/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : le 27 juillet 2001

   Module de cration d'un modle structural
   Scission d'une molcule en fragments ou en sous-molcules
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "systeme.h"
#include "atomes.h"
#include "txt_utils.h" /* Pour TXT_genere_chaine */

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static unsigned long suit_liaisons(Molecule *molecule, Atome *atome);

/* ~~~~~~~~ Ralisation pratique du module ~~~~~~~ */

/* -------- Isoler une entit covalente dans une molcule et en faire une nouvelle molcule
     Renvoie la nouvelle molcule cre

     Attention, comme un atome ne peut appartenir  plus d'une molcule,
     il faut en fait recopier tous les atomes concerns
*/

Molecule *MOL_isoler(Molecule *molecule, Atome *atome)
{
  char *nom;
  unsigned long nbr_atomes = 0;
/*   unsigned int j; */
  Atome *atm;
  Molecule *nv_molecule = NULL;

  if ( (molecule == NULL) || (atome == NULL) )
    return NULL;

  nom = TXT_genere_chaine("Mol"e_AIGU"cule isol"e_AIGU"e de %s", molecule->nom);
  nv_molecule = MOL_creer( nom );
  free( nom );
  if (nv_molecule == NULL) {
    (void) fprintf(stderr, "MOL_isoler : impossible de crer la molcule rsultat !\n");
    return NULL;
  }

  /* On enlve toutes les marques des atomes de la molcule */
  MOL_tt_demarquer(molecule);

  /* On marque tous les atomes lis de faon covalente  l'atome indiqu
     Fonction rcursive pour marquer en fait tous les atomes...

     La marque mk_1 indique que l'atome est en cours d'analyse (vite les cycles)
     La marque mk_2 indique que l'atome a dj t trait, de faon complte
 */
  nbr_atomes = suit_liaisons(molecule, atome);
#ifdef _DEVELOPPE_
  printf("MOL_isoler : %lu atomes connects dans cette molcule\n", nbr_atomes);
#endif

  if (nbr_atomes < 2) {
    (void) fprintf(stderr, "Pas d'entit covalente trouve\n");
    MOL_effacer(nv_molecule, VRAI, VRAI);
    return NULL;
  }
  if (nbr_atomes == molecule->nbr_atomes) {
    (void) fprintf(stderr, "La molcule est dj une molcule isole !\n");
    MOL_effacer(nv_molecule, VRAI, VRAI);
    return NULL;
  } 

  /* On parcourt la liste pour copier les atomes concerns */
  for ( atm = molecule->premier ; atm != NULL ; atm = atm->suivant ) {
    if ( VRAI == atm->marques.mk_2 ) {
      Atome *nv_atome;

      nv_atome = ATM_copier(atm, nv_molecule);
      if ( VRAI == atm->marques.absorbeur ) {
	creer_absorbeur(nv_atome, "COPIER ABSORBEUR", 1.0 /*molecule->absorbeurs[j].nom, molecule->absorbeurs[j].contribution.valeur_initiale */);
      }
    }
  }

  /* On reconstruit les liaisons covalentes */
  printf("MOL_isoler :  FAIRE, recopie intelligente de la table de liaison !\n");
  MOL_chercher_liaisons(nv_molecule);

  /* On prpare l'affichage */
  MOL_cherche_barycentre(nv_molecule, FAUX);
  MOL_initialise_vue(nv_molecule);

  return nv_molecule;
}

static unsigned long suit_liaisons(Molecule *molecule, Atome *atome)
{
  unsigned long nbr_marques;
  Fragment *fragment;
  Liaison *liaison;

  atome->marques.mk_1 = VRAI; /* Atome en cours d'analyse */
  nbr_marques = 1;

  /* On parcours tout le schma de liaisons */
  /* Liaisons inter-fragments en premier */
  for (liaison = molecule->liaisons ; liaison != NULL ; liaison = liaison->suivante) {
    if (liaison->type != LIAISON_COVALENTE)
      continue;

    if (liaison->atome_1 == atome) {
      if ( (liaison->atome_2->marques.mk_2 == FAUX) && (liaison->atome_2->marques.mk_1 == FAUX) ) {
	/* Atome pas encore trait */
	nbr_marques += suit_liaisons(molecule, liaison->atome_2);
      }
#ifdef _DEVELOPPE_
      else if (liaison->atome_2->marques.mk_1 == VRAI)
	printf("Cycle dtect...\n");
#endif
    } else if (liaison->atome_2 == atome) {
      if ( (liaison->atome_1->marques.mk_2 == FAUX) && (liaison->atome_1->marques.mk_1 == FAUX) ) {
	/* Atome pas encore trait */
	nbr_marques += suit_liaisons(molecule, liaison->atome_1);
      }
#ifdef _DEVELOPPE_
      else if (liaison->atome_1->marques.mk_1 == VRAI)
	printf("Cycle dtect...\n");
#endif
    }
  }

  /* On s'occupe maintenant des liaisons dans les fragments */
  for (fragment = molecule->fragments ; fragment != NULL ; fragment = fragment->suivant) {
    for (liaison = fragment->liaisons ; liaison != NULL ; liaison = liaison->suivante) {
      if (liaison->type != LIAISON_COVALENTE)
	continue;

      if (liaison->atome_1 == atome) {
	if ( (liaison->atome_2->marques.mk_2 == FAUX) && (liaison->atome_2->marques.mk_1 == FAUX) ) {
	  /* Atome pas encore trait */
	  nbr_marques += suit_liaisons(molecule, liaison->atome_2);
	}
#ifdef _DEVELOPPE_
	else if (liaison->atome_2->marques.mk_1 == VRAI)
	  printf("Cycle dtect...\n");
#endif
      } else if (liaison->atome_2 == atome) {
	if ( (liaison->atome_1->marques.mk_2 == FAUX) && (liaison->atome_1->marques.mk_1 == FAUX) ) {
	  /* Atome pas encore trait */
	  nbr_marques += suit_liaisons(molecule, liaison->atome_1);
	}
#ifdef _DEVELOPPE_
	else if (liaison->atome_1->marques.mk_1 == VRAI)
	  printf("Cycle dtect...\n");
#endif
      }
    }
  }

  atome->marques.mk_2 = VRAI; /* Atome : analyse termine */

  return nbr_marques;
}
