/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2007, Emmanuel CURIS
   Cration : janvier 2007

   Fonctions d'automatisation

   Routines gnriques (gestion de la structure)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "automatise.h"
#include "graphes.h"
#include "images.h"

/* -------- Variables locales -------- */

/* -------- Dclaration des fonctions locales -------- */

struct automatiser *AUTO_ReserverStructure(void);

/* -------- Rservation de la structure -------- */

struct automatiser *AUTO_ReserverStructure(void)
{
  struct automatiser *lot;

  /* Rservation */
  lot = (struct automatiser *) calloc( 1, sizeof( struct automatiser ) );
  if ( NULL == lot ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la structure\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Initialisation [autre que celle faite par calloc] */
  lot->type_objet      = 19750221;	/* Srement jamais atteint : sera repr si pas initialis ! */
  lot->classe_fonction = 19741910;
  lot->genre           = 20040911;
  lot->type            = 12345678;
  lot->numero_fonction = 11223344;

  lot->reference.graphe = NULL;

  lot->objets = NULL;
  lot->resultats = NULL;

  lot->methode = AUTOMATISE_METHODE_ROBOT; /* Reproduction bte par dfaut */
  lot->infos_necessaires = NULL;

  lot->nom_serie = NULL;

  /* Termin */
  return lot;
}

/* -------- Prparer une automatisation  partir d'une liste de graphes -------- */

struct automatiser *AUTO_GRF_PreparerStructure(unsigned long nombre_graphes, struct graphe **graphes)
{
  struct automatiser *lot;
  unsigned long i;

  if ( ( 0 == nombre_graphes ) || ( NULL == graphes ) ) {
    /* Aucun graphe indiqu ! */
    return NULL;
  }

  /* On prpare la structure */
  lot = AUTO_ReserverStructure();
  if ( NULL == lot ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en rservant la structure d'automatisation\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On l'initialise pour un travail sur des graphes */
  lot->type_objet = AUTOMATISE_GRAPHE;

  /* On rserve la table des objets */
  lot->objets = (struct objet_generique *) malloc( sizeof( struct objet_generique ) * nombre_graphes );
  if ( NULL == lot->objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver pour %lu graphes !\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre_graphes);
    (void) AUTO_LibererStructure( lot ); lot = NULL;
    return NULL;
  }

  /* On rserve la table des rsultats */
  lot->resultats = (struct objet_generique *) malloc( sizeof( struct objet_generique ) * nombre_graphes );
  if ( NULL == lot->resultats ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver pour %lu graphes (rsultats) !\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre_graphes);
    (void) AUTO_LibererStructure( lot ); lot = NULL;
    return NULL;
  }

  /* On initialise tout cela */
  lot->nombre_objets = nombre_graphes;
  for ( i = 0 ; i < nombre_graphes ; i++ ) {
    lot->objets[ i ].graphe = graphes[ i ];
    lot->resultats[ i ].graphe = NULL;
  }

  /* Termin */
  return lot;
}

/* -------- Librer une structure d'automatisation -------- */

int AUTO_LibererStructure(struct automatiser *lot)
{
  if ( NULL == lot ) return 0;

  /* On efface les rsultats, s'ils sont isols */
  if ( lot->resultats != NULL ) {
    unsigned long i;

    switch ( lot->type_objet )
    {
    case AUTOMATISE_GRAPHE:
      for ( i = 0 ; i < lot->nombre_objets ; i++ ) {
	Graphe *graphe;

	graphe = lot->resultats[ i ].graphe;
	if ( GRF_EstIsole( graphe ) ) {
	  GRF_eff_isole( graphe );
	}
      }
      break;

    case AUTOMATISE_IMAGE:
      for ( i = 0 ; i < lot->nombre_objets ; i++ ) {
	Image *image;

	image = lot->resultats[ i ].image;
	if ( IMG_EstIsolee( image ) ) {
	}
      }
      break;

    case AUTOMATISE_MOLECULE:
      break;

    default:
      AVERTISSEMENT( "Type d'objet non reconnu [%u]", lot->type_objet );
    }
  }

  /* On libre les structures */
  free( lot->resultats ); lot->resultats = NULL;
  free( lot->objets ); lot->objets = NULL;

  if ( lot->infos_necessaires ) {
    (void) fprintf(stderr, "%s,%s l.%u : Les infos complmentaires n'ont pas t libres !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    free( lot->infos_necessaires ); lot->infos_necessaires = NULL;
  }

  /* On libre le contenant */
  free( lot );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Ralisation des calculs, en fonction de la routine de calcul choisie -------- */

int AUTO_EffectuerCalculs(struct automatiser *lot)
{
  if ( NULL == lot ) return 0;

  switch ( lot->type_objet )
  {
  case AUTOMATISE_GRAPHE:
    return AUTO_EffectuerCalculs_GRF( lot );

  case AUTOMATISE_IMAGE:
    OPTION_A_FAIRE( "Appel de AUTO_EffectuerCalculs_IMG( lot )" );
    break;

  case AUTOMATISE_MOLECULE:
    OPTION_A_FAIRE( "Appel de AUTO_EffectuerCalculs_IMG( lot )" );
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Type d'objet non gr [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, lot->type_objet );
  }

  /* Problme dans l'aiguillage */
  return -250;
}
