/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des ascenseurs
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <math.h> /* Pour log10 */

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int ARB_CreerAscenseur(unsigned int arbre, unsigned objet_1, int x, int y, int sens, unsigned int taille)
{
  int retour;
  OBJECT *objets;

  /* Les contrles seront effectus dans ARB_CreerObjet */
  retour = ARB_CreerObjet( arbre, objet_1, G_BUTTON, x, y, 2, 1 );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : erreur dans ARB_CreerObjet [code %d]\n",
		    __FILE__, __FUNCTION__, __LINE__, retour );
    return -1;
  }
  objets = arbres[ arbre ].adresse;

  if ( ASC_VERTICAL == sens ) {
    (void) ARB_CreerObjet( arbre, objet_1 + 1, G_BOX, x, y + 1, 2, taille - 2 );
    (void) ARB_CreerObjet( arbre, objet_1 + 2, G_BOX, 0, 0, 2, taille - 2 );

    retour = ARB_CreerBouton( arbre, objet_1 + 3, x, y + (int) (taille - 1), 2, "\002" );
    if ( retour != 0 ) {
      (void) fprintf( stderr, "%s, %s l.%u : erreur dans ARB_CreerObjet [code %d]\n",
		      __FILE__, __FUNCTION__, __LINE__, retour );
      return -1;
    }

    objets[ objet_1 ].ob_spec.texte = "\001";
  } else {
    (void) ARB_CreerObjet(arbre, objet_1 + 1, G_BOX, x, y + 1, taille - 2, 1);
    (void) ARB_CreerObjet(arbre, objet_1 + 2, G_BOX, 0, 0, taille - 2, 1);

    retour = ARB_CreerBouton( arbre, objet_1 + 3, x + (int) (taille - 1), y, 2, "\004" );
    if (retour != 0) {
      (void) fprintf(stderr, "ARB_CreerAscenseur : erreur dans ARB_CreerObjet [code %d]\n", retour);
      return -1;
    }

    objets[ objet_1 ].ob_spec.texte = "\003";
  }

  /* Le bouton est fils du fond */
  objets[ objet_1 + 1 ].ob_next = objet_1 + 3;
  objets[ objet_1 + 1 ].ob_head = objet_1 + 2;
  objets[ objet_1 + 1 ].ob_tail = objet_1 + 2;
  OBJ_couleur( arbre, objet_1 + 1, GRIS_FONCE, COULEUR_FOND, METTRE);
  objets[ objet_1 + 1 ].ob_spec.gbox.motif = 6;	 /* Trame, pour voir la couleur, pas complte, pour ne pas avoir la 3D */

  objets[ objet_1 + 2 ].ob_next = objet_1 + 1;
  OBJ_couleur( arbre, objet_1 + 2, GRIS, COULEUR_FOND, METTRE);

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Mise  la taille d'un ascenseur, dans un arbre GEM
    Renvoie la position de l'ascenseur, en index par rapport  la liste indique, commenant  ZRO

  num_arbre  : l'arbre o se trouve l'ascenseur
  obj_avant  : la flche allant vers le prcdent
  obj_fond   : le fond de l'ascenseur
  obj_asc    : le poussoir de l'ascenseur
  obj_apres  : la flche allant vers le suivant
  sens       : ascenseur horizontal (ASC_HORIZONTAL), vertical (ASC_VERTICAL) ?
  nbr_objets : nombre d'objets dans la liste
  nbr_vis    : nombre d'objets visibles dans le dfileur associ
  idx_prem   : numro du premier objet visible (commence  0)
  objet      : l'objet sur lequel on a cliqu
  x, y       : position de la souris dans cet objet
  mode       : veut-on le positionner et adapter (METTRE), juste lire (QUESTION) ou grer l'ascenseur (ECHANGE) ?
  -------- */

unsigned long ARB_ascenseur(unsigned int num_arbre, unsigned int obj_avant, unsigned int obj_fond, unsigned int obj_asc, unsigned int obj_apres, int sens,
			    unsigned long nbr_objets, unsigned int nbr_visibles, unsigned long idx_premier,
			    unsigned int objet, int x, int y, boolean mode)
{
  unsigned int taille_fond;

  /* On rcupre la taille du fond, sert toujours */
  if ( ASC_VERTICAL == sens ) {
    OBJ_taille( num_arbre, obj_fond, -1, -1, NULL, &taille_fond );
  } else if ( ASC_HORIZONTAL == sens ) {
    OBJ_taille( num_arbre, obj_fond, -1, -1, &taille_fond, NULL );
  }

  if ( nbr_visibles >= nbr_objets ) { /* Moins d'objets que l'on ne peut en afficher */
    if ( METTRE == mode ) {
      OBJ_active( num_arbre, obj_avant, FORCE_NON );
      OBJ_active( num_arbre, obj_fond , FORCE_NON );
      OBJ_active( num_arbre, obj_asc  , FORCE_NON );
      OBJ_active( num_arbre, obj_apres, FORCE_NON );
      OBJ_position( num_arbre, obj_asc, 0, 0, NULL, NULL );
      if ( ASC_VERTICAL == sens ) {
	OBJ_taille( num_arbre, obj_asc, -1, taille_fond, NULL, NULL );
      } else if ( ASC_HORIZONTAL == sens ) {
	OBJ_taille( num_arbre, obj_asc, taille_fond, -1, NULL, NULL );
      }
    }
    return 0; /* On affiche toujours le premier lment de la liste en premier, quand on peut tout voir */
  }

  if ( idx_premier + nbr_visibles > nbr_objets )
    idx_premier = nbr_objets - nbr_visibles;

  /* Si l'on est l, il y a plus d'objets que l'on ne peut en afficher */
  if ( METTRE == mode ) { /* On initialise l'ascenseur */
    float taille, position;

    OBJ_active( num_arbre, obj_avant, idx_premier > 0 );
    OBJ_active( num_arbre, obj_apres, idx_premier + nbr_visibles < nbr_objets );
    OBJ_active( num_arbre, obj_fond, FORCE_OUI );
    OBJ_active( num_arbre, obj_asc , FORCE_OUI );

    taille = (float) (taille_fond) * (float) nbr_visibles / (float) nbr_objets;
    if ( taille < 2.0 ) taille = 2.0; /* Pour ne pas avoir un poussoir trop petit */
    
    position = (taille_fond - taille) * idx_premier / (float) (nbr_objets - nbr_visibles);
#ifdef _DEVELOPPE_
    if ( ( position < 0.0 ) || ( position > (float) taille_fond ) )
      (void) fprintf( stderr, "ARB_ascenseur / METTRE : position du bouton absurde, %f\n", position );
#endif
    
    if ( ASC_VERTICAL == sens ) {
      OBJ_taille  ( num_arbre, obj_asc, -1, (int) taille  , NULL, NULL );
      OBJ_position( num_arbre, obj_asc, -1, (int) position, NULL, NULL );
    } else if ( ASC_HORIZONTAL == sens ) {
      OBJ_taille  ( num_arbre, obj_asc, (int) taille  , -1, NULL, NULL );
      OBJ_position( num_arbre, obj_asc, (int) position, -1, NULL, NULL );
    }
  } else if (mode == ECHANGE) { /* On utilise l'ascenseur : on n'a que la position  changer */
    unsigned int taille;
    float position;

    if (objet == obj_avant) {
      if (idx_premier > 0) {
	if (idx_premier + nbr_visibles == nbr_objets) /* On tait  la fin */
	  OBJ_active(num_arbre, obj_apres, FORCE_OUI);
	idx_premier--;
      }
      if (idx_premier == 0)
	OBJ_active(num_arbre, obj_avant, FORCE_NON);
    } else if (objet == obj_fond) {
      int pos_bouton;

      if (sens == ASC_HORIZONTAL) {
	OBJ_position(num_arbre, obj_asc, -1, -1, &pos_bouton, NULL);
	if (x < pos_bouton) {/* Clic avant */
	  if (idx_premier > 10)
	    idx_premier -= 10;
	  else
	    idx_premier = 0;
	} else {
	  if (idx_premier + nbr_visibles + 10 < nbr_objets)
	    idx_premier += 10;
	  else
	    idx_premier = nbr_objets - nbr_visibles;
	}
      } else {
	OBJ_position(num_arbre, obj_asc, -1, -1, NULL, &pos_bouton);
	if (y < pos_bouton) {/* Clic aprs */
	  if (idx_premier > 10)
	    idx_premier -= 10;
	  else
	    idx_premier = 0;
	} else {
	  if (idx_premier + nbr_visibles + 10 < nbr_objets)
	    idx_premier += 10;
	  else
	    idx_premier = nbr_objets - nbr_visibles;
	}
      }

      OBJ_active(num_arbre, obj_avant, idx_premier > 0);
      OBJ_active(num_arbre, obj_apres, idx_premier + nbr_visibles < nbr_objets);
    } else if (objet == obj_asc) {
      int nv_position;
      float nv_premier;

      nv_position = X_ascenseur( arbres[ num_arbre ].objets[ obj_fond ],
				 arbres[ num_arbre ].objets[ obj_asc ],
				 (sens == ASC_VERTICAL) ? (VERTICAL) : (HORIZONTAL) );
      nv_premier = (float) nv_position * (float) (nbr_objets - nbr_visibles) / 1000.0;
      if (nv_premier < 0.0) nv_premier = 0.0;
      if (nv_premier + nbr_visibles > nbr_objets) nv_premier = (float) (nbr_objets - nbr_visibles);
      idx_premier = (unsigned long) nv_premier;

      OBJ_active(num_arbre, obj_avant, idx_premier > 0);
      OBJ_active(num_arbre, obj_apres, idx_premier + nbr_visibles < nbr_objets);
    } else if (objet == obj_apres) {
      if (idx_premier + nbr_visibles < nbr_objets) {
	if (idx_premier == 0) /* On tait au dbut */
	  OBJ_active(num_arbre, obj_avant, FORCE_OUI);
	idx_premier++;
      }
      if (idx_premier + nbr_visibles == nbr_objets)
	OBJ_active(num_arbre, obj_apres, FORCE_NON);
    }

    if (sens == ASC_VERTICAL)
      OBJ_taille(num_arbre, obj_asc, -1, -1, NULL, &taille);
    else
      OBJ_taille(num_arbre, obj_asc, -1, -1, &taille, NULL);

    position = (taille_fond - taille) * idx_premier / (float) (nbr_objets - nbr_visibles);
#ifdef _DEVELOPPE_
    if ( (position < 0.0) || (position > (float) taille_fond) )
      (void) fprintf(stderr, "ARB_ascenseur / ECHANGE : position du bouton absurde, %f\n", position);
#endif
    
    if (sens == ASC_VERTICAL) {
      OBJ_position(num_arbre, obj_asc, -1, (int) position, NULL, NULL);
    } else if (sens == ASC_HORIZONTAL) {
      OBJ_position(num_arbre, obj_asc, (int) position, -1, NULL, NULL);
    }

  } else { /* On lit la position "relle" (il peut y avoir des problmes d'arrondi) */
    int position;
    unsigned int taille;

    if (sens == ASC_VERTICAL) {
      OBJ_taille(num_arbre, obj_fond, -1, -1, NULL, &taille);
      OBJ_position(num_arbre, obj_asc, -1, -1, NULL, &position);
    } else if (sens == ASC_HORIZONTAL) {
      OBJ_taille(num_arbre, obj_fond, -1, -1, &taille, NULL);
      OBJ_position(num_arbre, obj_asc, -1, -1, &position, NULL);
    }

    idx_premier = (unsigned long) (position * (float) (nbr_objets - nbr_visibles) / (float) (taille_fond - taille));
  }

  return idx_premier;
}

/* -------- Gestion d'un ascenseur entier, dans un arbre GEM
   Renvoie la nouvelle valeur aprs gestion de l'ascenseur

 num_arbre                  : l'arbre contenant l'ascenseur
 avant, fond, bouton, apres : les objets constituant l'ascenseur
 mode                       : ASC_VERTICAL ou ASC_HORIZONTAL
 min, max, pas              : les bornes de variation de la variable
 valeur                     : la valeur initiale (utilise seulement si obj_clic = ROOT)
 obj_clic, x, y             : l'objet cliqu et la position de la souris sur cet objet
 ------------------ */

int gere_ascenseur(unsigned int num_arbre, unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres,
		   int mode, int min, int max, int pas, int valeur, unsigned int obj_clic, int x, int y)
{
  OBJECT *adresse;
  int larg_asc;

  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "gere_ascenseur : numro d'arbre incorrect [%u / max. %u]\n", num_arbre, BI_nbr_arbres);
    return valeur;
  }

  adresse = arbres[ num_arbre ].adresse;
  if ( NULL == adresse ) {
    (void) fprintf( stderr, "gere_ascenseur : arbre inexistant [%u]\n", num_arbre );
    return valeur;
  }

#ifdef _DEVELOPPE_
  (void) printf( "gere_ascenseur : arbre %u, objets %u / %u / %u / %u - cliqu %u\n",
		 num_arbre, avant, fond, bouton, apres, obj_clic );
#endif

  if ( obj_clic > ROOT ) {
    valeur = OBJ_LireEntier_S( num_arbre, bouton ); /* Valeur actuelle */
#ifdef _DEVELOPPE_
/*     (void) printf("gere_ascenseur : valeur = %d...", valeur); */
#endif
  }
  if (obj_clic == avant) { /* Clic sur flche gauche / basse */
    if (valeur == max) /* On tait sur la dernire */
      OBJ_active(num_arbre, apres, FORCE_OUI);
    valeur -= pas; /* Valeur prcdente */
    if (valeur <= min) {
      OBJ_active(num_arbre, avant, FORCE_NON);
      valeur = min;
    }
  } else if (obj_clic == fond) { /* Clic sur le fond de l'ascenseur */
    if ( mode == ASC_VERTICAL ) { /* Ascenseur vertical */
      if ( y < OB_Y(adresse, bouton) ) { /* Au-dessus du poussoir */
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      } else {
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      }
    } else { /* Ascenseur horizontal */
      if ( x < OB_X(adresse, bouton) ) { /* Avant le poussoir */
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      } else {
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      }
    }
    OBJ_active(num_arbre, avant, valeur > min);
    OBJ_active(num_arbre, apres, valeur < max);
  } else if (obj_clic == bouton) { /* Clic sur le poussoir */
    valeur = graf_slidebox(adresse, (int) fond, (int) bouton, (mode == ASC_VERTICAL) );
    if (mode == ASC_VERTICAL) { /* Ascenseur vertical */
      valeur = max - valeur * (max - min) / 1000;
    } else {
      valeur = min + valeur * (max - min) / 1000;
    }
    OBJ_active(num_arbre, avant, valeur > min);
    OBJ_active(num_arbre, apres, valeur < max);
  } else if (obj_clic == apres) { /* Clic sur la flche droite / haute */
    if (valeur == min)
      OBJ_active(num_arbre, avant, FORCE_OUI);
    valeur += pas;
    if (valeur >= max) {
      OBJ_active(num_arbre, apres, FORCE_NON);      
      valeur = max;
    }
  }

#ifdef _DEVELOPPE_
/*     (void) printf(" devient %d", valeur); */
#endif

  /* Positionnement du poussoir */
  OBJ_MettreEntier_S( num_arbre, bouton, valeur ); /* Nouvelle valeur */
  if ( mode == ASC_VERTICAL ) { /* Ascenseur vertical */
    int y_asc;

    larg_asc = OB_HEIGHT(adresse, fond) - OB_HEIGHT(adresse, bouton);
    y_asc = ((float) (max - valeur)) / ((float) (max - min)) * (float) larg_asc;
    OBJ_position(num_arbre, bouton, -1, y_asc, &y_asc, &y_asc);
  } else {
    int x_asc;

    larg_asc = OB_WIDTH(adresse, fond) - OB_WIDTH(adresse, bouton);
    x_asc = (int) ( ((float) (valeur - min)) / ((float) (max - min)) * (float) larg_asc );
#ifdef _DEVELOPPE_
/*     (void) printf(" [asc.en x = %d]\n", x_asc); */
#endif
    OBJ_position(num_arbre, bouton, x_asc, -1, &x_asc, &x_asc);
  }

  return valeur;
}
