/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système
   Fichier d'inclusion générique, qui se charge d'inclure en fonction du système
*/

#ifndef BI_COMMUNS_H_
# define BI_COMMUNS_H_ /* Pas d'inclusion multiple */

#include "deboguage.h"		/* Pour le log de fonctions, éventuellement... */

/* La partie spécifique du système */
#ifdef _GEM_
# include "bib_gem.h"
#endif
#ifdef _UNIX_
# include "Bib_X/Bib_X.h"
#endif
#ifdef _WINDOWS_
# include "Bib_W32/Bib_W32.h"
# ifdef BI_LOG_FONCTIONS_
#  undef stderr
#  define stderr BI_Log_Fonctions
#  undef stdout
#  define stdout BI_Log_Fonctions
# endif
#endif

#if defined(AVEC_OPEN_GL_)
# include <GL/gl.h>
# include <GL/glu.h>
#endif

/* ~~~~~~~~ La partie commune ~~~~~~~~ */

/* ------ Démarrage & fin du programme ------ */

boolean BI_Initialiser(const char *nom_ressource, unsigned int nbr_arbres, int *nbr_parametres, char **liste_parametres[]);
void    BI_Terminer   (void);

/* ------ Gestion des événements ------ */

#define AC_BLOQUER_SOURIS 1

void BI_EtatSouris(int *x, int *y, int *boutons, unsigned int *n_clics, int *etat);

/* ------ Gestion du sélecteur de fichiers ------ */

#define BI_SELECTEUR_MULTIPLE            1 /* Bit 0 ; mis : sélection multiple autorisée */
#define BI_SELECTEUR_FILTRE_UTILISATEUR  2 /* Bit 1 ; mis : on peut entrer un filtre manuellement */
#define BI_SELECTEUR_AJOUTER_PREDEFINI   4 /* Bit 2 ; mis : on peut modifier des chemins prédéfinis */
#define BI_SELECTEUR_UTILISER_APERCU     8 /* Bit 3 ; mis : il y a une routine d'aperçu disponible */
#define BI_SELECTEUR_AVEC_JOKERS        16 /* Bit 4 ; mis : les jokers servent pour indiquer une liste de fichiers */
#define BI_SELECTEUR_AVEC_SAUVEGARDE    32 /* Bit 5 ; mis : on fait une copie du fichier s'il existe, avant de le sauver */
#define BI_SELECTEUR_CHEMIN_SEULEMENT   64 /* Bit 6 ; mis : on ne s'intéresse qu'au chemin final, on ne sélectionne aucun fichier */
#define BI_SELECTEUR_MEMORISER_CHEMIN  128 /* Bit 7 ; mis : on mémorise le chemin en fermant le sélecteur [ie il est renvoyé à l'utilisateur] */

#define BI_SELECTEUR_ANNULER        -1975  /* Code spécial : annulation */

typedef struct {
  char *titre;		   /* Le titre à utiliser pour le sélecteur [pas const car peut être réservé avec malloc()... */
  unsigned int proprietes;	/* Masque de bits : cf. les #define ci-dessus */
  char *chemin;			/* Le chemin de travail initial [NULL = chemin de travail de l'application...] */
  unsigned int nbr_filtres;	/* Nombre de filtres */
  const char **filtres;		/* Les filtres */
  unsigned int nbr_predefinis;	/* Nombre de chemins prédéfinis [modifiable !] */
  char **predefinis;		/* Table des chemins prédéfinis [peut changer pendant l'appel] */
  void (*apercu)(unsigned int numero_arbre, unsigned int numero_objet, const char *nom_fichier);
} BI_InfosSelecteur;

int BI_SelecteurDeFichiers(BI_InfosSelecteur *infos, unsigned long *nombre_choisis, char ***fichiers_choisis, Fenetre *fenetre_mere);

int   BI_SelecteurChargement(BI_InfosSelecteur *infos, unsigned long *nombre_fichiers, char ***noms_fichiers, Fenetre *fenetre_mere);
char *BI_SelecteurSauvegarde(BI_InfosSelecteur *infos, int *code_erreur, Fenetre *fenetre_mere);
char *BI_SelecteurChemin    (BI_InfosSelecteur *infos, int *code_erreur, Fenetre *fenetre_mere);

int BI_Selecteur_AjouterChemin(char ***table_chemins, unsigned int *nombre_chemins, const char *chemin_a_ajouter);

char **CH_selecteur  (const char *masque, const char *titre, unsigned int nbr_ext,
		      const char **extension, unsigned int *nbr_fichiers, Fenetre *fenetre_mere);
char  *CHEM_selecteur(const char *titre, Fenetre *fenetre_mere);

/* ------ Autres fonctions... ------ */

#include "arbres.h"
#include "demande.h"
#include "erreur.h"
#include "evenements.h"
#include "dessins.h"
#include "fenetres.h"
#include "objets.h"
#include "rsc.h"

#define FIN_ERREUR_GRAVE      0 /* Erreur grave, il faut trouver pourquoi => on sort de toute façon */
#define FIN_ERREUR_MINEURE    1 /* Erreur moyenne, en version de développement on quitte */
#define FIN_ERREUR_COSMETIQUE 2 /* Erreur minime, point qu'il faudrait fixer... */
#define FIN_OPTION_MANQUANTE  3 /* Option non-implémentée */
#define FIN_AVERTISSEMENT     4 /* Quelque chose de vraiment, vraiment mineur ! */

#ifdef _DEVELOPPE_
# ifdef AVEC_DB_MALLOC_
#  include <dbmalloc.h>
# endif
# define FIN_ANTICIPEE(niveau) {\
  if ( niveau <= _DEVELOPPE_ ) { /* Suivant la gravité de l'erreur */\
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR GRAVE / Sortie anticipée brutale pour déboguage\n",\
	  	   __FILE__, __FUNCTION__, __LINE__ );\
    ( * (char *) NULL ) = 0; /* Seg. fault assurée => déboguage facile & core dump */\
    abort(); /* Si les signaux sont bloqués... */\
  }\
}
#else
# define FIN_ANTICIPEE(niveau) /* On fait comme si de rien n'était... */
#endif

#define OPTION_A_FAIRE(texte) {\
  (void) fprintf(stderr, "%s,%s l.%u : À FAIRE --- %s\n", __FILE__, __FUNCTION__, __LINE__, texte );\
  FIN_ANTICIPEE( FIN_OPTION_MANQUANTE );\
}

#define AVERTISSEMENT(texte,...) {\
  (void) fprintf(stderr, "%s,%s l.%u : ATTENTION --- ", __FILE__, __FUNCTION__, __LINE__ );\
  (void) fprintf(stderr, texte, __VA_ARGS__);\
  (void) fputc( '\n', stderr);\
  FIN_ANTICIPEE( FIN_AVERTISSEMENT );\
}

#endif /* BI_COMMUNS_H_ */
