/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Arbre cr au vol : pour demander des valeurs
   Dclarations (indpendantes de LASE)
*/

#ifndef BI_DEMANDE_H_
# define BI_DEMANDE_H_ /* Protection / inclusion multiple */

/* ~~~~~~~~ Demander des choses classiques  l'utilisateur ~~~~~~~~ */

short int BI_DemanderMarque    (unsigned int numero_arbre, unsigned int numero_objet);
short int BI_DemanderStyleLigne(unsigned int numero_arbre, unsigned int numero_objet);

/* ~~~~~~~~ Demander une ou plusieurs valeurs  l'utilisateur ~~~~~~~~ */

#define BI_CHAMP_DEMANDE_ENTIER_U 0 /* Demander un entier positif [unsigned long] */
#define BI_CHAMP_DEMANDE_ENTIER_S 1 /* Demander un entier sign   [long] */
#define BI_CHAMP_DEMANDE_REEL     2 /* Demander un nombre rel    [quelconque, avec restrictions sur les bornes ; erreur associe autorise !] */
#define BI_CHAMP_DEMANDE_COMPLEXE 3 /* Demander un nombre complexe */
#define BI_CHAMP_DEMANDE_CHAINE   4 /* Demander une chane de caractres, quelconque */
#define BI_CHAMP_DEMANDE_LISTE    5 /* Slection dans une liste de valeurs */
#define BI_CHAMP_DEMANDE_BINAIRE  6 /* Choix OUI/NON */

union valeur {
  unsigned long entier_U;	/* Entier positif */
  long entier_S;		/* Entier quelconque */
  double reel;		/* Nombre rel */
  struct { double Re, Im; } complexe;	/* Nombre complexe */
  char *chaine;		/* Texte quelconque [ librer avec free aprs !] */
  unsigned long numero_option; /* choisie dans la liste */
};

typedef struct infos_demande {
  unsigned int type;		/* Cf. codes BI_CHAMP_DEMANDE_xxx ci-dessus */

  const char *texte;     /* Texte  afficher avant */
  const char *unite;     /* Unit de rfrence [champs numriques uniquement] */

  /* -- complments pour champs numriques -- */
  int proprietes;        /* Quel contrainte a-t-on sur la valeur attendue ? Cf. codes BI_EDITABLE_xxx ci-dessous */
  double min, max;       /* Bornes autorises */
  double erreur;	 /* Erreur associe, ventuellement */

  /* -- complments pour liste de choix */
  unsigned long nombre_options;
  const char **options;

  /* La valeur initiale */
  union valeur valeur_initiale;

  /* La valeur par dfaut */
  union valeur valeur_defaut;

  /* La valeur indique par l'utilisateur */
  union valeur valeur_donnee;
} BI_Champ_Demande;

typedef struct {
  const char *titre; /* Titre de la bote de dialogue */
  const char *texte; /* Texte prparatoire de la bote de dialogue */
  unsigned int nbr_boutons, bouton_ok, bouton_annuler, bouton_reinit;
  const char *boutons[3]; /* Trois boutons au maximum */
} BI_Boite_Demande;

/* -------- Les proprits d'un champ ditable numrique -------- */

#define BI_EDITABLE_MIN        1 /* Bit mis : il y a une valeur minimale impose */
#define BI_EDITABLE_MAX        2 /* Bit mis : il y a une valeur maximale impose */
#define BI_EDITABLE_ENTIER     4 /* Bit mis : la valeur doit tre entire */
#define BI_EDITABLE_ANGLE      8 /* Bit mis : la valeur est un angle (en degrs) */
#define BI_EDITABLE_FRACTION  16 /* Bit mis : la valeur est comprise entre 0 et 1 */
#define BI_EDITABLE_FORMULE   32 /* Bit mis : les formules (simples) sont autorises */
#define BI_EDITABLE_FONCTION  64 /* Bit mis : les formules (complexes) sont autorises */
#define BI_EDITABLE_ERREUR   128 /* Bit mis : une erreur peut tre associe */
#define BI_EDITABLE_UNITE    256 /* Bit mis : une unit peut tre associe */
#define BI_EDITABLE_PROBLEME 512 /* Bit mis : valeur entre non-valable [=> afficher en rouge, p. ex.] */
#define BI_EDITABLE_TEXTE   1024 /* Bit mis : on attend un texte */

char  *BI_Demande_Texte(Fenetre *mere, const BI_Boite_Demande *contenu, const char *texte_initial, ...);

int BI_DemandeValeur  (Fenetre *mere, const BI_Boite_Demande *contenu, struct infos_demande *valeur,...);
int BI_DemandeMultiple(Fenetre *mere, const BI_Boite_Demande *contenu, unsigned int nbr_valeurs, struct infos_demande *valeurs,
		       ...);

#endif
