/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Arbres simplistes d'usage courant : demander une valeur (texte, entier, flottant,...)
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <math.h> /* Pour log10 */

#include "txt_utils.h"

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LARGEUR_ICONE                  10
#define LARGEUR_NON_TEXTE_BOITE_ALERTE (3+LARGEUR_ICONE) /* 1 + icone = 10 + 1 [+texte] + 1 */
#define LARGEUR_MINIMALE_LIGNE_TEXTE   50 /* en caracteres */
#define LARGEUR_MAXIMALE_LIGNE_TEXTE   (LARGEUR_MAXIMALE_BOITE_ALERTE - LARGEUR_NON_TEXTE_BOITE_ALERTE)	/* en caracteres */
#define LARGEUR_MINIMALE_BOITE_ALERTE  (LARGEUR_MINIMALE_LIGNE_TEXTE+LARGEUR_NON_TEXTE_BOITE_ALERTE) /* en caracteres ; texte + marges, icone... */
#define MARGE_ECRAN_BOITE_ALERTE       50 /* en pixels ; marge laterale minimale de chaque cote de la boite de dialogue */

static unsigned int LARGEUR_MAXIMALE_BOITE_ALERTE = 0;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int BI_vERR_alerte(Fenetre *mere, const BI_Boite_Alerte *boite, va_list parametres);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Affichage d'un message en cas d'annulation d'une opration -------- */

void BI_ERR_annuler(const char *texte,...)
{
  BI_Boite_Alerte message = { ERR_ICONE_INFO, "Op"e_AIGU"ration annul"e_AIGU"e", NULL, 1, 1, 1, { "Annuler", NULL, NULL, NULL, NULL} };
  va_list parametres;
  int retour;

  if ( NULL == texte ) return;

  message.texte = texte;

  va_start(parametres, texte);
  retour = BI_vERR_alerte( NULL, &message, parametres );
  va_end(parametres);

  if ( retour < 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans BI_vERR_alerte, code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
  }
}

/* -------- Fonction d'affichage d'un message d'erreur - plus gnrique que le form_alert initial
   Renvoi le numro du bouton utilis pour sortir

mere  : fentre appelante, que l'on bloque. NULL = on bloque tout.
boite : la structure dcrivant la bote d'alerte

Ensuite : valeurs des paramtres du texte, comme printf
 -------- */

int BI_ERR_alerte(Fenetre *mere, const BI_Boite_Alerte *boite, boolean bloquante,...)
{
  va_list parametres;
  int retour;

  ENTREE_FONCTION( __FUNCTION__, "mere, boite=%p, bloquante=%d,...", (void *) boite, bloquante );

  if ( NULL == boite ) {
    (void) fprintf(stderr, "%s,%s l.%u : appel avec une bote NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    SORTIE_FONCTION( __FUNCTION__, "%d [boite = NULL]", 0 );
    return 0;
  }
  if ( NULL == boite->texte ) {
    (void) fprintf(stderr, "%s,%s l.%u : appel avec une bote sans texte !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    SORTIE_FONCTION( __FUNCTION__, "%d [boite->texte = NULL]", 0 );
    return 0;
  }
  if ( ( boite->nbr_boutons < 1 ) || ( boite->nbr_boutons > BI_NOMBRE_MAX_BOUTONS_ALERTE ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : nombre de boutons incorrect, doit valoir entre 1 et %u !\n",
		   __FILE__, __FUNCTION__, __LINE__, BI_NOMBRE_MAX_BOUTONS_ALERTE);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    SORTIE_FONCTION( __FUNCTION__, "%d [Nombre de boutons incorrect, %d]", 0, boite->nbr_boutons );
    return 0;
  }

  /* On appelle la vraie fonction */
  va_start(parametres, bloquante);
  retour = BI_vERR_alerte( mere, boite, parametres );
  va_end(parametres);

  /* On renvoie la rponse obtenue */
  return retour;
}

/* ~~~~~~~~ La vraie routine qui se charge d'afficher et grer la bote de dialogue ~~~~~~~~ */

static int BI_vERR_alerte(Fenetre *mere, const BI_Boite_Alerte *boite, va_list parametres)
{
  unsigned int haut_boite, larg_boite;
  unsigned long nbr_lignes, longueur_ligne;
  unsigned int i, nbr_objets, bouton;
  unsigned int obj_icone;
  int retour;
  char *texte_reel = NULL, **lignes;
  OBJECT *objets;

  /* On vrifie qu'il n'y a pas un appel rcursif... */
  if ( arbres[ BI_ArbreAlerte ].drapeaux.ouvert == VRAI ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : appel rcursif impossible!\n * Texte de l'appel rcursif : %s\n",
		   __FILE__, __FUNCTION__, __LINE__, boite->texte );
#endif
    SORTIE_FONCTION( __FUNCTION__, "%d [Appel rcursif]", -1 );
    return -1;
  }

  /* Si pas encore fait : largeur maximale possible pour une boite d'alerte */
  if ( 0 == LARGEUR_MAXIMALE_BOITE_ALERTE ) {
    LARGEUR_MAXIMALE_BOITE_ALERTE = ( BI_larg_ecran - 2 * MARGE_ECRAN_BOITE_ALERTE ) / BI_larg_car;
#ifdef _DEVELOPPE_
    (void) printf( "%s : Largeur maximale de boite de dialogue = %u caracteres\n", __FUNCTION__, LARGEUR_MAXIMALE_BOITE_ALERTE );
#endif
  }

  /* ------ Prparation du texte ------ */
  texte_reel = TXT_vGenereChaine( boite->texte, parametres );
  if ( NULL == texte_reel ) {
    (void) fprintf( stderr, "%s : impossible de prparer le texte !\n", __FUNCTION__ );
    SORTIE_FONCTION( __FUNCTION__, "%d [texte_reel = NULL]", -2 );
    return -2;
  }
#ifdef _DEVELOPPE_
  (void) printf( "%s : texte du message, avant coupures :\n%s\n", __FUNCTION__, texte_reel );
#endif

  /* On compte le nombre de lignes et on dcoupe en morceaux... */
  lignes = TXT_GenereLignes( texte_reel, LARGEUR_MAXIMALE_LIGNE_TEXTE, '\n', &nbr_lignes );
  if ( NULL == lignes ) {
    (void) fprintf( stderr, "%s : impossible de prparer les lignes ! Message :\n%s\n",
		    __FUNCTION__, texte_reel );
    SORTIE_FONCTION( __FUNCTION__, "%d [Cration des lignes impossible]", -3 );
    return -3;
  }
  DANS_FONCTION( "Lignes prtes" );
#ifdef _DEVELOPPE_
  (void) printf( "%s : texte du message, aprs traitement :\n", __FUNCTION__ );
  for ( i = 0 ; i < nbr_lignes ; i++ ) {
    (void) printf("  %s\n", lignes[ i ]);
  }
#endif

  /* --- Taille de la bote --- */
  haut_boite = nbr_lignes + 2; /* Une ligne vide avant et aprs */
  if ( haut_boite < 5 ) /* Au moins 5 lignes pour avoir la place pour l'icne */
    haut_boite = 5;
  haut_boite += 2; /* Ligne des boutons */
  if ( boite->nbr_boutons > 3 ) /* Il faut une 2e range de boutons */
    haut_boite += 2;

  /* largeur : en fonction de la ligne la plus longue */
  {
    unsigned long lg_ligne_max = 0, lg_ligne;

    for ( i = 0 ; i < nbr_lignes ; i++ ) {
      lg_ligne = strlen( lignes[ i ] );
      if (  lg_ligne > lg_ligne_max ) lg_ligne_max = lg_ligne;
    }

    /* Si plus courtes que ce que peut l'ecran : on adapte la boite ! */
    if ( lg_ligne_max < LARGEUR_MAXIMALE_LIGNE_TEXTE ) {
      if ( lg_ligne_max < LARGEUR_MINIMALE_LIGNE_TEXTE ) lg_ligne_max = LARGEUR_MINIMALE_LIGNE_TEXTE;

      larg_boite = lg_ligne_max + LARGEUR_NON_TEXTE_BOITE_ALERTE;
      longueur_ligne = lg_ligne_max;
    } else {
      larg_boite     = LARGEUR_MAXIMALE_BOITE_ALERTE;
      longueur_ligne = LARGEUR_MAXIMALE_LIGNE_TEXTE;
    }
  }

  nbr_objets = nbr_lignes + 1 + boite->nbr_boutons + 1; /* Un par ligne, 1 par bouton, icne, ROOT */
  retour = BI_CreerArbre( BI_ArbreAlerte, nbr_objets, larg_boite, haut_boite );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : erreur dans BI_creer_arbre (arbre alerte : %u --- %u)\n",
		    __FUNCTION__, BI_ArbreAlerte, BI_nbr_arbres - 3 );
    (void) fprintf( stderr, "   Message : %s\n", texte_reel );
    free( texte_reel ); texte_reel = NULL;
    free( lignes ); lignes = NULL;
    SORTIE_FONCTION( __FUNCTION__, "%d [Cration de l'arbre impossible]", 0 );
    return 0;
  }
  DANS_FONCTION( "Arbre %d cr [%p; objets en %p]", BI_ArbreAlerte, arbres + BI_ArbreAlerte, objets = arbres[ BI_ArbreAlerte ].adresse );
  arbres[ BI_ArbreAlerte ].drapeaux.interne = VRAI; /* On ne libre pas les chaines en meme temps que l'arbre */
  objets = arbres[ BI_ArbreAlerte ].adresse;

  /* On s'occupe des objets */

  /* Les boutons de sortie */
  for ( i = 1 ; i <= boite->nbr_boutons ; i++ ) {
    objets[ i ].ob_next = i + 1;
    objets[ i ].ob_head = -1;
    objets[ i ].ob_tail = -1;
    objets[ i ].ob_type = G_BUTTON;
    objets[ i ].ob_spec.texte = boite->boutons[i - 1];
    objets[ i ].ob_flags = SELECTABLE | EXIT;
    objets[ i ].ob_state = 0;
    if ( i == boite->bouton_ok )
      objets[ i ].ob_flags |= DEFAULT;
    objets[ i ].ob_y = haut_boite - 2;
    objets[ i ].ob_height = 1;
  }
  if ( boite->nbr_boutons == 1 ) {
    objets[ 1 ].ob_x = 16;
    objets[ 1 ].ob_width = 30;
  } else if ( boite->nbr_boutons == 2 ) {
    objets[ 1 ].ob_x = 4;
    objets[ 1 ].ob_width = 25;
    objets[ 2 ].ob_x = 33;
    objets[ 2 ].ob_width = 25;
  } else if ( boite->nbr_boutons == 3 ) {
    objets[ 1 ].ob_x = 3;
    objets[ 1 ].ob_width = 16;
    objets[ 2 ].ob_x = 22;
    objets[ 2 ].ob_width = 16;
    objets[ 3 ].ob_x = 41;
    objets[ 3 ].ob_width = 16;
  } else if ( boite->nbr_boutons == 4 ) {
    objets[ 1 ].ob_x = 4;
    objets[ 1 ].ob_width = 25;
    objets[ 1 ].ob_y -= 2;
    objets[ 2 ].ob_x = 33;
    objets[ 2 ].ob_width = 25;
    objets[ 2 ].ob_y -= 2;
    objets[ 3 ].ob_x = 4;
    objets[ 3 ].ob_width = 25;
    objets[ 4 ].ob_x = 33;
    objets[ 4 ].ob_width = 25;
  } else if ( boite->nbr_boutons == 5 ) {
    objets[ 1 ].ob_x = 3;
    objets[ 1 ].ob_width = 16;
    objets[ 1 ].ob_y -= 2;
    objets[ 2 ].ob_x = 22;
    objets[ 2 ].ob_width = 16;
    objets[ 2 ].ob_y -= 2;
    objets[ 3 ].ob_x = 41;
    objets[ 3 ].ob_width = 16;
    objets[ 3 ].ob_y -= 2;
    objets[ 4 ].ob_x = 4;
    objets[ 4 ].ob_width = 25;
    objets[ 5 ].ob_x = 33;
    objets[ 5 ].ob_width = 25;
  }

  /* L'icne */
  obj_icone = i;
  retour = ARB_CreerObjet( BI_ArbreAlerte, obj_icone, G_ICON, 1, 1, LARGEUR_ICONE, 4 );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : erreur dans ARB_creer_objet / icne (objet %u)\n",
		    __FILE__, __FUNCTION__, __LINE__, obj_icone );
    free( lignes ); lignes = NULL;
    free( texte_reel ); texte_reel = NULL;
    (void) BI_LibererArbre( BI_ArbreAlerte );
    SORTIE_FONCTION( __FUNCTION__, "%d [Cration de l'icone impossible]", -5 );
    return -5;
  }
  OBJ_icone( BI_ArbreAlerte, obj_icone, boite->icone );

  /* Les lignes de texte */
  (void) ARB_CreerNTextes( BI_ArbreAlerte, obj_icone + 1, nbr_lignes, ROOT, 10, 1, longueur_ligne );

  /* Mise en place des textes, avec adaptations eventuelles */
  for ( i = 0 ; i < nbr_lignes ; i++ ) {
    if ( NULL == lignes[ i ] ) {
      AVERTISSEMENT( "La ligne %u est NULL !", i );
      continue;
    }
    if ( ( '-' == *lignes[ i ] ) && ( '-' == *(lignes[ i ] + 1) ) ) {
      /* Separateur... */
      OBJ_ChangerType( BI_ArbreAlerte, obj_icone + 1 + i, G_SEPARATEUR );
    } else {
      objets[ obj_icone + 1 + i ].ob_spec.texte = lignes[ i ];
    }
  }

#ifdef _DEVELOPPE_
/*   printf("Boite d'alerte : arbre %u / %u objets, %lu lignes\n", BI_ArbreAlerte, nbr_objets, nbr_lignes); */
/*   for (i = 0 ; i < nbr_objets ; i++) { */
/*     printf("Objet %d :\n", i); */
/*     printf("  Suivant : %d - Fils 1 : %d - Fils n : %d\n", objets[i].ob_next, objets[i].ob_head, objets[i].ob_tail); */
/*     if ( (objets[i].ob_type == G_BUTTON) || (objets[i].ob_type == G_STRING) ) */
/*       printf("  Texte : %s\n", objets[i].ob_spec.texte); */
/*     else if ( objets[i].ob_type == G_ICON ) */
/*       printf("  Icne...\n"); */
/*     if ( objets[i].ob_flags & LASTOB ) */
/*       printf("** Dernier objet **\n"); */
/*   } */
#endif

  arbres[ BI_ArbreAlerte ].objet_annuler = boite->bouton_esc;

  DANS_FONCTION( "On va initialiser l'arbre" );
  ARB_Initialiser( BI_ArbreAlerte, NULL, ( int (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );

  DANS_FONCTION( "On va grer l'arbre" );
  bouton = ARB_Bloquant( BI_ArbreAlerte, boite->titre, mere, BI_ArbreAlerte );

  free( lignes ); lignes = NULL;
  free( texte_reel ); texte_reel = NULL;
  (void) BI_LibererArbre( BI_ArbreAlerte );

  SORTIE_FONCTION( __FUNCTION__, "%d [Choix utilisateur]", bouton );
  return bouton;
}
