/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des fentres
   Dclarations (indpendantes de LASE)
   Remarque : la structure Fenetre dpend du systme
              ==> elle est dfinie dans xx_fenetres.h
*/

#ifndef BI_FENETES_COMMUN_H_
# define BI_FENETRES_COMMUN_H_ /* Protection / inclusion multiple */

/* Les types de fentre prdfinis */

#define BI_FENETRE_SIMPLE  0 /* Fentre entirement gre par l'utilisateur */
#define BI_FENETRE_ARBRE   1 /* Fentre contenant uniquement un arbre GEM */
#define BI_FENETRE_IMAGE   2 /* Fentre contenant uniquement une image    */
#define BI_FENETRE_GL      3 /* Fentre contenant uniquement un affichage Open GL */

#define BI_FENETRE_MULTIPLE        10 /* Fentre contenant plusieurs sous-fentres */
#define BI_FENETRE_SIMPLE_ARBRE    11 /* Fentre multiple, cas particulier : simple  + arbre GEM */
#define BI_FENETRE_GL_ARBRE        12 /* Fentre multiple, cas particulier : Open GL + arbre GEM */
#define BI_FENETRE_GL_ARBRE_SIMPLE 13 /* Fentre multiple, cas particulier : Open GL + arbre GEM + rgion utilisateur */
#define BI_FENETRE_GL_SIMPLE       14 /* Fentre multiple, cas particulier : Open GL + simple */
#define BI_FENETRE_ONGLETS         15 /* Fentre contenant plusieurs arbre GEM avec des onglets pour choisir celui affich */

/* Position de l'arbre pour une fentre mixte ou triple
   De la partie Open-GL pour une fentre mixte/Open-GL ou triple */

#define BI_FEN_ARBRE_AUCUN   0 /* Arbre de fentre mixte de taille fixe */
#define BI_FEN_ARBRE_GAUCHE  1 /*   "    "    "      "   occupe toute la hauteur  gauche */
#define BI_FEN_ARBRE_DROITE  2 /*   "    "    "      "      "     "    "    "    " droite */
#define BI_FEN_ARBRE_HAUT    3 /*   "    "    "      "      "     "    " largeur en haut  */
#define BI_FEN_ARBRE_BAS     4 /*   "    "    "      "      "     "    "    "    "  bas   */
#define BI_FEN_ARBRE_COIN_HG 5
#define BI_FEN_ARBRE_COIN_HD 6
#define BI_FEN_ARBRE_COIN_BG 7
#define BI_FEN_ARBRE_COIN_BD 8

/* ~~~~~~~~~~ Cration, fermeture, ... de fentres ~~~~~~~~~~~~ */

/* -------- Briques de base de la cration -------- */

int      FEN_CreerFenetre      ( Fenetre *fenetre, Fenetre *mere );
int      FEN_DechainerFenetre  ( Fenetre *fenetre );
int      FEN_DetruireFenetre   ( Fenetre *fenetre, boolean memoriser_taille );
Fenetre *FEN_FenetreMere       ( Fenetre *fenetre );
int      FEN_InitialiserFenetre( Fenetre *fenetre );
int      FEN_LibererFenetre    ( Fenetre *fenetre );
Fenetre *FEN_NouvelleFenetre   ( int type, int attributs, Fenetre *mere );

void FEN_RedessinDefaut    (Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);
void FEN_GL_redessin_defaut(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);

int  FEN_ImposerTaille     (Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);
int  FEN_OptimiserDimensionsInternes( Fenetre *fenetre );

/* -------- Cration : plusieurs types. ATTENTION, l'ouverture est  la charge de l'appelant ! -------- */

Fenetre *FEN_CreerFenetreSimple(int attributs, const char *titre, const char *infos,
				void (*redessin) (Fenetre *, int, int, unsigned int, unsigned int),
				void (*ascenseur)(Fenetre *, int, int, int),
				int  (*clavier)  (Fenetre *, unsigned int, int),
				int  (*clic)     (Fenetre *, int, int, int , int, int),
				int  (*message)  (Fenetre *, Message *message),
				void (*rect)     (Fenetre *, int, int, int, int),
				int  (*temps)    (Fenetre *) );

Fenetre *FEN_CreerFenetreGL(int attributs, const char *titre,
			    void (*redessin) (Fenetre *, int, int, unsigned int, unsigned int),
			    void (*ascenseur)(Fenetre *, int, int, int),
			    int  (*clavier)  (Fenetre *, unsigned int, int),
			    int  (*clic)     (Fenetre *, int, int, int, int, int),
			    int  (*message)  (Fenetre *, Message *message),
			    void (*rect)     (Fenetre *, int, int, int, int),
			    int  (*temps)    (Fenetre *) );

Fenetre *FEN_CreerAvecArbre(int attributs, const char *titre, const char *infos,
			    unsigned int numero_arbre, int position_arbre, int (*init_arbre)(unsigned int),
			    void (*redessin)(Fenetre *, int, int, unsigned int, unsigned int),
			    int  (*clavier) (Fenetre *, unsigned int, int),
			    int  (*clic)    (Fenetre *, int, int, int, int, int),
			    int  (*message) (Fenetre *, Message *message),
			    void (*rect)    (Fenetre *, int, int, int, int),
			    int  (*temps)   (Fenetre *) );

Fenetre *FEN_CreerAvecGL(int attributs, const char *titre, int position_gl,
			 void (*redessin)   (Fenetre *, int, int, unsigned int, unsigned int),
			 void (*redessin_gl)(Fenetre *, int, int, unsigned int, unsigned int),
			 void (*ascenseur)  (Fenetre *, int, int, int),
			 int  (*clavier)    (Fenetre *, unsigned int, int),
			 int  (*clic)       (Fenetre *, int, int, int, int, int),
			 int  (*message)    (Fenetre *, Message *message),
			 void (*rect)       (Fenetre *, int, int, int, int),
			 int  (*temps)      (Fenetre *) );

Fenetre *FEN_CreerFenetreTriple(int attributs, const char *titre,
				unsigned int numero_arbre, int position_arbre, int (*init_arbre)(unsigned int),
				int position_gl,
				void (*redessin)   (Fenetre *, int, int, unsigned int, unsigned int),
				void (*redessin_gl)(Fenetre *, int, int, unsigned int, unsigned int),
				void (*ascenseur)  (Fenetre *, int, int, int),
				int  (*clavier)    (Fenetre *, unsigned int, int),
				int  (*clic)       (Fenetre *, int, int, int, int, int),
				int  (*message)    (Fenetre *, Message *message),
				void (*rect)       (Fenetre *, int, int, int, int),
				int  (*temps)      (Fenetre *) );

Fenetre *FEN_CreerOnglets(int attributs, unsigned int nombre_onglets, struct bi_onglet *onglets);

Fenetre *FEN_OuvrirArbre(Fenetre *mere, int attributs, unsigned int numero_arbre, const char *titre, int x, int y);
Fenetre *FEN_CreerFenetreMultiple(const char *titre, unsigned int nbr_fenetres, int attributs[], int types[]);

void FEN_AssocierEvenements(Fenetre *fenetre,
			    void (*redessin) (Fenetre *, int, int, unsigned int, unsigned int),
			    void (*ascenseur)(Fenetre *, int, int, int),
			    int  (*clavier)  (Fenetre *, unsigned int, int),
			    int  (*clic)     (Fenetre *, int, int, int, int, int),
			    int  (*message)  (Fenetre *, Message *message),
			    void (*rect)     (Fenetre *, int, int, int, int),
			    int  (*temps)    (Fenetre *) );

void FEN_GL_evntsgl(Fenetre *fenetre,
		    void (*redessin)(Fenetre *, int, int, unsigned int, unsigned int),
		    void (*rect)    (Fenetre *, int, int, int, int),
		    int  (*clic)    (Fenetre *, int, int, int, int, int),
		    int  (*deplace) (Fenetre *, int, int, int, int),
		    int  (*temps)   (Fenetre *) );

#define FEN_ContientArbre(type) ( ( BI_FENETRE_ARBRE == (type) ) || ( BI_FENETRE_SIMPLE_ARBRE == (type) ) || ( BI_FENETRE_GL_ARBRE == (type) ) || ( BI_FENETRE_GL_ARBRE_SIMPLE == (type) ) || ( BI_FENETRE_ONGLETS == (type) ) )

/* -------- Ouverture -------- */

/* int FEN_Afficher(Fenetre *fenetre); */
int FEN_Reouvrir(Fenetre *fenetre);
int FEN_Ouvrir  (Fenetre *fenetre);

Fenetre *FEN_premierplan(Fenetre *fenetre);

/* -------- Fermeture -------- */

void FEN_Fermer  (Fenetre *fenetre, boolean detruit); /* Juste la fentre   */
int  FEN_Detruire(Fenetre *fenetre); /* Fentre _et_ structure associe */
/* int  FEN_Cacher  (Fenetre *fenetre); */

#define BI_FEN_FERMER_FORCER_NORMAL  0 /* Message de fermeture "normal" */
#define BI_FEN_FERMER_FORCER_OK     -1 /* Message de fermeture en forant une rponse de type OK */
#define BI_FEN_FERMER_FORCER_ANNULE -2 /* Message de fermeture en forant une rponse de type Annuler */

int FEN_MessageFermeture(Fenetre *fenetre, int mode_fermeture);

/* -------- Aspect -------- */

int FEN_ascenseur(Fenetre *fenetre, int asc, int pos, int taille);

int FEN_infos(Fenetre *fenetre, const char *infos);
int FEN_titre(Fenetre *fenetre, const char *titre);

boolean FEN_dimensions(Fenetre *fenetre, int x, int y, int larg, int haut);

boolean FEN_palette(Fenetre *fenetre, Palette palette);

int FEN_AdapterArbre      (Fenetre *contenant, Fenetre *arbre);
int FEN_AdapterOpenGL     (Fenetre *contenant, Fenetre *opengl, unsigned int proportion);
int FEN_AdapterSousFenetre(Fenetre *contenant, Fenetre *arbre, Fenetre *utilisateur);

/* -------- Contenu -------- */

int FEN_QuelArbre(Fenetre *fenetre);

#define FEN_COMPLEMENT_1 (1)	/* Premire information complmentaire */
#define FEN_COMPLEMENT_2 (2)	/* Seconde information complmentaire */

void *FEN_LireComplement   (Fenetre *fenetre, int numero);
void  FEN_StockerComplement(Fenetre *fenetre, int numero, const void *info);

/* -------- Redessins -------- */

#define BI_RDS_UTIL    (1)     /* Redessiner la portion utilisateur */
#define BI_RDS_GL      (2)     /* Redessiner la portion Open-GL     */
#define BI_RDS_FILLES  (4)     /* Redessiner aussi les filles !     */
#define BI_RDS_ONGLETS (8)     /* Redessiner juste les onglets      */

#define BI_RDS_TOUT (BI_RDS_UTIL | BI_RDS_GL | BI_RDS_FILLES | BI_RDS_ONGLETS)

boolean FEN_redessin(Fenetre *fenetre, unsigned int portion);

/* -------- Aides pour le dveloppement -------- */

#ifdef _DEVELOPPE_
void    BI_AfficherFenetre        (Fenetre *fenetre);
boolean BI_VerifierChainageFenetre(Fenetre *fenetre);
#endif

#endif /* BI_FENETRES_COMMUN_H_ */
