/* Bibliothèque GEM / X-Window - (C) 1998, Emmanuel CURIS
   Gestion des arbres d'objet
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>		/* Pour isspace() */

#include "bib_interface.h"

static int ARB_ConvertitTEDINFO(OBJECT *objet, unsigned int num_arbre, unsigned int num_objet);

/* -------- Initialisation d'une boîte de dialogue : partie spécifique X-Window --------\
 | Renvoie 0 si tout s'est bien passé, -1 sinon                                         |
 |  APPEL : on suppose les premiers contrôles faits => adresse != NULL, nombre d'objets connu...
 |                                                                                      |
 | Il y a deux boucles sur les objets, mais comme la fonction n'est appelée normalement |
 | qu'une seule fois par arbre et qu'un arbre n'est jamais immense, ce n'est pas gênant |
 \------------------------------------------------------------------------------------- */

int ARB_InitSpecifique(unsigned int arbre)
{
  OBJECT *objet;
  int retour;
  unsigned int i;

  /* Création de la table des fenêtres X */
  arbres[ arbre ].objets = (Window *) calloc( arbres[ arbre ].nbr_objets, sizeof(Window) );
  if ( NULL == arbres[ arbre ].objets ) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante pour le tableau des fenêtres X des objets\n",
		    __FILE__, __FUNCTION__, __LINE__);
    (void) fprintf( stderr, " --- Opération annulée ---\n" );
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "Arbre : %d - %u objets\n", arbre, arbres[ arbre ].nbr_objets );
#endif
    return -2;
  }

  /* Analyse objet par objet pour améliorer la gestion */
  for ( i = 0, objet = arbres[ arbre ].adresse ; i < arbres[ arbre ].nbr_objets ; i++, objet++ ) {
    switch ( objet->ob_type )
    {
    case G_TITLE:
      if ( objet->ob_spec.txt_malloc != NULL ) {
	char *texte;
	unsigned long lg_texte;

	/* On ôte les espaces finaux */
	texte = objet->ob_spec.txt_malloc;
	lg_texte = strlen( texte );
	while ( ' ' == texte[ lg_texte - 1 ] ) {
	  texte[ lg_texte - 1 ] = 0;
	  if ( lg_texte > 1 )
	    lg_texte--;
	}

	/* On ajoute un caractère à la largeur, pour que cela soit plus joli */
	objet->ob_width += BI_larg_car;
      }
      break;

    case G_BUTTON:
      if ( objet->ob_state & SHADOWED ) {
	OPTION_A_FAIRE("Conversions bouton -> sous_menu [?]");
      }
      break;

    case G_FTEXT: case G_FBOXTEXT:
      retour = ARB_ConvertitTEDINFO( objet, arbre, i );
      if ( retour != 0 ) {
	(void) fprintf( stderr, "%s, %s l.%u : ERREUR survenue dans la conversion\n",
			__FILE__, __FUNCTION__, __LINE__ );
      }
      break;

    case G_USERDEF:
      if ( objet->ob_spec.util != NULL ) { /* Structure présente */
	if ( ( int (*)(struct parm_blk *) ) NULL == objet->ob_spec.util->ub_code ) { /* Mais pas de code d'affichage */
	  OBJ_ChangerType( arbre, i, G_SEPARATEUR, NULL );
	}
      } else { /* Pas de structure présente */
	OBJ_ChangerType( arbre, i, G_SEPARATEUR, NULL );
      }
      break;
    }
  }

  /* Rien à signaler, initialisation réussie */
  return 0;
}

/* -------- Convertit une TEDINFO en champ éditable interne -------- */

static int ARB_ConvertitTEDINFO(OBJECT *objet, unsigned int num_arbre, unsigned int num_objet)
{
  boolean av_unite = FAUX;
  const char *masque;
  int retour = 0;
  unsigned int nbr_separateurs = 0;
  char separateur = 0;
  char *unite = NULL;

  if ( NULL == objet->ob_spec.tedinfo ) {
    (void) fprintf( stderr, "%s, %s l.%u : Aucune TEDINFO associée à l'arbre !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return -1;
  }

  /* Quel est le masque ? */
  masque = objet->ob_spec.tedinfo->te_ptmplt;
  if ( *masque != '_' ) {
    /* On vérifie qu'il ne s'agirait pas simplement d'espaces... */
    while ( isspace( *masque ) ) {
      masque++;
      if ( 0 == *masque ) break; /* Masque avec que des espaces : paraît absurde */
    }
  }

  if ( *masque != '_' ) {
    AVERTISSEMENT( "Texte éditable, avec texte avant [%s]", objet->ob_spec.tedinfo->te_ptmplt );
    return +1; /* Pas de conversion possible actuellement ! */
  }

  /* On compte le nombre de séparateurs connus */
  while ( *masque != 0 ) {
    if ( ( '.' == *masque ) || ( ','== *masque ) || ( '/'== *masque ) ) {
      nbr_separateurs++;
      separateur = *masque;
    } else if ( *masque != '_') {
      /* On s'arrête, et on contrôlera après, car on peut avoir des . ou / dans les unités
	 qui fausseraient la recherche des séparateurs... */
      av_unite = VRAI;
      break;
    }

    masque++;
  }

  /* on détecte l'unité à utiliser */
  if ( VRAI == av_unite ) {
    const char *pos_unite;

    pos_unite = masque;

    /* Est-ce bien une unité, pas un texte au milieu ? */
    while ( *masque != 0 ) {
      if ( '_'== *masque ) {
	/* Éditable de la forme __truc__ -> ignoré ! */
	AVERTISSEMENT("Texte éditable avec texte au milieu [%s]", objet->ob_spec.tedinfo->te_ptmplt);
	return +2;
      }
      masque++;
    }

    /* On repère donc l'unité proprement dite */
    while ( ' ' == *pos_unite ) {
      pos_unite++;
      if ( 0 == *pos_unite ) {
	AVERTISSEMENT("Texte éditable fini par des espaces [%s]", objet->ob_spec.tedinfo->te_ptmplt);
	pos_unite = NULL;
	break;
      }
    }

    /* On a bien trouvé une unité : on en fait une copie
       (quand on libèrera la TEDINFO, on perdra l'original !) */
    if ( pos_unite != NULL ) {
      unite = strdup( pos_unite );
      if ( NULL == unite ) {
	(void) fprintf(stderr, "%s, %s l.%u : Impossible de créer l'unité. Conversion annulée\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -5;
      }
    }
  }

  /* Suivant le nombre de séparateurs... */
  if ( 0 == nbr_separateurs ) { /* "______" ou "_____texte" */
    unsigned int proprietes;

    /* Quels caractères sont autorisés ? */
    masque = objet->ob_spec.tedinfo->te_pvalid;
    if ( '9' == *(masque + 1) ) { /* Chiffre en 2e position */
      int premier_caractere;

      /* On mémorise le 1er caractère : indiquera si le signe est autorisé... */
      premier_caractere = *masque;
      masque++;

      /* On regarde s'il n'y a bien ensuite que des chiffres */
      while ( *masque != 0 ) {
	if ( *masque != '9' ) {
	  AVERTISSEMENT("Champ éditable inconnu [masque=%s, filtre=%s]", objet->ob_spec.tedinfo->te_ptmplt, objet->ob_spec.tedinfo->te_pvalid);
	  free( unite ); unite = NULL;
	  return +10;
	}
	masque++;
      }

      /* Nombre entier */
      proprietes = BI_EDITABLE_ENTIER;
      if ( '9' == premier_caractere ) { /* 1er caractère, chiffre obligatoire */
	proprietes |= BI_EDITABLE_MIN;
      }
      if ( unite != NULL ) {
	proprietes |= BI_EDITABLE_UNITE;
      }

      /* Conversion en nombre */
      retour = OBJ_ChangerType( num_arbre, num_objet, G_NOMBRE );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s, %s l.%u : ERREUR en voulant changer le type de l'objet [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, retour);
	free( unite ); unite = NULL;
	return -20;
      }

      /* Adaptation des options */
      OBJ_OptionsEditable( num_arbre, num_objet, proprietes, 0.0, 0.0, unite );
    } else if ( 0 == *(masque + 1) ) { /* Un seul caractère éditable */
      if ( '9' == *masque ) {
	retour = OBJ_ChangerType( num_arbre, num_objet, G_NOMBRE );
	if ( retour != 0 ) {
	  (void) fprintf(stderr, "%s, %s l.%u : ERREUR en voulant changer le type de l'objet [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
	  free( unite ); unite = NULL;
	  return -20;
	}

	proprietes = BI_EDITABLE_ENTIER | BI_EDITABLE_MIN | BI_EDITABLE_MAX;
	if ( unite != NULL ) {
	  proprietes |= BI_EDITABLE_UNITE;
	}
	OBJ_OptionsEditable( num_arbre, num_objet, proprietes, 0.0, 9.0, unite );
      }
    } else {
      if ( unite != NULL ) {
	AVERTISSEMENT("Champ suivi d'un texte [%s], non-géré", objet->ob_spec.tedinfo->te_ptmplt);
	free( unite ); unite = NULL;
	return +10;
      }

      /* Conversion en texte infini */
      retour = OBJ_ChangerType( num_arbre, num_objet, G_TXT_INFINI );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de créer l'éditable infini / arbre %u, objet %u\n",
		       __FILE__, __FUNCTION__, __LINE__, num_arbre, num_objet);
	return -1;
      }
    }
  } else if ( 1 == nbr_separateurs ) {
    unsigned int proprietes = 0;

    if ( '/' == separateur ) { /* __/__ texte : fraction, pas encore géré */
      AVERTISSEMENT("Texte éditable pour fraction rationnelle exacte [%s]", masque);
      free( unite ); unite = NULL;
      return +3;
    }

    /* On suppose arbitrairement un nombre réel... */
    masque = objet->ob_spec.tedinfo->te_pvalid;
    if ( '9' == *masque ) { /* On ne peut pas entrer de signe */
      proprietes |= BI_EDITABLE_MIN;
    }
    if ( unite != NULL ) {
      proprietes |= BI_EDITABLE_UNITE;
    }

    /* Conversion en nombre */
    retour = OBJ_ChangerType( num_arbre, num_objet, G_NOMBRE );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR en voulant changer le type de l'objet [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      free( unite ); unite = NULL;
      return -20;
    }

    /* Adaptation des options */
    OBJ_OptionsEditable( num_arbre, num_objet, proprietes, 0.0, 0.0, unite );
  } else {
    AVERTISSEMENT("Texte éditable avec plus de 2 séparateurs [%s]", masque);
    return +5;
  }

  /* On libère l'unité [une copie a été faite dans la structure] */
  free( unite ); unite = NULL;

  /* Conversion réussie */
  return 0;
}

/* -------- Renvoie le numéro d'objet correspondant à la fenêtre X indiquée -------- */

unsigned int ARB_QuelObjet(unsigned int num_arbre, Window window)
{
  unsigned int objet = ROOT;
  unsigned int i;

  /* Contrôles préliminaires */
  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf( stderr, "%s,%s l.%u : numéro d'arbre incorrect [%u, max %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, num_arbre, BI_nbr_arbres - 1 );
    return ROOT;
  }
  if ( FAUX == arbres[num_arbre].drapeaux.ouvert ) {
    (void) fprintf( stderr, "%s,%s l.%u : arbre fermé [%u]\n",
		    __FILE__, __FUNCTION__, __LINE__, num_arbre );
    return ROOT;
  }
  if ( NULL == arbres[num_arbre].objets ) {
    (void) fprintf( stderr, "%s,%s l.%u : arbre ouvert mais sans fenêtres X ! [%u]\n",
		    __FILE__, __FUNCTION__, __LINE__, num_arbre );
    return ROOT;
  }

  /* A-t-on cliqué sur l'un des objets ? */
  for ( i = 0 ; i < arbres[ num_arbre ].nbr_objets ; i++ ) {
    if ( window == arbres[ num_arbre ].objets[ i ] ) {
      objet = i;
      break;
    }
  }

  if ( ROOT == objet ) { /* Non trouvé, ou clic sur le fond */
#ifdef _DEVELOPPE_
    if (i > 0) { /* Non trouvé ! */
      (void) fprintf( stderr, "%s,%s l.%u : la fenêtre X [%lu] ne correspond à aucun objet dans l'arbre [%u]\n",
		      __FILE__, __FUNCTION__, __LINE__, (unsigned long) window, num_arbre );
    }
#endif
    return ROOT;
  }

  if ( FAUX == OBJ_active( num_arbre, objet, QUESTION ) ) /* Objet inactif ! */
    return ROOT;

  return objet;
}

/* ----- Clic sur un arbre d'objets GEM --------\
 | Renvoie un des BI_EVENEMENT_*                |
 |                                              |
 | arbre  : pointeur sur l'arbre traité         |
 | souris : l'événement souris qui a eu lieu    |
 | bouton : contient le bouton qui a été cliqué |
 \--------------------------------------------- */

int CLIC_arbre(Arbre *arbre, XButtonEvent *souris, unsigned int *bouton)
{
  unsigned int numero_objet;
  int retour = BI_EVENEMENT_NON_TRAITE;

  numero_objet = ARB_QuelObjet( (unsigned int) (arbre - arbres), souris->window );

  *bouton = numero_objet;

  /* Maintenant que l'on a récupéré l'arbre et l'objet, on peu utiliser la gestion standard */
  retour = ARB_GererClic( arbre,  numero_objet, souris->button, souris->x, souris->y );
  if ( BI_EVENEMENT_ERREUR == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant cliquer [arbre=%u, objet=%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, (unsigned int) (arbre - arbres), numero_objet);
  }

  /* c. On renvoie ce que l'on a pu faire... */
  return retour;
}

/* ------------- Appelle un arbre GEM -------------------------------------- */

Fenetre *ARB_Appeler(unsigned int arbre, int attributs, const char *titre, void (*init_arbre)(void))
{
  if ( arbre >= BI_nbr_arbres ) {
    (void) fprintf( stderr, "%s,%s l.%u : numéro d'arbre incorrect [%u / max %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, arbre, BI_nbr_arbres - 1 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* Préparation éventuelle de l'arbre */
  if ( FAUX == arbres[ arbre ].drapeaux.prepare ) {
    /* D'abord, l'init personelle, car peut se charger d'appeler ARB_Initialiser, ou préparer avant */
    if ( init_arbre != ( void (*)(void) ) NULL ) {
      init_arbre();
    } else {
      OPTION_A_FAIRE( "Préparation par défaut de l'arbre" );
    }
  }

  /* Si l'arbre n'a pas été initialisé */
  if ( FAUX == arbres[ arbre ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( arbre, NULL,
			      ( boolean (*)(unsigned int, int, int  ) ) NULL,
			      ( int     (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) {
      (void) fprintf( stderr, "%s,%s l.%u : ERREUR %d dans ARB_Initialiser() [arbre %u]\n",
		      __FILE__, __FUNCTION__, __LINE__, retour, arbre );
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
      return NULL;
    }
  }

  /* Arbre en fenêtre      */
  return FEN_OuvrirArbre( NULL, attributs, arbre, titre, -1, -1 );
}

/* -------- Gestion d'un arbre bloquant -----------\
 | Renvoie le bouton utilisé pour quitter          |
 |   codes <0 en cas de problème                   |
 |                                                 |
 | numero_arbre : le numéro de l'arbre à traiter      |
 | titre        : titre associé à l'arbre             |
 | fenetre_mere : fenêtre dont on appelle l'arbre
 | numero_arbre_pere : arbre appelant [utilisé seulement si NULL pour fenetre_mere]
 \------------------------------------------------ */

int ARB_Bloquant(unsigned int numero_arbre, const char *titre, Fenetre *fenetre_mere, unsigned int numero_arbre_pere)
{
  Arbre *arbre = arbres + numero_arbre; /* Arbre utilisé */
  Fenetre *fen_arbre;
  int sortie = FAUX, retour;
  unsigned int bouton_sortie = 0;

  if ( numero_arbre >= BI_nbr_arbres ) {
    (void) fprintf( stderr, "%s,%s l.%u : numéro d'arbre incorrect [%u / max %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, numero_arbre, BI_nbr_arbres - 1 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }
  retour = ARB_Initialiser( numero_arbre, NULL,
			    ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : impossible d'initialiser l'arbre %u [code %d]\n",
		    __FILE__, __FUNCTION__, __LINE__, numero_arbre, retour );
    return -3;
  }

  /* On crée la fenêtre de l'arbre & on l'ouvre */
  if ( NULL == titre )
    titre = "Arbre bloquant";
  fen_arbre = FEN_OuvrirArbre( NULL, NAME, numero_arbre, titre, -1, -1 );
  if ( NULL == fen_arbre ) {
    (void) fprintf( stderr, "%s,%s l.%u : Impossible de créer la fenêtre [arbre %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, numero_arbre );
    return -1;
  }

  /* On indique qu'elle est modale pour la fenêtre mère */
  if ( NULL == fenetre_mere ) {
    /* Pas de fenêtre indiquée : est-ce depuis un arbre bloquant (qui sera forcément ouvert...) ? */
    if ( ( numero_arbre_pere != numero_arbre ) && ( numero_arbre_pere < BI_nbr_arbres ) ) {
      fenetre_mere = ARB_QuelleFenetre( arbres + numero_arbre_pere, FAUX );
    }
  }
  if ( fenetre_mere != NULL ) {
    XSetTransientForHint( BI_Display, fen_arbre->handle, fenetre_mere->handle );
  }

  do {
    Fenetre *fen_evenement;
    XEvent evenement;
    Window fenetre_X = None;
    Bool est_filtre;
    int retour;

    XNextEvent( BI_Display, &evenement );
    fen_evenement = BIX_QuelleFenetre( evenement.xbutton.window );
    if ( fen_evenement != NULL ) 
      fenetre_X = fen_evenement->handle;

    est_filtre = XFilterEvent( &evenement, fenetre_X );
    if ( True == est_filtre ) {
      (void) fprintf( stderr, "%s,%s l.%u : Événement X filtré, ignoré\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      continue;
    }

    switch ( evenement.type )
    {
    case KeyPress:
      if ( fen_evenement != fen_arbre ) {
	FEN_premierplan( fen_arbre );
      } else {
	/* La touche est toujours associée à l'arbre, on ne gère pas les menus */
	retour = EVNT_clavier( fen_arbre, &(evenement.xkey), FAUX );
/* 	printf("%s : retour EVNT_clavier = %d\n", __FUNCTION__, retour ); */
	if ( retour >= BI_EVENEMENT_CODE_OBJET ) {
	  bouton_sortie = retour - BI_EVENEMENT_CODE_OBJET;
	  sortie = BI_EVENEMENT_FERMER_ARBRE;
	}
      }
      break;

    case KeyRelease:
      break;

    case Expose: /* Redessin : rien de spécial */
      {
	Window ex_redessin = BI_Fen_redessin;
	BI_Fen_redessin = evenement.xexpose.window;
	EVNT_redessin(fen_evenement, &(evenement.xexpose) );
	BI_Fen_redessin = ex_redessin;
      }
      break;

    case ButtonPress:
      if ( fen_evenement == fen_arbre ) { /* Seuls les clics sur l'arbre sont enregistrés */
	if ( evenement.xbutton.window != fen_arbre->handle) { /* Clic sur un truc sympa... */
	  sortie = CLIC_arbre(arbre, &(evenement.xbutton), &bouton_sortie);
	}
      } else {
	FEN_premierplan( fen_arbre ); /* On remet l'arbre au premier plan, pour le rappeler à l'utilisateur */
      }
      break;

    case ConfigureNotify: /* Changement de taille de la fenêtre - XConfigureEvent */
      if ( ( fen_evenement != NULL ) && ( fen_evenement != fen_arbre ) ) { /* Il y a une fenêtre classique associée */
	int delta_x, delta_y;

	delta_x = evenement.xconfigure.width - fen_evenement->larg;
	delta_y = evenement.xconfigure.height - fen_evenement->haut;
	if ( ( 0 == delta_x ) && ( 0 == delta_y ) ) {
#ifdef _DEVELOPPE_
	  printf("Configure Notify sans changement de taille. Changement de l'ordre ?\n");
#endif
	  break;
	}

	/* Si fenêtre demande un message spécial en cas de changement de taille, on le fait */
	if ( fen_evenement->msg_masque.taille == VRAI )	{
	  Message message;

	  message.type = WM_SIZED;
	  message.fenetre.application = 0;
	  message.fenetre.en_plus = 0;
	  message.fenetre.handle = 0; /*quel_handle(fen_evenement);*/
	  message.fenetre.x = fen_evenement->x;
	  message.fenetre.y = fen_evenement->y;
	  message.fenetre.larg = evenement.xconfigure.width;
	  message.fenetre.haut = evenement.xconfigure.height;
	  (fen_evenement->message)(fen_evenement, &message);
	}

	/* On effectue le changement de taille */
	change_taille_fenetre( fen_evenement, evenement.xconfigure.width, evenement.xconfigure.height );
      }
      break;

    case MotionNotify: /* Déplacement du curseur */
      EVNT_pointeur( fen_evenement, &(evenement.xmotion) );
      break;

    case EnterNotify:
      EVNT_entree( fen_evenement, &(evenement.xcrossing) );
      break;

    case LeaveNotify: /* Entrée ou sortie d'une fenêtre - pour le pointeur en arrière plan */
      EVNT_sortie( fen_evenement, &(evenement.xcrossing) );
      break;

    case CirculateNotify: /* Pour empêcher l'arbre de passer à l'arrière plan */
      if ( ( ( fen_evenement == fen_arbre ) && ( evenement.xcirculate.place == PlaceOnBottom ) )
	|| ( ( fen_evenement != fen_arbre ) && ( evenement.xcirculate.place == PlaceOnTop ) ) ) {
	FEN_premierplan( fen_arbre );
      }
#ifdef _DEVELOPPE_
      else
	printf("Événement CirculateNotify pour la fenêtre %ld\n", fen_evenement->handle);
#endif
      break;

    case ClientMessage:
      if ( evenement.xclient.window == fen_arbre->handle ) {
	if ( (Atom) (evenement.xclient.data.l[ 0 ]) == BI_ProtocoleFermerFenetre ) {
#ifdef _DEVELOPPE_
	  (void) printf("Demande de fermeture forcée de l'arbre\n");
#endif
	  if ( arbres[ numero_arbre ].objet_annuler != ROOT ) { /* Il y a un objet pour annuler */
	    OBJ_choisit( numero_arbre, arbres[ numero_arbre ].objet_annuler, FORCE_OUI );
	    bouton_sortie = arbres[ numero_arbre ].objet_annuler;
	    sortie = BI_EVENEMENT_FERMER_ARBRE;
	  } else { /* Chercher le bouton de sortie par défaut... */
	    OPTION_A_FAIRE("Chercher le bouton de sortie par défaut... ou indiquer le bouton d'annulation !");
	  }
	}
      }
      break;

    case FocusIn:
      EVNT_Activation( fen_evenement, &evenement.xfocus );
      break;

#ifdef _DEVELOPPE_
    case ButtonRelease:		/* Bouton souris relâché */
    case DestroyNotify:
    case MapNotify:		/* La fenêtre devient affichée */
    case ReparentNotify:
    case UnmapNotify:		/* La fenêtre devient invisible */
    case FocusOut:		/* La fenêtre perd le premier plan */
      break;

    case NoExpose:		/* Rôle ?? */
      break;

    default:
      (void) fprintf(stderr, "%s,%s l.%u : type %d\n",
		     __FILE__, __FUNCTION__, __LINE__, evenement.xany.type);
#endif
    }
  } while ( sortie != BI_EVENEMENT_FERMER_ARBRE ); /* Tant qu'on a pas cliqué sur un objet de sortie */

  OBJ_choisit( numero_arbre, bouton_sortie, FORCE_NON );

  /* On ne détruit pas la structure, car appel récursif d'arbres possible... */
  FEN_Fermer( fen_arbre, VRAI );
  return bouton_sortie;
}

/* --------------- Renvoie l'arbre associé à un objet ---------------- */

Arbre *quel_arbre(OBJECT *objet)
{
  Arbre *arbre = arbres;
  unsigned int i;

  /* On vérifie que ce n'est pas un objet racine */

  for (i = 0, arbre = arbres ; i < BI_nbr_arbres ; i++, arbre++) {
    if (arbre->adresse == objet)
      return arbre;
  }

  /* Il faut maintenant vérifier que ce n'est pas un objet interne

     PAS ENCORE RÉALISÉ : PAS L'USAGE POUR L'INSTANT... */

  /* Aucun arbre trouvé associé à cet objet */
  (void) fprintf(stderr, "quel_arbre : aucun arbre associé à cet objet !\n");
  return NULL;
}

/* -------- Indique si un arbre est déjà ouvert ou non... -------- */

boolean ARB_EstOuvert(unsigned int numero_arbre)
{
  if ( numero_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : Numéro d'arbre incorrect [%u, max = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, BI_nbr_arbres );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return FAUX;
  }

  if ( VRAI == arbres[ numero_arbre ].drapeaux.ouvert ) return VRAI; /* Arbre marqué comme ouvert ! */

  return FAUX;
}

/* ------------- Affichage d'un message dans un arbre GEM pendant un temps donné ----\
 |   Renvoie <0 en cas de problème, VRAI sinon
 |   Bloque l'utilisation du programme (côté interface) tant qu'il est affiché
 |
 | num_arbre : le numéro de l'arbre à afficher comme message
 | temps : -> si bit 0 mis (impair), on attends ce temps-la (en s) pour effacer + clic/touche
 |         -> sinon (pair) : on attends le temps sans contrôler clic/touche
 |         -> 0 : on ouvre juste, utilisateur ferme quand il n'en a plus besoin (en fenêtre alors)
 |         -> 1 : on ouvre jusqu'à ce que l'utilisateur clique ou appuie sur une touche
 |         -> 2 : on ferme un message précédemment ouvert
 \---------------------------------------------------------------------------------- */

int ARB_message(unsigned int num_arbre, unsigned int temps)
{
  XEvent evenement;
  Window vx_fenetre = None;
  Fenetre *fenetre, *fen_message; /* La fenêtre associée au message */
  unsigned long compteur __attribute__((__unused__)) = 0UL;

  if ( temps == 2 ) { /* On ferme ce qui a été ouvert */
    fenetre = ARB_QuelleFenetre( arbres + num_arbre, FAUX );
    FEN_Detruire( fenetre );
    return VRAI;
  }

  fenetre = FEN_OuvrirArbre( NULL, CLOSER|NAME|MOVER, num_arbre, "Message", -1, -1 );
  if ( fenetre == NULL )
    return -1; /* Ouverture a échouée */
  fen_message = fenetre;

  if ( 0 == temps ) /* On ne fait qu'ouvrir une fenêtre avec le message */
    return VRAI;

  if ( temps > 2 ) { /* C'est que l'on limite à un certain temps */
#ifdef _DEVELOPPE_
    printf("ARB_message : on est avec timer\n");
#endif
    compteur = temps * 1000; /* Temps en millisecondes */

    /* Il faudra le faire, avec un timer => un select */
  } else {
    boolean sortie = FAUX;
    Bool est_filtre;

    do {
      XNextEvent( BI_Display, &evenement );

      est_filtre = XFilterEvent( &evenement, None );
      if ( True == est_filtre ) {
	(void) fprintf( stderr, "%s,%s l.%u : Événement X filtré, ignoré\n",
			__FILE__, __FUNCTION__, __LINE__ );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	continue;
      }


      BI_Fen_redessin = evenement.xany.window; /* Fenêtre concernée -> pour les redessins, ok... */
      if (BI_Fen_redessin != vx_fenetre) {
	vx_fenetre = BI_Fen_redessin;
	fenetre = BIX_QuelleFenetre( BI_Fen_redessin );
      }

      switch (evenement.type)
      {
      case ButtonPress:
      case KeyPress:
	if (temps & 1) { /* Impair = clic ferme la fenêtre */
	  sortie = VRAI;
	}
	break;

      case ConfigureNotify: /* Changement de taille de la fenêtre - XConfigureEvent */
	if (fenetre) { /* Il y a une fenêtre classique associée */
	  int delta_x, delta_y, d_x, d_y;

	  delta_x = evenement.xconfigure.width - fenetre->larg;
	  delta_y = evenement.xconfigure.height - fenetre->haut;
	  d_x = evenement.xconfigure.x - fenetre->x;
	  d_y = evenement.xconfigure.y - fenetre->y;
	  if ( (delta_x == 0) && (delta_y == 0) ) {
	    if ( (d_x == 0) && (d_y == 0) ) {
	      /*	  printf("Configure Notify, sans changement de taille ni position - (%d, %d)\n",
			  evenement.xconfigure.x, evenement.xconfigure.y); */
	    } else {
	      /*	  printf("Déplacement de la fenêtre\n"); */
	      fenetre->x += d_x;
	      fenetre->y += d_y;
	    }
	    return VRAI;
	  }

	  /* On effectue le changement de taille */
	  change_taille_fenetre(fenetre, evenement.xconfigure.width, evenement.xconfigure.height);	  
	  fenetre->x += d_x;
	  fenetre->y += d_y;

	  /* Si fenêtre demande un message spécial en cas de changement de taille, on le fait */  
	  if ( fenetre->msg_masque.taille == VRAI ) {
	    Message message;

	    message.type = WM_SIZED;
	    message.fenetre.application = 0;
	    message.fenetre.en_plus = 0;
	    message.fenetre.handle = 0; /*quel_handle(fenetre);*/
	    message.fenetre.x = fenetre->x;
	    message.fenetre.y = fenetre->y;
	    message.fenetre.larg = evenement.xconfigure.width;
	    message.fenetre.haut = evenement.xconfigure.height;
	    (fenetre->message)(fenetre, &message);
	  }
	}
	break;

      case Expose: /* On affiche la fenêtre = redessin  - XExposeEvent */
	EVNT_redessin(fenetre, (XExposeEvent *) &evenement );
	break;

      case MotionNotify: /* Déplacement du curseur */
	EVNT_pointeur(fenetre, &(evenement.xmotion) );
	break;

      case EnterNotify:  /* Entrée dans une fenêtre */
	if ( fenetre && (evenement.xcrossing.subwindow == None) ) {
	  XRectangle rectangle  __attribute__((__unused__));

	  rectangle.x = (short) (fenetre->w_x);
	  rectangle.y = (short) (fenetre->w_y);
	  rectangle.width = (unsigned short) (fenetre->w_larg);
	  rectangle.height = (unsigned short) (fenetre->w_haut);
	  
	  /* XSetClipRectangles(BI_Display, _PRG_station->gr_contexte, 0, 0, &rectangle, 1, Unsorted); */
	  if ( fenetre->rect ) {
	    fenetre->vx_x = evenement.xcrossing.x - fenetre->w_x;
	    fenetre->vx_y = evenement.xcrossing.y - fenetre->w_y;
	    (fenetre->rect)(fenetre, fenetre->vx_x, fenetre->vx_y, -1, -1);
	  }
	}
	break;

      case LeaveNotify:  /* Sortie d'une fenêtre */
	if ( (fenetre) && (evenement.xcrossing.subwindow == None) ) {
	  if ( fenetre->rect ) {
	    (fenetre->rect)(fenetre, -1, -1, fenetre->vx_x, fenetre->vx_y);
	    fenetre->vx_x = -1; fenetre->vx_y = -1;
	  }
	}
	break;
      }
    } while (sortie == FAUX);
  }

  FEN_Detruire( fen_message ); /* Fini avec ce message */

  return VRAI;
}
