/* Bibliothèque GEM / X_windows étendue
   (C) Emmanuel CURIS, mai 2006

   Primitives de dessin
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct degrade {
  Colormap palette;
  XColor *couleurs;
};

static struct degrade degrades[ BI_NOMBRE_DEGRADES ] = { {None, NULL}, {None, NULL}, {None, NULL}, {None, NULL}, {None, NULL}, {None, NULL}, {None, NULL}, {None, NULL} };

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean BIX_Degrade_DemandePalette(void);
static inline void BIX_Z_MettreClip(Fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip);

/* ~~~~~~~~ Réalisation pratique des fonctions ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Changement des propriétés de dessin ~-~-~-~-~-~-~-~-~ */

/* -------- Changer la couleur du GC. Joue sur le dessin de TOUT -------- */

void X_TRC_Couleur(unsigned int couleur)
{
  if ( couleur < BI_NOMBRE_COULEURS ) {
    /* On installe la couleur dans le GC */
    XSetForeground( BI_Display, BI_GC_programme, BI_Couleurs[ couleur ].pixel );
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Index de couleur incorrect [%u, max = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, couleur, BI_NOMBRE_COULEURS );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
}

/* -------- Créer un dégradé et, si nécessaire, la palette associée ; installer cette palette. Joue sur le dessin de TOUT -------- */

int X_TRC_CreerDegrade(Fenetre *fenetre, unsigned int base)
{
  boolean reussi __attribute__((__unused__)) = VRAI;

  if ( base >= BI_NOMBRE_DEGRADES ) {
    (void) fprintf(stderr, "%s,%s l.%u : Base de dégradé incorrecte [%u, 0 à %u valides]\n",
		   __FILE__, __FUNCTION__, __LINE__, base, BI_NOMBRE_DEGRADES);
    return -1;
  }

  /* Est-ce que la table des couleurs existe ? */
  if ( NULL == degrades[ base ].couleurs ) {
    int valeur, retour;
    unsigned int i;

    /* On réserve la table des couleurs */
    degrades[ base ].couleurs = (XColor *) malloc( 256 * sizeof( XColor ) );
    if ( NULL == degrades[ base ].couleurs ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de créer la table des couleurs\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }

    /* Doit-on créer une palette virtuelle ? */
    if ( VRAI == BIX_Degrade_DemandePalette() ) {
      Window fenetre_X;

      /* Sur quelle fenêtre travaille-t-on ? */
      if ( NULL == fenetre ) {
	AVERTISSEMENT( "Pas de fenêtre X précisé [base : %u]", base );
	fenetre_X = BI_Fen_racine;
      } else {
	fenetre_X = fenetre->handle;
      }

      /* On crée une palette virtuelle */
      degrades[ base ].palette = XCreateColormap( BI_Display, fenetre_X, BI_Visual, AllocNone );
    } else {
      degrades[ base ].palette = BI_Palette;
    }

    /* On installe les couleurs dans la palette */
    switch ( base )
    {
    case BI_DEGRADE_FAUSSES_COULEURS:
      /* Essai de dégradé noir -> bleu -> jaune -> rouge */
      degrades[ base ].couleurs[ 0 ].red   = 65535;
      degrades[ base ].couleurs[ 0 ].green = 65535;
      degrades[ base ].couleurs[ 0 ].blue  = 65535;
      retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs );
      if ( FAUX == retour ) {
	(void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer le noir, fausses couleurs\n",
		       __FILE__, __FUNCTION__, __LINE__);
	reussi = FAUX;
      }

      for ( i = 0 ; i < 255 / 3 ; i++ ) { /* noir -> bleu */
	degrades[ base ].couleurs[ i ].red   = 0;
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = (short int) (65535.0 * (i / 85.0));

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, fausses couleurs\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      for ( ; i < 2 * 85 ; i++) { /* bleu -> jaune */
	degrades[ base ].couleurs[ i ].red   = (short int) (65535.0 * ((i - 85.0) / 85.0));
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = 65535;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, fausses couleurs\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      for ( ; i < 256 ; i++) { /* jaune -> rouge */
	degrades[ base ].couleurs[ i ].red   = 65535;
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = (short int) (65535.0 * ((255.0 - i) / 85.0));

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, fausses couleurs\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_GRIS:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = valeur;
	degrades[ base ].couleurs[ i ].green = valeur;
	degrades[ base ].couleurs[ i ].blue  = valeur;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, niveaux de gris\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_ROUGE:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = valeur;
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = 0;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, rouges\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_VERT:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	    degrades[ base ].couleurs[ i ].red   = 0;
	    degrades[ base ].couleurs[ i ].green = valeur;
	    degrades[ base ].couleurs[ i ].blue  = 0;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, verts\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_BLEU:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = 0;
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = valeur;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, bleus\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_CYAN:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = 0;
	degrades[ base ].couleurs[ i ].green = valeur;
	degrades[ base ].couleurs[ i ].blue  = valeur;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, cyans\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_JAUNE:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = valeur;
	degrades[ base ].couleurs[ i ].green = valeur;
	degrades[ base ].couleurs[ i ].blue  = 0;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, jaunes\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;

    case BI_DEGRADE_MAGENTA:
      for ( i = 0 ; i < 256 ; i++ ) {
	valeur = i * 257;

	degrades[ base ].couleurs[ i ].red   = valeur;
	degrades[ base ].couleurs[ i ].green = 0;
	degrades[ base ].couleurs[ i ].blue  = valeur;

	retour = XAllocColor( BI_Display, degrades[ base ].palette, degrades[ base ].couleurs + i );
	if ( FAUX == retour ) {
	  (void) fprintf(stderr, "%s,%s l.%u : impossible d'allouer la couleur %u, magentas\n",
			 __FILE__, __FUNCTION__, __LINE__, i);
	  reussi = FAUX;
	}
      }
      break;
    }
  }

  /* On installe, si nécessaire, la palette associée */
  if ( ( degrades[ base ].palette != BI_Palette ) && ( fenetre != NULL ) ) {
    OPTION_A_FAIRE( "Assigner la palette à la fenêtre indiquée !" );
  }

  /* Le dégradé est prêt, on a donc terminé */
  return 0;
}

/* -------- Changer la couleur du GC, dans un dégradé dont la base est indiquée. Joue sur le dessin de TOUT -------- */

void X_TRC_CouleurDegrade(unsigned int base, unsigned int niveau)
{
#ifdef _DEVELOPPE_
  if ( base >= BI_NOMBRE_DEGRADES ) {
    (void) fprintf(stderr, "%s,%s l.%u : Base de dégradé incorrecte [%u, 0 à %u valides]\n",
		   __FILE__, __FUNCTION__, __LINE__, base, BI_NOMBRE_DEGRADES);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  if ( NULL == degrades[ base ].couleurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel sans appel préalable à X_TRC_CreerDegrade, ou sans contrôle de succès de l'appel [base : %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, base);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  if ( niveau >= 256 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Niveau de dégradé incorrect [0 à 255, ici %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, niveau);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
#endif

  /* On installe la couleur dans le GC */
  XSetForeground( BI_Display, BI_GC_programme, degrades[ base ].couleurs[ niveau ].pixel );
}

/* -------- Changer l'épaisseur des lignes -------- */

void X_TRC_EpaisseurLigne(unsigned int epaisseur)
{
  XGCValues parametres;

/* Pour les pointeurs & autres, épaisseur nulle utilisable
   Attention, pas pour des dessins plus complexes car peut donner des résultats faux
    (exemple : machines linux/fedora Neoware de Soleil) */
/*   if ( 0 == epaisseur ) { */
/*     AVERTISSEMENT( "Épaisseur de ligne à éviter : %u [affichages faux sur certains serveurs, dont Fedora/SOLEIL]", */
/* 		   epaisseur ); */
/*     epaisseur = 1; */
/*   } */

  parametres.line_width = epaisseur;

  XChangeGC( BI_Display, BI_GC_programme, GCLineWidth, &parametres );
}

/* -------- Changer le mode d'écriture -------- */

void X_TRC_ModeEcriture(unsigned int mode)
{
  XGCValues parametres;

  switch (mode)
  {
  case MD_REMPLACE:
    parametres.function = GXcopy;
    break;

  case MD_TRANSPARENT:
    parametres.function = GXor;
    break;

  case MD_INVERSE:
    parametres.function = GXinvert;
    break;

  case MD_EFFACE:
    parametres.function = GXorInverted;
    break;

  default:
    AVERTISSEMENT( "Mode de tracé inconnu : %u", mode );
    parametres.function = GXset; /* Ça n'existe pas, ce mode -> n'importe quoi ;-) */
  }

  XChangeGC( BI_Display, BI_GC_programme, GCFunction, &parametres );
}

/* -------- Changer le style de tracé des lignes -------- */

void X_TRC_MotifRemplissage(unsigned int motif __attribute__((__unused__)))
{
  OPTION_A_FAIRE( "Définir le motif de remplissage" );
}

/* -------- Changer le style de tracé des lignes -------- */

void X_TRC_StyleLigne(unsigned int style)
{
  XGCValues parametres;

  if ( style <= BI_CONTINU ) {
    parametres.line_style = LineSolid;
  } else {
    BIX_InstallerPointilles( BI_GC_programme, style );

    parametres.line_style = LineOnOffDash;
  }

  XChangeGC( BI_Display, BI_GC_programme, GCLineStyle, &parametres );
}

/* ~-~-~-~-~-~-~-~-~ Routines de dessin fondamentales [sans contrôle] ~-~-~-~-~-~-~-~-~ */

/* Sous X, il n'existe que XDrawArc qui trace aussi bien des ellipses que des cercles ; complets ou partiels
   ==> seule la fonction ci-dessous est utile, les autres sont des macros : cf. X_dessin.h */

inline void X_TRC_ArcEllipse(struct fenetre *fenetre, int x_centre, int y_centre, unsigned int rayon_H, unsigned int rayon_V,
			     int angle_debut, int angle_fin)
{
  Window fenetre_dessin;
  int x_HG, y_HG;
  unsigned int largeur, hauteur;

  if ( ( 0 == rayon_H ) || ( 0 == rayon_V ) ) {
    AVERTISSEMENT( "Rayon absurde --- rH = %u, rv = %u", rayon_H, rayon_V );
    return;
  }

  if ( fenetre != NULL ) {
    fenetre_dessin = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    x_centre += fenetre->w_x; y_centre += fenetre->w_y;
  } else {
    fenetre_dessin = BI_Fen_redessin;
  }
  /* X : on donne les dimensions du rectangle contenant l'ellipse ; centre au centre du rectangle */
  x_HG = x_centre - rayon_H;
  y_HG = y_centre - rayon_V;
  largeur = rayon_H << 1;
  hauteur = rayon_V << 1;
  XDrawArc( BI_Display, fenetre_dessin, BI_GC_programme, x_HG, y_HG, largeur, hauteur, angle_debut * 64, (angle_fin - angle_debut) * 64);
}

inline void X_TRC_Disque(struct fenetre *fenetre, int x_centre, int y_centre, unsigned int rayon)
{
  Window fenetre_dessin __attribute__((__unused__));

  if ( 0 == rayon ) {
    AVERTISSEMENT( "Rayon absurde --- r = %u", rayon );
    return;
  }

  if ( fenetre != NULL ) {
    fenetre_dessin = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    x_centre += fenetre->w_x; y_centre += fenetre->w_y;
  } else {
    fenetre_dessin = BI_Fen_redessin;
  }

  XFillArc( BI_Display, fenetre->handle, BI_GC_programme, x_centre - rayon, y_centre - rayon,
	    rayon << 1, rayon << 1, 0, 360 * 64);
}

inline void X_TRC_Ligne(Fenetre *fenetre, int x0, int y0, int x1, int y1)
{
  Drawable fenetre_X;

  if ( NULL == fenetre ) {
    fenetre_X = BI_Fen_redessin;
  } else {
    fenetre_X = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    x0 += fenetre->w_x; y0 += fenetre->w_y;
    x1 += fenetre->w_x; y1 += fenetre->w_y;
  }

  XDrawLine( BI_Display, fenetre_X, BI_GC_programme, x0, y0, x1, y1 );
}

inline void X_TRC_LigneBrisee(Fenetre *fenetre, unsigned long nombre, Point_Ecran *points)
{
  Drawable fenetre_X;

  if ( ( 0 == nombre ) || ( NULL == points ) ) return;

  if ( NULL == fenetre ) {
    fenetre_X = BI_Fen_redessin;
  } else {
    unsigned long i;

    fenetre_X = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    for ( i = 0 ; i < nombre ; i++ ) {
      points[ i ].x += fenetre->w_x;
      points[ i ].y += fenetre->w_y;
    }
  }

#ifdef _DEVELOPPE_
  /* Contrôle de valeurs aberrantes : certains serveurs X détestent */
  {
    unsigned long i;

    for ( i = 0 ; i < nombre ; i++ ) {
      if ( ( points[ i ].x < 0 ) || ( points[ i ].x > 16000 ) ) {
	(void) fprintf(stderr, "%s,%s l.%u : Valeur de X absurde [%d, points %lu : 0 -> %lu]\n",
		       __FILE__, __FUNCTION__, __LINE__, points[ i ].x, i, nombre - 1 );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
      if ( ( points[ i ].y < 0 ) || ( points[ i ].y > 16000 ) ) {
	(void) fprintf(stderr, "%s,%s l.%u : Valeur de Y absurde [%d, points %lu : 0 -> %lu]\n",
		       __FILE__, __FUNCTION__, __LINE__, points[ i ].y, i, nombre - 1 );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
    }
  }
#endif

  /* On appelle la fonction X adéquate ; possible car Point_Ecran défini comme XPoint ! */
  XDrawLines( BI_Display, fenetre_X, BI_GC_programme, points, nombre, CoordModeOrigin );

  /* On remet les coordonnées d'origine : si par hasard le tableau ressert, il ne faut pas perturber ! */
  if ( fenetre != NULL ) {
    unsigned long i;

    /* Annulation de la correction des coordonnées */
    for ( i = 0 ; i < nombre ; i++ ) {
      points[ i ].x -= fenetre->w_x;
      points[ i ].y -= fenetre->w_y;
    }
  }
}

void X_TRC_Marque(Fenetre *fenetre, int x, int y, unsigned int marque, unsigned int taille)
{
  XGCValues valeurs_GC;

  if ( BI_MARQUE_AUCUNE == marque ) return; /* Aucune marque à afficher ! */
  if ( taille < 4 ) {
    /* Moins de 4 pixel : on ne verra rien, autant mettre un point ! */
    marque = PM_POINT;
  }

  /* On récupère le style et l'épaisseur de ligne actuels */
  XGetGCValues( BI_Display, BI_GC_programme, GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle, &valeurs_GC );

  /* On passe en ligne de style simple */
  XSetLineAttributes( BI_Display, BI_GC_programme, 0, LineSolid, CapButt, JoinMiter );

  if ( NULL == fenetre ) {
    BIX_TracerMarque( BI_Fen_redessin, BI_GC_programme, x, y, marque, taille );
  } else {
    BIX_TracerMarque( fenetre->handle, BI_GC_programme, fenetre->w_x + x, fenetre->w_y + y, marque, taille );
  }

  /* On repasse en ligne définie dans le GC */
  XSetLineAttributes( BI_Display, BI_GC_programme, valeurs_GC.line_width, valeurs_GC.line_style, valeurs_GC.cap_style, valeurs_GC.join_style );
}

inline void X_TRC_Rectangle(Fenetre *fenetre, int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  Drawable fenetre_X __attribute__((__unused__));

  if ( ( 0 == largeur ) || ( 0 == hauteur ) ) {
    AVERTISSEMENT( "Dimensions absurdes --- largeur = %u, hauteur = %u", largeur, hauteur );
    return;
  }

  if ( NULL == fenetre ) {
    fenetre_X = BI_Fen_redessin;
  } else {
    fenetre_X = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    x_HG += fenetre->w_x; y_HG += fenetre->w_y;
  }

  XDrawRectangle( BI_Display, fenetre->handle, BI_GC_programme, x_HG, y_HG, largeur, hauteur );
}

inline void X_TRC_RectanglePlein(struct fenetre *fenetre, int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  Drawable fenetre_X;

  if ( ( 0 == largeur ) || ( 0 == hauteur ) ) {
    AVERTISSEMENT( "Dimensions absurdes --- largeur = %u, hauteur = %u", largeur, hauteur );
    return;
  }

  if ( NULL == fenetre ) {
    fenetre_X = BI_Fen_redessin;
  } else {
    fenetre_X = fenetre->handle;

    /* Correction des coordonnées : zone de travail */
    x_HG += fenetre->w_x; y_HG += fenetre->w_y;
  }

  XFillRectangle( BI_Display, fenetre_X, BI_GC_programme, x_HG, y_HG, largeur, hauteur );
}

/* ~-~-~-~-~-~-~-~-~ Routines de dessin fondamentales [avec contrôle] ~-~-~-~-~-~-~-~-~ */

void X_TRC_Z_Ligne(Fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip,
		   int x0, int y0, int x1, int y1)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace la ligne comme si de rien n'était... */
  X_TRC_Ligne( fenetre, x0, y0, x1, y1 );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

void X_TRC_Z_Cercle(struct fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip, int x, int y, unsigned int rayon)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace le cercle comme si de rien n'était... */
  X_TRC_Cercle( fenetre, x, y, rayon );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

void X_TRC_Z_Disque(struct fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip,
		    int x, int y, unsigned int rayon)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace le disque comme si de rien n'était... */
  X_TRC_Disque( fenetre, x, y, rayon );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

void X_TRC_Z_Ellipse(struct fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip,
		     int x, int y, unsigned int rayon_H, unsigned int rayon_V)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace l'ellipse comme si de rien n'était... */
  X_TRC_Ellipse( fenetre, x, y, rayon_H, rayon_V );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

void X_TRC_Z_Rectangle(Fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip,
		       int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace le rectangle comme si de rien n'était... */
  X_TRC_Rectangle( fenetre, x_HG, y_HG, largeur, hauteur );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

void X_TRC_Z_Texte(Fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip,
		   int x, int y, unsigned int mode_H, unsigned int mode_V, unsigned int angle, const char *texte)
{
  /* On installe le clip */
  BIX_Z_MettreClip( fenetre, x_HG_clip, y_HG_clip, larg_clip, haut_clip );

  /* On trace le texte comme si de rien n'était... */
  X_TRC_Texte( fenetre, x, y, mode_H, mode_V, angle, texte );

  /* On coupe le clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );
}

/* ~-~-~-~-~-~-~-~-~ Aides pour les routines de dessin ~-~-~-~-~-~-~-~-~ */

/* -------- Savoir si le Visual par défaut demande l'utilisation de palettes... -------- */

static boolean BIX_Degrade_DemandePalette(void)
{
  int classe;
  XVisualInfo *infos_visuals = NULL, masque = { NULL, XVisualIDFromVisual( BI_Visual ), 0, 0, 0, 0, 0, 0, 0, 0 };
  int nombre_visuals;

  /* On récupère la liste des Visuals pour l'écran par défaut */
  infos_visuals = XGetVisualInfo( BI_Display, VisualIDMask, &masque, &nombre_visuals );
  if ( NULL == infos_visuals ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun Visual par défaut, absurde !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return FAUX;
  }
  if ( nombre_visuals > 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Il y a %d visuals avec l'Id de celui par défaut, absurde !\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre_visuals);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return FAUX;
  }

  /* On récupère la classe */
  classe = infos_visuals->class;

  /* Plus besoin des infos */
  XFree( infos_visuals );

  /* On déduit de la classe la nécessité d'une palette */
  switch ( classe )
  {
  case StaticGray:
  case StaticColor:
  case PseudoColor:
  case GrayScale:
    return VRAI;

  case DirectColor:
  case TrueColor:
    /* Normalement, il y a toutes les couleurs possibles ! */
    return FAUX;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Classe de Visual inconnue [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, classe);
    return FAUX;
  }
}

int BIX_InstallerPointilles(GC contexte, unsigned int style)
{
  char tirets[ 6 ] = {5, 5, 6, 6, 8, 8};
  int nbr_tirets = 1;

  /* On définit les pointillés */
  switch ( style )
  {
  case BI_CONTINU:
    OPTION_A_FAIRE("Passer en mode continu avec XChangeGC ou XSetLineAttributes");
    return -3;

  case BI_TIRET_LONG:
    tirets[ 0 ] =  4;
    tirets[ 1 ] =  4;
    nbr_tirets = 2;
    break;

  case BI_POINTS:
    tirets[ 0 ] = 2;
    tirets[ 1 ] = 6;
    nbr_tirets = 2;
    break;

  case BI_TIRET_POINT:
    tirets[ 0 ] = 8;
    tirets[ 1 ] = 3;
    tirets[ 2 ] = 2;
    tirets[ 3 ] = 3;
    nbr_tirets = 4;
    break;

  case BI_TIRET_COURT:
    tirets[ 0 ] = 8;
    tirets[ 1 ] = 8;
    nbr_tirets = 2;
    break;

  case BI_TIRET_2POINTS:
    tirets[ 0 ] = 5;
    tirets[ 1 ] = 3;
    tirets[ 2 ] = 2;
    tirets[ 3 ] = 2;
    tirets[ 4 ] = 1;
    tirets[ 5 ] = 3;
    nbr_tirets = 6;
    break;

  default:
    AVERTISSEMENT( "Code de style inconnu [%u]", style );
    return -1;
  }

  XSetDashes( BI_Display, contexte, 0, tirets, nbr_tirets );
  return 0;
}

void BIX_TracerMarque(Drawable fenetreX, GC contexte, int x, int y, unsigned int marque, unsigned int taille)
{
  XSegment segments[ 4 ];
  XPoint   points[ 5 ];

  switch ( marque )
  {
  case BI_MARQUE_AUCUNE:	/* Aucune marque : rien à faire ! */
    return;

  case BI_MARQUE_PLUS:			/* Croix de type classique */
    taille /= 2; /* Puisque mesure de débord de part et d'autre */

    segments[ 0 ].x1 = (short) (x - taille);
    segments[ 0 ].y1 = (short) y;
    segments[ 0 ].x2 = (short) (x + taille);
    segments[ 0 ].y2 = (short) y;

    segments[ 1 ].x1 = (short) x;
    segments[ 1 ].y1 = (short) (y - taille);
    segments[ 1 ].x2 = (short) x;
    segments[ 1 ].y2 = (short) (y + taille);

    XDrawSegments( BI_Display, fenetreX, contexte, segments, 2 );
    break;

  case BI_MARQUE_ASTERISQUE: /* Étoile */
    taille /= 2; /* Puisque mesure de débord de part et d'autre */

    segments[ 0 ].x1 = (short) (x - taille);
    segments[ 0 ].y1 = (short) (y - taille);
    segments[ 0 ].x2 = (short) (x + taille);
    segments[ 0 ].y2 = (short) (y + taille);

    segments[ 1 ].x1 = (short)  x;
    segments[ 1 ].y1 = (short) (y - taille);
    segments[ 1 ].x2 = (short)  x;
    segments[ 1 ].y2 = (short) (y + taille);

    segments[ 2 ].x1 = (short) (x - taille);
    segments[ 2 ].y1 = (short) (y + taille);
    segments[ 2 ].x2 = (short) (x + taille);
    segments[ 2 ].y2 = (short) (y - taille);

    segments[ 3 ].x1 = (short) (x - taille);
    segments[ 3 ].y1 = (short)  y;
    segments[ 3 ].x2 = (short) (x + taille);
    segments[ 3 ].y2 = (short)  y;

    XDrawSegments( BI_Display, fenetreX, contexte, segments, 4 );
    break;

  case BI_MARQUE_CARRE:
    taille /= 2; /* Puisque mesure de débord de part et d'autre */

    XDrawRectangle( BI_Display, fenetreX, contexte, x - taille, y - taille, 2 * taille, 2 * taille );
    break;

  case BI_MARQUE_CROIX:		/* × */
    taille /= 2; /* Puisque mesure de débord de part et d'autre */

    segments[ 0 ].x1 = (short) (x - taille);
    segments[ 0 ].y1 = (short) (y - taille);
    segments[ 0 ].x2 = (short) (x + taille);
    segments[ 0 ].y2 = (short) (y + taille);

    segments[ 1 ].x1 = (short) (x - taille);
    segments[ 1 ].y1 = (short) (y + taille);
    segments[ 1 ].x2 = (short) (x + taille);
    segments[ 1 ].y2 = (short) (y - taille);

    XDrawSegments( BI_Display, fenetreX, contexte, segments, 2 );
    break;

  case BI_MARQUE_LOSANGE: /* Carré tourné de 45° */
    taille /= 2; /* Puisque mesure de débord de part et d'autre */

    points[ 0 ].x = (short) x;
    points[ 0 ].y = (short) y - taille;
    points[ 1 ].x = (short) x + taille;
    points[ 1 ].y = (short) y;
    points[ 2 ].x = (short) x;
    points[ 2 ].y = (short) y + taille;
    points[ 3 ].x = (short) x - taille;
    points[ 3 ].y = (short) y;
    points[ 4 ] = points[0]; /* On connecte */

    XDrawLines( BI_Display, fenetreX, contexte, points, 5, CoordModeOrigin );
    break;

  case BI_MARQUE_CERCLE:	/* Cercle centré sur le point, de diamètre la taille de la marque */
    taille /= 2;
    XDrawArc( BI_Display, fenetreX, contexte, x - taille, y - taille, 2 * taille, 2 * taille, 0, 23040 );
    break;

  case BI_MARQUE_DISQUE:	/* Idem, mais plein */
    XFillArc( BI_Display, fenetreX, contexte, x - taille/2, y - taille/2, taille, taille, 0, 23040 );
    break;

  default:
    if ( marque != BI_MARQUE_POINT ) {
      AVERTISSEMENT("Mode de marque non supporté : %u", marque );
    }
    XDrawPoint( BI_Display, fenetreX, BI_GC_programme, x, y );
  }
}

static inline void BIX_Z_MettreClip(Fenetre *fenetre, int x_HG_clip, int y_HG_clip, unsigned int larg_clip, unsigned int haut_clip)
{
  XRectangle region;

  if ( ( 0 == larg_clip ) || ( 0 == haut_clip ) ) {
    AVERTISSEMENT( "Zone de clip vide ! [L=%u, H=%u]", larg_clip, haut_clip );
    return;
  }

  if ( fenetre != NULL ) {
    /* Correction des coordonnées : zone de travail */
    x_HG_clip += fenetre->w_x; y_HG_clip += fenetre->w_y;
  }

  /* On coupe un éventuel ancien clip */
  XSetClipMask( BI_Display, BI_GC_programme, None );

  /* On installe le clip */
  region.x      = 0;
  region.y      = 0;
  region.width  = larg_clip;
  region.height = haut_clip;
  XSetClipRectangles( BI_Display, BI_GC_programme, x_HG_clip, y_HG_clip, &region, 1, Unsorted );
}
