/**** Bibliothèque mixte GEM / X_windows
  (C) 1998-2003 Emmanuel CURIS

  Gestion des fenêtres : ouverture [mais pas création], paramètres,...
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"

/* -------- Ascenseurs de la fenêtre -------- */

int FEN_ascenseur(Fenetre *fenetre, int asc, int position, int taille)
{
  int x_asc, y_asc;
  unsigned int larg_asc, haut_asc;

  if (fenetre == NULL) return -1;

#ifdef _DEVELOPPE_
  /* On vérifie que la fenêtre appartient bien au chaînage [PAS FORCÉ AVEC LES SOUS-FENÊTRES] */
/*   if ( BI_VerifierChainageFenetre( fenetre ) == FAUX ) */
/*     (void) fprintf(stderr, "Fonction appelante : FEN_ascenseur\n"); */

  if ( position > 1000 )
    (void) fprintf(stderr, "FEN_ascenseur : position de l'ascenseur absurde, %d\n", position);
  if ( taille > 1000 )
    (void) fprintf(stderr, "FEN_ascenseur : taille de l'ascenseur absurde, %d\n", taille);
#endif

  if (asc == ASC_VERTICAL) { /* Cas de l'ascenseur vertical */
    int i;
    unsigned int haut_fond, j;
    Window w;

    if (fenetre->fond_asc_vertical == 0) return -2; /* Pas d'ascenseur */

    XGetGeometry(BI_Display, fenetre->fond_asc_vertical, &w, &i, &i, &j, &haut_fond, &j, &j);
    XGetGeometry(BI_Display, fenetre->bouton_asc_vertical, &w,
		 &x_asc, &y_asc, &larg_asc, &haut_asc, &j, &j);

    if (taille > 0) {
      haut_asc = (unsigned int) ( (((float) taille) / 1000.0) * haut_fond );
    }

    if (haut_asc < 5) haut_asc = 5; /* Pour être sûr d'avoir une taille minimale */

    if (position >= 0) {
      haut_fond -= haut_asc;
      y_asc = (int) ( (((float) position) / 1000.0) * haut_fond );
    }

    XMoveResizeWindow(BI_Display, fenetre->bouton_asc_vertical, x_asc, y_asc, larg_asc, haut_asc);
    return VRAI;
  }

  if (asc == ASC_HORIZONTAL) { /* Cas de l'ascenseur vertical */
    int i;
    unsigned int larg_fond, j;
    Window w;

    if (fenetre->fond_asc_horizontal == 0) return -2; /* Pas d'ascenseur */

    XGetGeometry(BI_Display, fenetre->fond_asc_horizontal, &w, &i, &i, &larg_fond, &j, &j, &j);
    XGetGeometry(BI_Display, fenetre->bouton_asc_horizontal, &w,
		 &x_asc, &y_asc, &larg_asc, &haut_asc, &j, &j);

    if (taille > 0) {
      larg_asc = (unsigned int) ( (((float) taille) / 1000.0) * larg_fond );
    }

    if (larg_asc < 5) larg_asc = 5; /* Pour être sûr d'avoir une taille minimale */

    if (position >= 0) {
      larg_fond -= larg_asc;
      x_asc = (int) ( (((float) position) / 1000.0) * larg_fond );
    }

    XMoveResizeWindow(BI_Display, fenetre->bouton_asc_horizontal, x_asc, y_asc, larg_asc, haut_asc);
    return VRAI;
  }

  return -2; /* Ascenseur de type inconnu */
}

/* -------- Barre d'informations -------- */

int FEN_infos(Fenetre *fenetre, const char *infos)
{
  OBJECT obj_infos = { -1, -1, -1, G_INFOS, NONE, NORMAL, {NULL}, 0, 0, 1, 1 };

  if ( NULL == fenetre ) return -1;
  if ( infos == NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FEN_infos : pas de chaîne d'infos [Fenêtre : %s]\n", fenetre->titre);
#endif
    return -2;
  }

  /* La fenêtre doit-elle avoir des infos ? */
  if ( 0 == ( fenetre->attributs & INFO ) ) {
    Fenetre *mere;

    /* Pas de barre d'info ==> on cherche pour la mère */
    mere = FEN_FenetreMere( fenetre );
    if ( mere != fenetre ) {
      return FEN_infos( mere, infos );
    }

    AVERTISSEMENT( "Infos indiquées pour une fenêtre sans barre d'info ! [%s, fenêtre %s]", infos, fenetre->titre );
    return -5;
  }
  free( fenetre->infos ); /* On libère la précédente */
  fenetre->infos = strdup( infos ); /* On mémorise une copie pour éviter des modifs imprévues */

  if ( fenetre->barre_d_infos == None ) return -3; /* Pas de barre d'infos créée ! */
  if ( fenetre->handle ==  None ) return 0;
  if ( fenetre->drapeaux.ouverte == FAUX ) return 0;

  obj_infos.ob_spec.texte = fenetre->infos;
  OBJ_dessine( fenetre->barre_d_infos, &obj_infos );

  /* Pas de problème particulier */
  return 0;
}

/* -------- Titre (nom) de la fenêtre -------- */

int FEN_titre(Fenetre *fenetre, const char *titre)
{
#ifndef _utf8_
  XTextProperty titre_X;
#endif

  if ( NULL == fenetre ) return -1;
  if ( NULL == titre   ) return -3;

  free( fenetre->titre ); fenetre->titre = NULL;

  fenetre->titre = strdup( titre ); /* On garde une copie locale, car si on efface l'original, gênant ! */

  if ( 0 == fenetre->handle ) return -2;

#ifdef _utf8_
# ifdef X_HAVE_UTF8_STRING
  /* On installe la chaîne en UTF-8, pour le nom et pour l'icône... */
  printf( "\n\nOn installe le nom de fenêtre UTF-8 %s\n\n\n", titre );
  Xutf8SetWMProperties( BI_Display, fenetre->handle, titre, titre,
			NULL, 0, NULL, NULL, NULL );
# else
  XStoreName( BI_Display, fenetre->handle, titre );
# endif
#else
  if ( XStringListToTextProperty( (char **) &titre, 1, &titre_X ) != 0 ) {
    XSetWMName( BI_Display, fenetre->handle, &titre_X );

    /* Plus besoin de la propriété : on la libère */
    XFree( titre_X.value ); titre_X.value = NULL;
  } else {
    (void) fprintf( stderr, "%s,%s l.%u : Problème dans XStringListToTextProperty() ==> on passe sur XStoreName()\n",
		    __FILE__, __FUNCTION__, __LINE__);
    XStoreName( BI_Display, fenetre->handle, titre );
  }
#endif

  /* Terminé */
  return 0;
}

/* -------- Passer la fenêtre au premier plan -------- */

Fenetre *FEN_premierplan(Fenetre *fenetre)
{
  XWindowAttributes attr_arbre;
  XWindowChanges config_fenetre = { 0, 0, 0, 0, 0, None, TopIf };

  if ( NULL == fenetre ) return NULL;
  if ( None == fenetre->handle ) { /* Fenêtre X inexistante */
    (void) fprintf(stderr, "%s,%s l.%u : Fenêtre X inexistante. Réouvrir ?\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On utilise, si WM compatible, les messages avancés
     Si présent, fonctionne => inutile d'essayer le reste */
  if ( ( VRAI == BI_Freedesktop_OK ) && ( _NET_ACTIVE_WINDOW != None ) ) {
    XClientMessageEvent message;

    message.type = ClientMessage;
    message.serial = 0;
    message.send_event = True;
    message.display = BI_Display;
    message.window = fenetre->handle;
    message.message_type = _NET_ACTIVE_WINDOW;
    message.format = 32;
    message.data.l[ 0 ] = 0;	/* 1 = envoyé par l'application mais ne marche pas */
    message.data.l[ 1 ] = 0;	/* => on laisse 0 "ancien protocole" => fonctionne */
    message.data.l[ 2 ] = 0;
    message.data.l[ 3 ] = 0;
    message.data.l[ 4 ] = 0;

    XSendEvent( BI_Display, BI_Fen_racine, False, SubstructureRedirectMask, (XEvent *) &message );

    return fenetre;
  }

  /* Si absent : on le fait « à la main », en espérant que cela fonctionnera... */

  /* Essai : on ferme/réouvre la fenêtre pour être sûr qu'elle soit sur l'écran courant ?
     Semble ne pas fonctionner !

     XUnmapWindow( BI_Display, fenetre->handle );
     XMapWindow( BI_Display, fenetre->handle );
  */

  /* XMapRaised ? */

  /* On monte la fenêtre au premier plan */
  XRaiseWindow( BI_Display, fenetre->handle );

  /* On y met le clavier... mais attention, risque d'erreur X quand la fenêtre n'est pas visible
     ==> comment savoir si la fenêtre est sur le même écran ?
     Essai 0 : fermeture/réouverture, ne fonctionne pas
     Essai 1 : la fenêtre racine ==> identique dans les deux cas
     Essai 2 : attributs des fenêtres => flag map_state OK */
  XGetWindowAttributes( BI_Display, fenetre->handle, &attr_arbre );
  if ( attr_arbre.map_state == IsViewable ) { /* On voit bien la fenêtre */
    XSetInputFocus( BI_Display, fenetre->handle, RevertToParent, CurrentTime /* Il faudrait un temps correct, mais...*/ );
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : À FAIRE --- Faire revenir la fenêtre map_state = %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, attr_arbre.map_state);
  }

  /* On utilise la version avec WM pour s'assurer que cela fonctionne...
     ==> fonctionne toujours pas sous KDE :( */
  XReconfigureWMWindow( BI_Display, fenetre->handle, BI_Num_ecran, CWStackMode, &config_fenetre );

  /* Terminé... Normalement, la fenêtre est au premier plan ! */
  return fenetre;
}

/* -------- Forcer le redessin d'une fenêtre -------- */

boolean FEN_redessin(Fenetre *fenetre, unsigned int portion)
{
  XEvent redessin;
  unsigned int i;

  if (fenetre == NULL) /* Fenêtre n'existe pas... */
    return FAUX;

  if (fenetre->handle == None) { /* Fenêtre X n'existe pas */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : La fenêtre X n'existe pas pour %s\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->titre);
#endif
    return FAUX;
  }

  if (fenetre->drapeaux.ouverte == FAUX) { /* Fenêtre pas ouverte */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FEN_redessin(%p, %u) : fenêtre fermée !\n", (void *) fenetre, portion);
#endif
    return FAUX;
  }

  if ( portion == 0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : pas de redessin demandé\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return FAUX; /* On ne veut rien redessiner ! */
  }

  if ( VRAI == fenetre->drapeaux.rds_dem ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FEN_redessin(%p, %u) : fenêtre attend déjà un redessin !\n", (void *) fenetre, portion);
#endif
    return FAUX;
  }

#ifdef _DEVELOPPE_ /* On vérifie que la fenêtre appartient bien au chaînage [NON car sous-fenêtres !]*/
/*   if ( BI_VerifierChainageFenetre( fenetre ) == FAUX ) */
/*     fprintf(stderr, "Fonction appelante : FEN_redessin\n"); */
#endif

  redessin.type = Expose; /* Message de redessin unique */
  redessin.xexpose.count  = 0;

  switch ( fenetre->type )
  {
  case BI_FENETRE_GL:		/* Version OpenGL */
    fenetre->open_gl->refaire = VRAI; /* Si l'on demande le redessin OpenGL, il y a une raison */

  case BI_FENETRE_SIMPLE:	/* Fenêtres simples */
  case BI_FENETRE_ARBRE:
  case BI_FENETRE_IMAGE:
    redessin.xexpose.window = fenetre->handle;
    redessin.xexpose.x = fenetre->w_x;
    redessin.xexpose.y = fenetre->w_y;
    redessin.xexpose.width  = fenetre->w_larg;
    redessin.xexpose.height = fenetre->w_haut;
    XSendEvent( BI_Display, fenetre->handle, False, 0, &redessin );
    break;

  case BI_FENETRE_SIMPLE_ARBRE:	/* Arbre + utilisateur : on ne redessine que la partie utilisateur ! */
    if ( portion & BI_RDS_UTIL ) FEN_redessin( fenetre->ss_fenetres[ 1 ], portion );
    break;

  case BI_FENETRE_GL_SIMPLE:    /* Open GL + utilisateur */
    if ( portion & BI_RDS_GL   ) FEN_redessin( fenetre->ss_fenetres[ 0 ], portion );
    if ( portion & BI_RDS_UTIL ) FEN_redessin( fenetre->ss_fenetres[ 1 ], portion );
    break;

  case BI_FENETRE_GL_ARBRE_SIMPLE: /* Arbre + Open GL + utilisateur */
    if ( portion & BI_RDS_GL   ) FEN_redessin( fenetre->ss_fenetres[ 1 ], portion );
    if ( portion & BI_RDS_UTIL ) FEN_redessin( fenetre->ss_fenetres[ 2 ], portion );
    break;

  case BI_FENETRE_MULTIPLE:	/* Fenêtre multiple */
    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      if ( portion & BI_RDS_FILLES ) FEN_redessin( fenetre->ss_fenetres[ i ], portion );
    }
    break;

  case BI_FENETRE_ONGLETS:
    if ( portion & BI_RDS_ONGLETS ) {
      struct fen_onglets *infos_onglets;

      infos_onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
      if ( NULL == infos_onglets ) return FAUX;

      for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
	redessin.xexpose.window = infos_onglets->onglets[ i ];
	redessin.xexpose.x = 0;
	redessin.xexpose.y = 0;
	redessin.xexpose.width  = fenetre->w_larg;
	redessin.xexpose.height = BI_haut_car;
	XSendEvent( BI_Display, infos_onglets->onglets[ i ], False, 0, &redessin );
      }
    }
    if ( portion & BI_RDS_FILLES ) {
      int onglet_actif;

      onglet_actif = BI_QuelOngletEstActif( fenetre );
      if ( onglet_actif >= 0 ) {
	FEN_redessin( fenetre->ss_fenetres[ onglet_actif ], portion );
      }
    }
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Type de fenêtre inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->type );
    return FAUX;
  }

  fenetre->drapeaux.rds_dem = VRAI; /* On a demandé un redessin */
  return VRAI;
}

/* -------- Gestion des onglets -------- */

int BI_QuelOngletEstActif(Fenetre *fenetre)
{
  struct fen_onglets *infos_onglets;

  if ( NULL == fenetre ) return -1;
  if ( BI_FENETRE_ONGLETS != fenetre->type ) {
    (void) fprintf(stderr, "%s,%s l.%u : La fenêtre indiquée [%s] ne contient pas d'onglets [type %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->titre, fenetre->type );
    return -2;
  }

  infos_onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos_onglets ) {
    AVERTISSEMENT( "Pas d'infos X pour les onglets [fenêtre %s]", fenetre->titre );
    return -3;
  }

  if ( infos_onglets->onglet_actif >= fenetre->nbr_ss_fenetres ) {
    (void) fprintf(stderr, "%s,%s l.%u : Numéro d'onglet actif absurde [%u, mais %lu sous-fenêtres !]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos_onglets->onglet_actif, fenetre->nbr_ss_fenetres);
    return -4;
  }

  return infos_onglets->onglet_actif;
}

int BI_MemoriserOngletActif(Fenetre *fenetre, unsigned int nouvel_actif)
{
  struct fen_onglets *infos_onglets;
  int ancien_actif;

  /* En profite pour contrôler fenetre, la présence d'infos,... */
  ancien_actif = BI_QuelOngletEstActif( fenetre );
  if ( ancien_actif < 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans BI_QuelOngletEstActif() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, ancien_actif);
    return -1;
  }

  /* On récupère les infos */
  infos_onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  /* ne peut pas être NULL, sinon détecté dans BI_QuelOngletEstActif() */

  /* On mémorise le nouvel onglet actif */
  infos_onglets->onglet_actif = nouvel_actif;

  /* On redessine les onglets */
  FEN_redessin( fenetre, BI_RDS_ONGLETS );

  /* Terminé, sans souci */
  return 0;
}


#ifdef _DEVELOPPE_
static const char *types_fenetres[] = {
  "Simple",
  "Arbre",
  "Image",
  "Open-GL",
  "Non-défini",
  "Non-défini",
  "Non-défini",
  "Non-défini",
  "Non-défini",
  "Non-défini",
  "Multiple",
  "Simple + Arbre",
  "Open-GL + Arbre",
  "Open-GL + Arbre + Simple",
  "Open-GL + Simple",
  "Onglets"
};

void BI_AfficherFenetre(Fenetre *fenetre)
{
  fprintf( stderr, " * Suivante : %p\n", (void *) fenetre->suivante );
  fprintf( stderr, " * Handle X : %lu\n", (unsigned long) (fenetre->handle) );
  fprintf( stderr, " * Type : %d [%s]\n", fenetre->type, types_fenetres[ fenetre->type ] );
  fprintf( stderr, " * Attributs : %d\n", fenetre->attributs );
  fprintf( stderr, " * Position dans la mère : %d\n", fenetre->position );
  fprintf( stderr, " * Position : (%d, %d) - %d * %d\n", fenetre->x, fenetre->y,
	   fenetre->larg  , fenetre->haut );
  fprintf( stderr, " * Travail : (%d, %d) - %d * %d\n", fenetre->w_x, fenetre->w_y,
	   fenetre->w_larg, fenetre->w_haut);
  fprintf( stderr, " * Nom : %p / %s\n", (const void *) (fenetre->titre), fenetre->titre );
  fprintf( stderr, " * Infos : %p / %s\n", (const void *) (fenetre->infos), fenetre->infos );
  fprintf( stderr, " * Arbre : numéro %u soit %p\n", fenetre->numero_arbre, (void *) fenetre->arbre );
  fprintf( stderr, " * Sous-fenêtres : %lu [%p]\n", fenetre->nbr_ss_fenetres, (void *) fenetre->ss_fenetres );
}
#endif
