/* Bibliothque GEM pour X-Windows
   (C) 1998, Emmanuel CURIS

   Bibliothque de formulaires (FORM lib)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "bib_interface.h"
#include "txt_utils.h"

int form_do(OBJECT *adresse, unsigned int prem_edit)
{
  unsigned int num_arbre;

  if (adresse == NULL)
    return -1;

  for (num_arbre = 0 ; arbres[num_arbre].adresse != adresse ; num_arbre++) {
    if (num_arbre >= BI_nbr_arbres) { /* On a dpass */
      /* Il faudrait voir  initialiser un nouvel arbre = arbre 3 p. ex. avec pour l'utiliser */
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "form_do : adresse d'arbre incorrecte - %p\n", (void *) adresse);
#endif
      return -2;
    }
  }

#ifdef _DEVELOPPE_
  if ( prem_edit > arbres[ num_arbre ].nbr_objets ) {
    (void) fprintf(stderr, "form_do : le premier objet ditable indiqu n'est pas correct\n");
  }
  if ( ( prem_edit != ROOT ) && ( ( adresse[ prem_edit ].ob_flags & EDITABLE ) == 0 ) ) {
    (void) fprintf(stderr, "form_do : le premier objet ditable indiqu n'est pas marqu comme ditable dans l'arbre\n");
  }
#endif

  return ARB_Bloquant( num_arbre, "Arbre bloquant", NULL, num_arbre );
}

int form_dial(int dtype, int ix, int iy, int iw, int ih, int x, int y, int w, int h)
{
  Fenetre *fenetre;

  switch (dtype)
  {
  case FMD_FINISH:
    for (fenetre = BI_premiere_fenetre ; fenetre ; fenetre = fenetre->suivante)
      FEN_redessin(fenetre, BI_RDS_TOUT);

  case FMD_START: /* Rien  faire */
    return 1;

  case FMD_GROW:
    printf("form_dial :  FAIRE, bote croissante (%d,%d  %d%d) -> (%d,%d %d%d)\n",
	   ix, iy, iw, ih, x, y, w, h);
    return 1;

  case FMD_SHRINK:
    printf("form_dial :  FAIRE, bote dcroissante (%d,%d  %d%d) -> (%d,%d %d%d)\n",
	   ix, iy, iw, ih, x, y, w, h);
    return 1;

  default:
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "form_dial : commande inconnue = %d\n", dtype);
#endif
    return 0;
  }
}

/* -------- Botes d'alerte -------- */

int form_alert(int defbut, const char *chaine)
{
  char *pos_debut, *texte;
  const char *pos_fin;
  char **boutons;
  int num_icone, retour;
  unsigned int i;
  unsigned long nbr_boutons;
  BI_Boite_Alerte message = { ERR_ICONE_QUESTION, "Avertissement", NULL, 0, 0, 0, { NULL, NULL, NULL, NULL, NULL } };

  OPTION_A_FAIRE( "Appeler directement BI_ERR_alerte" );

  /* 1. On analyse la chane pour trouver icne, texte et boutons */
  /* a. Icne */
  if ( NULL == chaine ) {
    (void) fprintf(stderr, "%s,%s l.%u : pas de message\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( *chaine != '[' ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s : format du message incorrect, le premier caractre n'est pas [ \"%s\"\n", __FUNCTION__, chaine);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -2;
  }
  num_icone = TXT_CNV_vNombre_L( chaine + 1, &pos_fin );
  if ( NULL == pos_fin ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s : code icone non termin \"%s\"\n", __FUNCTION__, chaine);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -12;
  }

  if ( 0 == num_icone )      message.icone = ERR_ICONE_AUCUN;
  else if ( 1 == num_icone ) message.icone = ERR_ICONE_EXCLAME;
  else if ( 2 == num_icone ) message.icone = ERR_ICONE_QUESTION;
  else if ( 3 == num_icone ) message.icone = ERR_ICONE_STOP;
  else {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "form_alert : Attention, type d'icone inconnu sous TOS [%d]\n", num_icone);
#endif
    message.icone = ERR_ICONE_DISQUE;
  }

  /* b. Texte */
  pos_debut = strchr( pos_fin, '[' );
  if ( NULL == pos_debut ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s : pas de texte \"%s\"\n", __FUNCTION__, chaine);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -3;
  }
  pos_fin = strchr( pos_debut + 1, ']' );
  if ( NULL == pos_fin ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s : texte non termin \"%s\"\n", __FUNCTION__, chaine);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -4;
  }
  texte = strdup( pos_debut + 1 );
  if ( NULL == texte ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de copier le texte !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -5;
  }
  texte[ pos_fin - pos_debut - 1 ] = 0; /* Pour ne pas avoir le ] final */
  message.texte = texte;

  /* c. Boutons */
  pos_debut = strchr( pos_fin, '[' );
  if ( NULL == pos_debut ) {
    free( texte ); message.texte = NULL; texte = NULL;
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s : pas de boutons \"%s\"\n", __FUNCTION__, chaine);
#endif
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -6;
  }
  boutons = TXT_DecomposerChaine( pos_debut + 1, '|', &nbr_boutons );
  if ( NULL == boutons ) {
    free( texte ); message.texte = NULL; texte = NULL;
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des boutons !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -7;
  }
  *(boutons[ nbr_boutons - 1 ] + strlen( boutons[ nbr_boutons - 1 ] ) - 1 ) = 0; /* On se dbarrasse du ] final */
  message.nbr_boutons = ( nbr_boutons > 5 ) ? ( 5 ) : ( nbr_boutons );
  for ( i = 0 ; i < message.nbr_boutons ; i++ ) { /* Pas plus de 5 boutons */
    message.boutons[ i ] = boutons[ i ];
  }
  message.bouton_ok = defbut;

  /* On appelle la bote d'alerte */
  retour = BI_ERR_alerte( NULL, &message, VRAI );

  /* On libre tout */
  free( texte ); message.texte = NULL; texte = NULL;
  free( boutons[ 0 ] );		/* Le texte des boutons est copi dans le premier lment ! Ensuite, pointeurs dans le texte */
  free( boutons ); boutons = NULL;

  /* On renvoie le bouton cliqu */
  return retour;
}

int form_error(int errnum)
{
  char texte[100];

  OPTION_A_FAIRE( "Appeler directement BI_ERR_alerte" );

  switch (errnum)
  {
  case 2:
  case 3:
  case 18:
    return form_alert(1, "[2][Cette application ne peut pas|trouver l'objet auquel vous|essayez d'acc"e_AIGU"der.][Confirmer]");

  case 4:
    return form_alert(1, "[1][Cette application n'a pas|suffisamment de place pour"
		            "|ouvrir un autre document.|Fermez un document que vous|n'utilisez pas.]"
		            "[Confirmer]");

  case 5:
    return form_alert(1, "[1][Un objet ayant ce nom existe|d"e_AIGU"j"a_GRAVE", ou bien il n'autorise"
		            "|pas l'"e_AIGU"criture.][Confirmer]");

  case 8:
  case 10:
  case 11:
    return form_alert(1, "[1][M"e_AIGU"moire insuffisante pour|l'application en cours.][Confirmer]");

  case 15:
    return form_alert(1, "[1][Le lecteur sp"e_AIGU"cifi"e_AIGU"|n'existe pas.][Annuler]");

  default:
    sprintf(texte, "[3][Erreur TOS #%d.][Annuler]", errnum);
    return form_alert(1, texte);
  }
}

/* -------- Centrer un arbre  l'cran - sans grand intrt en gnral :-) ---- */

int form_center(OBJECT *adresse, int *x, int *y, int *larg, int *haut)
{
  if (adresse == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "form_button : pas d'arbre transmis\n");
#endif
    return -1;
  }

  *larg = OB_WIDTH(adresse, ROOT);
  *haut = OB_HEIGHT(adresse, ROOT);

  *x = ( WidthOfScreen(BI_Ecran) - *larg ) >> 1;
  *y = ( HeightOfScreen(BI_Ecran) - *haut ) >> 1;

  /* Et on ne touche surtout pas  ob_x et ob_y car RELATIF  LA FENTRE -> toujours en 0,0... */
  return 0;
}

int form_keybd(OBJECT *arbre, unsigned int objet, unsigned int objet_suivant, unsigned int touche,
	       unsigned int *nv_suivant, unsigned int *nv_touche)
{
  printf("form_keybd : FONCTION  FAIRE -- form_keybd(%p, %u, %u, %u [%c], %p, %p)\n",
	 (void *) arbre, objet, objet_suivant, touche, touche, (void *) nv_suivant, (void *) nv_touche);
  return 0;
}

int form_button(OBJECT *adresse, unsigned int objet, int n_clics, unsigned int *pnxt_obj)
{
  unsigned int num_arbre;
  int i;

  if (adresse == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "form_button : pas d'arbre transmis\n");
#endif
    return -1;
  }

  if ( OB_STATE(adresse, objet) & DISABLED ) /* Object inactif : rien  faire */
    return 0;

  for (num_arbre = 0 ; arbres[num_arbre].adresse != adresse ; num_arbre++)
    if (num_arbre >= BI_nbr_arbres) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "form_button : pas d'arbre  l'adresse %p\n", (void *) adresse);
#endif
      return -2;
    }

  if ( objet >= arbres[num_arbre].nbr_objets ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "form_button : pas d'objet %d dans cet arbre (qui en contient %d)\n",
		   objet, arbres[num_arbre].nbr_objets);
#endif
    return -3;
  }

  *pnxt_obj = objet; /* cas le plus frquent */

  for ( i = n_clics ; i > 0 ; i-- ) {
    OBJ_choisit(num_arbre, objet, ECHANGE);
  }

  return 0;
}
