/* Bibliothèque GEM / X_windows étendue
   (C) Emmanuel CURIS, avril 1998
   Dernière modification : 30 octobre 1998

   Gestion des objets « GEM »

   30 octobre 1998
     * Essai de dessin des boutons désactivés sur Alpha : marche bien mieux !
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#ifdef _DEVELOPPE_
# include <math.h>
# ifdef _DEC_UNIX_ /* Pas envie de refaire ici les différentes versions de fp_class... */
#  include <fp_class.h>
# endif
#endif

#include "txt_utils.h"
#include "bib_interface.h"

#include "X_motifs.h"

/* -------- Fonctions locales -------- */

static int OBJ_MetValeur_Editable_L(unsigned int arbre, unsigned int objet, long valeur);

/* ~~~~~~~~ Réalisation pratique des fonctions ~~~~~~~~ */

/* ~~~~~~~~~~~~~~~~ Écriture de valeurs diverses dans les objets ~~~~~~~~~~~~~~~~ */

/* -------------------- Installation d'une chaîne de caractères ---------------------------------\
 |   Ne renvoie rien
 |
 | arbre   : arbre contenant l'objet à modifier
 | objet   : numéro de l'objet à modifier
 | l_max   : nombre maximal de caractères à copier (pour les chaînes, boutons,...)
 | texte   : pointeur sur la chaine à installer
 | mode    : TXT_COPIER   ==> on copie la chaîne à la place de l'ancienne                        |
 |           TXT_REMPLACE ==> on change l'adresse de l'objet pour qu'il pointe sur texte         |
 \---------------------------------------------------------------------------------------------- */

void met_chaine(unsigned int arbre, unsigned int objet, unsigned long l_max, const char *texte, int mode)
{
  OBJECT *adresse;
  char *chaine;
  unsigned long i;
  int retour;

  if (texte == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Attention, texte NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    texte = "> texte = null <";
#else
    texte = "";
#endif
/*     FIN_ANTICIPEE( FIN_ERREUR_MINEURE ); */
  }
  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Arbre ou objet inexistant\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  adresse = arbres[arbre].adresse;

  if ( ( OB_TYPE(adresse, objet) == G_TXT_INFINI ) ||
       ( G_NOMBRE == OB_TYPE(adresse, objet) ) ) { /* Texte infini : traitement spécial... */
    Texte_Infini *txt_infini;

    l_max = strlen(texte); /* Puisqu'il est infini, de toute façon... */

    if ( G_TXT_INFINI == OB_TYPE(adresse, objet) ) {
      txt_infini = adresse[ objet ].ob_spec.txt_infini;
      if ( NULL == txt_infini ) {
	(void) fprintf(stderr, "%s, %s l.%u : Texte infini NULL\n",
		       __FILE__, __FUNCTION__, __LINE__);
	FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
	return;
      }
    } else {
      txt_infini = &( adresse[ objet ].ob_spec.nombre->texte );
    }

    if ( l_max >= txt_infini->max_cars ) { /* Il faut rallonger le texte (>= = le 0 final...) */
      /* Note : si aucun réservé, txt_infini->texte = NULL et realloc <=> malloc donc Ok */
      char *nv_texte = (char *) realloc(txt_infini->texte, l_max + 10);
      if (nv_texte != NULL) {
	txt_infini->texte = nv_texte;
	txt_infini->max_cars = l_max + 9; /* Réservation a réussi */
	txt_infini->texte[txt_infini->max_cars] = 0; /* On est sûr d'avoir un 0 final */
      } else {
	(void) fprintf(stderr, "met_chaine : mémoire insuffisante -- nouveau texte infini\n");
	l_max = txt_infini->max_cars;
      }
    }

    if (txt_infini->max_cars == 0 ) /* Aucun caractère disponible */
      return;

    /* On copie maintenant les l_max cars dans la chaîne réservée */
    strncpy(txt_infini->texte, texte, l_max + 1); /* Le +1 pour être sûr d'avoir le 0 final */
    txt_infini->prem_car = 0; /* On réaffiche depuis le début du texte */
    BI_DessinerObjet(arbre, objet); /* On force le redessin */
    if ( objet == arbres[ arbre ].edit_courant ) { /* Il faut aussi adapter le curseur */
      BI_PlacerCurseur( arbres + arbre, BI_InitialiserCurseur( adresse + objet ) );
    }
    return;
  }

  if (mode == TXT_REMPLACE) { /* On se contente de changer l'adresse */
    switch ( OB_TYPE(adresse, objet) )
    {
    case G_TEXT:     /* Genre texte, non éditable */
    case G_BOXTEXT:
      adresse[objet].ob_spec.tedinfo->te_ptext.constant = texte;
      break;

    case G_FTEXT:    /* Genre texte, éditable : ATTENTION, on modifie en fait
			LE MASQUE et non le texte édité */
    case G_FBOXTEXT:
      OPTION_A_FAIRE( "Stockage d'un nouveau masque dans un objet éditable" );
/*       adresse[objet].ob_spec.tedinfo->te_ptmplt = texte; */
      break;

    case G_BUTTON: /* Genre texte direct */
    case G_STRING:
    case G_TITLE:
      if ( FAUX == arbres[ arbre ].drapeaux.interne ) {
	/* La chaine a été réservée avec malloc() */
	free( adresse[objet].ob_spec.txt_malloc );
	adresse[objet].ob_spec.txt_malloc = strdup( texte );
      } else {
	/* Chaine statique : on peut la mettre telle qu'elle */
	adresse[objet].ob_spec.texte = texte;
      }
      break;
    }

    BI_DessinerObjet(arbre, objet); /* On force le redessin */
    return;
  }

  /* On fait une recopie */
  switch ( OB_TYPE(adresse, objet) )
  {
  case G_TEXT:     /* Genre texte avec curseurs */
  case G_FTEXT:
  case G_BOXTEXT:
  case G_FBOXTEXT:
    chaine = adresse[objet].ob_spec.tedinfo->te_ptext.editable; /* Le texte */
    l_max  = adresse[objet].ob_spec.tedinfo->te_txtlen - 1; /* Longueur maximale */
    break;

  case G_STRING:   /* Genre chaîne ou bouton   */
  case G_BUTTON:
  case G_TITLE:
    if ( VRAI == arbres[ arbre ].drapeaux.interne ) {
      (void) fprintf(stderr, "%s,%s l.%u : Écriture dans un texte statique [arbre interne]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }
    chaine = adresse[ objet ].ob_spec.txt_malloc;
    if ( ( NULL == chaine ) && ( l_max > 0 ) ) {
      /* On a perdu la chaîne : on va donc en refaire une, de la taille maximale indiquée */
      adresse[ objet ].ob_spec.txt_malloc = (char *) calloc( l_max, sizeof( char ) );
      chaine = adresse[ objet ].ob_spec.txt_malloc;
    }
    break;
    
  default:        /* Genre inconnu, donc pas de recopie - on quitte */
    return;
  }

  if ( l_max > strlen(texte) ) { /* Aucun problème */
    l_max = strlen(texte);
  }

  if ( NULL == chaine ) {
    (void) fprintf(stderr, "%s, %s l.%u : chaine = NULL, impossible d'y copier le texte !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  for (i = 0 ; i < l_max ; i++)
    chaine[i] = texte[i];
  chaine[l_max] = 0; /* Chaîne doit finir par un octet nul */
  
  if ( (OB_TYPE(adresse, objet) == G_FTEXT) || (OB_TYPE(adresse, objet) == G_FBOXTEXT) ) { /* Objet éditable */
    for ( ; i < (unsigned) (adresse[objet].ob_spec.tedinfo->te_txtlen - 1) ; i++)
      chaine[i] = 0; /* On termine par des 0 pour être tranquille */

    if ( objet == (unsigned int) arbres[arbre].edit_courant ) {
      BI_PlacerCurseur( arbres + arbre, BI_InitialiserCurseur( arbres[ arbre ].adresse + objet) );
    }
  }

  if ( arbres[ arbre ].drapeaux.ouvert == VRAI) {
    BI_DessinerObjet( arbre, objet );
  }
}

/* -------------- Met une valeur "long" dans un champ GEM ----------
   Note : code aussi pour met_valeur avec la macro qui va bien...

   ATTENTION : aucune vérification de débordement de capacité pour les boutons et autres...
*/

void met_lvaleur(unsigned int arbre, unsigned int objet, long valeur)
{
  OBJECT *adresse;
  char *chaine;
  int type, retour;

  adresse = arbres[arbre].adresse;

  type = (arbres[arbre].adresse)[objet].ob_type;

  switch ( type )
  {
  case G_TXT_INFINI:
    chaine = TXT_genere_chaine("%l", valeur );
    met_chaine( arbre, objet, 0, chaine, TXT_COPIER );
    free( chaine );
    return;

  case G_BUTTON:
  case G_STRING:
#ifdef _DEVELOPPE_
    printf("met_lvaleur : écriture d'un chiffre dans un bouton/une chaîne\n");
    printf("              pas de vérification de débordement possible -> c'est dangereux !\n");
#endif
    chaine = adresse[objet].ob_spec.txt_malloc;
    sprintf(chaine, "%ld", valeur);
    break;

  case G_ICON:
#ifdef _DEVELOPPE_
    printf("met_lvaleur : écriture d'un chiffre dans un icône\n");
    printf("              pas de vérification de débordement possible -> c'est dangereux !\n");
#endif
    chaine = adresse[objet].ob_spec.icone->ib_ptext;
    sprintf(chaine, "%ld", valeur);
    break;

  case G_BOXCHAR:
    if (valeur < 0) {
      (void) fprintf(stderr, "met_lvaleur : Valeur trop petite pour un objet BOXCHAR [%ld]\n", valeur);
      valeur = 0;
    }
    if (valeur > 9) {
      (void) fprintf(stderr, "met_lvaleur : Valeur trop grande pour un objet BOXCHAR [%ld]\n", valeur);
      valeur = 9;
    }

    adresse[objet].ob_spec.gbox.caractere = '0' + valeur;
    break;

  case G_TEXT:
  case G_BOXTEXT:
    (void) fprintf(stderr, "met_lvaleur : À FAIRE --- objet de type G_TEXT ou G_BOXTEXT [valeur = %ld]\n", valeur);
    break;

  case G_FTEXT:
  case G_FBOXTEXT:
    retour = OBJ_MetValeur_Editable_L( arbre, objet, valeur );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "met_lvaleur : ERREUR dans OBJ_MetValeur_Editable_L [code %d]\n", retour);
      return;
    }
    if ( objet == arbres[ arbre ].edit_courant ) {
      BI_PlacerCurseur( arbres + arbre, BI_InitialiserCurseur( arbres[ arbre ].adresse + objet ) );
    }
    break;

  default:
    (void) fprintf(stderr, "met_lvaleur : objet de type non géré = %d [valeur : %ld]\n", type, valeur);
    return;
  }

  /* Si nécessaire, on redessine */
  if ( VRAI == arbres[ arbre ].drapeaux.ouvert ) {
    BI_DessinerObjet( arbre, objet );
  }
}

static int OBJ_MetValeur_Editable_L(unsigned int arbre, unsigned int objet, long valeur)
{
  TEDINFO *tedinfo;
  char *chaine;  /* Adresse de la chaîne affichée */
  const char *masque; /* Adresse du masque             */
  char *pos_debut, *pos_fin;
  char inter[50]; /* M'étonnerai qu'il y ait plus de 50 chiffres en usage courant ! */
  size_t nbr_chiffres;
  double valeur_maximale;

  /* Initialisation des variables */
  tedinfo = arbres[ arbre ].adresse[ objet ].ob_spec.tedinfo;
  if ( NULL == tedinfo ) {
    (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Pas de TEDINFO !\n");
    return -1;
  }
  chaine = tedinfo->te_ptext.editable;
  if ( NULL == chaine ) {
    (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Pas de chaîne !\n");
    return -1;
  }
  masque = tedinfo->te_ptmplt;
  if ( NULL == chaine ) {
    (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Pas de masque !\n");
    return -1;
  }

  pos_debut = strchr(masque, '_');  /* Position du premier caractère éditable */
  if ( pos_debut == NULL ) {        /* Pas de caractère éditable              */
    (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Aucun objet éditable [masque : \"%s\"]\n", masque);
    return -2;
  }

  /* Cas particulier, facile */
  if (valeur == 0) {
    chaine[ 0 ] = '0';
    chaine[ 1 ] = 0;
    return 0;
  }

  /* On cherche le dernier caractère éditable de l'ensemble */
  pos_fin = pos_debut + 1;
  while ( *pos_fin == '_' )
    pos_fin++;

  /* Ici, on a donc pos_debut qui pointe sur le premier _ et pos_fin sur le caractère après le dernier _
     du même bloc (ie, si c'est un truc pour flottant, on ignore la fin et ainsi de suite) */
  nbr_chiffres = pos_fin - pos_debut;
  if ( valeur < 0 ) {
    if ( 1 == nbr_chiffres ) {
      (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Nombre trop petit [%ld]\n", valeur);
      chaine[ 0 ] = '0';
      chaine[ 1 ] = 0;
      return +1; /* Erreur, mais mineure */
    } else {
      chaine[0] = '-';
      pos_debut++;
      nbr_chiffres--;
      valeur = -valeur;
      chaine++;
    }
  }
  valeur_maximale = pow(10.0, (double) (nbr_chiffres + 1)) - 1.0;
  if ( (double) valeur > valeur_maximale ) {
    (void) fprintf(stderr, "OBJ_MetValeur_Editable_L : Valeur trop grande, %ld [%g max.]\n", valeur, valeur_maximale);
    valeur = (long) valeur_maximale;
  }
  
  sprintf(inter, "%ld", valeur);
  strncpy(chaine, inter, nbr_chiffres);

  /* Tout s'est bien passé */
  return 0;
}

/* -------------- Met une valeur "float" dans un champ GEM --------- */

void met_fvaleur(unsigned int arbre, unsigned int objet, double valeur)
{
  int retour;

  OPTION_A_FAIRE( "Appel à remplacer par appel à OBJ_MettreNombre" );

#ifdef _DEVELOPPE_
# ifdef _DEC_UNIX_ /* Pas envie de faire les différents cas de fp_class & variantes */
  if ( (isnan(valeur)) || 
       (fp_class(valeur) == FP_POS_DENORM) || (fp_class(valeur) == FP_NEG_DENORM) ||
       (fp_class(valeur) == FP_SNAN) || (fp_class(valeur) == FP_QNAN) ) {
    (void) fprintf(stderr, "met_fvaleur : appel avec un flottant incorrect. Arbre : %d - objet %d\n",
		   arbre, objet);
  }
# elif defined(_LINUX_)
  if ( (isnan(valeur)) || (isinf(valeur)) ) {
    (void) fprintf(stderr, "met_fvaleur : appel avec un flottant incorrect. Arbre : %d - objet %d\n",
		   arbre, objet);
  }
# endif
#endif

  retour = OBJ_MettreNombre( arbre, objet, valeur, -1.0 );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans OBJ_MettreNombre(), code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    /* Valeur dans un bouton, une chaîne */
    if ( +1 == retour ) {
      char *chaine;

      chaine = arbres[ arbre ].adresse[ objet ].ob_spec.txt_malloc;
#ifdef _DEVELOPPE_
      printf("met_fvaleur : écriture d'un chiffre dans un bouton/une chaîne\n");
      printf("              pas de vérification de débordement possible -> c'est dangereux !\n");
      OPTION_A_FAIRE( "Remplacer l'objet par un autre type d'objet" );
#endif
      sprintf( chaine, "%3.2f", valeur );
    }
  }
}

/* ----------- Met une valeur "double" dans un champ GEM en notation scientifique ---------\
 |   Ne renvoie rien                                                                       |
 |                                                                                         |
 | Suppose 1) que le texte des boutons et chaînes accepte au moins 7 + exposant = 11 cars. |
 |         2) que le texte des TEDINFO est de la forme ___.______ (tout sauf _) ____       |
 |                              au moins 1--------------^   ^--------- au plus 10          |
 \---------------------------------------------------------------------------------------- */

void met_svaleur(unsigned int arbre, unsigned int objet, double valeur)
{
  OBJECT *adresse;
  const char *pos_point, *pos_debut, *pos_fin, *pos_exposant, *pos_fin_exposant;
  char nombre[50]; /* Doit largement suffire pour une précision de 10 chiffres ! */

  adresse = arbres[arbre].adresse;
  if ( ( OB_TYPE(adresse, objet) == G_BUTTON )
    || ( OB_TYPE(adresse, objet) == G_STRING ) )
  {
    char *chaine = (adresse + objet)->ob_spec.txt_malloc;
#ifdef _DEVELOPPE_
    printf("met_svaleur : écriture d'un chiffre dans un bouton/une chaîne\n");
    printf("              pas de vérification de débordement possible -> c'est dangereux !\n");
#endif
    sprintf(chaine, "%.5e", valeur);
  }

  if ( G_NOMBRE == OB_TYPE(adresse, objet) ) {
    OPTION_A_FAIRE( "valeur dans un nombre" );
    return;
  }

  if ( ( OB_TYPE(adresse, objet) == G_TEXT     )
    || ( OB_TYPE(adresse, objet) == G_BOXTEXT  )
    || ( OB_TYPE(adresse, objet) == G_FTEXT    )
    || ( OB_TYPE(adresse, objet) == G_FBOXTEXT ) )
  {
    TEDINFO *tedinfo = (adresse + objet)->ob_spec.tedinfo;
    char *texte = tedinfo->te_ptext.editable;  /* Adresse de la chaîne affichée */
    const char *masque = tedinfo->te_ptmplt; /* Adresse du masque             */
    char *txt_deb_decimal, *txt_deb_exposant;
    int i;

    OPTION_A_FAIRE("Corriger la mise en place du texte");

    pos_debut = strchr(masque, '_');   /* Position du premier caractère éditable */
    if (pos_debut == NULL)            /* Pas de caractère éditable              */
      return;

    pos_point = strchr(masque, '.');   /* Position du point décimal              */
    if (pos_point == NULL)
      pos_point = strchr(masque, ','); /* Ou de la virgule décimale */
    if (pos_point == NULL)            /* Ni l'un, ni l'autre -> on met un entier... */
      OBJ_MettreEntier_S( arbre , objet, (long) valeur );

    if (pos_point == pos_debut + 1) {   /* Un seul caractère avant le point */
      if ( valeur < 0.0 ) {
	AVERTISSEMENT( "Valeur négative [%g] à placer dans la chaîne _.*, objet %u, arbre %u", valeur, objet, arbre );
      }
      valeur = 0.0;
    }

    pos_fin_exposant = masque + strlen(masque) - 1;
    while (*pos_fin_exposant != '_') /* Dernier caractère éditable */
      pos_fin_exposant--;

    if (pos_fin_exposant == pos_debut) /* Un seul caractère éditable */
    {
      if (valeur < 0) valeur = 0;
      if (valeur > 9) valeur = 9;
      *texte = '0' + (int) valeur;
      *(texte + 1) = 0;
      BI_DessinerObjet(arbre, objet);
      return; /* On a fini */
    }

    pos_exposant = pos_fin_exposant; /* Début de l'exposant */
    while ( *(pos_exposant - 1) == '_' )
      pos_exposant--;

    pos_fin = pos_exposant - 1; /* Car. non modifiable juste avant le dernier champ éditable */
    if ( pos_fin == pos_point ) /* Pas d'exposant en réalité ! */
    {
      met_fvaleur(arbre, objet, valeur);
      return;
    }

    while ( *pos_fin != '_' )
      pos_fin--;

    /* pos_fin pointe sur le dernier _ avant l'exposant = fin de la partie décimale */

    /* Maintenant, on connaît les limites des trois zones */

    txt_deb_decimal  = texte + (pos_point - pos_debut);
    txt_deb_exposant = texte + (pos_fin - pos_debut);

    for (i = 0 ; i < (txt_deb_decimal - masque) ; i++)
      texte[i] = ' '; /* Chaîne pleine d'espace avant le point décimal */

    sprintf(nombre, "%+.10e", valeur); /* Nombre contient +x.xxxxxxxxxxe+xxxx */

    /* On met ce qui est avant la virgule */

    *(txt_deb_decimal - 1) = nombre[1];
    if (valeur < 0)
    {
      *(txt_deb_decimal - 2) = '-'; /* On est sûr qu'il existe, sinon forcé à >0 */
    }

    /* On met la partie décimale */

    if (strchr(nombre, '.') != NULL) {
      strncpy(txt_deb_decimal, (const char *) (strchr(nombre, '.') + 1),
	      (size_t) ( pos_fin - pos_point + 1 ) );
    } else {
      strncpy(txt_deb_decimal, "          ", (size_t) ( pos_fin - pos_point + 1 ) );
    }

    if (strchr(nombre, 'e') != NULL) {
      strncpy(txt_deb_exposant,
	      (const char *) (strchr(nombre, 'e') + 1),
	      (size_t) ( pos_fin_exposant - pos_exposant + 1 ) );
    } else {
      strncpy(txt_deb_exposant, "          ", (size_t) ( pos_fin_exposant - pos_exposant + 1 ) );
    }
  }
  BI_DessinerObjet(arbre, objet);
}

/* ~~~~~~~~~~~~~~~~~~~~ Lecture d'une valeur dans un objet contenant du texte ~~~~~~~~~~~~~~~~~~ */

/* -------------------- Lecture d'une chaîne de caractères --------------------------------------\
 |   Renvoie l'adresse de la chaîne affichée dans l'objet (informatif surtout...)                |
 |
 | arbre : arbre contenant l'objet à lire
 | objet : numéro de l'objet à lire
 | l_max : nombre maximal de caractères que l'on peut écrire dans texte (0 final non compris)    |
 | texte : pointeur sur la chaîne contenant la chaîne lu                                         |
 | mode  : TXT_COPIER   ==> on copie la chaîne à la place de l'ancienne                          |
 |         TXT_REMPLACE ==> on ne fait que renvoyer l'adresse de la chaîne que l'on veut lire    |
 \---------------------------------------------------------------------------------------------- */

char *lit_chaine(unsigned int arbre, unsigned int objet, unsigned long l_max, char *texte, int mode)
{
  OBJECT *adresse;
  char *chaine;

  adresse = arbres[arbre].adresse;
  switch ( adresse[objet].ob_type )
  {
  case G_STRING:   /* Chaîne ou bouton : OB_SPEC contient l'adresse de la chaine */
  case G_BUTTON:
    chaine = adresse[objet].ob_spec.txt_malloc; /* pour avoir un (char *), pas un (const char *) */
    break;
    
  case G_TEXT:     /* Texte et apparentés : structure TEDINFO */
  case G_BOXTEXT:
  case G_FTEXT:
  case G_FBOXTEXT:
    chaine = adresse[objet].ob_spec.tedinfo->te_ptext.editable;
    break;

  case G_TXT_INFINI:
    chaine = adresse[objet].ob_spec.txt_infini->texte;
    break;

  case G_SSMENU:
    chaine = (char *) adresse[ objet ].ob_spec.menu->texte;
    if ( ( NULL == chaine ) && ( adresse[ objet ].ob_spec.menu->options != NULL ) ) {
      chaine = (char *) adresse[ objet ].ob_spec.menu->options[ adresse[ objet ].ob_spec.menu->option_choisie ];
    }
    break;

  default:         /* Objet de type inconnu, ou sans texte associé -> on quitte */
    (void) fprintf(stderr, "%s,%s l.%u : objet de type inconnu ou sans texte [objet %d, type %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, objet, adresse[ objet ].ob_type);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  if ( NULL == chaine ) {
    AVERTISSEMENT( "Chaîne nulle dans l'objet %u, arbre %u", objet, arbre );
    if ( texte != NULL ) texte[ 0 ] = 0;
    return NULL;
  }

  if ( mode == TXT_REMPLACE ) /* On ne veut que l'adresse */
    return chaine;

  /* Là, on veut la copie de la chaîne */
  if ( texte != NULL ) {
    unsigned long l_chaine, i;

    /* On copie dans la chaîne indiquée */
    l_chaine = strlen( chaine );
    if ( l_max < l_chaine )
      l_chaine = l_max; /* Nombre de caractères au maximum, 0 non compris */

    for ( i = 0 ; i <= l_chaine ; i++ ) /* On recopie dans la chaîne voulue */
      texte[ i ] = chaine[ i ];

    chaine = texte;
  } else {
    /* On crée la copie avec strdup() --- à libérer avec free() */
    chaine = strdup( chaine );
  }

  return chaine;
}

/* -------------------- Redimensionner un objet ---------------------------------\
 |  Les valeurs d'agrandissement sont supposées exprimées en pixel               |
 |                                                                               |
 | arbre : l'arbre contenant l'objet à redimensionner                            |
 | objet : l'objet à redimensionner                                              |
 | x, y  : valeur des redimensionnements horizontaux et verticaux                |
 \------------------------------------------------------------------------------ */

void OBJ_agrandit(unsigned int arbre, unsigned int objet, int x, int y)
{
  int retour;
  OBJECT *adresse = arbres[arbre].adresse;

  /* On s'assure de travailler en coordonnées écran */
  if ( FAUX == arbres[ arbre ].drapeaux.init ) {
    (void) fprintf(stderr, "%s,%s l.%u : Arbre %u non initialisé\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    OPTION_A_FAIRE("Arbre non initialisé lors de l'appel !");
    retour = ARB_Initialiser( arbre, NULL, ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Initialisation de l'arbre a échoué [arbre %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre);
      return;
    }
  }
  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Valeurs absurdes / arbre %u, objet %u\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, objet);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  OB_WIDTH (adresse, objet) += x;
  OB_HEIGHT(adresse, objet) += y;

  if ( arbres[arbre].drapeaux.ouvert == VRAI ) { /* On redimensionne aussi la fenêtre - taille _absolue_ */
    Window fenetre = arbres[arbre].objets[objet];
    unsigned int larg, haut;

#ifdef _DEVELOPPE_
    if ( (OB_WIDTH(adresse, objet) <= 0) || (OB_HEIGHT(adresse, objet) <= 0) ) {
      fprintf(stderr, "Paramètre incorrect : taille négative ou nulle\n");
      fprintf(stderr, "OBJ_agrandit, arbre = %d, objet = %d\n", arbre, objet);

      XSynchronize(BI_Display, True); /* On force la synchro pour vérifier */
    }
    if ( fenetre == None ) {
      fprintf(stderr, "Fenêtre de l'objet inexistante / arbre marqué comme ouvert !\n");
      fprintf(stderr, "OBJ_agrandit, arbre = %d, objet = %d\n", arbre, objet);

      XSynchronize(BI_Display, True); /* On force la synchro pour vérifier */
    }
#endif

    larg = (unsigned int) OB_WIDTH(adresse, objet);
    haut = (unsigned int) OB_HEIGHT(adresse, objet);
    XResizeWindow(BI_Display, fenetre, larg, haut);
  }
}

/* -------------------- Objet visible ou invisible ----------------------------------------------\
 |    Renvoie l'état de l'objet après l'opération (VRAI : visible, FAUX : invisible)             |
 |                                                                                               |
 | num_arbre : numéro de l'arbre contenant l'objet à modifier                                    |
 | num_objet : numéro de l'objet à modifier                                                      |
 | mettre    : opération à faire. FORCE_NON = on rend visible, FORCE_OUI = on rend invisible,    |
 |                               ECHANGE = on bascule de l'un à l'autre, QUESTION = on lit juste |
 \---------------------------------------------------------------------------------------------- */

boolean OBJ_cache(unsigned int num_arbre, unsigned int num_objet, boolean mettre)
{
  OBJECT *adresse = arbres[num_arbre].adresse;

  /* Ici, le résultat est indépendant du type d'objet... */

  if (mettre == ECHANGE)
  {
    if ( OB_FLAGS(adresse, num_objet) & HIDETREE ) /* L'objet est invisible */
      mettre = FORCE_NON; /* On rend donc visible */
    else
      mettre = FORCE_OUI; /* On rend invisible */
  }

  if (mettre == FORCE_NON) /* On force à être non-caché = on rend visible */
  {
    OB_FLAGS(adresse, num_objet) &= ~HIDETREE; /* On rend visible */
    if ( arbres[num_arbre].drapeaux.ouvert == VRAI ) /* Arbre effectivement ouvert */
      XMapWindow(BI_Display, arbres[num_arbre].objets[num_objet]);
  }
  else if (mettre == FORCE_OUI) /* On force à être caché */
  {
    OB_FLAGS(adresse, num_objet) |=  HIDETREE; /* On rend invisible */
    if ( arbres[num_arbre].drapeaux.ouvert == VRAI ) /* Arbre effectivement ouvert */
      XUnmapWindow(BI_Display, arbres[num_arbre].objets[num_objet]);
  }
    
  if ( OB_FLAGS(adresse, num_objet) & HIDETREE ) /* Si QUESTION, on doit tester... */
    return VRAI;
  else
    return FAUX;
}

/* -------- Modifier la couleur d'un objet ---------------------------------\
 |  Renvoie la couleur actuelle de l'objet (-1 = pas de couleur possible)   |
 |                                                                          |
 | arbre   : numéro de l'arbre contenant l'objet à modifier                 |
 | objet   : numéro de l'objet à modifier                                   |
 | couleur : le code AES (donc 1 à 16 uniquement a priori) de la couleur    |
 | partie  : portion de l'objet que l'on veut modifier                      |
 |               COULEUR_BORD  = couleur du bord                            |
 |               COULEUR_FOND  = couleur du fond                            |
 |               COULEUR_TEXTE = couleur du texte                           |
 | mode    : QUESTION = on ne fait qu'interroger, sinon on met la couleur   |
 \------------------------------------------------------------------------- */

int OBJ_couleur(unsigned int arbre, unsigned int objet, int couleur, boolean partie, boolean mode)
{
  OBJECT *adresse = arbres[arbre].adresse;
  int couleur_mise = -1;

  if ( (couleur > 15) || (couleur < 0) )
    mode = QUESTION; /* Index de couleur incorrect pour l'AES */

  switch ( OB_TYPE(adresse, objet) )
  {
  case G_BOX:
  case G_IBOX:
  case G_BOXCHAR:
    if (mode != QUESTION) { /* On ne fait pas que questionner */
      if (partie == COULEUR_BORD) {
	(adresse + objet)->ob_spec.gbox.couleur_cadre = (couleur & 0xF);
      } else if (partie == COULEUR_FOND) {
	(adresse + objet)->ob_spec.gbox.couleur_fond = couleur; /* (couleur & 0xFL); */
      } else {
	(adresse + objet)->ob_spec.gbox.couleur_texte = (couleur & 0xF);
      }
    }

    if (partie == COULEUR_BORD) {
      couleur_mise = (adresse + objet)->ob_spec.gbox.couleur_cadre;
    } else if (partie == COULEUR_FOND) {
      couleur_mise = (adresse + objet)->ob_spec.gbox.couleur_fond;
    } else if (partie == COULEUR_TEXTE) {
      couleur_mise = (adresse + objet)->ob_spec.gbox.couleur_texte;
    }
    break;

  case G_TEXT:
  case G_BOXTEXT:
  case G_FTEXT:
  case G_FBOXTEXT:
    if (mode != QUESTION) { /* De toute façon, seul le texte est modifiable ici */
      (adresse + objet)->ob_spec.tedinfo->te_color = couleur;
    }

    couleur_mise = (adresse + objet)->ob_spec.tedinfo->te_color;
    break;

  case G_IMAGE: /* Couleur de premier plan */
    if (mode != QUESTION) {
      (adresse + objet)->ob_spec.image->bi_color = couleur;
    }
    couleur_mise = (adresse + objet)->ob_spec.image->bi_color;
    break;

  case G_STRING: /* De toute façon, pas de possibilité de choisir la couleur */
  case G_USERDEF:
  case G_BUTTON:
  case G_ICON:
  case G_TITLE:
  case G_CICON:
    break;
  }

  /* Doit-on redessiner l'ensemble ? */
  if ( ( mode != QUESTION ) && ( VRAI == arbres[ arbre ].drapeaux.ouvert ) ) {
    BI_DessinerObjet( arbre, objet );
  }

  return couleur_mise;
}

/* -------------------- Déplacer un objet --------------------------------------------------\
 |                                                                                          |
 | arbre : l'arbre contenant l'objet à déplacer                                             |
 | objet : l'objet à déplacer                                                               |
 | x, y  : valeur des déplacements horizontaux et verticaux                                 |
 | mode  : unité des déplacements (DEPLACE_PIXELS ou DEPLACE_RELATIF = en 0/00 de la taille |
 \----------------------------------------------------------------------------------------- */

void OBJ_deplace(unsigned int arbre, unsigned int objet, int x, int y, boolean mode)
{
  int delta_x, delta_y;
  int retour;
  OBJECT *adresse = arbres[arbre].adresse;

  /* On s'assure de travailler en coordonnées écran */
  if ( FAUX == arbres[ arbre ].drapeaux.init ) {
    (void) fprintf(stderr, "%s,%s l.%u : Arbre %u non initialisé\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    OPTION_A_FAIRE("Arbre non initialisé lors de l'appel !");
    retour = ARB_Initialiser( arbre, NULL, ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Initialisation de l'arbre a échoué [arbre %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre);
      return;
    }
  }
  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Valeurs absurdes / arbre %u, objet %u\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, objet);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  if (mode == DEPLACE_PIXELS) { /* x, y = déplacements en pixels */
    delta_x = x;
    delta_y = y;
  } else {
    delta_x = ( x * (int) OB_WIDTH(adresse, objet) ) / 1000;
    delta_y = ( y * (int) OB_HEIGHT(adresse, objet) ) / 1000;
  }

  if ( OB_X(adresse, objet) > delta_x ) {
    OB_X(adresse, objet) += delta_x; /* Empeche de sortir si les delta sont négatifs */
  } else {
    OB_X(adresse, objet) = 0;
  }
  if ( OB_Y(adresse, objet) > delta_y ) {
    OB_Y(adresse, objet) += delta_y;
  } else {
    OB_Y(adresse, objet) = 0;
  }

  if ( arbres[arbre].drapeaux.ouvert == VRAI ) { /* On déplace aussi la fenêtre - coordonées _absolues_ */
    Window fenetre = arbres[arbre].objets[objet];

#ifdef _DEVELOPPE_
    if ( fenetre == None ) {
      fprintf(stderr, "Fenêtre de l'objet inexistante / arbre marqué comme ouvert !\n");
      fprintf(stderr, "OBJ_agrandit, arbre = %d, objet = %d\n", arbre, objet);

      XSynchronize(BI_Display, True); /* On force la synchro pour vérifier */
    }
#endif
    XMoveWindow(BI_Display, fenetre, OB_X(adresse, objet), OB_Y(adresse, objet));
  }
}

/* -------- Définir une icône dans celles par défaut -------- */

#ifdef AVEC_XPM_ /* Les icônes, au format XPM */
# include "disquette.xpm"
# include "exclame.xpm"  /* Pas terrible : voir à améliorer */
# include "trombone.xpm" /* Ça c'est pour Patrick... */
# include "question.xpm" /* Pris de QT, voir si on trouve mieux ? */
# include "stop.xpm"     /* Pas terrible : voir à améliorer */
# include "infos.xpm"
#endif

void OBJ_icone(unsigned int arbre, unsigned int objet, int icone)
{
  int retour;

  retour = OBJ_ControlerValeurs(arbre, objet);
  if (retour != 0) {
    (void) fprintf(stderr, "OBJ_icone : erreur dans OBJ_ControlerValeurs [arbre %u, objet %u]\n", arbre, objet);
    return;
  }

  if ( arbres[ arbre ].adresse[ objet ].ob_type != G_ICON ) {
    (void) fprintf(stderr, "OBJ_icone : l'objet doit être un icône [objet %u, arbre %u]\n", objet, arbre);
    return;
  }

#ifdef AVEC_XPM_
  if ( ERR_ICONE_DISQUE == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = disquette;
  else if ( ERR_ICONE_EXCLAME == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = exclame;
  else if ( ERR_ICONE_QUESTION == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = question;
  else if ( ERR_ICONE_TROMBONE == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = trombone;
  else if ( ERR_ICONE_STOP == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = stop;
  else if ( ERR_ICONE_INFO == icone )
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = infos_xpm;
  else
    arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = NULL;
#else
  arbres[ arbre ].adresse[ objet ].ob_spec.icone_xpm = NULL;
  icone++;			/* Idiot, mais évite un warning "paramètre inutilisé" à la compilation */
#endif
}

/* -------------------- Modifier la position de l'objet -----------------------------------------\
 |   On suppose que l'on travaille en coordonnées relatives au père
 |   Valeur négative -> on ne modifie pas
 |
 | arbre : l'arbre contenant l'objet
 | objet : l'objet à modifier
 | x, y  : nouvelles coordonnées à modifier
 | nv_x, nv_y : contient les nouvelles coordonnées
 \---------------------------------------------------------------------------------------------- */

void OBJ_position(unsigned int arbre, unsigned int objet, int x, int y, int *nv_x, int *nv_y)
{
  OBJECT *adresse;
  int retour;

  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans OBJ_ControlerValeurs [arbre %u, objet %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, objet);
    return;
  }

  adresse = arbres[arbre].adresse;

  if (x >= 0)
    OB_X(adresse, objet) = x;
  else
    x = OB_X(adresse, objet);

  if (y >= 0)
    OB_Y(adresse, objet) = y;
  else
    y = OB_Y(adresse, objet);

  if (nv_x) *nv_x = OB_X(adresse, objet); /* On lit les nouvelles coordonnées   */
  if (nv_y) *nv_y = OB_Y(adresse, objet); /* Sauf si NULL -> pas intéressant... */
  
  if ( (x >= 0) || (y >= 0) ) { /* Si on a bien changé quelque chose... */
    if ( arbres[arbre].drapeaux.ouvert == VRAI ) { /* Arbre ouvert -> déplacer la fenêtre associée */
      Window fenetre = (arbres[arbre].objets)[objet];

      XMoveWindow(BI_Display, fenetre, x, y); 

      if (objet != ROOT) { /* Autre chose que la racine */
	/* On cherche le père */
	while ( OB_NEXT(adresse, objet) > (int) objet )
	  objet++;

	objet = OB_NEXT(adresse, objet);
	OBJ_dessine( (arbres[arbre].objets)[objet], adresse + objet );
      }
    }
  }
}

/* -------------------- Modifier les dimensions d'un objet ---------------------------------------\
 |   On suppose que l'on travaille en coordonnées relatives au père
 |   Valeur négative -> on ne modifie pas
 |
 | arbre            : l'arbre contenant l'objet
 | objet            : l'objet à modifier
 | larg, haut       : nouvelles dimensions de l'objet
 | nv_larg, nv_haut : contient les nouvelles dimensions
 \---------------------------------------------------------------------------------------------- */

void OBJ_taille(unsigned int arbre, unsigned int objet, int larg, int haut, unsigned int *nv_larg, unsigned int *nv_haut)
{
  OBJECT *adresse;
  int retour;

  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Paramètres absurdes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  adresse = arbres[arbre].adresse;

  if (larg >= 0)
    OB_WIDTH(adresse, objet) = larg;

  if (haut >= 0)
    OB_HEIGHT(adresse, objet) = haut;

  if (nv_larg) *nv_larg = OB_WIDTH (adresse, objet); /* On lit les nouvelles dimensions */
  if (nv_haut) *nv_haut = OB_HEIGHT(adresse, objet); /* sauf si NULL : pas intéressant  */

  if ( (larg >= 0) || (haut >= 0) ) { /* Si on a bien changé quelque chose... */
    larg = OB_WIDTH (adresse, objet); /* Dimensions à utiliser en réalité */
    haut = OB_HEIGHT(adresse, objet);

    if ( arbres[ arbre ].objets != NULL ) {
      /* Il y a une table d'objets : peut-être une fenêtre ? */
      Window fenetre;

      fenetre = ( arbres[ arbre ].objets)[ objet ];
      if ( fenetre != None ) {
	/* Changer la taille de la fenêtre associée */
	if ( (larg <= 0) || (haut <= 0) ) {
	  (void) fprintf(stderr, "%s,%s l.%u : taille négative ou nulle [arbre = %d, objet = %d ; L=%d, H=%d]\n",
			 __FILE__, __FUNCTION__, __LINE__, arbre, objet, larg, haut);
#ifdef _DEVELOPPE_
	  XSynchronize(BI_Display, True); /* On force la synchro pour vérifier */
#else
	  return;
#endif
	}

	printf("%s : on redimensionne la fenêtre %u [objet %u ; L=%d, H=%d\n",
	       __FUNCTION__, (unsigned int) fenetre, objet, larg, haut );
	XResizeWindow( BI_Display, fenetre, (unsigned int) larg, (unsigned int) haut );
#ifdef _DEVELOPPE_
      } else {
	(void) fprintf(stderr, "%s,%s l.%u : Pas de fenêtre associée [arbre %u, objet %u]\n",
		       __FILE__, __FUNCTION__, __LINE__, arbre, objet);
#endif
      }
#ifdef _DEVELOPPE_
    } else {
      /* Cela peut arriver dans les pré-initialisations des arbres... */
      (void) fprintf(stderr, "%s,%s l.%u : Arbre sans table d'objets [arbre %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre);
#endif
    }
#ifdef _DEVELOPPE_
  } else {
    if ( ( NULL == nv_larg ) && ( NULL == nv_haut ) ) {
      AVERTISSEMENT( "Appel inutile : on ne change rien, on ne demande pas les dimensions [arbre %u, objet %u]", arbre, objet );
    } else {
      /* Ce cas peut se produire lorsque l'on demande juste la taille de l'objet existant... en particulier pour la gestion des ascenseurs
	 ==> pas de message, juste rien à faire ! */
    }
#endif
  }
}

/* -------- Modifier les paramètres d'un objet utilisateur ----------\
 |    Renvoi le paramètre associé à cet objet                        |
 |                                                                   |
 | arbre, objet = caractérisent l'objet à modifier ou lire           |
 | fonction     = fonction de redessin à utiliser (NULL = inchangée) |
 | parametre    = paramètre à mettre                                 |
 | mode         = QUESTION si on demande juste le paramètre          |
 |                sinon, on installe la nouvelle valeur du paramètre |
 \------------------------------------------------------------------ */

long OBJ_utilisateur(unsigned int arbre, unsigned int objet, int (*fonction)(PARMBLK *), long parametre, boolean mode)
{
  OBJECT *adresse;
  int retour;

  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s, %s l. %u: arbre ou objet inexistant\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return 0;
  }
  adresse = arbres[ arbre ].adresse + objet;

  if ( adresse->ob_type != G_USERDEF ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : l'objet n'est pas un Userdef\n",
		    __FILE__, __FUNCTION__, __LINE__ );
#endif
    return 0;
  }

  if ( NULL == adresse->ob_spec.util ) { /* Non initialisé, ne devrait jamais arriver... */
    adresse->ob_spec.util = (USERBLK *) malloc( sizeof( USERBLK ) );
    if ( NULL == adresse->ob_spec.util ) {
      (void) fprintf(stderr, "%s,%s l.%u : mémoire insuffisante pour réserver l'USERBLK\n",
		     __FILE__, __FUNCTION__, __LINE__ );
      return 0;
    }

    adresse->ob_spec.util->ub_code = ( int (*)(PARMBLK *) ) NULL;
    adresse->ob_spec.util->ub_parm = 0;
  }

  if ( QUESTION == mode ) { /* On demande juste la valeur du paramètre */
    return adresse->ob_spec.util->ub_parm;
  }

  adresse->ob_spec.util->ub_parm = parametre;
  if ( fonction != ( int (*)(PARMBLK *) ) NULL )
    adresse->ob_spec.util->ub_code = fonction;

  if ( VRAI == arbres[arbre].drapeaux.ouvert ) /* Arbre ouvert, on redessine donc */
    OBJ_dessine( (arbres[ arbre ].objets)[ objet ], adresse);

  return parametre;
}

/* ~~~~~~~~ Traduction des texte dans un arbre ~~~~~~~~
    Cette fonction ne doit être appelée qu'une fois juste après la création de l'arbre
     pour traduire les textes des boutons

    A priori, n'a d'intérêt que pour un arbre chargé depuis un RSC...
     (sinon, textes codables en dur en C...)
 */

void OBJ_TraduireTexte(unsigned int arbre, unsigned int objet, const char *chaine)
{
  OBJECT *adr_objet;
  boolean copier = VRAI;
  int retour;

  /* Si aucune chaîne indiquée : rien à faire */
  if ( NULL == chaine ) return;

  /* Peut-on faire confiance aux valeurs d'arbre et d'objet indiquées ? */
  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : erreur %d dans OBJ_ControlerValeurs [arbre %u, objet %u]\n",
                    __FILE__, __FUNCTION__, __LINE__, retour, arbre, objet );
    return;
  }

  if ( VRAI == arbres[ arbre ].drapeaux.interne ) {
    /* Si l'arbre est interne, les chaînes sont a priori en dur,
       enregistrés dans le code : il vaut mieux traduire ces chaînes directement...
     */
    (void) fprintf( stderr, "%s, %s l.%u : Ne pas utiliser pour traduire un arbre interne.\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    (void) fprintf( stderr, "%s, %s l.%u : Codez directement les traductions, comme les textes de l'arbre\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }

  /* Quel est l'objet considéré ? */
  adr_objet = arbres[ arbre ].adresse + objet;

  /* On veut remplacer le texte de l'objet par la traduction.
     Faut-il copier la traduction [avec strdup()] ou l'installer telle quelle ? */
  if ( G_SSMENU == adr_objet->ob_type ) {
    copier = FAUX;
  }

  /* On remplace le texte de l'objet d'origine, si possible en le copiant par malloc pour être libérable... */
  retour = OBJ_RemplacerTexte( arbre, objet, chaine, copier );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : erreur %d dans OBJ_RemplacerTexte [arbre %u, objet %u]\n",
		    __FILE__, __FUNCTION__, __LINE__, retour, arbre, objet );
    return;
  }
}

/* ~~~~~~~~~~~~~~~~ Quelques objets USERDEF prédéfinis ~~~~~~~~~~~~~~~~ */

int USR_separateur(PARMBLK *parametres)
{
  int x_debut, y_debut, x_fin, y_fin;

  if ( parametres->pb_h > parametres->pb_w ) { /* Objet plus haut que large => séparateur vertical */
    x_debut = parametres->pb_x + (parametres->pb_w >> 1);
    y_debut = parametres->pb_y ;
    x_fin   = x_debut;
    y_fin   = parametres->pb_y + parametres->pb_h;

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS_FONCE ].pixel );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut + 1, y_debut, x_fin + 1, y_fin );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut - 1, y_fin  , x_fin + 1, y_fin );

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut - 1, y_debut, x_fin - 1, y_fin   );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut - 1, y_debut, x_fin - 1, y_debut );
  } else { /* Objet plus large que haut => séparateur horizontal */
    x_debut = parametres->pb_x;
    y_debut = parametres->pb_y + (parametres->pb_h >> 1);
    x_fin   = parametres->pb_x + parametres->pb_w;
    y_fin   = y_debut;

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS_FONCE ].pixel);
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut, y_debut - 1, x_fin, y_fin   - 1 );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_fin  , y_debut - 1, x_fin, y_debut + 1 );

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut, y_debut + 1, x_fin  , y_fin   + 1 );
    XDrawLine( BI_Display, BI_Fen_redessin, BI_GC_objets, x_debut, y_debut - 1, x_debut, y_debut + 1 );
  }

  return parametres->pb_currstate; /* État courant du bouton */
}
