/* Bibliothque X-Window pour les interfaces [pour LASE essentiellement]
   (C) 1998-2003, Emmanuel CURIS - Cration le 3 janvier 2003

   Gestion des redessins dans les fentres
*/

#define _LIBRAIRIE_X_ 

#include <stdio.h>
#include <stdlib.h>

#include "bib_interface.h"

/* -------- Variables locales -------- */

static Region region_redessin; /* Pour les redessins X : essai... */

/* -------- Fonctions locales -------- */

int BI_InitRedessins(void);
int BI_TermineRedessins(void);

static int EVNT_RDS_GadgetFenetre(Fenetre *fenetre, Window fenetre_X);
static int EVNT_RDS_GL           (Fenetre *fenetre, BI_infos_GL *infos_gl, XExposeEvent *redessin);
static int EVNT_RDS_Menu         (Window fenetre_X);

/* -------- Ralisation pratique ----- */

/* ~-~-~-~-~-~-~-~-~ Fonction gnrale : s'occupe d'appeler ce qu'il faut... ~-~-~-~-~-~-~-~-~  */

int EVNT_redessin(Fenetre *fenetre, XExposeEvent *redessin)
{
  Fenetre *mere;
  XRectangle rectangle;
  int retour;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p redessin=%p", fenetre, redessin );

  rectangle.x = (short) (redessin->x);
  rectangle.y = (short) (redessin->y);
  rectangle.width = (unsigned short) (redessin->width);
  rectangle.height = (unsigned short) (redessin->height);

  if ( NULL == fenetre ) { /* C'est probablement un menu qu'il faut redessiner */
    if (redessin->count > 0) return BI_EVENEMENT_TRAITE; /* Inutile de redessiner si des messages suivent ! */

    retour = EVNT_RDS_Menu( redessin->window );
    SORTIE_FONCTION( __FUNCTION__, "%d [fenetre = NULL]", retour );
    return retour;
  }

  /* Fentre ferme, mais rouverture possible */
  if ( None == fenetre->handle ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [fenetre->handle = None]", VRAI );
    return BI_EVENEMENT_TRAITE;
  }

  fenetre->drapeaux.rds_dem = FAUX; /* On traite le message de redessin */
  /* Attention, le message de redessin a pu tre demand sur la mre, qui peut ne jamais tre redessine */
  mere = FEN_FenetreMere( fenetre );
  if ( mere != NULL ) mere->drapeaux.rds_dem = FAUX;

  /* S'agit-il de la partie "principale" de la fentre ? */
  if ( redessin->window == fenetre->handle ) {
    if ( BI_FENETRE_GL == fenetre->type ) {
      retour = EVNT_RDS_GL( fenetre, fenetre->open_gl, redessin );

      SORTIE_FONCTION( __FUNCTION__, "%d [GL_FENETRE]", retour );
      return retour;
    } else {
      if ( fenetre->redessin ) { /* Il y a une fonction de redessin */
	XUnionRectWithRegion( &rectangle, region_redessin, region_redessin );
	if ( redessin->count == 0 ) {
	  XSetRegion( BI_Display, _PRG_station->gr_contexte, region_redessin );
	  DANS_FONCTION( "Appel  fenetre->redessin()" );
	  /* On travaille en coordonnes relatives  la zone de travail */
	  (fenetre->redessin)( fenetre, 0, 0, fenetre->w_larg, fenetre->w_haut );
	  if ( fenetre->rect ) {
	    DANS_FONCTION( "Appel  fenetre->rect()" );
	    (fenetre->rect)( fenetre, fenetre->vx_x, fenetre->vx_y, -1, -1 );
	  }

	  XDestroyRegion( region_redessin );
	  region_redessin = XCreateRegion(); /* Suppose que l'on n'a pas de mlange de messages Expose
						pour diffrentes fentres... */
 	  XSetClipMask( BI_Display, _PRG_station->gr_contexte, None ); /* On annule le clipping */
	}

	SORTIE_FONCTION( __FUNCTION__, "%d [fentre racine, non Open-GL]", VRAI );
	return BI_EVENEMENT_TRAITE;
      }
    }

    /* Si on est l, c'est qu'il y a eu un incident quelconque... */
    (void) fprintf(stderr, "EVNT_redessin : fentre sans fonction de redessin [%p / %s]\n",
		   (void *) fenetre, fenetre->titre);
    SORTIE_FONCTION( __FUNCTION__, "%d [ANORMAL]", 0 );
    return BI_EVENEMENT_NON_TRAITE;
  }

  /* Non : on s'occupe des autres lments de la fentre */
  if ( redessin->count > 0 ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Messages suivront]", VRAI );
    return BI_EVENEMENT_TRAITE; /* Inutile de redessiner s'il y en a aprs */
  }

  /* 1re tentative : les gadgets de fentre */
  if ( BI_EVENEMENT_NON_TRAITE != EVNT_RDS_GadgetFenetre( fenetre, redessin->window ) ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Gadgets]", VRAI );
    return BI_EVENEMENT_TRAITE;
  }

  /* 2e tentative : l'arbre d'une fentre GEM */
  if ( fenetre->arbre != NULL ) {
    Arbre *arbre;
    unsigned int i;

    arbre = arbres + fenetre->numero_arbre; /* Arbre contenu dans la fentre */
    for (i = 0 ; i < arbre->nbr_objets ; i++) {
      if ( redessin->window == (arbre->objets)[i] ) { /* Redessiner l'objet i */
	OBJ_dessine(redessin->window, arbre->adresse + i);
	if ( ( arbre->edit_courant !=  ROOT ) && ( i == arbre->edit_courant ) ) { /* On affiche le curseur texte */
	  BI_DessinerCurseur( arbre );
	}
	
	SORTIE_FONCTION( __FUNCTION__, "%d [Objet d'un arbre]", VRAI );
	return BI_EVENEMENT_TRAITE;
      }
    }
  }

  /* Ici : fentre proprement dite */
#ifdef _DEVELOPPE_
  printf("EVNT_redessin : redessin non effectu --- fentre %p [%s]\n", (void *) fenetre, fenetre->titre );
#endif

  SORTIE_FONCTION( __FUNCTION__, "%d [non-trait]", VRAI );
  return BI_EVENEMENT_NON_TRAITE; /* Redessin non trait */
}

/* ~-~-~-~-~-~-~-~-~ Fonctions spcifiques de certaines fentres ou rgions ~-~-~-~-~-~-~-~-~ */

/* -------- Les gadgets de fentre -------- */

static int EVNT_RDS_GadgetFenetre(Fenetre *fenetre, Window fen_a_redessiner)
{
  OBJECT gadget = { -1, -1, -1, 0, NONE, NORMAL, {NULL}, 0, 0, 1, 1 };

  /* On cherche si un gadget convient */

  if ( fen_a_redessiner == fenetre->barre_d_infos ) { /* Barre d'infos */
    gadget.ob_type = G_INFOS;
    gadget.ob_spec.texte = fenetre->infos;
#ifdef _DEVELOPPE_
/*     printf( "EVNT_RDS_GadgetFenetre : barre d'infos --- fentre %s\n", fenetre->titre ); */
#endif
#ifdef XGEM_AVEC_CLOSER_
  } else if ( fen_a_redessiner == fenetre->closer ) { /* Bouton de fermeture */
    gadget.ob_type = G_CLOSER;
#endif
  } else if ( fen_a_redessiner == fenetre->fleche_haute) { /* Flche verticale / ascenseur */
    gadget.ob_type = G_FLECHE;
    gadget.ob_spec.autre = BI_OBJ_FLECHE_HAUTE;
  } else if ( ( fen_a_redessiner == fenetre->bouton_asc_vertical ) || /* L'un des poussoirs */
	      ( fen_a_redessiner == fenetre->bouton_asc_horizontal ) ) {
    gadget.ob_type = G_ASCENSEUR;
  } else if ( ( fen_a_redessiner == fenetre->fond_asc_vertical ) || /* L'un des fonds d'ascenseur */
	      ( fen_a_redessiner == fenetre->fond_asc_horizontal ) ) {
    /* Pour l'instant, rien  redessiner ! */
    return VRAI;
  } else if ( fen_a_redessiner == fenetre->fleche_basse ) {
    gadget.ob_type = G_FLECHE;
    gadget.ob_spec.autre = BI_OBJ_FLECHE_BASSE;
  } else if ( fen_a_redessiner == fenetre->fleche_gauche) {
    gadget.ob_type = G_FLECHE;
    gadget.ob_spec.autre = BI_OBJ_FLECHE_GAUCHE;
  } else if ( fen_a_redessiner == fenetre->fleche_droite ) {
    gadget.ob_type = G_FLECHE;
    gadget.ob_spec.autre = BI_OBJ_FLECHE_DROITE;
  }

  /* Est-ce qu'un gadget a t trouv ? */
  if ( gadget.ob_type != 0 ) { /* Si oui, on le dessine */
    OBJ_dessine( fen_a_redessiner, &gadget );
    return BI_EVENEMENT_TRAITE;
  }

  /* Non : on essaye si, par hasard, il s'agit d'un onglet */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    int numero_onglet;
    boolean est_actif = FAUX;

    numero_onglet = BIX_QuelOnglet( fenetre, fen_a_redessiner, &est_actif );
    if ( numero_onglet >= 0 ) {
      BI_Onglet *onglets;

      onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
      if ( onglets != NULL ) {
	Window racine;
	int x, y;
	unsigned int larg, haut, profondeur, lg_bord;

	/* On rcupre les dimensions de la fentre */
	XGetGeometry( BI_Display, fen_a_redessiner, &racine, &x, &y, &larg, &haut, &lg_bord, &profondeur );

	/* On initialise l'objet */
	gadget.ob_type = G_BUTTON;
	gadget.ob_state = onglets[ numero_onglet ].etat;
	gadget.ob_spec.texte = onglets[ numero_onglet ].texte;
	gadget.ob_width = larg;
	gadget.ob_height = haut;
	if ( VRAI == est_actif ) {
	  gadget.ob_state |= SELECTED;
	} else {
	  gadget.ob_state &= ~SELECTED;
	}

	/* On redessine */
	OBJ_dessine( fen_a_redessiner, &gadget );

	return BI_EVENEMENT_TRAITE;
      }
    }
  }

  /* Non : on n'a pas pu traiter le message */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Menu droulant -------- */

static int EVNT_RDS_Menu( Window fenetre_X )
{
  unsigned int i;

  /* Y a-t-il un menu ? */
  if ( NULL == BI_Menu ) {
    (void) fprintf(stderr, "EVNT_RDS_Menu : aucun menu activ !\n");
    return BI_EVENEMENT_ERREUR;
  }

  if ( fenetre_X == BI_Menu->racine ) /* Fentre racine : rien  faire */
    return BI_EVENEMENT_TRAITE;

  for ( i = 0 ; i < BI_Menu->nbr_titres ; i++ ) {
    if ( fenetre_X == (BI_Menu->titres)[ i ] ) {
      OBJ_dessine(fenetre_X, BI_Menu->adresse + BI_Menu->prem_titre + i);
      return BI_EVENEMENT_TRAITE;
    }
  }

  (void) fprintf(stderr, "EVNT_RDS_Menu : fentre X %x non trouve dans le menu\n", (unsigned int) fenetre_X);
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Partie OpenGL d'une fentre -------- */

static int EVNT_RDS_GL(Fenetre *fenetre, BI_infos_GL *infos_gl, XExposeEvent *redessin)
{
  if ( NULL == infos_gl ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos Open GL passes !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p infos_gl=%p redessin=%p", fenetre, infos_gl, redessin );

  if ( fenetre->handle != redessin->window ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Redessin pour une autre fentre]", FAUX );
    return BI_EVENEMENT_NON_TRAITE;
  }

#ifdef _DEVELOPPE_
/*   printf("EVNT_RDS_GL( %s, %p, %p )\n", fenetre->titre, (void *) infos_gl, (void *) redessin ); */
#endif
#ifdef AVEC_OPEN_GL_
  if ( BI_OpenGL_actif == VRAI ) {
    DANS_FONCTION( "Open-GL actif" );
    if ( ( infos_gl->refaire == VRAI ) && ( BI_GL_direct == FAUX ) ) {
      const char *vx_infos;

      DANS_FONCTION( "Mode indirect" );

      vx_infos = fenetre->infos;
      fenetre->infos = NULL;
      FEN_infos( fenetre, "Calcul Open-GL en cours... Patientez SVP" );
      if ( ( infos_gl->image == None) || ( infos_gl->fond == None ) ) {
	BI_GL_direct = VRAI; /* On force en mode direct */
      } else {
	Bool reponse;

	DANS_FONCTION( "Appel  glXMakeCurrent [display=%p, drawable=%p, ctx=%p]", BI_Display, infos_gl->image, infos_gl->gl_contexte );
	reponse = glXMakeCurrent( BI_Display, infos_gl->image, infos_gl->gl_contexte );
	if ( False == reponse ) {  
	  (void) fprintf(stderr, "Affichage Open-GL : impossible de travailler en mode indirect\n");
	  BI_GL_direct = VRAI; /* Mode indirect ne fonctionne pas */
	} else {
	  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  if ( fenetre->redessin != ( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL ) {
	    ( fenetre->redessin )(fenetre, redessin->x, redessin->y, (unsigned) redessin->width, (unsigned) redessin->height);
	  } else {
	    (void) fprintf(stderr, "EVNT_RDS_GL : fentre sans fonction de redessin OpenGL !\n");
	  }
	  glFlush();
	  if ( BI_OpenGL_dBuffer == VRAI ) {
#ifdef _DEVELOPPE_
	    printf("On va changer les buffers\n");
#endif
	    glXSwapBuffers( BI_Display, infos_gl->image );
	  }
	  infos_gl->refaire = FAUX;

	  /* On force l'affichage de ce qui a t dessin */
	  XClearWindow( BI_Display, fenetre->handle );
	}
      }
      FEN_infos( fenetre, vx_infos );
    }
    if ( VRAI == BI_GL_direct ) { /* Affichage direct dans la fentre */
      DANS_FONCTION( "Appel  glXMakeCurrent [display=%p, drawable=%p, ctx=%p]", BI_Display, fenetre->handle, infos_gl->gl_contexte );
      glXMakeCurrent( BI_Display, fenetre->handle, infos_gl->gl_contexte );
      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
      if ( fenetre->redessin != ( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL ) {
	( fenetre->redessin )(fenetre, redessin->x, redessin->y, (unsigned) redessin->width, (unsigned) redessin->height);
      } else {
	(void) fprintf(stderr, "EVNT_RDS_GL : fentre OpenGL sans fonction de redessin OpenGL !\n");
      }
      glFlush();
      if (BI_OpenGL_dBuffer == VRAI) {
	glXSwapBuffers(BI_Display, fenetre->handle);
      }
    }
    /* Doit-on faire des dessins X aprs ? */
    if ( fenetre->open_gl->rds_post ) {
      (fenetre->open_gl->rds_post)( fenetre, redessin->x, redessin->y,
				    (unsigned) redessin->width, (unsigned) redessin->height );
    }
  } else {
    XSetForeground( BI_Display, BI_GC_objets, WhitePixel(BI_Display, BI_Num_ecran));
    XFillRectangle( BI_Display, fenetre->handle, BI_GC_objets,
		    redessin->x, redessin->y, (unsigned) redessin->width, (unsigned) redessin->height);
    XSetForeground( BI_Display, BI_GC_objets, BlackPixel(BI_Display, BI_Num_ecran));
    XDrawString( BI_Display, fenetre->handle, BI_GC_objets, 
		 redessin->x + (redessin->width >> 1), redessin->y + (redessin->height >> 1),
		 "Pas de serveur OpenGL actif", 27 );
  }
#else /* OPEN_GL_ non dfini */
# ifdef _DEVELOPPE_
  printf("%s( %s ) --- Pas de support OpenGL compil\n", __FUNCTION__, ( fenetre != NULL ) ? ( fenetre->titre ) : ( "FENTRE NULL !" ) );
# endif
  XSetForeground(BI_Display, BI_GC_objets, WhitePixel(BI_Display, BI_Num_ecran));
  XFillRectangle(BI_Display, fenetre->handle, BI_GC_objets,
		 redessin->x, redessin->y, (unsigned) redessin->width, (unsigned) redessin->height);
  XSetForeground(BI_Display, BI_GC_objets, BlackPixel(BI_Display, BI_Num_ecran));
  XDrawString(BI_Display, fenetre->handle, BI_GC_objets, 
	      redessin->x + (redessin->width >> 1), redessin->y + (redessin->height >> 1),
	      "Logiciel compil sans support OpenGL : aucune vue 3D possible.", 27);
#endif

  return BI_EVENEMENT_TRAITE;
}

/* ~-~-~-~-~-~-~-~-  appeler au dbut pour prparer la rgion de redessin... ~-~-~-~-~-~-~-~- */

int BI_InitRedessins(void)
{
  region_redessin = XCreateRegion();
  if ( None == region_redessin ) {
    (void) fprintf(stderr, "BI_InitRedessins : impossible de crer la rgion pour redessins\n");
    return -1;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~-  appeler  la fin pour librer la rgion de redessin... ~-~-~-~-~-~-~-~- */

int BI_TermineRedessins(void)
{
  XDestroyRegion( region_redessin );
  region_redessin = None;

  return 0;
}
