/* Bibliothèque GEM / X-Windows
   (C) 1998, Emmanuel CURIS

   Chargement des ressources : coeur ; prévu pour plusieurs ressources dès l'origine...

   Inspiré, en partie, de xrsrc.c par Oliver Groeger & Olaf Meisiek

 10 avril 1998
   * Fonctions RSC_creer() et RSC_effacer()

 1er avril 1998
   * Début du travail
*/

#define _LIBRAIRIE_X_

#include <errno.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "bib_interface.h"
#include "txt_utils.h"

#if ISO_8859_1 /* Codage latin-1 et non atari */
const char conv_8859_1[ 128 ] = {
                                          '', '', /* 128 - 129 */
  '', '', '', '', '', '', '', '', '', '', /* 130 - 139 */
  '', '', '', '', '', '', '', '', '', '', /* 140 - 149 */
  '', '', '', '', '', '', '', '', '', 'f', /* 150 - 159 */
  '', '', '', '', '', '', '', '', '', '?', /* 160 - 169 */
  '', '', '', '', '', '', '', '', '', '', /* 170 - 179 */
  '?', '?', '', '', '', '', '', '', '', '', /* 180 - 189 */
  '', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 190 - 199 */
  '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 200 - 209 */
  '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 210 - 219 */
  '?', '', '?', '?', 'a', '', 'G', 'p', 'S', 's', /* 220 - 229 */
  '', 't', 'F', 't', 'O', 'd', 'f', 'f', 'E', '?', /* 230 - 239 */
  '=', '', '>', '<', '?', '?', '', '~', '', '?', /* 240 - 249 */
  '', '?', 'n', '', '', ''                      /* 250 - 255 */
};
#endif /* ISO_8859_1 */

/* Conversion TOS vers codage utilisé */
static const char *conversion[ 128 ] = {
  C_CEDILLE    , u_TREMA   , e_AIGU , a_CIRC , a_TREMA   , a_GRAVE, a_ROND      , c_CEDILLE   , /* 80 -> 87 */
  e_CIRC       , e_TREMA   , e_GRAVE, i_TREMA, i_CIRC    , i_GRAVE, A_TREMA     , A_ROND      , /* 88 -> 8F */
  E_AIGU       , LIG_ae    , LIG_AE , o_CIRC , o_TREMA   , o_GRAVE, u_CIRC      , u_GRAVE     , /* 90 -> 97 */
  y_TREMA      , O_TREMA   , U_TREMA, CENT_AMERICAIN, LIVRE_STERLING, YEN, BETA , "f"         , /* 98 -> 9F */
  a_AIGU       , i_AIGU    , o_AIGU , u_AIGU , n_TILDE   , N_TILDE, a_EXPOSANT  , o_EXPOSANT  , /* A0 -> A7 */
  INTERROGE_INVERSE, "?"   , "?"    , UN_DEMI, UN_QUART  , EXCLAME_INVERSE, GUILLEMET_OUVRANT, GUILLEMET_FERMANT, /* A8 AF */
  a_TILDE      , o_TILDE   , O_BARRE, o_BARRE, LIG_oe    , LIG_OE, A_GRAVE      , A_TILDE     , /* B0 -> B7 */
  O_TILDE      , TREMA     , ACCENT_AIGU,OBELE, FIN_LIGNE, COPYRIGHT, ENREGISTRE, MARQUE_DEPOSEE, /* B8 -> BF */
  LIG_ij       , LIG_IJ    , ALEPH  , "?"    , "?"       , "?"   , "?"          , "?"         , /* C0 -> C7 */
  "?"          , "?"       , "?"    , "?"    , "?"       , "?"   , "?"          , "?"         , /* C8 -> CF */
  "?"          , "?"       , "?"    , "?"    , "?"       , "?"   , "?"          , "?"         , /* D0 -> D7 */
  "?"          , "?"       , "?"    , "?"    , "?"       , PARAGRAPHE, ET_LOGIQUE, INFINI     , /* D8 -> DF */
  ALPHA        , BETA      , GAMMA_MAJUSCULE, PI, SIGMA_MAJUSCULE, SIGMA, MU    , TAU         , /* E0 -> E8 */
  PHI_MAJUSCULE, THETA     , OMEGA_MAJUSCULE, DELTA, "?", PHI, APPARTIENT       , INTERSECTION, /* E9 -> EF */
  EQUIVALENT   , PLUS_MOINS, SUPERIEUR_EGAL, INFERIEUR_EGAL, "?", "?", DIVISER  , "?"         , /* F0 -> F7 */
  "?"          , "?"       , "?"    , "?"    , n_EXPOSANT, DEUX_EXPOSANT, TROIS_EXPOSANT, "?"   /* F8 -> FF */
};

#ifdef _XRSRC_ORIGINAL_

/****** Compilerswitches ******************************************************/

#define COLOR_ICONS FALSE	/* icônes couleur ?                       */
#define SAVE_MEMORY FALSE	/* réserver de la place pour les icônes ? */

/****** TYPES ****************************************************************/

#ifndef __MYDIAL__
typedef struct
{
  USERBLK	ublk;
  UWORD		old_type;
} OBBLK;
#endif

/****** VARIABLES ************************************************************/

static int  xgl_wbox, xgl_hbox;
static GRECT xdesk;

#if COLOR_ICONS == TRUE
WORD xvdi_handle;
#endif

static int    *rs_global;
static RSXHDR *rs_hdr;
static RSXHDR *hdr_buf;

#if COLOR_ICONS == TRUE
WORD    farbtbl[256][32];
ULONG   farbtbl2[32];
WORD    is_palette;
WORD    rgb_palette[256][4];
WORD    xpixelbytes;
WORD    xscrn_planes;
#endif

#endif /* _XRSRC_ORIGINAL_ */

static Ressource *premier_RSC = NULL;

/****** FUNCTIONS ************************************************************/

/* --------------- Création des infos nécesaires à un nouveau ressource ----------------- */

Ressource *RSC_creer(void)
{
  Ressource *ressource;

  ressource = (Ressource *) malloc( sizeof(Ressource) );
  if ( NULL == ressource ) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  ressource->Bitblk   = NULL; /* Tous les champs internes pouvant servir */
  ressource->Cicon    = NULL;
  ressource->Ciconblk = NULL;
  ressource->Iconblk  = NULL;
  ressource->Object   = NULL;
  ressource->Tedinfo  = NULL;
  ressource->Chaines  = NULL;
  ressource->Chaines_libres = NULL;
  ressource->Images_libres  = NULL;

  /* On met à jour la chaîne */

  ressource->suivant = premier_RSC;
  premier_RSC = ressource;

  return ressource;
}

/* --------------- Effacement des infos d'un ressource ---------------------------------- */

int RSC_effacer(Ressource *ressource)
{
  if ( NULL == ressource )
    return 0; /* Rien de spécial */

  if ( ressource == premier_RSC ) { /* On est sur le premier */
    premier_RSC = ressource->suivant;
  } else {
    Ressource *avant = premier_RSC;

    while ( avant->suivant != ressource ) {
      avant = avant->suivant;
      if ( NULL == avant ) {
	(void) fprintf( stderr, "%s,%s l.%u : le ressource demandé n'est pas dans la liste !\n",
			__FILE__, __FUNCTION__, __LINE__ );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return -1; /* Erreur survenue */
      }
    }
    avant->suivant = ressource->suivant;
  }

  free( ressource->Bitblk   ); ressource->Bitblk   = NULL;
  free( ressource->Cicon    ); ressource->Cicon    = NULL;
  free( ressource->Ciconblk ); ressource->Ciconblk = NULL;
  free( ressource->Iconblk  ); ressource->Iconblk  = NULL;
  free( ressource->Object   ); ressource->Object   = NULL;
  free( ressource->Tedinfo  ); ressource->Tedinfo  = NULL;
  free( ressource->Chaines  ); ressource->Chaines  = NULL;
  free( ressource->Chaines_libres ); ressource->Chaines_libres = NULL;
  free( ressource->Images_libres  ); ressource->Images_libres  = NULL;
  free( ressource ); ressource = NULL;

  return 0; /* Tout s'est bien passé */
}

/* --------------- Effacer tous les ressources en mémoire ------------------------------- */

void RSC_tt_effacer(void)
{
  while ( premier_RSC != NULL )
    RSC_effacer( premier_RSC );
}

/* --------------- Lecture de l'adresse d'un objet du RSC ------------------------------- */

int RSC_lit_adresse(Ressource *ressource, int type, unsigned int arbre, void *adresse)
{
  OBJECT *adr_objet;
/*   TEDINFO *adr_tedinfo; */
  unsigned int i;

  switch (type) /* Type d'objet que l'on cherche */
  {
  case R_TREE: /* On cherche un arbre d'objet */
    if ( arbre >= ressource->nombre_arbres ) { /* Pas assez d'arbres */
      (void) fprintf( stderr, "%s,%s l.%u : le ressource indiqué ne contient pas d'arbre %u (seulement %lu)\n",
		      __FILE__, __FUNCTION__, __LINE__, arbre, ressource->nombre_arbres );
      return -2;
    }

    i = 0;
    adr_objet = ressource->Object; /* Adresse de l'arbre */
    while ( i < arbre ) { /* Tant que ce n'est pas le bon */
      int dern_objet = 0;

      /* On commence par aller le plus loin possible dans cet arbre */
      while ( OB_TAIL(adr_objet, dern_objet) != -1 ) /* On saute le plus d'objets fils possible */
	dern_objet = OB_TAIL(adr_objet, dern_objet);

      /* On termine en parcourant « à la main » les derniers */
      while ( ( OB_FLAGS(adr_objet, dern_objet) & LASTOB ) == FAUX )
	dern_objet ++; /* On passe donc au suivant */

      adr_objet += dern_objet + 1; /* On se place sur le début de l'arbre suivant */
      i++;
    }

    *( (OBJECT **) adresse) = adr_objet; /* Pointeur sur le premier objet de l'arbre */
    return 0; /* Tout va bien */
  }

  return -3; /* Arrivé à la fin sans avoir rien fait */
}

/* -------- Lire une chaîne dans le fichier RSC -------- */

char *BI_RSC_LireChaine(struct rsc_infos *infos, unsigned long position)
{
  long ex_position;
  int c;
  char *chaine;
  unsigned long i, nbr_cars;

  /* On mémorise la position courante */
  ex_position = ftell( infos->fichier );

  /* On se met à la position indiquée */
  if ( -1 == fseek( infos->fichier, position, SEEK_SET ) ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR dans fseek - %s\n",
		    __FILE__, __FUNCTION__, __LINE__, strerror( errno ) );
    return NULL;
  }

  /* On cherche la fin de la chaîne : un 0 */
  nbr_cars = 0;
  do {
    c = fgetc( infos->fichier );
    if ( EOF == c ) {
      (void) fprintf( stderr, "%s,%s l.%u : Fin de fichier atteinte !\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      break;
    }
    if ( 127 == c ) {        /* Code 127 (7F) : Delta majuscule sur atari */
      nbr_cars += strlen( DELTA_MAJUSCULE );
    } else if ( c & 0x80 ) { /* Bit 7 mis : caractère non-ASCII, région 128 -> 255 (80 -> FF) */
      nbr_cars += strlen( conversion[ c - 128 ] );
    } else {		     /* Code 0 à 126 (00 à 7E) : ASCII standard */
      nbr_cars++;
    }
  } while ( c != 0 );		/* Chaîne atari du RSC terminée par un 0... */

  /* On réserve la chaîne correspondante */
  chaine = (char *) malloc( nbr_cars + 1 ); /* 0 final ! */
  if ( NULL == chaine ) {
    (void) fprintf( stderr, "%s,%s l.%u : Mémoire insuffisante pour créer la chaîne [%lu cars] !\n",
		    __FILE__, __FUNCTION__, __LINE__, nbr_cars );
    return NULL;
  }

  /* On se remet au début de la chaîne */
  fseek( infos->fichier, position, SEEK_SET );

  /* On lit la chaîne */
  i = 0;
  do {
    c = fgetc( infos->fichier );
    if ( EOF == c ) {
      (void) fprintf( stderr, "%s,%s l.%u : Fin de fichier atteinte !\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      break;
    }
    if ( 127 == c ) {		/* Code 127 (7F) : Delta majuscule sur atari */
      strcpy( chaine + i, DELTA_MAJUSCULE );
      i += strlen( DELTA_MAJUSCULE );
    } else if ( c & 0x80 ) {		/* Bit 7 mis : caractère non-ASCII, région 128 -> 255 (80 -> FF) */
      strcpy( chaine + i, conversion[ c - 128 ] );
      i += strlen( conversion[ c - 128 ] );
    } else {				/* Code 0 à 126 (00 à 7E) : ASCII standard */
      chaine[ i++ ] = c;
    }
  } while ( c != 0 );		/* Chaîne atari du RSC terminée par un 0... */

  chaine[ nbr_cars ] = 0; /* On s'assure qu'il y a bien un 0 en fin */

  /* fread( chaine, 1, nbr_cars, infos->fichier ); */

  /* On se remet à l'ancienne position */
  fseek( infos->fichier, ex_position, SEEK_SET );

  /* On traduit la chaîne */
/*   for ( i = 0 ; i < nbr_cars ; i++ ) { */
/*     c = * ( ( (unsigned char *) chaine ) + i ); /\* Pour être sûr d'avoir un codage > 127 des accents *\/ */
/*     if ( 127 == c ) { /\* Delta majuscule sur atari *\/ */
/*       chaine[ i ] = 'D'; */
/*     } else if ( c > 127 ) { /\* Accents, codage atari *\/ */
/* #if ISO_8859_1 == 1 /\* Codage latin-1 *\/ */
/*       chaine[ i ] = conv_8859_1[ c - 128 ]; */
/* #endif */
/*     } */
/*   } */

  /* Terminé */
#ifdef _DEVELOPPE_
/*   printf("%s : chaîne = %s\n", __FUNCTION__, chaine ); */
#endif
  return chaine;
}

/*****************************************************************************/

/* -------- Cherche le fichier RSC -------- */

char *BI_TrouverRSC(const char *nom_fichier)
{
  FILE *fichier;
  char *nom_rsc;

  if ( NULL == nom_fichier ) return NULL;

  /* Premier essai, le plus simple : le fichier tel-quel existe-t-il ? */
  fichier = fopen( nom_fichier, "r");
  if ( fichier != NULL ) {
    (void) fclose( fichier );
    return strdup( nom_fichier ); /* Rien à faire */
  }

  OPTION_A_FAIRE("Chercher dans les variables d'environnement le nom du répertoire des RSC");
  OPTION_A_FAIRE("Chercher dans les ressources X le nom du répertoire des RSC");

  /* Non-trouvé : on essaye dans le dossier codé en dur, s'il y en a un */
#ifdef LIB_RSC_
  nom_rsc = TXT_genere_chaine( "%s/%s", LIB_RSC_, nom_fichier );
  if (nom_rsc == NULL) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante - recherche du fichier RSC annulée [%s]\n",
		    __FILE__, __FUNCTION__, __LINE__, nom_fichier );
    return NULL;
  }
#ifdef _DEVELOPPE_
  printf("Nouveau lieu de recherche du RSC : %s\n", nom_rsc );
#endif
  fichier = fopen( nom_rsc, "r");
  if ( fichier != NULL ) { /* Trouvé */
    (void) fclose( fichier );
    return nom_rsc;
  }
  free( nom_rsc );
#endif /* LIB_RSC_ */

  /* Fichier non-trouvé */
  (void) fprintf( stderr, "%s,%s l.%u : Impossible de trouver le fichier RSC [%s]\n",
		  __FILE__, __FUNCTION__, __LINE__, nom_fichier );
  return NULL;
}

int RSC_charge(const char *fname, Ressource *ressource)
{
  int retour;
  unsigned int nbr_arbres;

  retour = BI_ChargerRSC( fname, 0, &nbr_arbres );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : Impossible de charger le RSC \"%s\" [code %d]\n",
		    __FILE__, __FUNCTION__, __LINE__, fname, retour );
    return -10;
  }

  ressource->nombre_arbres = nbr_arbres;

  return 0; /* Tout s'est bien passé */
}

/* -------- Conversion des coordonnées : caractère -> écran -------- */

int RSC_CarVEcran(unsigned int arbre, unsigned int objet)
{
  OBJECT *adresse;
  int coord, frac;

  adresse = arbres[ arbre ].adresse;

  coord =  OB_X(adresse, objet) & 0xFF ; /* Premier octet = caractères */
  frac = ( OB_X(adresse, objet) & 0xFF00 ) >> 8; /* Deuxième octet = fractions de caractère */
  OB_X( adresse, objet ) = coord * BI_larg_car + frac * BI_larg_car / 256;

  coord =  OB_Y(adresse, objet) & 0xFF ; /* Premier octet = caractères */
  frac = ( OB_Y(adresse, objet) & 0xFF00 ) >> 8; /* Deuxième octet = fractions de caractère */
  OB_Y(adresse, objet) = coord * BI_haut_car + frac * BI_haut_car / 256;

  OB_WIDTH(adresse, objet)  *= BI_larg_car;
  OB_HEIGHT(adresse, objet) *= BI_haut_car;

  /* Tout s'est bien passé */
  return 0;
}

#ifdef _XRSRC_ORIGINAL_
/*****************************************************************************/

void rs_sglobal (int *base)
{
  rs_global = base;
  hdr_buf = (RSXHDR *)*(LONG *)&rs_global[7];
  (LONG)rs_hdr = (LONG)hdr_buf + sizeof (RSXHDR);

  return;
}

/*****************************************************************************/

int rs_free (int *base)
{
#if COLOR_ICONS == TRUE
  WORD     i;
  CICON    *color_icn;
#endif

  rs_global = base;

#if COLOR_ICONS == TRUE

  if ((color_icn = (CICON *)*(LONG *)&rs_global[2]) != NULL)
  {
    for (i = 0; i < rs_global[4]; i++)
    {
      if (color_icn[i].num_planes > 1)
      {
	if (color_icn[i].col_data != NULL)
	  free (color_icn[i].col_data);
	if (color_icn[i].sel_data != NULL)
	  free (color_icn[i].sel_data);
	if (color_icn[i].sel_data == NULL && color_icn[i].sel_mask != NULL)
	  free (color_icn[i].sel_mask);
      }
    }
    free (color_icn);
  }
  if (*(LONG *)rs_global)
    free ((void *)*(LONG *)rs_global);
#endif

  free ((RSXHDR *)*(LONG *)&rs_global[7]);

  return (TRUE);
}

/*****************************************************************************/

int rs_sadd (int *base, WORD rs_stype, WORD rs_sindex, OBJECT *re_saddr)
{
  OBJECT *old_addr;

  rs_sglobal (base);

  old_addr = get_address (rs_stype, rs_sindex);

  if (old_addr == (OBJECT *)NULL)
    return (FALSE);

  *old_addr = *re_saddr;

  return (TRUE);
}

/*****************************************************************************/

void *get_address (WORD type, WORD index)
{
  void *the_addr = (void *)NULL;
  union
  {
    void	*dummy;
    char	*string;
    OBJECT     **dpobject;
    OBJECT	*object;
    TEDINFO	*tedinfo;
    ICONBLK	*iconblk;
    BITBLK	*bitblk;
  } all_ptr;

  switch (type)
  {
  case R_TREE:
    all_ptr.dpobject = (OBJECT **)(*(long **)&rs_global[5]);
    the_addr = all_ptr.dpobject[index];
    break;

  case R_OBJECT:
    the_addr = get_sub (index, hdr_buf->rsh_object, sizeof(OBJECT));
    break;

  case R_TEDINFO:
  case R_TEPTEXT:
    the_addr = get_sub (index, hdr_buf->rsh_tedinfo, sizeof(TEDINFO));
    break;

  case R_ICONBLK:
  case R_IBPMASK:
    the_addr = get_sub (index, hdr_buf->rsh_iconblk, sizeof(ICONBLK));
    break;

  case R_BITBLK:
  case R_BIPDATA:
    the_addr = get_sub (index, hdr_buf->rsh_bitblk, sizeof(BITBLK));
    break;

  case R_OBSPEC:
    all_ptr.object = get_address(R_OBJECT, index);
    the_addr = &all_ptr.object->ob_spec;
    break;

  case R_TEPVALID:
  case R_TEPTMPLT:
    all_ptr.tedinfo = get_address(R_TEDINFO, index);
    if (type == R_TEPVALID)
      the_addr = &all_ptr.tedinfo->te_pvalid;
    else
      the_addr = &all_ptr.tedinfo->te_ptmplt;
    break;

  case R_IBPDATA:
  case R_IBPTEXT:
    all_ptr.iconblk = get_address(R_ICONBLK, index);
    if (type == R_IBPDATA)
      the_addr = &all_ptr.iconblk->ib_pdata;
    else
      the_addr = &all_ptr.iconblk->ib_ptext;
    break;
    
  case R_STRING:
    the_addr = get_sub (index, hdr_buf->rsh_frstr, sizeof (char *));
    the_addr = (void *) * (char *)the_addr;
    break;

  case R_IMAGEDATA:
    the_addr = get_sub (index, hdr_buf->rsh_imdata, sizeof (char *));
    the_addr = (void *)*(char *)the_addr;
    break;

  case R_FRIMG:
    the_addr = get_sub (index, hdr_buf->rsh_frimg, sizeof (char *));
    the_addr = (void *)*(char *)the_addr;
    break;

  case R_FRSTR:
    the_addr = get_sub (index, hdr_buf->rsh_frstr, sizeof (char *));
    break;
  }

  return (the_addr);
}

/*****************************************************************************/

void *get_sub (WORD index, LONG offset, WORD size)
{
  unsigned char *ptr = (unsigned char *)rs_hdr;

  ptr += offset;
  ptr += (index * size);

  return ((void *)ptr);
}

/*****************************************************************************/

void fix_chp (int *pcoord, WORD flag)
{
  WORD ncoord;

  ncoord = *pcoord & 0xff;

  if (!flag && ncoord == 0x50)
    ncoord = xdesk.g_w;				/* xdesk.w = largeur de l'cran en points */
  else
    ncoord *= (flag ? xgl_hbox : xgl_wbox);	/* xgl_wbox, xgl_hbox = largeur & hauteur du dessin en points */

  if (((*pcoord >> 8) & 0xff) > 0x80)
    ncoord += (((*pcoord >> 8) & 0xff) | 0xff00);
  else
    ncoord += ((*pcoord >> 8) & 0xff);
  
  *pcoord = ncoord;
}

/*****************************************************************************/
/* initialiser les icônes couleur pour la résolution actuelle                */
/*****************************************************************************/

void do_ciconfix (ULONG header, RSXHDR *rsxhdr, LONG rs_len)
{
  LONG   *cicon_liste;
  WORD   i;
  OBJECT *obj;

  cicon_liste = (LONG *)(*(LONG *)(rsxhdr->rsh_rssize + (rsxhdr->rsh_rssize & 1L) + header + sizeof (LONG)) + header);
  if ((LONG)cicon_liste - header > rsxhdr->rsh_rssize && (LONG)cicon_liste - header < rs_len)
  {
    if (fill_cicon_liste (cicon_liste, header, rsxhdr) != NIL)
    {
#if COLOR_ICONS == TRUE
      WORD nub = 0, work_out [57], *palette;
			
      if (*(LONG *)rs_global && *(LONG *)&rs_global[2])
      {
	vq_extnd (xvdi_handle, TRUE, work_out);	/* Nombre de plans */
	xscrn_planes = work_out[4];

	xpixelbytes = test_rez ();
	palette = (WORD *)*(LONG *)(rsxhdr->rsh_rssize + (rsxhdr->rsh_rssize & 1L) + header + 2 * sizeof (LONG));
	if (palette != NULL)
	{
	  (LONG)palette += header;
	  memcpy (rgb_palette, palette, sizeof (rgb_palette));
	  is_palette = TRUE;
	}
	else
	  is_palette = FALSE;
	
	xfill_farbtbl ();
			
	for (i = 0; i < rsxhdr->rsh_nobs; i++)
        {
	  obj = &((OBJECT *)(rsxhdr->rsh_object + header))[i];
	  if ((obj->ob_type & 0xff) == G_CICON)
	  {
	    if (xadd_cicon ((CICONBLK *)obj->ob_spec, obj, nub++) == FALSE)
	    {
	      memset (&((CICON *)*(LONG *)&rs_global[2])[nub-1], 0, sizeof (CICON));
	      obj->ob_type = (obj->ob_type & 0xff00) | G_ICON;
	    }
	    obj->ob_spec -= header;
	  }
	}
      }
      else
#endif
      {
	for (i = 0; i < rsxhdr->rsh_nobs; i++)
	{
	  obj = &((OBJECT *)(rsxhdr->rsh_object + header))[i];
	  if ((obj->ob_type & 0xff) == G_CICON)
	  {
	    obj->ob_type = (obj->ob_type & 0xff00) | G_ICON;
	    obj->ob_spec -= header;
	  }
	}
      }
    }
  }
}

/*****************************************************************************/
/* initialisation du pointeur pour les icônes couleurs dans le RSC           */
/*****************************************************************************/

int fill_cicon_liste (LONG *cicon_liste, ULONG header, RSXHDR *rsxhdr)
{
  WORD     i, i2, num = 0;
  char     *p;
  LONG     iclen, num_cicon, ob, p2;
  CICONBLK *cblk;
  CICON    *cicon, *cold;
  OBJECT   *pobject;

  while (!cicon_liste[num])
    num++;
	
  if (cicon_liste[num] != -1L)
    return (NIL);
  
  cblk = (CICONBLK *)&cicon_liste[num+1];
	
  for (i = 0; i < num; i++)
  {
    cicon_liste[i] = (LONG)cblk;
    p = (char *)&cblk[1];
    cblk->monoblk.ib_pdata = (int *)p;
    iclen = cblk->monoblk.ib_wicon / 8 * cblk->monoblk.ib_hicon;
    p += iclen;
    cblk->monoblk.ib_pmask = (int *)p;
    p += iclen;
    p2 = (LONG)cblk->monoblk.ib_ptext;
    if (!p2 || header + p2 == (LONG)p || p2 < rsxhdr->rsh_string || p2 > rsxhdr->rsh_rssize)
      cblk->monoblk.ib_ptext = (char *) p;
    else
      (LONG)cblk->monoblk.ib_ptext = header + (LONG)cblk->monoblk.ib_ptext;
    
    cicon = (CICON *)&p[12];
    p += 12L;
    cold = cicon;
    if ((num_cicon = (LONG)cblk->mainlist) > 0)
    {
      cblk->mainlist = cicon;
	
      for (i2 = 0; i2 < num_cicon; i2++)
      {
	p = (char *)&cicon[1];
	cicon->col_data = (WORD *)p;
	p += iclen * cicon->num_planes;
	cicon->col_mask = (WORD *)p;
	p += iclen;
	if (cicon->sel_data != NULL)
	{
	  cicon->sel_data = (WORD *)p;
	  p += iclen * cicon->num_planes;
	  cicon->sel_mask = (WORD *)p;
	  p += iclen;
	}
	cicon->next_res = (CICON *)p;
	cold = cicon;
	cicon = (CICON *)p;
      }
      cold->next_res = NULL;
    }
    cblk = (CICONBLK *)p;
  }
  
  if (num != NIL)
  {
    pobject = (OBJECT *)(header + rsxhdr->rsh_object);
    
    for (ob = 0; ob < rsxhdr->rsh_nobs; ob++)
      if ((pobject[ob].ob_type & 0xff) == G_CICON)
	pobject[ob].ob_spec = cicon_liste[pobject[ob].ob_spec];
    
    if ((*(LONG *)rs_global = (LONG)malloc (num * sizeof (OBBLK))) != 0L)
      memset ((void *)*(LONG *)rs_global, 0, num * sizeof (OBBLK));
    if ((*(LONG *)&rs_global[2] = (LONG)malloc (num * sizeof (CICON))) != 0L)
      memset ((void *)*(LONG *)&rs_global[2], 0, num * sizeof (CICON));
    rs_global[4] = num;
  }
  
  return (num);
}

#if COLOR_ICONS == TRUE /* Toute la suite ne concerne que les icônes couleurs */

/*****************************************************************************/
/* changer le format de l'icône et l'adapter aux autres résolutions.         */
/*****************************************************************************/

WORD xadd_cicon (CICONBLK *cicnblk, OBJECT *obj, WORD nub)
{
  WORD     x, y, line, xmax, best_planes, find_planes;
  CICON    *cicn, *color_icn, *best_icn = NULL;
  LONG     len, *next;
  MFDB     d;
  OBBLK    *ub;
#if SAVE_MEMORY == TRUE
  CICON    *max_icn = NULL;
#endif

  len = cicnblk->monoblk.ib_wicon / 8 * cicnblk->monoblk.ib_hicon;
  
  color_icn = &((CICON *)*(LONG *)&rs_global[2])[nub];

  best_planes = 1;
  if (xscrn_planes > 8)
    find_planes = 4;
  else
    find_planes = xscrn_planes;

  cicn = cicnblk->mainlist;
  next = (LONG *)&cicnblk->mainlist;
  
  while (cicn != NULL)
  {
    *next = (LONG)cicn;
    next = (LONG *)&cicn->next_res;

#if SAVE_MEMORY == TRUE
    if (cicn->num_planes > xscrn_planes)
      max_icn = cicn;
#endif
    if (cicn->num_planes >= best_planes && cicn->num_planes <= find_planes)
    {
      best_planes = cicn->num_planes;
      best_icn = cicn;
    }
    cicn = cicn->next_res;
  }
	
  if (best_icn == NULL)		/* aucune icône couleur trouvée */
    return (FALSE);
  else
    *color_icn = *best_icn;
  
  if (best_planes > 1)
    color_icn->num_planes = xscrn_planes;
  else
    color_icn->num_planes = 1;
	
  /* Allouer la place pour le format universel */
  if ((color_icn->col_data = malloc (len * color_icn->num_planes)) == NULL)
    return (FALSE);
  if (color_icn->sel_data)
  {
    if ((color_icn->sel_data = malloc (len * color_icn->num_planes)) == NULL)
    {
      free (color_icn->col_data);
      return (FALSE);
    }
  }

  if (best_planes > 1)
  {
    if (best_icn->sel_data == NULL)
    {
      /* Prparer le masque slectionn */
      if ((color_icn->sel_mask = malloc (len)) == NULL)
	{
	  free (color_icn->col_data);
	  if (color_icn->sel_data)
	    free (color_icn->sel_data);
	  return (FALSE);
	}
      
      xmax = cicnblk->monoblk.ib_wicon / 16;
      
      for (y = 0; y < cicnblk->monoblk.ib_hicon; y++)
      {
	line = y * xmax;
	
	for (x = 0; x < xmax; x++)
	{
	  if (y & 1)
	    color_icn->sel_mask[line + x] = best_icn->col_mask[line + x] & 0xaaaa;
	  else
	    color_icn->sel_mask[line + x] = best_icn->col_mask[line + x] & 0x5555;
	}
      }
    }
		
    d.mp  = color_icn->col_data;
    d.fwp = cicnblk->monoblk.ib_wicon;
    d.fh	= cicnblk->monoblk.ib_hicon;
    d.fww = d.fwp >> 4;
    d.ff	= TRUE;
    d.np	= xscrn_planes;
	
    xfix_cicon ((UWORD *)best_icn->col_data, len, best_planes, xscrn_planes, &d);
    if (best_icn->sel_data)
    {
      d.mp = color_icn->sel_data;
      xfix_cicon ((UWORD *)best_icn->sel_data, len, best_planes, xscrn_planes, &d);
    }
  }
  else
  {
    memcpy (color_icn->col_data, best_icn->col_data, len);
    memcpy (color_icn->sel_data, best_icn->sel_data, len);
  }

#if SAVE_MEMORY == TRUE
  if (best_icn->num_planes < color_icn->num_planes && max_icn != NULL)
  {
    if (best_icn->sel_data == NULL || max_icn->sel_data != NULL)
    {
      best_icn->col_data = max_icn->col_data;
      best_icn->sel_data = max_icn->sel_data;
      best_icn->num_planes = xscrn_planes;
    }
  }
	
  if (best_icn->num_planes == color_icn->num_planes)
  {
    memcpy (best_icn->col_data, color_icn->col_data, len * color_icn->num_planes);
    free (color_icn->col_data);
    color_icn->col_data = NULL;
    if (best_icn->sel_data != NULL)
    {
      memcpy (best_icn->sel_data, color_icn->sel_data, len * color_icn->num_planes);
      free (color_icn->sel_data);
      color_icn->sel_data = NULL;
    }
    else
    {
      memcpy (cicnblk->monoblk.ib_pmask, color_icn->sel_mask, len);
      best_icn->sel_mask = cicnblk->monoblk.ib_pmask;
      free (color_icn->sel_mask);
      color_icn->sel_mask = NULL;
    }
  }
  else
  {
    if (best_icn->sel_data == NULL)
    {
      memcpy (cicnblk->monoblk.ib_pmask, color_icn->sel_mask, len);
      free (color_icn->sel_mask);
      color_icn->sel_mask = NULL;
      best_icn->col_data = color_icn->col_data;
      best_icn->sel_mask = cicnblk->monoblk.ib_pmask;
      best_icn->num_planes = color_icn->num_planes;
    }
    else
      *best_icn = *color_icn;
  }
  cicnblk->mainlist = best_icn;
#else
  color_icn->next_res = cicnblk->mainlist;
  cicnblk->mainlist = color_icn;
#endif
  ub = (OBBLK *)*(LONG *)rs_global;
  ub[nub].old_type = G_CICON;
  ub[nub].ublk.ub_parm = obj->ob_spec;
  ub[nub].ublk.ub_code = xdraw_cicon;
  obj->ob_spec = (LONG)&ub[nub].ublk;
  obj->ob_type = (obj->ob_type & 0xff00) | G_USERDEF;
  
  return (TRUE);
}

/*****************************************************************************/
/* combiens de bytes par pixels seront changs au format                    */
/*****************************************************************************/

LOCAL WORD test_rez ()

{	WORD     i, np, color, pxy[8], rgb[3], bpp = 0;
	UWORD    backup[32], test[32];
	WORD     black[3] = {0, 0, 0};
	WORD     white[3] = {1000, 1000, 1000};
	MFDB     screen;
	MFDB     pixel = {NULL, 16, 1, 1, 0, 1, 0, 0, 0};
	MFDB     stdfm = {NULL, 16, 1, 1, 1, 1, 0, 0, 0};

	if (xscrn_planes >= 8)
	{
		stdfm.np = pixel.np = xscrn_planes;

		if (xscrn_planes == 8)
		{
			color = 0xff;
			memset (test, 0, xscrn_planes * sizeof (WORD));
			for (np = 0; np < xscrn_planes; np++)
				test[np] = (color & (1 << np)) << (15 - np);
	
			pixel.mp = stdfm.mp = test;
			vr_trnfm (xvdi_handle, &stdfm, &pixel);
			
			for (i = 1; i < xscrn_planes; i++)
				if (test[i])	break;
			
			if (i >= xscrn_planes && !(test[0] & 0x00ff))
				bpp = 1;
		}
		else
		{
			xrect2array (&xdesk, pxy);
			vs_clip (xvdi_handle, FALSE, pxy);
			screen.mp = NULL;
			
			memset (backup, 0, sizeof (backup));
		
			vswr_mode (xvdi_handle, MD_REPLACE);
			vsl_ends (xvdi_handle, 0, 0);
			vsl_type (xvdi_handle, 1);
			vsl_width (xvdi_handle, 1);
			memset (pxy, 0, sizeof (pxy));
			
			graf_mouse (M_OFF, NULL);
		
			pixel.mp = backup;	
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
		
			/* sauver l'ancienne couleur */
			vq_color (xvdi_handle, 15, 1, rgb);
	
			/* tester le format universel */
			pixel.mp = test;
			vsl_color (xvdi_handle, 15);
			vs_color (xvdi_handle, 15, white);
			v_pline (xvdi_handle, 2, pxy);
			
			memset (test, 0, xscrn_planes * sizeof (WORD));
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
			
			for (i = (xscrn_planes + 15) / 16 * 2; i < xscrn_planes; i++)
				if (test[i])	break;
			
			if (i >= xscrn_planes)
			{
				vs_color (xvdi_handle, 15, black);
				v_pline (xvdi_handle, 2, pxy);
				
				memset (test, 0, xscrn_planes * sizeof (WORD));
				vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
				
				for (i = (xscrn_planes + 15) / 16 * 2; i < xscrn_planes; i++)
					if (test[i])	break;
				
				if (i >= xscrn_planes)
					bpp = (xscrn_planes + 7) / 8;
			}

			/* restaurer l'ancienne couleur */
			vs_color (xvdi_handle, 15, rgb);
	
			pixel.mp = backup;	/* restaurer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &pixel, &screen);
		
			graf_mouse (M_ON, NULL);
	
			xrect2array (&xdesk, pxy);
			vs_clip (xvdi_handle, TRUE, pxy);
		}
	}

	return (bpp);
}

/*****************************************************************************/
/* En mode True-Color, passer la valeur de point en palette RVB              */
/*****************************************************************************/

LOCAL VOID xfill_farbtbl ()

{	WORD np, color, pxy[8], backup[32], rgb[3];
	MFDB screen;
	MFDB pixel = {NULL, 16, 1, 1, 0, 1, 0, 0, 0};
	MFDB stdfm = {NULL, 16, 1, 1, 1, 1, 0, 0, 0};
	WORD pixtbl[16] = {0, 2, 3, 6, 4, 7, 5, 8, 9, 10, 11, 14, 12, 15, 13, 16};
	
	if (xscrn_planes >= 8)
	{
		if (xscrn_planes > 8)
		{
			if (is_palette == FALSE)	/* pas de palette dans le Ressource */
			{
				for (color = 0; color < 255; color++)
				{	if (color < 16)
					{	vq_color (xvdi_handle, pixtbl[color], 1, rgb_palette[color]);
						rgb_palette[color][3] = pixtbl[color];
					}
					else
					{	vq_color (xvdi_handle, color - 1, 1, rgb_palette[color]);
						rgb_palette[color][3] = color - 1;
					}
				}
				vq_color (xvdi_handle, 1, 1, rgb_palette[255]);
				rgb_palette[255][3] = 1;
				is_palette = TRUE;
			}
			
			vs_clip (xvdi_handle, FALSE, pxy);
			graf_mouse (M_OFF, NULL);

			memset (backup, 0, sizeof (backup));
	 		memset (farbtbl, 0, 32 * 256 * sizeof (WORD));
			screen.mp = NULL;
			stdfm.np = pixel.np = xscrn_planes;
		
			vswr_mode (xvdi_handle, MD_REPLACE);
			vsl_ends (xvdi_handle, 0, 0);
			vsl_type (xvdi_handle, 1);
			vsl_width (xvdi_handle, 1);
			memset (pxy, 0, sizeof (pxy));
			
			pixel.mp = backup;	/* calculer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
		
			/* calculer l'ancienne couleur */
			vq_color (xvdi_handle, 15, 1, rgb);

			for (color = 0; color < 256; color++)
			{
				vs_color (xvdi_handle, 15, rgb_palette[color]);
				vsl_color (xvdi_handle, 15);
				v_pline (xvdi_handle, 2, pxy);
				
				stdfm.mp = pixel.mp = farbtbl[color];
		
				/* vro_cpyfm,  car v_get_pixel ne fonctionne pas en TrueColor (>=24 Plans) */
				vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
	
				if (farbtbl2 != NULL && xpixelbytes)
				{	farbtbl2[color] = 0L;
					memcpy (&farbtbl2[color], pixel.mp, xpixelbytes);
				}
					
				vr_trnfm (xvdi_handle, &pixel, &stdfm);
				for (np = 0; np < xscrn_planes; np++)
					if (farbtbl[color][np])
						farbtbl[color][np] = 0xffff;
			}
		
			/* restaurer l'ancienne couleur */
			vs_color (xvdi_handle, 15, rgb);

			pixel.mp = backup;	/* restaurer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &pixel, &screen);
		
			graf_mouse (M_ON, NULL);
		}
	}
}

/*****************************************************************************/
/* adapter l'icne  la rsolution graphique actuelle                        */
/* (par exemple : une icne de 4 Plans vers 24 Plans TrueColor)              */
/*****************************************************************************/

LOCAL VOID xfix_cicon (UWORD *col_data, LONG len, WORD old_planes, WORD new_planes, MFDB *s)

{	LONG  x, i, old_len, rest_len, mul[32], pos;
	UWORD np, *new_data, mask, pixel, bit, color, back[32], old_col[32], maxcol;
	WORD  got_mem = FALSE;
	MFDB  d;
	
	len >>= 1;

	if (old_planes == new_planes)
	{	if (s != NULL)
		{	if (new_planes == xscrn_planes)
			{
				d = *s;
				d.ff = FALSE;
				s->mp = col_data;
				if (d.mp == s->mp)
				{	if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
						d.mp = s->mp;
					else
						got_mem = TRUE;
				}
				
				vr_trnfm (xvdi_handle, s, &d);
				if (d.mp != s->mp && got_mem == TRUE)
				{
					memcpy (s->mp, d.mp, len * 2 * new_planes);
					free (d.mp);
				}
			}
			else
				memcpy (s->mp, col_data, len * 2 * new_planes);
		}
		return;
	}
	
	if (new_planes <= 8)
	{
		old_len  = old_planes * len;
		rest_len = new_planes * len - old_len;

		if (s != NULL)
		{
			new_data = &((UWORD *)s->mp)[old_len];
			memset (new_data, 0, rest_len * 2);
			memcpy (s->mp, col_data, old_len * 2);
			col_data = s->mp;
		}
		else
			new_data = (UWORD *)&col_data[old_len];
		
		for (x = 0; x < len; x++)
		{
			mask = 0xffff;
	
			for (i = 0; i < old_len; i += len)
				mask &= (UWORD)col_data[x+i];
			
			if (mask)
				for (i = 0; i < rest_len; i += len)
					new_data[x+i] |= mask;
		}

		if (s != NULL)	/* convertir au format universel */
		{
			d = *s;
			d.ff = 0;
			if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
				d.mp = s->mp;
			
			vr_trnfm (xvdi_handle, s, &d);
			if (d.mp != s->mp)
			{
				memcpy (s->mp, d.mp, len * 2 * new_planes);
				free (d.mp);
			}
		}
	}
	else	/* TrueColor, valeur de points en RVB */
	{
		if (!xpixelbytes || s == NULL)
		{
			for (i = 0; i < new_planes; i++)
				mul[i] = i * len;
			
			if (old_planes < 8)
			{
				maxcol = (1 << old_planes) - 1;
				memcpy (old_col, farbtbl[maxcol], new_planes * sizeof (WORD));
				memset (farbtbl[maxcol], 0, new_planes * sizeof (WORD));
			}
	
			if (s != NULL)
			{
				new_data = &((UWORD *)s->mp)[old_len];
				memset (new_data, 0, rest_len * 2);
				memcpy (s->mp, col_data, old_len * 2);
				col_data = s->mp;
			}
			
			for (x = 0; x < len; x++)
			{
				bit = 1;
				for (np = 0; np < old_planes; np++)
					back[np] = col_data[mul[np] + x];
				
				for (pixel = 0; pixel < 16; pixel++)
				{
					color = 0;
					for (np = 0; np < old_planes; np++)
					{
						color += ((back[np] & 1) << np);
						back[np] >>= 1;
					}
					
					for (np = 0; np < new_planes; np++)
					{	pos = mul[np] + x;
						col_data[pos] = (col_data[pos] & ~bit) | (farbtbl[color][np] & bit);
					}
					
					bit <<= 1;
				}
			}
			if (old_planes < 8)
				memcpy (farbtbl[maxcol], old_col, new_planes * sizeof (WORD));

			if (s != NULL)	/* convertir au format universel */
			{
				d = *s;
				d.ff = 0;
				if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
					d.mp = s->mp;
				
				vr_trnfm (xvdi_handle, s, &d);
				if (d.mp != s->mp)
				{
					memcpy (s->mp, d.mp, len * 2 * new_planes);
					free (d.mp);
				}
			}
		}
		else
			std_to_byte (col_data, len, old_planes, farbtbl2, s);
	}
}

/*****************************************************************************/
/* std_to_byte transforme un dessin du format standard au format             */
/* universel (pour les rsolutions >= 16 Plans)                              */
/*****************************************************************************/

LOCAL VOID std_to_byte (col_data, len, old_planes, farbtbl2, s)
UWORD *col_data;
LONG  len;
WORD  old_planes;
ULONG *farbtbl2;
MFDB  *s;

{	LONG  x, i, mul[32], pos;
	UWORD np, *new_data, pixel, color, back[32];
	WORD  memflag = FALSE;
	UBYTE *p1, *p2;
	ULONG  colback;

	if (s->mp == col_data)
	{
		if ((col_data = malloc (len * 2 * s->np)) == NULL)
			return;
		memcpy (col_data, s->mp, len * 2 * s->np);
		memflag = TRUE;
	}
	new_data = (UWORD *)s->mp;
	p1 = (UBYTE *)new_data;

	if (old_planes < 8)
	{
		colback = farbtbl2[(1 << old_planes) - 1];
		farbtbl2[(1 << old_planes) - 1] = farbtbl2[255];
	}
		
	for (i = 0; i < old_planes; i++)
		mul[i] = i * len;
	
	pos = 0;
	
	for (x = 0; x < len; x++)
	{
		for (np = 0; np < old_planes; np++)
			back[np] = col_data[mul[np] + x];
		
		for (pixel = 0; pixel < 16; pixel++)
		{
			color = 0;
			for (np = 0; np < old_planes; np++)
			{
				color |= ((back[np] & 0x8000) >> (15 - np));
				back[np] <<= 1;
			}
			
			switch (xpixelbytes)
			{
				case 2:
					new_data[pos++] = *(UWORD *)&farbtbl2[color];
					break;

				case 3:
					p2 = (UBYTE *)&farbtbl2[color];
					*(p1++) = *(p2++);
					*(p1++) = *(p2++);
					*(p1++) = *(p2++);
					break;

				case 4:
					((ULONG *)new_data)[pos++] = farbtbl2[color];
					break;
			}
		}
	}

	if (old_planes < 8)
		farbtbl2[(1 << old_planes) - 1] = colback;

	if (memflag)
		free (col_data);
}

/*****************************************************************************/
/* dessine l'icne couleur                                                   */
/*****************************************************************************/

int CDECL xdraw_cicon (PARMBLK *pb)
{
  int	   ob_x, ob_y, x, y, dummy, pxy[4], m_mode, i_mode, mskcol, icncol;
  long	   ob_spec;
  ICONBLK *iconblk;
  CICON   *cicn;
  WORD    *mask, *data, *dark = NULL;
  char     letter[2];
  WORD     selected, mindex[2], iindex[2], buf, xy[4];
  boolean  invert = FALSE;
	
  selected = pb->pb_currstate & SELECTED;
	
  xrect2array ((GRECT *)&pb->pb_xc, xy);
  vs_clip (xvdi_handle, TRUE, xy);   /* dfinit la zone rectangulaire */

  ob_spec	= pb->pb_parm;
  ob_x		= pb->pb_x;
  ob_y		= pb->pb_y;

  iconblk = (ICONBLK *)ob_spec;
  cicn    = ((CICONBLK *)ob_spec)->mainlist;
  m_mode  = MD_TRANS;

  if (selected) /* it was an objc_change */
  {
    if (cicn->sel_data != NULL)
    {
      mask = cicn->sel_mask;
      data = cicn->sel_data;
      if (cicn->num_planes > 1)
      {
	if (cicn->num_planes > 8)	/* TrueColor, carte couleur en mode RVB? */
	  i_mode = S_AND_D;
	else
	  i_mode = S_OR_D;
      }
      else
	i_mode = MD_TRANS;
    }
    else
    {
      mask = cicn->col_mask;
      data = cicn->col_data;
      
      if (cicn->num_planes > 1)
      {
	if (cicn->num_planes > 8)
	  i_mode = S_AND_D;
	else
	  i_mode = S_OR_D;
	dark = cicn->sel_mask;
      }
      else
	invert = TRUE;
    }
  }
  else
  {
    mask = cicn->col_mask;
    data = cicn->col_data;
	
    if (cicn->num_planes > 1)
    {
      if (cicn->num_planes > 8)
	i_mode = S_AND_D;
      else
	i_mode = S_OR_D;
    }
    else
      i_mode = MD_TRANS;
  }
	
  mindex [0] = ((iconblk->ib_char & 0x0f00) != 0x0100) ? (iconblk->ib_char & 0x0f00) >> 8 : WHITE;
  mindex [1] = WHITE;
	
  icncol = iindex[0] = (WORD)(((UWORD)iconblk->ib_char & 0xf000U) >> 12U);
  iindex[1] = WHITE;
  
  mskcol = (iconblk->ib_char & 0x0f00) >> 8;
  
  x = ob_x + iconblk->ib_xicon;
  y = ob_y + iconblk->ib_yicon;
  
  if (invert)
  {
    buf       = iindex[0];
    iindex[0] = mindex[0];
    mindex[0] = buf;
    i_mode    = MD_TRANS;
  }
  if (selected)
  {
    buf    = icncol;
    icncol = mskcol;
    mskcol = buf;
  }
	
  draw_bitblk (mask, x, y, iconblk->ib_wicon, iconblk->ib_hicon, 1, m_mode, mindex);
  draw_bitblk (data, x, y, iconblk->ib_wicon, iconblk->ib_hicon, cicn->num_planes, i_mode, iindex);
	
  if (dark)
  {
    mindex [0] = BLACK;
    mindex [1] = WHITE;
    draw_bitblk (dark, x, y, iconblk->ib_wicon, iconblk->ib_hicon, 1, MD_TRANS, mindex);
  }

  if (iconblk->ib_ptext[0])
  {
    x = ob_x + iconblk->ib_xtext;
    y = ob_y + iconblk->ib_ytext;
    
    pxy[0] = x;
    pxy[1] = y;
    pxy[2] = x + iconblk->ib_wtext - 1;
    pxy[3] = y + iconblk->ib_htext - 1;
		
    vswr_mode     (xvdi_handle, MD_REPLACE);		/* dessiner la boite texte (TextBox) */
    vsf_color     (xvdi_handle, mskcol);
    vsf_interior  (xvdi_handle, FIS_SOLID);
    vsf_perimeter (xvdi_handle, FALSE);
    v_bar         (xvdi_handle, pxy);
  }

  vswr_mode     (xvdi_handle, MD_TRANS);
  vst_font      (xvdi_handle, 1);	/* Systemfont */
  vst_height    (xvdi_handle, 4, &dummy, &dummy, &dummy, &dummy);
  vst_color     (xvdi_handle, icncol);
  vst_effects   (xvdi_handle, TXT_NORMAL);
  vst_alignment (xvdi_handle, ALI_LEFT, ALI_TOP, &dummy, &dummy);
  vst_rotation  (xvdi_handle, 0);
	
  if (iconblk->ib_ptext[0])
  {
    x += (iconblk->ib_wtext - strlen (iconblk->ib_ptext) * 6) / 2;
    y += (iconblk->ib_htext - 6) / 2;
  
    v_gtext (xvdi_handle, x, y, iconblk->ib_ptext);
  }

  letter[0] = iconblk->ib_char & 0xff;
  if (letter[0])
  {
    letter[1] = '\0';
    x = ob_x + iconblk->ib_xicon + iconblk->ib_xchar;
    y = ob_y + iconblk->ib_yicon + iconblk->ib_ychar;
    
    v_gtext (xvdi_handle, x, y, letter);
  }
	
  return (pb->pb_currstate & ~ SELECTED);
} /* draw_userdef */

/*****************************************************************************/

void draw_bitblk (WORD *p, WORD x, WORD y, WORD w, WORD h, WORD num_planes, WORD mode, WORD *index)
{
  WORD	 	pxy[8];
  MFDB	 	s, d;

  d.mp	= NULL; /* screen */
  s.mp	= (VOID *)p;
  s.fwp = w;
  s.fh	= h;
  s.fww = w >> 4;
  s.ff	= FALSE;
  s.np	= num_planes;

  pxy[0] = 0;
  pxy[1] = 0;
  pxy[2] = s.fwp - 1;
  pxy[3] = s.fh - 1;
  
  pxy[4] = x;
  pxy[5] = y;
  pxy[6] = pxy[4] + pxy [2];
  pxy[7] = pxy[5] + pxy [3];

  if (num_planes > 1)
    vro_cpyfm (xvdi_handle, mode, pxy, &s, &d);
  else
    vrt_cpyfm (xvdi_handle, mode, pxy, &s, &d, index);	 /* copy it */
}

/*****************************************************************************/

void xrect2array (const GRECT *rect, int *array)
{
  *array++ = rect->g_x;
  *array++ = rect->g_y;
  *array++ = rect->g_x + rect->g_w - 1;
  *array   = rect->g_y + rect->g_h - 1;
} /* xrect2array */

/* #endif */
#endif

#endif /* def _XRSRC_ORIGINAL_ */
