/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2001, Emmanuel CURIS
   Cration du fichier : 21 dcembre 2001

   Tout ce qui concerne l'extraction des oscillations EXAFS

   Changement d'espace : E <-> k (voire autres ?) [rcupr du vieux EXAFS.c]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "extrait.h"
#include "science.h"

#include "conv_abscisses.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static double CONV_CalculeCoefficient(const char *unite_energie, const char *unite_k, int sens);

static void prepare_convert(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Conversion de l'nergie en nombres d'onde (eV -> 1/A) --------\
 |   Renvoie 0 si pas d'erreur, -xxx en cas d'erreur                      |
 |                                                                        |
 | spectre : le spectre dont on veut faire la conversion                  |
 | e_0     : la valeur de e_0, toujours pour cette conversion             |
 \----------------------------------------------------------------------- */

int EXF_e_v_k(Graphe *spectre, double e_0, boolean historique)
{
  unsigned long i;
  register double *zone_x;
  double coefficient, x;
  int retour = 0; /* Pas d'erreur */
  const char *unite;

  if (spectre == NULL) return E_V_K_PAS_GRAPHE; /* Erreur : pas de graphe en entre */
  if (e_0 < 0.0)       return E_V_K_E0_FAUX;    /* Erreur : E0 Absurde */

  /* Coefficient de conversion */
  coefficient  = 2 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rel */

  /* Analyse de l'unit en axe des X */
  unite = GRF_QuelleUnite( spectre, AXE_X );
  if ( unite != NULL ) {
    int type_unite;
    double facteur;

    facteur = TXT_NormaliserUnite( unite, FAUX, &type_unite, NULL );
    if ( ( type_unite != UNITE_ENERGIE ) || ( facteur <= 0.0 ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Unit incorrecte en X [%s type %d, on attend une nergie %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, unite, type_unite, UNITE_ENERGIE);
      return -58;
    }
    coefficient *= facteur;	/* Conversion des nergies en joules */ 
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'unit en X, on suppose des eV\n",
		   __FILE__, __FUNCTION__, __LINE__);
    coefficient *= EV_V_JOULE;	/* Conversion des nergies en joules */ 
  }

  zone_x = spectre->x;
  if ( NULL == spectre->x ) {
    (void) fprintf(stderr, "%s,%s l.%u : Graphe sans X ! Conversion des X impossible\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return E_V_K_PAS_GRAPHE;	/* Pas vraiment cette erreur, mais un peu comme si... */
  }
  
  for (i = 0 ; i < spectre->nombre_point ; i++) {
    x = coefficient * ( *(zone_x) - e_0 );
    if (x < 0.0) { /* Normalement, ne devrait pas arriver */
      *(zone_x) = - sqrt(-x) / M_V_ANGSTROM;
      retour = E_V_K_VAL_NEGATIVE; /* On a d convertir des valeurs de E bizarres */
    } else {
      *(zone_x) =   sqrt(x) / M_V_ANGSTROM; /* Donne des m^-1, on convertit en angstrom^-1 */
    }
    zone_x++;
  }

  /* On met les nouvelles bornes en X */  
  x  = (spectre->x_min - e_0); /* nergie en joules */
  x *= coefficient;
  if (x < 0.0)
    spectre->x_min = -sqrt(-x) / M_V_ANGSTROM;
  else
    spectre->x_min =  sqrt( x) / M_V_ANGSTROM;
  
  x  = (spectre->x_max - e_0); /* nergie en joules */
  x *= coefficient;
  if (x < 0.0)
    spectre->x_max = -sqrt(-x) / M_V_ANGSTROM;
  else
    spectre->x_max =  sqrt( x) / M_V_ANGSTROM;

  /* On a maintenant des A^-1 en X */
  GRF_ChangerUnite( spectre, AXE_X, ANGSTROM"^-1" );

  /* Avec les nouvelles barres d'erreur : les X sont implicites
     => si on change les X du graphe, change automatiquement ceux des erreurs en X et en Y
     => pas d'adaptation  faire ! */

  /* S'il y a des erreurs en X, il faut les adapter ! */
  if ( spectre->erreur_x != NULL ) {
    OPTION_A_FAIRE( "Adapter les erreurs en X" );
  }

  /* On ne touche pas aux Y : les erreurs en Y ne sont pas modifies */

  /* Log dans l'historique */
  if ( VRAI == historique ) {
    HIST_Titre(1, "Conversion E -> k", "CONV");
    HIST_Texte(1, 0, "Graphe : %s\n", spectre->nom);
    HIST_Texte(3, 0, "E0 = %f eV\n", e_0);
  }
  
  return retour;
}

/* -------- Conversion des nombres d'onde en  l'nergie (1/A -> eV) --------\
 |   Renvoie 0 si pas d'erreur, -xxx en cas d'erreur                        |
 |                                                                          |
 | spectre : le spectre dont on veut faire la conversion                    |
 | e_0     : la valeur de e_0, toujours pour cette conversion               |
 \------------------------------------------------------------------------- */

int EXF_k_v_e(Graphe *spectre, double e_0, boolean historique)
{
  unsigned long i;
  register double *zone_x;
  double coefficient, x, vers_inv_metre = 1.0;
  int retour = 0; /* Pas d'erreur */
  const char *unite;

  if (spectre == NULL) return E_V_K_PAS_GRAPHE; /* Erreur : pas de graphe en entre */
  if (e_0 < 0.0)       return E_V_K_E0_FAUX;    /* Erreur : E0 Absurde. Comme ne fait qu'une translation, la valeur 0 est accepte. */

  /* Coefficient de conversion */
  coefficient  = 2 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rel */
  coefficient *= EV_V_JOULE;              /* Conversion des nergies en eV [sinon : J] */ 

  /* Analyse de l'unit en axe des X */
  unite = GRF_QuelleUnite( spectre, AXE_X );
  if ( unite != NULL ) {
    int type_unite;
    double facteur;

    facteur = TXT_NormaliserUnite( unite, FAUX, &type_unite, NULL );
    if ( ( type_unite != UNITE_NOMBRE_ONDE ) || ( facteur <= 0.0 ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Unit incorrecte en X [%s type %d, on attend une nergie %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, unite, type_unite, UNITE_NOMBRE_ONDE);
      return -58;
    }
    vers_inv_metre = facteur;	/* Conversion des nergies en joules */ 
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'unit en X, on suppose des A^-1\n",
		   __FILE__, __FUNCTION__, __LINE__);
    vers_inv_metre = M_V_ANGSTROM;
/*     coefficient *= EV_V_JOULE; */	/* Conversion des nergies en joules */ 
  }

  zone_x = spectre->x;	
  for (i = 0 ; i < spectre->nombre_point ; i++, zone_x++) {
    x = *zone_x * vers_inv_metre; /* Conversion en m^-1 */
    x *= x / coefficient;
    if ( *zone_x < 0 ) {
      *zone_x = e_0 - x;
      retour = E_V_K_VAL_NEGATIVE;
    } else {
      *zone_x = e_0 + x;
    }
  }
  
  /* On met les nouvelles bornes en X */	
  x  = spectre->x_min * vers_inv_metre;
  x *= x / coefficient;
  if (spectre->x_min < 0.0)
    spectre->x_min = e_0 - x;
  else
    spectre->x_min = e_0 + x;

  x  = spectre->x_max * vers_inv_metre;
  x *= x / coefficient;
  if (spectre->x_max < 0.0)
    spectre->x_max = e_0 - x;
  else
    spectre->x_max = e_0 + x;

  /* On a maintenant des eV en X */
  GRF_ChangerUnite( spectre, AXE_X, "eV" );

  /* Avec les nouvelles barres d'erreur : les X sont implicites
     => si on change les X du graphe, change automatiquement ceux des erreurs en X et en Y
     => pas d'adaptation  faire ! */

  /* S'il y a des erreurs en X, il faut les adapter ! */
  if ( spectre->erreur_x != NULL ) {
    OPTION_A_FAIRE( "Adapter les erreurs en X" );
  }

  /* On ne touche pas aux Y : les erreurs en Y ne sont pas modifies */

  /* Log dans l'historique */
  if ( VRAI == historique ) {
    HIST_Titre(1, "Conversion k -> E", "CONV");
    HIST_Texte(1, 0, "Graphe : %s\n", spectre->nom);
    HIST_Texte(3, 0, "E0 = %f eV\n", e_0);
  }

  return retour;
}

/* -------- Calcul du coefficient multiplicatif pour la conversion --------
   Renvoie la valeur du coefficient (0.0 en cas d'erreur !)

   unite_energie : l'unit utilise pour indiquer les nergies
   unite_k       : l'unit utilise pour les nombres d'onde
   sens          : si > 0, conversion E -> k
                   si < 0, conversion k -> E

                               _____________________________
                              /    masse_electron (        )
   Formule utilise : k = \  / 2 A -------------- ( E - E  )    o A est le coefficient de conversion d'units :
                           \/          h * h      (      0 )    E  convertir en J, rsultat en m^-1  convertir vers l'unit souhaite...
                        = C * racine(E - E0)
 */

static double CONV_CalculeCoefficient(const char *unite_energie, const char *unite_k, int sens)
{
  double coefficient = 1.0, facteur_E, facteur_k;
  int type_E, type_k;

  if ( 0 == sens ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune conversion demande ! [sens = 0]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0.0;
  }
  if ( ( NULL == unite_energie ) || ( NULL == unite_k ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'unit indique pour les nergies [%s] ou les k [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, unite_energie, unite_k);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );

    /* On prend les valeurs par dfaut dans LASE */
    if ( NULL == unite_energie ) unite_energie = "eV";
    if ( NULL == unite_k ) unite_k = ANGSTROM"^-1";
  }

  /* On normalise les units : unit des nombres d'onde */
  facteur_k = TXT_NormaliserUnite( unite_k, FAUX, &type_k, NULL );
  if ( ( type_k != UNITE_NOMBRE_ONDE ) || ( facteur_k <= 0.0 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Unit incorrecte [%s type %d], on attend un nombre d'onde [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, unite_k, type_k, UNITE_NOMBRE_ONDE);
    return 0.0;
  }

  /* On normalise les units : unit des nergies */
  facteur_E = TXT_NormaliserUnite( unite_energie, FAUX, &type_E, NULL );
  if ( ( type_E != UNITE_ENERGIE ) || ( facteur_E <= 0.0 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Unit incorrecte [%s type %d], on attend une nergie [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, unite_energie, type_E, UNITE_ENERGIE );
    return 0.0;
  }

  /* On calcule le _carr_ du coefficient C tel que dfini ci-dessus */

  coefficient = 2.0 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rel : en kgJ^-2s^-2 */
  coefficient *= facteur_E; /* On convertit les nergies en joules, il reste du kgJ^-1s^-2 = m^-2, puisque J = kgms^-2 */
  coefficient /= facteur_k * facteur_k; /* On convertit le rsultat en l'unit souhaite */

  if ( sens < 0.0 ) {
    /* Conversion des k en nergie : on prend 1/C en fait, puisque E = E0 + k/C ! */
    coefficient = 1.0 / coefficient;
  } else {
    /* Conversion des nergies en k : on prend C directement, puisque k = C * racine |E - E0| */
    coefficient = sqrt( coefficient );
  }

  /* On a termin */
  return coefficient;
}

/* -------- Conversions ponctuelles E <-> k -------- */

double CONV_E_vers_k(double energie, double e0, const char *unite_e0, const char *unite_k)
{
  double coefficient;
  double k;

  coefficient = CONV_CalculeCoefficient( unite_e0, unite_k, +1 );
  if ( coefficient <= 0.0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans CONV_CalculeCoefficient() [code %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, coefficient);
    return 0.0;
  }

  /* On calcule k */
  k = coefficient * sqrt( fabs( energie - e0 ) );
  if ( energie < e0 ) {
    /* Par convention, on prend alors k < 0 */
    k = -k;
  }

  /* Termin */
  return k;
}

double CONV_k_vers_E(double k, const char *unite_k, double e0, const char *unite_e0)
{
  double coefficient;
  double energie;

  coefficient = CONV_CalculeCoefficient( unite_e0, unite_k, -1 );
  if ( coefficient <= 0.0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans CONV_CalculeCoefficient() [code %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, coefficient);
    return 0.0;
  }

  /* On calcule k */
  if ( k < 0 ) {
    /* Convention : k < 0 ssi E < E0 */
    energie = e0 - coefficient * ( k * k );
  } else {
    energie = e0 + coefficient * ( k * k );
  }

  /* Termin */
  return energie;
}

/* ---------------- Interface GEM pour l'appel de ces fonctions -------------------------------- */

Graphe *convert(Graphe *graphe)
{
  int retour;
  Graphe *dest;
  double e_0;   /* nergie de seuil */
  int type_unite;

  if (graphe == NULL) return NULL; /* Rien  faire */

  dest = graphe; /* Par dfaut: graphe converti = graphe de dpart */

  if (arbres[CONVERT_K_E].drapeaux.prepare == FAUX)
    prepare_convert();

  /* On propose un sens de conversion */
  type_unite = TXT_NormaliserUnite( GRF_QuelleUnite( graphe, AXE_X ), FAUX, &type_unite, NULL );
  if ( UNITE_ENERGIE == type_unite ) {
    OBJ_choisit(CONVERT_K_E, CONV_EVK, FORCE_OUI);
  } else if ( UNITE_NOMBRE_ONDE == type_unite ) {
    OBJ_choisit(CONVERT_K_E, CONV_KVE, FORCE_OUI);
  }

  retour = ARB_Bloquant( CONVERT_K_E, txt_titre_arbre, principale, 0 );
  if (retour == CONV_ANNULE) return NULL; /* On annule : anomalie... */

  e_0  = OBJ_LireNombre( CONVERT_K_E, VAL_CONVERT, NULL );
  if ( OBJ_choisit(CONVERT_K_E, CONV_NV_GRAPHE, QUESTION) & SELECTED ) {
    dest = GRF_Copier( graphe, NULL, FAUX );
    if (dest == NULL) return graphe; /* Conversion n'a pas pu avoir lieu => retour au graphe de dpart */
  } else
    dest = graphe;

  if ( OBJ_choisit(CONVERT_K_E, CONV_EVK, QUESTION) & SELECTED ) { /* Conversion de E vers k */
    int erreur = EXF_e_v_k(dest, e_0, VRAI);
    if (erreur) { /* Erreur lors de la conversion */
      if (E_V_K_VAL_NEGATIVE == erreur) {
	(void) BI_ERR_alerte(NULL, &MSG_conversion, VRAI);
      } else {
	(void) BI_ERR_alerte(NULL, &ERR_conversion, VRAI, erreur);
      }
    }
  } else {
    int erreur = EXF_k_v_e(dest, e_0, VRAI );
    if (erreur) {
      if (E_V_K_VAL_NEGATIVE == erreur) {
	(void) BI_ERR_alerte(NULL, &MSG_conversion, VRAI);
      } else {
	(void) BI_ERR_alerte(NULL, &ERR_conversion, VRAI, erreur);
      }
    }
  }
  
  return dest;
}

/* -------- Prparation des interfaces -------- */

static void prepare_convert(void)
{
  OBJ_deplace(CONVERT_K_E, CONV_EVK, 0, +1, DEPLACE_PIXELS);

#ifdef AVEC_TRADUCTION_
  met_chaine(CONVERT_K_E, CONV_NV_GRAPHE, 0, txt_arbre_nv_graphe, TXT_REMPLACE);
  met_chaine(CONVERT_K_E, VAL_CONVERT, 0, txt_arbre_e0, TXT_REMPLACE);
#endif

  arbres[CONVERT_K_E].drapeaux.prepare = VRAI;
}
