/* LASE : Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1996 - 2002
   Cration du fichier : 16 novembre 1998

   Extraction des oscillations EXAFS

   Reproduction d'une extraction avec les paramtres dj connus

 22 juin 2000 : on effaait le (mat) dans la repro. automatique si demand,
      mais son nom tait utilis pour crer les noms des graphes suivants -> seg. fault possible !
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "serie.h"

#include "absorption.h"
#include "extrait.h"

#include "fichiers.h" /* Pour ignore_ligne */
#include "filtrage.h" /* Pour TF_ouvre */
#include "xanes.h"

#include "exf_repro.txt"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

#define LG_MAX_NOM_MU0 50

#define ETAPE_3_REFAIRE   0 /* Rouvrir la fentre pour optimiser l'tape 3 */
#define ETAPE_3_SAUTER    1 /* Ne pas faire la 3e tape */
#define ETAPE_3_IDENTIQUE 2 /* Refaire la 3e tape avec le mme paramtre de lissage */

static char *chemin_parametres = NULL; /* Dossier de sauvegarde des paramtres de reproduction */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static struct serie *EXF_ExtractionTotale(Graphe *spectre, MAT_infos *infos_mum, MU1_infos *infos_I, MU2_infos *infos_II, int etape_III, double p,
					  int normalisation, boolean garder_intermediaires, struct serie *serie_resultats);

static int GER_reproduit(unsigned int objet, int x, int y);
static void    prepare_arbre_reproduit(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~~~~~~~~~~ */

/* -------- Extraction en une seule tape du signal E.X.A.F.S. ----------------------------\
 |      Ne renvoie rien                                                                    |
 |                                                                                         |
 | brut : le graphe brut qu'il faudra traiter                                              |
 \---------------------------------------------------------------------------------------- */

void EXF_reproduit(Graphe *brut)
{
  MAT_infos infos_mum = { MUM_VICTORINE_2, UN_POINT, 0.0, 0.0, NULL, 0, NULL, NULL, NULL}, *p_infos_mum = NULL;
  MU1_infos infos_I, *p_infos_I = NULL;
  MU2_infos infos_II;
  int etape_III = ETAPE_3_SAUTER, normalisation = NORM_AUCUNE, retour;
  boolean mu0_existe;
  extern const char *txt_aucun_graphe, *txt_aucun_choisi; /* Dans variables.c */
  struct serie *serie_resultats = NULL;

  if ( NULL == brut ) return;

  if ( FAUX == arbres[ EXAFS_GENERAL ].drapeaux.prepare )
    prepare_arbre_reproduit();

#ifdef _DEVELOPPE_ /* Car option toujours inactive pour l'instant */
  mu0_existe = GRF_type_existe(GR_MU0, VRAI, NULL);

  OBJ_active(EXAFS_GENERAL, EXF_MU0_CALC, mu0_existe);
  OBJ_active(EXAFS_GENERAL, EXV_TITRE_MU0, FORCE_NON);
  if (mu0_existe > 0)
    met_chaine(EXAFS_GENERAL, EXV_TITRE_MU0, LG_MAX_NOM_MU0, txt_aucun_choisi, TXT_COPIER);
  else
#endif
    met_chaine(EXAFS_GENERAL, EXV_TITRE_MU0, LG_MAX_NOM_MU0, txt_aucun_graphe, TXT_COPIER);

  do {
    retour = ARB_Bloquant( EXAFS_GENERAL, txt_titre_arbre, principale, 0 );
    if ( retour == EXF_ANNULER ) /* On annule le calcul */
      return;

    if ( retour == EXF_SAUVER ) { /* On sauve les paramtres de calcul */
      EXF_svparams();
    }

    if ( retour == EXF_CHARGER ) { /* On recharge les paramtres de calcul */
      EXF_chparams(NULL);
    }
  } while ( retour != EXF_CALCULER );

  /* On mmorise les infos de l'extraction
     1. Soustraction du fond continu */
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU_MATRICE, QUESTION) == VRAI ) { /* On limine la matrice */
    p_infos_mum = &infos_mum;

    /* On dfinit la mthode d'extraction */
    infos_mum.reference = NULL; /* Pas de rfrence */
    if (OBJ_choisit(EXAFS_GENERAL, E1F_MUM_2_PTS, QUESTION) == VRAI) { /* 2 points de contrle */
      infos_mum.mode = DEUX_POINTS;
      infos_mum.e_1p = OBJ_LireNombre(EXAFS_GENERAL, E1F_MUM_VAL_E1P, NULL);
    } else {
      infos_mum.mode = UN_POINT;
    }
    infos_mum.e_1 = OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E1, NULL);
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_DROITE, QUESTION) ) /* On utilise une droite */
      infos_mum.methode = MUM_DROITE;
    else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT1, QUESTION) ) /* On utilise une victorine */
      infos_mum.methode = MUM_VICTORINE_1;
    else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT2, QUESTION) ) /* On utilise une victorine */
      infos_mum.methode = MUM_VICTORINE_2;
    else if ( OBJ_choisit(EXAFS_GENERAL, E1F_MUM_POLY, QUESTION) ) { /* On utilise un polynme */
      infos_mum.degre = OBJ_LireEntier_S(EXAFS_GENERAL, EG_MUM_DEGRE);
      infos_mum.methode = MUM_POLYNOME;
    }
  }

  /* 2. Extraction des oscillations EXAFS, normalisation subsquente */
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_REFAIT, QUESTION) == VRAI ) { /* ter l'absorption atomique */
    p_infos_I = &infos_I;

    /* ---- Premire tape ---- */
    infos_I.e_0   = OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E0, NULL);
    infos_I.e_2   = OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E2, NULL);
    infos_I.degre = OBJ_LireEntier_S (EXAFS_GENERAL, EXV_DEGRE_MU0_I);
    infos_I.deg_affiche = OBJ_LireEntier_S(EXAFS_GENERAL, EXF_VALEUR_K);

    if ( OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, QUESTION) == VRAI ) {
      infos_I.avec_e3 = VRAI;
      infos_I.e_3 = OBJ_LireNombre(EXAFS_GENERAL, E1F_MU1_VAL_E2P, NULL);
    } else {
      infos_I.avec_e3 = FAUX;
      infos_I.e_3 = brut->x_max;
    }

    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, QUESTION) )
      infos_I.methode = ELIM_MU0_RAPIDE;
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT  , QUESTION) )
      infos_I.methode = ELIM_MU0_LENT;

    /* ---- Deuxime tape ---- */
    infos_II.degre = OBJ_LireEntier_S(EXAFS_GENERAL, EXF_DEGRE_MU0_II); /* Degr du polynme */
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, QUESTION) )  /* Objet slectionn */
      infos_II.methode = ELIM2_RAPIDE;
    else
      infos_II.methode = ELIM2_PRECISE;

    /* ---- Troisime tape ---- */
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_ETAPE3_VAR, QUESTION) == VRAI ) { /* tape refaite manuellement */
      etape_III = ETAPE_3_REFAIRE;
    } else if ( OBJ_choisit(EXAFS_GENERAL, EXF_ETAPE3_FIXE, QUESTION) == VRAI) { /* tape automatise */
      etape_III = ETAPE_3_IDENTIQUE;
    } else {
      etape_III = ETAPE_3_SAUTER;
    }
  }

  /* 3. Normalisation */
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_NORMALISER, QUESTION) == VRAI) { /* On veut normaliser */
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_NORM_THEORIE, QUESTION) == FAUX ) {
      normalisation = NORM_EXPERIENCE;
    } else {
      normalisation = MODELE_MU0_CONSTANT; /* De toute faon, pour l'instant, sans relle importance */
    }
  }

  /* On raliste l'extraction proprement dite */
  if ( VRAI == OBJ_choisit( EXAFS_GENERAL, EG_SERIE, QUESTION ) ) {
    Graphe **bruts;
    unsigned long i, nbr_bruts;

    /* On construit la table des graphes  traiter */
    bruts = GRF_ChoisirType( GRF_QuelleSerie( brut ), brut->type, VRAI, &nbr_bruts );
    if ( NULL == bruts ) {
      (void) fprintf(stderr, "%s : aucun graphe trouv, erreur probable dans GRF_ChoisirType()\n", __FUNCTION__);
      return;
    }

    /* On les traite */
    for ( i = 0 ; i < nbr_bruts ; i++ ) {
      serie_resultats = EXF_ExtractionTotale( bruts[ i ], p_infos_mum, p_infos_I, &infos_II, etape_III,
					      OBJ_LireNombre(EXAFS_GENERAL, EXF_VAL_S, NULL),
					      normalisation, OBJ_choisit(EXAFS_GENERAL, EXF_EFFACER, QUESTION) == FAUX, serie_resultats );
    }

    /* Plus besoin de cette table */
    free( bruts ); bruts = NULL;
  } else {
    serie_resultats = EXF_ExtractionTotale( brut, p_infos_mum, p_infos_I, &infos_II, etape_III,
					    OBJ_LireNombre(EXAFS_GENERAL, EXF_VAL_S, NULL),
					    normalisation, OBJ_choisit(EXAFS_GENERAL, EXF_EFFACER, QUESTION) == FAUX, NULL );
  }

  /* On libre la mmoire ventuellement rserve dans les routines */
  free( infos_mum.coefficients ); infos_mum.coefficients = NULL;

  OPTION_A_FAIRE( "Renvoyer la srie pour affichage des rsultats..." )
}

/* -------- Ralisation effective de la reproduction de l'extraction -------- */

static struct serie *EXF_ExtractionTotale(Graphe *spectre, MAT_infos *infos_mum, MU1_infos *infos_I, MU2_infos *infos_II, int etape_III, double p,
					  int normalisation, boolean garder_intermediaires, struct serie *serie_resultats)
{
  Graphe *brut;
  Graphe *mu_0 = NULL; /* Le polynme soustrait */

  if ( NULL == spectre ) {
    (void) fprintf(stderr, "%s,%s l.%u : pas de spectre indiqu\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( NULL == serie_resultats ) {
    /* Par dfaut : on se place dans la mme srie que l'original */
    serie_resultats = GRF_QuelleSerie( spectre );
  }
  brut = spectre;

  /* ~~~~ Soustraction du fond continu ~~~~ */
  if ( NULL != infos_mum ) {
    Graphe  *moins_mum = NULL;

    /* Les autres paramtres ont t dfinis avant appel de la fonction */

    /* On fait la soustraction proprement dite (s'occupe aussi de l'historique) */
    moins_mum = XAN_SoustraireMatrice(brut, infos_mum, VRAI);
    if ( NULL == moins_mum ) { /* Problme survenu */
      (void) fprintf(stderr, "EXF_ExtractionTotale : erreur dans XAN_SoustraireMatrice()\n");
      return NULL;
    }

    /* Le spectre soustrait devient la rfrence */
    brut = moins_mum;
  }

  /* ~~~~ Extraction des oscillations EXAFS ~~~~ */
  if ( NULL != infos_I ) {
    Graphe *exafs_I, *exafs_II, *exafs_III, *exafs;
    int retour;

    /* ---- Premire tape ---- */
    infos_I->exafs = NULL;
    infos_I->mu_0  = NULL;
    exafs_I = MU1_ExtraitEXAFS( brut, infos_I, VRAI ); /* Extraction : s'occupe des erreurs, de l'historique,... */
    if ( NULL == exafs_I ) {
      (void) fprintf(stderr, "EXF_ExtractionTotale : erreur dans MU1_ExtraitEXAFS()\n");
      return NULL; /* Problme apparu... */
    }

    retour = XAS_MU01_PostExtraction( brut, exafs_I, infos_I );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur dans le post-traitement de l'EXAFS (code %d)\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
    }

    if ( FAUX == garder_intermediaires ) { /* Plus besoin du (matrice) */
      if ( brut != spectre ) {
	GRF_efface(brut, VRAI, VRAI); /* On efface le (matrice) */
      }
    }

    /* ---- Deuxime tape ---- */
    infos_II->exafs = NULL;
    infos_II->mu_0  = NULL;
    infos_II->mu_0_ref = infos_I->mu_0;
    exafs_II = MU2_ExtraitEXAFS(exafs_I, infos_II, VRAI);
    if ( NULL == exafs_II ) {
      (void) fprintf(stderr, "EXF_ExtractionTotale : erreur dans MU2_ExtraitEXAFS()\n");
      return NULL; /* Problme survenu */
    }

    exafs = exafs_II; /* EXAFS actuel */
    mu_0  = infos_II->mu_0; /* Modle de 0 aprs les tapes I et II */

    if ( FAUX == garder_intermediaires ) { /* Plus besoin des rsultats de la 1re tape */
      GRF_efface(infos_I->mu_0 , VRAI, VRAI); /* 0  l'issue de la premire tape */
      GRF_efface(infos_I->exafs, VRAI, VRAI); /* signal  l'issue de la deuxime tape */
    }

    /* ---- Troisime tape, seulement si ncessaire ---- */
    if ( ETAPE_3_REFAIRE == etape_III ) {
      exafs_III = MU3_ouvre(exafs_II, mu_0, FAUX);
      (void) TF_ouvre(exafs_III, FAUX); /* Car aidera au choix */
      return NULL; /* La normalisation sera faite en fermant la fentre de 3e tape */
    }

    if ( ETAPE_3_IDENTIQUE == etape_III ) {
      MU3_infos infos_3;

      /* Paramtres de l'extraction */
      infos_3.parametre = p;
      infos_3.exafs = NULL;
      infos_3.mu_0  = NULL;
      infos_3.mu_0_ref = mu_0; /* Modle de 0 dj obtenu */

      /* Extraction : s'occupe des erreurs, de l'historique,... */
      exafs_III = MU3_ExtraitEXAFS(exafs, &infos_3, VRAI);
      if ( NULL == exafs_III ) {
	(void) fprintf(stderr, "EXF_ExtractionTotale : erreur dans MU3_ExtraitEXAFS()\n");
	return NULL; /* Problme survenu */
      }

      if ( FAUX == garder_intermediaires ) { /* Plus besoin des rsultats de la 2e tape */
	GRF_efface(exafs_II, VRAI, VRAI);
	GRF_efface(infos_II->mu_0, VRAI, VRAI);
      }

      exafs = exafs_III;
      mu_0 = infos_3.mu_0;
    }

    brut = exafs;
  }

  /* ~~~~~~~~ Normalisation ~~~~~~ */
  if ( NORM_EXPERIENCE == normalisation ) {
    int retour;

    if ( NULL == mu_0 ) {
      (void) fprintf( stderr, "%s,%s l.%u : pas de (0) exprimental pour normaliser\n",
		      __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }

    /* Normalisation en divisant par le (0) exprimental */
    retour = EXF_NormaliseExp( brut, mu_0, VRAI );
    if ( retour != 0 ) {
      OPTION_A_FAIRE( "Normalisation incorrecte --- Prvenir du pb" )
    }
    if ( FAUX == garder_intermediaires ) { /* Plus besoin des rsultats de la 2e tape */
      GRF_efface( mu_0, VRAI, VRAI ); /* Plus besoin du modle de 0 */
    }

    /* On insre le graphe rsultat dans la srie indique */
    serie_resultats = I_GRF_InsererDansSerie( serie_resultats, NULL, brut );
  } else if ( normalisation != NORM_AUCUNE ) {
    I_XAS_NormaliserEXAFS( brut );
  }

  /* Tout s'est bien pass : on renvoie la srie dans laquelle on a insr le graphe extrait */
  return serie_resultats;
}

/* -------- Chargement des paramtres d'extraction --------
    Valable aussi pour les paramtres de la TF
   -------- */

#define MODE_MUM       1
#define MODE_MU1       2
#define MODE_MU2       3
#define MODE_MU3       4
#define MODE_NORM      5
#define MODE_TFDIRECTE 6
#define MODE_TFINVERSE 7

#define ERR_LECTURE { SYS_ERREUR(ERREUR_LECTURE, nom_fichier[0], fichier, NULL); \
                      free(nom_fichier[0]); free(nom_fichier); \
                      (void) fclose(fichier); return FAUX; }

#define LGMAX_TXT_FENETRE 36 /* Dans l'arbre de la TFI */

boolean EXF_chparams(struct infos_reproduit_filtre *infos_filtre)
{
  FILE *fichier;
  char **nom_fichier;
  char chaine[11];
  int c_lu, mode = 0;
  unsigned int nbr_fichiers;

  nom_fichier = CH_selecteur( "*.CNF", txt_titre_fsel_charger, 0, NULL, &nbr_fichiers,
			      ARB_QuelleFenetre( arbres + EXAFS_GENERAL, FAUX ) );
  if ( (nom_fichier == NULL) || (nbr_fichiers == 0) ) return FAUX;

  if (nbr_fichiers > 1) {
    unsigned int i;

    /* On ne garde que le premier */
    for (i = 1 ; i < nbr_fichiers ; i++)
      free(nom_fichier[i]);
  }

  fichier = fopen(nom_fichier[0], "r");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier[0], NULL, NULL);
    free(nom_fichier[0]); free(nom_fichier);
    return FAUX;
  }

  if ( fgets(chaine, 10, fichier) == NULL ) {
    ERR_LECTURE;
  }
  if ( strcmp(chaine, "#LASE-EXT") != 0 ) {
#ifdef _DEVELOPPE_
    printf("EXF_chparams : format inconnu, chane : %s\n", chaine);
#endif
    EXF_ERREUR(FORMAT_INCONNU);
    free(nom_fichier[0]);
    free(nom_fichier);
    (void) fclose(fichier);
    return FAUX;
  }
  if (ignore_ligne(fichier) == EOF) { /* On ignore la fin de la ligne */
    ERR_LECTURE;
  }

  c_lu = fgetc(fichier); /* Premier caractre de la ligne suivante */
  while (c_lu != EOF) {
    switch (c_lu)
    {
    case  '#':  /* Ligne de commentaire : ignore */
      /*      printf("On doit sauter 1 ligne de commentaires\n"); */
      break;

    case '-': /* Changement de mode */
      if (fgets(chaine, 5, fichier) == NULL) {
	ERR_LECTURE;
      }
      if ( chaine[0] == 'M' ) {
	if ( chaine[2] == 'M' ) {
	  OBJ_choisit(EXAFS_GENERAL, E1F_MUM_2_PTS, FORCE_NON); /* Par dfaut, 1 pt seulement pour (m) */
	  OBJ_active(EXAFS_GENERAL, E1F_MUM_VAL_E1P, FORCE_NON);
	  mode = MODE_MUM;
	} else if ( chaine[3] == '1' ) {
	  OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, FORCE_NON); /* Idem pour 0,I */
	  OBJ_active(EXAFS_GENERAL, E1F_MU1_VAL_E2P, FORCE_NON);
	  mode = MODE_MU1;
	} else if ( chaine[3] == '2' )
	  mode = MODE_MU2;
	else if ( chaine[3] == '3' )
	  mode = MODE_MU3;
	else {
#ifdef _DEVELOPPE_
	  fprintf(stderr, "EXF_chparam : mode inconnu -%s\n", chaine);
#endif
	  ERR_LECTURE;
	}
      } else if ( chaine[0] == 'N' ) {
	mode = MODE_NORM;
      } else if ( chaine[0] == 'T' ) {
	if ( chaine[2] == 'D' ) {
	  OBJ_choisit(TF_GENERAL, RF_FAIRE_TF, FORCE_OUI);
	  OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, FORCE_NON);
	  mode = MODE_TFDIRECTE;
	} else if ( chaine[2] == 'I' ) {
	  mode = MODE_TFINVERSE;
	} else {
#ifdef _DEVELOPPE_
	  fprintf(stderr, "EXF_chparam : mode inconnu -%s\n", chaine);
#endif
	  ERR_LECTURE;
	}
      } else {
#ifdef _DEVELOPPE_
	fprintf(stderr, "EXF_chparam : mode inconnu -%s\n", chaine);
#endif
	ERR_LECTURE;
      }
      break;

    case 'a': /* Fonction d'apodisation */
      {
	int fenetre = -1;
	boolean etat;

	(void) fscanf(fichier, "%*s : %d", &fenetre);
	if ( (fenetre < 0) || (fenetre >= NOMBRE_FENETRES) ) {
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "EXF_chparams : fentre inconnue, %d\n", fenetre);
#endif
	  ERR_LECTURE;
	}
	met_chaine(TF_GENERAL, RF_FEN_TF, LGMAX_TXT_FENETRE, Types_fenetres[fenetre], TXT_COPIER);
	etat = Autorise_Symetrique(fenetre);
	OBJ_active(TF_GENERAL, RF_TF_SYMETRIQUE, etat);
	OBJ_active(TF_GENERAL, RF_VAL_K1, etat);
	OBJ_active(TF_GENERAL, TFG_TF_PARAM, Avec_Parametre(fenetre));
	if (infos_filtre != NULL)
	  infos_filtre->type_fen_tf = fenetre;
      }
      break;

    case 'k': /* Bornes des fentres */
      c_lu = fgetc(fichier);
      if ( c_lu == '1' ) { /* k1 -> fentre  plateau */
	double k1 = 0.0;

	(void) fscanf(fichier, ": %lf", &k1);
	OBJ_MettreNombre(TF_GENERAL, RF_VAL_K1, k1, 0.0 );
      } else if ( c_lu == '2' ) { /* k2 -> fentre non symtrique */
	double k2 = 0.0;

	OBJ_active(TF_GENERAL, RF_VAL_K2, FORCE_OUI );
	OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, FORCE_NON);
	(void) fscanf(fichier, ": %lf", &k2);
	OBJ_MettreNombre(TF_GENERAL, RF_VAL_K2, k2, 0.0 );
      } else if ( c_lu == 'm' ) {
	double k = 0.0;

	c_lu = fgetc(fichier);
	(void) fscanf(fichier, "%*s : %lf", &k);
	if (c_lu == 'i') { /* kmin */
	  if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, RF_VAL_KMIN, k, 0.0 );
	  else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, TFG_TFI_KMIN, k, 0.0 );
	} else if (c_lu == 'a') { /* kmax */
	  if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, RF_VAL_KMAX, k, 0.0 );
	  else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, TFG_TFI_KMAX, k, 0.0 );
	}
      }
      break;

    case 'D': /* Degr des divers polynmes */
      {
	int degre;
	fscanf(fichier, "%*s = %d", &degre);
	if (mode == MODE_MU1) {
	  OBJ_MettreEntier_U( EXAFS_GENERAL, EXV_DEGRE_MU0_I, degre );
	} else if (mode == MODE_MU2) {
	  OBJ_MettreEntier_U( EXAFS_GENERAL, EXF_DEGRE_MU0_II, degre );
	} else if (mode == MODE_MUM) {
	  OBJ_MettreEntier_U( EXAFS_GENERAL, EG_MUM_DEGRE, degre );
	}
      }
      break;

    case 'E': /* Valeur des diffrentes nergies */
      c_lu = fgetc(fichier);
      if (c_lu == '1') { /* E1 ou E1' */
	double val_e1;

	c_lu = fgetc(fichier);
	fscanf(fichier, " = %lf", &val_e1);
	if ( is_nan(val_e1) || is_inf(val_e1) ) {
	  ERR_LECTURE;
	}
	if (c_lu == '\'') {
	  OBJ_choisit(EXAFS_GENERAL, E1F_MUM_2_PTS, FORCE_OUI);
	  OBJ_active(EXAFS_GENERAL, E1F_MUM_VAL_E1P, FORCE_OUI);
	  OBJ_MettreNombre(EXAFS_GENERAL, E1F_MUM_VAL_E1P, val_e1, 0.0 );
	} else
	  OBJ_MettreNombre(EXAFS_GENERAL, EXF_VALEUR_E1, val_e1, 0.0 );
      } else if (c_lu == '0') { /* E0 */
	double val_e0;
	fscanf(fichier, " = %lf", &val_e0);
	if ( is_nan(val_e0) || is_inf(val_e0) ) {
	  ERR_LECTURE;
	}
	OBJ_MettreNombre(EXAFS_GENERAL, EXF_VALEUR_E0, val_e0, 0.0 );
      } else if (c_lu == '2') { /* E2 ou E2' */
	double val_e2;

	c_lu = fgetc(fichier);
	fscanf(fichier, " = %lf", &val_e2);
	if ( is_nan(val_e2) || is_inf(val_e2) ) {
	  ERR_LECTURE;
	}
	if (c_lu == '\'') {
	  OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, FORCE_OUI);
	  OBJ_active(EXAFS_GENERAL, E1F_MU1_VAL_E2P, FORCE_OUI);
	  OBJ_MettreNombre(EXAFS_GENERAL, E1F_MU1_VAL_E2P, val_e2, 0.0 );
	} else
	  OBJ_MettreNombre(EXAFS_GENERAL, EXF_VALEUR_E2, val_e2, 0.0 );
      }
      break;

    case 'f': /* Filtre */
      {
	int filtre = -1;
	boolean etat;

	(void) fscanf(fichier, "%*s : %d", &filtre);
	if ( (filtre < 0) || (filtre >= NOMBRE_FENETRES) ) {
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "EXF_chparams : filtre inconnue, %d\n", filtre);
#endif
	  ERR_LECTURE;
	}
	met_chaine(TF_GENERAL, RF_TFI_FENETRE, LGMAX_TXT_FENETRE, Types_fenetres[filtre], TXT_COPIER);
	etat = Autorise_Symetrique(filtre);
	OBJ_active(TF_GENERAL, RF_TFI_SYMETRIK, etat);
	OBJ_active(TF_GENERAL, RF_VAL_R1, etat);
	OBJ_active(TF_GENERAL, TFG_TFI_PARAM, Avec_Parametre(filtre));
	if (infos_filtre != NULL)
	  infos_filtre->type_fen_tfi = filtre;
      }
      break;

    case 'M': /* Infos sur les mthodes de calcul */
      fscanf(fichier, "%*s = %*[ ]");
      c_lu = fgetc(fichier);
      if (c_lu == 'c') /* Normalisation : avec le 0 calcul */
	GER_reproduit(EXF_NORM_MU0, 0, 0);
      else if (c_lu == 'i') /* Ignorer la 3e tape */
	GER_reproduit(EXF_ETAPE3_SANS, 0, 0);
      else if (c_lu == 'n') { /* normale */
	if (mode == MODE_MU1) OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, FORCE_OUI);
	else if (mode == MODE_MU2) OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, FORCE_OUI);
	else if (mode == MODE_TFDIRECTE) OBJ_choisit(TF_GENERAL, RF_TF_NORMALE, FORCE_OUI);
	else if (mode == MODE_TFINVERSE) OBJ_choisit(TF_GENERAL, RF_TFI_NORMALE, FORCE_OUI);
      } else if (c_lu == 'p') { /* pondre */
	if (mode == MODE_MU1) OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT, FORCE_OUI);
	else if (mode == MODE_MU2) OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_LENT, FORCE_OUI);
      } else if (c_lu == 'r') {
	if (mode == MODE_MU3) GER_reproduit(EXF_ETAPE3_VAR, 0, 0); /* III : refaire */
	if (mode == MODE_TFDIRECTE) OBJ_choisit(TF_GENERAL, RF_TF_RAPIDE, FORCE_OUI); /* Rapide */
	else if (mode == MODE_TFINVERSE) OBJ_choisit(TF_GENERAL, RF_TFI_RAPIDE, FORCE_OUI); /* Rapide */
      } else if (c_lu == 't') /* Normalisation thorique */
	GER_reproduit(EXF_NORM_THEORIE, 0, 0);
      else if (c_lu == 'v') /* III : valeur fixe */
	GER_reproduit(EXF_ETAPE3_FIXE, 0, 0);
      else {
#ifdef _DEVELOPPE_
	fprintf(stderr, "EXF_chparams : mthode inconnue : %c\n", c_lu);
#endif
	ERR_LECTURE;
      }
      break;

    case 'n': /* Pondration de travail */
      {
	int degre;
	fscanf(fichier, " = %d", &degre);
	OBJ_MettreEntier_U( EXAFS_GENERAL, EXF_VALEUR_K, degre );
      }
      break;

    case 'p':
      c_lu = fgetc(fichier);
      if (c_lu == ' ') { /* Valeur du paramtre */
	double p;

	if (fscanf(fichier, " : %lf", &p) != 1) {
	  fprintf(stderr, "Impossible de lire le paramtre");
	  ERR_LECTURE;
	}
	if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, TFG_TF_PARAM, p, 0.0 );
	else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, TFG_TFI_PARAM, p, 0.0 );
      } else if (c_lu == 'a') { /* Pas */
	double pas;

	if (fscanf(fichier, "%*s : %lf", &pas) != 1) {
	  ERR_LECTURE;
	}
	if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, TF_RF_PAS, pas, 0.0 );
	else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, TFG_TFI_PAS, pas, 0.0 );
      }
      break;

    case 'R':
      c_lu = fgetc(fichier);
      if ( c_lu == '1' ) { /* R1 -> fentre  plateau */
	double R1 = 0.0;

	if (fscanf(fichier, ": %lf", &R1) != 1) {
	  ERR_LECTURE
	}
	OBJ_MettreNombre(TF_GENERAL, RF_VAL_R1, R1, 0.0 );
      } else if ( c_lu == '2' ) { /* R2 -> fentre non symtrique */
	double R2 = 0.0;

	if (fscanf(fichier, ": %lf", &R2) != 1) {
	  ERR_LECTURE
	}
	OBJ_active(TF_GENERAL, RF_VAL_R2, FORCE_OUI);
	OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, FORCE_NON);
	OBJ_MettreNombre(TF_GENERAL, RF_VAL_R2, R2, 0.0 );
      } else if ( c_lu == 'm' ) {
	double R = 0.0;

	c_lu = fgetc(fichier);
	if (fscanf(fichier, "%*s : %lf", &R) != 1) {
	  ERR_LECTURE
	}
	if (c_lu == 'i') { /* kmin */
	  if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, RF_TF_RMIN, R, 0.0 );
	  else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, RF_VAL_RMIN, R, 0.0 );
	} else if (c_lu == 'a') { /* kmax */
	  if (mode == MODE_TFDIRECTE) OBJ_MettreNombre(TF_GENERAL, RF_TF_RMAX, R, 0.0 );
	  else if (mode == MODE_TFINVERSE) OBJ_MettreNombre(TF_GENERAL, RF_VAL_RMAX, R, 0.0 );
	}
      }
      break;

    case 'S': /* Paramtre pour III */
      {
	double s;
	fscanf(fichier, " = %lf", &s);
	if ( is_nan(s) || is_inf(s) ) {
	  ERR_LECTURE;
	}
	if (s < 0) s = 0.0;
	OBJ_MettreNombre( EXAFS_GENERAL, EXF_VAL_S, s, 0.0 );
      }
      break;

    case 'T': /* Choix : type de modle pour la matrice */
      fscanf(fichier, "%*s = ");
      c_lu = fgetc(fichier);
      if (c_lu == 'd') { /* Droite */
	OBJ_choisit(EXAFS_GENERAL, EXF_MUM_DROITE, FORCE_OUI);
      } else if (c_lu == 'p') { /* polynme */
	OBJ_choisit(EXAFS_GENERAL, E1F_MUM_POLY, FORCE_OUI);
      } else if (c_lu == 'v') { /* victorine */
	fscanf(fichier, "%*s%*[ ]");
	c_lu = fgetc(fichier);
	if (c_lu == 'r') { /* rduite */
	  OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT1, FORCE_OUI);
	} else if (c_lu == 'c') { /* complte */
	  OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT2, FORCE_OUI);
	} else {
#ifdef _DEVELOPPE_
	  fprintf(stderr, "EXF_chparams : type de victorine inconnue : %c\n", c_lu);
#endif
	  ERR_LECTURE;
	}
      } else {
#ifdef _DEVELOPPE_
	fprintf(stderr, "EXF_chparams : type de mu matrice inconnu : %c\n", c_lu);
#endif
	ERR_LECTURE;
      }
      break;

#ifdef _DEVELOPPE_
    default:
      fprintf(stderr, "EXF_chparams : commande inconnue : %c - la ligne est ignore\n", c_lu);
#endif
    }

    c_lu = ignore_ligne(fichier);
    if (c_lu != EOF) c_lu = fgetc(fichier);
  }

  (void) fclose(fichier);
  free(nom_fichier[0]);
  free(nom_fichier);
  return VRAI;
}

/* -------- Sauvegarde des paramtres d'extraction ----------------------------------- */

boolean EXF_svparams(void)
{
  FILE *fichier;
  char *nom_fichier;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    2, NULL,
    0, NULL,
    NULL
  };

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) "Sauver les param"e_GRAVE"tres d'extraction";
  infos_selecteur.chemin = chemin_parametres;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, ARB_QuelleFenetre( arbres + EXAFS_GENERAL, FAUX ) );
  /* On mmorise les chemins */
  chemin_parametres = infos_selecteur.chemin;

  if ( NULL == nom_fichier ) return FAUX;

  fichier = fopen(nom_fichier, "w");
  if ( NULL == fichier ) {
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, NULL, NULL);
    free( nom_fichier ); nom_fichier = NULL;
    return FAUX;
  }

  if (fprintf(fichier, "#LASE-EXT ("COPYRIGHT" 1999-2000 E. Curis)\n") <= 0) { /* En-tte pour reconnatre */
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, fichier, NULL);
    (void) fclose(fichier);
    free(nom_fichier); nom_fichier = NULL;
    return FAUX;
  }

  fprintf(fichier, "#Param"e_GRAVE"tres pour "MU"matrice\n");
  fprintf(fichier, "-MUM-\n");
  fprintf(fichier, "Type = ");
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_DROITE, QUESTION) == VRAI )
    fprintf(fichier, "droite\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT1, QUESTION) == VRAI )
    fprintf(fichier, "victorine r"e_AIGU"duite\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT2, QUESTION) == VRAI )
    fprintf(fichier, "victorine compl"e_GRAVE"te\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, E1F_MUM_POLY, QUESTION) == VRAI ) {
    fprintf(fichier, "polyn"o_CIRC"me\n");
    fprintf(fichier, "Deg. = %ld\n", OBJ_LireEntier_S(EXAFS_GENERAL, EG_MUM_DEGRE));
  }
  fprintf(fichier, "E1   = %f\n", OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E1, NULL));
  if ( OBJ_choisit(EXAFS_GENERAL, E1F_MUM_2_PTS, QUESTION) == VRAI)
    fprintf(fichier, "E1'  = %f\n", OBJ_LireNombre(EXAFS_GENERAL, E1F_MUM_VAL_E1P, NULL));

  fprintf(fichier, "#Valeurs pour la premi"e_GRAVE"re "e_AIGU"tape de l'extraction\n");
  fprintf(fichier, "-MU01-\n");
  fprintf(fichier, "E0   = %f\n", OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E0, NULL));
  fprintf(fichier, "E2   = %f\n", OBJ_LireNombre(EXAFS_GENERAL, EXF_VALEUR_E2, NULL));
  fprintf(fichier, "Deg. = %ld\n", OBJ_LireEntier_S(EXAFS_GENERAL, EXV_DEGRE_MU0_I));
  fprintf(fichier, "n    = %ld\n", OBJ_LireEntier_S(EXAFS_GENERAL, EXF_VALEUR_K));
  if ( OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, QUESTION) == VRAI ) {
    fprintf(fichier, "E2'  = %f\n", OBJ_LireNombre(EXAFS_GENERAL, E1F_MU1_VAL_E2P, NULL));
  }
  fprintf(fichier, "Met. = ");
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, QUESTION) == VRAI )
    fprintf(fichier, "normal\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT, QUESTION) == VRAI )
    fprintf(fichier, "pond"e_AIGU"r"e_AIGU"\n");

  fprintf(fichier, "# Valeurs pour la deuxi"e_GRAVE"me "e_AIGU"tape de l'extraction\n");
  fprintf(fichier, "-MU02-\n");
  fprintf(fichier, "Deg. = %ld\n", OBJ_LireEntier_S(EXAFS_GENERAL, EXF_DEGRE_MU0_II));
  fprintf(fichier, "Met. = ");
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, QUESTION) == VRAI )
    fprintf(fichier, "normal\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_LENT, QUESTION) == VRAI )
    fprintf(fichier, "pond"e_AIGU"r"e_AIGU"\n");

  fprintf(fichier, "# Valeurs pour la troisi"e_GRAVE"me "e_AIGU"tape de l'extraction\n");
  fprintf(fichier, "-MU03-\n");
  fprintf(fichier, "Met. = ");
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_ETAPE3_FIXE, QUESTION) == VRAI ) {
    fprintf(fichier, "valeur fixe\n");
    fprintf(fichier, "S    = %g\n", OBJ_LireNombre( EXAFS_GENERAL, EXF_VAL_S, NULL ));
  } else if ( OBJ_choisit(EXAFS_GENERAL, EXF_ETAPE3_SANS, QUESTION) == VRAI )
    fprintf(fichier, "ignorer\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_ETAPE3_VAR, QUESTION) == VRAI )
    fprintf(fichier, "refaire "a_GRAVE" chaque fois\n");

  fprintf(fichier, "# Param"e_GRAVE"tres de la normalisation\n");
  fprintf(fichier, "-NORM-\n");
  fprintf(fichier, "Met. = ");
  if ( OBJ_choisit(EXAFS_GENERAL, EXF_NORM_THEORIE, QUESTION) == VRAI )
    fprintf(fichier, "th"e_AIGU"orique\n");
  else if ( OBJ_choisit(EXAFS_GENERAL, EXF_NORM_MU0, QUESTION) == VRAI )
    fprintf(fichier, "coefficient d'absorption estim"e_AIGU"\n");

  free( nom_fichier ); nom_fichier = NULL;
  (void) fclose( fichier ); fichier = NULL;

  return VRAI; /* On a pu sauver */
}

/* ~~~~~~~~ Ralisation de l'interface ~~~~~~~~~~~~~~~~ */

/* -------- Gestion de la bote de dialogue -------------------------- */

static int GER_reproduit(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  boolean etat;

  switch (objet)
  {
/*   case EXF_MU_MATRICE: */
/*   case EXF_MU0_CALC: */
/*   case EXF_MU0_REFAIT: */
/*   case EXF_EFFACER: */
/*   case EXF_NORMALISER: */
/*   case EG_SERIE: */
/*     OBJ_choisit(EXAFS_GENERAL, objet, ECHANGE); */
/*     return VRAI; */

  case EXF_MUM_DROITE:
  case EXF_MUM_VICT1:
  case EXF_MUM_VICT2:
    OBJ_choisit(EXAFS_GENERAL, objet, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EG_MUM_DEGRE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

/*   case EXF_MU0_I_RAPIDE: */
/*   case EXF_MU0_I_LENT: */
/*   case EXF_MU0_II_RAPID: */
/*   case EXF_MU0_II_LENT: */
/*   case EXF_NORM_THEORIE: */
/*   case EXF_NORM_MU0: */
/*     OBJ_choisit(EXAFS_GENERAL, objet, FORCE_OUI); */
/*     return VRAI; */

  case E1F_MUM_POLY:
    OBJ_choisit(EXAFS_GENERAL, E1F_MUM_POLY, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EG_MUM_DEGRE, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case E1F_MUM_2_PTS:
    etat = OBJ_choisit(EXAFS_GENERAL, E1F_MUM_2_PTS, ECHANGE);
    OBJ_active(EXAFS_GENERAL, E1F_MUM_VAL_E1P, etat);
    return BI_EVENEMENT_TRAITE;

  case E1F_MU1_2_PTS:
    etat = OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, ECHANGE);
    OBJ_active(EXAFS_GENERAL, E1F_MU1_VAL_E2P, etat);
    return BI_EVENEMENT_TRAITE;

  case EXF_ETAPE3_SANS:
  case EXF_ETAPE3_FIXE:
    OBJ_choisit(EXAFS_GENERAL, objet, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EXF_NORMALISER, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EXF_NORM_THEORIE, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EXF_NORM_MU0, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EXF_VAL_S, objet == EXF_ETAPE3_FIXE);
    return BI_EVENEMENT_TRAITE;

  case EXF_ETAPE3_VAR:
    OBJ_choisit(EXAFS_GENERAL, objet, FORCE_OUI);
    OBJ_active(EXAFS_GENERAL, EXF_NORMALISER, FORCE_NON);
    OBJ_active(EXAFS_GENERAL, EXF_NORM_THEORIE, FORCE_NON);
    OBJ_active(EXAFS_GENERAL, EXF_NORM_MU0, FORCE_NON);
    OBJ_active(EXAFS_GENERAL, EXF_VAL_S, FORCE_NON);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre de reproduction -------- */

static void prepare_arbre_reproduit(void)
{
  if ( FAUX == arbres[ EXAFS_GENERAL ].drapeaux.init ) {
    int retour;

    OPTION_A_FAIRE( "Documentation, " );
    retour = ARB_Initialiser( EXAFS_GENERAL, "menu_exafs.html#reproduire", GER_reproduit, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  OBJ_deplace(EXAFS_GENERAL, E1F_MUM_2_PTS, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, E1F_MUM_VAL_E1P, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, EG_MUM_DEGRE, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(EXAFS_GENERAL, E1F_MU1_2_PTS, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, E1F_MU1_VAL_E2P, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, EXF_DEGRE_MU0_II, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, (EXF_MU0_II_RAPID - 1), 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, EXF_TXT_ETAPE3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, EXF_FND_ETAPE3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(EXAFS_GENERAL, EXF_VAL_S, 0, +3, DEPLACE_PIXELS);

#ifdef AVEC_TRADUCTION_
#endif

#ifndef _DEVELOPPE_
  OBJ_active(EXAFS_GENERAL, EXF_MU0_CALC, FORCE_NON);
  OBJ_active(EXAFS_GENERAL, EXV_TITRE_MU0, FORCE_NON);
#endif

  arbres[EXAFS_GENERAL].drapeaux.prepare = VRAI;
}
