/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 1999, Emmanuel CURIS
   Cration du fichier   :  8 dcembre 1999
   Dernire modification :  8 dcembre 1999

   Tout ce qui concerne l'extraction du signal EXAFS  partir du signal d'absorption brut

   Soustraction d'un 0 en une tape, car calcul par ailleurs
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "extrait.h"
#include "interface.h" /* Pour RCT_pointeur */
#include "lissage.h"
#include "science.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void RDS_mu0(Fenetre *fen, int x, int y, unsigned int larg, unsigned int haut);
static int  MSG_mu0(Fenetre *fen, Message *msg);
static int  GER_mu0(unsigned int num_objet,int x, int y);
static int  CLC_mu0(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

static int prepare_arbre_mu0(unsigned int);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Prparation de 0 avant extraction ------------------------------- */

void MU0_prepare(MU0_infos *infos)
{
  free(infos->b); free(infos->c); free(infos->d); /* On libre les anciens tableaux */
  SPL_tt_prep(infos->mu_0, &(infos->b), &(infos->c), &(infos->d));
}

/* -------- Extraction proprement dite ------------------------------------------ */

void MU0_extrait(Graphe *brut, MU0_infos *infos)
{
  register double *x;
  unsigned long nbr_points = 0, i;

  x = brut->x;
  for (i = 0, x = brut->x ; i < brut->nombre_point ; i++, x++) {
    if ( *x >= infos->e_0 ) /* Le point convient */
      nbr_points++;
  }

  infos->exafs->nombre_point = nbr_points; /* Nombre de points rel du spectre */

  if (infos->methode == MU0CALC_ECH_K) { /* 0 donn en chelle des k (1/A) */
    register double *zone_x = brut->x, *zone_y = brut->y;
    unsigned long idx_2 = 0;

    /* On rcupre la partie intressante */
    for (i = 0 ; i < brut->nombre_point ; i++) {
      if ( *(zone_x) >= infos->e_0 ) { /* Le point convient */
	*(infos->exafs->x + idx_2) = *(zone_x); /* Valeur en X */
	*(infos->exafs->y + idx_2) = *(zone_y);
	idx_2++;
      }
      zone_x++; zone_y++;
    }

    /* On convertit en chelle des k */
    EXF_e_v_k(infos->exafs, infos->e_0, FAUX);

    /* Et on soustrait _0 */
    zone_x = infos->exafs->x; zone_y = infos->exafs->y;
    for (i = 0 ; i < infos->exafs->nombre_point ; i++) {
      double y_mu0;  /* Valeur de mu_0 pour cette nergie */

      y_mu0 = SPL_tt_calc(infos->mu_0, *zone_x, infos->b, infos->c, infos->d);
      *zone_y -= y_mu0; /* On soustrait */
      
      if (y_mu0 != 0.0)
	*zone_y /= y_mu0;     /* On normalise */
      else {
	EXF_ERREUR(DIV_ZERO, MU"0 nul", txt_err_val_nulle);
	*zone_y = 0.0;
      }

      zone_x++; zone_y++;
    }	
  } else if (infos->methode == MU0CALC_ECH_EV) { /* 0 donn en chelle des E (eV) */
    register double *zone_x = brut->x, *zone_y = brut->y;
    unsigned long idx_2 = 0;

    for (i = 0 ; i < brut->nombre_point ; i++) {
      if ( *(zone_x) >= infos->e_0 ) { /* Le point convient */
	double y_mu0; /* Valeur de 0 pour cette nergie */

	*(infos->exafs->x + idx_2) = *(zone_x); /* Valeur en X */
	y_mu0 = SPL_tt_calc(infos->mu_0, *zone_x, infos->b, infos->c, infos->d);
	*(infos->exafs->y + idx_2) = *(zone_y) - y_mu0;
	
	if (y_mu0)
	  *(infos->exafs->y + idx_2) /= y_mu0;
	else {
	  EXF_ERREUR(DIV_ZERO, MU"0 nul", txt_err_val_nulle);
	}

	idx_2++;
      }
      zone_x++; zone_y++;
    }
    
    /* Reste  convertir en chelle des k le spectre EXAFS */
    EXF_e_v_k(infos->exafs, infos->e_0, FAUX);
  }
  
  /* On multiplie par k^n le rsultat */  
  EXF_k_p_n(infos->exafs, infos->degre, FAUX);
  
  GRF_bornes(infos->exafs, DESORDONNE, DESORDONNE); /* Calcul des bornes */
}

/* ~-~-~-~- Interface ~-~-~-~-~-~-~-~- */

/* -------- Ouverture de la fentre associe  cette tape -------- */

void MU0_ouvre(Graphe *brut, Graphe *mu_0)
{
  Fenetre *fenetre;
  MU0_infos *infos = NULL;
  char *titre;

  if (brut == NULL) return; /* Vraiment rien  faire */
  if (mu_0 == NULL) return;

  if ( arbres[OTE_MU0_CALC].drapeaux.ouvert == VRAI ) { /* Arbre dj ouvert */
    Graphe *vieux; /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_QuelleFenetre( &(arbres[OTE_MU0_CALC]), FAUX );
    vieux = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
    infos = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

    if ( (reponse == 1) || (vieux == brut) ) { /* On active seulement */
      FEN_premierplan(fenetre); /* Fentre au premier plan */
      return; /* Rien d'autre  faire */
    } else if (reponse == 2) { /* On rappelle, pour le nouveau graphe */
      if ( brut->drapeaux.protege ) {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      /* On dlie les informations */
      FEN_GRF_LierGraphe( fenetre, NULL );
      FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

      /* On libre les anciens graphes */
      vieux->drapeaux.protege = FAUX;
      infos->mu_0->drapeaux.protege = FAUX;
      GRF_efface(infos->exafs, VRAI, VRAI);
      
      FEN_premierplan(fenetre); /* Fentre au premier plan */
    } else { /* On annule l'appel */
      return;
    }
  } else { /* Arbre non ouvert : on l'ouvre */
    if ( brut->drapeaux.protege ) {
      EXF_ERREUR(GRF_PROTEGE);
      return;
    }

    fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, NULL, NULL,
				  OTE_MU0_CALC, BI_FEN_ARBRE_HAUT, prepare_arbre_mu0,
				  RDS_mu0, ( int (*)() ) NULL, CLC_mu0,
				  MSG_mu0, RCT_pointeur, ( int (*)() ) NULL);
    if (fenetre == NULL) return; /* Rien  faire... */

    FEN_Ouvrir( fenetre );
  }

  if (infos == NULL) { /* Aucune info n'existe dj */
    infos = (MU0_infos *) malloc( (size_t) sizeof(MU0_infos) );
    if (infos == NULL) {
      FEN_Detruire( fenetre );
      SYS_ERREUR(MANQUE_MEMOIRE, "EXAFS/theorique.c", "MU0_ouvre", "infos");
      return;
    }
    infos->b = NULL; infos->c = NULL; infos->d = NULL;
  }

  titre = TXT_genere_chaine( "Spectre E.X.A.F.S. de %s", brut->nom );
  infos->exafs = GRF_nouveau( brut, brut->nombre_point, GR_EXAFS1, GR_NORMAL, titre );
  free( titre ); titre = NULL;
  if (infos->exafs == NULL) {
    free( infos ); infos = NULL;
    FEN_Detruire( fenetre );
    return;
  }
  infos->mu_0 = mu_0;

  /* On lie les informations */
  FEN_GRF_LierGraphe( fenetre, brut );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos );

  infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
  brut->drapeaux.protege = VRAI; /* Calcul en cours sur ces spectres -> rien de faisable */
  mu_0->drapeaux.protege = VRAI;

  if ( OBJ_choisit(OTE_MU0_CALC, MU0CALC_ECH_K, QUESTION) == VRAI ) { /* chelle des nombres d'onde */
    infos->methode = MU0CALC_ECH_K;
    GRF_ChangerLegende( infos->exafs, AXE_X, "Nombre d'onde, k", VRAI);
    GRF_ChangerUnite( infos->exafs, AXE_X, NULL ); /* Unit par dfaut ! */
  }
  if ( OBJ_choisit(OTE_MU0_CALC, MU0CALC_ECH_EV, QUESTION) == VRAI ) { /* chelle des nergies */
    infos->methode = MU0CALC_ECH_EV;
    GRF_ChangerLegende( infos->exafs, AXE_X, E_AIGU"nergie", VRAI);
    GRF_ChangerUnite( infos->exafs, AXE_X, "eV" );
  }

  /* On cherche la valeur de E0 la plus plausible */
  infos->e_0   = cherche_e0  (brut, FAUX);                  /* E0 : maximum de la drive */
  infos->degre = OBJ_LireEntier_U(OTE_MU0_CALC, MU0CALC_DEGRE);

  MU0_prepare(infos);          /* Prparation */
  MU0_extrait(brut, infos);    /* Extraction  */

  fenetre->drapeaux.redessin = VRAI;
  fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */

  OBJ_MettreNombre( OTE_MU0_CALC, MU0CALC_E0, infos->e_0, 0.0 );
}

/* ------------- Redessin de la fentre ------------------------\
 |   Le clipping est dj install correctement lors de l'appel |
 |                                                              |
 | fenetre    : structure associe  la fentre considre      |
 | x, y       : coordonnes de la zone de redessin              |
 | larg, haut : dimensions de la zone de redessin               |
 \-------------------------------------------------------------- */

static void RDS_mu0(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  Graphe *graphe;
  MU0_infos *infos;
  double x_echelle, y_echelle;
  int x_barre, y_axe;

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  graphe = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  if ( NULL == graphe ) {
    TRC_Message( fenetre, "Pas de signal exp"e_AIGU"rimental !" );
  }

  if ( (graphe->x_max == graphe->x_min) || (graphe->y_max == graphe->y_min) ) {
    TRC_Message( fenetre, "Signal exp"e_AIGU"rimental nul" );
    return;
  }

  x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
  y_echelle = ( (double) fenetre->w_haut ) / (graphe->y_max - graphe->y_min);

  /* On trace la courbe exprimentale - en noir */
  TRC_Graphe( fenetre, graphe, NOIR, 1, 0, 0, fenetre->w_x, fenetre->w_y, 
	      graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle );

  /* On trace la courbe  soustraire - en bleu */
  if (infos->methode == MU0CALC_ECH_EV) { /* chelle d'nergie -> pas de problme */
    TRC_Graphe( fenetre, infos->mu_0, BLEU, 1, 0, 0, fenetre->w_x, fenetre->w_y,
		graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle );
  } else if (infos->methode == MU0CALC_ECH_K) { /* chelle des k ==> convertir avant de dessiner */
#ifdef ancien
    double coefficient = 2 * MASSE_ELECTRON * EV_V_JOULE / (H_BARRE * H_BARRE);
    double x;
    register double *zone_x = infos->mu_0->x, *zone_y = infos->mu_0->y;
    unsigned long i, compteur = 0;

    vsl_color(_PRG_station->handle, BLEU);

    for (i = 0 ; i < infos->mu_0->nombre_point ; i++, zone_x++, zone_y++) {
      x = *zone_x * M_V_ANGSTROM; /* Conversion en m^-1 */
      x *= x / coefficient;
      if ( x < 0 ) {
	x = infos->e_0 - x;
      } else {
	x += infos->e_0;
      }

      points[compteur]     = fenetre->w_x + x_echelle * (x - graphe->x_min);
      points[compteur + 1] = fenetre->w_y + y_echelle * (graphe->y_max - *zone_y);
      compteur += 2;
      if (compteur >= 256) { /* Tableau de points complet */
	v_pline(_PRG_station->handle, 128, points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }

    v_pline(_PRG_station->handle, (int) (compteur+1)/2, points);
#endif
    OPTION_A_FAIRE( "Reprendre l'affichage avec graphe isol !" );
  }

  /* On met les barres */
  TRC_Couleur( ROUGE ); /* Barre rouge : E0 */
  x_barre = TRC_ConvertitX_PvE( infos->e_0, graphe->x_min, 0, x_echelle );
  TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );

  /* On trace le graphe EXAFS extrait, en gris - en fin, car chelle diffrente */
  if ( (infos->exafs->x_max == infos->exafs->x_min) ||
       (infos->exafs->y_max == infos->exafs->y_min) ) {
#ifdef _ENGLISH_
    v_gtext(_PRG_station->handle,
	    fenetre->w_x + (fenetre->w_larg >> 1), fenetre->w_y + (fenetre->w_haut >> 1),
	    "The EXAFS obtained is nul");
#else
    TRC_Message( fenetre, "Signal obtenu nul" );
#endif
    return;
  }

  x_echelle = ( (double) fenetre->w_larg ) / (infos->exafs->x_max - infos->exafs->x_min);
  y_echelle = ( (double) fenetre->w_haut ) / (infos->exafs->y_max - infos->exafs->y_min);

  TRC_Graphe( fenetre, infos->exafs, GRIS_FONCE, 1, 0, 0, fenetre->w_x, fenetre->w_y,
	      infos->exafs->x_min, infos->exafs->x_max, infos->exafs->y_min, infos->exafs->y_max,
	      x_echelle, y_echelle );

  /* On trace l'axe des X associ, en violet */
  y_axe = TRC_ConvertitY_PvE( 0.0, infos->exafs->y_min, fenetre->w_haut, 1.0/y_echelle );
  TRC_Couleur( VIOLET );
  TRC_LigneH( fenetre, 0, y_axe, fenetre->w_larg );
}

/* -------------- Messages reus par la fentre ---------- */

static int MSG_mu0(Fenetre *fenetre, Message *message)
{
  int retour;
  Graphe *brut, *exafs = NULL, *mu_0 = NULL;
  MU0_infos *infos;
  char *titre;

  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  if (message->type == MSG_GRAPHE_MODIFIE) {
    if ( (Graphe *) (message->divers.pointeur) == brut ) {
      /* On a modifi le brut ==> on refait une extraction */
      MU0_extrait( brut, infos); /* Et on refait */
      FEN_redessin( fenetre, BI_RDS_UTIL );
    }

    if ( (Graphe *) (message->divers.pointeur) == infos->mu_0 ) {
      /* On a modifi le 0 */
      MU0_prepare( infos ); /* On recalcule donc les splines */
      MU0_extrait( brut, infos); /* Et on refait */
      FEN_redessin( fenetre, BI_RDS_UTIL );
    }

    return BI_EVENEMENT_TRAITE;
  }

  switch ( message->type ) /* Nature du message */
  {
  case WM_CLOSED: /* On veut fermer la fentre */
    if ( message->fenetre.haut == -1) /* Fermeture force */
      retour = 2;
    else {
      retour = EXF_ERREUR(FIN_MODULE);
      if (retour == 3) /* On annule la fermeture */
	return BI_EVENEMENT_TRAITE;
    }

    /* On dlie les informations */
    FEN_GRF_LierGraphe( fenetre, NULL );
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

    /* De toute faon, on ferme */
    if (infos != NULL) {
      exafs = infos->exafs;
      mu_0  = infos->mu_0;
      free( infos ); infos = NULL;
    }
    FEN_Detruire( fenetre );

    if (retour == 2) { /* On ferme sans soustraire */
      GRF_efface(exafs, VRAI, VRAI);

      if (mu_0 != NULL)
	mu_0->drapeaux.protege = FAUX;  /* On n'utilise plus le mu0   */
      if (brut != NULL)
	brut->drapeaux.protege = FAUX;

      GRF_aller(brut);                /* On va au spectre de dpart */
      return BI_EVENEMENT_FEN_FERMEE;
    }

    /* On veut extraire le spectre */
    titre = TXT_genere_chaine( "%s - "MU"0 pr"e_AIGU"calcul"e_AIGU, brut->nom );
    GRF_titre( exafs, titre );
    free( titre ); titre = NULL;

    exafs->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
    exafs->drapeaux.protege = FAUX; /* Graphe non protg a priori */
    GRF_aller(exafs); /* On va  ce graphe */
    mu_0->drapeaux.protege = FAUX;  /* Graphe non protg a priori */

    brut->drapeaux.protege = FAUX;

    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non trait [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------------- Clic sur la fentre ---------------------- */

static int CLC_mu0(Fenetre *fenetre, int bouton, int n_clic __attribute__((__unused__)), int etat __attribute__((__unused__)), int x_clic, int y_clic __attribute__((__unused__)))
{
  MU0_infos *infos;
  Graphe *graphe;
  double x_echelle;

#ifdef _TOS_
  int handle, _rien_;

  wind_get(0, WF_TOP, &handle, &n_clic, &etat, &_rien_);
  if (handle != fenetre->handle) {
    wind_set(fenetre->handle, WF_TOP);
    return VRAI;
  }

  if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + (int) fenetre->w_larg)
    || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + (int) fenetre->w_haut) ) {
    return VRAI; /* Clic en-dehors de la zone de travail */
  }
#endif

  graphe = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
  if (x_echelle) {
    /* graf_mouse(S_ABEILLE, NULL); */ /* Souris en abeille  */
    if (bouton == 1) { /* Clic du bouton gauche -> E0 */
      infos->e_0 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
      OBJ_MettreNombre( OTE_MU0_CALC, MU0CALC_E0, infos->e_0, 0.0 );
    }

    MU0_extrait(graphe, infos); /* On rextrait l'EXAFS */
/*     graf_mouse(S_FLECHE, NULL); */

    FEN_redessin(fenetre, BI_RDS_UTIL); /* On redessine */
  }

  return BI_EVENEMENT_TRAITE;
}

/* -------- Gestion de l'arbre ------------ */

static int GER_mu0(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  MU0_infos *infos;
  Graphe *brut;
  Fenetre *fenetre;

  fenetre = ARB_QuelleFenetre(arbres + OTE_MU0_CALC, FAUX); /* Fentre actuelle */
  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos = (MU0_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  switch (objet)
  {
  case MU0CALC_E0: /* Demande de la valeur de E0  la main */
    (void) fprintf(stderr, "%s,%s l.%u : A REFONDRE compltement = utilisation d'un 0 thorique\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_TRAITE;

  case MU0CALC_ECH_K:  /* chelle du 0 utilis */
  case MU0CALC_ECH_EV:
    OBJ_choisit(OTE_MU0_CALC, objet, FORCE_OUI);
    infos->methode = objet;
    MU0_extrait(brut, infos);   /* On refait l'extraction */
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;
    
  case MU0CALC_NOUVEAU: /* On veut changer de polynme */
    {
      Graphe *nv_mu0 = GRF_choisir(GR_MU0, VRAI, VRAI, NULL);
      if (nv_mu0 == NULL) return VRAI; /* Rien d'autre  faire */
      
      infos->mu_0->drapeaux.protege = FAUX; /* On n'utilise plus celui-ci */
      infos->mu_0 = nv_mu0;
      infos->mu_0->drapeaux.protege = VRAI;
      MU0_prepare(infos);
      MU0_extrait(brut, infos);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case MU0CALC_DEGRE: /* Degr de k^n * khi(k) pour le spectre final */
    {
      int vx_degre = infos->degre;
			
      (void) fprintf(stderr, "%s,%s l.%u : A REFONDRE compltement = utilisation d'un 0 thorique\n",
		     __FILE__, __FUNCTION__, __LINE__);

      EXF_k_p_n(infos->exafs, infos->degre - vx_degre, FAUX); /* Juste  modifier l'affichage */
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre -------- */

static int prepare_arbre_mu0(unsigned int numero_arbre __attribute__((__unused__)))
{
  if ( FAUX == arbres[ OTE_MU0_CALC ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( OTE_MU0_CALC, "menu_exafs.html#mu0_calc", GER_mu0, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  }

  arbres[ OTE_MU0_CALC ].drapeaux.prepare = VRAI;
  return 0;
}
