/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 1998, Emmanuel CURIS
   Cration de ce fichier : 26 octobre 1998
   Dernire modification : 26 octobre 1998

   Tout ce qui concerne l'import de fichiers contenant des graphes  des formats autres

   Import des fichiers crs  Hasylab
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "systeme.h"

#include "serie.h"

#include "ascii_utils.h"
#include "fichiers.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int     H_IgnoreBloc (FILE *fichier);
static Graphe *H_LireSpectre(const char *nom_fichier, FILE *fichier, Infos_Extraction *infos);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Graphe *IMP_Hasylab(const char *nom_fichier, Infos_Extraction *infos, MOD_Extrait *type);
{
  Graphe *spectre = NULL;
  FILE *fichier;
  int c;

  if ( NULL == nom_fichier ){
    (void) fprintf(stderr, "IMP_Hasylab : pas de fichier indiqu !\n");
    return NULL;
  }

  fichier = fopen( nom_fichier, "rb");
  if ( NULL == fichier ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "IMP_Hasylab : impossible d'ouvrir le fichier \"%s\"\n", nom_fichier);
#endif
    (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, NULL);
    return NULL;
  }

  /* Lecture proprement dite. Fichier organis par blocs... */
  c = fgetc( fichier );
  do {
    if ( '!' == c ) { /* Ligne de commentaires */
      c = ignore_ligne( fichier );
      (void) fgetc(fichier); /* Pour tre sr que le 1er caractre soit toujours lu, mais sans erreur en cas de EOF */
    } else if ( '%' == c ) { /* Bloc d'informations... */
      int type;

      type = fgetc( fichier );
      if ( EOF == type ) {
	(void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, fichier, "Fin de fichier atteinte, inattendue (type de bloc manquent)\n");
	break;
      } else if ( 'd' == type ) { /* Contient des donnes : spectre de type xanes ? */
	spectre = H_LireSpectre( nom_fichier, fichier, infos );
	c = fgetc( fichier );
      } else {
	c = H_IgnoreBloc( fichier );
      }
    } else if ( '$' == c ) { /* Bloc de donnes utilisables */
    } else {
      (void) fprintf(stderr, "IMP_Hasylab : type de bloc inconnu --- %c [%d]\n", c, c);
    }
  } while ( c != EOF );

  /* On a fini... */
  (void) fclose( fichier );
  return spectre;
}

/* -------- Ignorer un bloc de commentaires -------- */

static int H_IgnoreBloc( FILE *fichier )
{
  int c;

  /* Pour l'instant, on ignore purement et simplement le bloc ! */
  c = ignore_ligne(fichier);
  while (' ' == c )
    c = ignore_ligne( fichier );

  /* On est pass  autre chose... */
  return c;
}

/* -------- Lire un spectre dans un bloc %d -------- */

static Graphe *H_LireSpectre(const char *nom_fichier, FILE *fichier, Infos_Extraction *infos)
{
  int c, retour;
  unsigned int i, nbr_colonnes = 0;
  unsigned long nbr_lignes;
  struct graphe **graphes;

  if ( NULL == infos ) {
    (void) fprintf(stderr, "H_LireSpectre : pas d'infos pour calculer I0, I1,...\n");
    /* Pour bien faire, il faudrait passer en import ASCII pur */
    return NULL;
  }

  /* On ignore ventuellement la fin de la ligne du %d */
  (void) ignore_ligne(fichier);

  /* On compte le nombre de colonnes */
  do {
    (void) fgetc(fichier); /* espace initial, non informatif*/ 
    c = fgetc( fcihier );
    if ( 'C' == c ) { /* C = Col, indique une colonne... */
      nbr_colonnes++;
      (void) ignore_ligne( fichier );
    }
  } while ( 'C' == c );
  if ( 1 >= nbr_colonnes ) {
    (void) fprintf(stderr, "H_LireSpectre : nombre de colonne incorrect [%u]\n", nbr_colonnes);
    return NULL;
  }
  /* On est donc l au dbut des donnes... */

  /* Combien de lignes  prendre en compte ? */
  nbr_lignes = compte_lignes( fichier, '!' );

  /* Prparation des graphes contenant le rsultat */
  infos->nbr_colonnes = nbr_colonnes;
  retour = ELEM_PreparerGraphes( infos, type, nbr_lignes, nom_fichier );
  if ( 0 != retour ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "H_LireSpectre : ERREUR dans ELEM_PreparerGraphes()\n");
    return NULL;
#endif
  }

  /* Chargement proprement dit */
}
