/* LASE : Logiciel d'Analyse des Spectres E.X.A.F.S.
   (c) Emmanuel CURIS 1998-2002

   Relecture des fichiers FEFF.INP

 Commandes de dbogage : D_FEFF_

 16, 25, 30 mars ; 28 & 29 janvier 1998
   * Chargement des fichiers FEFF : ligne TITLE vide provoquait un dcalage -> chargement foireux...
   * Remplac graphes.h par erreurs.h : pour dfinition de is_nan()
   * Utilisation de FEFF_defauts = paramtres par dfaut
   * Chargement : compteur de lignes tait trop incrment... (2 fois par tour !)
   * Fonction d'appel du dcodage d'une ligne (reprise  lit_ccdb)
   * Dcodage d'une ligne : reconnat tout sauf OVERLAP (les paramtres)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h> /* Pour isalpha */

#include "systeme.h"

#include "atomes.h"
#include "feff.h"
#include "fichiers.h"
#include "modeles.h"

#include "feff_charger.txt"

/* ~~~~~~~~ variables locales ~~~~~~~~ */

struct feff_infos {
  Modele_FEFF *modele;
  Molecule *molecule;
  int mode;
  unsigned int Z_absorbeur;
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Modele_FEFF *FEFF_fichier(const char *nom);

static boolean FEFF_ControleModele(struct feff_infos *infos);

static boolean FEFF_LireCommande  (FILE *fichier, char *ligne, struct feff_infos *infos);
static boolean FEFF_LireAtomes    (FILE *fichier, struct feff_infos *infos);
static boolean FEFF_LireOverlap   (FILE *fichier, struct feff_infos *infos, int potentiel);
static boolean FEFF_LirePotentiels(FILE *fichier, struct feff_infos *infos);

static boolean FEFF_LireAbsorbeur(struct feff_infos *infos, Vecteur3D *position);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Prparation du chargement : - choix du fichier
                                        - effacement de l'ancien modle
                                        - chargement effectif -------------------- */

#define TOUT_VA_BIEN  -1 /* Retours possibles de la fonction de chargement */
#define NOM_VIDE       1

Modele_FEFF *FEFF_charge(void)
{
  Modele_FEFF *modele;
  char **liste_noms;
  const char *ext[1] = {"*.inp"};
  unsigned int nbr_fichiers;

  liste_noms = CH_selecteur( "feff.inp", titre_selecteur_fichier, 1, ext, &nbr_fichiers, principale );
  if (liste_noms == NULL) return NULL; /* Aucun fichier  traiter */

  if (nbr_fichiers > 1) {
    unsigned int i;

    (void) SYS_ERREUR(TROP_FICHIERS);
    for (i = 1 ; i < nbr_fichiers ; i++) {
      free(liste_noms[i]);
    }
  }

  /* On charge maintenant le nouveau fichier */
  modele = FEFF_fichier( liste_noms[ 0 ] );
  free( liste_noms[ 0 ] ); /* Plus besoin d'eux */
  free( liste_noms );

  if ( NULL == modele ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur survenue\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }
  return modele;
}

/* ************** Lecture proprement dite du fichier ************************ */

#define FEFF_NORMAL     0 /* Lecture normale d'un fichier FEFF           */
#define FEFF_ATOMS      1 /* Liste de coordonnes atomiques              */
#define FEFF_POTENTIALS 2 /* Liste des potentiels */
#define FEFF_OVERLAP    3 /* Liste de couches pour l'instruction OVERLAP - 3 : potentiel 0
                                                                           4 : potentiel 1 ... */

#define COMMENTAIRE    1
#define LIGNE_VIDE    -5

Modele_FEFF *FEFF_fichier(const char *nom)
{
  Absorbeur *absorbeur;
  FILE *feff_inp;
  char *ligne;
  int i;
  struct feff_infos infos = { NULL, NULL, FEFF_NORMAL, 0 };
  struct modele_ajuste *modele;
  boolean fin_fichier = FAUX;

  if ( NULL == nom ) return NULL;

  feff_inp = fopen( nom, "rb" );
  if ( feff_inp == NULL ) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom, NULL, NULL);
    return NULL;
  }

  infos.modele = FEFF_CreerModele();
  if ( NULL == infos.modele ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le modle initial\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On initialise les potentiels pour les distinguer des Z et entre eux :
     si on lit d'abord les atomes, puis les potentiels, on peut faire la table aprs... */
  for ( i = 0 ; i < 8 ; i++ ) {
    infos.modele->potentiels[ i ] = -i;
  }

  /* On fait la lecture */
  do {
    ligne = lit_ligne(feff_inp, 0); /* Pas de longueur de ligne prvisible... */
    if (NULL == ligne) { /* Fin de fichier atteinte */
      fin_fichier = VRAI;
    } else if ( '*' == *ligne ) { /* Ligne de commentaire */
      /* Rien  faire */
    } else if ( isalpha( *ligne ) ) { /* Commence par une lettre : commande */
      fin_fichier = FEFF_LireCommande(feff_inp, ligne, &infos);
    } else {
      (void) fprintf(stderr, "FEFF_fichier : Ligne de rle inconnu [%s] dans %s\n", ligne, nom);
    }
    free(ligne);
  } while ( fin_fichier != VRAI );

#ifdef _DEVELOPPE_
  printf( "%s : fichier termin...\n", __FUNCTION__ );
#endif

  (void) fclose(feff_inp);

  /* Contrle de la correspondance potentiel / Z */
  if ( FAUX == FEFF_ControleModele( &infos) ) {
    FEFF_EffacerModele( infos.modele );
    return NULL;
  }

  /* Maintenant, il faut assigner le modle  l'atome absorbeur choisi */
  /* 1. si ce n'est pas un absorbeur, on le fait */
  if ( FAUX == infos.modele->absorbeur->marques.absorbeur ) {
    absorbeur = creer_absorbeur( infos.modele->absorbeur, "Absorbeur FEFF", 1.0 );
    if ( NULL == absorbeur ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer l'absorbeur pour l'atome absorbeur\n",
		     __FILE__, __FUNCTION__, __LINE__);
      /* On n'efface pas la molcule, car peut avoir t cre avant le modle FEFF ! */
      FEFF_EffacerModele( infos.modele );
      return NULL;
    }
  } else {
    absorbeur = ATM_QuelAbsorbeur( infos.molecule, infos.modele->absorbeur );
  }

  /* 3. on y associe le modle [si dj un : on demande si on remplace ou ajoute] */
  if ( NULL != absorbeur->modeles ) {
    struct modele_ajuste *modele;

    /* Les autres modles sont-ils de type FEFF ? */
    for ( modele = absorbeur->modeles ; modele != NULL ; modele = modele->suivant ) {
      if ( MODELE_EXAFS_CHEMINS == modele->type ) break;
    }
    if ( modele != NULL ) {
      int reponse;

      reponse = BI_ERR_alerte( NULL, &Q_RemplacerModele, VRAI );
      if ( 1 == reponse ) {
	Modele_FEFF *vx_modele;

	vx_modele = modele->infos_modele.feff;
	modele->infos_modele.feff = infos.modele;

	/* Plus besoin de l'ancien */
	FEFF_EffacerModele( vx_modele );
	return infos.modele; /* Fini */
      }
    }
  }

  modele = MODELE_CreerModele( NULL, "Mod"e_GRAVE"le FEFF", MODELE_EXAFS_CHEMINS, NULL, FAUX );
  if ( NULL == modele ) {
    /* On n'efface pas la molcule, car peut avoir t cre avant le modle FEFF ! */
    FEFF_EffacerModele( infos.modele );
    return NULL;
  }
  modele->infos_modele.feff = infos.modele;
  ABS_AjouterModele(absorbeur, modele);

  return infos.modele;
}

/* -------- Analyse d'une ligne comportant une commande --------\
 |   Renvoie VRAI si l'on a atteint la fin du fichier           |
 |                                                              |
 | fichier = le fichier o lire les infos complmentaires       |
 | ligne   = la ligne  analyser                                |
 | infos   = infos pour construire le modle FEFF               |
 \------------------------------------------------------------- */

static boolean FEFF_LireCommande(FILE *fichier, char *ligne, struct feff_infos *infos)
{
  char *commande, *parametres;

  commande = ligne; /* Dbut de la commande */
  *commande = toupper( *commande ); /* FEFF travaille en majuscules */

  /* On cherche la fin de la commande */
  parametres = ligne + 1;
  while ( isalpha( *parametres ) ) {
    *parametres = toupper( *parametres ); /* FEFF travaille en majuscules */
    parametres++;
  }
  if ( *parametres != 0 ) {
    *parametres = 0; /* Fin de ligne : fin de la commande */
    parametres++;    /* Dbut des paramtres */
  } else {
    parametres = NULL;
  }

#ifdef _DEVELOPPE_
  (void) fprintf(stderr, "feff.inp : commande = %s, paramtres = %s\n", commande, parametres);
#endif

  /* On essaye de reconnatre la commande
     Remarque : FEFF n'utilise que les 4 premiers caractres, mais je prfre utiliser la
     commande complte... */

  if ( strcmp(commande, "END") == 0 ) { /* commande END */
    /* On ne change pas le mode, car peut se trouver au milieu des atomes ou... */
    if ( 2 == BI_ERR_alerte(NULL, &Q_CommandeEND, VRAI) )
      return VRAI; /* On fait comme si on tait arriv  la fin du fichier */
  }

  /* Toutes les autres commandes indiquent que l'on a fini avec la prcdente */
  infos->mode = FEFF_NORMAL;

  if ( strcmp(commande, "AFOLP") == 0 ) { /* commande AFOLP */
    infos->modele->drapeaux.av_AFOLP = VRAI;
  } else if ( strcmp(commande, "ATOMS") == 0 ) { /* commande ATOMS */
    infos->mode = FEFF_ATOMS;
    return FEFF_LireAtomes(fichier, infos);
  } else if ( strcmp(commande, "CONTROL") == 0 ) { /* commande CONTROL */
    int ctrl[4], i;

    sscanf(parametres, "%d %d %d %d", ctrl, ctrl + 1, ctrl + 2, ctrl + 3);
    for( i = 0 ; i < 4 ; i++) {
      if ( 0 == ctrl[i] )
	infos->modele->parametres.CONTROL[i] = NON_LANCE; /* Sinon, on garde la valeur existante */
    }
  } else if ( strcmp(commande, "CORRECTIONS") == 0 ) { /* commande CORRECTIONS */
    sscanf(parametres, "%f %f", &infos->modele->parametres.CORRECTIONS.decalage,
	   &infos->modele->parametres.CORRECTIONS.elargissement);
  } else if ( strcmp(commande, "CRITERIA") == 0 ) {    /* commande CRITERIA */
    sscanf(parametres, "%f %f", &infos->modele->parametres.CRITERIA.critcw, &infos->modele->parametres.CRITERIA.critpw );
  } else if ( strcmp(commande, "CSIG") == 0 ) {        /* commande CSIG : inutile d'aprs rdinp.f FEFF 7.0 */
    (void) BI_ERR_alerte(NULL, &INF_CommandeObsolete, VRAI, "CSIG");
  } else if ( strcmp(commande, "CUSTOMCRIT") == 0 ) {  /* commande CUSTOMCRIT : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande CUSTOMCRIT %%f %%f %%f, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "DEBYE") == 0 ) {       /* commande DEBYE */
    sscanf(parametres, "%f %f", &infos->modele->parametres.DEBYE.temperature, &infos->modele->parametres.DEBYE.t_debye);
  } else if ( strcmp(commande, "EDGE") == 0 ) {        /* commande EDGE : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande EDGE %%s %%f, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "EGRID") == 0 ) {       /* commande EGRID : FEFF 7 modifi xanes */
    (void) fprintf(stderr, "FEFF_LireCommande : commande EGRID, Feff 7.02 modif xanes.  FAIRE\n");
  } else if ( strcmp(commande, "ELLIPTICITY") == 0 ) { /* commande ELLIPTICITY */
    sscanf(parametres, "%d %lf %lf %lf", &infos->modele->parametres.ELLIPTICITY.circulaire,
	   &infos->modele->parametres.ELLIPTICITY.vecteur.x, &infos->modele->parametres.ELLIPTICITY.vecteur.y, &infos->modele->parametres.ELLIPTICITY.vecteur.z );
  } else if ( strcmp(commande, "EMESH") == 0 ) {       /* commande EMESH : FEFF 7 modifi xanes */
    (void) fprintf(stderr, "FEFF_LireCommande : commande EMESH, Feff 7.02 modif xanes.  FAIRE\n");
  } else if ( strcmp(commande, "EXCHANGE") == 0 ) {    /* commande EXCHANGE */
    sscanf(parametres, "%d %f %f", &infos->modele->parametres.EXCHANGE.type, &infos->modele->parametres.EXCHANGE.decalage,
	   &infos->modele->parametres.EXCHANGE.p_imag);
  } else if ( strcmp(commande, "FOLP") == 0 ) {        /* commande FOLP */
    int potentiel;
    float recouvre;

    sscanf(parametres, "%d %f", &potentiel, &recouvre);
    if ( (potentiel < 0) || (potentiel > 8) ) {
      (void) BI_ERR_alerte(NULL, &ERR_PotentielFaux, VRAI, potentiel, "FOLP");
    } else
      infos->modele->parametres.FOLP[potentiel] = recouvre - 1.0;
  } else if ( strcmp(commande, "HOLE") == 0 ) {        /* commande HOLE */
    int seuil;
    double s02;

    sscanf(parametres, "%d %lf", &seuil, &s02);
    infos->modele->S0_2.valeur_initiale = s02;
    infos->modele->seuil = seuil - 1; /* FEFF : 1 = Seuil K */
  } else if ( strcmp(commande, "ION") == 0 ) {         /* commande ION */
    int potentiel;
    float charge;
    
    sscanf(parametres, "%d %f", &potentiel, &charge);
    if ( (potentiel < 0) || (potentiel > 8) ) {
      (void) BI_ERR_alerte(NULL, &ERR_PotentielFaux, VRAI, potentiel, "ION");
    } else
      infos->modele->parametres.ION[potentiel] = charge;
  } else if ( strcmp(commande, "IORDER") == 0 ) {      /* commande IORDER */
    (void) BI_ERR_alerte(NULL, &INF_CommandeIORDER, VRAI);
  } else if ( strcmp(commande, "INTCALC") == 0 ) {     /* commande INTCALC : FEFF 7/8 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande INTCALC, non documente dans Feff 6.  FAIRE\n"
		   "  [Remarque : marque comme non implmente dans rdinp.f / feff 7.02]\n");
  } else if ( strcmp(commande, "JUMPRM") == 0 ) {      /* commande JUMPRM : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande JUMPRM, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "KLIMIT") == 0 ) {      /* Commande KLIMIT : obsolte */
    (void) BI_ERR_alerte(NULL, &INF_CommandeObsolete, VRAI, "KLIMIT");
  } else if ( strcmp(commande, "MBCONV") == 0 ) {      /* commande MBCONV : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande , non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "NEMAX") == 0 ) {       /* commande NEMAX */
    sscanf(parametres, "%d", &infos->modele->parametres.NEMAX);
  } else if ( strcmp(commande, "NLEG") == 0 ) {        /* commande NLEG */
    sscanf(parametres, "%d", &infos->modele->parametres.NLEG);
  } else if ( strcmp(commande, "NOGEOM") == 0 ) {      /* commande NOGEOM */
    infos->modele->drapeaux.av_NOGEOM = VRAI;
  } else if ( strcmp(commande, "OVERLAP") == 0 ){      /* commande OVERLAP */
    int potentiel;

    sscanf(parametres, "%d", &potentiel);
    if ( (potentiel < 0) || (potentiel > 8) ) {
      (void) BI_ERR_alerte(NULL, &ERR_PotentielFaux, VRAI, potentiel, "OVERLAP");
    }
    return FEFF_LireOverlap(fichier, infos, potentiel);
  } else if ( strcmp(commande, "PCRITERIA") == 0 ) {   /* commande PCRITERIA */
    sscanf(parametres, "%f %f", &infos->modele->parametres.PCRITERIA.conservation, &infos->modele->parametres.PCRITERIA.tas );
  } else if ( strcmp(commande, "POLARIZATION") == 0 ) {/* commande POLARIZATION */
    infos->modele->parametres.POLARIZATION.presente = FAUX;
    sscanf(parametres, "%lf %lf %lf",
	   &infos->modele->parametres.POLARIZATION.vecteur.x, &infos->modele->parametres.POLARIZATION.vecteur.y, &infos->modele->parametres.POLARIZATION.vecteur.z );
  } else if ( strcmp(commande, "POTENTIALS") == 0) {   /* Commande POTENTIALS */
    return FEFF_LirePotentiels(fichier, infos);
  } else if ( strcmp(commande, "PRINT") == 0 ) {       /* Commande PRINT */
    short int *ctrl;

    ctrl = infos->modele->parametres.CONTROL;
    sscanf(parametres, "%hd %hd %hd %hd", ctrl, ctrl + 1, ctrl + 2, ctrl + 3);
  } else if ( strcmp(commande, "NOHOLE") == 0 ) {      /* commande NOHOLE : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande NOHOLE, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "NSTAR") == 0 ) {       /* commande NSTAR : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande NSTAR, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "REQUIRE") == 0 ) {     /* Commande REQUIRE : obsolte */
    (void) BI_ERR_alerte(NULL, &INF_CommandeObsolete, VRAI, "REQUIRE");
  } else if ( strcmp(commande, "RGRID") == 0 ) {       /* commande RGRID : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande RGRID %%f, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "RMAX") == 0 ) {        /* Commande RMAX */
    sscanf(parametres, "%f", &infos->modele->parametres.RMAX);
  } else if ( strcmp(commande, "RMULTIPLIER") == 0 ) { /* Commande RMULTIPLIER */
    sscanf(parametres, "%f", &infos->modele->parametres.RMULTIPLIER);
  } else if ( strcmp(commande, "RPHASES") == 0 ) {     /* commande RPHASES : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande RPHASES, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "SIG2") == 0 ) {        /* Commande SIG2 */
    sscanf(parametres, "%f", &infos->modele->parametres.SIG2);
  } else if ( strcmp(commande, "SIG3") == 0 ) {        /* commande SIG3 : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande SIG3 %%f, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "SPIN") == 0 ) {        /* commande SPIN : FEFF 7 ? */
    (void) fprintf(stderr, "FEFF_LireCommande : commande SPIN, non documente dans Feff 6.  FAIRE\n");
  } else if ( strcmp(commande, "SS") == 0 ) {          /* commande SS */
    unsigned int numero;
    int potentiel, degenerescence;
    double distance;

    sscanf(parametres, "%u %d %d %lf", &numero, &potentiel, &degenerescence, &distance);
    FEFF_AjouterCouche(infos->modele, numero, potentiel, degenerescence, distance);
  } else if ( strcmp(commande, "TITLE") == 0 ) {       /* commande TITLE */
    int num_titre = 0;

    while (infos->modele->parametres.titre[num_titre] != NULL) {
      num_titre++;
      if (num_titre > 9) { /* Dix commandes TITLE au maximum */
	(void) fprintf(stderr, "FEF_LireCommande : le fichier contient trop de commandes TITLE, on l'ignore [%s]\n", parametres);
	return VRAI;
      }
    }

    while ( isspace( *parametres ) )
      parametres++;
    if ( *parametres != 0 ) { /* Le titre n'est pas vide */
      infos->modele->parametres.titre[num_titre] = strdup(parametres);
    }
  } else if ( strcmp(commande, "VINT") == 0 ) {        /* commande VINT : FEFF 7 modifi xanes */
    (void) fprintf(stderr, "FEFF_LireCommande : commande VINT, Feff 7.02 modif xanes.  FAIRE\n");
  } else if ( strcmp(commande, "XANES") == 0 ) {       /* commande XANES */
    infos->modele->drapeaux.av_XANES = VRAI;
    (void) fprintf(stderr, "FEFF_LireCommande : commande XANES, paramtres Feff 7.02 modif xanes  FAIRE\n");
  } else if ( strcmp(commande, "XION") == 0 ) {        /* commande XION : FEFF 7 modifi xanes */
    (void) fprintf(stderr, "FEFF_LireCommande : commande XION, Feff 7.02 modif xanes.  FAIRE\n");
  } else { /* Autre commande -> on regarde si on n'est pas dans un mode spcial... */
    int reponse;

    reponse = BI_ERR_alerte(NULL, &ERR_CommandeInconnue, VRAI, commande);
    if (2 == reponse) return VRAI; /* On annule... */
  }

  return FAUX;
}

/* -------- Lecture des coordonnes des atomes ----------\
 |  Renvoie VRAI s'il reste des lignes aprs, FAUX sinon |
 \------------------------------------------------------ */

static boolean FEFF_LireAtomes(FILE *fichier, struct feff_infos *infos)
{
  Vecteur3D position;
  int potentiel;
  int n_lu;

  do {
    n_lu = fscanf(fichier, "%lf %lf %lf %d", &(position.x), &(position.y), &(position.z), &potentiel);
    if (0 == n_lu) {
      /* On est sur autre-chose qu'un atome */
      return VRAI;
    } else if ( EOF == n_lu ) {
      /* On a atteint la fin du fichier */
      return FAUX;
    } else if (4 == n_lu) {
      if ( 0 == potentiel ) { /* On lit l'absorbeur */
	if ( FAUX == FEFF_LireAbsorbeur( infos, &position ) ) {
	  (void) fprintf(stderr, "%s,%s l.%u : Erreur en lisant l'absorbeur\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  return FAUX;
	}
      } else if ( ( potentiel < 0 ) || ( potentiel > 8 ) ) {
	FEFF_ERREUR( POTENTIEL_FAUX, potentiel );
	(void) fprintf(stderr, "%s,%s l.%u : potentiel incorrect (%d), atome (%f, %f, %f) ignor\n",
		       __FILE__, __FUNCTION__, __LINE__, potentiel, position.x, position.y, position.z );
      } else { /* On lit un autre atome */
	if ( NULL == infos->molecule ) {
	  (void) fprintf(stderr, "FEFF_LireAtomes : le premier atome lu n'est pas l'atome absorbeur.\n");
	  (void) fprintf(stderr, "\t==> pour limiter les risques, on force la cration d'une nouvelle molcule\n");

	  infos->molecule = MOL_creer( infos->modele->parametres.titre[1] ); /* 2e titre = nom du compos */
	  if ( NULL == infos->molecule ) {
	    (void) fprintf(stderr, "FEFF_LireAtomes : impossible de crer la molcule !\n");
	    return FAUX;
	  }
	}

	(void) ATM_creer( infos->molecule, NULL, infos->modele->potentiels[ potentiel ], &position, 0.0, -1.0 );
      }
    }

    (void) ignore_ligne(fichier); /* Fin de la ligne : commentaire */
  } while (4 == n_lu);

  /* Si on est l, il y a eu un problme de lecture ! */
  (void) fprintf(stderr, "FEFF_LireAtomes : erreur de lecture, on a pu lire seulement %d valeurs\n", n_lu);
  (void) SYS_ERREUR(ERREUR_LECTURE, "<feff.inp>", fichier, "Format de fichier incorrect");
  return VRAI;
}

/* -------- Lecture de l'absorbeur, la plus intelligente possible -------- */

static boolean FEFF_LireAbsorbeur(struct feff_infos *infos, Vecteur3D *position)
{
  Modele_FEFF *modele;

  modele = infos->modele;

  /* 1. Est-on bien le premier (et unique) absorbeur ? */
  if ( modele->absorbeur != NULL ) {
    (void) fprintf(stderr, "FEFF_LireAbsorbeur : absorbeur en double.\n\tOriginal : (%f, %f, %f) - Copie : (%f, %f, %f)\n",
		   modele->absorbeur->position.x, modele->absorbeur->position.y, modele->absorbeur->position.z, position->x, position->y, position->z);
    (void) BI_ERR_alerte(NULL, &ERR_DejaAbsorbeur, VRAI);
    return VRAI; /* On ignore l'atome, mais n'empche pas la lecture de la suite */
  }

  /* 2. Quelle molcule contient le modle ? */
  if ( infos->molecule == NULL ) {    
    /* Aucune actuellement : on cherche, parmi les existantes, si une contient dj l'atome absorbeur indiqu */
    if ( modele->potentiels[0] == 0 ) {
      (void) fprintf(stderr, "FEFF_LireAbsorbeur : atome lu, potentiels pas encore dfinis ! Rsultat douteux...\n");
      (void) fprintf(stderr, "\t==> pour limiter les risques, on force la cration d'une nouvelle molcule\n");
    } else {
      /* D'abord, dans la molcule courante */
      modele->absorbeur = ATM_chercher( position, modele->potentiels[0], mol_courante );
      if ( NULL == modele->absorbeur ) {
	/* Si a rate, dans toutes les molcules */
	modele->absorbeur = ATM_chercher( position, modele->potentiels[0], NULL );
      }
    }

    /* Impossible de trouver un absorbeur : on en cre un, avec une nouvelle molcule */
    if ( NULL == modele->absorbeur ) {
      Molecule *molecule;

      molecule = MOL_creer( modele->parametres.titre[1] ); /* 2e titre = nom du compos */
      if ( NULL == molecule ) {
	(void) fprintf(stderr, "FEFF_LireAbsorbeur : impossible de crer la molcule !\n");
	return FAUX;
      }

      modele->absorbeur = ATM_creer( molecule, NULL, modele->potentiels[0], position, 0.0, -1.0 );
      if ( NULL == modele->absorbeur ) {
	(void) fprintf(stderr, "FEFF_LireAbsorbeur : impossible de crer l'atome absorbeur !\n");
	MOL_effacer(molecule, VRAI, VRAI);
	return FAUX;
      }

      infos->molecule = molecule;
    } else {
      infos->molecule = ATM_QuelleMolecule( modele->absorbeur );
    }
  } else {
    /* S'il existe dj une molcule, c'est que l'on a charg des atomes avant
       ==> c'est une molcule cre exprs pour : on doit y crer l'absorbeur */

    modele->absorbeur = ATM_creer( infos->molecule, NULL, modele->potentiels[0], position, 0.0, -1.0 );
    if ( NULL == modele->absorbeur ) {
      (void) fprintf(stderr, "FEFF_LireAbsorbeur : impossible de crer l'atome absorbeur !\n");
      MOL_effacer(infos->molecule, VRAI, VRAI);
      return FAUX;
    }
  }

  return VRAI; /* Tout s'est bien pass, on peut continuer  lire */
}

/* -------- Lecture d'une commande OVERLAP -------- */

static boolean FEFF_LireOverlap(FILE *fichier, struct feff_infos *infos, int potentiel)
{
  (void) fprintf(stderr, "FEFF_LireOverlap : ENCORE  FAIRE, fichier = %p, infos = %p, potentiel %d\n",
		 (void *) fichier, (void *) infos, potentiel);
  return VRAI;
}

/* -------- Lecture des potentiels --------
   Renvoie VRAI s'il reste des lignes aprs, FAUX sinon
*/

static boolean FEFF_LirePotentiels(FILE *fichier, struct feff_infos *infos)
{
  int n_lu;

#ifdef _DEVELOPPE_
  printf("==> On entre dans FEFF_LirePotentiels\n");
#endif

  do {
    int potentiel;
    unsigned int Z;

    n_lu = fscanf(fichier, "%d %u", &potentiel, &Z);
    if ( 0 == n_lu ) {
#ifdef _DEVELOPPE_
      printf("==> On sort de FEFF_LirePotentiels (table de potentiels lue)\n");
#endif
      return FAUX; /* Autre chose qu'un potentiel => il reste des lignes ! */
    } else if ( EOF == n_lu ) {
      return VRAI; /* Fin de fichier atteinte */
    } else if ( 2 == n_lu ) { /* Lecture correcte */
      if ( (potentiel >= 0) && (potentiel <= 8) ) {
	if ( infos->modele->potentiels[ potentiel ] > 0 ) {
	  (void) BI_ERR_alerte(NULL, &ERR_PotentielDejaDefini, VRAI, potentiel, Z);
	}
	infos->modele->potentiels[ potentiel ] = Z;
#ifdef _DEVELOPPE_
	printf("Potentiel lu : %d / %d\n", potentiel, Z);
#endif
      } else {
	(void) BI_ERR_alerte(NULL, &ERR_PotentielFaux, VRAI, potentiel, "POTENTIALS");
      }
    }

    (void) ignore_ligne(fichier); /* On ignore les commentaires qui suivent */
  } while (2 == n_lu);

  /* Si on est l, il y a eu un problme de lecture ! */
  (void) fprintf(stderr, "FEFF_LirePotentiels : erreur de lecture, on a pu lire seulement %d valeurs\n", n_lu);
  (void) SYS_ERREUR(ERREUR_LECTURE, "<feff.inp>", fichier, "Format de fichier incorrect");
  return FAUX; /* Mais il reste, a priori, des lignes */
}

/* -~-~-~-~-~-~-~-~ Contrle final du modle -~-~-~-~-~-~-~-~ */

/* -------- Contrle de la correspondance absorbeur/potentiel -------- */

static boolean FEFF_ControleModele(struct feff_infos *infos)
{
  Atome *atome;

  if ( NULL == infos ) return FAUX;

  /* 1. Y a-t-il une molcule associe ? */
  if ( NULL == infos->molecule ) { /* NON ! */
    if ( 0 == infos->modele->potentiels[0] ) {
#ifdef _DEVELOPPE_
      unsigned int i;

      (void) fprintf(stderr, "FEFF_ControleModele : aucun potentiel dfini, aucune molcule !\n");
      for (i = 0 ; i < 8 ; i++) {
	printf("Potentiel %d : %d\n", i, infos->modele->potentiels[i]);
      }
#endif
      (void) BI_ERR_alerte(NULL, &ERR_StructureVide, VRAI);
      return FAUX;
    }

    infos->molecule = MOL_creer("Mod"e_GRAVE"le FEFF");
    if ( NULL == infos->molecule ) {
      (void) fprintf(stderr, "FEFF_ControleModele : Impossible de crer la molcule\n");
      return FAUX;
    }
  }

  /* 2. Y a-t-il un atome pour le potentiel 0 ? */
  if ( NULL == infos->modele->absorbeur ) { /* NON ! */
    if ( NULL != infos->molecule->premier ) { /* On avait commenc une molcule, qui a dj d'autres atomes */
      (void) BI_ERR_alerte(NULL, &INF_AtomesSansAbsorbeur, VRAI, infos->modele->potentiels[0]);
    }

    infos->modele->absorbeur = ATM_creer( infos->molecule, NULL, infos->modele->potentiels[0], NULL,
					  infos->modele->parametres.ION[0], -1.0 );
    if ( NULL == infos->modele->absorbeur ) {
      (void) fprintf(stderr, "FEFF_fichier : Impossible de crer l'atome absorbeur\n");
      if (NULL == infos->molecule->premier) { /* Cre exprs pour : on l'efface donc... */
	MOL_effacer( infos->molecule, VRAI, VRAI );
	infos->molecule = NULL;
      }
      return FAUX;
    }
  }

  /* 3. A-t-on des atomes de Z encore inconnu ? */
  for ( atome = infos->molecule->premier ; atome != NULL ; atome = atome->suivant ) {
    int Z;

    Z = atome->num_atomique;
    if ( Z < HYDROGENE ) { /* Z inconnu : dfini avant les potentiels ? */
      if ( Z > -9 ) { /* Le potentiel a-t-il t dfini depuis ? */
	Z = infos->modele->potentiels[ -Z ];
      }
    }

    if ( Z < HYDROGENE ) {
      (void) BI_ERR_alerte(NULL, &INF_AtomeNonDefini, VRAI, atome->position.x, atome->position.y, atome->position.z, Z);
    } else {
      atome->num_atomique = Z;
      atome->rayon = ATM_Z_vers_rayon(Z, ATM_RAYON_COVALENT);
    }
  }

  /* Contrle termin... */
  return VRAI;
}
