/* LASE : Logiciel d'Analyse des Spectres Exprimentaux
   (c) Emmanuel CURIS 1998-2002

   Interface FEFF
   Modles d'ajustement par chemins, prvus pour FEFF

   Construction et gestion des modles d'ajustement
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "atomes.h"
#include "feff.h"
#include "modeles.h"
#include "absorption.h" /* Pour SEUIL_K  */
#include "theorie.h"    /* Pour les modles de libre parcours moyen */

#include "feff_modele.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Absorbeur *FEFF_TrouveAbsorbeur(Molecule *molecule);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Modele_FEFF *FEFF_CreerModele(void)
{
  Modele_FEFF *modele, md_init = {
    SEUIL_K, NULL, {0, 0, 0, 0, 0, 0, 0, 0, 0},
    {
      0, NULL,                   /* Pas d'atomes dfinis au dbut - ATOMS   */
      { 0, 0, 0, 3, 0, 0 },      /* On lance tous les modules     - CONTROL */
                                 /*     en sortie minimale        - PRINT   */
				 /* FEFF 8.2 : mode 3 pour le module PATHS pour avoir les feffXXXX.dat */
      { 0.0, 0.0 },              /* Option absente...             - DEBYE   */
      0.0,                       /* Option absente...             - SIG2    */
      { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, /* Aucun titre - TITLE */
      { 0.0, 0.0 },              /* Option absente...             - CORRECTIONS */
      { 4.0, 2.5 },              /* Critres de slection         - CRITERIA    */
      { ABSENT, { 0.0, 0.0, 0.0 } }, /* Ellipticit : aucune     - ELLIPTICITY */
      49,                        /* Conditionne l'intervalle      - NEMAX */
      8,                         /* Nombre maximal de rflexions  - NLEG  */
      ABSENT,                    /* Option absente                - XANES     */
      { 0.0, 0.0 },              /* Option absente                - PCRITERIA */
      { ABSENT, { 0.0, 0.0, 0.0 } }, /* Polarisation : aucune    - POLARIZATION */
      0.0,                       /* Option absente                - RMAX */
      1.0,                       /* Option absente                - RMULTIPLIER */
      0, NULL,                   /* Aucune commande               - SS */
      {ABSENT, ABSENT, ABSENT, ABSENT, ABSENT, ABSENT, ABSENT, ABSENT, ABSENT}, /* Aucune commande - FOLP */
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}, /* Aucune commande  - ION  */
      {ABSENT, 0.0, 0.0},        /* Option absente                - EXCHANGE */
      { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, /* Aucune commande - OVERLAP */
      FAUX,                      /* On n'utilise pas les extensions Xanes */
      { 0, {0.0, 0.0, 0.0, 0.0, 0.0}, {0.0, 0.0, 0.0, 0.0, 0.0} }, /* EGRID et EMESH */
      0.0                        /* Aucun dcalage d'origine      - VINTFIX */
    },
    {1.0,  1.0, 1.0, 1e-5, VAR_CONSTANTE, {1.0}}, /* S0 */
    {0.0, -5.0, 5.0, 1e-5, VAR_LIBRE    , {0.0}}, /* DE0 */
    {0.0,  0.0, 0.0, 0.0 , VAR_CONSTANTE, {0.0}}, /* Gamma */
    {VRAI, VRAI, FAUX, FAUX, FAUX},
    0, 0, NULL, /* Couches*/
    2, MODELE_LPM_INCLUS, NULL, NULL, NULL, NULL, 0.0, /* Pondration par k^2, lpm inclus, ta = 0 */
    NULL /* phase de l'absorbeur incluse */
  };

  /* Rservation */
  modele = (Modele_FEFF *) malloc( sizeof(Modele_FEFF) );
  if (NULL == modele) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "FEFF/modele.c", "FEFF_CreerModele", "modele");
    return NULL;
  }

  /* Initialisation */
  *modele = md_init;

  return modele;
}

/* -------- Choisir le modle FEFF  diter -------------------\
 |   Renvoie NULL en cas d'erreur                              |
 |                                                             |
 | proposer : VRAI = on demande s'il faut en crer un nouveau  |
 \------------------------------------------------------------ */

Modele_FEFF *FEFF_ChoisirModele(boolean proposer)
{
  Absorbeur *absorbeur;
  struct modele_ajuste *modele;

  if ( NULL == mol_courante ) { /* Aucune molcule en mmoire */
    Atome *atome;
    Molecule *molecule;
    char *symbole;
    int Z;

    /* On demmande le Z de l'absorbeur */
    symbole = BI_Demande_Texte(NULL, &Q_PotentielAbsorbeur, "Zn");
    if (NULL == symbole) { /* On a annul */
      return NULL;
    }
    Z = ATM_num_atomique( symbole );
    if ( SYMBOLE_CHIMIQUE_INCONNU == Z ) {
      Z = atoi(symbole);
    }
    if ( ( Z < HYDROGENE ) || ( Z > PLUTONIUM ) ) { /* Z absurde */
      (void) fprintf(stderr, "%s,%s l.%u : numro atomique ou symbole inconnu [%s = %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, symbole, Z);
      free( symbole ); symbole = NULL;
      return NULL;
    }
    free( symbole ); symbole = NULL;

    /* On cre une molcule avec juste cet atome,  l'origine */
    molecule = MOL_creer( "Mod"e_GRAVE"le FEFF" );
    if ( NULL == molecule ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la molcule, modle FEFF !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
    atome = ATM_creer( molecule, NULL, Z, NULL, 0.0, -1.0 );
    if ( NULL == atome ) {
      MOL_effacer( molecule, VRAI, VRAI );
      return NULL;
    }

    /* Cet atome est l'absorbeur de la molcule */
    absorbeur = creer_absorbeur( atome, "Absorbeur FEFF", 100.0 );
    if ( NULL == absorbeur ) {
      MOL_effacer( molecule, VRAI, VRAI );
      return NULL;
    }

    /* On cre le modle d'ajustement, type FEFF */
    modele = ABS_CreerModele( absorbeur, MODELE_EXAFS_CHEMINS, NULL );
    if ( NULL == modele ) {
      MOL_effacer( molecule, VRAI, VRAI );
      return NULL;
    }

    /* La molcule est la premire, on y va ! */
    MOL_aller( molecule );

    /* C'est le modle que l'on tudiera... */
    return modele->infos_modele.feff;
  }

  /* L, il y a une molcule */
  absorbeur = FEFF_TrouveAbsorbeur( mol_courante );
  if ( NULL == absorbeur ) {
    AVERTISSEMENT( "Impossible de trouver un absorbeur dans la molcule courante [%s]", mol_courante->nom );
    (void) BI_ERR_alerte( NULL, &ERR_AucunAbsorbeur, VRAI );
    return NULL;
  }

  if ( NULL == absorbeur->modeles ) { /* Aucun modle dans l'absorbeur : on le cre, ce sera le modle tudi */
    modele = ABS_CreerModele(absorbeur, MODELE_EXAFS_CHEMINS, NULL);
    if ( NULL == modele ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer un modle pour l'absorbeur actif\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }

    return modele->infos_modele.feff;
  }

  /* Il y a dj des modles dans l'absorbeur */
  modele = absorbeur->modele_actif;
  if ( MODELE_EXAFS_CHEMINS == modele->type ) {
    if (VRAI == proposer) { /* Faut-il en faire un nouveau ? */
      int reponse;

      reponse = BI_ERR_alerte( NULL, &Q_NouveauModele, VRAI );
      if ( 3 == reponse ) /* On annule */
	return NULL;

      if ( 2 == reponse ) { /* On en veut un nouveau */
	modele = ABS_CreerModele( absorbeur, MODELE_EXAFS_CHEMINS, NULL );
	if (NULL == modele) {
	  (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer un nouveau modle pour l'absorbeur actif\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  modele = absorbeur->modele_actif;
	}
      }
    }

    /* On modifie le modle actuel */
    return modele->infos_modele.feff;
  }

  /* Le modle actuel n'est pas FEFF : en chercher d'autres, proposer */
  OPTION_A_FAIRE( "Modle actif autre que FEFF" );
  return NULL;
}

/* ---- Reprage des atomes absorbeurs ---- */

static Absorbeur *FEFF_TrouveAbsorbeur(Molecule *molecule)
{
  Absorbeur *absorbeur;

  if ( NULL == molecule ) return NULL;

  if (molecule->nbr_absorbeurs == 0) { /* Pas d'absorbeur dfini : on regarde les absorbeurs potentiels */
    Atome *absorbeur_feff;

    absorbeur_feff = ABS_ProposerAbsorbeur(mol_courante, FAUX);
    if (NULL == absorbeur_feff) return NULL;

    absorbeur = creer_absorbeur(absorbeur_feff, "Absorbeur FEFF", 100.0);
  } else {
    absorbeur = molecule->absorbeurs + molecule->absorbeur_courant;
  }

  return absorbeur;
}

/* -------- Effacer un modle FEFF (et les infos qu'il utilise) -------- */

int FEFF_EffacerModele(Modele_FEFF *modele)
{
  unsigned int i;

  if (NULL == modele) return 0;

  /* Titres */
  for (i = 0 ; i < 10 ; i++) {
    free( modele->parametres.titre[ i ] );
  }

  /* Couches de rflexion simple */
  while ( modele->parametres.prem_ss != NULL )
    FEFF_EffacerCouche(modele, modele->parametres.prem_ss, VRAI);

  /* Atomes */
  while ( modele->parametres.prem_atome != NULL )
    FEFF_EffacerAtome(modele, modele->parametres.prem_atome, FAUX);

  /* L'absorbeur */
  modele->absorbeur = NULL;

  /* Commandes OVERLAP */
  fprintf(stderr, "FEFF_EffacerModele : Il faudrait s'occuper des commandes OVERLAP\n");
  for (i = 0 ; i < 8 ; i++) {
  }

  /* Couches d'ajustement */
  fprintf(stderr, "FEFF_EffacerModele : Il faudrait s'occuper des couches d'ajustement\n");

  /* Le modle lui-meme ! */
  free( modele ); modele = NULL;

  /* Aucun problme  l'effacement */
  return 0;
}
