/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS, 1996-1999
   
  12, 16 janvier ; 16, 25 mars 1998
    * TFI : graphes en cours de calcul maintenant protgs contre l'effacement...

  21, 28 novembre ; 15 dcembre 1997
    * TF inverse : corrig le calcul des X -> rsultat plus correct...
                   adapt le nombre de points au rsultat intressant
                   amlior l'interface : corrig pb. de redessin du pointeur
                                          dessine graphe * fentre aussi si graphe rel
    * la TF rapide pour un nombre quelconque de points fonctionne
    * chang quelques affectations de noms en GRF_titre
    * corrig le test de nbr_points = 2^k dans la TF inverse (appel de l'arbre)

  Filtrage des spectres exprimentaux
  Calcul des fentres pour la TF directe et inverse
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "systeme.h"

#include "exafs.h"
#include "filtrage.h"
#include "science.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Calcul de la fentre --------------------------------------------------------------------- */

/* -------- Calcul de la fentre dans un graphe dj dfini ---------------\
 |    Ne renvoie rien                                                      |
 |                                                                         |
 | fenetre : le graphe contenant la fentre (ses X servent pour le calcul) |
 | infos   : comment calculer la fentre                                   |
 \------------------------------------------------------------------------ */

void TF_fenetre(Graphe *fenetre, Infos_FonctionApodisation *infos)
{
  double moyen, k_2;
  double inter, coefficient, commun;
  register double *x, *zone_y;
  unsigned long i;
  unsigned short int inc = 1;

  if (fenetre == NULL) return; /* Pas de graphe fentre   */
  if ( GRF_FCT_COMPLEXE == fenetre->genre_donnees ) {
    (void) fprintf(stderr, "%s,%s l.%u : On ne devrait plus avoir ce cas\n",
		   __FILE__, __FUNCTION__, __LINE__);
/*     FIN_ANTICIPEE( FIN_ERREUR_MINEURE ); */

    inc = 2;
    for (i = 0 ; i < fenetre->nombre_point ; i++) {
      (( (Complexe *) (fenetre->y) )[i]).im = 0.0; /* Partie imaginaire nulle */
    }
  }

  /* Toutes les fentres sont comprises entre 0 et 1 */
  /* pour le dessin : on triche en disant qu'elles sont comme la courbe */

  fenetre->x_min = *(fenetre->x); /* On met les bornes */
  fenetre->x_max = *(fenetre->x + fenetre->nombre_point - 1);
  fenetre->y_min = 0.0;
  fenetre->y_max = 1.0;

  x = fenetre->x;
  zone_y = fenetre->y;

  moyen = (infos->x_min + infos->x_max) / 2.0; /* Valeur moyenne, sur laquelle est centre la fentre */
  
  switch ( infos->type )
  {
  case PAS_DE_FENETRE: /* Aucun fentre utilise */
    for (i = 0 ; i < fenetre->nombre_point ; i++, x++) {
      *(zone_y) = 1.0;      
      zone_y += inc;
    }
    break;

  case FEN_RECTANGLE: /* Fentre rectangulaire */
    for (i = 0 ; i < fenetre->nombre_point ; i++, x++) {
      if ( (*x >= infos->x_min) && (*x <= infos->x_max) )
	*(zone_y) = 1.0;
      else
	*(zone_y) = 0.0;
      
      zone_y += inc;
    }
    break;
	
  case FEN_BESSEL: /* Kaiser-Bessel */
    {
      double v_bessel;

      inter = M_PI * infos->parametre;
      v_bessel = bessel2( inter );
      if ( 0.0 == v_bessel ) {
	(void) fprintf(stderr, "%s, %s l.%u : Division par zro [p=%g, inter=%g]\n",
		       __FILE__, __FUNCTION__, __LINE__, infos->parametre, inter);
	return;
      }
      /* Le paramtre est trop grand -> donne pratiquement un pic de Dirac */
      if ( isinf( v_bessel ) ) {
	for ( i = 0 ; i < fenetre->nombre_point ; i++, zone_y += inc ) {
	  *zone_y = 0.0;
	}
	return;
      }
      coefficient = 1.0 / v_bessel;

      for (i = 0 ; i < fenetre->nombre_point ; i++, x++) {
	if ( ( *x < 0.0 ) || ( *x > ( infos->x_min + infos->x_max) ) )
	  *(zone_y) = coefficient; /* Bessel(0.0) = 1.0 */
	else {
	  commun = 1.0 - *x / moyen;
	  v_bessel = bessel2( inter * sqrt(1 - commun * commun) );
	  if ( isinf( v_bessel ) ) {
	    *zone_y = 1.0; /* On a une valeur infinie si le paramtres est trop grand */
	  } else {
	    *(zone_y) = coefficient * v_bessel ;
#ifdef _DEVELOPPE_
	    if ( isnan( *zone_y ) ) {
	      (void) fprintf(stderr, "%s, %s l.%u : Nan ! [X=%g -> commun = %g, inter=%g, p=%g, B(%g)=%g]\n",
			     __FILE__, __FUNCTION__, __LINE__, *x, commun, inter, infos->parametre,
			     inter * sqrt( 1.0 - commun * commun ), bessel2( inter * sqrt( 1.0 - commun * commun ) ) );
	      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
	    }
#endif
	  }
	}
			
	zone_y += inc;
      }
    }
    break;
	
  case FEN_GAUSSIENNE:  /* Gaussienne */
    coefficient = - M_PI_2 * infos->parametre;
    for (i = 0 ; i < fenetre->nombre_point ; i++, x++) {
      inter = 1 - *x / moyen;
      *(zone_y) = exp( coefficient * inter * inter );
      
      zone_y += inc;
    }
    break;
	
  case FEN_HAMMING: /* Hamming */
    if (infos->symetrique)
      k_2 = infos->x_max - (infos->x_1 - infos->x_min);
    else
      k_2 = infos->x_2;

    TFEN_hamming(fenetre, infos->x_min, infos->x_1, k_2, infos->x_max, infos->parametre);
    break;
    
  case FEN_HANNING: /* Normalement : hanning = hamming (0,5) */
    if (infos->symetrique)
      k_2 = infos->x_max - (infos->x_1 - infos->x_min);
    else
      k_2 = infos->x_2;

    TFEN_hanning(fenetre, infos->x_min, infos->x_1, k_2, infos->x_max);
    break;
	
  case FEN_NORTON_F3:   /* Norton-Beer F3 */
    for (i = 0 ; i < fenetre->nombre_point ; i++, x++) {
      if ( (*x < 0.0) || (*x > infos->x_min + infos->x_max) )
	inter = 0.0;
      else {
	inter  = 1.0 - *(x) / moyen;
	inter *= inter;
	inter  = 1.0 - inter;
	inter *= inter;    /* Donc inter = [1-(1-k/km)^2]^2 */
      }

      *(zone_y) = 0.045335 + 0.554883 * inter + 0.399782 * inter * inter;
      
      zone_y += inc;
    }
    break;
	
  case FEN_BARTLETT:
    if (infos->symetrique == VRAI) /* Fentre symtrique */
      k_2 = infos->x_max - (infos->x_1 - infos->x_min);
    else
      k_2 = infos->x_2;

    TFEN_bartlett(fenetre, infos->x_min, infos->x_1, k_2, infos->x_max);
    break;
    
  case FEN_BLACKMAN: /* Fentre de Blackman : pas encore faite */
    break;
  }
	
  return;
}

/* -------- Sous fonctions du calcul des fentre en une fois ---------------------- */

int TFEN_bartlett(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4)
{
  unsigned long i;
  double pente_1, pente_2;
  register double *x;

  pente_1 = (k_2 - k_1);
  pente_2 = (k_3 - k_4);
  if ( (pente_1 == 0) || (pente_2 == 0) ) {
    EXF_ERREUR(DIV_ZERO, "Pente nulle", txt_err_calc_annule);
    return -DIV_ZERO;
  }

  if ( GRF_EstReel( fenetre ) == VRAI ) {
    register double *y = fenetre->y;

    for (i = 0, x = fenetre->x ; i < fenetre->nombre_point ; i++, x++, y++) {
      if ( (*x <= k_1) || (*x >= k_4) )
	*y = 0;
      else if ( (*x > k_1) && (*x < k_2) )
	*y = (*x - k_1) / pente_1;
      else if ( (*x >= k_2) && (*x <= k_3) )
	*y = 1;
      else
	*y = (*x - k_4) / pente_2;
    }
  } else if ( GRF_EstComplexe( fenetre ) == VRAI ) {
    register Complexe *y = (Complexe *) fenetre->y;

    for (i = 0, x = fenetre->x ; i < fenetre->nombre_point ; i++, x++, y++) {
      y->im = 0; /* Toujours */
      if ( (*x <= k_1) || (*x >= k_4) )
	y->re = 0;
      else if ( (*x > k_1) && (*x < k_2) )
	y->re = (*x - k_1) / pente_1;
      else if ( (*x >= k_2) && (*x <= k_3) )
	y->re = 1;
      else
	y->re = (*x - k_4) / pente_2;
    }
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe non-support\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  return 0; /* Pas de problme */
}

int TFEN_hamming(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4, double parametre)
{
  double coef_1, coef_2;
  unsigned long i;
  register double *x = fenetre->x;

  if ( (k_1 == k_2) || (k_3 == k_4) ) {
    EXF_ERREUR(DIV_ZERO, "Zone de transition nulle", txt_err_calc_annule);
    return -DIV_ZERO;
  }
  if ( ( parametre > 1.0 ) || ( parametre < 0.5 ) ) {
    (void) fprintf(stderr, "TFEN_hamming : valeur du paramtre absurde, rsultats non garantis [%g]\n", parametre);
  }

  coef_1 = M_PI / (k_2 - k_1);
  coef_2 = M_PI / (k_3 - k_4);

  if ( GRF_EstReel( fenetre ) == VRAI ) {
    register double *y = fenetre->y;

    for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++) {
      if ( (*x <= k_1) || (*x >= k_4) )
	*y = 2 * parametre - 1;
      else if ( (*x > k_1) && (*x < k_2) ) {
	*y = parametre + (parametre - 1) * cos( (*x - k_1) * coef_1 );
      } else if ( (*x >= k_2) && (*x <= k_3) )
	*y = 1.0;
      else {
	*y = parametre + (parametre - 1) * cos( (*x - k_4) * coef_2 );
      }
    }
  } else if ( GRF_EstComplexe( fenetre ) == VRAI ) {
    register Complexe *y = (Complexe *) fenetre->y;

    for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++) {
      y->im = 0.0;
      if ( (*x <= k_1) || (*x >= k_4) )
	y->re = 2 * parametre - 1;
      else if ( (*x > k_1) && (*x < k_2) ) {
	y->re =  parametre + (parametre - 1) * cos( (*x - k_1) * coef_1 );
      } else if ( (*x >= k_2) && (*x <= k_3) ) {
	y->re = 1.0;
      } else {
	y->re =  parametre + (parametre - 1) * cos( (*x - k_4) * coef_2 );
      }
    }
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe non-support\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  return 0; /* Pas de problme */
}

int TFEN_hanning(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4)
{
  double coef_1, coef_2;
  unsigned long i;
  register double *x = fenetre->x;

  if ( (k_1 == k_2) || (k_3 == k_4) ) {
    EXF_ERREUR(DIV_ZERO, "Zone de transition nulle", txt_err_calc_annule);
    return -DIV_ZERO;
  }

  coef_1 = M_PI_2 / (k_2 - k_1);
  coef_2 = M_PI_2 / (k_3 - k_4);

  if ( GRF_EstReel( fenetre ) == VRAI ) {
    register double *y = fenetre->y;

    for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++) {
      double inter;

      if ( (*x <= k_1) || (*x >= k_4) )
	*y = 0.0;
      else if ( (*x > k_1) && (*x < k_2) ) {
	inter = sin( (*x - k_2) * coef_1 );
	*y = 1.0 - inter * inter;
      } else if ( (*x >= k_2) && (*x <= k_3) )
	*y = 1.0;
      else {
	inter = sin( (*x - k_3) * coef_2 );
	*y = 1.0 - inter * inter;
      }
    }
  } else if ( GRF_EstComplexe( fenetre ) == VRAI ) {
    register Complexe *y = (Complexe *) fenetre->y;

    for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++) {
      double inter;
			
      y->im = 0.0;
      if ( (*x <= k_1) || (*x >= k_4) )
	y->re = 0.0;
      else if ( (*x > k_1) && (*x < k_2) ) {
	inter = sin( (*x - k_2) * coef_1 );
	y->re = 1.0 - inter * inter;
      } else if ( (*x >= k_2) && (*x <= k_3) )
	y->re = 1.0;
      else {
	inter = sin( (*x - k_3) * coef_2 );
	y->re = 1.0 - inter * inter;
      }
    }
  } else {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe non-support\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  return 0; /* Pas de problme */
}

/* -------- Calcul de la fentre en un point prcis -----------\
 |   Renvoie la valeur de la fentre en ce point               |
 |                                                             |
 | x     : point o calculer la fentre                        |
 | infos : type de la fentre  utiliser                       |
 \------------------------------------------------------------ */

double TF_calcfen(double x, Infos_FonctionApodisation *infos)
{
  double moyen, inter;
  double pente_1, pente_2;
  double parametre;
  double k_2;

  if (NULL == infos) return 1.0; /* Pas de fentre ! */

  parametre = infos->parametre; /* Peut tre modifi pour le calcul... */
  if (infos->symetrique)
    k_2 = infos->x_max - (infos->x_1 - infos->x_min); /* On symtrise tout a... */
  else
    k_2 = infos->x_2;

  moyen = (infos->x_min + infos->x_max) / 2.0; /* K_moyen, revient souvent... */

  switch (infos->type)
  {
  case FEN_RECTANGLE:         /* Rectangulaire : la plus simple... */
    if (x < infos->x_min) return 0.0;
    if (x > infos->x_max) return 0.;
    return 1.0;

  case FEN_GAUSSIENNE:        /* Gaussienne    */
    parametre *= - M_PI_2;
    x = 1.0 - 2.0 * x / (infos->x_min + infos->x_max);
    return exp( parametre * x * x );

  case FEN_BESSEL:            /* Kaiser-Bessel */
    parametre *= M_PI;

    if (x < 0.0) return bessel2(0.0) / bessel2( parametre );
    if (x > infos->x_min + infos->x_max) return bessel2(0.0) / bessel2( parametre );
    
    x = 1.0 - x / moyen;
    return bessel2( parametre * sqrt(1.0 - x * x) ) / bessel2( parametre );

  case FEN_HANNING: /* Hanning = Hamming (0,5) en ralit... */
    parametre = 0.5;
  case FEN_HAMMING: /* Hamming */

    if ( (infos->x_min == infos->x_1) || (infos->x_max == k_2) ) {
      EXF_ERREUR(DIV_ZERO, "Zone de transition nulle", txt_err_calc_annule);
      return 0.0;
    }

    pente_1 = M_PI / (infos->x_1 - infos->x_min);
    pente_2 = M_PI / (k_2 - infos->x_max);

    if ( (x <= infos->x_min) || (x >= infos->x_max) )
      return	2.0 * parametre - 1.0;
    if ( (x > infos->x_min) && (x < infos->x_1) )
      return parametre + (parametre - 1.0) * cos( (x - infos->x_min) * pente_1 );
    if ( (x >= infos->x_1) && (x <= k_2) )
      return 1;
    return parametre + (parametre - 1.0) * cos( (x - infos->x_max) * pente_2 );

  case FEN_NORTON_F3:   /* Norton-Beer F3 */
    if ( x < 0.0 ) return 0.0;
    if ( x > infos->x_min + infos->x_max ) return 0.0;

    inter  = 1.0 - x / moyen;
    inter *= inter;
    inter  = 1.0 - inter;
    inter *= inter;    /* Donc inter = [1-(1-k/km)^2]^2 */
    
    return 0.045335 + 0.554883 * inter + 0.399782 * inter * inter;

  case FEN_BARTLETT:
    pente_1 = (infos->x_1 - infos->x_min);
    pente_2 = (k_2 - infos->x_max);

    if ( (pente_1 == 0.0) || (pente_2 == 0.0) ) {
      EXF_ERREUR(DIV_ZERO, "Pente nulle", txt_err_val_nulle);
      return 0.0;
    }

    if ( (x <= infos->x_min) || (x >= infos->x_max) )
      return 0;
    if ( (x > infos->x_min) && (x < infos->x_1) )
      return (x - infos->x_min) / pente_1;
    if ( (x >= infos->x_1) && (x <= k_2) )
      return 1.0;
    return (x - infos->x_max) / pente_2;

  case FEN_BLACKMAN: /* Fentre de Blackman : pas encore faite */
    break;
  }

  return 1.0; /* On est incapable de calculer la fentre */
}
