/* LASE : logiciel d'analyse des spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : 29 septembre 1998

   Module de  Calcul formel 

   Tout ce qui tourne autour d'une variable
*/
#define VARIABLES_CALCUL_FORMEL /* Pour ne pas avoir de doublon en dclarant les variables */

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "c_formel.h"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* Visibles de l'extrieur : facilite l'analyse des formules */
Variable Var_X, Var_Y, Var_Z, Var_T, Var_I, Var_R;
Variable Var_sX, Var_sY;

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique du module ~~~~~~~ */

/* -------- Initialiser une variable en fonction de son type -------- */

int VAR_Initialiser(Variable *variable, unsigned int type, double valeur, double min, double max, double pas)
{
  if ( NULL == variable ) return 0; /* Rien  faire : variable inexistante... */

  /* On commence par mettre le type de la variable */
  variable->type = type;

  /* On initialise la partie commune avec les valeurs indiques */
  variable->valeur_initiale = valeur;
  variable->min = min;
  variable->max = max;
  variable->delta = pas;

  /* On se sert du type pour le reste */
  switch( type ) {
  case VAR_CONSTANTE:
    variable->contenu.valeur = valeur;
    break;

  case VAR_LIBRE:
    variable->contenu.valeur = valeur;
    break;

  case VAR_COPIE:
    variable->contenu.original = NULL;
    break;

  case VAR_EXPRESSION:
  case VAR_RELATION:
    variable->contenu.expression.type = EXP_CTE_CTE;
    variable->contenu.expression.code = EXP_NOMBRE_ISOLE;
    variable->contenu.expression.arg_1.valeur = valeur;
    variable->contenu.expression.arg_2.valeur = 0.0;
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : type de variable inconnu [%d ; = %g = %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, variable->type, variable->valeur_initiale, variable->contenu.valeur);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }

  return PAS_DE_PROBLEME;
}


/* -------- Rcupration de la valeur d'une variable --------
 *  Renvoie 0.0 en cas de problme
 *          la valeur demande si tout va bien
 *
 * variable      = la variable dont on veut lire une valeur
 * classe_valeur = quelle valeur veut-on lire ? [VARIABLE_LIRE_VALEUR_xxxx, cf c_formel.h]
 */

double VAR_LireValeur(Variable *variable, unsigned int classe_valeur)
{
  if ( NULL == variable ) return 0.0;

  switch ( variable->type )
  {
  case VAR_CONSTANTE:
    /* Peu importe le type : seule la valeur initiale est utile, puisqu'invariable ! */
    return variable->valeur_initiale;

  case VAR_LIBRE:
    switch ( classe_valeur )
    {
    case VARIABLE_LIRE_VALEUR_INITIALE:
      return variable->valeur_initiale;

    case VARIABLE_LIRE_VALEUR_MINIMALE:
      return variable->min;

    case VARIABLE_LIRE_VALEUR_MAXIMALE:
      return variable->max;

    case VARIABLE_LIRE_VALEUR_AJUSTEE:
      return variable->contenu.valeur;

    default:
      AVERTISSEMENT( "Classe de valeur de variable inconnu [%u]", classe_valeur );
    }
    return 0.0;

  case VAR_COPIE:
    return VAR_LireValeur( variable->contenu.original, classe_valeur );

  case VAR_EXPRESSION:
  case VAR_RELATION:
    OPTION_A_FAIRE( "Variable de type expression ou relation" );
    /* utiliser l'expression pour calculer la valeur de la variable */
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : type de variable inconnu [%d ; = %g = %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, variable->type, variable->valeur_initiale, variable->contenu.valeur);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }

  return 0.0; /* Serait franchement trange */
}

/* -------- Fixer la valeur d'une variable [mmes codes que ci-dessus] -------- */

int VAR_FixerValeur(Variable *variable, unsigned int classe_valeur, double nouvelle_valeur)
{
  if ( NULL == variable ) return -1;

  switch ( variable->type )
  {
  case VAR_CONSTANTE:
    /* Peu importe le type : seule la valeur initiale est utile, puisqu'invariable ! */
    variable->valeur_initiale = nouvelle_valeur;

  case VAR_LIBRE:
    switch ( classe_valeur )
    {
    case VARIABLE_LIRE_VALEUR_INITIALE:
      variable->valeur_initiale = nouvelle_valeur;
      break;

    case VARIABLE_LIRE_VALEUR_MINIMALE:
      variable->min = nouvelle_valeur;
      break;

    case VARIABLE_LIRE_VALEUR_MAXIMALE:
      variable->max = nouvelle_valeur;
      break;

    case VARIABLE_LIRE_VALEUR_AJUSTEE:
      variable->contenu.valeur = nouvelle_valeur;
      break;

    default:
      AVERTISSEMENT( "Type de valeur de variable inconnu [%u]", classe_valeur );
    }
    break;

  case VAR_COPIE:
    return VAR_FixerValeur( variable->contenu.original, classe_valeur, nouvelle_valeur );

  case VAR_EXPRESSION:
  case VAR_RELATION:
    /* utiliser l'expression pour calculer la valeur de la variable
       ==> on ne peut pas en fixer la valeur ! */
    AVERTISSEMENT( "Impossible de fixer la valeur d'une variable dfinie  partir d'une autre ! Valeur %g ignore",
		   nouvelle_valeur );
    return +1;			/* Erreur mineure... */

  default:
    (void) fprintf(stderr, "%s,%s l.%u : type de variable inconnu [%d ; = %g = %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, variable->type, variable->valeur_initiale, variable->contenu.valeur);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }

  return 0; /* Pas de problme rencontr */
}

/* -------- Contrle que les bornes ne sont pas atteintes, lors d'un ajustement -------- */

int VAR_ControleBornes(Variable *variable)
{
  double valeur;
  double tolerance;

  if ( NULL == variable ) return 0;

  if ( VAR_CONSTANTE == variable->type ) return 0; /* Le problme ne se pose pas pour une variable fixe ! */

  valeur = VAR_LireValeur( variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
  tolerance = variable->max - variable->min;
  tolerance /= 100.0; /* Si l'on est  moins de 1/100 de l'intervalle d'une borne, c'est que l'on a atteint la borne */

  if ( valeur <= variable->min + tolerance )
    return -1; /* Borne infrieure atteinte */
  if ( valeur >= variable->max - tolerance )
    return +1; /* Borne suprieure atteinte */

  /* On est bien dans l'intervalle autoris */
  return 0;
}

/* -------- Sauvegarde du contenu d'une variable dans l'historique -------- */

void HIST_VAR_ValeurVariable(Variable *variable, const char *texte)
{
  double valeur;

  if ( NULL == variable ) return;

  valeur = VAR_LireValeur( variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
  if ( variable->type == VAR_CONSTANTE ) {  /* Variables non ajustes : seulement si historique exhaustif */
    HIST_Texte( 5, 0, "%s : %g [constante]\n", texte, valeur );
    return;
  } else if ( VAR_EXPRESSION == variable->type ) {
    HIST_Texte( 5, 0, "%s : %g [calcul"e_AIGU"e apr"e_GRAVE"s ajustement]\n", texte, valeur );
    return;
  }

  HIST_Texte( 3, 0, "%s : %g", texte, valeur );
  if ( variable->type == VAR_LIBRE ) {
    HIST_Texte( 5, 0, " [min : %g, max : %g, d"e_AIGU"part : %g]", variable->min, variable->max, variable->valeur_initiale );
  } else if ( VAR_COPIE == variable->type ) {
    HIST_Texte( 5, 0, " [copie d'une autre variable]" );
  } else if ( VAR_RELATION == variable->type ) {
    HIST_Texte( 5, 0, " [fonction d'autres variables]" );
  }
  HIST_Texte( 3, 0, "\n" );
}
