/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Slection d'une image dans la liste
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "images.h"
#include "interface.h"
#include "tracer.h" /* Pour dessin de l'objet utilisateur */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct infos_ChoixImage {
  Image **table_images;
  unsigned long nbr_images;
  unsigned long img_choisie;
  unsigned long prem_affichee;
  boolean strict;
  int type;
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static const char *OBJ_NVersTexte(unsigned int num_arbre, unsigned long num_objet);

static int CLV_ImgChoix(unsigned int touche, int etat, int *renvoi);
static int GER_ImgChoix(unsigned int num_objet, int x, int y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Image *I_IMG_ChoisirUnique(Image *modele, unsigned int masque __attribute__((__unused__)), boolean strict, boolean multi_serie, Serie_Images *serie_ou_choisir)
{
  struct infos_ChoixImage infos;
  Image *image_choisie = NULL;
  int retour;

  if ( NULL == IMG_serie_courante ) return NULL; /* Aucune image en mmoire */
  if ( NULL == serie_ou_choisir ) {
    serie_ou_choisir = IMG_serie_courante;
  }
  if ( NULL == modele ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de modle indiqu !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  OPTION_A_FAIRE( "Adapter pour avoir des filtres qui tiennent compte du masque et du modle !" );

  /* On filtre la liste des images pour avoir la table dans laquelle slectionner */
  infos.table_images = IMG_FiltrerType( serie_ou_choisir->premiere_image, modele->type, strict, &(infos.nbr_images) );
  if ( NULL == infos.table_images ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la table des images ou aucune image\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* et on termine de prparer les infos d'affichage */
  infos.img_choisie = 0;
  infos.prem_affichee = 0;
  infos.type = modele->type;
  infos.strict = strict;

  /* Prparation de l'arbre */
  OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, USR_image, (long) ( infos.table_images[ infos.img_choisie ] ), METTRE);
  ARB_PrepareSelecteurObjets( FAUX, GER_ImgChoix, CLV_ImgChoix );
  OBJ_active( CHX_GRAPHE, CHXG_SERIE, multi_serie );
  OBJ_active(CHX_GRAPHE, CHXG_OK, FORCE_OUI);

  ARB_PreparerDefileur_O( CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR, CHXG_AVANT, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES,
			  infos.nbr_images, NULL, infos.img_choisie, infos.prem_affichee, OBJ_NVersTexte );

  /* Gestion */
  ARB_MettreComplement( CHX_GRAPHE, &infos );
  retour = ARB_Bloquant( CHX_GRAPHE, "Choisir une image", principale, 0 );
  ARB_MettreComplement( CHX_GRAPHE, NULL );

  if ( CHXG_ANNULE == retour ) { /* On annule */
    free( infos.table_images ); infos.table_images = NULL;
    return NULL;
  }

  image_choisie = infos.table_images[ infos.img_choisie ];

  /* Plus besoin de la table d'images */
  free( infos.table_images ); infos.table_images = NULL;

  return image_choisie;
}

Image **I_IMG_ChoisirMultiples(int type __attribute__((__unused__)), boolean strict __attribute__((__unused__)), boolean multi_serie __attribute__((__unused__)), unsigned long *nombre_images_choisies)
{
  if ( NULL == nombre_images_choisies ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de mmoriser le nombre de choix\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  OPTION_A_FAIRE( "Faire un slecteur multiple d'images !" );
  return NULL;
}

/* -------- Gestion du clavier -------- */

static int CLV_ImgChoix(unsigned int touche, int etat __attribute__((__unused__)), int *renvoi __attribute__((__unused__)) )
{
  (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Clavier (touche %c)\n",
		 __FILE__, __FUNCTION__, __LINE__, touche);

  return FAUX;
}

/* -------- Gestion du dfileur ------- */

static int GER_ImgChoix(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  struct infos_ChoixImage *infos;

  /* On rcupre les infos de slection */
  infos = ( struct infos_ChoixImage * ) ARB_LireComplement( CHX_GRAPHE );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos associes  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_TRAITE;
  }

  /* 1. Traitement du changement de srie (car on peut avoir des sries vides) */
  if ( CHXG_SERIE == objet ) {
    OPTION_A_FAIRE( "Changement de srie" );
    return BI_EVENEMENT_TRAITE;
  }

  /* Si aucune image, alors on s'arrte */
  if ( ( NULL == infos->table_images ) || ( 0 == infos->nbr_images ) ) return VRAI;

  /* Pour le dcalage des noms : ascenseur horizontal */
  if ( ( objet == CG_TXT_AVANT ) || ( objet == CG_FND_ASC_TXT ) ||
       ( objet == CG_ASC_TXT   ) || ( objet == CG_TXT_APRES ) ) {
    OPTION_A_FAIRE( "Dfileur des noms" );
    return BI_EVENEMENT_TRAITE;
  }

  /* Pour le dcalage de la liste : ascenseur vertical */
  if ( ( objet == CHXG_AVANT ) || ( objet == CHXG_FOND_ASC ) ||
       ( objet == CHXG_APRES ) || ( objet == CHXG_POUSSOIR ) ) {
    (void) ARB_GererDefileur_O( CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
				CHXG_AVANT, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES,
				infos->nbr_images, NULL, &(infos->img_choisie), &(infos->prem_affichee),
				OBJ_NVersTexte, objet, y );
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) ( infos->table_images[ infos->img_choisie ]), METTRE);
    return BI_EVENEMENT_TRAITE;
  }

  /* Clic dans la liste : slection */
  if ( ( objet >= CHXG_PREMIER_NOM ) && ( objet <= CHXG_DERNIER_NOM ) ) {
    (void) ARB_GererDefileur_O( CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
				CHXG_AVANT, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES,
				infos->nbr_images, NULL, &(infos->img_choisie), &(infos->prem_affichee),
				OBJ_NVersTexte, objet, y );
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) ( infos->table_images[ infos->img_choisie ]), METTRE);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Quel texte afficher ? -------- */

static const char *OBJ_NVersTexte(unsigned int num_arbre, unsigned long num_objet)
{
  struct infos_ChoixImage *infos;

  infos = ( struct infos_ChoixImage * ) ARB_LireComplement( num_arbre );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos associes  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  if ( num_objet >= infos->nbr_images ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec un index d'objet trop grand [%lu, max %lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_objet, infos->nbr_images - 1);
#endif
    return " ";
  }

  /* Nom de l'image */
  return infos->table_images[ num_objet ]->nom;
}

/* -------- Redessin d'un objet utilisateur contenant une image -------- */

int cdecl USR_image(USR_FONCTION *parametres)
{
  Image *image;

  /* Prparation du dessin */
  TRC_InitDessins( NULL, parametres->pb_xc, parametres->pb_yc, parametres->pb_wc, parametres->pb_hc );

  /* Trac de l'image : optimal dans ce cadre */
  image = (Image *) (parametres->pb_parm);
  TRC_ImageDansRectangle( NULL, image, NOIR, parametres->pb_x, parametres->pb_y, parametres->pb_w, parametres->pb_h );

  /* On a fini */
  return parametres->pb_currstate;
}
