/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS
   Cration : 13 juin 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Produit d'une image par un nombre rel
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "bin_utils.h"

#include "graphes.h" /* Pour GRF_FacteurMultiplicatif() */
#include "images.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int IMG_FacteurMultiplicatif_Z(Image *image, double facteur);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int IMG_FacteurMultiplicatif(Image *image, double facteur, unsigned int axe)
{
  if ( NULL == image ) return -1;
  if ( 1.0 == facteur ) return 0; /* Produit par 1 : rien ne change */

  /* Cas spciaux : axe X & axe Y, ce sont la grille et la valeur du coin BG qui changent */
  if ( axe & AXE_X ) {
    if ( facteur >= 0.0 ) {
      /* Transformation croissante : le min reste le min */
      image->x_min *= facteur;
    } else {
      /* Transformation dcroissante : le max devient le min */
      image->x_min = facteur * ( image->x_min + image->larg * image->l_reel );
    }

    /* Le pas de la grille est forcment positif */
    image->l_reel *= fabs( facteur );
  }

  if ( axe & AXE_Y ) {
    if ( facteur >= 0.0 ) {
      /* Transformation croissante : le min reste le min */
      image->y_min *= facteur;
    } else {
      /* Transformation dcroissante : le max devient le min */
      image->y_min = facteur * ( image->y_min + image->haut * image->h_reel );
    }

    /* Le pas de la grille est forcment positif */
    image->h_reel *= fabs( facteur );
  }

  /* Axe Z : on touche aux valeurs, cette fois */
  if ( axe & AXE_Z ) {
    int retour;

    retour = IMG_FacteurMultiplicatif_Z( image, facteur );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme dans le produit sur l'axe Z [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return retour;
    }
  }

  /* Pas de problme */
  return 0;
}

static int IMG_FacteurMultiplicatif_Z(Image *image, double facteur)
{
  unsigned long i, nbr_cases;

  /* Combien de valeurs ? */
  nbr_cases = image->larg * image->haut;

  /* On fait le produit pour les valeurs */
  switch ( image->genre )
  {
  case IMG_GENRE_REELLE:
    for ( i = 0 ; i < nbr_cases ; i++ ) {
      image->valeurs.R[ i ] *= facteur;
    }
    break;

  case IMG_GENRE_COMPLEXE:
    for ( i = 0 ; i < nbr_cases ; i++ ) {
      image->valeurs.C[ i ].re *= facteur;
      image->valeurs.C[ i ].im *= facteur;
    }
    break;

  case IMG_GENRE_GRAPHE:
    for ( i = 0 ; i < nbr_cases ; i++ ) {
      GRF_FacteurMultiplicatif( image->valeurs.G[ i ], facteur, AXE_Y, NULL );
    }
    break;

  case IMG_GENRE_DIAGRAMME:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE / Diagramme\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -4;

  case IMG_GENRE_PHOTO:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE / photo\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -8;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Genre inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->genre);
    return -167;
  }

  /* On adapte les bornes */
  if ( facteur > 0.0 ) {
    image->val_min *= facteur;
    image->val_max *= facteur;
  } else {
    double tmp;

    tmp = image->val_min * facteur;
    image->val_min = image->val_max * facteur;
    image->val_max = tmp;
  }

  /* S'il y a des erreurs : elles sont multiplies par la valeur absolue */
  if ( image->erreurs ) {
    int retour;

    retour = IMG_FacteurMultiplicatif_Z( image->erreurs, fabs( facteur ) );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'adaptation des erreurs [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);

      return +1; /* Erreur mineure */
    }
  }

  /* Calcul russi */
  return 0;
}
