/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Imports/export, des fichiers exprimentaux entre autres
   Import d'un fichier au format PCA (v.1, v.2 ou v.3)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "fichiers.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define PCA3_BLOC_CANAUX 1014
#define PCA3_BLOC_ETALON 1015
#define PCA3_BLOC_DATE   1016
#define PCA3_BLOC_TITRE  1017

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Fichier au format PCA v 3 (adapt du code en BASIC venant de Wapi) --------
     Renvoie le spectre de fluo ainsi obtenu

   Apparemment, il s'agit d'un format  champs successifs, chacun commenant par un code
   Format binaire de codage de type Intel, a priori
*/

Graphe *Import_PCA3(const char *nom_fichier, Serie_Graphes *serie, unsigned int num_bloc)
{
  Graphe *graphe = NULL;
  FILE *fichier;
  char date[14], heure[14];
  char titre[25];
  unsigned int numero_bloc = 0;
  double A = 0.0, B = 0.0, C = 0.0; /* Coefficients de l'talonnage en nergie */

  if (NULL == nom_fichier) return NULL;
  if (NULL == serie) serie = serie_courante; /* On ajoute  la srie courante par dfaut */

  fichier = fopen(nom_fichier, "rb");
  if (NULL == fichier) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, NULL);
    (void) fprintf(stderr, "Import_PCA3 : impossible d'ouvrir le fichier %s\n", nom_fichier);
    return NULL;
  }

  /* On lit bloc par bloc */

  do {
    short int type_bloc;  /* Suppose sizeof(short int) = 2 */
    int lg_bloc;          /* Suppose sizeof(int) = 4 */
    size_t n_lu;

    n_lu = fread( &type_bloc, sizeof(short int), 1, fichier);
    if (n_lu != 1) {
      (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, fichier, NULL);
      (void) fprintf(stderr, "Import_PCA3 : impossible de lire le code de bloc [fichier : %s]\n", nom_fichier);
      return NULL;
    }
    n_lu = fread( &lg_bloc, sizeof(int), 1, fichier);
    if (n_lu != 1) {
      (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, fichier, NULL);
      (void) fprintf(stderr, "Import_PCA3 : impossible de lire la taille du bloc [fichier : %s, code : %d]\n", nom_fichier, type_bloc);
      return NULL;
    }

    switch ( type_bloc )
    {
    case PCA3_BLOC_CANAUX: /* Donnes */
      {
	int c1, c2;
	unsigned long nbr_canaux, i;
	char *titre_spectre;
	double *x, *y;

	fseek(fichier, +4, SEEK_CUR); /* On saute 4 caractres */
	c1 = fgetc(fichier);
	c2 = fgetc(fichier);
	nbr_canaux = (c2 << 8) + c1;
/* 	printf(" ==> Donnes pour %lu canaux\n", nbr_canaux); */
	lg_bloc -= 6;
	numero_bloc++; /* Un bloc de donnes en plus */

	if ( (0 == num_bloc) || (numero_bloc == num_bloc) ) { /* On garde ce spectre */
	  titre_spectre = TXT_genere_chaine("%F : %s, %s - %s", nom_fichier, date, heure, titre);
	  graphe = Serie_nv_graphe(serie, graphe, nbr_canaux, GR_FLUOX, GR_NORMAL, titre_spectre);
	  free(titre_spectre);
	  if (NULL == graphe) {
	    (void) fclose(fichier);
	    return NULL;
	  }

	  /* On lit canal par canal */
	  for (i = 0, x = graphe->x, y = graphe->y ; i  < nbr_canaux ; i++, x++, y++) {
	    unsigned int valeur; /* Suppose sizeof(int) = 4 */

	    fread(&valeur, sizeof(int), 1, fichier);
	    *x = (double) i;
	    *y = (double) valeur;
	  }
	  GRF_bornes(graphe, ORDONNE, DESORDONNE);
	  lg_bloc -= 4 * nbr_canaux;

	  if (num_bloc > 0) { /* On ne s'intresse qu' un bloc : on a donc fini ! */
	    (void) fclose(fichier);
	    return graphe;
	  }

	  /* Pour tre sr de tout avoir dans la mme srie ! */
	  if (NULL == serie) { /* Il n'y avait donc aucune srie en mmoire */
	    serie = serie_courante;
	  }

	  /* talonnage ventuel */
	  if ( (A != 0.0) || (B != 0.0) || (C != 0.0) ) {
	    for (i = 0, x = graphe->x ; i  < nbr_canaux ; i++, x++) {
	      *x = C + B * ((double) i) + A * ((double) i) * ((double) i);
	      *x *= 1000.0; /* l'talonnage parat tre en keV, LASE travaille en eV */
	    }
	    GRF_bornes(graphe, ORDONNE, NE_PAS_FAIRE);
	  }
	}
      }
      break;

    case PCA3_BLOC_DATE: /* Une date & une heure */
      fseek(fichier, +6, SEEK_CUR); /* On saute 6 caractres */
      fread(heure, sizeof(char), 13, fichier);
      heure[13] = 0;
      fread(date, sizeof(char), 13, fichier);
      date[13] = 0;
/*       printf(" ==> le %s  %s\n", date, heure); */
      lg_bloc -= (6 + 13 + 13);
      break;

    case PCA3_BLOC_ETALON: /* talonnage en nergie des canaux [parabolique : E = A * n + B * n + C] */
      fseek(fichier, +10, SEEK_CUR); /* On saute 10 caractres */
      /* Suppose une taille de double de 8, codage Intel ! */
      fread(&C, sizeof(double), 1, fichier); /* A0 */
      fread(&B, sizeof(double), 1, fichier); /* A1 */
      fread(&A, sizeof(double), 1, fichier); /* A2 */
      lg_bloc -= (34);
      break;

    case PCA3_BLOC_TITRE: /* En-tte du fichier ou intitul */
      fseek(fichier, +6, SEEK_CUR); /* On saute 6 caractres */
      fread(titre, sizeof(char), 24, fichier);
      titre[24] = 0;
/*       printf(" ==> titre : %s\n", titre); */
      lg_bloc -= (6 + 24);
      break;

    default:
#ifdef _DEVELOPPE_
      printf("Code bloc : %d - longueur : %d [hors en-tte de bloc]\n", type_bloc, lg_bloc);
      printf(" ==> INCONNU : on ignore\n");
#endif
      break;
    }

    /* On saute la fin du bloc */
    fseek(fichier, lg_bloc, SEEK_CUR);

  } while ( 1 );

  /* Chargement termin */
  (void) fclose(fichier);

  return graphe;
}
