/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 2003, Emmanuel CURIS
   Cration de ce fichier : 26 octobre 1998

   Tout ce qui concerne l'import de fichiers contenant des graphes  des formats autres

   Import gnrique de fichiers au format ASCII, organiss en colonnes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "systeme.h"

#include "fichiers.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* #include "imp_ascii.txt" */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

boolean import_asc_3d(const char *nom, int nbr_colonnes, int x_colonne,
		      int y_colonne, int z_colonne,
		      unsigned int en_tete, char commentaire);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Routine d'appel des imports ASCII, cas gnrique -------- */

int ASCII_Generique(const char *nom_fichier, int format, struct infos_import_grf *infos)
{
  FILE *fichier;
  boolean liberer_table_graphes = FAUX;
  Graphe **graphes = NULL, *graphe = NULL;
  int retour;
  unsigned long nombre_graphes = 1;
  unsigned long nombre_lignes;

  /* On essaye d'ouvrir le fichier */
  fichier = fopen( nom_fichier, "rb" );
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir le fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    return -1;
  }

  /* On saute l'en-tte [et les commentaires initiaux] */
  switch ( format )
  {
  case ASCII_FORMAT_FEFF_F:
  case ASCII_FORMAT_FEFF_C:
  case ASCII_FORMAT_FEFF_Cp:
  case ASCII_FORMAT_FEFF_X:
    retour = FEFF_SauterEnTete( fichier );
    break;

  default:
    retour = ASCII_SauterEnTete( fichier, infos->lg_en_tete, infos->commentaire );
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant passer l'en-tte [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    (void) fclose( fichier );
    return -2;
  }

  /* On compte le nombre de lignes dans le fichier */
  nombre_lignes = compte_lignes( fichier, infos->commentaire );
#ifdef _DEVELOPPE_
  printf( "%s : %lu lignes comptes dans %s\n", __FUNCTION__, nombre_lignes, nom_fichier );
#endif
  if ( nombre_lignes < 3 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Trop peu de lignes utiles --- Chargement annul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return -4;
  }

  /* Pralable au chargement : l'historique */
  if ( infos->proprietes & IA_AVEC_HISTORIQUE ) {
    HIST_Titre(1, "Importer un fichier", "E/S");
    HIST_Texte(2, 0, "Fichier : %s\n", nom_fichier);
    HIST_Texte(4, 0, "Lignes  ignorer : %u au dbut, celles introduites par %c (%d)\n",
	       infos->lg_en_tete, infos->commentaire, (int) infos->commentaire);
  }

  if ( infos->colonne_Im > 0 ) { /* Graphe  valeurs complexes */
    graphe = IA_Complexe( fichier, nom_fichier, nombre_lignes, infos );
    graphes = &graphe;
  } else if ( infos->colonne_Z > 0 ) { /* Graphe  valeurs relles dans l'espace [3D] */
    OPTION_A_FAIRE( "Mettre  jour l'import des graphes 3D" );

    import_asc_3d( nom_fichier, infos->nbr_colonnes, infos->colonne_X, infos->colonne_Y, infos->colonne_Z, infos->lg_en_tete, infos->commentaire);
  } else if ( infos->proprietes & IA_MULTI_COLONNES ) { /* Plusieurs graphes,  X commun */
    unsigned long nbr_intervalles;
    unsigned int nbr_colonnes;

    /* Combien de colonnes ? */
    nbr_colonnes = infos->nbr_colonnes;
    if ( 0 == nbr_colonnes ) {
      nbr_colonnes = compte_colonnes( fichier, infos->separateur );
    }
    if ( infos->proprietes & IA_AVEC_HISTORIQUE ) {
      HIST_Texte(4, 0, "Nombre total de colonnes : %d\n", nbr_colonnes);
    }

    /* Table des graphes  charger */
    infos->charger_grf = (boolean *) malloc( sizeof( boolean ) * nbr_colonnes );
    if ( infos->charger_grf != NULL ) {
      Intervalle_I *colonnes;
      unsigned int i_col;

      /* On gnre les intervalles */
      colonnes = TXT_genere_intervalles( infos->filtre , &nbr_intervalles );
      if (NULL == colonnes) { /* Erreur survenue */
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer une table d'intervalle de slection -> on essaye de tout charger\n",
		       __FILE__, __FUNCTION__, __LINE__);

	free( infos->charger_grf ); infos->charger_grf = NULL;
      } else {
	for ( i_col = 0 ; i_col < nbr_colonnes ; i_col++ ) {
	  /* Rq. Tableau commence  0, mais colonnes  1 [utilisateur...] */
	  infos->charger_grf[ i_col ] = TXT_est_dans_intervalle( i_col + 1, colonnes, nbr_intervalles);
	}

	/* Plus besoin de la table des intervalles */
	free(colonnes); colonnes = NULL;
      }
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la table -> on essaye de tout charger !\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    /* On charge */
    graphes = IA_ReelsMulti( fichier, nom_fichier, nombre_lignes, infos );
    free( infos->charger_grf ); infos->charger_grf = NULL;

    nombre_graphes = nbr_colonnes;
    liberer_table_graphes = VRAI;
  } else { /* Graphe -- unique --  valeurs relles */
    graphe = IA_ReelUnique( fichier, nom_fichier, nombre_lignes, infos );
    graphes = &graphe;
  }

  /* Plus besoin du fichier */
  (void) fclose( fichier );

  if ( NULL == graphes ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le chargement du fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
  }

  /* Traitement post-chargement */
  switch ( format )
  {
  case ASCII_FORMAT_FEFF_F:
  case ASCII_FORMAT_FEFF_C:
  case ASCII_FORMAT_FEFF_Cp:
  case ASCII_FORMAT_FEFF_X:
    retour = FEFF_PostChargement( format, infos->colonne_X, nombre_graphes, graphes, nom_fichier );
    break;

  default:
    retour = ASCII_PostChargement( infos->type_import, nombre_graphes, graphes );
  }

  /* Multi-graphes -> on libre la table des graphes chargs */
  if ( VRAI == liberer_table_graphes ) {
    free( graphes ); graphes = NULL;
  }

  /* Chargement russi */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Routines lmentaires ~-~-~-~-~-~-~-~-~ */

/* -------- Sauter l'en-tte -------- */

int ASCII_SauterEnTete(FILE *fichier, unsigned long nbr_lg_en_tete, int commentaire)
{
  int c;

  if ( NULL == fichier ) return -1;

  /* On passe le nombre de lignes indiques */
  c = sauter_en_tete( fichier, nbr_lg_en_tete, commentaire );
  if ( EOF == c ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant sauter %lu lignes d'en-tte\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_lg_en_tete);
    return -2;
  }

  /* On ignore toutes les lignes de commentaire qui suivent */
  while ( c == commentaire ) {
    c = ignore_ligne( fichier );
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Le fichier ne contient que des commentaires\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -3;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Mise en couleur et tout et tout -------- */

int ASCII_PostChargement(int type, unsigned int nombre, struct graphe **table)
{
  unsigned int i;

  if ( NULL == table ) {
    (void) fprintf(stderr, "%s,%s l.%u : Table des graphes nulle\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On met les types correctement */
  for ( i = 0 ; i < nombre ; i++ ) {
    GRF_ChangerType( table[ i ], type );
  }

  /* On met les couleurs, si en multi-colonnes */
  if ( nombre > 1 ) {
    unsigned int couleur = NOIR;
    unsigned int style = 1, marque = 0;

    for ( i = 0 ; i < nombre ; i++ ) {
      if ( table[ i ] != NULL ) {
	GRF_couleur( table[ i ], couleur, style, -1 );
	couleur++;
	if ( couleur >= 16 ) {
	  couleur = NOIR;
	  style++;
	}
	if ( style > BI_NOMBRE_STYLES_LIGNES ) {
	  style = 1;
	  marque++;
	}
      }
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Graphe  valeurs complexes : X (Re, Im), + erreurs ventuellement -------- */

Graphe *IA_Complexe(FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos)
{
  Complexe *y, *y_err = NULL;
  Graphe *graphe;
  char *nom_graphe;
  double *valeurs;
  int c, retour;
  unsigned int nbr_colonnes;
  unsigned int colonne_sRe;
  unsigned long i_ligne = 0;

  /* Vrifications */
  if ( NULL == infos ) return NULL;
  if ( ( 0 == infos->colonne_X ) || ( 0 == infos->colonne_Re ) || ( 0 == infos->colonne_Im ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Une des colonnes n'a pas t indique [X : %u, Re : %u, Im : %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos->colonne_X, infos->colonne_Re, infos->colonne_Im);
    return NULL;
  }

  /* Si ncessaire, on cherche le nombre de colonnes */
  if ( 0 == infos->nbr_colonnes ) {
    nbr_colonnes = compte_colonnes( fichier, infos->separateur );
    if ( 0 == nbr_colonnes ) {
      (void) fprintf(stderr, "%s,%s l.%u : Nombre de colonnes dtectes absurde\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
  } else { 
    nbr_colonnes = infos->nbr_colonnes;
  }

  /* On rserve pour les valeurs */
  valeurs = (double *) malloc( sizeof( double ) * nbr_colonnes );
  if ( NULL == valeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver les valeurs  lire ! [%u colonnes]\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_colonnes);
    return NULL;
  }

  /* On prpare le graphe */
  nom_graphe = TXT_genere_chaine( "%F : (#%u + i*#%u) = f(#%u)", nom_fichier, infos->colonne_Re, infos->colonne_Im, infos->colonne_X );
  graphe = Serie_nv_graphe( infos->serie, NULL, nbr_lignes, GR_SSTYPE, GRF_FCT_COMPLEXE, nom_graphe );
  free( nom_graphe ); nom_graphe = NULL;
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR -- Impossible de crer le graphe\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( valeurs );
    return NULL;
  }
  y = ( Complexe * ) ( graphe->y );

  if ( infos->proprietes & IA_AVEC_HISTORIQUE ) {
    /* On complte l'historique */
    HIST_Texte(4, 0, "Nombre total de colonnes : %d\n", nbr_colonnes);
    HIST_Texte(2, 0, "Graphe complexe, Z=f(X) --- X : col. %d ; Re(Z) : col. %d ; Im(Z) : col. %d\n", infos->colonne_X, infos->colonne_Re, infos->colonne_Im );
    HIST_Texte(2, 0, "Graphe cr : %s\n", graphe->nom );
  }

  /* Faut-il aussi crer des erreurs ? */
  if ( ( infos->colonne_sX > 0 ) && ( infos->colonne_sX <= nbr_colonnes ) ) { /* pour les X : oui */
    Graphe *err_X;

    err_X = GRF_creer_erreurs( graphe, AXE_X );
    if ( NULL == err_X ) { /* On annule le chargement des erreurs */
      OPTION_A_FAIRE( "Avertir que les erreurs en X ne seront pas charges" );
    }
  }

  colonne_sRe = infos->colonne_sY; /* Changement de nom par commodit ! */
  if ( ( ( colonne_sRe > 0 ) || ( infos->colonne_sIm > 0 ) ) &&
       ( ( colonne_sRe <= nbr_colonnes ) && ( infos->colonne_sIm <= nbr_colonnes ) ) ) {
    Graphe *err_Y;

    err_Y = GRF_creer_erreurs( graphe, AXE_Y );
    if ( NULL == err_Y ) { /* On annule le chargement des erreurs */
      OPTION_A_FAIRE( "Avertir que les erreurs ne seront pas charges" );
      y_err = NULL;
    } else {
      y_err = (Complexe *) ( err_Y->y );
    }
  }

  do {
    c = fgetc( fichier );
    ungetc( c, fichier );

    /* On saute les commentaires ventuels */
    while ( infos->commentaire == c ) {
      c = ignore_ligne( fichier );
    }
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte [%s : l. %lu / %lu]\n",
		     __FILE__, __FUNCTION__, __LINE__, nom_fichier, i_ligne + 1, nbr_lignes);
      break;
    }

    /* On lit la ligne de valeurs [en ignorant les erreurs associes...] */
    retour = lit_valeurs( fichier, nbr_colonnes, infos->separateur, valeurs, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR fichier %s l. %lu / %lu [%g, %g, %g]\n",
		     __FILE__, __FUNCTION__, __LINE__, nom_fichier, i_ligne + 1, nbr_lignes,
		     valeurs[ infos->colonne_X - 1 ], valeurs[ infos->colonne_Re - 1 ], valeurs[ infos->colonne_Im - 1 ]);
      break;
    }

    /* On mmorise les valeurs */
    if ( infos->colonne_X <= nbr_colonnes ) {
      graphe->x[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
      if ( graphe->erreur_y != NULL ) {
	graphe->erreur_y->x[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
      }
    }
    if ( infos->colonne_Re <= nbr_colonnes ) {
      y[ i_ligne ].re = valeurs[ infos->colonne_Re - 1 ];
    }
    if ( infos->colonne_Im <= nbr_colonnes ) {
      y[ i_ligne ].im = valeurs[ infos->colonne_Im - 1 ];
    }
    if ( ( infos->colonne_sX > 0 ) && ( infos->colonne_sX <= nbr_colonnes ) ) {
      if ( graphe->erreur_x != NULL ) {
	graphe->erreur_x->y[ i_ligne ] = valeurs[ infos->colonne_sX - 1 ];
      }
    }
    if ( y_err != NULL ) {
      if ( ( colonne_sRe > 0 ) && ( colonne_sRe <= nbr_colonnes ) ) {
	y_err[ i_ligne ].re = valeurs[ colonne_sRe - 1 ];
      }
      if ( ( infos->colonne_sIm > 0 ) && ( infos->colonne_sIm <= nbr_colonnes ) ) {
	y_err[ i_ligne ].im = valeurs[ infos->colonne_sIm - 1 ];
      }
    }

/*     (void) printf("%s : X = %g -> Re = %g, Im = %g\n", __FUNCTION__, valeurs[ infos->colonne_X - 1 ], valeurs[ infos->colonne_Re - 1 ], valeurs[ infos->colonne_Im - 1 ]); */

    i_ligne++;
  } while ( i_ligne < nbr_lignes );

  /* Plus besoin de la table des valeurs */
  free( valeurs );

  /* On met  jour les bornes */
  GRF_bornes( graphe, DESORDONNE, DESORDONNE );
  GRF_bornes( graphe->erreur_x, DESORDONNE, DESORDONNE );
  GRF_bornes( graphe->erreur_y, DESORDONNE, DESORDONNE );

  /* Le chargement a russi */
  return graphe;
}

/* -------- Import ASCII d'un fichier contenant plusieurs graphes -------- */

Graphe **IA_ReelsMulti(FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos)
{
  Graphe **table_graphes = NULL;
  unsigned int i_grf, nbr_grf = 0;
  unsigned int nbr_colonnes;
  unsigned long i_ligne = 0;
  int genre = -1, c;
  double *valeurs, *erreurs = NULL;

  if ( ( NULL == fichier ) || ( NULL == infos ) ) return NULL;
  if ( 0 == infos->nbr_colonnes ) {
    nbr_colonnes = compte_colonnes( fichier, infos->separateur );
    if ( 0 == nbr_colonnes ) {
      (void) fprintf(stderr, "%s,%s l.%u : Nombre de colonnes dtectes absurde\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
  } else { 
    nbr_colonnes = infos->nbr_colonnes;
  }
  if ( NULL == infos->serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Srie pour le chargement non prcise.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* De quel genre seront les graphes crs ? */
  if ( 0 == infos->colonne_X ) {
    genre = GRF_SERIE; /* Pas de X prcis */
  } else if ( infos->proprietes & IA_SERIE_X_COMMUNS ) {
    if ( infos->serie->table_X != NULL ) {
      if ( infos->serie->nbr_points != nbr_lignes ) {
	(void) fprintf(stderr, "%s,%s l.%u : La srie contient dj des X en nombre diffrent -> chargement impossible\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return NULL;
      } else {
	OPTION_A_FAIRE( "Poser la question, nouveaux ou anciens X ?" );
      }
    } else {
      infos->serie->table_X = (double *) calloc( nbr_lignes, sizeof( double ) );
      if ( NULL == infos->serie->table_X ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des X\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return NULL;
      }
      infos->serie->nbr_points = nbr_lignes;
      infos->serie->drapeaux.table_X = VRAI;
    }

    genre = GRF_SERIE; /* Puisque les X sont dans la srie ! */
  } else {
    genre = GRF_FCT_REELLE;
  }

  /* On rserve la table des graphes */
  table_graphes = ( Graphe ** ) calloc( nbr_colonnes, sizeof( Graphe * ) );
  if ( NULL == table_graphes ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver une table pour %u graphes\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_colonnes);
    return NULL;
  }

  /* On rserve la table pour les valeurs & les erreurs */
  valeurs = (double *) malloc( sizeof( double ) * nbr_colonnes );
  if ( NULL == valeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver les valeurs  lire ! [%u colonnes]\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_colonnes);
    free( table_graphes );
    return NULL;
  }
  if ( infos->proprietes & IA_UTILISER_ERREURS ) {
    erreurs = (double *) malloc( sizeof( double ) * nbr_colonnes );
    if ( NULL == erreurs ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver les erreurs  lire ! [%u colonnes]\n",
		     __FILE__, __FUNCTION__, __LINE__, nbr_colonnes);
    }
  }

  /* On rserve les graphes, si possible */
  for ( i_grf = 0 ; i_grf < nbr_colonnes ; i_grf++ ) {
    /* Avec l'valuation paresseuse, ne devrait pas planter */
    if ( ( NULL == infos->charger_grf ) || ( VRAI == infos->charger_grf[ i_grf ] ) ) {
      char *nom;

      nom = TXT_genere_chaine( "%F : #%u = f(#%u)", nom_fichier, i_grf + 1, infos->colonne_X );
      table_graphes[ i_grf ] = Serie_nv_graphe( infos->serie, NULL, nbr_lignes, GR_SSTYPE, genre, nom );
      free( nom );
      if ( table_graphes[ i_grf ] != NULL ) nbr_grf++;
    }
  }
  if ( 0 == nbr_grf ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun graphe charg !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( valeurs ); free( erreurs );
    free( table_graphes );
    return NULL;
  }

  /* On fait la lecture */
  do {
    c = fgetc( fichier );
    ungetc( c, fichier );

    /* On saute les commentaires ventuels */
    while ( infos->commentaire == c ) {
      c = ignore_ligne( fichier );
    }
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte [%s : l. %lu / %lu]\n",
		     __FILE__, __FUNCTION__, __LINE__, nom_fichier, i_ligne + 1, nbr_lignes);
      break;
    }

    /* On lit la ligne de valeurs [avec si besoin est les erreurs associes...] */
    lit_valeurs( fichier, nbr_colonnes, infos->separateur, valeurs, erreurs );

    /* On mmorise les valeurs */
    for ( i_grf = 0 ; i_grf < nbr_colonnes ; i_grf++ ) {
      if ( NULL == table_graphes[ i_grf ] )
	continue;

      table_graphes[ i_grf ]->y[ i_ligne ] = valeurs[ i_grf ];
      if ( ( infos->colonne_X > 0 ) && ( infos->colonne_X <= nbr_colonnes ) ) {
	if ( table_graphes[ i_grf ]->x != NULL ) {
	  table_graphes[ i_grf ]->x[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
	  if ( ( erreurs != NULL ) && ( erreurs[ infos->colonne_X - 1 ] > 0.0 ) ) {
	    OPTION_A_FAIRE( "Mmoriser l'erreur sur les X aussi" );
	  }
	} else if ( infos->serie->table_X != NULL ) {
	  infos->serie->table_X[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
	}
	if ( table_graphes[ i_grf ]->erreur_y != NULL ) {
	  if ( table_graphes[ i_grf ]->erreur_y->x != NULL ) { /* NULL si type GRF_SERIE ! */
	    table_graphes[ i_grf ]->erreur_y->x[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
	  }
	}
      }

      /* Erreurs  prendre en compte ? */
      if ( ( erreurs != NULL ) && ( erreurs[ i_grf ] > 0.0 ) ) {
	if ( NULL == table_graphes[ i_grf ]->erreur_y ) {
	  (void) GRF_creer_erreurs( table_graphes[ i_grf ], AXE_Y );
	}
	if ( table_graphes[ i_grf ]->erreur_y != NULL ) { /* En cas d'chec de la cration, ne pas remplir ! */
	  table_graphes[ i_grf ]->erreur_y->y[ i_ligne ] = erreurs[ i_grf ];
	}
      }
    }

    i_ligne++;
  } while ( i_ligne < nbr_lignes );

  /* Filtre pour liminer les NaN */
  for ( i_grf = 0 ; i_grf < nbr_colonnes ; i_grf++ ) {
    (void) GRF_OterValeursManquantes( table_graphes[ i_grf ] );
  }

  /* Bornes */
  for ( i_grf = 0 ; i_grf < nbr_colonnes ; i_grf++ ) {
    GRF_bornes( table_graphes[ i_grf ], DESORDONNE, DESORDONNE );
    if ( table_graphes[ i_grf ] != NULL ) {
      GRF_bornes( table_graphes[ i_grf ]->erreur_y, DESORDONNE, DESORDONNE );
    }
  }

  /* Termin, tout s'est bien pass */
  free( valeurs );
  free( erreurs );
  return table_graphes;
}

/* -------- Import ASCII d'un fichier contenant un seul graphe -------- */

Graphe *IA_ReelUnique(FILE *fichier, const char *nom_fichier, unsigned long nbr_lignes, struct infos_import_grf *infos)
{
  Graphe *graphe;
  char *nom_graphe;
  double *valeurs;
  int c;
  unsigned int nbr_colonnes;
  unsigned long i_ligne = 0;

  /* Vrifications */
  if ( 0 == infos->colonne_Y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Une des colonnes n'a pas t indique [X : %u, Y : %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos->colonne_X, infos->colonne_Y);
    return NULL;
  }

  /* Si ncessaire, on cherche le nombre de colonnes */
  if ( 0 == infos->nbr_colonnes ) {
    nbr_colonnes = compte_colonnes( fichier, infos->separateur );
    if ( 0 == nbr_colonnes ) {
      (void) fprintf(stderr, "%s,%s l.%u : Nombre de colonnes dtectes absurde\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
  } else { 
    nbr_colonnes = infos->nbr_colonnes;
  }

  /* On rserve pour les valeurs */
  valeurs = (double *) malloc( sizeof( double ) * nbr_colonnes );
  if ( NULL == valeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver les valeurs  lire ! [%u colonnes]\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_colonnes);
    return NULL;
  }

  /* On prpare le graphe */
  nom_graphe = TXT_genere_chaine( "%F : #%u = f(#%u)", nom_fichier, infos->colonne_Y, infos->colonne_X );
  graphe = Serie_nv_graphe( infos->serie, NULL, nbr_lignes, GR_SSTYPE, ( infos->colonne_X > 0 ) ? ( GRF_FCT_REELLE ) : ( GRF_SERIE ), nom_graphe );
  free( nom_graphe ); nom_graphe = NULL;
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR -- Impossible de crer le graphe\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( valeurs );
    return NULL;
  }
  /* Il n'a aucune unit indique ! */
  GRF_ChangerUnite( graphe, AXE_X, "" );
  GRF_ChangerUnite( graphe, AXE_X, "" );

  if ( infos->proprietes & IA_AVEC_HISTORIQUE ) {
    /* On complte l'historique */
    HIST_Texte(4, 0, "Nombre total de colonnes : %d\n", nbr_colonnes);
    HIST_Texte(2, 0, "Graphe rel, Y=f(X) --- X : col. %d ; Y : col. %d\n", infos->colonne_X, infos->colonne_Y );
    HIST_Texte(2, 0, "Graphe cre : %s\n", graphe->nom );
  }

  /* Faut-il aussi crer des erreurs ? */
  if ( ( infos->colonne_sX > 0 ) && ( infos->colonne_sX <= nbr_colonnes ) ) { /* pour les X : oui */
    Graphe *err_X;

    err_X = GRF_creer_erreurs( graphe, AXE_X );
    if ( NULL == err_X ) { /* On annule le chargement des erreurs */
      OPTION_A_FAIRE( "Avertir que les erreurs en X ne seront pas charges" );
    }
  }

  if ( ( infos->colonne_sY > 0 ) && ( infos->colonne_sY <= nbr_colonnes ) ) {
    Graphe *err_Y;

    err_Y = GRF_creer_erreurs( graphe, AXE_Y );
    if ( NULL == err_Y ) { /* On annule le chargement des erreurs */
      OPTION_A_FAIRE( "Avertir que les erreurs en Y ne seront pas charges" );
    }
  }

  do {
    c = fgetc( fichier );
    ungetc( c, fichier );

    /* On saute les commentaires ventuels */
    while ( infos->commentaire == c ) {
      c = ignore_ligne( fichier );
    }
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Fin de fichier atteinte [%s : l. %lu / %lu]\n",
		     __FILE__, __FUNCTION__, __LINE__, nom_fichier, i_ligne + 1, nbr_lignes);
      break;
    }

    /* On lit la ligne de valeurs [en ignorant les erreurs associes...] */
    lit_valeurs( fichier, nbr_colonnes, infos->separateur, valeurs, NULL );

    /* On mmorise les valeurs */
    if ( ( infos->colonne_X <= nbr_colonnes ) && ( infos->colonne_X > 0 ) ) {
      graphe->x[ i_ligne ] = valeurs[ infos->colonne_X - 1 ];
      /* ... fin d' faire sauter ... */
    }
    if ( infos->colonne_Y <= nbr_colonnes ) {
      graphe->y[ i_ligne ] = valeurs[ infos->colonne_Y - 1 ];
    }
    if ( ( infos->colonne_sX > 0 ) && ( infos->colonne_sX <= nbr_colonnes ) ) {
      if ( graphe->erreur_x != NULL ) {
	graphe->erreur_x->y[ i_ligne ] = valeurs[ infos->colonne_sX - 1 ];
      }
    }
    if ( ( infos->colonne_sY > 0 ) && ( infos->colonne_sY <= nbr_colonnes ) ) {
      if ( graphe->erreur_y != NULL ) {
	graphe->erreur_y->y[ i_ligne ] = valeurs[ infos->colonne_sY - 1 ];
      }
    }

    i_ligne++;
  } while ( i_ligne < nbr_lignes );

  /* Plus besoin de la table des valeurs */
  free( valeurs );

  /* On met  jour les bornes */
  GRF_bornes( graphe, DESORDONNE, DESORDONNE );
  GRF_bornes( graphe->erreur_x, DESORDONNE, DESORDONNE );
  GRF_bornes( graphe->erreur_y, DESORDONNE, DESORDONNE );

  /* Si srie (sans X), on utilise des marqueurs pour tracer [car le trac continu ne fonctionne pas encore !] */
  if ( NULL == graphe->x ) {
    GRF_couleur( graphe, -1, 0, 1 );
  }

  /* Le chargement a russi */
  return graphe;
}

/* -------- Import d'un graphe en 3d - pas d'erreurs, de toute faon. -------- */

boolean import_asc_3d(const char *nom, int nbr_colonnes, int x_colonne,
		      int y_colonne, int z_colonne,
		      unsigned int en_tete, char commentaire)
{
  FILE *fichier;
  int c;
  register double *x, *y, *z;
  unsigned long nbr_lignes = 0, i;
  Graphe *graphe;

  fichier = fopen(nom, "rb");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom, NULL, NULL); /* Erreur  la lecture */
    return FAUX;
  }

  /* On commence par passer les lignes  ignorer */

  if ( sauter_lignes(fichier, en_tete) == FAUX ) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom, fichier, "Impossible de sauter les premires lignes"); /* Erreur  la lecture */
    (void) fclose(fichier);
    return FAUX;
  }

  nbr_lignes = compte_lignes(fichier, commentaire);
  if (nbr_lignes < 2) { /* Trop peu de donnes */
    (void) fprintf(stderr, "import_asc_3d : le fichier ne contient pas assez de lignes\n");
    (void) fclose(fichier);
    return FAUX;
  }

  graphe = GRF_nouveau( NULL, nbr_lignes, GR_SSTYPE, GRF_COURBE_ESPACE, nom );
  if (graphe == NULL) {
    (void) fclose(fichier);
    return FAUX;
  }

  x = graphe->x; y = graphe->y; z = graphe->z.z;

  i = 0;
  do {
    c = fgetc(fichier);

    if (c != commentaire) { /* Ce n'est pas une ligne de commentaire */
      int j;

      ungetc(c, fichier); /* On recule d'un caractre */
      
      for (j = 1 ; j <= nbr_colonnes ; j++) {
	if (j == x_colonne) {
	  fscanf(fichier, "%lf", x);
	} else if (j == y_colonne) {
	  fscanf(fichier, "%lf", y);
	} else if (j == z_colonne) {
	  fscanf(fichier, "%lf", z);
	} else {
	  fscanf(fichier, "%*s");
	}
      }
      if (x_colonne == 0) { /* On utilise en x l'index dans le fichier */
	*x = (double) i;
      }
      i++; /* On vient de lire une ligne de donnes */
      x++; y++; z++;
    }

    ignore_ligne(fichier);

  } while (i < nbr_lignes); /* Tant que l'on n'a pas tout lu */

  (void) fclose(fichier); /* C'est bon */
  HIST_Texte(1, 0, "Graphe : %s\n", graphe->nom);

  GRF_bornes(graphe, DESORDONNE, DESORDONNE);
  GRF_aller(graphe);
  return VRAI;
}
