/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1998-2015

   Définitions pour les modèles de décomposition sur une base de spectres
*/

#ifndef MODELES_DECOMPOSE_H_
# define MODELES_DECOMPOSE_H_ /* Protection contre l'inclusion multiple */

# include "c_formel.h"		/* Obligatoire pour le coefficient dans la décomposition linéaire */

/* ------ Rappel de définitions extérieures, au cas où... ------ */

struct graphe;			/* Définie dans "graphes.h" */
struct modele_ajuste;		/* Définie dans "modeles.h" */
struct variable;		/* Définie dans "coefficients.h" */

/* -~-~-~-~-~-~-~-~ Les structures nécessaires  -~-~-~-~-~-~-~-~ */

/* ------ Chaque spectre de base ------ */

struct element_base {
  struct graphe *graphe;       /* Le graphe servant de référence */
  struct variable coefficient; /* Coefficient de la combinaison linéaire */
  double erreur;	       /* Sur le coefficient */
  double *a, *b, *c;           /* Les coefficients de la spline lissant le graphe */
};

/* ------ La décomposition linéaire utilisée ------ */

typedef struct decomposition {
  unsigned int methode;	        /* Comment chercher les coefs ? [cf #define ci-dessous] */

  unsigned long nombre_bases;	/* Nombre de graphes de référence */
  struct element_base *base;	/* Tableau dynamique */

  struct graphe *alpha, *beta;  /* Dans le cas où on fait un "suivi" point par point */
} DCMP_infos;

/* -~-~-~-~-~-~-~-~ Les différentes méthodes de décomposition  -~-~-~-~-~-~-~-~ */

#define MODELE_DECOMPOSITION_MOINDRES_CARRES  0 /* Moindres carrés classiques */
#define MODELE_DECOMPOSITION_MDR_CARR_SOMME1  1 /* Moindres carrés avec contrainte : la somme a+b=1 */
#define MODELE_DECOMPOSITION_MOYENNES         2 /* Cf. fonction ; méthode peu convainquante ; limitée à 2 spectres de référence */
#define MODELE_DECOMPOSITION_MOYENNE_VARIANCE 3	/* Idem */
#define MODELE_DECOMPOSITION_SUIVI            4 /* Suivi "cinétique" */
#define MODELE_DECOMPOSITION_SUIVI_LINEAIRE   5 /* Suivi "cinétique" avec contrainte linéaire */
#define MODELE_DECOMPOSITION_ACP              6 /* Analyse en composantes principales */

#define MODELE_DECOMPOSITION_NOMBRE_DE_METHODES  7

/* -~-~-~-~-~-~-~-~ Les fonctions de gestion de ces modèles  -~-~-~-~-~-~-~-~ */

struct decomposition    *MODELE_NouveauDecomposition(void);

int MODELE_Decomposition_AjouterVecteurDeBase  (struct decomposition *decomposition, struct graphe *graphe_a_ajouter);
int MODELE_Decomposition_EnleverVecteurDeBase  (struct decomposition *decomposition, unsigned long numero);
int MODELE_Decomposition_RemplacerVecteurDeBase(struct decomposition *decomposition, unsigned long numero, struct graphe *nouveau_graphe);

int MODELE_Decomposition_Liberer             (struct decomposition *decomposition);

int MODELE_Decomposition_Controler           (struct decomposition *decomposition);
int MODELE_Decomposition_Decomposer          (struct graphe *graphe, struct decomposition *decomposition);
int MODELE_Decomposition_PreparerAjustement  (struct modele_ajuste *modele);

/* ------- Les codes d'erreur possible ------ */

#define MODELE_DECOMPOSITION_ERREUR_DEJA_DANS_BASE (+1) /* Graphe indiqué déjà dans la base... */
#define MODELE_DECOMPOSITION_ERREUR_DECOMPOSE      (+2) /* Graphe indiqué est celui décomposé... */

#endif	/* MODELES_DECOMPOSE_H_ */
