/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 28 juin 2002

   Import d'un fichier ralis  l'ESRF
*/

#ifndef LASE_ESRF_H_

#define LASE_ESRF_H_ /* Protection */

/* Definition des differentes lignes de l'ESRF */

#define LG_MAX_LIGNE_ESRF 21

#define ESRF_ID01   0 /* Diffusion anomale */
#define ESRF_ID02   1 /* Haute brillance */
#define ESRF_ID03   2 /* Surfaces */
#define ESRF_ID08   3
#define ESRF_ID09   4
#define ESRF_ID10A  5
#define ESRF_ID10B  6
#define ESRF_ID11   7
#define ESRF_ID12   8
#define ESRF_ID13   9
#define ESRF_ID141 10
#define ESRF_ID142 11
#define ESRF_ID143 12
#define ESRF_ID144 13
#define ESRF_ID15A 14
#define ESRF_ID15B 15
#define ESRF_ID16  16
#define ESRF_ID17  17
#define ESRF_ID18  18
#define ESRF_ID19  19
#define ESRF_ID20  20
#define ESRF_ID21  21 /* fluo X, elements legers */
#define ESRF_ID22  22 /* fluo X, elements lourds */
#define ESRF_ID23  23
#define ESRF_ID24  24
#define ESRF_ID26  25
#define ESRF_ID27  26
#define ESRF_ID28  27
#define ESRF_ID29  28
#define ESRF_ID30  29
#define ESRF_ID31  30
#define ESRF_ID32  31

#define ESRF_BM01A 32
#define ESRF_BM01B 33 /* SNBL, exafs */
#define ESRF_BM02  34
#define ESRF_BM05  35
#define ESRF_BM08  36
#define ESRF_BM14  37
#define ESRF_BM16  38
#define ESRF_BM20  39
#define ESRF_BM25A 40
#define ESRF_BM25B 41
#define ESRF_BM26  42
#define ESRF_BM28  43
#define ESRF_BM29  44
#define ESRF_BM30A 45
#define ESRF_BM30B 46

#define ESRF_NOMBRE_LIGNES 47

/* ~~~~~~~~ Fonctions gnriques ~~~~~~~~~~~~~ */

#include "txt_utils.h" /* Pour les intervalles */

int lit_esrf(const char *nom, Intervalle_I *intervalles, unsigned long nbr_intervalles, unsigned int ligne);

unsigned long ESRF_CompterLignesScan (FILE *fichier);
boolean       ESRF_TrouverScanSuivant(FILE *fichier);

/* ~~~~~~~~ Fonctions ligne par ligne ~~~~~~~~ */

/* Les differents types de commandes possibles */

#define ESRF_AUTRE  801 /* Arbitraires... */ 
#define SCAN_ABSOLU 800
#define SCAN_RELATIF  1 /* Numero du texte dscan    */
#define ESRF_MCA      3 /* Numero du texte mcaacq   */
#define GRILLE_MESH   4 /* Numero du texte mesh     */
#define ZAP_IMAGE     7 /* Numero du texte zapimage */
#define ZAP_LIGNE     8 /* Numero du texte zapline  */

struct serie; /* Dans Include/serie.h */

int bm01b_lit_exafs(FILE *fichier, int numero_scan, const char *nom_fichier_spec);

struct image  *id21_lit_zapimg(FILE *fichier, int numero_scan, const char *nom_fichier_spec);
struct graphe *id22_lit_scan  (FILE *fichier, int mode, int numero_scan, char *mouvement, const char *nom_fichier, boolean *ok);
struct graphe *id22_lit_mca   (FILE *fichier, int numero_scan, const char *nom_fichier, struct serie *serie_mca);
struct image  *id22_lit_mesh  (FILE *fichier, int numero_scan, const char *nom_fichier);
struct image  *id22_lit_mesh_MCA(FILE *fichier, int numero_scan, const char *nom_fichier);

int id21_ligne_image(const char *nom_fichier, struct image *image, unsigned int num_ligne, unsigned int nbr_colonnes,
		     struct serie *serie_mca);

void fin_colonnes (void);

#endif /* LASE_ESRF_H_ */
