/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Fichier  inclure : dfinitions gnrales
*/

#ifndef IMAGES_H_
# define IMAGES_H_ /* Protection contre l'inclusion multiple */

# include "complexe.h"
# include "txt_utils.h" /* Pour les titres, peut servir... */

struct composant;		/* Dfinie dans echantillon.h */
struct dess_image;		/* Dfinie dans tracer.h */
struct expr;			/* Dfinie dans c_formel.h */
struct graphe;			/* Dfinie dans graphes.h */
struct matiere;			/* Dfinie dans echantillon.h */
struct matrice;			/* Dfinie dans matrices.h */
struct orientation_3d;		/* Dfinie dans Utilitaires/utils_3d.h */
struct serie;			/* Dfinie dans serie.h */

/* -------- Variables gnrales au programme ------ */

#define MM_V_INTERNE(a) ( (a) * 1000.0 )
#define INTERNE_V_MM(a) ( (a) / 1000.0 )

/* Structure dfinissant une image.

   Les images sont les quivalents (simples) des graphes pour les surfaces z = f(x,y).
   Deux faons de travailler : grille fixe [le plus frquent] ou grille libre.

   En pratique : les valeurs sont (x, y, f(x, y)) ; seul f(x,y) est stocke, car on suppose
                 les valeurs des (x,y) rparties sur une grille rectangulaire rgulire, donc
    il y a larg valeurs de x,  partir de x_min, espaces de l_reel
           haut            y              y_min,             h_reel
   Le repre est un repre mathmatique classique.

   Les valeurs sont numrotes dans l'ordre 0  1   2  ...  L-1
                                            L L+1 L+2 ... 2L-1

   On trouve (x, y)  partir du numro de case : la case (i,j) contient
   la valeur en (x_min + (i + 1/2) * l_reel, y_min + (j + 1/2) * h_reel)

   La valeur dans la case (i,j)
    correspond  la valeur _au centre de la case_ : +-----+  X = index de la case considre
                                                    |  O  |  0 = point prenant la valeur mmorise
                                                    X-----+  + = intersection avec les autres cases

   La nature des donnes contenues dans l'image (de z) est indique par le champ type (comme un graphe)
   Le codage de ces donnes est indiqu par le champ genre (comme un graphe)

   Les images sont agences en sries (listes simplements chanes), comme les graphes.
*/

/* Nature des donnes codes dans l'image */

#define IMG_GENRE_REELLE    0 /* Valeurs de type double     */
#define IMG_GENRE_COMPLEXE  1 /* Valeurs de type complexe   */
#define IMG_GENRE_GRAPHE    2 /* En chaque point, un graphe [NULL ventuellement].
				 Les graphes appartiennent tous  une mme srie, protge.
				 ATTENTION, deux images peuvent tre associes  la mme srie !
			      */
#define IMG_GENRE_DIAGRAMME 3 /* Collection de graphes dfinissant des frontires.
			       Mme remarque quand aux graphes & sries, mais NULL interdit.
			      */
#define IMG_GENRE_PHOTO     4 /* Image pour illustration... */
#define IMG_GENRE_COMPO     5 /* Carte de compositions chimiques -> struct matiere en chaque point
				 Si NULL en un point, indique un trou (donc NULL autoris !)
			       Min = 0 (trou), Max = nombre de compositions diffrentes */
#define IMG_GENRE_ERREUR    6 /* Valeurs des incertitudes en chaque point */
#define IMG_GENRE_CONTINGENCE 7	/* Valeur de type double ; contient une table de contingence : X et Y sont des modalits...
				   Normalement entier >0, mais peut tre non-entier quand
				    - effectif thorique ;
				    - variable rponse en fait quantitative (!)
				 */

#define NOMBRE_GENRE_IMAGES (IMG_GENRE_CONTINGENCE + 1)

const char *IMG_TXT_Genre(int genre);

/* Type des donnes : pour faciliter le travail de l'utilisateur uniquement */

#define IMG_AUTRE_TYPE       -3 /* Type spcial, non reconnu par les cas prcdents */
#define IMG_SANS_TYPE        -2 /* Pas de type particulier : fonction mathmatique p. ex. */

#define IMG_TYPE_CARTOGRAPHIE 0 /* Carte. Si graphes associs, tous de mme type : dfinit le type de carte.
				          Sinon, suppose carte de concentration
				   X, Y : distances, supposes donnes en MICRONS */
#define IMG_TYPE_TF           1 /* TF d'une carte. Valeurs complexes en gnral */
#define IMG_TYPE_ERREURS      2 /* carts-types pour chaque point d'une carte moyenne */
#define IMG_TYPE_MATRICE      3 /* Visualisation 3D d'une matrice [relle ou complexe] : (i,j) = lment de la matrice */
#define IMG_TYPE_INTENSITES   4 /* Coupe de faisceau [pourrait tre une carte, mais facilite la slection ainsi...] */
#define IMG_TYPE_DENSITE      5 /* Coupe d'chantillon : variations de densit */
#define IMG_TYPE_COMPOSITION  6 /* Coupe d'chantillon : variations de composition */
#define IMG_TYPE_REPONSE_DET  7 /* Rponse d'un dtecteur */

#define IMG_NOMBRE_TYPES 8 /* Les types spciaux ne sont pas compts ! */

const char *IMG_TXT_Type(int genre);

/* Contrle de l'affichage */

#define IMG_AVEC_PALETTE     1 /* Bit mis : il y a une palette -> correspondance  assurer [sinon, par dfaut] */
#define IMG_AVEC_COUPE       2 /* Bit mis : on dessine la trace de la coupe */
#define IMG_AVEC_CENTRE      4 /* Bit mis : on dessine le centre du repre */
#define IMG_AVEC_POINTS      8 /* Bit mis : on dessine les centres des cases */
#define IMG_AVEC_GRILLE     16 /* Bit mis : on dessine une grille passant par le centre des cases */
#define IMG_AVEC_CADRE      32 /* Bit mis : on dessine avec un cadre autour de l'image */
#define IMG_AVEC_GRADUATION 64 /* Bit mis : on gradue le cadre / les axes */
#define IMG_AVEC_AXES      128 /* Bit mis : on dessine les axes [repre absolu] */

#define IMG_FAUSSES_COULEURS BI_DEGRADE_FAUSSES_COULEURS
#define IMG_PALETTE          -2 /* Photo : contient sa palette ! */

#ifndef AXE_X  /*  conserver synchrone avec graphes.h */
# define AXE_X  1
# define AXE_Y  2
# define AXE_Z  4
#endif

struct img_palette {
  unsigned long nbr_teintes;
  unsigned int *RVB;
};

struct aff_image {
  int proprietes; /* Masque : que doit-on faire pour afficher ? */

  int couleur; /* Quelle couleur pour le dgrad ? */
  struct img_palette palette; /* Correspondance valeur -> couleur */

  char *legende_X; /* Lgende, axe des X */
  char *legende_Y; /* Lgende, axe des Y */
  char *legende_Z; /* Lgende, axe des Z */

  char *unite_X, *unite_Y, *unite_Z; /* Units associes */

  struct graphe *coupe;
  double xp_1, yp_1, xp_2, yp_2;  /* Coordonnes de dbut & fin de la coupe correspondant au profil
				     DANS LE REPRE MICROMTRIQUE
				     Si elles sont gales, aucun point n'est encore dfini */

  double *moyennes;		/* Pour une image avec un graphe par point, valeur a utiliser pour l'affichage */

  char *textes_X;		/* Nom des modalits en X [pour une table de contingence] */
  double *valeurs_X;		/* Valeurs en X [si var. quantitative] */

  char *textes_Y;		/* Nom des modalits en X [pour une table de contingence] */
  double *valeurs_Y;		/* Valeurs en Y [si var. quantitative] */
};

struct img_comp_graphes {
  struct graphe *grf_moyen;	/* Le graphe moyen associe a l'image, si necessaire */
};

typedef struct image {
  struct image *suivante;   /* Liste simplement chane d'images */
  int genre;                /* Codage des donnes dans l'image */
  int type;                 /* Type de l'image */
  unsigned int larg, haut;  /* Taille de l'image, en nombre de points
			       Grille libre : nombre de "lignes" et de "colonnes"
			    */

  double x_min, y_min;      /* Dmarrage de la grille, dans le repre absolu
			       Grille libre : point BG du rectangle englobant la grille
			    */
  double l_reel, h_reel;     /* Pas de la grille (en m par dfaut)
			       Grille libre : point HD du rectangle englobant la grille
			     */
  double val_max, val_min;   /* Valeurs extrmes, pour l'affichage */

  union {
    double *R;         /* Valeurs relles */
    Complexe *C;       /* Valeurs complexes */
    struct graphe **G; /* Graphe */
    void *I;           /* Image gif/jpeg/... */
    struct matiere **M;/* Matire = composition */
  } valeurs;

  struct {
    unsigned sauvee       :1; /* VRAI : l'image a t sauve */
    unsigned isolee       :1; /* VRAI : l'image n'appartient pas  un chanage */
    unsigned grille_libre :1; /* VRAI : f(x, y) pour (x, y) libres */
  } drapeaux;

  double *x, *y;/* Valeurs des (x,y) pour une grille libre */
  int    *ddl; /* Degrs de libert en chaque point, pour images de type erreur */

  struct image *erreurs; /* Si des erreurs sont associes aux points en chaque case
			    [Ncessairement isole, associe juste  cette image */

  char *nom; /* Cr au vol -> longueur libre */

  struct aff_image *infos_affichage; /* Images non-isoles seulement */
} Image;

/* Les sries forment une srie simplement chane */

typedef struct serie_images {
  struct serie_images *suivante; /* NULL == dernire srie */
  char *nom;			/* Cr au vol -> longueur libre */

  unsigned long nbr_images;	/* Nombre dans la srie */
  struct image *premiere_image;	/* Premire image de la liste */

  struct {
    unsigned homogene :1; /* VRAI : toutes les images sont de mme type         */
    unsigned commun   :1; /* VRAI : toutes les images sont traites simultanment
			     N'a de sens que si le prcdent est VRAI aussi    */
    unsigned protegee :1; /* VRAI : aucun graphe effaable/ajoutable par l'utilisateur */
  } drapeaux;
} Serie_Images;

#ifndef VARIABLES_GENERALES_
extern Serie_Images *IMG_premiere_serie, *IMG_serie_courante;
extern Image *image_courante;
extern Fenetre *fen_images;
#endif

/* ~-~-~-~-~-~-~-~- Travail sur les sries d'images ~-~-~-~-~-~-~-~- */

/* Cration, effacement */
Serie_Images *IMG_SERIE_CreerSerie  (Serie_Images *serie_avant, const char *nom_serie);
int           IMG_SERIE_EffacerSerie(Serie_Images *serie, boolean question, boolean eff_images);

/* Gestion de l'agencement des images dans la srie */
int IMG_InsererDansSerie(Image *image, Serie_Images *serie, Image *img_avant);
int IMG_IsolerImage     (Image *image);

int IMG_ChangerSerie(Image *image, Serie_Images *nouvelle_serie, Image *avant);

Serie_Images *IMG_QuelleSerie(Image *image);

/* Chargement, sauvegarde */
int IMG_SERIE_Sauver      (Serie_Images *serie, FILE *fichier, unsigned int options);
int IMG_SERIE_SauverIsolee(Serie_Images *serie, const char *nom_fichier, unsigned int options);

/* Modifier les paramtres internes */
int IMG_SERIE_Titre(Serie_Images *serie, const char *nouveau_nom_serie);

/* Proprits d'une srie d'images */
#define IMG_SERIE_EstProtegee(serie) ( (serie != NULL) ? ( serie->drapeaux.protegee ) : ( FAUX ) )

/* ~-~-~-~-~-~-~-~- Travail sur les images ~-~-~-~-~-~-~-~- */

/* -------- Cration, manipulation des structures -------- */

/* Cration, effacement */
Image *IMG_CreerImage      (Serie_Images *serie, Image *avant, int genre, int type, unsigned int larg, unsigned int haut);
Image *IMG_CreerImageIsolee(int genre, int type, unsigned int larg, unsigned int haut);

Image *IMG_CopierImage (Image *originale, Serie_Images *serie_arrivee, boolean isolee);

int    IMG_EffacerImage(Image *image, boolean eff_graphes);

struct serie *IMG_TrouverSerieGraphes(Image *image);

int IMG_ChangerGenre(Image *image, int nouveau_genre, struct composant *matrice, struct composant *compose);
int IMG_ChangerType (Image *image, int nouveau_type);

/* Proprits d'une image */
inline boolean IMG_EstIsolee(Image *image);

/* Affichage et associes */
void    IMG_AllerImage(Image *image);
void    IMG_AllerSerie(Serie_Images *image);

int IMG_AssocierCoupe(Image *image, struct graphe *coupe, double x1, double y1, double x2, double y2);

Fenetre *IMG_OuvreFenetreAffichage(void);
void     IMG_AdapterAffichage(struct dess_image *dessin_image);

void        IMG_Couleur       (Image *image, int couleur);
int         IMG_ChangerLegende(Image *image, unsigned int axe, const char *nouvelle_legende);
void        IMG_Titre         (Image *image, const char *nom);
int         IMG_ChangerUnite  (Image *image, unsigned int axe, const char *nouvelle_unite);

void        IMG_Moyennes      (Image *image);
const char *IMG_QuelleLegende (const Image *image, unsigned int axe);
const char *IMG_QuelleUnite   (const Image *image, unsigned int axe);
const char *IMG_TexteTypeImage(Image *image);

/* Chargement et sauvegarde */
int exp_gnuplot(Image *image, char *nom_fichier);

Image *IMG_Charger(FILE *fichier, int version, const char *nom_fichier);
int    IMG_Sauver (FILE *fichier, Image *image);
int    IMG_SauverIsolee(const char *nom_fichier, Image *image);

/* Slection */

Image **IMG_FiltrerType(Image *premiere, int type, boolean strict, unsigned long *nombre);

/* -------- Accs aux valeurs -------- */

inline int IMG_XY_v_N (Image *image, double x, double y, unsigned long *num_case);
inline int IMG_N_v_XY (Image *image, unsigned long num_case, double *x, double *y);
inline int IMG_XY_v_IJ(Image *image, double x, double y, unsigned int *i, unsigned int *j);
inline int IMG_IJ_v_XY(Image *image, unsigned int i, unsigned int j, double *x, double *y);
inline int IMG_N_v_IJ (Image *image, unsigned long num_case, unsigned int *i, unsigned int *j);
inline int IMG_IJ_v_N (Image *image, unsigned int i, unsigned int j, unsigned long *num_case);

void IMG_bornes(Image *image);

int IMG_ChangerEchelle(Image *image, int axes, double echelle_X, double echelle_Y);
int IMG_ChangerRepere (Image *image, int axes, double x_min, double y_min);

/* -------- Fonctions de gestion des images -------- */

/* Quelques utilitaires */

#define IMG_CENTRE_DANS_IMAGE   0
#define IMG_CENTRE_COIN_HG      1 /* Centre sur un coin */
#define IMG_CENTRE_COIN_HD      2
#define IMG_CENTRE_COIN_BG      3
#define IMG_CENTRE_COIN_BD      4
#define IMG_CENTRE_BORD_B       5 /* Centre sur un bord */
#define IMG_CENTRE_BORD_G       6
#define IMG_CENTRE_BORD_H       7
#define IMG_CENTRE_BORD_D       8
#define IMG_CENTRE_COTE_H      20 /* Centre latralement  l'image */
#define IMG_CENTRE_COTE_G      21
#define IMG_CENTRE_COTE_D      22
#define IMG_CENTRE_COTE_B      23
#define IMG_CENTRE_QUADRANT_HG 24 /* Centre "en diagonale" */
#define IMG_CENTRE_QUADRANT_HD 25
#define IMG_CENTRE_QUADRANT_BG 26
#define IMG_CENTRE_QUADRANT_BD 27
#define IMG_CENTRE_EXT_BORD_HD 28 /* Sur un bord horizontal, mais  l'extrieur */
#define IMG_CENTRE_EXT_BORD_HG 29
#define IMG_CENTRE_EXT_BORD_BD 30
#define IMG_CENTRE_EXT_BORD_BG 31
#define IMG_CENTRE_EXT_BORD_DH 32 /* Sur un bord vertical, mais  l'extrieur */
#define IMG_CENTRE_EXT_BORD_DB 33
#define IMG_CENTRE_EXT_BORD_GH 34
#define IMG_CENTRE_EXT_BORD_GB 35

#define IMG_CentreEstDansImage(position) ( (position >= IMG_CENTRE_DANS_IMAGE) && (position <= IMG_CENTRE_BORD_D) )

short int IMG_Position_Centre(Image *image);

#define IMG_LIT_SIMPLE           0 /* Valeur dans la case sans traitement */
#define IMG_INTERPOLE_BILINEAIRE 1 /* Valeur interpole, bilinaire */
#define IMG_INTERPOLE_2T         2 /* Valeur interpole en coupant la case en 2 triangles puis utilisation du plan */
#define IMG_INTERPOLE_4T         3 /* Valeur interpole en coupant la case en 4 triangles puis utilisation du plan */
#define IMG_INTERPOLE_SPLINE     4 /* Utilisation d'une spline 2D pour interpoler la valeur */

double IMG_lit_valeur(Image *image, double x, double y, int mode, boolean *dehors);

Image *IMG_interpole(Image *image, unsigned int nv_larg, unsigned int nv_haut);
Image *IMG_plaquer  (Image *ref, Image *source);
Image *IMG_lissage  (Image *source);

int IMG_CalerSurImage(Image *image, Image *originale);

Image *IMG_equation(double x_min, double pas_x, unsigned long nbr_x,
		    double y_min, double pas_y, unsigned long nbr_y,
		    struct expr *formule);

/* Oprations mathmatiques sur une image */

int IMG_FacteurMultiplicatif(Image *image, double facteur, unsigned int axe);

int    cherche_centre(Image *image, unsigned int *x, unsigned int *y);
int    IMG_decouper  (Image *source);
double img_interpole (Image *image, double x, double y, char *dehors);

/* Transformations gomtriques gnriques */

Image *IMG_rotation_quelconque(Image *source, double angle, double pas_x, double pas_y);
Image *IMG_tourner(Image *source);

Image *IMG_Miroir_Horizontal(Image *source, double position);
Image *IMG_Miroir_Vertical  (Image *source, double position);

Image *IMG_ProjeterImage(Image *source, struct orientation_3d *orientation_image, struct orientation_3d *orientation_projection,
			 double pas_x, double pas_y);

Image *IMG_TransformationQuelconque(Image *source, double *matrice, double pas_x, double pas_y);

#define IMG_MIROIR_VERTICAL   1
#define IMG_MIROIR_HORIZONTAL 2

int IMG_PseudoMiroir(Image *image, int sens);

/* Autres fonctions */

typedef struct {
  Image  *tranche;
  unsigned int x, y, larg, haut;
  struct graphe *spectre;
  double x_min, x_max;
} Infos_tranche;

int  IMG_moyenne_rectangle(Image *source, Infos_tranche *infos, boolean erreurs);
int  IMG_moyenne_tranches (Image *source, Infos_tranche *infos);
void IMG_choisit_tranche  (Image *image);

Image *IMG_CovarianceVersImage(struct graphe *graphe, boolean correlation);
Image *IMG_MatriceVersImage   (struct matrice *matrice);
 
/* ------ Effets 3D de l'chantillon ------ */
Image *effets_epaisseur(Image *source);
Image *ECH_radial(Image *tranche, double epais_tranche, double epais_totale, 
		  double angle_faisceau, double inclinaison, double rotation);

Image *IMG_corrige_epaisseur(Image *source);
Image *IMG_epais_lignes(Image *source, double epais_tranche, double epais_totale, 
			double angle_faisceau, double inclinaison, double rotation);

/* ------ Effets du faisceau tendu ------- */
Image *convoluer_image(Image *source);


double dist_img_simple(Image *source, Image *dest);

/* -------- Fonctions associes aux profils -------- */

struct graphe *IMG_profil(Image *image); /* L'interface */

struct graphe *init_profil(Image *image, int angle, double pas); /* Image -> profil */
struct graphe *IMG_coupe(Image *image, double x1, double y1, double x2, double y2, double pas);

typedef struct profil {
  unsigned int angle; /* En degrs, entre 0 et 360 */
  float unite; /* Unit en X, en microns - idalement, 1 */
} Profil;

#define IMG_GENERER_RADIAL     0
#define IMG_GENERER_COUPE      1
#define IMG_GENERER_ELLIPTIQUE 2

Image *IMG_generer(struct graphe *profil);

#if defined(_TURBO_C_) || defined(_PURE_C_)
# include "c_formel.h"
# include "graphes.h"
#endif

#endif /* DECONV_H_ */
