/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1998-2002

   Dfinitions pour les modles d'ajustement
*/

#ifndef MODELES_AJUSTE_H_

#define MODELES_AJUSTE_H_ /* Protection contre l'inclusion multiple */

# include "c_formel.h"		/* Obligatoire pour les variables */

/* ------ Structures spcifiques d'un type de modle, pas forcment utilises ------ */

struct atome;        /* Dfinie dans "atomes.h"  */
struct couche;       /* Dfinie dans "couches.h" */
struct graphe;       /* Dfinie dans "graphes.h" */
struct matrice;      /* Dfinie dans "matrices.h"*/
struct modele_feff;  /* Dfinie dans "feff.h"    */
struct modele_gnxas; /* --  DFINIR -- */
struct serie;        /* Dfinie dans "serie.h" */

/* -~-~-~-~-~-~-~-~ Structure gnrale pour les modles  -~-~-~-~-~-~-~-~ */

/* ------ Les algorithmes disponibles ------ */
#define ALGO_PSEUDOGRADIENT 0 /* Algorithme rapide inspir d'EDA */
#define ALGO_LEVENBERG      1 /* Algorithme de type Levenberg-Maquardt */
#define ALGO_GRADIENT       2
#define ALGO_SIMPLEXE       3
#define ALGO_SIMP_PUIS_GR   4 /* Simplexe, puis gradient en fin */

#define MOD_NOMBRE_ALGORITHMES 1 /* Seul le pseudo-gradient est gr actuellement ! */

/* ------ Les types de modle ------ */
#define MODELE_MULTI_MODELES -1 /* Ajustement simultan de plusieurs modles */
#define MODELE_CL_MODELES     0 /* Combinaison linaire des modles indiqus */
#define MODELE_EXAFS_CHEMINS  1 /* Somme de contribution de chemins : type FEFF / exafs */
#define MODELE_EXAFS_GNXAS    2 /* Contributions groupes par nombre d'atomes impliqus, type GNXAS */
#define MODELE_SEUIL_BASIQUE  3 /* Modle d'un seuil  l'aide de gaussiennes & Arc tg */
#define MODELE_FLUOX          4 /* Modle de composition, rmission de fluorescence X */
#define MODELE_DECOMPOSITION  5 /* Modle de dcomposition linaire d'un spectre sur une base de spectres*/
#define MODELE_GENERIQUE      6 /* Modle gnrique, multi-paramtres, non-linaire */
#define MODELE_POLYNOMIAL     7	/* y=a + bX + cX + ... */

struct cl_modeles;	   /* Pour CL de modles, voir plus loin */
struct multi_modeles;	   /* Pour modles multiples, voir ci-aprs */
struct decomposition;	   /* Dfinie dans "decomposition.h" */
struct modele_generique;   /* Dfinitions pour un ajustement non-linaire  plusieurs variables */
struct modele_polynomial;  /* Paramtres pour un ajustement d'un polynome */

struct pg_variable;   /* Variables pour un ajustement par un pseudo-gradient */

typedef struct modele_ajuste {
  struct modele_ajuste *suivant; /* On peut avoir plusieurs modles pour une mme entit */

  int    type;           /* Type de modle considr */
  char  *nom_modele;     /* Nom du modle (chane  librer par free) */

  double x_min, x_max;   /* L'intervalle sur lequel est dfini le modle */

  struct graphe *experience; /* Le graphe exprimental  modliser */
  struct graphe *reference;  /* Id, mais juste la partie importante en X */
  struct graphe *modele;     /* Le rsultat du modle considr */
  struct graphe *residu;     /* Le rsidu qui va avec [exprience - modle] */

  union { /* Infos spcifiques  une classe de modles */
    struct multi_modeles *modeles; /* Pour les modles multiples */
    struct cl_modeles    *cl_modeles;
    struct modele_feff   *feff;    /* Dfini dans feff.h */
    struct modele_gnxas  *gnxas;
    struct decomposition *decomposition;  /* Dfini dans decomposition.h */
    struct modele_generique *generique;
    struct modele_polynomial *polynome;
  } infos_modele;

  double        erreur;        /* Entre le modle et l'exprience, telle qu'elle sera minimise */
  unsigned int  algorithme;    /* Le type d'algorithme utilis */
  unsigned int  iteration;     /* Le nombre d'itrations effectues */
  unsigned int  max_iteration; /*  ... et le maximum autoris */
  unsigned int  nbr_variables; /* Ajusts dans le modle */
  int condition_arret;         /* Quand arrte-t-on l'ajustement ? */
  short int err_min, derr_min; /* -log10 de l'erreur/variation de l'erreur minimale autorise */
  struct {
    unsigned pondere     :1; /* VRAI : pondration par les barres d'erreur */
    unsigned contraintes :1; /* VRAI : il y a des contraintes entre les variables */
    unsigned actif       :1; /* VRAI : modle "courant" (si plusieurs successifs) */
    unsigned en_cours    :1; /* VRAI : modle en cours d'ajustement */
    unsigned isole       :1; /* VRAI : modle isol pour ajustement simple */
    unsigned int preparation:1; /* VRAI : modle en cours de prparation */
    unsigned int arret_force:1; /* VRAI : on demande un arrt anticip */
  } drapeaux;
  union {
    struct pg_variable *pseudogradient; /* Ajustement par le pseudo-gradient */
  } infos_variables;

  struct matrice *covariance; /* Matrice de covariance  l'issue de l'ajustement */
} ModeleAjustement;

/* Informations complmentaires pour les modles CL d'autres modles */
struct cl_modeles {
  unsigned int nbr_termes;
  struct variable      *coefs;   /* Tableau dynamique */
  struct modele_ajuste **modeles; /* Tableau dynamique */
};

/* Informations complmentaires pour les modles multiples  indpendants  */
struct multi_modeles {
  unsigned int nbr_modeles;
  unsigned int modele_actif;
  struct modele_ajuste **modeles; /* Tableau dynamique */
};

/* Informations complmentaires pour un modle non-linaire simple, gnrique */
struct parametre {
  char *symbole;		/* Du paramtre dans la formule */
  char *unite;			/* Pour contrles et affichages seulement */
  char *nom;			/* Pour affichage uniquement */
  char *derivee;		/* La formule de calcul analytique de la drive */
  double erreur;		/* Erreur associe au paramtre [<=0 : aucune] */
  struct variable variable;	/* La variable associe : donne la valeur, min & max */
  struct expr *arbre_derivee;	/* L'arbre pour calculer la drive */
};

struct var_modele {
  char *unite;			/* Unit attendue pour cette variable [NULL : quelconque] */
  char *type;			/* Type de grandeur physique */
  char *legende;		/* Lgende  utiliser pour la variable */
  char *formule;		/* Formule dfinissant la variable */
  char *formule_erreur;		/* Formule dfinissant l'erreur sur la variable */
};

struct modele_generique {
  unsigned int type;		/* Type de modle == genres de graphes ! */
  char *nom;			/* Nom du modle [Pour affichage seulement !] */
  struct var_modele X, Y;	/* Les variables X & Y du modle */
  unsigned long nombre_parametres; /* Nombre de paramtres dans le modle */
  struct parametre *parametres;	/* Table des paramtres du modle */
  struct expr *arbre_formule;	/* Expression Y=f(X),  calculer... */
};

/* Informations pour un modle polynomial */
struct modele_polynomial {
  unsigned int degre;
  double *coefficients;
};

ModeleAjustement *MODELE_CreerModele  (ModeleAjustement *avant, const char *nom, int type, struct graphe *modele, boolean complement);
int               MODELE_EffacerModele(ModeleAjustement **premier, ModeleAjustement *modele, boolean efface_graphes);

int   MODELE_ListeDomaines (unsigned long *nombre_domaines, const char ***noms_domaines);
char *MODELE_DossierDomaine(const char *nom_domaine);
int   MODELE_ListeModeles  (const char *nom_dossier, unsigned long *nombre, char ***noms_modeles, char ***nom_fichiers);

struct modele_generique *MODELE_LireModele          (const char *nom_fichier);
struct modele_generique *MODELE_NouveauGenerique    (const char *nom_modele);

int MODELE_Generique_AfficherResultats (struct modele_generique *modele, boolean historique);
int MODELE_Generique_AjouterParametres (struct modele_generique *modele, unsigned long nbr_parametres);
int MODELE_Generique_CalculerModele    (ModeleAjustement *modele);
int MODELE_Generique_Controler         (struct modele_generique *modele);
int MODELE_Generique_IncrementsPG      (ModeleAjustement *modele);
int MODELE_Generique_Liberer           (struct modele_generique *modele);
int MODELE_Generique_PreparerAjustement(ModeleAjustement *modele);
int MODELE_Generique_PGInitVariables   (ModeleAjustement *modele, unsigned int prem_variable);
int MODELE_Generique_Verifier          (struct modele_generique *modele, unsigned int type_validation);


/* Codes d'action : que faire pour les anciens rsultats ? */
#define Q_MODELE_ANCIENS_RESULTATS_EFFACER   1
#define Q_MODELE_ANCIENS_RESULTATS_GARDER    2
#define Q_MODELE_ANCIENS_RESULTATS_ANNULER   3
#define Q_MODELE_ANCIENS_RESULTATS_DEMANDER 10

int MODELE_AjustementComplet (ModeleAjustement *modele, int (*calcule_modele)(struct modele_ajuste *),
			      boolean dessins, boolean afficher_resultats, int action_ancien_modele);

int MODELE_AfficherResultats (ModeleAjustement *modele);
int MODELE_AjusterModele     (ModeleAjustement *modele, int (*calcule_modele)(struct modele_ajuste *));
int MODELE_CalculerModele    (ModeleAjustement *modele);
int MODELE_ControlerModele   (ModeleAjustement *modele, boolean fin_ajustement);
int MODELE_Initialiser       (ModeleAjustement *modele);
int MODELE_InitVariables     (ModeleAjustement *modele);
int MODELE_MettreReference   (ModeleAjustement *modele, struct graphe *experience, boolean eff_anciens);
int MODELE_PreparerAjustement(ModeleAjustement *modele, int action_ancien_modele);
int MODELE_OterReference     (ModeleAjustement *modele, boolean effacer_modele);
int MODELE_TerminerAjustement(ModeleAjustement *modele, boolean afficher_resultats);

unsigned int MODELE_CompterVariables(ModeleAjustement *modele);

char  *MODELE_InfosVariable       (ModeleAjustement *modele, unsigned int variable);
char  *MODELE_UniteVariable       (ModeleAjustement *modele, unsigned int variable);
double MODELE_ValeurVariable      (ModeleAjustement *modele, unsigned int variable, unsigned int type);
int    MODELE_FixerValeurVariable (ModeleAjustement *modele, unsigned int variable, unsigned int type, double valeur);
int    MODELE_ReinitialiseVariable(ModeleAjustement *modele, unsigned int variable);

int  MODELE_Nommer                  (ModeleAjustement *modele, const char *nom);

ModeleAjustement *MODELE_ChercherModele(struct graphe *graphe);

/* ~~~~~~~~ Routines d'ajustement ~~~~~~~~ */

double MODELE_CalculeDerivee(struct modele_ajuste *modele, unsigned int variable, double x);

/* -------- Les conditions d'arrt -------- */

#define ARRET_AJUSTEMENT_ERREUR          1 /* Si l'erreur est suffisamment faible  */
#define ARRET_AJUSTEMENT_DELTA_ERREUR    2 /* Si l'erreur varie trop peu           */
#define ARRET_AJUSTEMENT_DELTA_VARIABLE  4 /* Si une variable varie trop peu       */
#define ARRET_AJUSTEMENT_ITERATIONS      8 /* Aprs un certain nombre d'itrations */
#define ARRET_AJUSTEMENT_LIMITES        16 /* Une variable atteint ses limites     */
#define ARRET_AJUSTEMENT_FORCE          32 /* Erreur ou interrompu                 */

#define ARRET_AJUSTEMENT_TESTS          (1 << 6) /* Aprs l'ajustement, faire un test de validit */
#define ARRET_AJUSTEMENT_COVARIANCE     (1 << 7) /* Aprs l'ajustement, estimer la matrice de covariance */

/* -------- Pseudo-gradient ("rapide") -------- */

typedef struct pg_variable
{
  struct variable *var;
  double ex_valeur; /* Valeur avant le tour d'ajustement, mmorise si algo. diverge */
  double increment;
  double num, den ; /* Numrateur et dnominateur           */
  int type;         /* Le type de variable dont il s'agit   */
  struct modele_ajuste *modele; /* Le modle dont elle dpend */
  union {
    struct couche *couche; /* Pour un modle FEFF ! */
    struct parametre *parametre; /* Pour un modle gnrique */
  } origine;
} PG_variable;

char *AJ_PseudoGradient_Infos(unsigned int j, char **nom, PG_variable *variable); /* ex infos_rapide*/
int   MODELE_AjustementPG    (struct modele_ajuste *modele, Fenetre *fenetre,
			      double (*calcule_point)(struct modele_ajuste *modele, double x),
			      int (*calcule_modele)(struct modele_ajuste *modele),
			      int (*calcule_increments)(struct modele_ajuste *modele),
			      void (*affichage)(struct modele_ajuste *modele));

/* ~~~~~~~~ Routines gnrales d'analyse des rsultats ~~~~~~~~ */

#define MODELE_ERREUR_NORME2 -2 /* On calcule en fait la somme des |y| */
#define MODELE_ERREUR_NORME1 -1 /* On calcule en fait la somme des |y|  */

#define MODELE_ERREUR_ABSOLUE 1 /* Erreur absolue par les valeurs absolues */
#define MODELE_ERREUR_CARRES  2 /* Erreur absolue par les moindres carrs  */

double MODELE_CalculerResidu(struct modele_ajuste *modele);
double MODELE_CalculerErreur(struct modele_ajuste *modele, int type_erreur, boolean stat, boolean relative);

struct matrice *MODELE_CreerMatriceCovariance(struct modele_ajuste *modele);
struct matrice *MODELE_MatriceCovariance     (struct modele_ajuste *modele);

void          MODELE_InterfaceMonteCarlo(struct modele_ajuste *modele);
struct serie *MODELE_MonteCarlo         (ModeleAjustement *modele, struct serie *tirages,
					 unsigned long nbr_tirages, boolean garder_tirages, boolean faire_matrice);

#define MODELE_VAL_BORNES     0 /* Les bornes autorises sont-elles atteintes ? */
#define MODELE_VAL_PHYSIQUE   1 /* Les valeurs obtenues ont-elles un sens physique ? */
#define MODELE_VAL_STATS      2 /* Contrle statistique du rsidu : test du khi-deux */
#define MODELE_VAL_MONTECARLO 3 /* Lancer une simulation Monte-Carlo */
#define MODELE_VAL_MC_RESIDUS 4 /* id., mais sur le modle thorique, pour tudier les rsidus */
#define MODELE_VAL_INITIALES  5	/* Etudier l'influence du choix des valeurs initiales sur les valeurs obtenues */
#define MOD_NOMBRE_METHODES_VALIDATION 6

int MODELE_ControlerBornes  (struct modele_ajuste *modele);
int MODELE_ControlerPhysique(struct modele_ajuste *modele);

double MODELE_TestKhiDeux       (struct modele_ajuste *modele);
double MODELE_TestFisherSnedecor(struct modele_ajuste *modele);

int           MODELE_VALIDITE_PreparerModele     (struct modele_ajuste *modele);
struct serie *MODELE_VALIDITE_ValeursInitiales_MC(struct modele_ajuste *modele, unsigned long nombre_tirages);

/* ~~~~~~~~ Les codes d'erreur ~~~~~~~~ */

#define ERREUR_MODELE_AJUSTEMENT_EN_COURS -1 /* Ajustement de ce modle dj en cours */

/* -~-~-~-~-~-~-~-~ Modles trs simples qui peuvent tre ajusts sans la structure complte ci-dessus -~-~-~-~-~-~-~-~ */

int MODELE_AjusterDroite(struct graphe *donnees, struct graphe **modele, struct graphe **residus, double *pente, double *o_a_lo);

#endif /* MODELES_AJUSTE_H_ */
