/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2001, Emmanuel CURIS
   Cration du fichier : 11 fvrier 2001

   Oprations diverses sur les graphes, plus gnrales que l'EXAFS lui-mme
   (aussi bien mathmatiques que statistiques simples : tri en particulier)
   (les oprations statistiques avances sont dans ../Stats)

   Dfinitions gnrales pour ces oprations [repris de graphes.h en gnral]
*/

#ifndef OPERATIONS_H_
#define OPERATIONS_H_ /* vite l'inclusion multiple */

#include "graphes.h"
struct expr;			/* Dans c_formel.h */

/* ~~~~~~~~ Transformations simples d'un graphe ~~~~~ */

/* -------- Transformations affines -------- */

boolean GRF_xaffine (Graphe *graphe, double a, double b, double c);
boolean GRF_yaffine (Graphe *graphe, double a, double b, double c);
boolean GRF_xyaffine(Graphe *graphe, double a, double b, double c);

/* -------- Transformations quelconques ---- */

#define TRANSFORME_LOG_BASE_E 0.0	/* Code spcial pour le log. de base e (nprien) : vite de donner la valeur exacte de e ! */

int GRF_SemiLog_Y(Graphe *graphe, double base);

boolean GRF_Xfonction(Graphe *graphe, const char *texte);
boolean GRF_Yfonction(Graphe *graphe, const char *texte);

Graphe *GRF_Transformer_R(Graphe *graphe, struct expr *tr_X, struct expr *tr_sX, struct expr *tr_Y, struct expr *tr_sY);

/* ~~~~~~~~ Traitement des graphes complexes ~~~~~~~~ */

/* ------------ Dcomposition d'un graphe en ses lments rels ------ */

void CMP_decompose (Graphe *complexe);

Graphe *CMP_reelle    (Graphe *complexe);
Graphe *CMP_imaginaire(Graphe *complexe);
Graphe *CMP_module    (Graphe *complexe);
Graphe *CMP_argument  (Graphe *complexe);

/* ~~~~~~~~ Estimation des drives ~~~~~~~~ */

#define DER_RAPIDE     1 /* Drive rapide : delta_y/delta_x */
#define DER_SPLINE     2 /* Drive par calcul de la spline */

Graphe *GRF_Deriver      (const Graphe *graphe, int methode);
Graphe *GRF_DeriveeSimple(const Graphe *graphe);
Graphe *GRF_DeriveeSpline(const Graphe *graphe, unsigned long nombre_points, const double *a, const double *b, const double *c);
int     GRF_Derivee_PostCalcul(const Graphe *graphe, Graphe *derivee, int methode);

Graphe *DER_seconde(const Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d);

/* ~~~~~~~~ Estimation de l'aire sous la courbe ~~~~~~ */

#define INTG_RECTANGLE 0
#define INTG_MEDIAN    1
#define INTG_TRAPEZE   2
#define INTG_SIMPSON   3
#define INTG_BODE      4
#define INTG_SPLINE    6

double  GRF_Integrer_R(Graphe *graphe, double x_min, double x_max, int methode, double *erreur);
int     GRF_Integrer_C(Graphe *graphe, double x_min, double x_max, int methode, Complexe *valeur, Complexe *erreur);
Graphe *GRF_Primitive (const Graphe *graphe, int methode, union valeur constante, double erreur_constante);

/* ~~~~~~~~ Tronquer un graphe (en X, en Y ou les deux) ~~~~~~~~ */

#define TRQ_GARDER_INTERIEUR 0
#define TRQ_GARDER_EXTERIEUR 1

typedef struct
{
  double x_min, x_max; /* Les limites de troncature */
  double y_min, y_max;
  int axes;            /* Sur quel axe porte la troncature ? (X, Y ou les deux) */
  int mode_x, mode_y;  /* Ce que l'on conserve en X et en Y */
} TRQ_infos;

void GRF_tronque(Graphe *graphe);

int  TRQ_Tronquer(Graphe *graphe, TRQ_infos *infos, boolean en_place, boolean historique);

/* ~~~~~~~~ liminer un point aberrant par interpolation ~~~~~~~~ */

typedef struct ab_param
{
  struct ab_param *suivant; /* La suivante, si travail avec plusieurs fentres simultanment */
  Graphe *graphe;    /* Le graphe sur lequel on travaille */
  double x_borne[4]; /* Les bornes pour les intervalles de lissage : x1min, x1max, x2min, x2max */
  char degre;        /* Degr du polynme de lissage */
  double coefs[11];  /* Coefficients de ce polynme  */
} ABER_param;

void aberrance(Graphe *graphe);

Graphe *GRF_recale_y   (Graphe *graphe, double valeur, boolean nouveau, double err_valeur);
Graphe *GRF_mrecale_y  (Graphe *graphe, double valeur, boolean nouveau, double err_valeur);

/* ------------ Oprations mathmatiques sur deux graphes --------------------------------- */

/* version sans contrle des X */

boolean GRF_clin(const Graphe *gr_1, double coef_1, const Graphe *gr_2, double coef_2, Graphe *resultat);

/* version avec calage sur la rfrence */

boolean GRF_Rclin(Graphe *reference, double coef_1, Graphe *gr_2, double coef_2);

/* ------------ Oprations mathmatiques sur plus de 2 graphes ---------------------------- */

Graphe *op_diverses(Graphe *reference);

#endif /* ifndef OPERATIONS_H_ */
