/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1998 - 2001
   Cration du fichier : 8 dcembre 2001

   Dfinitions pour le chargement de structures PDB, version volue
*/

#ifndef PDB_H_

#define PDB_H_ /* Protection contre l'inclusion multiple */

struct fragment;
struct molecule; /* Dans atome.h */

/* -------- Une chane protique -------- */

typedef struct chaine {
  struct chaine *suivante;
  int code;
  unsigned long nbr_residus;
  int delta_num; /* Dcalage de numrotation entre le 1er rsidu et son numro PDB */
  struct molecule *chaine;
  struct fragment *prem_fragment; /* Car pas forcment le premier de la molcule, si chanes groupes */
} PDB_Chaine;

PDB_Chaine *trouver_chaine(PDB_Chaine **premiere, char code_chaine);

/* -------- Une molcule d'un autre type (htro) -------- */

typedef struct hetero {
  struct hetero *suivant;
  char code[4];
  char *nom;
  unsigned long nbr_atomes; /* On pourrait mme faire la formule, mais bon... */
} PDB_hetero;

PDB_hetero *trouver_hetero(PDB_hetero **premier, char *code_hetero);

struct fragment *PDB_trouver_fragment(PDB_Chaine *chaine, struct fragment *avant, int numero, char *nom);

/* -------- Informations gnrales pour le chargement -------- */

#define PDB_OPTIONS_CHARGER_H     1 /* Bit mis s'il faut charger les H */
#define PDB_OPTIONS_CHARGER_EAU   2 /* Bit mis s'il faut charger les molcules d'eau isoles */
#define PDB_OPTIONS_UNE_MOLECULE  4 /* Bit mis : on ne veut qu'une molcule avec toutes les chanes */
#define PDB_OPTIONS_UNE_CHAINE    8 /* Bit mis : on ne veut charger qu'une seule chane */
#define PDB_OPTIONS_TOUS_MODELES 16 /* Bit mis : on veut charger tous les modles du fichier */

typedef struct infos_PDB {
  char *base_nom;             /* Nom de la protine globale  */
  unsigned int drapeaux;      /* Options utilises pour le chargement */
  PDB_Chaine *premiere_chaine;
  PDB_hetero *premier_groupe;
  int ref_type, type;         /* Chane  charger, chane en cours */
  int ref_modele, num_modele; /* Idem, pour les modles */
  int num_aa;                 /* Acide amin en cours de chargement */
  int code_retour;            /* Si lecture a chou, pourquoi ? */
  char code[5];               /* Code lettres de la protine */ 
} PDB_infos;

/* Fonctions pour lire les diffrentes parties d'un fichier 
   Renvoient la 1re ligne qui n'est pas de la bonne section
   NULL en cas de problme quelconque...
*/

char *PDB_lit_Titre        (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Primaire     (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Heterogene   (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Secondaire   (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Liaisons     (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Divers       (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Cristallo    (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Coordonnees  (FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_TableLiaisons(FILE *fichier, char *ligne, PDB_infos *infos_pdb);
char *PDB_lit_Fin          (FILE *fichier, char *ligne, PDB_infos *infos_pdb);

#endif /* PDB_H_ */
