/* LASE - Logiciel d'Analyse des Spectres E.X.A.F.S.
   (c) 1996 - 2004 - Emmanuel CURIS

   Dfinitions pour les fonctions de tri (toutes donnes confondues)
   Contient aussi les routines d'limination de doublons & de fusion
*/

#ifndef TRI_H_
# define TRI_H_  /* Protection contre l'inclusion multiple */

struct graphe; /* Dans graphes.h */

/* -------- Tri d'un simple tableau de valeurs -------- */

double *TRI_Tableau(double *x, unsigned long nbr_elements);

/* -------- Tri & fusion par blocs, avec moyenne ventuelle -------- */

typedef struct {
  double *x;			/* Pointeur de dpart - X */
  double *y;			/* Pointeur de dpart - Y */
  unsigned long position;	/* Point que l'on est en train de traiter */
  unsigned long nbr_points;	/* Nombre de points dans le bloc */
  int axe_erreurs;		/* AXE_X | AXE_Y si erreurs sur les deux axes */
  double *erreurs_x;		/* Erreurs sur les X */
  double *erreurs_y;		/* Erreurs sur les Y */
} TRI_infos;

int TRI_Blocs(TRI_infos *trie, TRI_infos *blocs, unsigned long nbr_blocs);
unsigned long MOY_blocs(double *x_tri, double *y_tri, TRI_infos *blocs, unsigned long nbr_blocs, unsigned long total);

/* -------- Tri des graphes proprement dit -------- */

#define GRF_TRI_RAPIDE    0 /* Rapide, mais utilise de la mmoire en plus */
#define GRF_TRI_UNIVERSEL 1 /* Plus lent, mais tri simultan des incertitudes */

int GRF_Trier(struct graphe *graphe, int methode);

/* -------- Traitement des doublons -------- */

#define GRF_DOUBLONS_SIMPLE 0	/* Doublons exacts seulement */
#define GRF_DOUBLONS_GROUPE 1	/* Groupes de k points successifs, quels que soient les X */
#define GRF_DOUBLONS_SOUPLE 2	/* Quasi-doublons, ie Dx infrieur  un seuil */

unsigned long GRF_TraiterDoublons(struct graphe *graphe, int methode, double parametre, boolean moyenner, boolean garder_erreurs);

unsigned long DBL_simple(struct graphe *graphe, boolean moyenner);
unsigned long DBL_diff  (struct graphe *graphe, boolean moyenner, double difference);

/* -------- Fusion de graphes -------- */

struct graphe *GRF_Fondre(unsigned long nombre, struct graphe **graphes, boolean isole);

struct graphe *fondre(void);	/* Fusion  - renvoi le graphe obtenu */

boolean GER_fusion(int num_objet, int x_souris, int y_souris);

/* void tri_en_place(struct graphe *graphe); */

#endif /* TRI_H_ */
