/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2006, Emmanuel CURIS
   Cration : mars 2006

   Interface graphique avec l'utilisateur

   Ajuster un modle (quelconque) sur un spectre exprimental
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "graphes.h"
#include "interface.h"
#include "modeles.h"

#include "int_ajuster.txt"

/* -------- Variables locales -------- */

static ModeleAjustement *premier_modele = NULL;

struct infos_generique {
  unsigned long parametre_affiche;
  struct modele_generique *modele;
  unsigned long nombre_domaines;
  unsigned long domaine_choisi;
  const char **domaines;
  unsigned long nombre_modeles;	/* Le nombre de modles dans le domaine choisi */
  char **modeles;		/* Les noms des modles */
  char **fichiers_modeles; /* Les noms des fichiers associs  ces modles */
};

/* -------- Fonctions locales -------- */

static int               I_MOD_Modifier  (unsigned long numero_modele);
static ModeleAjustement *I_MOD_N_V_Modele(unsigned long numero_modele);
static const char       *I_MOD_N_V_Texte (unsigned int arbre, unsigned long numero_modele);
static unsigned long     I_MOD_NombreModeles(void);

static void I_MG_AfficherModele   (struct infos_generique *infos);
static void I_MG_AfficherParametre(struct infos_generique *infos);
static void I_MG_CouperParametre  (boolean couper);
static void I_MG_CouperConstante  (boolean couper);
static void I_MG_EnregistrerParametre(struct infos_generique *infos);

static int  I_MG_GererArbre   (unsigned int objet, int x, int y);
static void I_MG_PreparerArbre(void);

static ModeleAjustement *I_MG_Choisir(void);

static void I_MG_LibererModeles(struct infos_generique *infos);

/* ~-~-~-~-~-~-~-~-~ Ajustement de modles simples... ~-~-~-~-~-~-~-~-~ */

Graphe *I_GRF_AjusterDroite(Graphe *experience)
{
  char *titre;
  double pente = 0.0, o_a_lo = 0.0;
  int retour;
  Graphe *modele = NULL;
  struct serie *serie;

  if ( NULL == experience ) return NULL;

  /* On trouve la meilleure droite au sens des moindres carrs */
  retour = MODELE_AjusterDroite( experience, &modele, NULL, &pente, &o_a_lo );
  if ( retour != 0 ) {
    OPTION_A_FAIRE( "Afficher un message" );
    if ( retour < 0 ) {
      GRF_eff_isole( modele );
      return NULL;
    }
  }

  /* On affiche l'quation de la droite */
  retour = BI_ERR_alerte( principale, &MSG_ValeursAjusteesDroite, FAUX, o_a_lo, pente );

  /* On insre le rsultat juste aprs l'exprience */
  serie = I_GRF_InsererDansSerie( NULL, experience, modele );
  if ( NULL == serie ) {
    (void) fprintf(stderr, "%s, %s l.%u : Insertion dans une srie impossible : on efface le rsultat\n",
		   __FILE__, __FUNCTION__, __LINE__);
    GRF_eff_isole( modele );
    return NULL;
  }

  /* On met les lgendes, styles,... */
  GRF_CopierAffichage( modele, experience, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_LEGENDE_Y | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_UNITE_Y );
  GRF_CouleurSuivante( modele, experience );
  GRF_couleur( modele, -1, BI_CONTINU, -1 );  /* On force l'existence d'un trait continu : c'est une droite, donc normal ! */

  /* On met le titre */
  titre = TXT_genere_chaine( titre_droite_mc, experience->nom );
  GRF_titre( modele, titre );
  free( titre ); titre = NULL;

  /* Termin : on renvoie le graphe ainsi cr */
  return modele;
}

/* ~-~-~-~-~-~-~-~-~ Slectionner un modle et ajuster ce modle... ~-~-~-~-~-~-~-~-~ */

Graphe *I_GRF_ModeliserGenerique(Graphe *experience)
{
  int retour;
  ModeleAjustement *modele = NULL;

  if ( NULL == experience ) return NULL;

  if ( NULL == premier_modele ) {
    const char *nom_modele;

    /* On demande le nom du modle  crer */
    nom_modele = BI_Demande_Texte( principale, &Q_NomModele, txt_nouveau_modele );
    if ( NULL == nom_modele ) {
      /* On annule, d'une faon ou d'une autre */
      return NULL;
    }

    /* On cre ce modle gnrique */
    premier_modele = MODELE_CreerModele( NULL, nom_modele, MODELE_GENERIQUE, graphe_courant, VRAI );
    modele = premier_modele;

    /* On dite ce modle */
    I_ModeleGenerique( &(modele->infos_modele.generique) );
  } else {
    modele = I_MG_Choisir();
  }

  if ( NULL == modele ) {
    (void) fprintf( stderr, "%s,%s l.%u : Aucun modle slectionn !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  if ( ( modele->experience != NULL ) && ( modele->experience != graphe_courant ) ) {
    retour = BI_ERR_alerte( principale, &Q_ChangerReference, VRAI );
    if ( I_AJ_CHANGERREFERENCE_ANNULER == retour ) {
      /* On annule */
      return NULL;
    }
    MODELE_MettreReference( modele, graphe_courant, QUESTION );
  }

  /* On ralise l'ajustement */
  retour = MODELE_AjustementComplet( modele, NULL, VRAI, VRAI, Q_MODELE_ANCIENS_RESULTATS_DEMANDER );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : Erreur dans MODELE_AjustementComplet, code %d\n",
		    __FILE__, __FUNCTION__, __LINE__, retour );
    return NULL;
  }

  /* On renvoie le modle thorique obtenu */
  return modele->modele;
}

static ModeleAjustement *I_MG_Choisir(void)
{
  int retour;
  InfosSelecteur infos;

  /* Construire la liste des modles */
  infos.nbr_objets = I_MOD_NombreModeles();
  infos.premier    = 0;
  infos.obj_choisi = 0;

  infos.nouveau  = I_ModeleGenerique_Nouveau;
  infos.copier   = NULL;
  infos.modifier = I_MOD_Modifier;
  infos.sauver   = ( int (*)(unsigned long) ) NULL;
  infos.charger  = ( int (*)(void) ) NULL;
  infos.effacer  = NULL;
  infos.compter  = I_MOD_NombreModeles;
  infos.objet_v_texte = I_MOD_N_V_Texte;

  retour = I_SelecteurUniversel( titre_selecteur, &infos, fenetre_onglets );
  if ( retour != 0 ) {
    /* Annulation ou erreur dans le slecteur */
    return NULL;
  }

  return I_MOD_N_V_Modele( infos.obj_choisi );
}

/* -------- Routines ncessaires pour la gestion de l'arbre -------- */

struct modele_ajuste *I_ModeleGenerique_Premier(void)
{
  return premier_modele;
}

static int I_MOD_Modifier(unsigned long numero_modele)
{
  ModeleAjustement *modele;

  /* On repre le modle actif */
  modele = I_MOD_N_V_Modele( numero_modele );

  /* On le paramtre */
  I_ModeleGenerique( &(modele->infos_modele.generique) );

  /* Termin sans erreur */
  return 0;
}

static unsigned long I_MOD_NombreModeles(void)
{
  unsigned long i;
  ModeleAjustement *modele;

  for ( modele = premier_modele, i = 0 ; modele != NULL ; i++, modele = modele->suivant );
  return i;
}

int I_ModeleGenerique_Nouveau(unsigned long numero_avant)
{
  ModeleAjustement *modele_avant, *nouveau_modele;
  const char *nom_modele;

  /* On demande le nom du modle  crer */
  nom_modele = BI_Demande_Texte( principale, &Q_NomModele, txt_nouveau_modele );
  if ( NULL == nom_modele ) {
    /* On annule, d'une faon ou d'une autre */
    return -1;
  }

  /* On repre celui  mettre avant */
  modele_avant = I_MOD_N_V_Modele( numero_avant );

  /* On cre le modle */
  nouveau_modele = MODELE_CreerModele( NULL, nom_modele, MODELE_GENERIQUE, graphe_courant, VRAI );
  if ( NULL == nouveau_modele ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer un nouveau modle !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* On fait le chanage */
  if ( NULL == modele_avant ) {	/* C'est que cela devient le premier ! */
    nouveau_modele->suivant = premier_modele;
    premier_modele = nouveau_modele;
  } else {
    nouveau_modele->suivant = modele_avant->suivant;
    modele_avant->suivant = nouveau_modele;
  }

  /* On le paramtre */
  I_ModeleGenerique( &(nouveau_modele->infos_modele.generique) );

  /* Termin sans erreur */
  return 0;
}

static ModeleAjustement *I_MOD_N_V_Modele(unsigned long numero_modele)
{
  unsigned long i;
  ModeleAjustement *modele;

  for ( modele = premier_modele, i = 0 ; modele != NULL ;
	i++, modele = modele->suivant ) {
    if ( i == numero_modele ) return modele;
  }

  (void) fprintf(stderr, "%s,%s l.%u : Numro de modle incorrect [%lu]\n",
		 __FILE__, __FUNCTION__, __LINE__, numero_modele);
  return NULL;
}

static const char *I_MOD_N_V_Texte(unsigned int arbre __attribute__((__unused__)), unsigned long numero_modele)
{
  ModeleAjustement *modele;

  modele = I_MOD_N_V_Modele( numero_modele );
  if ( NULL == modele ) return NULL;

  return modele->nom_modele;
}

/* ~-~-~-~-~-~-~-~-~ Paramtrer un modle gnrique ~-~-~-~-~-~-~-~-~ */

#define MG_LGMAX_NOM_DOMAINE 43
#define MG_LGMAX_NOM_MODELE  43

/* -------- Ouverture de l'arbre --------
   On passe par un pointeur de pointeur, car le modle peut tre libr et recr, quand on passe par la bibliothque
 */

void I_ModeleGenerique(struct modele_generique **modele)
{
  unsigned int i;
  int reponse;
  struct infos_generique infos = { 0, NULL, 0, 0, NULL, 0, NULL, NULL };

  if ( NULL == modele ) {
    (void) fprintf(stderr, "%s,%s l.%u : modele = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  if ( NULL == *modele ) return;

  /* On prpare la bote de dialogue */
  I_MG_PreparerArbre();

  /* On cherche les modles prdfinis [Remplace car un ventuel prcdent aura t libr !] */
  OBJ_RemplacerTexte( MOD_GENERIQUE, MG_DOMAINE, txt_choisir_domaine, VRAI );
  OBJ_active( MOD_GENERIQUE, MG_MODELE, FORCE_NON );
  if ( MODELE_ListeDomaines( &(infos.nombre_domaines), &(infos.domaines) ) != 0 ) {
    infos.nombre_domaines = 0;
    infos.domaine_choisi = 0;
    infos.domaines = NULL;
    OBJ_active( MOD_GENERIQUE, MG_DOMAINE, FORCE_NON );
  } else {
    OBJ_active( MOD_GENERIQUE, MG_DOMAINE, FORCE_OUI );
  }

  /* On initialise les valeurs */
  infos.modele = *modele;

  /* On les place dans la bote de dialogue */
  I_MG_AfficherModele( &infos );

  /* On appelle la bote de dialogue */
  ARB_MettreComplement( MOD_GENERIQUE, &infos );
  reponse = ARB_Bloquant( MOD_GENERIQUE, titre_params_generique, fenetre_onglets, 0 );
  ARB_MettreComplement( MOD_GENERIQUE, NULL );

  /* On indique quel est le modle rellement utilis */
  *modele = infos.modele;

  /* On libre les noms de domaines */
  /* On met au pralable un texte sr dans le bouton ! */
  OBJ_RemplacerTexte( MOD_GENERIQUE, MG_DOMAINE, txt_choisir_domaine, VRAI );

  for ( i = 0 ; i < infos.nombre_domaines ; i++ ) {
    free( (char *) infos.domaines[ i ] ); infos.domaines[ i ] = NULL;
  }
  free( infos.domaines ); infos.domaines = NULL;

  /* On libre les infos de modles */
  I_MG_LibererModeles( &infos );

  if ( MG_ANNULER == reponse ) return;

  /* On mmorise l'quation indique */
  free( (*modele)->Y.formule ); (*modele)->Y.formule = NULL;
  (*modele)->Y.formule = strdup( lit_chaine( MOD_GENERIQUE, MG_FORMULE_Y, 0, NULL, TXT_REMPLACE ) );

  /* On mmorise les valeurs du paramtre affich */
  I_MG_EnregistrerParametre( &infos );
}

/* -------- Gestion de l'arbre ------------ */

static int I_MG_GererArbre(unsigned int objet, int x, int y)
{
  int etat;
  struct infos_generique *infos;

  /* On rcupre les infos associes  la bote */
  infos = (struct infos_generique *) ARB_LireComplement( MOD_GENERIQUE );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune information associe  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_TRAITE;
  }

  /* En fonction de l'objet cliqu */
  switch ( objet )
  {
  case MG_PARAM_AVANT:
  case MG_FOND_ASC_PARA:
  case MG_BOUTON_ASC:
  case MG_PARAM_APRES:
    /* On mmorise les valeurs du paramtre affich */
    I_MG_EnregistrerParametre( infos );

    /* On repre quel est le nouveau paramtre */
    infos->parametre_affiche = ARB_ascenseur( MOD_GENERIQUE, MG_PARAM_AVANT, MG_FOND_ASC_PARA, MG_BOUTON_ASC, MG_PARAM_APRES, ASC_VERTICAL, 
					      infos->modele->nombre_parametres, 1, infos->parametre_affiche, objet, x, y, ECHANGE );
    /* On affiche le nouveau paramtre */
    I_MG_AfficherParametre( infos );
    return BI_EVENEMENT_TRAITE;

  case MG_PARAM_CONST:
    etat = OBJ_choisit( MOD_GENERIQUE, objet, ECHANGE );
    I_MG_CouperConstante( etat );
    return BI_EVENEMENT_TRAITE;

  case MG_DOMAINE:
    OBJ_choisit( CREER_GRAPHE, objet, FORCE_OUI );
    {
      int retour;

      retour = SS_menu_texte( infos->domaines, infos->nombre_domaines, titre_menu_domaines, MOD_GENERIQUE, MG_DOMAINE );
      if ( retour >= 0 ) {
	char *nom_dossier = NULL;

	infos->domaine_choisi = retour;
	OBJ_RemplacerTexte( MOD_GENERIQUE, objet, infos->domaines[ infos->domaine_choisi ], VRAI );

	/* On efface les anciens modles */
	I_MG_LibererModeles( infos );

	/* On rcupre le nom du dossier de cette catgorie */
	nom_dossier = MODELE_DossierDomaine( infos->domaines[ infos->domaine_choisi ] );
	if ( NULL == nom_dossier ) {
	  OBJ_choisit( MOD_GENERIQUE, MG_MODELE, FORCE_NON );
	  return BI_EVENEMENT_TRAITE;
	}

	/* On lit les modles disponibles dans cette catgorie */
	retour = MODELE_ListeModeles( nom_dossier, &(infos->nombre_modeles), &(infos->modeles), &(infos->fichiers_modeles) );
	if ( ( retour != 0 ) || ( 0 == infos->nombre_modeles ) ) {
	  /* Aucun modle */
	  OBJ_active( MOD_GENERIQUE, MG_MODELE, FORCE_NON );
	} else {
	  OBJ_active( MOD_GENERIQUE, MG_MODELE, FORCE_OUI );
	  OBJ_RemplacerTexte( MOD_GENERIQUE, MG_MODELE, infos->modeles[ 0 ], VRAI );
	}

	/* Termin */
	free( nom_dossier ); nom_dossier = NULL;
      }
    }
    OBJ_choisit( MOD_GENERIQUE, objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case MG_MODELE:
    OBJ_choisit( MOD_GENERIQUE, objet, FORCE_OUI );
    {
      int retour;

      retour = SS_menu_texte( (const char **) infos->modeles, infos->nombre_modeles, titre_menu_modeles, MOD_GENERIQUE, MG_MODELE );
      if ( retour >= 0 ) {
	struct modele_generique *modele = NULL;

	/* On charge le modle correspondant ! */
	modele = MODELE_LireModele( infos->fichiers_modeles[ retour ] );
	if ( modele != NULL ) {
	  /* On installe les paramtres de ce modle  la place des anciens */
	  MODELE_Generique_Liberer( infos->modele );
	  infos->modele = NULL;

	  infos->modele = modele;

	  /* On installe les paramtres de ce modle dans l'arbre */
	  I_MG_AfficherModele( infos );
	} else {
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant lire le modle dans %s\n",
			 __FILE__, __FUNCTION__, __LINE__, infos->fichiers_modeles[ retour ]);
	}
      }
    }
    OBJ_choisit( MOD_GENERIQUE, objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case MG_NV_PARAMETRE:
    /* On mmorise les valeurs du paramtre affich */
    I_MG_EnregistrerParametre( infos );

    /* On cre un nouveau paramtre */
    MODELE_Generique_AjouterParametres( infos->modele, 1 );

    /* Ce paramtre devient le nouvel affich */
    if ( 1 == infos->modele->nombre_parametres ) {
      /* On vient de crer le premier paramtre */
      I_MG_CouperParametre( FAUX );
      infos->parametre_affiche = 0;
    } else {
      infos->parametre_affiche++;
    }

    /* On refait l'ascenseur */
    ARB_ascenseur( MOD_GENERIQUE, MG_PARAM_AVANT, MG_FOND_ASC_PARA, MG_BOUTON_ASC, MG_PARAM_APRES, ASC_VERTICAL, 
		   infos->modele->nombre_parametres, 1, infos->parametre_affiche, ROOT, 0, 0, METTRE );

    /* Et on affiche ce nouveau paramtre */
    I_MG_AfficherParametre( infos );
    return BI_EVENEMENT_TRAITE;

  case MG_EFFACER_PARAM:
    OPTION_A_FAIRE( "Effacer le paramtre du modle" );

    /* Quel paramtre devient le nouvel affich ? */
    if ( 0 == infos->modele->nombre_parametres ) {
      /* On vient d'effacer le dernier paramtre */
      I_MG_CouperParametre( VRAI );
      infos->parametre_affiche = 0;
    } else {
      infos->parametre_affiche++;
    }

    /* On refait l'ascenseur */
    ARB_ascenseur( MOD_GENERIQUE, MG_PARAM_AVANT, MG_FOND_ASC_PARA, MG_BOUTON_ASC, MG_PARAM_APRES, ASC_VERTICAL, 
		   infos->modele->nombre_parametres, 1, infos->parametre_affiche, ROOT, 0, 0, METTRE );
    return BI_EVENEMENT_TRAITE;
  }

  /* Autres objets : traitement par dfaut */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Affichages dans l'arbre -------- */

static void I_MG_AfficherModele(struct infos_generique *infos)
{
  /* On affiche l'quation du modle [modle Y=f(X), donc dans l'quation associe  X !] */
  OBJ_CopierTexte( MOD_GENERIQUE, MG_FORMULE_Y, infos->modele->Y.formule, 0 );

  /* Y a-t-il des paramtres ? */
  if ( 0 == infos->modele->nombre_parametres ) {
    /* Aucun paramtre : tout couper ! */
    I_MG_CouperParametre( VRAI );
    return;
  }

  I_MG_CouperParametre( FAUX );

  /* On met  jour l'ascenseur des paramtres */
  ARB_ascenseur( MOD_GENERIQUE, MG_PARAM_AVANT, MG_FOND_ASC_PARA, MG_BOUTON_ASC, MG_PARAM_APRES, ASC_VERTICAL, 
		 infos->modele->nombre_parametres, 1, infos->parametre_affiche, ROOT, 0, 0, METTRE );

  /* On affiche le paramtre en cours d'dition */
  I_MG_AfficherParametre( infos );
}

static void I_MG_AfficherParametre(struct infos_generique *infos)
{
  struct modele_generique *modele;
  struct parametre *parametre;

  modele = infos->modele;
  if ( NULL == modele ) return;
  if ( 0 == modele->nombre_parametres ) {
    /* Aucun paramtre : tout couper ! */
    return;
  }

  /* Afficher les valeurs */
  if ( infos->parametre_affiche >= modele->nombre_parametres ) {
    (void) fprintf(stderr, "%s,%s l.%u : Numro de paramtre affich absurde [%lu, max %lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos->parametre_affiche, modele->nombre_parametres - 1 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    infos->parametre_affiche = 0;
  }
  parametre = modele->parametres + infos->parametre_affiche;

  OBJ_CopierTexte( MOD_GENERIQUE, MG_NOM_PARAMETRE, parametre->nom, 0 );
  OBJ_CopierTexte( MOD_GENERIQUE, MG_SYMBOLE_PARAM, parametre->symbole, 0 );
  OBJ_CopierTexte( MOD_GENERIQUE, MG_UNITE_PARAM  , parametre->unite, 0 );

  OBJ_MettreNombre( MOD_GENERIQUE, MG_VALEUR_INIT, parametre->variable.valeur_initiale, 0.0 );

  if ( VAR_CONSTANTE == parametre->variable.type ) {
    OBJ_choisit( MOD_GENERIQUE, MG_PARAM_CONST, FORCE_OUI );
    I_MG_CouperConstante( VRAI );
  } else {
    OBJ_choisit( MOD_GENERIQUE, MG_PARAM_CONST, FORCE_NON );
    I_MG_CouperConstante( FAUX );

    OBJ_CopierTexte( MOD_GENERIQUE, MG_DERIVEE, parametre->derivee, 0 );

    OBJ_MettreNombre( MOD_GENERIQUE, MG_BORNE_INF, parametre->variable.min, 0.0 );
    OBJ_MettreNombre( MOD_GENERIQUE, MG_BORNE_SUP, parametre->variable.max, 0.0 );
  }
}

/* -------- Mmorisation des valeurs -------- */

static void I_MG_EnregistrerParametre(struct infos_generique *infos)
{
  char *texte;
  struct parametre *parametre;

  if ( 0 == infos->modele->nombre_parametres ) return; /* Pas de paramtre, donc rien  mmoriser ! */

  /* Quel est le paramtre affich ? */
  if ( infos->parametre_affiche >= infos->modele->nombre_parametres ) {
    (void) fprintf(stderr, "%s, %s l.%u : Index de paramtre affich absurde !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  parametre = infos->modele->parametres + infos->parametre_affiche;

  /* Enregistrement proprement dit */
  free( parametre->nom ); parametre->nom = NULL;
  texte = lit_chaine( MOD_GENERIQUE, MG_NOM_PARAMETRE, 0, NULL, TXT_REMPLACE );
  if ( texte != NULL ) {
    parametre->nom = strdup( texte );
  }

  free( parametre->unite ); parametre->unite = NULL;
  texte = lit_chaine( MOD_GENERIQUE, MG_UNITE_PARAM, 0, NULL, TXT_REMPLACE );
  if ( texte != NULL ) {
    parametre->unite = strdup( texte );
  }

  free( parametre->symbole ); parametre->symbole = NULL;
  texte = lit_chaine( MOD_GENERIQUE, MG_SYMBOLE_PARAM, 0, NULL, TXT_REMPLACE );
  if ( texte != NULL ) {
    parametre->symbole = strdup( texte );
  }

  parametre->variable.contenu.valeur  = OBJ_LireNombre( MOD_GENERIQUE, MG_VALEUR_INIT, NULL );
  parametre->variable.valeur_initiale = parametre->variable.contenu.valeur;
  if ( OBJ_choisit( MOD_GENERIQUE, MG_PARAM_CONST, QUESTION ) ) {
    /* Paramtre constant : les bornes et la drive n'ont pas d'importance */
    parametre->variable.type = VAR_CONSTANTE;
  } else {
    /* Rel paramtre ajustable : on mmorise aussi les bornes & la drive */
    parametre->variable.type = VAR_LIBRE;
    parametre->variable.min = OBJ_LireNombre( MOD_GENERIQUE, MG_BORNE_INF, NULL );
    parametre->variable.max = OBJ_LireNombre( MOD_GENERIQUE, MG_BORNE_SUP, NULL );

    free( parametre->derivee ); parametre->derivee = NULL;
    texte = lit_chaine( MOD_GENERIQUE, MG_DERIVEE, 0, NULL, TXT_REMPLACE );
    if ( texte != NULL ) {
      parametre->derivee = strdup( texte );
    }
  }
}

/* -------- Activations/inactivations dans l'arbre -------- */

static void I_MG_CouperParametre(boolean couper)
{
  /* Si couper = VRAI, on inactive toutes les infos du paramtre [pas de paramtre prsent...] */
  OBJ_active( MOD_GENERIQUE, MG_EFFACER_PARAM, ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );
  OBJ_active( MOD_GENERIQUE, MG_PARAM_CONST  , ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );

  OBJ_active( MOD_GENERIQUE, MG_NOM_PARAMETRE, ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );
  OBJ_active( MOD_GENERIQUE, MG_VALEUR_INIT  , ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );
  OBJ_active( MOD_GENERIQUE, MG_UNITE_PARAM  , ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );
  OBJ_active( MOD_GENERIQUE, MG_SYMBOLE_PARAM, ( VRAI == couper ) ? ( FORCE_NON ) : ( FORCE_OUI ) );

  /* Pour le min et max, si on active : seulement si paramtre non constant */
  if ( VRAI == couper ) {
    I_MG_CouperConstante( VRAI );
  } else {
    if ( VRAI == OBJ_choisit( MOD_GENERIQUE, MG_PARAM_CONST, QUESTION ) ) {
      I_MG_CouperConstante( VRAI );
    } else {
      I_MG_CouperConstante( FAUX );
    }
  }
}

static void I_MG_CouperConstante(boolean couper)
{
  /* Si on coupe, on dsactive -> force_non */
  if ( VRAI == couper ) {
    /* Paramtre constant : ni les bornes, ni la drive n'ont de sens */
    OBJ_active( MOD_GENERIQUE, MG_BORNE_INF, FORCE_NON );
    OBJ_active( MOD_GENERIQUE, MG_BORNE_SUP, FORCE_NON );
    OBJ_active( MOD_GENERIQUE, MG_DERIVEE  , FORCE_NON );
  } else {
    /* Paramtre variable : il faut les bornes et la drive */
    OBJ_active( MOD_GENERIQUE, MG_BORNE_INF, FORCE_OUI );
    OBJ_active( MOD_GENERIQUE, MG_BORNE_SUP, FORCE_OUI );
    OBJ_active( MOD_GENERIQUE, MG_DERIVEE  , FORCE_OUI );
  }
}

/* -------- Gestion des modles prdfinis -------- */

static void I_MG_LibererModeles(struct infos_generique *infos)
{
  unsigned long i;

  /* On met au pralable un texte sr dans le bouton ! */
  OBJ_RemplacerTexte( MOD_GENERIQUE, MG_MODELE, txt_aucun_modele, VRAI );

  /* Libration des noms de modles */
  for ( i = 0 ; i < infos->nombre_modeles ; i++ ) {
    free( infos->modeles[ i ] ); infos->modeles[ i ] = NULL;
    free( infos->fichiers_modeles[ i ] ); infos->fichiers_modeles[ i ] = NULL;
  }

  /* Libration des tables associes */
  free( infos->modeles ); infos->modeles = NULL;
  free( infos->fichiers_modeles ); infos->fichiers_modeles = NULL;

  /* Il ne reste donc plus aucun modle */
  infos->nombre_modeles = 0;
}

/* -------- Prparation de l'arbre -------- */

static void I_MG_PreparerArbre(void)
{
  int retour;

  if ( VRAI == arbres[ MOD_GENERIQUE ].drapeaux.prepare ) return;

  /* Initialisation interne */
  retour = ARB_Initialiser( MOD_GENERIQUE, NULL, I_MG_GererArbre, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur  l'initialisation...\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }
  
  /* Enjolivation */
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_PARAMETRE, 0, +500, DEPLACE_RELATIF );
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_BIBLIO   , 0, +500, DEPLACE_RELATIF );

  ARB_EnjoliverAscenseur( MOD_GENERIQUE, MG_PARAM_AVANT, MG_FOND_ASC_PARA, MG_BOUTON_ASC, MG_PARAM_APRES,
			  ASC_VERTICAL, 0 );

  OBJ_deplace( MOD_GENERIQUE, MG_TXT_SYMBOLE  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_SYMBOLE_PARAM, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_VALEUR   , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_VALEUR_INIT  , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_BORNES   , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_BORNE_INF    , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_MAX      , 0, +4, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_BORNE_SUP    , 0, +4, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_TXT_UNITE    , 0, +5, DEPLACE_PIXELS );
  OBJ_deplace( MOD_GENERIQUE, MG_UNITE_PARAM  , 0, +5, DEPLACE_PIXELS );

  arbres[ MOD_GENERIQUE ].drapeaux.prepare = VRAI;
}
