/* LASE - Logiciel d'analyse des spectres exprimentaux
   (C) 1998-2004, Emmanuel CURIS

   Module d'interface utilisateur

   Gestion graphique des blocs d'une figure
*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "figures.h"
#include "graphes.h"
#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Realisation pratique ~~~~~ */

/* -------- Changer la taille d'un bloc  la souris --------
   Renvoie  0 si changement effectu
           <0 en cas d'erreur [-1 : annulation]
           >0 en cas d'absence de traitement

   Fenetre  : la fentre dans laquelle travailler
   region2D : la rgion de dessin 2D  modifier
   xBG_zone : rgion de la fentre autorisant les dplacements
   yBG_zone
   larg_zone: dimensions maximale de la rgion [= 100%]
   haut_zone
   position : position de la souris sur le bloc
 */

int I_BLOC_ChangerTaille_Souris(Fenetre *fenetre, struct infos_dessin2D *region2D,
				int xBG_zone, int yBG_zone, unsigned int larg_zone, unsigned int haut_zone,
				unsigned int position)
{
  int retour;

  if ( TRC_EST_COIN_HG == position ) {
    int nv_x_HG, nv_y_HG;

    nv_x_HG = region2D->x_BG;
    nv_y_HG = region2D->y_BG - region2D->hauteur;
    retour = prepare_coin( fenetre, region2D->x_BG + region2D->largeur, region2D->y_BG,
			   &nv_x_HG, &nv_y_HG, TRC_EST_COIN_HG );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_HG < (int) ( region2D->x_BG + region2D->largeur ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_HG - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG + region2D->largeur - nv_x_HG ) / (double) larg_zone;
    } else {
      region2D->x_BG_pc = 100.0 * ( region2D->x_BG + region2D->largeur ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( nv_x_HG - ( region2D->x_BG + region2D->largeur ) ) / (double) larg_zone;
    }

    if ( nv_y_HG < (int) region2D->y_BG ) {
      region2D->haut_pc = 100.0 * ( region2D->y_BG - nv_y_HG ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_HG ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_HG - region2D->y_BG ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_COIN_BG == position ) {
    int nv_x_BG, nv_y_BG;

    nv_x_BG = region2D->x_BG;
    nv_y_BG = region2D->y_BG;
    retour = prepare_coin( fenetre, region2D->x_BG + region2D->largeur, region2D->y_BG - region2D->hauteur,
			   &nv_x_BG, &nv_y_BG, TRC_EST_COIN_BG );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_BG < (int) ( region2D->x_BG + region2D->largeur ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_BG - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG + region2D->largeur - nv_x_BG ) / (double) larg_zone;
    } else {
      region2D->x_BG_pc = 100.0 * ( region2D->x_BG + region2D->largeur ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( nv_x_BG - ( region2D->x_BG + region2D->largeur ) ) / (double) larg_zone;
    }

    if ( nv_y_BG < (int) ( region2D->y_BG - region2D->hauteur ) ) {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( region2D->y_BG - region2D->hauteur - nv_y_BG ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_BG ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_BG - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_COIN_HD == position ) {
    int nv_x_HD, nv_y_HD;

    nv_x_HD = region2D->x_BG + region2D->largeur;
    nv_y_HD = region2D->y_BG - region2D->hauteur;
    retour = prepare_coin( fenetre, region2D->x_BG, region2D->y_BG,
			   &nv_x_HD, &nv_y_HD, TRC_EST_COIN_HD );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_HD < (int) ( region2D->x_BG ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_HD - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG - nv_x_HD ) / (double) larg_zone;
    } else {
      region2D->larg_pc = 100.0 * ( nv_x_HD - region2D->x_BG ) / (double) larg_zone;
    }

    if ( nv_y_HD < (int) region2D->y_BG ) {
      region2D->haut_pc = 100.0 * ( region2D->y_BG - nv_y_HD ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_HD ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_HD - region2D->y_BG ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_COIN_BD == position ) {
    int nv_x_BD, nv_y_BD;

    nv_x_BD = region2D->x_BG + region2D->largeur;
    nv_y_BD = region2D->y_BG;
    retour = prepare_coin( fenetre, region2D->x_BG, region2D->y_BG - region2D->hauteur,
			   &nv_x_BD, &nv_y_BD, TRC_EST_COIN_BD );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_BD < (int) ( region2D->x_BG ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_BD - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG - nv_x_BD ) / (double) larg_zone;
    } else {
      region2D->larg_pc = 100.0 * ( nv_x_BD - region2D->x_BG ) / (double) larg_zone;
    }

    if ( nv_y_BD < (int) ( region2D->y_BG - region2D->hauteur ) ) {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( region2D->y_BG - region2D->hauteur - nv_y_BD ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_BD ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_BD - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_MILIEU_BORD_HAUT == position ) {
    int nv_y_H;

    nv_y_H = region2D->y_BG - region2D->hauteur;
    retour = prepare_barre_horizontale( fenetre, region2D->x_BG, region2D->largeur, &nv_y_H );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_y_H < (int) region2D->y_BG ) {
      region2D->haut_pc = 100.0 * ( region2D->y_BG - nv_y_H ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_H ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_H - region2D->y_BG ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_MILIEU_BORD_BAS == position ) {
    int nv_y_B;

    nv_y_B = region2D->y_BG;
    retour = prepare_barre_horizontale( fenetre, region2D->x_BG, region2D->largeur, &nv_y_B );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_y_B < (int) ( region2D->y_BG - region2D->hauteur ) ) {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( region2D->y_BG - region2D->hauteur - nv_y_B ) / (double) haut_zone;
    } else {
      region2D->y_BG_pc = 100.0 * ( yBG_zone - nv_y_B ) / (double) haut_zone;
      region2D->haut_pc = 100.0 * ( nv_y_B - ( region2D->y_BG - region2D->hauteur ) ) / (double) haut_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_MILIEU_BORD_GAUCHE == position ) {
    int nv_x_G;

    nv_x_G = region2D->x_BG;
    retour = prepare_barre_verticale( fenetre, region2D->y_BG - region2D->hauteur, &nv_x_G, region2D->hauteur );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_G < (int) ( region2D->x_BG + region2D->largeur ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_G - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG + region2D->largeur - nv_x_G ) / (double) larg_zone;
    } else {
      region2D->x_BG_pc = 100.0 * ( region2D->x_BG + region2D->largeur ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( nv_x_G - ( region2D->x_BG + region2D->largeur ) ) / (double) larg_zone;
    }

    return 0;			/* Changement effectu */
  }

  if ( TRC_EST_MILIEU_BORD_DROIT == position ) {
    int nv_x_D;

    nv_x_D = region2D->x_BG + region2D->largeur;
    retour = prepare_barre_verticale( fenetre, region2D->y_BG - region2D->hauteur, &nv_x_D, region2D->hauteur );
    if ( FAUX == retour ) return -1; /* Dplacement annul : rien  faire */

    /* Conversion cran -> % */
    if ( nv_x_D < (int) ( region2D->x_BG ) ) {
      region2D->x_BG_pc = 100.0 * ( nv_x_D - xBG_zone ) / (double) larg_zone;
      region2D->larg_pc = 100.0 * ( region2D->x_BG - nv_x_D ) / (double) larg_zone;
    } else {
      region2D->larg_pc = 100.0 * ( nv_x_D - region2D->x_BG ) / (double) larg_zone;
    }

    return 0;
  }

  /* Si on est l : on n'a pas cliqu sur une "poigne" -> rien de spcial  faire ! */
  return +1;
}

/* -------- Changer les axes  la souris --------
   bloc    : le bloc  modifier
   position : position de la souris sur le bloc
*/

void I_BLOC_ChangerAxes_Souris(Fenetre *mere, Bloc_Figure *bloc, unsigned int position)
{
  if ( TRC_EST_AU_CENTRE == position ) {
    I_ParametrerBornes( &(bloc->dessin.axe_X), &(bloc->dessin.axe_Y), NULL, BLOC_ChoisirBornesOptimales, bloc, mere );
  }

  if ( position & TRC_EST_BORD_GAUCHE ) {
    I_ParametrerLegendes( &(bloc->dessin.axe_Y), AXE_Y, VRAI, mere );
  }
  if ( position & TRC_EST_BORD_DROIT ) {
    OPTION_A_FAIRE( "Paramtrer l'axe des Y secondaire" );
  }

  if ( position & TRC_EST_BORD_HAUT ) {
    OPTION_A_FAIRE( "Paramtrer l'axe des X secondaire" );
  }
  if ( position & TRC_EST_BORD_BAS ) {
    I_ParametrerLegendes( &(bloc->dessin.axe_X), AXE_X, VRAI, mere );
  }
}

/* -------- Dplacer le bloc dans la fentre --------
   Renvoie  0 si changement effectu
           <0 en cas d'erreur [-1 : annulation]
           >0 en cas d'absence de traitement

   Fenetre  : la fentre dans laquelle travailler
   region2D : la rgion de dessin 2D  modifier
   xBG_zone : rgion de la fentre autorisant les dplacements
   yBG_zone
   larg_zone: dimensions maximale de la rgion [= 100%]
   haut_zone
   delta_x  : dcalage entre le coin BG du bloc et la position de la souris [>0 quand on est dans le bloc]
   delta_y
 */

int I_BLOC_Deplacer_Souris(Fenetre *fenetre, struct infos_dessin2D *region2D,
			   int xBG_zone, int yBG_zone, unsigned int lg_zone, unsigned int ht_zone, int delta_x, int delta_y)
{
  int nv_x_HG, nv_y_HG, retour;

  if ( ( NULL == region2D ) || ( NULL == fenetre ) ) return -20;

  /* On dplace le rectangle */
  retour = prepare_deplace( fenetre, -delta_x, delta_y - region2D->hauteur, region2D->largeur, region2D->hauteur, &nv_x_HG, &nv_y_HG );
  if ( FAUX == retour ) return -1;

/*   printf("Y reus     : Y HG=%d -> BG=%d (H=%u), Zone : B=%d, H=%d, haut=%u\n", */
/* 	 nv_y_HG, nv_y_HG + region2D->hauteur, region2D->hauteur, yBG_zone, yBG_zone - ht_zone, ht_zone ); */

  /* Contrles de la nouvelle position : on doit rester dans la zone ! */
  if ( nv_x_HG < xBG_zone )                                   nv_x_HG = xBG_zone;
  if ( (nv_x_HG + region2D->largeur) > (xBG_zone + lg_zone) ) nv_x_HG = (xBG_zone + lg_zone) - region2D->largeur;

  if ( (nv_y_HG + (int) region2D->hauteur) > yBG_zone ) nv_y_HG = yBG_zone - region2D->hauteur;
  if ( nv_y_HG < (yBG_zone - (int) ht_zone) )           nv_y_HG = (yBG_zone - ht_zone);

/*   printf("Y contrls : Y HG=%d -> BG=%d (H=%u), Zone : B=%d, H=%d, haut=%u\n", */
/* 	 nv_y_HG, nv_y_HG + region2D->hauteur, region2D->hauteur, yBG_zone, yBG_zone - ht_zone, ht_zone ); */

  /* On mmorise la nouvelle position */
  region2D->x_BG_pc = 100.0 * ( nv_x_HG - xBG_zone ) / (double) lg_zone;
  region2D->y_BG_pc = 100.0 * ( yBG_zone - ( nv_y_HG + region2D->hauteur ) ) / (double) ht_zone;
#ifdef _DEVELOPPE_
  if ( ( region2D->y_BG_pc < 0.0 ) || ( region2D->y_BG_pc > 100.0 ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Valeur de nv_Y_HG absurde [%g %%]; nv_y_HG=%d, ht_zone=%d, yBG_zone=%d\n",
		   __FILE__, __FUNCTION__, __LINE__, region2D->y_BG_pc, nv_y_HG, ht_zone, yBG_zone );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
  }
#endif

  return 0;
}

/* -------- Raliser un zoom  la souris dans une rgion du place --------
 |  Renvoie 0 si le zoom a pu tre effectu
           <0 en cas d'erreur [-1 : annulation]
           >0 en cas d'absence de traitement

   fenetre  : la fentre dans laquelle travailler
   region   : la rgion de dessin 2D  modifier
   axes_secondaires : doit-on travailler sur certains axes secondaires ?
   x_clic, y_clic : position du premier clic (coordonnes cran ABSOLUES)
 */

int I_BLOC_ZoomerSouris(Fenetre *fenetre, struct infos_dessin2D *region, unsigned int axes_secondaires, int x_clic, int y_clic)
{
  double x_min, x_max, y_min, y_max;
  int retour, x_fin, y_fin;
  InfosAxe *axe_X, *axe_Y;

  if ( ( NULL == region ) || ( NULL == fenetre ) ) return -20;

  /* On prpare le zoom  la souris */
  retour = prepare_zoom( fenetre, x_clic, y_clic, &x_fin, &y_fin );
  if ( retour != VRAI ) return +1; /* Annulation ! */

  /* Contrle de cohrence : clic = coin HG [donc x_min, mais y_max !], fin = coin BD [donc x_max, mais y_min !] */
  if ( x_fin < x_clic ) {
    int tmp = x_fin; x_fin = x_clic; x_clic = tmp; 
  }
  if ( y_fin < y_clic ) {
    int tmp = y_fin; y_fin = y_clic; y_clic = tmp; 
  }

  /* Conversion de ces points en vrais points */
  retour = TRC_Z2D_EcranVXY( region, axes_secondaires, x_clic, y_clic, &x_min, &y_max );
  if ( retour != 0 ) return -1;
  (void) TRC_Z2D_EcranVXY( region, axes_secondaires, x_fin, y_fin, &x_max, &y_min );

  /* Slection des axes modifis */
  if ( ( axes_secondaires & AXE_X ) && ( region->X_secondaire != NULL ) ) {
    axe_X = region->X_secondaire;
  } else {
    axe_X = &(region->axe_X);
  }

  if ( ( axes_secondaires & AXE_Y ) && ( region->Y_secondaire != NULL ) ) {
    axe_Y = region->Y_secondaire;
  } else {
    axe_Y = &(region->axe_Y);
  }

  /* Mmorisation des changements */
  axe_X->min = x_min;
  axe_X->max = x_max;

  axe_Y->min = y_min;
  axe_Y->max = y_max;

  /* Modification termine sans encombre */
  return 0;
}
