/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 2006, Emmanuel CURIS
   Cration : 21 septembre 2010

   Interface graphique

   Comparaison de 2 ou plus graphes
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

/* #include "automatise.h" */
/* #include "extrait.h" */
#include "interface.h"
/* #include "operations.h"	*/	/* Pour la drive */
/* #include "science.h" */
/* #include "serie.h" */
#include "statistiques.h"
/* #include "tracer.h" */

/* -------- Variables locales -------- */


/* -------- Les textes [isols pour traduction] -------- */

#include "int_comparer.txt"

/* -------- Fonctions locales -------- */

/* ~-~-~-~-~-~-~-~-~ Appel par l'utilisateur ~-~-~-~-~-~-~-~-~ */

/* -------- Routine d'appel principale : initialisation, calculs & cration d'une figure -------- */

struct graphe *I_GRF_ComparerGraphes(struct graphe *graphe_1)
{
  Graphe **graphes_possibles = NULL, **graphes_choisis;
  Graphe *resultat = NULL, *difference = NULL;
  unsigned long nombre_graphes_possibles = 0, nombre_graphes_choisis;

  /* 1. Choisir les graphes de mme nature que celui indiqu */
  if ( FAUX == GRF_EstFonctionReelle( graphe_1 ) ) {
    /* Pas fonction relle : la procdure de comparaison n'a pas de sens */
    (void) BI_ERR_alerte( principale, &ERR_GrapheNonReel, VRAI );
    return NULL;
  }

  /* 2. On demande la liste des graphes compatibles avec la rfrence :
        mme type (au sens large) - mme genre - mmes units */
  graphes_possibles = GRF_ChoisirMasque( NULL, graphe_1,
					 GRF_CHOIX_GENRE | GRF_CHOIX_TYPE | GRF_CHOIX_UNITE_X | GRF_CHOIX_UNITE_Y,
					 GRF_CHOIX_UNITE_X | GRF_CHOIX_UNITE_Y, &nombre_graphes_possibles );
  if ( NULL == graphes_possibles ) {
    /* Problme survenu, car la rfrence est forcment compatible avec elle-mme ! */
    (void) fprintf(stderr, "%s,%s l.%u : Problme survenu dans la slection des graphes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( 1 == nombre_graphes_possibles ) {
    /* Un seul graphe compatible : inutile de chercher  faire une comparaison... */
    (void) BI_ERR_alerte( principale, &ERR_GrapheUnique, VRAI );
    return NULL;
  }

  /* -- parmi tous les compatibles, quels sont ceux que l'on veut comparer ? -- */
  graphes_choisis = GRF_ChoisirDansTable( graphes_possibles, nombre_graphes_possibles, OPTIONS_CHOIX_MULTIPLE, &nombre_graphes_choisis );

  /* Plus besoin de la liste des graphes possibles */
  free( graphes_possibles ); graphes_possibles = NULL;

  /* Combien de graphes choisis ? */
  if ( 1 == nombre_graphes_choisis ) {
    /* Un seul graphe compatible : inutile de chercher  faire une comparaison... */
    (void) BI_ERR_alerte( principale, &ERR_GrapheUnique, VRAI );
    return NULL;
  }

  /* On fait la comparaison */
  resultat =  STAT_GRF_ComparerGraphes( nombre_graphes_choisis, graphes_choisis, NULL, &difference );

  OPTION_A_FAIRE( "Ouvrir une figure pour prsenter les rsultats" );

  /* Plus besoin de la liste des graphes choisis */
  free( graphes_choisis ); graphes_choisis = NULL;

  /* Termin : on renvoie le graphe avec les p */
  return resultat;
}
