/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2003

   Initialisation de l'interface graphique [arbres, dialogues,...]
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h" /* pour l'init des listes d'atomes, de molcules,... */
#include "interface.h"
#include "theorie.h" /* Pour le calcul des phases - sera dplac un jour */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define NOMBRE_ONGLETS 4	/* Nombre d'onglets dans la fentre des listes d'objets */

static BI_Onglet onglets[ NOMBRE_ONGLETS ] = {
  { "Graphes", 0, CHOIX_GRAPHE, (int (*)(void) ) NULL },
  { "Images", 0, CHOIX_IMAGE, (int (*)(void) ) NULL },
  { "Mod"e_GRAVE"les", 0, LISTE_MODELES, (int (*)(void) ) NULL },
  { "Mol"e_AIGU"cules", 0, LISTE_MOLECULES, (int (*)(void) ) NULL },
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void prep_arbres(void);

/* ~~~~~~~~ Fonctions externes ~~~~~~~ */

extern int MEN_PreparerAjuste   (void);
extern int MEN_PreparerBanque   (void);
extern int MEN_PreparerExtraire (void);
extern int MEN_PreparerFluoX    (void);
extern int MEN_PreparerStructure(void);
extern int MEN_PreparerTheorie  (void);

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

/* -------- Prparation des arbres et tout a -------- */

int InitialiserInterface( void )
{
  int retour;

  if ( fenetre_onglets != NULL ) {
    AVERTISSEMENT( "Fentre d'onglets non-NULL [%s]. Interface dj initialise ?", fenetre_onglets->titre );
    return -20;
  }

  /* On prpare les menus */
  retour = MEN_PreparerExtraire();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerExtraire()\n");
    return retour;
  }
  retour = MEN_PreparerTheorie();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerTheorie()\n");
    return retour;
  }
  retour = MEN_PreparerAjuste();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerAjuste()\n");
    return retour;
  }
  retour = MEN_PreparerStructure();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerStructure()\n");
    return retour;
  }
  retour = MEN_PreparerBanque();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerBanque()\n");
    return retour;
  }
  retour = MEN_PreparerFluoX();
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans MEN_PreparerFluoX()\n");
    return retour;
  }

  /* On prpare la liste des graphes (pour avoir accs aux sous-menus !) */
  LGRF_PreparerArbre();
  LIMG_PreparerArbre();
  LMOD_PreparerArbre();
  LMOL_PreparerArbre();

  /* On prpare la fentre des onglets */
  fenetre_onglets = FEN_CreerOnglets( NAME|MOVER, NOMBRE_ONGLETS, onglets );
  if ( NULL == fenetre_onglets ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir la fentre des listes d'objets !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -21;
  }

  /* On ouvre cette fentre */
  FEN_Ouvrir( fenetre_onglets );

  prep_arbres();    /* On rend les arbres plus jolis...  part car long et sans intrt */

  MOL_PrepareListe(); /* Liste des molcules */
  ATM_PrepareListe(); /* Liste des atomes */

  /* Le menu par dfaut est le menu d'extraction des spectres */
  retour = BI_InstallerMenu(MENU, "LASE", MEN_extrait);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "InitialiserInterface : ERREUR dans BI_InstallerMenu()\n");
    return retour;
  }

  /* Initialisation russie */
  return 0;
}

static void prep_arbres(void)
{
  OBJECT *adresse;

  /* Calcul d'une phase thorique */
  arbres[CALC_PHASE].fonction.arbre = GER_phase;

  /* TFI, arbre en fentre */
  adresse = arbres[PARAM_FOUR_INVER].adresse;
  OB_X(adresse, TF_INV_FEN_AVANT) += 1;
  OB_X(adresse, TF_INV_FEN_APRES) += 1;
  OB_X(adresse, TF_INV_FEN_FOND)  += 1;
  OB_Y(adresse, TF_INV_FEN_FOND)  += 1;
  OB_HEIGHT(adresse, TF_INV_FEN_FOND) -= 2;
}
