/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2007

   Interface pour la gestion de base des molécules (copie, effacement,...)
*/

#include <stdio.h>
#include <stdlib.h>

#include "exafs.h"
#include "systeme.h"
#include "txt_utils.h"

#include "atomes.h"
#include "interface.h"
#include "statistiques.h"

#include "int_molecules.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~ Effacement ~-~-~-~-~-~-~-~ */

int I_MOL_EffacerMolecule(Molecule *molecule)
{
  if ( NULL == molecule ) return 0;

  if ( FAUX == MOL_EstSauvee( molecule ) ) {
    int reponse;

    /* La molécule n'a pas été sauvée : on demande confirmation avec proposition de sauvegarde */
    reponse = BI_ERR_alerte( NULL, &Q_Effacer_NonSauvee, VRAI );
    if ( 3 == reponse ) return +1;  /* Annulation */
    if ( 2 == reponse ) {
      /* On sauvegarde en premier */
      reponse = I_MOL_Sauver( molecule );
      if ( reponse != 0 ) {
	(void) fprintf(stderr, "%s, %s l.%u : Échec en voulant sauver la molécule : effacement annulé\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return -5;
      }
    }
  } else {
    int reponse;

    /* La molécule a déjà été sauvée : on se contente de demander confirmation */
    reponse = BI_ERR_alerte( NULL, &Q_Effacer_NonSauvee, VRAI );
    if ( 2 == reponse ) return +1;  /* Annulation */
  }

  /* On efface, s'il y a eu confirmation */
  MOL_effacer( molecule, VRAI, FAUX );

  /* Et on refait la liste du sélecteur de molécules */
  MOL_PrepareListe();

  /* Terminé sans problème */
  return 0;
}
