/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2005, Emmanuel CURIS
   Cration : 31 janvier 2005

   Menu prsentant les options associes aux graphes d'un certain type
   Ici : cration de graphes ex-nihilo
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "interface.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define OPT_CREATION_REEL_XY  1 /* 0 = sparateur "valeurs" */
#define OPT_CREATION_SERIE_Y  2
#define OPT_CREATION_REEL_FCT 4	/* 3 = sparateur "fonction" */

#define NOMBRE_OPTIONS_CREATION 5

#include "opt_creation.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int OPT_CreerSerieY(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int OPT_CreationGraphe(void)
{
  int option;

  /* On appelle le menu */
  option = SS_menu_texte( menu_creation, NOMBRE_OPTIONS_CREATION, titre_menu_creation, 0, ROOT );
  if ( option < 0 ) {
    return +1; /* Annul : erreur mineure... */
  }

  /* On fait la cration demande... */
  switch ( option )
  {
  case OPT_CREATION_REEL_XY:
    OPTION_A_FAIRE( "Crer un graphe (X,Y)" );
    break;

  case OPT_CREATION_SERIE_Y:
    (void) OPT_CreerSerieY();
    break;

  case OPT_CREATION_REEL_FCT:
    OPTION_A_FAIRE( "Crer un graphe Y=f(X)" );
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Option non traite [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, option);
    return -1;
  }

  /* Tout s'est bien pass */
  return 0;

}

/* -------- Crer une srie de valeurs simple -------- */

static int OPT_CreerSerieY(void)
{
  char *valeurs, **table_valeurs;
  unsigned long nombre_valeurs, i;
  Graphe *graphe;

  /* On demande les valeurs  l'utilisateur */
  valeurs = BI_Demande_Texte( principale, &Q_ValeursSerieY, NULL );
  if ( NULL == valeurs ) {
    return +1;			/* Cration annule ou erreur */
  }

  /* On les convertit en table */
  table_valeurs = TXT_DecomposerChaine( valeurs, ';', &nombre_valeurs );

  /* Plus besoin des valeurs... */
  free( valeurs ); valeurs = NULL;

  /* Contrles */
  if ( ( 0 == nombre_valeurs ) || ( NULL == table_valeurs ) ) {
    /* Aucune valeur entre ou erreur */
    return -1;
  }
  if ( 1 == nombre_valeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Il faut au moins 2 valeurs !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( table_valeurs[ 0 ] ); table_valeurs[ 0 ] = NULL;
    free( table_valeurs ); table_valeurs = NULL;
    return -2;
  }

  /* On cre le graphe dans la srie courante */
  graphe = GRF_nouveau( graphe_courant, nombre_valeurs, GR_SSTYPE, GRF_SERIE, NULL );
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans GRF_nouveau()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( table_valeurs[ 0 ] ); table_valeurs[ 0 ] = NULL;
    free( table_valeurs ); table_valeurs = NULL;
    return -3;
  }

  /* On mmorise les valeurs dans le graphe */
  for ( i = 0 ; i < nombre_valeurs ; i++ ) {
    double erreur;

    graphe->y[ i ] = TXT_CNV_vNombre_D( table_valeurs[ i ], NULL, &erreur );
    if ( erreur > 0 ) {
      /* On cre les incertitudes si ncessaire */
      if ( NULL == graphe->erreur_y ) {
	GRF_creer_erreurs( graphe, AXE_Y );
      }
      if ( graphe->erreur_y != NULL ) {
	graphe->erreur_y->y[ i ] = erreur;
	graphe->erreur_y->z.ddl[ i ] = 0;
      }
    }
  }

  /* On peut librer les valeurs */
  free( table_valeurs[ 0 ] ); table_valeurs[ 0 ] = NULL;
  free( table_valeurs ); table_valeurs = NULL;

  /* On recalcule les bornes */
  GRF_bornes( graphe, NE_PAS_FAIRE, DESORDONNE );

  /* Tout s'est bien pass */
  return 0;
}
