/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2008, Emmanuel CURIS
   Cration : 16 janvier 2008 par copie de l'ancien Images/gestion.c

   Interface utilisateur : affichage de l'image courante et options associs [fentre principale "image"]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "echantillon.h"	/* Pour la composition ! */
#include "fichiers.h"
#include "graphes.h"
#include "images.h"
#include "interface.h"
#include "tracer.h"

/* Dfinitions pour le menu contextuel */

#define MCI_NOMBRE_OPTIONS 15

#define MCI_INTERPOLER         0
#define MCI_LISSER             1
#define MCI_NORMALISER         2
#define MCI_ROTATION           4 /* avant : sparateur */
#define MCI_MIROIR_HV          5
#define MCI_MIROIR_AUTRE       6
#define MCI_DECOUPER           7
#define MCI_TRANSFORMATION     8
#define MCI_PROFIL_RADIAL     10 /* avant : sparateur */
#define MCI_COUPE_TRANVERSE   11
#define MCI_PROFIL_CURVILIGNE 12
#define MCI_ISOCONTOUR        13
#define MCI_COUPE_PLANE       14

#include "int_principale_image.txt"

/* ~~~~~~~~ Structures locales ~~~~~~~~ */

#ifdef AVEC_OPEN_GL_
typedef struct {
  GLfloat lon, lat, inc; /* Longitude, Latitude, Inclinaison */
  GLfloat distance;
  GLfloat echelle;
  boolean fil_de_fer;
} Camera;
#endif

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#ifdef AVEC_OPEN_GL_
static Camera cam_carto = {
  10.0, 0.0, 0.0,
  0.0, 1.3,
  FAUX
};
static boolean dessin_smooth = FAUX;
#endif

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean IMG_gere_menu(void);

static void RCT_image(Fenetre *, int, int, int, int);
static void RDS_image(Fenetre *, int, int, unsigned int, unsigned int);
static void RDS_img3d(Fenetre *, int, int, unsigned int, unsigned int);
static void ASC_image(Fenetre *, int, int, int);
static int  CLC_image(Fenetre *, int, int, int, int, int);
static int  CLC_img3d(Fenetre *, int, int, int, int, int);
static int  CLV_image(Fenetre *fen, unsigned int touche, int etat);
static int  MSG_image(Fenetre *fen, Message *message);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Ouverture de la fentre d'affichage -------- */

int I_OuvrirPrincipale_IMG(void)
{
  DES_image *dessin_image;
  Palette palette = None;

  /* On vrifie que la fentre d'images n'est pas dj ouverte */
  if ( fen_images != NULL ) {
    /* Fentre dj ouverte, on la passe au premier plan */
    FEN_premierplan( fen_images );
    return 0;
  }

  /* On cre donc la fentre : coupe en deux verticalement, droite = OpenGL ; pas de fermeture possible */
  fen_images = FEN_CreerAvecGL( TOUS_SF_ASC|ASC_VERTICAL|ASC_HORIZONTAL, txt_titre_fenetre, BI_FEN_ARBRE_DROITE,
				RDS_image, RDS_img3d,
				ASC_image, CLV_image, CLC_image,
				MSG_image, RCT_image, (int (*)(Fenetre *)) NULL );
  if ( NULL == fen_images ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir la fentre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* Gestion de la partie 3D */
  FEN_GL_evntsgl( fen_images, RDS_img3d, NULL, CLC_img3d,
		  (int (*)(Fenetre *, int, int, int, int) ) NULL, (int (*)(Fenetre *)) NULL);

  FEN_infos( fen_images, txt_titre_fenetre);
  fen_images->msg_masque.fermeture = VRAI;

  if (palette != None) /* Fentre aura donc sa propre palette */
    FEN_palette( fen_images, palette );

  /* Paramtres d'affichage */
  dessin_image = ( DES_image * ) malloc( sizeof( DES_image ) );
  if ( NULL == dessin_image ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FEN_Detruire( fen_images ); fen_images = NULL;
    return -2;
  }

  dessin_image->image = NULL; /* Pas encore d'image */
  dessin_image->couleur = IMG_FAUSSES_COULEURS;
  dessin_image->proprietes = IMG_AVEC_COUPE | IMG_AVEC_CENTRE | IMG_AVEC_CADRE | IMG_AVEC_GRADUATION | IMG_AVEC_AXES;
  dessin_image->x_min = 0.0; dessin_image->x_max = 0.0;
  dessin_image->y_min = 0.0; dessin_image->y_max = 0.0;

  /* Ouverture proprement dite */
  FEN_StockerComplement( fen_images, FEN_COMPLEMENT_1, dessin_image );
  FEN_Ouvrir( fen_images );

  /* On a russi, tout va bien */
  return 0;
}

/* -------- Redessin de la fentre 2D -------- */

static void RDS_image(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
/*   double echelle; */
  int x_zone, y_zone;
  unsigned int larg_zone, haut_zone;
  DES_image *des_image;
  Graphe *coupe = NULL;
/*   InfosAxe axe = { NULL, NULL, */
/* 		   AXE_AVEC_BORNES | AXE_AVEC_GRADUATIONS | AXE_AVEC_LEGENDE | AXE_ECRAN, GRADUATION_DECIMALE, 0, 0.0, 0.0, 0.0, 0, 5.0, 0.0, */
/* 		   NOIR, NOIR, NOIR, NOIR, 0 }; */

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* Utilisable si 16 * 16 microns minimum */
  if ( ( fenetre->w_larg < 16 ) || ( fenetre->w_haut < 16 ) ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Fentre trop petite\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return;
  }

  /* Sparateurs */
  /* a. Moiti haute [image] / moiti basse [lgende + coupe] */
  TRC_Separateur( fenetre, HORIZONTAL, 0, fenetre->w_haut / 2, fenetre->w_larg );

  /* b. Moiti basse : 1/3 suprieur [lgende] / 2/3 infrieurs [coupe] */
  TRC_Separateur( fenetre, HORIZONTAL, 0, fenetre->w_haut / 2 + fenetre->w_haut / 6, fenetre->w_larg );

  if ( image_courante == NULL ) {
    TRC_Message( fenetre, txt_err_aucune_image );
    return;
  }

  des_image = ( DES_image * ) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( NULL == des_image ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos pour le trac des images\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  if ( des_image->image != image_courante ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, Incohrence image courante / image associe  la fentre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

#ifdef _DEVELOPPE_
  printf("%s : rgion du plan X=[%g, %g] Y=[%g, %g], image : [%g, %g] Y=[%g, %g], couleur : %u\n", __FUNCTION__,
	 des_image->x_min, des_image->x_max, des_image->y_min, des_image->y_max,
	 des_image->image->x_min, des_image->image->x_min + des_image->image->l_reel * (des_image->image->larg - 0.5),
	 des_image->image->y_min, des_image->image->y_min + des_image->image->h_reel * (des_image->image->haut - 0.5),
	 des_image->couleur );
#endif

  /* Moiti haute de la fentre : l'image */
  x_zone = 2; y_zone = 2;
  larg_zone = fenetre->w_larg; if ( larg_zone > 4 ) larg_zone -= 4; else larg_zone = 0;
  haut_zone = fenetre->w_haut / 2; if ( haut_zone > 4 )  haut_zone -= 4; else haut_zone = 0;
  TRC_PortionImageDansRectangle( fenetre, des_image, x_zone, y_zone, larg_zone, haut_zone );

  /* ... et ses axes */
#ifdef a_controler
  echelle = TRC_Echelle2D( des_image->x_max - des_image->x_min, des_image->y_max - des_image->y_min, larg_zone, haut_zone );

  axe.min = des_image->image->x_min - des_image->image->l_reel * 0.5;
  axe.max = des_image->image->x_min + des_image->image->l_reel * (des_image->image->larg - 0.5);
  TRC_AxeGradue( fenetre, x_zone, y_zone + 20 /*  + (int) rint( ( des_image->y_max - des_image->y_min ) * echelle ) */,
		 x_zone + (int) rint( ( des_image->x_max - des_image->x_min ) * echelle ), y_zone + 20/*  + (int) rint( ( des_image->y_max - des_image->y_min ) * echelle ) */,
		 &axe, 5, 10 );

  axe.proprietes &= AXE_VERTICAL;
  axe.min = des_image->image->y_min - des_image->image->h_reel * 0.5;
  axe.max = des_image->image->y_min + des_image->image->h_reel * (des_image->image->haut - 0.5);
  TRC_AxeGradue( fenetre, x_zone, y_zone + (int) rint( ( des_image->y_max - des_image->y_min ) * echelle ), x_zone, y_zone, &axe, 5, 10 );
#endif

  /* Moiti basse de la fentre : coupe ou autres informations */
  /* 1. L'chelle : le tiers suprieur */
  y_zone =  ( fenetre->w_haut / 2 ) + 2;
  haut_zone = fenetre->w_haut / 6; if ( haut_zone > 4 ) haut_zone -= 4; else haut_zone = 0;
  TRC_EchelleImage( fenetre, des_image, x_zone, y_zone, larg_zone, haut_zone );

  /* 2. La coupe, ventuellement */
  y_zone = fenetre->w_haut / 2 + fenetre->w_haut / 6 + 2;
  haut_zone = (fenetre->w_haut - y_zone) - 2;
  if ( NULL == image_courante->infos_affichage ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, pas d'infos d'affichage pour l'image courante !\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return;
  }
  if ( NULL == image_courante->infos_affichage->coupe ) { /* Pas de coupe */
    TRC_Z_Message( fenetre, x_zone, y_zone, larg_zone, haut_zone, txt_msg_pas_de_coupe );
    return;
  }
  coupe = image_courante->infos_affichage->coupe;

  /* 3. L'axe gradu horizontal pour la coupe */
  TRC_Couleur( NOIR );
  TRC_Style( BI_CONTINU );
  TRC_LigneH( fenetre, 2, fenetre->w_haut - 2, fenetre->w_larg - 4 );

  /* Et on le gradue */
  TRC_gradue( fenetre, 2, fenetre->w_haut - 2, fenetre->w_larg - 4, fenetre->w_haut - 2,
	      coupe->x_min, coupe->x_max, GRADUE_DECIMAL, -1, 10, FAUX );

  /* 4. L'axe gradu vertical pour la coupe */
  TRC_LigneV( fenetre, 50, y_zone, haut_zone - 4 );

  /* Et on le gradue */
  TRC_gradue( fenetre, 50, y_zone + haut_zone - 2, 50, y_zone + 2, coupe->y_min, coupe->y_max, GRADUE_DECIMAL, 0, 10, VRAI );

  /* Tout  la fin, la coupe : pour ne pas qu'elle soit masque par les lgendes */
  TRC_GrapheDansRectangle( fenetre, coupe, VERT, 1, x_zone, y_zone, larg_zone, haut_zone );

#ifdef ancien
  /* Grille linaire ou ponctuelle, si demande */
  if ( OBJ_choisit(CHOIX_IMAGE, LI_GRILLE_LIGNES, QUESTION) == VRAI ) {
    /* Trac des colonnes de la grille - en rouge, pour tre mieux vu sur le fond noir */
    vsl_color(_PRG_station->handle, ROUGE_FONCE);
    if (l_case > 4) { /* Sinon, on ne voit plus rien ! */
      points[1] = fenetre->w_y + 3 + h_case/2.0;
      points[3] = points[1] + haut_image - h_case;
      for (i = 0 ; i < image_courante->larg ; i++) {
	points[0] = fenetre->w_x + 3 + (i + 0.5) * l_case;
	points[2] = points[0];
	v_pline(_PRG_station->handle, 2, points);
      }
    }

    /* Trac des lignes de la grille */
    if (h_case > 4) { /* Sinon, grille trop serre */
      points[0] = fenetre->w_x + 3 + l_case/2.0;
      points[2] = points[0] + larg_image - l_case;
      for (i = 0 ; i < image_courante->haut ; i++) {
	points[1] = fenetre->w_y + 3 + (i + 0.5) * h_case;
	points[3] = points[1];
	v_pline(_PRG_station->handle, 2, points);
      }
    }
  } else if (OBJ_choisit(CHOIX_IMAGE, LI_GRILLE_POINTS, QUESTION) == VRAI ) {
    vsl_color(_PRG_station->handle, VERT_FONCE);

    if ( (l_case > 5) && (h_case > 5) ) {/* Sinon, trop petit : sans intrt */
      for (i = 0 ; i < image_courante->larg ; i++) {
	points[0] = fenetre->w_x + 3.0 + (i + 0.5) * l_case;

	for (j = 0 ; j < image_courante->haut ; j++) {
	  points[1] = fenetre->w_y + 3.0 + (j + 0.5) * h_case;
	  v_circle(_PRG_station->handle, points[0], points[1], 1);
	}
      }
    }
  }
#endif
}

/* -------- Redessin de la fentre 3D -------- */

static void RDS_img3d(Fenetre *fenetre, int x __attribute__((__unused__)), int y __attribute__((__unused__)), unsigned int larg __attribute__((__unused__)), unsigned int haut __attribute__((__unused__)))
{
#ifdef AVEC_OPEN_GL_
  Image   *image;
  float    vue_l, vue_h;
  float    cntr_x, cntr_y, cntr_z;
  const float largeur_vue = 20.0;
  float    coef_m2gl;
  float    coef_haut, off_haut;
  GLfloat *prev_line;
  unsigned int xi, yi;
  float    f, prev_ecr_y;
  double   *valeurs;
  struct {
    GLfloat n[3];
  }       *Normales;
  static struct {
    GLfloat Position[4];
    GLfloat Diffuse[4];
    GLfloat Ambiante[4];
  } lumiere = {
    { -1.0, 1.0, 1.0, 0.0 }, /* Directive ( != Position ) */
    { 1.0, 1.0, 1.0, 1.0 }, /* Blanche */
    { 0.4, 0.5, 0.4, 1.0 }
  };

#ifdef _DEVELOPPE_
/*   printf("RDS_img3d : fenetre = %p, x = %d, y = %d, larg = %u, haut = %u\n", */
/* 	 (void *) fenetre, x, y, larg, haut); */
#endif

  if ( (image = image_courante) == NULL )
    return;
  if ( IMG_GENRE_COMPO == image->genre ) return; /* Pas de dessin pour les images / composition */
  if ( image->genre != IMG_GENRE_REELLE ) {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Image  valeurs non-relles\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  if ( (image->l_reel <= 0.0) || (image->h_reel <= 0.0) ) {
    (void) fprintf(stderr, "RDS_img3d : image avec des dimensions de case absurdes [l = %g, h = %g : %s]\n",
		   image->l_reel, image->h_reel, image->nom);
    return; /* Sinon, div par zro aprs ==> Nan ou Inf => le serveur X explose (!) */
  }

  prev_line = malloc(sizeof(GLfloat) * image->larg);
  if (prev_line == NULL) {
    (void) fprintf(stderr, "RDS_img3d : mmoire insuffisante pour rserver %u lignes\n", image->larg);
    return;
  }
  Normales = malloc(sizeof(*Normales) * image->larg);
  if (Normales == NULL) {
    (void) fprintf(stderr, "RDS_img3d : mmoire insuffisante pour rserver %u normales\n", image->larg);
    free(prev_line); /* EC -> CC : Attention  bien librer la mmoire ! */
    return;
  }
  /* Etape 1: Definition du mode de dessin */
  glClearColor(1.0, 1.0, 1.0, 0.0);
  glShadeModel( (dessin_smooth==FAUX)?(GL_FLAT):(GL_SMOOTH) );
  glPolygonMode(GL_FRONT_AND_BACK, (cam_carto.fil_de_fer==FAUX)?(GL_FILL):(GL_LINE) );
  glEnable(GL_DEPTH_TEST);
  glLineWidth(1.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
  glEnable(GL_COLOR_MATERIAL);

  /* Etape 2: placement de la lumire */
  glLoadIdentity();
  glLightfv(GL_LIGHT0, GL_AMBIENT,  lumiere.Ambiante);
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  lumiere.Diffuse );
  glLightfv(GL_LIGHT0, GL_POSITION, lumiere.Position);
  glEnable(GL_LIGHT0);

  /* Etape 3: Dfinition de la projection */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (fenetre->w_larg < fenetre->w_haut) { /* Problme: garder l'aspect */
    vue_l = largeur_vue / 2.0;
    vue_h = vue_l * fenetre->w_haut; vue_h /= fenetre->w_larg;
    if (vue_h < 2.0) vue_h = 2.0;
  } else {
    vue_h = largeur_vue / 2.0;
    vue_l = vue_h * fenetre->w_larg; vue_l /= fenetre->w_haut;
    if (vue_l < 2.0) vue_l = 2.0;
  }
  glFrustum(-vue_l, vue_l, -vue_h, vue_h, 50.0, 110.0);

  /* Etape 4: On place l'oeil */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0, 0, -80.0 - cam_carto.distance);
  glRotatef(cam_carto.lon, 0.0, 1.0, 0.0);
  glRotatef(cam_carto.lat, 1.0, 0.0, 0.0);
  glRotatef(cam_carto.inc, 0.0, 0.0, 1.0);
  glScalef(cam_carto.echelle, cam_carto.echelle, cam_carto.echelle);

  /* Etape 5: Calcul des coefficients d'adaptation */
  coef_m2gl = largeur_vue / image->l_reel; coef_m2gl /= image->larg-1;
  /*   m2gl = coef de Microns vers l'unit OpenGL */
  if (image->val_min < image->val_max)
    coef_haut = 16.0 / (image->val_max - image->val_min);
  else
    coef_haut = 2.0;
  off_haut = (image->val_min + image->val_max) / 2.0; off_haut *= coef_haut;

  /* Etape 6: Trac des axes */
  glDisable(GL_LIGHTING);
  glColor3ub(184, 0, 32);
  glNormal3f(0.0, 1.0, 0.0);
  cntr_x = - image->x_min + (image->l_reel/2.0); cntr_x *= coef_m2gl; cntr_x -= (largeur_vue / 2.0);
  cntr_y = IMG_lit_valeur(image, 0.0, 0.0, IMG_INTERPOLE_BILINEAIRE, NULL);
  cntr_y *= coef_haut; cntr_y -= off_haut; cntr_y += 0.1;
  cntr_z = image->y_min - (image->h_reel/2.0); cntr_z *= coef_m2gl; cntr_z += (largeur_vue / 2.0);
  glBegin(GL_LINES);
    glVertex3f( -50.0, cntr_y, cntr_z);
    glVertex3f(  50.0, cntr_y, cntr_z);

    glVertex3f(cntr_x,  -50.0, cntr_z);
    glVertex3f(cntr_x,   50.0, cntr_z);

    glVertex3f(cntr_x, cntr_y,  -50.0);
    glVertex3f(cntr_x, cntr_y,   50.0);
  glEnd();

  /* Etape 7: Trac de la 'montagne' */
  if (image->larg < 2) return; /* On ne trace pas s'il s'agit d'une simple ligne :-/ */
  if (image->haut < 2) return;

  glEnable(GL_LIGHTING);
  glColor3ub(32, 64, 212);
  valeurs = image->valeurs.R;
  for (xi=0;xi<image->larg;xi++) {
    f = *valeurs++;
    f *= coef_haut; f -= off_haut;
    prev_line[xi] = f;
  }

  glEnable(GL_NORMALIZE);

  prev_ecr_y = + (largeur_vue / 2.0) /* <==  INITIALISER CORRECTEMENT ! [Chang - en + par analogie  la suite] */;
  vue_l = image->l_reel * coef_m2gl;
  vue_h = image->h_reel * coef_m2gl;
  for (yi = /*0*/ 1 ; yi<image->haut ; yi++) { /* NB : 0 -> seg fault OpenGL */
    float py, ecr_x, ecr_y;
    float prev_f;

    py = yi * image->h_reel;
    ecr_y = - py * coef_m2gl; ecr_y += (largeur_vue / 2.0);
    py += image->y_min;
    prev_f = prev_line[0];

    glBegin(GL_QUAD_STRIP);
    for (xi=0;xi<image->larg;xi++) {
      f = *valeurs++;
      f *= coef_haut; f -= off_haut;

      ecr_x = xi * largeur_vue; ecr_x /= (image->larg - 1); ecr_x -= (largeur_vue / 2.0);
      /*       ***** A propos du vecteur normal... *****
	 on assimile le trac courant  la cration d'un quad :
	  0----1   +--X->  le point 2 est le point courant.
	  |    |   |       le point 1 est l'autre pour OpenGL:QUAD_STRIP
	  3----2   |Z      le point 3 sert pour le calcul de la normale

	  Pour calculer la normale, on pose les vecteurs :
	   ~ v1 : de [2] vers [1]
	   ~ v2 : de [2] vers [3]
	   ~ n : la 'normale', donne par le produit vectoriel v1 /\ v2
	  On obtient donc : (avec f la hauteur de la case)
	       | - ( f - prev_f ) * len_case_z
	   n = | len_case_z * len_case_x
	       | - ( f - prev_f ) * ( f - prev_line[xi] )
	  Comme ce vecteur est deja assez compliqu, on s'embetes pas avec sa
	  norme et on laise OpenGL le normaliser a notre place (GL_NORMALIZE)
       */
      if (yi>0) { /* EC : ce test me parat trs inutile... */
	glNormal3f(Normales[xi].n[0], Normales[xi].n[1], Normales[xi].n[2]);
      } else {
	glNormal3f( (prev_f - f) * vue_h,
		    vue_l * vue_h,
		    (prev_f - f) * (f - prev_line[xi]) ); /* EC : x -> xi ! */
      }
      glVertex3f(ecr_x, prev_line[xi], prev_ecr_y);

      glNormal3f( (prev_f - f) * vue_h,
		  vue_l * vue_h,
		  (prev_f - f) * (f - prev_line[xi]) ); /* EC : x -> xi ! */
      glVertex3f(ecr_x, f,             ecr_y);

      Normales[xi].n[0] = (prev_f - f) * vue_h;
      Normales[xi].n[1] = vue_l * vue_h;
      Normales[xi].n[2] = (prev_f - f) * (f - prev_line[xi]); /* EC : x -> xi ! */

      prev_line[xi] = f;
      prev_f = f;

    }
    glEnd();
    prev_ecr_y = ecr_y;
  }

  free(prev_line);
  free(Normales);

  /* Etape 7: On place une grille au sol */
  {
    float x, y, z, f;
    float max_x, min_z, max_z;

    y = -8.5;
    x = largeur_vue;  max_x = largeur_vue / 2.0;
    z = ((image->haut-1) * image->h_reel * coef_m2gl);
    min_z = - largeur_vue / 2.0; max_z = min_z + z;
    glDisable(GL_LIGHTING);

    glColor3ub(112, 132, 132);
    glBegin(GL_LINES);
    for (xi=0;xi<image->larg;xi++) {
      f  = xi * x;
      f /= image->larg-1;
      f -= max_x;
      glVertex3f(f, y, min_z);
      glVertex3f(f, y, max_z);
    }
    for (yi=0;yi<image->haut;yi++) {
      f  = yi * z;
      f /= image->haut-1;
      f += min_z;
      glVertex3f(-max_x, y, f);
      glVertex3f( max_x, y, f);
    }
    glEnd();
  }

  /* Etape 8: On trace la coupe */
  if ( NULL == image->infos_affichage ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos d'affichage [image %s]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom);
#endif
    return;
  }

  if ( image->infos_affichage->coupe != NULL ) {
    float vect_x, vect_y;
    int   nb_pts, i;
    float px, py;
    float off_x, off_y;

    glColor3ub(16, 208, 32);
    glLineWidth(2.0);
    vect_x = image->infos_affichage->xp_2 - image->infos_affichage->xp_1;
    vect_y = image->infos_affichage->yp_2 - image->infos_affichage->yp_1;

    /* nombre de poinst a prendre : on fait [ norme(vect_coupe) / norme(diag_une_case) ]
       et on multiplie par racine(8) pour avoir un peu plus de points, donc de prcision.
       -- le choix n'est pas fait (totalement) au hasard, il permet d'optimiser le nombre
       d'operations ncessaires pour les calculs :) -- */
    f  =        (vect_x * vect_x)        +        (vect_y * vect_y);
    f /= (image->l_reel * image->l_reel) + (image->h_reel * image->h_reel);
    nb_pts = sqrt(8.0 * f);
    glBegin(GL_LINE_STRIP);
    off_x = -image->x_min - image->l_reel/2.0;
    off_y = -image->y_min - image->h_reel/2.0;
    for (i = 0 ; i <= nb_pts ; i++) {
      px = vect_x * i; px /= nb_pts; px += image->infos_affichage->xp_1;
      py = vect_y * i; py /= nb_pts; py += image->infos_affichage->yp_1;
      f = IMG_lit_valeur(image, px, py, IMG_INTERPOLE_BILINEAIRE, NULL);
      px += off_x; px *= coef_m2gl; px -= largeur_vue/2.0;
      py += off_y; py *= coef_m2gl; py -= largeur_vue/2.0;
      f  *= coef_haut; f -= off_haut; f += 0.2;
      glVertex3f(px, f, py);
    }
    glEnd();
  }
#else
  TRC_Message( fenetre, "Support OpenGL absent !" );
#endif /* AVEC_OPEN_GL_ */
}

/* -------- Gestion de l'ascenseur, vertical -------- */

static void ASC_image(Fenetre *fenetre, int type, int ss_type, int position __attribute__((__unused__)))
{
  Image *nv_image;
  Serie_Images *serie;
  unsigned int compteur;

  if (fenetre == NULL) return;
  if ( NULL == IMG_serie_courante ) return; /* Aucune srie -> ascenseurs inutiles ! */

  if (type == WM_ARROWED) { /* Utilisation des flches */
    switch (ss_type)
    {
      /* ---- Ascenseur vertical : changement d'image dans la mme srie ---- */
    case WA_UPPAGE: /* 10 images avant */
      OPTION_A_FAIRE( "Remonter de 10 images !" );
      break;

    case WA_DNPAGE: /* 10 images aprs */
      compteur = 0;
      nv_image = image_courante;
      if ( NULL == nv_image ) return; /* Srie vide */
      while (compteur < 10) {
	if (nv_image->suivante == NULL) /* On est sur la dernire */
	  break;
	nv_image = nv_image->suivante;
	compteur++;
      }
      IMG_AllerImage(nv_image);
      break;

    case WA_UPLINE: /* Image prcdente */
      if ( image_courante != IMG_serie_courante->premiere_image ) {
	nv_image = IMG_serie_courante->premiere_image;
	if ( NULL == nv_image ) return; /* Srie vide */
	while (nv_image->suivante != image_courante)
	  nv_image = nv_image->suivante;
	IMG_AllerImage(nv_image);
      }
      break;

    case WA_DNLINE: /* Image suivante */
      if ( image_courante != NULL ) {
	IMG_AllerImage( image_courante->suivante );
      }
      break;

      /* ------ Ascenseur horizontal : passage d'une srie  l'autre */
    case WA_LFPAGE: /* 10 sries prcdentes */
      serie = IMG_serie_courante;
      for (compteur = 0 ; compteur < 10 ; compteur++) {
	if (serie != IMG_premiere_serie) {
	  Serie_Images *s;
	  s = IMG_premiere_serie;
	  while (s->suivante != serie)
	    s = s->suivante;
	  serie = s;
	}
      }
      IMG_AllerSerie( serie );
      break;

    case WA_RTPAGE: /* 10 sries suivantes */
      compteur = 0;
      serie = IMG_serie_courante;
      while ( (serie->suivante != NULL) && (compteur < 10) ) {
	serie = serie->suivante;
	compteur++;
      }
      IMG_AllerSerie( serie );
      break;

    case WA_LFLINE: /* Srie prcdente */
      if ( IMG_serie_courante != IMG_premiere_serie) {
	serie = IMG_premiere_serie;
	while ( serie->suivante != IMG_serie_courante )
	  serie = serie->suivante;
	IMG_AllerSerie( serie );
      }
      break;

    case WA_RTLINE: /* Srie suivante */
      IMG_AllerSerie( IMG_serie_courante->suivante);
      break;
    }
  } else if (type == WM_VSLID) {/* Ascenseur vertical dplac */
    OPTION_A_FAIRE( "Glissement ascenseur V" );
  } else if (type == WM_HSLID) { /* Ascenseur horizontal dplac */
    OPTION_A_FAIRE( "Glissement ascenseur H" );
  }
}

/* -------- Gestion d'un clic sur l'image -------- */

static int CLC_image(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{
  double echelle, x_souris, y_souris;
  unsigned int larg_image, haut_image;
  int x_image, y_image;
  int retour;
  DES_image *dessin_image;
  Image *image;

  if ( fenetre == NULL ) return BI_EVENEMENT_ERREUR;
  if ( NULL == IMG_serie_courante ) return BI_EVENEMENT_TRAITE; /* Aucune srie -> rien  faire */

  if ( 2 == bouton ) { /* Bouton le plus  droite : menu, fonctionne toujours */
    IMG_gere_menu();
    return BI_EVENEMENT_TRAITE; 
  }

  if ( ( fenetre->w_larg < 16 ) || ( fenetre->w_haut < 16 ) ) {
    return BI_EVENEMENT_TRAITE; /* Trop petite pour tre gre */
  }

  /* On rcupre les infos de dessin */
  dessin_image = ( DES_image * ) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( NULL == dessin_image ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos d'affichage\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }
  if ( dessin_image->image != image_courante ) { 
    (void) fprintf(stderr, "%s,%s l.%u : image affiche [%s] != image courante [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, dessin_image->image->nom, image_courante->nom);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }
  image = dessin_image->image;

  if ( 2 == n_clic ) {
    printf("CLC_image :  FAIRE, clic multiple --- n_clic = %d\n", n_clic);
  }
  if ( etat != 0 ) {
    printf("CLC_image :  FAIRE, touche-clic --- etat = %d\n", etat);
  }

  /* Agencement de la fentre : la moiti haute affiche l'image */
  x_image = fenetre->w_x + 2; y_image = fenetre->w_y + 2;
  larg_image = fenetre->w_larg - 4;
  haut_image = (fenetre->w_haut / 2 ) - 4;

  /* Clic en-dehors : sans effet ! */
  if ( x < x_image ) return BI_EVENEMENT_TRAITE; if ( y < y_image ) return BI_EVENEMENT_TRAITE;
  if ( x >= x_image + (int) larg_image ) return BI_EVENEMENT_TRAITE; if ( y >= y_image + (int) haut_image ) return BI_EVENEMENT_TRAITE;

  /* chelle utilise */
  echelle = TRC_Echelle2D( dessin_image->x_max - dessin_image->x_min, dessin_image->y_max - dessin_image->y_min, larg_image, haut_image );
  if ( echelle <= 0.0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_Echelle2D() [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, echelle);
#endif
    return BI_EVENEMENT_TRAITE;
  }

  /* Coordonnes de la souris relative  l'image, en microns, relative  l'origine de l'image */
  retour = TRC_EcranVersAbsolu( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
				x, y, &x_souris, &y_souris);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_EcranVersAbsolu() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return BI_EVENEMENT_TRAITE;
  }

  if ( 4 == bouton ) { /* Bouton le plus  droite : placer le centre du repre = le point (0,0) */
    if ( image->infos_affichage != NULL ) {
      image->infos_affichage->xp_1 -= x_souris; /* La coupe ne bouge pas ! */
      image->infos_affichage->yp_1 -= y_souris;
      image->infos_affichage->xp_2 -= x_souris;
      image->infos_affichage->yp_2 -= y_souris;
    }

    image->x_min -= x_souris;
    image->y_min -= y_souris;
    FEN_redessin( fenetre, BI_RDS_TOUT );
    return BI_EVENEMENT_TRAITE;
  }

  if ( bouton == 1 ) { /* Placer l'axe de coupe */
    struct aff_image *infos;

    infos = image->infos_affichage;
    if ( NULL == infos ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos d'affichage ==> pas de coupe\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return BI_EVENEMENT_TRAITE;
    }

    if ( NULL == infos->coupe ) { /* Pas encore de coupe */
      if ( ( infos->xp_1 == infos->xp_2) && ( infos->yp_1 == infos->yp_2 ) ) { /* Premier point */
	infos->xp_1 = x_souris;
	infos->yp_1 = y_souris;
      } else { /* On place le second point */
	infos->xp_2 = x_souris;
	infos->yp_2 = y_souris;
	infos->coupe = IMG_coupe( image, infos->xp_1, infos->yp_1, x_souris, y_souris, 0.1);
	FEN_redessin(fenetre, BI_RDS_TOUT);
      }
    } else {
      /* On essaye de dplacer les points */
      int x_mk_1, y_mk_1, x_mk_2, y_mk_2;
      boolean nv_coupe = FAUX;

      TRC_AbsoluVersEcran( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
			   infos->xp_1, infos->yp_1, &x_mk_1, &y_mk_1);
      TRC_AbsoluVersEcran( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
			   infos->xp_2, infos->yp_2, &x_mk_2, &y_mk_2);

      if ( (x > x_mk_1 - 5) && (x < x_mk_1 + 5) && /* On dplace le premier point */
	   (y > y_mk_1 - 5) && (y < y_mk_1 + 5) ) {
	retour = prepare_ligne(fenetre, x_mk_2, y_mk_2, &x_mk_1, &y_mk_1);
	if ( retour == VRAI ) {
	  retour = TRC_EcranVersAbsolu( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
					x_mk_1, y_mk_1, &(infos->xp_1), &(infos->yp_1) );
	  if ( retour != 0 ) {
	    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_EcranVersAbsolu() [code %d]\n",
			   __FILE__, __FUNCTION__, __LINE__, retour);
	  }
	  nv_coupe = VRAI;
	}
      } else if ( (x > x_mk_2 - 5) && (x < x_mk_2 + 5) && /* On dplace le second point */
		  (y > y_mk_2 - 5) && (y < y_mk_2 + 5) ) {
	retour = prepare_ligne(fenetre, x_mk_1, y_mk_1, &x_mk_2, &y_mk_2);
	if (retour == VRAI) {
	  retour = TRC_EcranVersAbsolu( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
					x_mk_2, y_mk_2, &(infos->xp_2), &(infos->yp_2) );
	  if ( retour != 0 ) {
	    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_EcranVersAbsolu() [code %d]\n",
			   __FILE__, __FUNCTION__, __LINE__, retour);
	  }
	  nv_coupe = VRAI;
	}
      }

      if ( VRAI == nv_coupe ) { /* On trace la nouvelle coupe, en effaant l'ancienne */
	Graphe *inter;

	inter = infos->coupe;
	infos->coupe = NULL; /* En effaant, on n'efface pas les points de la coupe... */
	GRF_efface( inter, VRAI, FAUX );
	infos->coupe = IMG_coupe( image, infos->xp_1, infos->yp_1, infos->xp_2, infos->yp_2, 0.1);
	FEN_redessin( fenetre, BI_RDS_TOUT );
      }
    }
  }

  return BI_EVENEMENT_TRAITE; /* Trait correctement */
}

/* -------- Clic sur l'image, partie Open-GL -------- */

static int CLC_img3d(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{
#ifdef _DEVELOPPE_
  printf("CLC_img3d : fenetre = %p, bouton = %d, n_clic = %d, etat = %d, x = %d, y = %d\n",
	 (void *) fenetre, bouton, n_clic, etat, x, y);
#endif

  return BI_EVENEMENT_TRAITE; /* Pour l'instant, sert juste  bloquer le clic sur la partie OpenGL */
}

/* -------- Gestion du clavier sur la fentre des images --------

 C = proposer un centre pour l'image
*/

static int CLV_image(Fenetre *fenetre, unsigned int touche, int etat)
{
  Fenetre *mere;

  if ( NULL == image_courante ) return BI_EVENEMENT_TRAITE; /* Rien  faire */

  /* On rcupre la fentre mre, pour tre sr de bien redessiner si besoin... */
  mere = FEN_FenetreMere( fenetre );

  switch (touche)
  {
  case 'a': /* Aucune grille sur l'image */
    if ( image_courante->infos_affichage != NULL ) {
      /* On enlve la grille & les points */
      image_courante->infos_affichage->proprietes &= ~(IMG_AVEC_GRILLE|IMG_AVEC_POINTS);

      /* On redessine pour voir le rsultat */
      FEN_redessin( fenetre, BI_RDS_UTIL );
    }
    return BI_EVENEMENT_TRAITE;

  case 'B': /* Importer une image PIXE de Bordeaux */
    IMG_AllerImage( PIXE_Bordeaux() );
    return BI_EVENEMENT_TRAITE;

  case 'C': /* Proposer un centre pour l'image */
    OPTION_A_FAIRE( "Proposer un centre pour l'image = translater la rgion 2D du plan" );
    return BI_EVENEMENT_TRAITE;

  case 'd': /* Decouper l'image */
    IMG_decouper(image_courante);
    return BI_EVENEMENT_TRAITE;

  case 'e': /* Corriger les effets d'epaisseur */
    IMG_corrige_epaisseur(image_courante);
    return BI_EVENEMENT_TRAITE;

  case 'g': /* Passer en niveaux de gris */
    image_courante->infos_affichage->couleur = BI_DEGRADE_GRIS;
    FEN_redessin( mere, BI_RDS_UTIL );
    return BI_EVENEMENT_TRAITE;

  case 'i': /* Interpoler une image */
    IMG_AllerImage( IMG_lissage( image_courante ) );
    return BI_EVENEMENT_TRAITE;

#ifdef AVEC_PNG_
  case 'I': /* On sauve une image de la vue en 3D */
    sv_image(fenetre);
    return BI_EVENEMENT_TRAITE;
#endif

  case 'r': /* Rotation */
    IMG_tourner(image_courante);
    return BI_EVENEMENT_TRAITE;

  case 's': /* Sauver */
    I_IMG_Sauver( image_courante );
    return BI_EVENEMENT_TRAITE;

  case 't': /* Changer la tranche pour une image de carto complexe */
    if ( IMG_GENRE_GRAPHE == image_courante->genre ) {
      IMG_choisit_tranche(image_courante);
    }
    return BI_EVENEMENT_TRAITE;

  case 127:
    I_IMG_Effacer( image_courante, etat & K_CTRL );
    return BI_EVENEMENT_TRAITE;

  case Fleche_Haut:
    ASC_image(fenetre, WM_ARROWED, WA_UPLINE, 0);
    return BI_EVENEMENT_TRAITE;

  case Fleche_Bas:
    ASC_image(fenetre, WM_ARROWED, WA_DNLINE, 0);
    return BI_EVENEMENT_TRAITE;

#ifdef AVEC_OPEN_GL_
  /* Gestion de la camera de la vue OpenGL */
  case '+': cam_carto.echelle += 0.1; FEN_redessin( mere, BI_RDS_GL); break;
  case '-': cam_carto.echelle -= 0.1; FEN_redessin( mere, BI_RDS_GL); break;

  case '6': cam_carto.lon += 5.0; FEN_redessin( mere, BI_RDS_GL); break;
  case '4': cam_carto.lon -= 5.0; FEN_redessin( mere, BI_RDS_GL); break;

  case '2': cam_carto.lat += 5.0; FEN_redessin( mere, BI_RDS_GL); break;
  case '8': cam_carto.lat -= 5.0; FEN_redessin( mere, BI_RDS_GL); break;

  case '1': cam_carto.inc += 5.0; FEN_redessin( mere, BI_RDS_GL); break;
  case '9': cam_carto.inc -= 5.0; FEN_redessin( mere, BI_RDS_GL); break;

  case '7': cam_carto.distance -= 1.0; FEN_redessin( mere, BI_RDS_GL); break;
  case '3': cam_carto.distance += 1.0; FEN_redessin( mere, BI_RDS_GL); break;

  case 'F':
    cam_carto.fil_de_fer = (cam_carto.fil_de_fer == FAUX)?(VRAI):(FAUX);
    FEN_redessin( mere, BI_RDS_GL);
    break;
  case 'S':
    dessin_smooth = (dessin_smooth == FAUX)?(VRAI):(FAUX);
    FEN_redessin( mere, BI_RDS_GL);
    break;
#endif
  }

  return BI_EVENEMENT_NON_TRAITE; /* Non trait */
}

/* -------- Gestion des messages pour la fentre -------- */

static int MSG_image(Fenetre *fenetre, Message *message)
{
  DES_image *des_image;

  des_image = ( DES_image * ) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );

  switch ( message->type )
  {
  case WM_CLOSED: /* On ferme la fentre */
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, NULL );
    FEN_Detruire( fenetre );

    free( des_image ); des_image = NULL;

    fen_images = NULL; /* La fentre est ferme */
    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non trait [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion du pointeur de la souris -------- */

#define EXREGION_DEHORS  0
#define EXREGION_COUPE   1
#define EXREGION_IMAGE   2
#define EXREGION_ECHELLE 3

static void RCT_image(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  char *infos;
  unsigned int larg_image, haut_image, larg_coupe, haut_coupe, larg_echelle, haut_echelle;
  int x_image, y_image, x_coupe, y_coupe, x_echelle, y_echelle;
  int ex_region = EXREGION_DEHORS;
  DES_image *dessin_image;
  Graphe *coupe = NULL;
  Image *image;

  if (image_courante == NULL) return;

  dessin_image = ( DES_image * ) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( NULL == dessin_image ) {
    /* Arrive quand on cre la fentre : peut tre appel avant d'avoir pu associer la structure... */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, pas d'infos pour le trac des images !\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return; /* Il n'y a alors rien  faire */
  }
  if ( dessin_image->image != image_courante ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, Incohrence image courante / image associe  la fentre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  image = dessin_image->image;
  if ( image->infos_affichage != NULL ) {
    coupe = image->infos_affichage->coupe;
  }

  /* Utilisable si 16 * 16 pixels minimum */
  if ( ( fenetre->w_larg < 16 ) || ( fenetre->w_haut < 16 ) ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Fentre trop petite\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return;
  }

  /* Zoom trop grand -> pas de rticule */
  if ( ( dessin_image->x_max == dessin_image->x_min ) || ( dessin_image->y_max == dessin_image->y_min ) ) {
    return;
  }

  /* Dcoupage de la fentre en zones */
  /* a. Moiti haute : image */
  x_image = 2; y_image = 2;
  larg_image = fenetre->w_larg - 4; haut_image = (fenetre->w_haut / 2) - 4;

  /* b. Un tiers de la moiti base : chelle */
  x_echelle = 2; y_echelle = (fenetre->w_haut / 2) + 2;
  larg_echelle = fenetre->w_larg - 4;
  haut_echelle = fenetre->w_haut / 6; if ( haut_echelle > 4 ) haut_echelle -= 4; else haut_echelle = 0;

  /* c. Deux-tiers bas de la moiti basse : coupe */
  if ( NULL == coupe ) { /* On ne peut pas tre sur la coupe s'il n'y en a pas ! */
    x_coupe = -50; y_coupe = -50;
    larg_coupe = 0; haut_coupe = 0;
  } else {
    x_coupe = 2;
    y_coupe = fenetre->w_haut / 2 + fenetre->w_haut / 6 + 2;
    larg_coupe = fenetre->w_larg - 4;
    haut_coupe = (fenetre->w_haut - y_coupe) - 2;
  }

  TRC_Mode( BI_MODE_INVERSE );
  TRC_Style( BI_CONTINU );
  TRC_Epaisseur( 0 );

  /* On efface l'ancien rticule, si ncessaire */
  if ( ( vx_x >= x_image ) && ( vx_x < x_image + (int) larg_image ) &&
       ( vx_y >= y_image ) && ( vx_y < y_image + (int) haut_image ) ) {
    TRC_Couleur( ROUGE ); /* Pointeur rouge pour augmenter le contraste sur l'image */

    TRC_LigneV( fenetre, vx_x, y_image, haut_image );
    TRC_LigneH( fenetre, x_image, vx_y, larg_image );

    ex_region = EXREGION_IMAGE;
  }
  if ( ( vx_x >= x_echelle ) && ( vx_x < x_echelle + (int) larg_echelle ) &&
       ( vx_y >= y_echelle ) && ( vx_y < y_echelle + (int) haut_echelle ) ) {
    TRC_Couleur( ROUGE ); /* Pointeur rouge pour augmenter le contraste sur l'chelle */

    if ( IMG_GENRE_REELLE == image->genre ) {
      TRC_LigneV( fenetre, vx_x, y_echelle, haut_echelle * 0.66666 + 10 ); /* Les couleurs n'occupent que les deux-tiers, puis l'axe */
    } else if ( IMG_GENRE_COMPO == image->genre ) {
    } else if ( IMG_GENRE_GRAPHE == image->genre ) {
    } else {
      OPTION_A_FAIRE( "Rticule dans l'chelle pour type d'image autre que R/Composition" );
    }

    ex_region = EXREGION_ECHELLE;
  }
  if ( ( vx_x >= x_coupe ) && ( vx_x < x_coupe + (int) larg_coupe ) &&
       ( vx_y >= y_coupe ) && ( vx_y < y_coupe + (int) haut_coupe ) ) {
    TRC_Couleur( NOIR ); /* Pointeur classique */

    TRC_LigneV( fenetre, vx_x, y_coupe, haut_coupe );
    TRC_LigneH( fenetre, x_coupe, vx_y, larg_coupe );

    ex_region = EXREGION_COUPE;
  }

  /* On met le nouveau rticule, si ncessaire */
  if ( ( x >= x_image ) && ( x < x_image + (int) larg_image ) &&
       ( y >= y_image ) && ( y < y_image + (int) haut_image ) ) {
    double echelle;
    double x_souris, y_souris;
    double valeur, valeur_interpolee;
    int retour;

    /* Dessin du rticule */
    TRC_Couleur( ROUGE ); /* Pointeur rouge pour augmenter le contraste sur l'image */

    TRC_LigneV( fenetre, x, y_image, haut_image );
    TRC_LigneH( fenetre, x_image, y, larg_image );

    /* Quelle chelle utiliser ? */
    echelle = TRC_Echelle2D( dessin_image->x_max - dessin_image->x_min, dessin_image->y_max - dessin_image->y_min, larg_image, haut_image );
    if ( echelle <= 0.0 ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_Echelle2D() [%g]\n",
		     __FILE__, __FUNCTION__, __LINE__, echelle);
#endif
      return;
    }

    /* Coordonnes du point sous la souris */
    retour = TRC_EcranVersAbsolu( x_image, y_image, larg_image, haut_image, echelle, dessin_image->x_min, dessin_image->y_max,
				  x, y, &x_souris, &y_souris);
    if ( retour < 0 ) { /* Erreur grave seulement ! */
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans TRC_EcranVersAbsolu() [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }

    /* Valeur Z=f(X,Y) en ce point */
    if ( IMG_GENRE_REELLE == image->genre ) {
      valeur = IMG_lit_valeur( image, x_souris, y_souris, IMG_LIT_SIMPLE, NULL );
      valeur_interpolee = IMG_lit_valeur( image, x_souris, y_souris, IMG_INTERPOLE_BILINEAIRE, NULL );

      /* On affiche les infos */
      infos = TXT_genere_chaine( txt_infos_dans_image_R,
				 (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
				 image->l_reel, image->h_reel, x_souris, y_souris, valeur, valeur_interpolee);
    } else if ( IMG_GENRE_COMPO == image->genre ) {
      struct matiere *composition;
      int retour;
      unsigned long numero_case;

      retour = IMG_XY_v_N( image, x_souris, y_souris, &numero_case );
      if ( retour != 0 ) {
	infos = TXT_genere_chaine( txt_infos_dans_image_M,
				   (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
				   image->l_reel, image->h_reel, x_souris, y_souris, "[Hors image]" );
      } else {
	composition = image->valeurs.M[ numero_case ];
	if ( composition != NULL ) {
	  infos = TXT_genere_chaine( txt_infos_dans_image_M,
				     (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
				     image->l_reel, image->h_reel, x_souris, y_souris, composition->nom );
	} else {
	  infos = TXT_genere_chaine( txt_infos_dans_image_M,
				     (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
				     image->l_reel, image->h_reel, x_souris, y_souris, "Trou" );
	}
      }
    } else {
      infos = TXT_genere_chaine( txt_infos_dans_image_A, (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
				 image->l_reel, image->h_reel, x_souris, y_souris );
    }
    FEN_infos( fenetre, infos );
    free( infos ); infos = NULL;
  } else if ( ( x >= x_echelle ) && ( x < x_echelle + (int) larg_echelle ) &&
	      ( y >= y_echelle ) && ( y < y_echelle + (int) haut_echelle ) ) {
    double echelle_X;
    double x_reel;

    if ( IMG_GENRE_COMPO == image->genre ) return;  /* Sur une composition, le rticule ne sert  rien dans l'chelle ! */
    if ( IMG_GENRE_GRAPHE == image->genre ) return;  /* Sur une cartographie de graphes non plus ! */

    /* Dessin du rticule */
    TRC_Couleur( ROUGE ); /* Pointeur rouge pour augmenter le contraste sur l'chelle */
    TRC_LigneV( fenetre, x, y_echelle, haut_echelle * 0.66666 + 10 ); /* Les couleurs n'occupent que les deux-tiers, puis l'axe */

    /* Coordonnes du point ss le rticule */
    echelle_X = (double) larg_echelle / ( image->val_max - image->val_min );
    x_reel = image->val_min + ( x - x_echelle ) / echelle_X;

    /* On affiche les infos */
    infos = TXT_genere_chaine( txt_infos_dans_echelle, x_reel );
    FEN_infos( fenetre, infos );
    free( infos ); infos = NULL;
  } else if ( ( x >= x_coupe ) && ( x < x_coupe + (int) larg_coupe ) &&
	      ( y >= y_coupe ) && ( y < y_coupe + (int) haut_coupe ) ) {
    double echelle_X, echelle_Y;
    double x_reel, y_reel;

    /* Dessin du rticule */
    TRC_Couleur( NOIR ); /* Pointeur classique */
    TRC_LigneV( fenetre, x, y_coupe, haut_coupe );
    TRC_LigneH( fenetre, x_coupe, y, larg_coupe );

    /* Coordonnes du point ss le rticule */
    echelle_X = (double) larg_coupe / (coupe->x_max - coupe->x_min);
    echelle_Y = (double) haut_coupe / (coupe->y_max - coupe->y_min);
    x_reel = coupe->x_min + ( x - x_coupe ) / echelle_X;
    y_reel = coupe->y_max - ( y - y_coupe ) / echelle_Y;

    /* On affiche les infos */
    infos = TXT_genere_chaine( txt_infos_dans_coupe, x_reel, y_reel );
    FEN_infos(fenetre, infos);
    free( infos ); infos = NULL;
  } else if ( ex_region != EXREGION_DEHORS ) { /* On est en-dehors, mais on vient de sortir */
    infos = TXT_genere_chaine( txt_infos_dehors,
			       (image->larg - 1) * image->l_reel, (image->haut - 1) * image->h_reel,
			       image->l_reel, image->h_reel);
    FEN_infos(fenetre, infos);
    free( infos ); infos = NULL;
  }
}

/* ~~~~~~~~ Sous-menu proposant diverses options ~~~~~~~~ */

static boolean IMG_gere_menu(void)
{
  Image *img_creee = NULL;
  Graphe *grf_cree = NULL;
  int option;

  if (NULL == image_courante) return FAUX;

  option = SS_menu_texte(menu_contextuel_images, MCI_NOMBRE_OPTIONS, txt_titre_ssmenu, -1, 0);
  if (option < 0) return FAUX;

  switch (option)
  {
  case MCI_INTERPOLER: /* Interpoler une image */
    img_creee = IMG_lissage(image_courante);
    break;

  case MCI_ROTATION: /* Faire tourner une image */
    img_creee = IMG_tourner(image_courante);
    break;

  case MCI_DECOUPER: /* Couper une portion de l'image */
    IMG_decouper(image_courante);
    break;

  case MCI_MIROIR_HV: /* Miroir horizontal ou vertical */
    {
      int reponse;

      reponse = BI_ERR_alerte(fen_images, &ERR_TypeMiroir, VRAI);
      if (1 == reponse) { /* Horizontal */
	img_creee = IMG_Miroir_Horizontal(image_courante, 0.0); /* Miroir / axe des X */
      } else if (2 == reponse) { /* Vertical */
	img_creee = IMG_Miroir_Vertical(image_courante, 0.0); /* Miroir / axe des Y */
      }
    }
    break;

  case MCI_PROFIL_RADIAL: /* Calculer un profil radial */
    grf_cree = IMG_profil(image_courante);
    break;

  default:
    return FAUX;
  }

  IMG_AllerImage(img_creee);
  GRF_aller(grf_cree);

  return VRAI;
}
