/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998-2002, Emmanuel CURIS - Cration de ce fichier: 23 juin 1998

   Module de lissage & inteprolation des graphes par diffrentes mthodes

   Interpolation / lissage d'un graphe quelconque (partie interface utilisateur)
   Source original rcupr de ../XANES.c
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"

#include "interface.h"
#include "lissage.h"
#include "serie.h" /* Pour quelle_serie() */

#include "lis_interpoler.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Routine d'appel gnrique ~-~-~-~-~-~-~-~-~ */

Graphe *GRF_Interpoler(Graphe *original, struct infos_lissage *comment)
{
  Graphe *resultat;

  if ( NULL == comment ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'informations de lissage indiques\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* Par dfaut, aucune erreur ! */
  comment->code_erreur = 0;

  if ( NULL == original ) {
    comment->code_erreur = ERR_LIS_AUCUN_GRAPHE;
    return NULL;
  }

  /* En fonction des mthodes, on choisit la bonne fonction */
  if ( VRAI == comment->table_fournie ) {
    /* On fournit une table des X */
    resultat = GRF_CalerSurTable( original, comment->nbr_points, comment->table_x, comment->methode, INTERPOLE_BORDS_0_0, &(comment->code_erreur) );
  } else {
    /* On fournit les informations pour construire une grille des X */
    resultat = GRF_CalerSurGrille( original, comment->methode, INTERPOLE_BORDS_0_0,
				   comment->x_min, comment->x_max, comment->nbr_points, &(comment->code_erreur) );
  }

  /* On renvoie le rsultat */
  return resultat;
}

/* ******** Ralisation effective du lissage ************************** */

/* -------- Lissage complet d'un spectre rel --------------------------\
 |  Renvoie le graphe liss (NULL si erreur)
 |
 | source    : le graphe  lisser [doit tre rel, tri, sans soublons] |
 | nv_points : le nombre de points dans le graphe liss                 |
 \--------------------------------------------------------------------- */

Graphe *lisse_complexe(Graphe *source, unsigned long nv_points)
{
  Graphe *resultat;
  int retour;
  Complexe *b, *c, *d; /* Coefficients pour le lissage */
  char *titre;
  unsigned long i, j = 0;
  register double *x_lisse, *x;
  register Complexe *y_lisse, *y;
  double pas;

  retour = CSPL_tt_prep(source, &b, &c, &d);
  if ( retour != 0 ) {
    LIS_ERREUR(retour, source->nom);
    return NULL;
  }

  /* On cre le nouveau graphe */
  titre = TXT_genere_chaine( "%s (interpol"e_AIGU")", source->nom);
  resultat = GRF_nouveau(source, nv_points, source->type, GRF_FCT_COMPLEXE, titre);
  free( titre ); titre = NULL;
  if (resultat == NULL) {
    free(b); free(c); free(d);
    return NULL;
  }

  /* On calcule le nouveau graphe - suppose que source est tri ! */

  x = source->x;
  y = (Complexe *) source->y;
  y_lisse = (Complexe *) resultat->y;

  /* Le premier point est identique... */

  resultat->x[0] = *x;
  *y_lisse = *y;

  x_lisse = resultat->x + 1;
  y_lisse++;

  pas = ( source->x_max - source->x_min ) / (nv_points - 1);

  for (i = 1 ; i < nv_points - 1 ; i++, x_lisse++, y_lisse++)
  {
    double x_apres, delta;

    *x_lisse = *(x_lisse - 1) + pas;

    if (j < source->nombre_point)
      x_apres = *(x + 1);
    else
      x_apres = *(x_lisse);

    while (*x_lisse > x_apres)
    {
      x++; j++; y++;
      if ( j >= source->nombre_point )
	x_apres = *x_lisse;
      else
	x_apres = *(x + 1);
    }

    delta = *x_lisse - *x;

    y_lisse->re  = delta * d[j].re;
    y_lisse->re += c[j].re;
    y_lisse->re *= delta;
    y_lisse->re += b[j].re;
    y_lisse->re *= delta;
    y_lisse->re += y->re;
    y_lisse->im  = delta * d[j].im;
    y_lisse->im += c[j].im;
    y_lisse->im *= delta;
    y_lisse->im += b[j].im;
    y_lisse->im *= delta;
    y_lisse->im += y->im;
  }

  /* Le dernier point est toujours conserv */

  *x_lisse = source->x_max;
  *y_lisse = *( ( (Complexe *) (source->y) ) + source->nombre_point );

  GRF_bornes(resultat, ORDONNE, DESORDONNE);

  free(b); free(c); free(d); /* Plus besoin d'eux */

  return resultat;
}

/* -------- Lissage de _tous_ les graphes d'une srie sur une table indique -------- */

int LIS_InterpolerSerie(struct serie *serie, struct infos_lissage *infos)
{
  unsigned long nombre_points;
  double *nv_table;
  struct graphe *graphe, *grf_interpole;
  int retour;

  /* Combien de points dans la table ? */
  if ( VRAI == infos->table_fournie ) {
    nombre_points = infos->nbr_points;
  } else {
    nombre_points = 1 + rint( ( infos->x_max - infos->x_min ) / ( infos->pas ) );
  }

  /* On rserve la table */
  nv_table = (double *) malloc( sizeof(double) * nombre_points );
  if ( NULL == nv_table ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de construire la nouvelle table !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On remplit la table */
  if ( VRAI == infos->table_fournie ) {
    memcpy( nv_table, infos->table_x, nombre_points * sizeof( double ) );
  } else {
    unsigned long i;

    for ( i = 0 ; i < nombre_points ; i++ ) {
      nv_table[ i ] = infos->x_min + infos->pas * (double) i;
    }
  }

  /* Pour chaque graphe de la srie convenable */
  for ( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    if ( graphe->x != serie->table_X ) continue;

    /* On cre un graphe vide isol de type srie */
    grf_interpole = GRF_nv_isole( nombre_points, graphe->type, GRF_SERIE );
    if ( NULL == grf_interpole ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer la version interpole !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      /* Ne pose pas de problme de continuer directement, mais le graphe non-modifi perdra ses X...
	 On ne peut pas les garder : la table est libre en fin de boucle.
	 Si on arrte, la table n'est pas libre, mais les graphes dj modifis auront une table flottante
	 ==> il y aurait des fuites de mmoire */
      continue;
    }

    /* On lui met comme X la table que l'on vient de crer */
    grf_interpole->x = nv_table;

    /* On remplit ce graphe avec la version interpole de l'original */
    retour = LIS_RecalerGraphe( graphe, grf_interpole, infos->methode, INTERPOLE_BORDS_Y_Y );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans le lissage [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      GRF_eff_isole( grf_interpole ); grf_interpole = NULL;
      /* Mme remarque que ci-dessus pour l'interruption prcoce */
      continue;
    }

    /* On adapte la taille du graphe dans la srie */
    retour = GRF_AdapteTaille( graphe, nombre_points );
    if ( FAUX == retour ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans l'adaptation de la taille [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      GRF_eff_isole( grf_interpole ); grf_interpole = NULL;
      /* Mme remarque que ci-dessus pour l'interruption prcoce */
      continue;
    }

    /* Les Y du isol ont les bonnes valeurs => on les copie dans la table des Y  la bonne taille */
    memcpy( graphe->y, grf_interpole->y, sizeof( double ) * grf_interpole->nombre_point );

    /* On met la bonne table des X */
    graphe->x = nv_table; grf_interpole->x = NULL;

    /* Plus besoin du graphe isol */
    GRF_eff_isole( grf_interpole ); grf_interpole = NULL;
  }

  /* On enlve l'ancienne table de la srie --- elle ne devrait plus tre utilise ! */
  Serie_EnleverTable( serie );

  /* On assigne la nouvelle */
  Serie_AssocierTable( serie, nombre_points, nv_table, FAUX );

  /* Tout s'est bien pass */
  return 0;
}
