/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2003, Emmanuel CURIS

   Oprations de base sur les matrices
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "systeme.h"

#include "matrices.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

/* -------- CL de colonnes : Ccible <- Ccible + coef2 * C2 -------- */

int MAT_CL_Colonnes(Matrice *matrice, unsigned int col_cible, unsigned int col_2, double coef_2)
{
  unsigned int i;
  double Ac, A2;

#ifdef _DEVELOPPE_
  if ( NULL == matrice ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une matrice NULLE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
  if ( ( col_cible >= matrice->n_colonnes ) || ( col_2 >= matrice->n_colonnes ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Index de colonnes absurdes [%u, %u --- %u colonnes]\n",
		   __FILE__, __FUNCTION__, __LINE__, col_cible, col_2, matrice->n_colonnes);
    abort();
  }
#endif

  if ( matrice->type != MATRICE_QUELCONQUE ) {
    int retour;

    retour = MAT_ChangerType( matrice, MATRICE_QUELCONQUE );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en obtenir une matrice quelconque\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }
  }

  for ( i = 0 ; i < matrice->n_lignes ; i++ ) {
    Ac = lit_element( matrice, i, col_cible );
    A2 = lit_element( matrice, i, col_2 );

    met_element( matrice, i, col_cible, Ac + coef_2 * A2 );
  }

  /* Calcul fini sans anicroche */
  return 0;
}

/* -------- CL de lignes : Lcible <- Lcible + coef2 * L2 -------- */

int MAT_CL_Lignes(Matrice *matrice, unsigned int lgn_cible, unsigned int lgn_2, double coef_2)
{
  unsigned int j;
  double Ac, A2;

#ifdef _DEVELOPPE_
  if ( NULL == matrice ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une matrice NULLE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
  if ( ( lgn_cible >= matrice->n_lignes ) || ( lgn_2 >= matrice->n_lignes ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Index de lignes absurdes [%u, %u --- %u lignes]\n",
		   __FILE__, __FUNCTION__, __LINE__, lgn_cible, lgn_2, matrice->n_lignes);
    abort();
  }
#endif

  if ( matrice->type != MATRICE_QUELCONQUE ) {
    int retour;

    retour = MAT_ChangerType( matrice, MATRICE_QUELCONQUE );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en obtenir une matrice quelconque\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }
  }

  for ( j = 0 ; j < matrice->n_colonnes ; j++ ) {
    Ac = lit_element( matrice, lgn_cible, j );
    A2 = lit_element( matrice, lgn_2, j );

    met_element( matrice, lgn_cible, j, Ac + coef_2 * A2 );
  }

  /* Calcul fini sans anicroche */
  return 0;
}

/* -------- Permuter deux colonnes ------ */

int MAT_EchangerColonnes(Matrice *matrice, unsigned int col_1, unsigned int col_2)
{
  unsigned int i;

  if ( col_1 == col_2 ) return 0; /* Rien  faire ! */

#ifdef _DEVELOPPE_
  if ( NULL == matrice ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une matrice NULLE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
  if ( ( col_1 >= matrice->n_colonnes ) || ( col_2 >= matrice->n_colonnes ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Index de colonnes absurdes [%u, %u --- %u colonnes]\n",
		   __FILE__, __FUNCTION__, __LINE__, col_1, col_2, matrice->n_colonnes);
    abort();
  }
#endif

  if ( matrice->type != MATRICE_QUELCONQUE ) {
    int retour;

    retour = MAT_ChangerType( matrice, MATRICE_QUELCONQUE );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en obtenir une matrice quelconque\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }
  }

  /* Permutation effective */
  for ( i = 0 ; i < matrice->n_lignes ; i++ ) {
    double tmp;

    tmp = lit_element( matrice, i, col_1 );
    met_element( matrice, i, col_1, lit_element( matrice, i, col_2 ) );
    met_element( matrice, i, col_2, tmp );
  }

  /* Tout s'est bien pass, joie ! */
  return 0;
}

/* -------- Permuter deux lignes ------ */

int MAT_EchangerLignes(Matrice *matrice, unsigned int lgn_1, unsigned int lgn_2)
{
  unsigned int i;

  if ( lgn_1 == lgn_2 ) return 0; /* Rien  faire ! */

#ifdef _DEVELOPPE_
  if ( NULL == matrice ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une matrice NULLE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
  if ( ( lgn_1 >= matrice->n_lignes ) || ( lgn_2 >= matrice->n_lignes ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Index de lignes absurdes [%u, %u --- %u lignes]\n",
		   __FILE__, __FUNCTION__, __LINE__, lgn_1, lgn_2, matrice->n_lignes);
    abort();
  }
#endif

  if ( matrice->type != MATRICE_QUELCONQUE ) {
    int retour;

    retour = MAT_ChangerType( matrice, MATRICE_QUELCONQUE );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en obtenir une matrice quelconque\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }
  }

  /* Permutation effective */
  for ( i = 0 ; i < matrice->n_colonnes ; i++ ) {
    double tmp;

    tmp = lit_element( matrice, lgn_1, i );
    met_element( matrice, lgn_1, i, lit_element( matrice, lgn_2, i ) );
    met_element( matrice, lgn_2, i, tmp );
  }

  /* Tout s'est bien pass, joie ! */
  return 0;
}
