/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Cration et gestion de modles en vue d'un ajustement

   Contrle de la validit & de la cohrence des modles avant de lancer l'ajustement
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "couches.h" /* Pour NC_... */
#include "modeles.h"
#include "decomposition.h"	/* Modles de dcomposition linaire sur une base de spectres */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation concrte ~~~~~ */

int MODELE_ControlerModele(ModeleAjustement *modele, boolean fin_ajustement)
{
  int retour = -1974;
  unsigned int i;

  if ( NULL == modele ) return -1;
  if ( NULL == modele->infos_modele.modeles ) {
    (void) fprintf( stderr, "%s,%s l.%u : pas d'informations complmentaires du modle\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  switch ( modele->type )
  {
  case MODELE_MULTI_MODELES: /* On vrifie chacun des sous-modles */
    for ( i = 0 ; i < modele->infos_modele.modeles->nbr_modeles ; i++ ) {
      retour = MODELE_ControlerModele( modele->infos_modele.modeles->modeles[ i ], fin_ajustement );
      if ( retour != 0 ) {
	(void) fprintf( stderr, "%s,%s l.%u : Erreur dans l'un des sous-modles [n %u / multi]\n",
			__FILE__, __FUNCTION__, __LINE__, i + 1 );
	return retour;
      }
    }
    break;

  case MODELE_CL_MODELES:
    for ( i = 0 ; i < modele->infos_modele.cl_modeles->nbr_termes ; i++ ) {
      retour = MODELE_ControlerModele( modele->infos_modele.cl_modeles->modeles[ i ], fin_ajustement );
      if (retour != 0) {
	(void) fprintf( stderr, "%s,%s l.%u : Erreur dans l'un des sous-modles [n %u / CL]\n",
			__FILE__, __FUNCTION__, __LINE__, i + 1 );
	return retour;
      }
    }
    break;

  case MODELE_EXAFS_CHEMINS:
    /* Attention, cette tape compte aussi le nombre de variables rellement disponibles dans le modle */
    retour = NC_ControleModele( modele->infos_modele.feff,
			       (modele->x_min > modele->experience->x_min) ? (modele->x_min) : (modele->experience->x_min),
				&(modele->nbr_variables) ); 
    break;

  case MODELE_EXAFS_GNXAS:
  case MODELE_SEUIL_BASIQUE:
  case MODELE_FLUOX:
    AVERTISSEMENT( "type de modle pas encore support [%d]\n", modele->type );
    break;

  case MODELE_DECOMPOSITION:
    retour = MODELE_Decomposition_Controler( modele->infos_modele.decomposition );
    break;
 
  case MODELE_GENERIQUE:
    retour = MODELE_Generique_Controler( modele->infos_modele.generique );
    break;

  default:
    (void) fprintf( stderr, "%s : type de modle inconnu [%d]\n",
		    __FUNCTION__, modele->type );
    return +5; /* Suppose que c'est fourni par l'utilisateur... */
  }

  /* On renvoie ce qu'a detecte la methode */
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : ERREUR en controlant le modele [code : %d]\n",
		    __FUNCTION__, retour );
  }
  return retour;
}

