/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Cration et gestion de modles en vue d'un ajustement

   Calcul des drives par rapport aux variables ajustes
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "couches.h"
#include "feff.h"
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static double PG_CalculerDerivee(struct modele_ajuste *modele, unsigned int variable, double x);

/* ~~~~~~~~ Ralisation concrte ~~~~~ */

/* -------- Calcule la drive de la fonction modle par rapport  une variable ajuste --------\
 |   Renvoie la valeur de la drive (0.0 en cas d'erreur)                                      |
 |                                                                                              |
 | modele   = le modle tudi (infos sur le mode d'ajustement,...)                             |
 | variable = le numro de la variable considre                                               |
 | x        = le point pour lequel on veut la drive de la fonction modle                     |
 \--------------------------------------------------------------------------------------------- */

double MODELE_CalculeDerivee(struct modele_ajuste *modele, unsigned int variable, double x)
{
  double derivee = 0.0;
/*   int type_variable; */

  if (NULL == modele) return 0.0;
  if (variable > modele->nbr_variables) {
    (void) fprintf(stderr, "MODELE_CalculeDerivee : variable inconnue ! (%u, max %u)\n",
		   variable, modele->nbr_variables);
    return 0.0;
  }

  switch ( modele->algorithme )
  {
  case ALGO_PSEUDOGRADIENT:
    derivee = PG_CalculerDerivee(modele, variable, x);
    break;

  default:
    (void) fprintf(stderr, "MODELE_CalculeDerivee : algorithme inconnu ! (%u)\n", modele->algorithme);
  }

  return derivee;
}

/* -------- Calcul dans le cadre d'un algorithme de type pseudo-gradient --------*/

static double PG_CalculerDerivee(struct modele_ajuste *modele, unsigned int num_variable, double x)
{
  PG_variable *variable;

  if (NULL == modele->infos_variables.pseudogradient) {
    (void) fprintf(stderr, "PG_CalculeDerivee : aucune variable !\n");
    return 0.0;
  }
  variable = modele->infos_variables.pseudogradient + num_variable;

  switch (modele->type)
  {
  case MODELE_CL_MODELES:
    break;

  case MODELE_EXAFS_CHEMINS:
    {
      struct modele_feff *feff;
      double delta_e = 0.0;

      feff = modele->infos_modele.feff;
      if ( VRAI == feff->drapeaux.DE0_general )
 	delta_e = VAR_LireValeur( &(feff->Delta_E), VARIABLE_LIRE_VALEUR_AJUSTEE );
      else if ( variable->origine.couche != NULL ) {
	delta_e = VAR_LireValeur( &(variable->origine.couche->Delta_E), VARIABLE_LIRE_VALEUR_AJUSTEE );
      }

      return calc_derivee(x, variable->origine.couche, modele->infos_modele.feff, variable->type, variable->ex_valeur,
			  calc_exafs_1point(x, feff, variable->origine.couche, delta_e), delta_e);
    }		

  case MODELE_EXAFS_GNXAS:
  case MODELE_SEUIL_BASIQUE:
  case MODELE_FLUOX:
  case MODELE_DECOMPOSITION:
    (void) fprintf(stderr, "PG_CalculeDerivee : type de modle non gr, %d\n", modele->type);
    break;

  default:
    (void) fprintf(stderr, "PG_CalculeDerivee : type de modle inconnu, %d\n", modele->type);
    break;
  }

  return 0.0;
}
