/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Cration et gestion de modles en vue d'un ajustement

   Calcul du rsidu entre un modle et l'exprience
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "graphes.h"
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int calcule_residu_reel    (Graphe *experience, Graphe *modele, Graphe *residu);
static int calcule_residu_complexe(Graphe *experience, Graphe *modele, Graphe *residu);

/* calcule_erreur_gntp : g indique le genre des donnes (R : rel   , C : complexe)
                         n indique la norme utilise    (C : carrs , A : absolue )
			 t indique le type de somme     (A : absolue, R : relative)
			 p indique la pondration       (  : aucune , S : erreurs )
   Remarque : la pondration n'a pas de sens pour une erreur relative...
 */

static double calcule_erreur_RCA (Graphe *residu);
static double calcule_erreur_RCAS(Graphe *residu, Graphe *experience);
static double calcule_erreur_RCR (Graphe *residu, Graphe *experience);
/* static double calcule_erreur_CCA (Graphe *residu); */
/* static double calcule_erreur_CCAS(Graphe *residu, Graphe *experience); */
/* static double calcule_erreur_CCR (Graphe *residu); */

/* ~~~~~~~~ Ralisation concrte ~~~~~ */

/* -------- Calculer le graphe contenant le rsidu point par point entre modle et exprience --------
      Renvoie l'erreur associe (<0 si problme), correspondant  la grandeur  minimiser
 */

double MODELE_CalculerResidu(struct modele_ajuste *modele)
{
  double erreur;

  if (NULL == modele) return -1.0;

  if ( (NULL == modele->experience) || (NULL == modele->reference) ) {
    (void) fprintf(stderr, "Modele_CalculerResidu : modle sans graphe exprimental !\n");
    return -2.0;
  }

  if (NULL == modele->modele) {
    (void) fprintf(stderr, "Modele_CalculerResidu : modle sans graphe modle !\n");
    return -3.0;
  }

  if (NULL == modele->residu) {
    (void) fprintf(stderr, "Modele_CalculerResidu : modle sans graphe pour contenir le rsidu !\n");
    return -4.0;
  }

  if ( modele->reference->genre_donnees == GRF_FCT_REELLE ) {
    calcule_residu_reel(modele->reference, modele->modele, modele->residu);
  } else if ( GRF_FCT_COMPLEXE == modele->reference->genre_donnees )  { /* Graphes  valeurs complexes ? */
    calcule_residu_complexe(modele->reference, modele->modele, modele->residu);
  } else {
    (void) fprintf(stderr, "Modele_CalculerResidu : graphe modle  valeurs de type inconnu !\n");
    return -5.0;
  }

  /* Maintenant que le rsidu est calcul, on peut s'en servir pour calculer la grandeur minimise : "erreur" */

  /* Actuellement : seuls les moindres carrs sont programms */
  erreur = MODELE_CalculerErreur(modele, MODELE_ERREUR_CARRES, modele->drapeaux.pondere, FAUX);

  /* Tout s'est bien pass */
  return erreur;
}

/* -------- Calcul effectif, dans le cas d'un modle rel -------- */

static int calcule_residu_reel(Graphe *experience, Graphe *modele, Graphe *residu)
{
  unsigned long i;
  double *y_exp, *y_modele, *y_residu;

  for ( i = 0, y_exp = experience->y, y_modele = modele->y, y_residu = residu->y ;
	i < modele->nombre_point ; i++, y_exp++, y_modele++, y_residu++) {
    *y_residu = *y_exp - *y_modele;
  }

  GRF_bornes(residu, NE_PAS_FAIRE, DESORDONNE);
  return 0; /* Tout s'est bien pass */
}

/* -------- Calcul effectif, dans le cas d'un modle complexe -------- */

static int calcule_residu_complexe(Graphe *experience, Graphe *modele, Graphe *residu)
{
  unsigned long i;
  Complexe *y_exp, *y_modele, *y_residu;

  for ( i = 0, y_exp = (Complexe *) experience->y, y_modele = (Complexe *) modele->y, y_residu = (Complexe *) residu->y ;
	i < modele->nombre_point ; i++, y_exp++, y_modele++, y_residu++) {
    y_residu->re = y_exp->re - y_modele->re;
    y_residu->im = y_exp->im - y_modele->im;
  }

  GRF_bornes(residu, NE_PAS_FAIRE, DESORDONNE);
  return 0; /* Tout s'est bien pass */
}

/* -~-~-~-~-~-~-~-~ Calcul d'un rsidu condens en une valeur (erreur) -~-~-~-~-~-~-~-~ */

/* -------- Appel depuis l'extrieur -------- */

double MODELE_CalculerErreur(struct modele_ajuste *modele, int type_erreur, boolean stat, boolean relative)
{
  Graphe *residu;
  double erreur = -20.0;

  if ( NULL == modele ) return -1.0;
  if ( type_erreur < 0 ) {
    residu = modele->reference; /* On veut la norme du spectre exprimental */
    type_erreur = -type_erreur;
  } else
    residu = modele->residu; /* On travaille sur le rsidu */

  if ( NULL == residu ) {
    (void) fprintf( stderr, "%s : pas de graphe rsiduel (modele->residu ou modele->reference) !\n",
		    __FUNCTION__ );
    return -2.0;
  }

  switch (type_erreur)
  {
  case MODELE_ERREUR_CARRES:
    if ( VRAI == stat ) {
      if ( VRAI == relative ) {
	/* La pondration se simplifie, puisque l'on a du (y-y_th)/y -> comme sans pondration ! */
	erreur = calcule_erreur_RCR( residu, modele->reference );
      } else {
	erreur = calcule_erreur_RCAS(residu, modele->reference);
      }
    } else {
      if (VRAI == relative) {
	erreur = calcule_erreur_RCR( residu, modele->reference );
      } else {
	erreur = calcule_erreur_RCA(residu);
      }
    }
    break;

  case MODELE_ERREUR_ABSOLUE:
    break;
  }

  return erreur;
}

/* -------- Somme des carrs, spectre rel, sans pondration -------- */

static double calcule_erreur_RCA(Graphe *residu)
{
  unsigned long i;
  double erreur = 0.0, *y;

  for (i = 0, y = residu->y ; i < residu->nombre_point ; i++, y++) {
    erreur += (*y) * (*y);
  }

  return erreur;
}

/* -------- Somme des carrs, spectre rel, pondration par les erreurs -------- */

static double calcule_erreur_RCAS(Graphe *residu, Graphe *experience)
{
  unsigned long i, ignores = 0;
  double erreur = 0.0, *y, *y_err, sigma;

  if ( NULL == experience ) {
    (void) fprintf( stderr, "%s : aucun graphe exprimental !\n", __FUNCTION__ );
    return -1.0;
  }

  if ( NULL == experience->erreur_y ) { /* Le rsidu n'a pas forcment d'erreurs associes, mais l'exprience si ! */
    (void) fprintf( stderr, "%s : aucune erreur associe !\n", __FUNCTION__ );
    return -1.0;
  }

  for ( i = 0, y = residu->y, y_err = experience->erreur_y->y ;
	i < residu->nombre_point ;
	i++, y++, y_err++ ) {
    sigma = *y_err * *y_err;
    if ( 0.0 == *y_err ) {
      ignores++; /* Pour viter une division par 0 */
    } else {
      erreur += (*y) * (*y) / sigma;
    }
  }

  if ( ignores > 0 ) {
    (void) fprintf( stderr, "%s : %lu points ignors dans le calcul [erreur associe nulle !]\n",
		    __FUNCTION__, ignores);
  }
  return erreur;
}

/* -------- Somme des carrs relatifs, spectre rel, sans pondration -------- */

static double calcule_erreur_RCR(Graphe *residu, Graphe *experience)
{
  unsigned long i;
  double erreur = 0.0, *y, *y_exp, tmp;

  for (i = 0, y = residu->y, y_exp = experience->y ; i < residu->nombre_point ; i++, y++, y_exp++) {
    if ( *y_exp != 0.0 ) {
      tmp = *y / *y_exp;
      erreur += tmp * tmp;
    }
  }

  return erreur;
}
