/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Cration              : 12 novembre 1999
   Dernire modification : 18 novembre 1999

   Module d'ajustement par un modle  n couches

   Calcul de la matrice de covariance
   estime par l'inverse de la matrice des drives secondes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "systeme.h"

#include "graphes.h"
#include "ERREURS.H"

#include "atomes.h"   /* Pour les absorbeurs */
#include "couches.h"
#include "lissage.h"
#include "matrices.h"
#include "modeles.h"
#include "science.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */


/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* static int calc_mat_exact(double *matrice, unsigned long nbr_elements, Infos_ncouches *infos);
 Pour le calcul par les drives secondes, s'il s'avre ncessaire */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* Notes : en pratique, cette fonction devrait tre appele assez rarement,
   juste en fin d'ajustement quand le modle est suppos correct.
   Aussi, il n'y a pas d'effort d'optimisation dans l'tat actuel des choses. */

boolean NC_covariance(struct modele_ajuste *modele)
{
#ifdef ancienne_version
  Matrice *mat_inv = NULL, *matrice;
  unsigned long nbr_elements, i, j;

  if (NULL == modele) return FAUX; /* Pas d'infos d'ajustement ! */

  /* Nombre total de variables = dimension totale de la matrice */
  for (i = 0, nbr_elements = 0 ; i < infos->nbr_modeles ; i++)
    nbr_elements += infos->modeles[i]->nbr_variables;

  if ( nbr_elements < 2) {
    (void) fprintf(stderr, "NC_covariance : trop peu de variables pour calculer une matrice de covariance !\n");
    return FAUX;
  }

  /* On initialise les modles */
  for (i = 0 ; i < infos->nbr_modeles ; i++) {
    int retour;

    retour = MODELE_Initialiser( infos->modeles[i] );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "NC_covariance : impossible d'initialiser le modle %ld (code %d).\n", i + 1, retour);
      return FAUX;
    }
  }

  for (i = 0 ; i < infos->nbr_modeles ; i++) {
    /* On initialise les variables, comme si algo. rapide */
  }

  /* Matrice de covariance : carre, symtrique : on ne stocke que la partie infrieure
     -> si matrice (n,n), cela revient  n(n+1)/2 lments */
  matrice = creer_matrice(nbr_elements, nbr_elements, MATRICE_SYMETRIQUE);
  if (matrice == NULL) {
/*     termine_rapide(infos); */
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "N_couches/covariance.c", "NC_covariance", "matrice");
    return FAUX;
  }

/* Calcul de la matrice des drives secondes. Mis  part dans le cas d'une option ventuelle
   pour le choix entre un calcul exact ou simplifi. */
  if ( calc_mat_simple(matrice->valeurs, nbr_elements, infos) == FAUX ) {
/*     termine_rapide(infos); */
    return FAUX;
  }

  /* En attendant, on affiche le rsultat du calcul */
#ifdef _DEVELOPPE_
  printf("------ Hessienne de l'ajustement -------\n");
  for (i = 0 ; i < nbr_elements ; i++)
    printf("Var %ld\t", i);
  printf("\n");

  for (i = 0 ; i < nbr_elements ; i++) {
/*     LM_variable var_i; */
    for (j = 0 ; j <= i ; j++) {
      printf("%g \t", matrice->valeurs[MAT_TINF_element(i, j)]);
    }

/*     (void) infos_rapide(i, NULL, &var_i); */
/*     printf("Var %ld (%d)\n", i, var_i.type); */
  }
#endif

  /* On n'a plus besoin des infos d'ajustement : voir  les librer ? */

  /* On inverse la matrice */
  mat_inv = inverse_matrice(matrice, VRAI);
  effacer_matrice(matrice); /* On n'a plus besoin d'elle non plus */
  if (mat_inv == NULL) {
/*      termine_rapide(infos); */
    return FAUX;
  }

  /* Affichage de la matrice inverse */
  printf("------ Pseudo-matrice de covariance -------\n");
  for (i = 0 ; i < nbr_elements ; i++)
    printf("Var %ld\t", i);
  printf("\n");

  for (i = 0 ; i < nbr_elements ; i++)
  {
/*     LM_variable var_i; */
    for (j = 0 ; j <= i ; j++)
    {
      printf("%g \t", mat_inv->valeurs[MAT_TINF_element(i, j)] * 2.0); /* S = 2 H^-1 */
    }
    for ( ; j < nbr_elements ; j++)
      printf("\t");

/*     (void) infos_rapide(i, NULL, &var_i); */
/*     printf("Var %ld (%d)\n", i, var_i.type); */
  }

  printf("------ Valeurs et incertitudes obtenues -------\n");
  for (i = 0 ; i < nbr_elements ; i++)
  {
/*     LM_variable var_i; */
/*     (void) infos_rapide(i, NULL, &var_i); */
/*     printf("Var %ld = %g  %g (%d)\n", i, VAR_prend_valeur(var_i.var), */
/* 	   sqrt( mat_inv->valeurs[MAT_TINF_element(i, i)] * 2.0 ), var_i.type); */
  }

  printf("------ Matrice de corrlation correspondante -------\n");
  for (i = 0 ; i < nbr_elements ; i++) {
    double val_1 = sqrt( mat_inv->valeurs[MAT_TINF_element(i, i)] );
    for (j = 0 ; j < i ; j++) {
      double val_2 = sqrt( mat_inv->valeurs[MAT_TINF_element(j, j)] );

      mat_inv->valeurs[ MAT_TINF_element(i, j) ] /= val_1 * val_2;
    }
  }
  for (i = 0 ; i < nbr_elements ; i++) /* La diagonale */
    mat_inv->valeurs[ MAT_TINF_element(i, i) ] = 1.0;

  for (i = 0 ; i < nbr_elements ; i++) /* Affichage */
    printf("Var %ld\t", i);
  printf("\n");

  for (i = 0 ; i < nbr_elements ; i++)
  {
/*     LM_variable var_i; */
    for (j = 0 ; j <= i ; j++)
    {
      printf("%g \t", mat_inv->valeurs[MAT_TINF_element(i, j)]); /* S = 2 H^-1 */
    }
    for ( ; j < nbr_elements ; j++)
      printf("\t");

/*     (void) infos_rapide(i, NULL, &var_i); */
/*     printf("Var %ld (%d)\n", i, var_i.type); */
  }

/*   termine_rapide(infos); */
  effacer_matrice(mat_inv);
#endif
  return VRAI;
}
