/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Ce fichier : cration le 6 juin 2002

   Module d'ajustement par un modle  n couches

   Liste des couches dans un modle et gestion de cette liste
   = paramtrage spcifique d'un modle de chemins  ajuster
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "atomes.h"
#include "couches.h"
#include "feff.h"
#include "interface.h"
#include "theorie.h" /* Pour les modles de lpm */

#include "nc_liste.txt"

/* ~~~~~~~~ Variables locales  ce fichier ~~~~~~~~ */

#define LGMAX_NOM_GRAPHE_LPM             30
#define LGMAX_NOM_GRAPHE_PHASE_ABSORBEUR 43
#define LGMAX_NOM_GRAPHE_PHASE           48
#define LGMAX_NOM_GRAPHE_AMPLITUDE       48

static int var_editee = 0;  /* S0 dit par dfaut */

/* ~~~~~~~~ Fonctions locales  ce fichier ~~~~~~~~ */

static void ASC_nc_couches(Fenetre *fenetre, int type, int ss_type, int position);
static int  CLV_nc_couches(unsigned int, int, int *);

static Variable *QuelleVariable(int variable_editee, Modele_FEFF *modele, Couche *couche);

static void enregistre_valeurs(Variable *var);
static void affiche_valeurs   (Variable *var);
static void affiche_couche    (Modele_FEFF *modele, Couche *couche);

static void active_tout(Modele_FEFF *modele);
static void prepare_liste_couches(void);

/* ~~~~~~~~ Ralisation pratique des diffrentes fonctions ~~~~~~~~ */

/* -------- Aller  une couche : la mettre comme couche dite dans la fentre -------- */

void COUCHE_Aller(Modele_FEFF *modele, Couche *couche)
{
  Fenetre *fenetre;
  unsigned int num_couche = 0;
  Modele_FEFF *modele_fenetre;

  if (NULL == modele) return;

  fenetre = ARB_QuelleFenetre(arbres + AJ_EXAFS_CHEMINS, FAUX);
  if (NULL == fenetre) {
    (void) fprintf(stderr, "%s,%s l.%u : diteur de couches ferm\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }
  modele_fenetre = (Modele_FEFF *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( modele != modele_fenetre ) {
    OPTION_A_FAIRE( "Changement de couche avec changement de modle FEFF." );
    return;
  }

  /* On enregistre l'ancienne couche, si ncessaire */
  if ( NULL != couche_editee ) {
    COUCHE_EnregistrerParametres(couche_editee);
  } else { /* On n'ditait rien avant */
    active_tout(modele);
  }

  /* Numro de la couche dans la liste ? */
  if ( NULL != couche ) {
    Couche *i;

    for (i = modele->couches ; i != couche ; i = i->suivante, num_couche++) {
      if ( NULL == i ) {
	(void) fprintf(stderr, "COUCHE_Aller : la couche %s n'est pas dans le modle indiqu !\n", couche->nom);
	return;
      }
    }
  } else {
    couche_editee = NULL;
    active_tout(NULL); /* Aucune option disponible */
    FEN_ascenseur(fenetre, ASC_HORIZONTAL, 0, 1000);
    return;
  }

  /* On va  la nouvelle couche */
  couche_editee = couche;
  affiche_couche(modele, couche_editee);

  /* L'ascenseur */
  if (modele->nbr_couches < 2) { /* 0 ou 1 couche */
    FEN_ascenseur(fenetre, ASC_HORIZONTAL, 0, 1000);
  } else {
    float pos_asc, taille_asc;

    taille_asc = 1000.0 / (float) (modele->nbr_couches);
    pos_asc = 1000.0 * num_couche / ((float) (modele->nbr_couches - 1));
    FEN_ascenseur(fenetre, ASC_HORIZONTAL, (int) pos_asc, (int) taille_asc);
  }
}

/* -------- Enregistrer les informations pour le modle dit -------- */

void COUCHE_EnregistrerModele(void)
{
  Variable *variable;
  struct modele_feff *modele;

  if ( FAUX == arbres[AJ_EXAFS_CHEMINS].drapeaux.ouvert ) /* Pas de modle  enregistrer */
    return;

  modele = (struct modele_feff *) ARB_LireComplement( AJ_EXAFS_CHEMINS );
  if (NULL == modele) {
    (void) fprintf(stderr, "COUCHE_EnregistrerModele : arbre ouvert, mais aucun modle associ !\n");
    return;
  }

  /* Presque tout est mmoris au vol... Sauf Rmin/Rmax, pas encore pris en compte, et la pondration */
  modele->ponderation = OBJ_LireEntier_U( AJ_EXAFS_CHEMINS, AEC_PONDERATION );
  if ( MODELE_LPM_GAMMA_ETA == modele->type_lpm )
    modele->lpm_val_eta = OBJ_LireNombre( AJ_EXAFS_CHEMINS, AEC_VAL_ETA, NULL );

  /* On sauve les valeurs de la variable courante... */
  variable = QuelleVariable(var_editee, modele, couche_editee);
  enregistre_valeurs( variable );

  /* Paramtres de la couche, si ncessaire */
  COUCHE_EnregistrerParametres(couche_editee);
}

/* -------- Enregistrer les informations pour la couche dite -------- */

void COUCHE_EnregistrerParametres(Couche *couche)
{
  if (couche == NULL) return;

  lit_chaine(AJ_EXAFS_CHEMINS, AEC_NOM_COUCHE, LG_MAX_NOM_COUCHE, couche->nom, TXT_COPIER);

  switch (var_editee) /* Suivant la variable qui est dite, on affiche telle ou telle chose */
  {
    /* Variables mixtes */
  case AEC_DELTAE0:
    if ( VRAI == OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_DE0_INDIVIDU, QUESTION) ) { /* DE0 individuel */
      enregistre_valeurs( &(couche->Delta_E) );
    }
    break;

  case AEC_GAMMA:
    OPTION_A_FAIRE( "Enregistrement uniquement si Gamma individuel" );
    enregistre_valeurs( &(couche->Gamma) );
    break;

  case AEC_N:
    enregistre_valeurs( &(couche->N) );
    break;

  case AEC_LONGUEUR:
    enregistre_valeurs( &(couche->R) );
    break;

  case AEC_DEBYEW:
    enregistre_valeurs( &(couche->Debye_Waller) );
    break;

  case AEC_C1:
    enregistre_valeurs( &(couche->Cumul_1) );
    break;
  case AEC_C2:
    enregistre_valeurs( &(couche->Cumul_2) );
    break;
  case AEC_C3:
    enregistre_valeurs( &(couche->Cumul_3) );
    break;
  case AEC_C4:
    enregistre_valeurs( &(couche->Cumul_4) );
    break;
  }

  couche->utilisee = OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, QUESTION);

  /* Les infos de phase, amplitude, exafs... ont t modifies au vol donc... */
}

/* -------- Ouverture de la fentre qui affiche les couches -------- */

void COUCHE_OuvrirEditeur(Modele_FEFF *modele)
{
  Fenetre *fenetre;

  if ( NULL == modele ) {
    modele = FEFF_ChoisirModele(FAUX); /* Modle actif si possible... */
    if ( NULL == modele ) { /* On coupe tout */
      (void) fprintf(stderr, "COUCHE_OuvrirEditeur : aucun modle FEFF trouv...\n");
      return;
    }
  }

  fenetre = ARB_QuelleFenetre(arbres + AJ_EXAFS_CHEMINS, FAUX);
  if (NULL == fenetre) { /* Fentre pas encore ouverte */
    if ( FAUX == arbres[AJ_EXAFS_CHEMINS].drapeaux.prepare ) {
      prepare_liste_couches();
      ARB_MettreComplement( AJ_EXAFS_CHEMINS, modele );
      ARB_SimulerClic( AJ_EXAFS_CHEMINS, AEC_S02, 1 );
    }

    fenetre = ARB_Appeler(AJ_EXAFS_CHEMINS, NAME|CLOSER|MOVER|ASC_HORIZONTAL, tit_arb_choix_chemins, 0);
    if (NULL == fenetre) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir la fentre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
    fenetre->ascenseur = ASC_nc_couches; /* Gestion de l'ascenseur */
  } else { /* Enregistrer les paramtres du modle prcdemment dit */
    COUCHE_EnregistrerModele();
    FEN_premierplan( fenetre );

    /* On fait sauter le lien vers l'ancien modle... Sinon, impossible de mettre le nouveau ! */
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, NULL );
    ARB_MettreComplement ( AJ_EXAFS_CHEMINS, NULL );
  }

  /* Le modle dit... */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, modele );
  ARB_MettreComplement ( AJ_EXAFS_CHEMINS, modele );

  /* Paramtres de ce modle */
  switch ( modele->type_lpm )
  {
  case MODELE_LPM_GAMMA:
    (void) ARB_SimulerClic( AJ_EXAFS_CHEMINS, AEC_LPM_GAMMA, 1 );
    break;

  case MODELE_LPM_GAMMA_ETA:
    (void) ARB_SimulerClic( AJ_EXAFS_CHEMINS, AEC_LPM_G_ETA, 1 );
    break;

  case MODELE_LPM_GRAPHE:
    (void) ARB_SimulerClic( AJ_EXAFS_CHEMINS, AEC_LPM_SEPARE, 1 );
    break;

  case MODELE_LPM_INCLUS:
    (void) ARB_SimulerClic( AJ_EXAFS_CHEMINS, AEC_LPM_INCLUS, 1 );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Type de libre parcours moyen inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, modele->type_lpm);
  }
  if ( modele->phase_central != NULL ) {
    met_chaine(AJ_EXAFS_CHEMINS, AEC_PHA_CENTRAL, LGMAX_NOM_GRAPHE_PHASE_ABSORBEUR, modele->phase_central->nom, TXT_COPIER);
  } else {
    met_chaine(AJ_EXAFS_CHEMINS, AEC_PHA_CENTRAL, LGMAX_NOM_GRAPHE_PHASE_ABSORBEUR, txt_phase_absorbeur_ds_phase, TXT_COPIER);
  }
  OBJ_MettreEntier_S( AJ_EXAFS_CHEMINS, AEC_PONDERATION, modele->ponderation );
  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_DE0_INDIVIDU, FAUX == modele->drapeaux.DE0_general);
  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_GAM_INDIVIDU, FAUX == modele->drapeaux.GAM_general);

  /* On se place sur la 1re couche */
  COUCHE_Aller( modele, modele->couches );
}

/* -------- Ascenseurs de la fentre = slection de la couche dite -------- */

static void ASC_nc_couches(Fenetre *fenetre, int type, int ss_type, int position)
{
  Modele_FEFF *modele;
  Couche *couche = couche_editee;

  if ( fenetre == NULL ) return; /* Ne devrait jamais arriver normalement... */

  modele = (Modele_FEFF *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if (NULL == modele) { /* Pas de modle : inutile de dfiler ! */
    return;
  }
  if (modele->nbr_couches < 2) return; /* Aucune ou une seule couche en mmoire : inutile de dfiler... */

  if (type == WM_ARROWED) { /* On a utilis une des flches de l'ascenseur */
    switch (ss_type) /* Action qui peut tre envisage */
    {
    case WA_LFPAGE: /* On descend de 10 couches */
      if (modele->nbr_couches <= 10) {
	couche = modele->couches;
      } else {
	unsigned long num_courante = 0;

	for (couche = modele->couches ; couche != couche_editee ; couche = couche->suivante, num_courante++);

	if (num_courante > 10) {
	  unsigned long i;
	  for (i = 0, couche = modele->couches ; i < (num_courante - 10) ; couche = couche->suivante, i++);
	} else {
	  couche = modele->couches;
	}
      }
      break;

    case WA_RTPAGE: /* On monte de 10 couches */
      if (modele->nbr_couches <= 10) {
	couche = modele->couches;
	while (couche->suivante != NULL) /* On se place sur la dernire couche */
	  couche = couche->suivante;
      } else {
	unsigned int compteur = 0;

	couche = couche_editee;
	while ( compteur < 10 )	{
	  couche = couche->suivante;
	  if (couche->suivante == NULL) /* On est sur la dernire couche */
	    break;
	  compteur++;
	}
      }
      break;

    case WA_LFLINE: /* On descend d'une couche */
      if (couche_editee != modele->couches)
	for (couche = modele->couches ; couche->suivante != couche_editee ; couche = couche->suivante);
      break;

    case WA_RTLINE: /* On monte d'une couche   */
      if (couche_editee->suivante != NULL) { /* Ce n'est pas la dernire */
	couche = couche_editee->suivante;
      }
      break;
    }
  } else if (type == WM_HSLID) { /* Ascenseur horizontal dplac */
    unsigned long num_couche, i;

    num_couche = position * (modele->nbr_couches - 1) / 1000;
    for (i = 0, couche = modele->couches ; i < num_couche ; i++, couche = couche->suivante);
  }

  /* On va  la nouvelle couche active */
  COUCHE_Aller(modele, couche);
}

/* -------- Clics sur un objet = paramtrage de la couche active ------------ */

/* Version clavier... */

static int CLV_nc_couches(unsigned int touche, int etat, int *renvoi)
{
  if ( renvoi != NULL ) *renvoi = 0;

  if ( etat & K_CTRL ) {
    Fenetre *fenetre;

    fenetre = ARB_QuelleFenetre( arbres + AJ_EXAFS_CHEMINS, FAUX );

    if ( Fleche_Gauche == touche ) {	       /* Ctrl - <- : couche prcdente */
      ASC_nc_couches( fenetre, WM_ARROWED, WA_LFLINE, 0 );

      return BI_EVENEMENT_TRAITE;
    } else if ( Fleche_Droite == touche ) { /* Ctrl - -> : couche suivante */
      ASC_nc_couches( fenetre, WM_ARROWED, WA_RTLINE, 0 );

      return BI_EVENEMENT_TRAITE;
    }
  }

  /* Touche non traite */
  return BI_EVENEMENT_NON_TRAITE;
}

/* Version souris */

int GER_nc_couches(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  Modele_FEFF *modele;
  Couche *couche;
  Variable *variable;

  modele = (Modele_FEFF *) ARB_LireComplement( AJ_EXAFS_CHEMINS );
  if ( NULL == modele ) {
    (void) fprintf(stderr, "GER_nc_couches : arbre ouvert, aucun modle associ !\n");
    return BI_EVENEMENT_TRAITE;
  }

  /* ---- D'abord les objets gnriques ---- */
  if (objet == AEC_NV_COUCHE) {        /* Cration d'une nouvelle couche */
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_NV_COUCHE, FORCE_OUI);
    if (couche_editee != NULL)
      COUCHE_EnregistrerParametres(couche_editee);
    couche = COUCHE_Creer(modele, couche_editee);
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_NV_COUCHE, FORCE_NON);
    if (couche == NULL) {
      (void) fprintf(stderr, "GER_nc_couches : ERREUR dans COUCHE_Creer !\n");
      return BI_EVENEMENT_TRAITE;
    }
    
    COUCHE_Aller(modele, couche);
    return BI_EVENEMENT_TRAITE;
  }

  /* ---- Ici, on s'occupe donc des paramtres d'une couche -> que si une couche au moins existe ! ---- */
  if (objet == AEC_UTILISER) {
    boolean etat;

    etat = OBJ_choisit(AJ_EXAFS_CHEMINS, objet, ECHANGE);
    if (BI_etat & K_CTRL) { /* Ctrl-clic : toutes couches actives ou inactives */
      for (couche = modele->couches ; couche ; couche = couche->suivante) {
	couche->utilisee = etat;
      }
    }
    if (couche_editee != NULL)
      couche_editee->utilisee = etat;
    return BI_EVENEMENT_TRAITE;
  }

  if (objet == AEC_EFF_COUCHE) {   /* Effacement de la couche courante */
    Couche *future;

    if ( couche_editee == NULL ) {
      (void) fprintf(stderr, "GER_nc_couches : effacer couche, mais aucune couche dite !\n");
      return BI_EVENEMENT_TRAITE;
    }

    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_EFF_COUCHE, FORCE_OUI);
    if ( couche_editee->suivante != NULL ) {
      future = couche_editee->suivante;
    } else {
      future = modele->couches;
      while ( future->suivante != couche_editee ) {
	future = future->suivante;
	if ( NULL == future ) {
	  (void) fprintf(stderr, "GER_nc_couches / effacer : la couche dite n'est pas dans le modle dit !\n");
	  return BI_EVENEMENT_TRAITE;
	}
      }
    }
    COUCHE_Effacer(modele, couche_editee, QUESTION, FAUX);
    COUCHE_Aller  (modele, future);
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_EFF_COUCHE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;
  }

  if (objet == AEC_TTEFFACER) { /* Effacer le modle complet */
    int reponse;

    reponse = BI_ERR_alerte(NULL, &Q_EffacerAmplitude, VRAI);
    if (reponse == 3) return BI_EVENEMENT_TRAITE; /* On annule l'effacement */

    while (modele->couches) {
      COUCHE_Effacer(modele, modele->couches, VRAI, (reponse == 1));
    }
    couche_editee = NULL;
    return BI_EVENEMENT_TRAITE;
  }

  /* Variable en cours d'dition - utilise aprs */
  variable = QuelleVariable(var_editee, modele, couche_editee);

  if (objet == AEC_RESULTAT) { /* Relire les rsultats existants */
    double nv_valeur;
    int reponse;

    OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_OUI);

    reponse = BI_ERR_alerte(NULL, &Q_QuelleVariable, VRAI);
    if (5 == reponse) {
      OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_NON);
      return BI_EVENEMENT_TRAITE; /* On annule */
    }

    if ( variable != NULL ) {
      if ( VAR_LIBRE == variable->type ) {
	nv_valeur = variable->contenu.valeur;
	variable->valeur_initiale = nv_valeur;
	OBJ_MettreNombre(AJ_EXAFS_CHEMINS, AEC_VAL_DEBUT, nv_valeur, 0.0 );
      }
    }
    if (1 == reponse) return BI_EVENEMENT_TRAITE; /* On ne veut que la variable actuelle */

    if (2 == reponse) { /* Couche actuelle uniquement */
      if (NULL == couche_editee) {
	(void) fprintf(stderr, "GER_nc_couches : aucune couche en cours d'dition !\n");
	return VRAI;
      }

      if ( couche_editee->N.type == VAR_LIBRE )
	couche_editee->N.valeur_initiale = couche_editee->N.contenu.valeur;
      if ( couche_editee->Debye_Waller.type == VAR_LIBRE )
        couche_editee->Debye_Waller.valeur_initiale = couche_editee->Debye_Waller.contenu.valeur;
      if ( couche_editee->R.type == VAR_LIBRE )
	couche_editee->R.valeur_initiale = couche_editee->R.contenu.valeur;
      if ( couche_editee->Cumul_1.type == VAR_LIBRE )
	couche_editee->Cumul_1.valeur_initiale = couche_editee->Cumul_1.contenu.valeur;
      if ( couche_editee->Cumul_2.type == VAR_LIBRE )
	couche_editee->Cumul_2.valeur_initiale = couche_editee->Cumul_2.contenu.valeur;
      if ( couche_editee->Cumul_3.type == VAR_LIBRE )
	couche_editee->Cumul_3.valeur_initiale = couche_editee->Cumul_3.contenu.valeur;
      if ( couche_editee->Cumul_4.type == VAR_LIBRE )
	couche_editee->Cumul_4.valeur_initiale = couche_editee->Cumul_4.contenu.valeur;

      /* Variables mixtes */
      if ( (FAUX == modele->drapeaux.DE0_general) && (VAR_LIBRE == couche_editee->Delta_E.type ) )
	couche_editee->Delta_E.valeur_initiale = couche_editee->Delta_E.contenu.valeur;
      if ( (FAUX == modele->drapeaux.GAM_general) && (VAR_LIBRE == couche_editee->Gamma.type ) )
	couche_editee->Gamma.valeur_initiale = couche_editee->Gamma.contenu.valeur;
    }

    if (3 == reponse) { /* On veut tous les rsultats pour toutes les couches */
      for (couche = modele->couches ; couche ; couche = couche->suivante) {
	if (couche->utilisee == FAUX) continue; /* Pas la peine... */
	
	if ( couche->N.type == VAR_LIBRE )
	  couche->N.valeur_initiale = couche->N.contenu.valeur;
	if ( couche->Debye_Waller.type == VAR_LIBRE )
	  couche->Debye_Waller.valeur_initiale = couche->Debye_Waller.contenu.valeur;
	if ( couche->R.type == VAR_LIBRE )
	  couche->R.valeur_initiale = couche->R.contenu.valeur;
	if ( couche->Cumul_1.type == VAR_LIBRE )
	  couche->Cumul_1.valeur_initiale = couche->Cumul_1.contenu.valeur;
	if ( couche->Cumul_2.type == VAR_LIBRE )
	  couche->Cumul_2.valeur_initiale = couche->Cumul_2.contenu.valeur;
	if ( couche->Cumul_3.type == VAR_LIBRE )
	  couche->Cumul_3.valeur_initiale = couche->Cumul_3.contenu.valeur;
	if ( couche->Cumul_4.type == VAR_LIBRE )
	  couche->Cumul_4.valeur_initiale = couche->Cumul_4.contenu.valeur;

	/* Variables mixtes */
	if ( (FAUX == modele->drapeaux.DE0_general) && (VAR_LIBRE == couche->Delta_E.type ) )
	  couche->Delta_E.valeur_initiale = couche->Delta_E.contenu.valeur;
	if ( (FAUX == modele->drapeaux.GAM_general) && (VAR_LIBRE == couche->Gamma.type ) )
	  couche->Gamma.valeur_initiale = couche->Gamma.contenu.valeur;
      }
    }

    if ( (3 == reponse) || (4 == reponse) ) { /* On veut les rsultats des variables gnrales du modle */
      if ( VAR_LIBRE == modele->S0_2.type )
	modele->S0_2.valeur_initiale = modele->S0_2.contenu.valeur;

      /* Variables mixtes */
      if ( (VRAI == modele->drapeaux.DE0_general) && (VAR_LIBRE == modele->Delta_E.type ) )
	modele->Delta_E.valeur_initiale = modele->Delta_E.contenu.valeur;
      if ( (VRAI == modele->drapeaux.GAM_general) && (VAR_LIBRE == modele->Gamma.type ) )
	modele->Gamma.valeur_initiale = modele->Gamma.contenu.valeur;
    }

    OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_NON);
    return BI_EVENEMENT_TRAITE;
  }

  /* ---- On sauve les valeurs de la variable en cours d'dition ---- */
  enregistre_valeurs( variable );

  OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_OUI);

  /* ---- On traite l'objet cliqu ---- */
  switch (objet)
  {
  case AEC_LPM_SEPARE: /* Le libre parcours moyen est donn par un graphe obtenu extrieurement */
    if ( modele->lpm != NULL ) {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, LGMAX_NOM_GRAPHE_LPM, modele->lpm->nom, TXT_COPIER);
    } else {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, LGMAX_NOM_GRAPHE_LPM, txt_lpm_dans_amplitude, TXT_COPIER);
    }
  case AEC_LPM_INCLUS: /* Le libre parcours moyen est inclus dans les amplitudes */
    OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_ETA, FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, objet == AEC_LPM_SEPARE);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_GAM_INDIVIDU, FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_GAMMA, FORCE_NON);
#ifdef a_reprendre
    if (var_editee == GENE_GAMMA) /* On passe  une autre variable */
    {
      enregistre_valeurs( &Gamma );
      var_editee = AEC_S02;
      affiche_valeurs( &S0_2 );
      OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
    }
#endif
    return BI_EVENEMENT_TRAITE;

  case AEC_LPM_G_ETA:
    OBJ_MettreNombre(AJ_EXAFS_CHEMINS, AEC_VAL_ETA, modele->lpm_val_eta, 0.0 );
  case AEC_LPM_GAMMA: /* Calcul thorique du lpm : la variable G intervient */
    OBJ_choisit(AJ_EXAFS_CHEMINS, objet, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_ETA, objet == AEC_LPM_G_ETA);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_GAM_INDIVIDU, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_GAMMA, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case AEC_LPM_EXTERNE:
    if (modele->lpm) /* Il y avait un spectre dj avant */
      modele->lpm->drapeaux.protege = FAUX; /* A pu tre protg au cours d'un ajustement */

    modele->lpm = GRF_choisir(GR_LIBREP, VRAI, VRAI, NULL);
    if ( modele->lpm != NULL ) {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, LGMAX_NOM_GRAPHE_LPM, modele->lpm->nom, TXT_COPIER);
    } else {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, LGMAX_NOM_GRAPHE_LPM, txt_lpm_dans_amplitude, TXT_COPIER);
      GER_nc_couches(AEC_LPM_G_ETA, 0, 0); /* Pas de lpm -> on travaille avec un modle thorique */
    }
    return BI_EVENEMENT_TRAITE;

  case AEC_PHA_CENTRAL: /* Phase de l'absorbeur dans un graphe  part */
    modele->phase_central = GRF_choisir(GR_EXAFSPHA, VRAI, VRAI, NULL);
    if ( modele->phase_central != NULL ) {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_PHA_CENTRAL, LGMAX_NOM_GRAPHE_PHASE_ABSORBEUR, modele->phase_central->nom, TXT_COPIER);
    } else {
      met_chaine(AJ_EXAFS_CHEMINS, AEC_PHA_CENTRAL, LGMAX_NOM_GRAPHE_PHASE_ABSORBEUR, txt_phase_absorbeur_ds_phase, TXT_COPIER);
    }
    return BI_EVENEMENT_TRAITE;

  case AEC_DE0_INDIVIDU:
    modele->drapeaux.DE0_general = ! (OBJ_choisit(AJ_EXAFS_CHEMINS, objet, ECHANGE));
    if ( AEC_DELTAE0 == var_editee ) { 
      if ( VRAI == modele->drapeaux.DE0_general ) { /* Paramtre gnral */
	/* Rq. Si c'est la variable dite et que l'on passe en variable gnral,
	   c'est qu'il y avait au moins une couche et que le bouton tait actif ! */
	affiche_valeurs( &(modele->Delta_E) );
      } else { /* Spcifique  chaque couche */
	if (NULL == couche_editee) {
	  GER_nc_couches(AEC_S02, 0, 0); /* On active S0 comme variable, forcment prsente ! */
	  OBJ_active(AJ_EXAFS_CHEMINS, AEC_DELTAE0, FORCE_OUI);
	} else
	  affiche_valeurs( &(couche_editee->Delta_E) );
      }
    }
    return BI_EVENEMENT_TRAITE;

  case AEC_GAM_INDIVIDU:
    modele->drapeaux.GAM_general = ! (OBJ_choisit(AJ_EXAFS_CHEMINS, objet, ECHANGE));
    if ( AEC_GAMMA == var_editee ) { 
      if ( VRAI == modele->drapeaux.GAM_general ) { /* Paramtre gnral */
	/* Rq. Si c'est la variable dite et que l'on passe en variable gnral,
	   c'est qu'il y avait au moins une couche et que le bouton tait actif ! */
	affiche_valeurs( &(modele->Gamma) );
      } else { /* Spcifique  chaque couche */
	if (NULL == couche_editee) {
	  GER_nc_couches(AEC_S02, 0, 0); /* On active S0 comme variable, forcment prsente ! */
	  OBJ_active(AJ_EXAFS_CHEMINS, AEC_GAMMA, FORCE_OUI);
	} else
	  affiche_valeurs( &(couche_editee->Gamma) );
      }
    }
    return BI_EVENEMENT_TRAITE;

  case AEC_S02:
  case AEC_GAMMA:
  case AEC_DELTAE0:
  case AEC_N:
  case AEC_DEBYEW:
  case AEC_LONGUEUR:
  case AEC_C1:
  case AEC_C2:
  case AEC_C3:
  case AEC_C4:
    var_editee = objet;
    variable = QuelleVariable(var_editee, modele, couche_editee);
    affiche_valeurs( variable );
    return BI_EVENEMENT_TRAITE;

  case AEC_CONSTANT:
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MINIMUM, FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MAXIMUM, FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_DELTA_MIN    , FORCE_NON);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_RELATION, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case AEC_VARIABLE:
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MINIMUM, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MAXIMUM, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_DELTA_MIN    , FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_RELATION, FORCE_NON);

    OBJ_MettreIntervalle( AJ_EXAFS_CHEMINS, AEC_MINIMUM, AEC_MAXIMUM, 0, 0, variable->min, variable->max, 0.0, 0 );
    return BI_EVENEMENT_TRAITE;

  case AEC_RELATION:
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MINIMUM, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_MAXIMUM, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_DELTA_MIN, FORCE_OUI);
    OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_RELATION, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case AEC_GRF_AMP:
    couche_editee->amplitude = GRF_choisir(GR_EXAFSAMP, VRAI, VRAI, NULL);
    if ( couche_editee->amplitude )
      met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_AMP, LGMAX_NOM_GRAPHE_AMPLITUDE, couche_editee->amplitude->nom, TXT_COPIER);
    else
      met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_AMP, LGMAX_NOM_GRAPHE_AMPLITUDE, txt_pas_amplitude, TXT_COPIER);
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_GRF_AMP, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case AEC_GRF_PHASE:
    couche_editee->phase = GRF_choisir(GR_EXAFSPHA, VRAI, VRAI, NULL);
    if ( couche_editee->phase )
      met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, LGMAX_NOM_GRAPHE_PHASE, couche_editee->phase->nom, TXT_COPIER);
    else
      met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, LGMAX_NOM_GRAPHE_PHASE, txt_pas_phase, TXT_COPIER);
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -~-~-~-~-~-~-~-~ Sous-routines de gestion de l'interface -~-~-~-~-~-~-~-~ */

/* -------- Affichage des donnes associes  une couche -------- */

static void affiche_couche(Modele_FEFF *modele, Couche *couche)
{
  Variable *variable;

  if (couche == NULL) return;

  met_chaine(AJ_EXAFS_CHEMINS, AEC_NOM_COUCHE, 0, couche->nom, TXT_COPIER);

  variable = QuelleVariable(var_editee, modele, couche);
  affiche_valeurs( variable );

  OBJ_active(AJ_EXAFS_CHEMINS, AEC_CHEMIN, NULL != couche->chemin);

  if ( couche->amplitude )
    met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_AMP, LGMAX_NOM_GRAPHE_AMPLITUDE, couche->amplitude->nom, TXT_COPIER);
  else
    met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_AMP, LGMAX_NOM_GRAPHE_AMPLITUDE, txt_pas_amplitude, TXT_COPIER);

  if ( couche->phase )
    met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, LGMAX_NOM_GRAPHE_PHASE, couche->phase->nom, TXT_COPIER);
  else
    met_chaine(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, LGMAX_NOM_GRAPHE_PHASE, txt_pas_phase, TXT_COPIER);

  OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_UTILISER, couche->utilisee);
}

/* -------- Affichage des donnes associes  une variable ------------------------------- */

static void affiche_valeurs(Variable *var)
{
  if (NULL == var) {
    (void) fprintf(stderr, "affiche_valeurs [N_couches/liste.c] : variable = NULL\n");
    return;
  }

  OBJ_MettreNombre( AJ_EXAFS_CHEMINS, AEC_VAL_DEBUT, var->valeur_initiale, 0.0 );
/*   printf("affiche_valeurs : Variable de type : %d\n", var->type); */

  OBJ_active (AJ_EXAFS_CHEMINS, AEC_MINIMUM  , var->type != VAR_CONSTANTE);
  OBJ_active (AJ_EXAFS_CHEMINS, AEC_MAXIMUM  , var->type != VAR_CONSTANTE);
  OBJ_active (AJ_EXAFS_CHEMINS, AEC_DELTA_MIN, var->type != VAR_CONSTANTE);
  OBJ_active (AJ_EXAFS_CHEMINS, AEC_RELATION , var->type == VAR_RELATION );

  if ( var->type == VAR_CONSTANTE ) {
    OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_CONSTANT, FORCE_OUI);
  } else {
    OBJ_MettreIntervalle( AJ_EXAFS_CHEMINS, AEC_MINIMUM, AEC_MAXIMUM, 0, 0, var->min, var->max, 0.0, 0 );
    OBJ_MettreEntier_U( AJ_EXAFS_CHEMINS, AEC_DELTA_MIN, var->delta );

    if ( var->type == VAR_RELATION ) {
      OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_RELATION, FORCE_OUI);
      /* Rafficher la relation */
    } else {
      OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_VARIABLE, FORCE_OUI);
    }
  }
}

/* -------- Mmorisation des donnes associes  une variable --------------------------- */

static void enregistre_valeurs(Variable *var)
{
  if (NULL == var) {
    (void) fprintf(stderr, "enregistre_valeurs [N_couches/liste.c] : variable = NULL\n");
    return;
  }

  var->valeur_initiale = OBJ_LireNombre( AJ_EXAFS_CHEMINS, AEC_VAL_DEBUT, NULL );

  if ( OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_CONSTANT, QUESTION) ) { /* Variable constante */
    var->type = VAR_CONSTANTE;
    return;
  }

  var->min = OBJ_LireNombre( AJ_EXAFS_CHEMINS, AEC_MINIMUM, NULL );
  var->max = OBJ_LireNombre( AJ_EXAFS_CHEMINS, AEC_MAXIMUM, NULL );
  if (var->min > var->max) {
    double i = var->min;
    var->min = var->max;
    var->max = i;
  }
  var->delta = OBJ_LireNombre( AJ_EXAFS_CHEMINS, AEC_DELTA_MIN, NULL );

  if ( OBJ_choisit(AJ_EXAFS_CHEMINS, AEC_VARIABLE, QUESTION) ) { /* Variable de type normal */
    var->type = VAR_LIBRE;
    var->contenu.valeur = var->valeur_initiale;
    return;
  }

  var->type = VAR_RELATION; /* Variable de type relation */
  /* Mmoriser la relation */
}

/* -------- Reprage de la variable en cours d'dition -------- */

static Variable *QuelleVariable(int variable_editee, Modele_FEFF *modele, Couche *couche)
{
  if (NULL == modele) return NULL;

  switch (variable_editee)
  {
    /* Variables du modle global */
  case AEC_CONTRIB:
    fprintf(stderr, "QuelleVariable :  FAIRE, gestion de la contribution du modle\n");
    return NULL;

  case AEC_S02:
    return &(modele->S0_2);

    /* Variables mixtes */
  case AEC_DELTAE0:
    if ( VRAI == modele->drapeaux.DE0_general )
      return &(modele->Delta_E);
    else if ( couche != NULL )
      return &(couche->Delta_E);
    break;

  case AEC_GAMMA:
    if ( VRAI == modele->drapeaux.GAM_general )
      return &(modele->Gamma);
    else if ( couche != NULL )
      return &(couche->Gamma);
    break;

    /* Variables d'une couche */
  case AEC_N:
    if ( couche != NULL )
      return &(couche->N);

  case AEC_LONGUEUR:
    if ( couche != NULL )
      return &(couche->R);

  case AEC_DEBYEW:
    if ( couche != NULL )
      return &(couche->Debye_Waller);

  case AEC_C1:
    if ( couche != NULL )
      return &(couche->Cumul_1);

  case AEC_C2:
    if ( couche != NULL )
      return &(couche->Cumul_2);

  case AEC_C3:
    if ( couche != NULL )
      return &(couche->Cumul_3);

  case AEC_C4:
    if ( couche != NULL )
      return &(couche->Cumul_4);
  }

  /* Variable non-trouve ou couche NULL */
  return NULL;
}

/* -------- Activation / dsactivation des boutons en fonction du modle -------- */

static void active_tout(Modele_FEFF *modele)
{
  boolean sens;

  sens = (modele != NULL);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_TTEFFACER , sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_EFF_COUCHE, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_NOM_COUCHE, sens);
  if (sens == FAUX) /* Aucune couche */
    met_chaine(AJ_EXAFS_CHEMINS, AEC_NOM_COUCHE, 0, txt_pas_couche, TXT_COPIER);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_N, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_DEBYEW, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_C1, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_C2, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_C3, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_C4, sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_LONGUEUR, sens);

#ifdef _DEVELOPPE_ /* Encore en dveloppement */
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_CHEMIN   , sens);
#endif

  OBJ_active(AJ_EXAFS_CHEMINS, AEC_GRF_AMP  , sens);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE, sens);
}

static void prepare_liste_couches(void)
{
  int retour;

  /* Pr-initialisation */
  retour = ARB_Initialiser( AJ_EXAFS_CHEMINS, NULL, GER_nc_couches, CLV_nc_couches );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR %d dans ARB_Initialiser() [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, AJ_EXAFS_CHEMINS );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }

/*  OBJ_choisit(AJ_EXAFS_CHEMINS, COUCHE_N, FORCE_OUI);*/
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_LPM_G_ETA , +1,  0, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_LPM_INCLUS,  0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_LPM_SEPARE, +1, +1, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_LPM_EXTERNE, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_PONDERE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_PONDERATION, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_GAM_INDIVIDU, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_GAM_IND , 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_RMAX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_RMAX    , 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_VAL_INIT, 0, -3, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_VAL_DEBUT   , 0, -3, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_RESULTAT    , 0, -3, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_CONSTANT    , 0, -2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_CONSTANT, 0, -2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_VARIABLE    , 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_VARMIN  , 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_MINIMUM     , 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_DX_MIN  , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_DELTA_MIN   , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_RELATION    , 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_RELATION, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_VAL_RELATION, 0, +3, DEPLACE_PIXELS);

  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_UTILISER  , 0, -2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_CHEMIN    , 0, -2, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_NOM   , 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_NOM_COUCHE, 0, -1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_TXT_PHASE , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(AJ_EXAFS_CHEMINS, AEC_GRF_PHASE , 0, +1, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_ /* Options pas encore ralises */
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_VAL_RELATION, FORCE_NON);
  OBJ_active(AJ_EXAFS_CHEMINS, AEC_RELATION , FORCE_NON);
#endif

  arbres[AJ_EXAFS_CHEMINS].drapeaux.prepare = VRAI;
}
