/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Cration : 27 juin 2002

   Ajustement d'un modle  N chemins (type FEFF)

   Affichage des rsultats de l'ajustement (avec ou sans historique)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "couches.h"
#include "feff.h"
#include "theorie.h" /* Pour les types de lpm */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int affiche_variable(Variable *variable, int type, boolean historique);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int NC_AfficherResultats(struct modele_feff *modele, boolean historique)
{
  Couche *couche;

  if (NULL == modele) return -1;

  /* Variables gnrales */
  affiche_variable( &(modele->S0_2), VAR_SO2, historique);
  if ( VRAI == modele->drapeaux.DE0_general ) {
    affiche_variable( &(modele->Delta_E), VAR_DELTAE_GENE, historique);
  }
  if ( (VRAI == modele->drapeaux.GAM_general ) &&
       ( (MODELE_LPM_GAMMA == modele->type_lpm) || ( MODELE_LPM_GAMMA_ETA == modele->type_lpm) ) ) {
    affiche_variable( &(modele->Gamma), VAR_GAMMA_GENE, historique);
  }

  /* Variables pour chaque couche */
  for (couche = modele->couches ; couche != NULL ; couche = couche->suivante) {
    if ( FAUX == couche->utilisee ) {
      if (VRAI == historique)
	HIST_Texte(5, 2, "Couche inutilise : %s\n", couche->nom);
      continue;
    }

    if (VRAI == historique) {
      HIST_Texte(3, 2, "Couche : %s\n", couche->nom);
    }
    (void) printf("-- Couche %s --\n", couche->nom);

    affiche_variable( &(couche->N), VAR_N, historique);
    affiche_variable( &(couche->R), VAR_R, historique);
    affiche_variable( &(couche->Debye_Waller), VAR_DEBYE, historique);
    affiche_variable( &(couche->Cumul_1), VAR_CUMUL_1, historique);
    affiche_variable( &(couche->Cumul_2), VAR_CUMUL_2, historique);
    affiche_variable( &(couche->Cumul_3), VAR_CUMUL_3, historique);
    affiche_variable( &(couche->Cumul_4), VAR_CUMUL_4, historique);

    if ( FAUX == modele->drapeaux.DE0_general ) {
      affiche_variable( &(couche->Delta_E), VAR_DELTAE, historique);
    }
    if ( (FAUX == modele->drapeaux.GAM_general ) &&
	 ( (MODELE_LPM_GAMMA == modele->type_lpm) || ( MODELE_LPM_GAMMA_ETA == modele->type_lpm) ) ) {
      affiche_variable( &(couche->Gamma), VAR_GAMMA, historique);
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

static int affiche_variable(Variable *variable, int type, boolean historique)
{
  char *nom_variable;

  /* Nom court de la variable */
  switch ( type )
  {
  case VAR_SO2:
    nom_variable = "S0"DEUX_EXPOSANT;
    break;

  case VAR_DELTAE_GENE:
  case VAR_DELTAE:
    nom_variable = "DE0";
    break;

  case VAR_GAMMA_GENE:
  case VAR_GAMMA:
    nom_variable = GAMMA_MAJUSCULE;
    break;

  case VAR_N:
    nom_variable = "N";
    break;

  case VAR_R:
    nom_variable = "R";
    break;

  case VAR_DEBYE:
    nom_variable = SIGMA""DEUX_EXPOSANT;
    break;

  case VAR_CUMUL_1:
    nom_variable = "C1";
    break;
  case VAR_CUMUL_2:
    nom_variable = "C2";
    break;
  case VAR_CUMUL_3:
    nom_variable = "C3";
    break;
  case VAR_CUMUL_4:
    nom_variable = "C4";
    break;

  default:
    (void) fprintf(stderr, "affiche_variable [N_couches/resultats.c] : variable de type inconnu (%d)\n", type);
    nom_variable = "??";
    break;
  }

  if (VAR_CONSTANTE == variable->type) {
    if (VRAI == historique) {
      HIST_Texte(5, 0, "%s : %g (Constante)\n", nom_variable, variable->valeur_initiale);
    }

    /* Inutile d'afficher  l'cran pour les variables non-ajustes */
    return 0;
  }

  /* Ici, la variable est ajuste */
  if (VRAI == historique) {
    HIST_Texte(3, 0, "%s : %g [min : %g, MAX : %g]\n", nom_variable,
	       variable->contenu.valeur, variable->min, variable->max);
  }
  printf("%s : %g [min : %g, MAX : %g, initialement %g]\n", nom_variable,
	 variable->contenu.valeur, variable->min, variable->max, variable->valeur_initiale);

  return 0;
}
