/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2003, Emmanuel CURIS

   Oprations diverses sur les graphes, plus gnrales que l'EXAFS lui-mme
   (aussi bien mathmatiques que statistiques simples : tri en particulier)
   (les oprations statistiques avances sont dans ../Stats)

   Calculer la primitive d'un graphe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
/* #include "exafs.h" */

#include "graphes.h"
/* #include "interface.h" */
#include "operations.h"

#include "op_primitive.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Graphe *GRF_Primitive_R(const Graphe *graphe, double constante, int methode);

static int GRF_PrimitiveTrapezes_R(const Graphe *graphe, Graphe *resultat);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Routine gnrique d'appel  ~-~-~-~-~-~-~-~-~ */

Graphe *GRF_Primitive(const Graphe *graphe, int methode, union valeur constante, double erreur_constante)
{
  Graphe *primitive = NULL;

  if ( NULL == graphe ) return NULL;

  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
    primitive = GRF_Primitive_R( graphe, constante.reel, methode );
    break;

  case GRF_FCT_COMPLEXE:
    OPTION_A_FAIRE( "Primitive d'une fonction complexe" );
    break;

  case GRF_SERIE:
    if ( NULL == graphe->x ) {
      OPTION_A_FAIRE( "Primitive d'une suite de valeurs, X implicites" );
    } else {
      /* On a en fait une pseudo-fonction relle */
      primitive = GRF_Primitive_R( graphe, constante.reel, methode );
    }
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Genre de graphe non intgrable ou inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->genre_donnees );
  }

  /* Faut-il ajouter aussi l'erreur de la constante ? */
  if ( erreur_constante > 0.0 ) {
    OPTION_A_FAIRE( "Tenir compte de l'erreur de la constante d'intgration" );
  }

  /* Termin : on renvoie le graphe ainsi obtenu */
  return primitive;
}

/* ~-~-~-~-~-~-~-~-~ Intgration des graphes rels ~-~-~-~-~-~-~-~-~ */

static Graphe *GRF_Primitive_R(const Graphe *graphe, double constante, int methode)
{
  char *titre;
  int retour = -5748;
  Graphe *primitive;

  /* Titre du graphe rsultat */
  titre = TXT_genere_chaine( texte_titre_primitive, graphe->nom);

  /* On cre le graphe somme :  la suite, type inconnu -> AUTRE ; il a les mmes X que l'original */
  primitive = GRF_nv_TableX( (Graphe *) graphe, GR_AUTRETYPE, GRF_FCT_REELLE, titre, FAUX, graphe->x, graphe->nombre_point, VRAI );
  free( titre );
  if ( NULL == primitive ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le graphe primitive\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On calcule en fonction de la mthode */
  switch ( methode )
  {
  case INTG_RECTANGLE:
    OPTION_A_FAIRE( "Mthode des rectangles" );
    break;

  case INTG_MEDIAN:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE : mthode du point mdian\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case INTG_TRAPEZE:
    retour = GRF_PrimitiveTrapezes_R( graphe, primitive );
    break;

  case INTG_SIMPSON:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE, mthode de Simpson\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case INTG_BODE:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE, mthode de Bode\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case INTG_SPLINE:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE, mthode des splines\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Mthode inconnue [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, methode);
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR survenue dans le calcul de l'intgrale [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    if ( retour < 0 ) { /* Erreur grave -> le rsultat n'ayant pas de sens, on l'efface */
      GRF_efface( primitive, FAUX, FAUX );
      return NULL;
    }
  }

  /* On ajoute la constante d'intgration */
  GRF_FacteurAdditif( primitive, constante, 0, AXE_Y );

  /* On mets les bornes --- les X sont identiques, donc dj faits ! */
  GRF_bornes( primitive, NE_PAS_FAIRE, DESORDONNE );

  /* Traitement ultrieur, essentiellement graphique ! */

  if ( graphe->affichage != NULL ) {
    GRF_CopierAffichage( primitive, graphe, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_COULEUR | GRF_CP_AFF_STYLE | GRF_CP_AFF_MARQUE );
  }

  /* Types spciaux : on peut deviner le type du graphe intgr */
  if (graphe->type == GR_DERIVEE2)
    GRF_ChangerType( primitive, GR_DERIVEE );
  else if ( (graphe->type == GR_DENSITE) || (graphe->type == GR_HISTOGRAMME) )
    GRF_ChangerType( primitive, GR_REPARTITION ); /* L'intgrale d'une densit de probabilit est une fonction de rpartition ! */

  if (graphe->type == GR_HISTOGRAMME) {
    register double *x, *x_intg;
    unsigned long i;

    (void) fprintf(stderr, "%s,%s l.%u :  reprendre, traitement & gestion des histogrammes => genre de graphes spcial !\n",
		   __FILE__, __FUNCTION__, __LINE__);

    /* Dans les histogrammes, les X indiquent le dbut de la classe, or ce qui est important
       est le milieu de la classe */
    for (i = 0, x = graphe->x, x_intg = primitive->x ; i < graphe->nombre_point - 1; i++, x++, x_intg++) {
      *x_intg = (*(x+1) + *x) / 2.0;
    }
    /* Avec cette mthode, le dernier point du spectre d'intgration est  un X trange, mais bon... */
  }

  /* Calcul termin, RAS */
  return primitive; /* On renvoie le graphe somme */
}

/* -------- Calcul par la mthode des trapzes -------- */

static int GRF_PrimitiveTrapezes_R(const Graphe *graphe, Graphe *primitive)
{
  double somme = 0.0;
  double inter;
  double *x, *y, *y_intg;
  unsigned long i;

  /* Premiere tape : calcul pas  pas de l'intgrale */
  for ( i = 0, x = graphe->x, y = graphe->y, y_intg = primitive->y ; i < graphe->nombre_point - 1 ; i++, x++, y++, y_intg++) {
    *y_intg = somme;
      
    /* On calcule pour le point suivant */
    inter  = *(y + 1) + *y;
    inter *= *(x + 1) - *x;
    inter /= 2.0;
    somme += inter;
  }
  /* On met le dernier point */
  *y_intg = somme;

  /* Cas d'une matrice de covariance en Y : plus compliqu */
  if ( graphe->covariance != NULL ) {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Matrice de covariance en Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +48;
  }

  /* Traitement des erreurs ventuelles */
  if ( graphe->erreur_x != NULL ) {
    if ( graphe->erreur_y != NULL ) {
      (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Erreurs en X et Y\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return +2;
    } else {
      (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Erreurs en X seulement\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return +3;
    }
  } else if ( graphe->erreur_y != NULL ) {
    Graphe *erreurs;

    /* On cre le graphe des erreurs en Y */
    erreurs = GRF_creer_erreurs( primitive, AXE_Y );
    if ( NULL == erreurs) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer les erreurs\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return +1; /* Erreur mineure */
    }

    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Calcul des erreurs en Y\n",
		   __FILE__, __FUNCTION__, __LINE__);

    /* Termin : calcul des bornes */
    GRF_bornes( erreurs, NE_PAS_FAIRE, DESORDONNE );
    return +45;
  }

  return 0; /* Tout s'est bien pass */
}
