/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2002
   Date de cration      : 3 aot 2002

   Toutes les fonctions de traitement statistique de l'E.X.A.F.S.

   Calcul des variances pour une table de valeurs
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "statistiques.h"

/* -------- Calcul de la variance commune d'une population  partir des variances intra-groupes --------\
 |   Renvoie la variance commune (<0 en cas d'erreur)                                                   |
 |                                                                                                      |
 | variances   : la table des variances intra-groupes                                                   |
 | ddl         : nombre de degrs de libert associs [ATTENTION, <0 si variance sur la moyenne]        |
 | nbr_groupes : le nombre de groupes                                                                   |
 | ddl_total   : nombre de degrs de libert associs  l'estimation de la variance inter-groupes       |
 \----------------------------------------------------------------------------------------------------- */

double STAT_VarianceCommune(double *variances, int *ddl, unsigned long nbr_groupes, unsigned long *ddl_total)
{
  unsigned long i, ddl_commun = 0;
  double var_commune = 0.0;

  if ( (NULL == variances) || (NULL == ddl) ) {
    return -1.0;
  }

  for (i = 0 ; i < nbr_groupes ; i++, ddl++, variances++) {
    double var;

    if (0 == *ddl) continue; /* Valeur unique, utiliser la variance n'a pas rellement de sens */

    var = *variances;
    var *= var; /* C'est l'cart-type qui est mmoris... */
    if ( *ddl < 0 ) { /* C'est que l'on a la variance sur la moyenne */
      var *= ( -*ddl + 1 ); /* On veut celle dans la population */
    }
    ddl_commun += (unsigned long) (abs( *ddl ));

    var_commune += var * (double) ddl_commun;
  }

  if (0 == ddl_commun) {
    fprintf(stderr, "STAT_VarianceCommune : degrs de liberts tous nuls, calcul impossible !\n");
    return -1.0;
  }
  var_commune /= (double) ddl_commun;

  /*      (n_1 - 1) S1 + (n_2 - 1) S2 + ... + (n_k - 1) Sk
     S = ---------------------------------------------------
                     (n_1 + n_2 + ... +n_k - k)               */

  if ( ddl_total != NULL ) *ddl_total = ddl_commun;
  return var_commune;
}
