/* LASE - Logiciel d'Analyse des Spectres EXAFS
   LASE - (C) 1998-2003, Emmanuel CURIS

   Modlisation d'une exprience : aspects purement gomtriques
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "detecteur.h"
#include "echantillon.h"
#include "faisceau.h"
#include "graphes.h" /* Pour GRF_nv_GrilleXP */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean Intersection_FaceX(Vecteur3D *u0, Vecteur3D *u, double x, Vecteur3D *sortie);
static boolean Intersection_FaceY(Vecteur3D *u0, Vecteur3D *u, double x, Vecteur3D *sortie);
static boolean Intersection_FaceZ(Vecteur3D *u0, Vecteur3D *u, double x, Vecteur3D *sortie);
static int     IndexCaseSuivante(Vecteur3D *u0, Vecteur3D *u, Vecteur3D *sortie, int *d_ix, int *d_iy, int *d_iz);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* Coordonnes de l'intersection entre le faisceau et le plan de l'chantillon  la profondeur donne */

int FAISCEAU_ProjeterCentre(Faisceau *faisceau, double profondeur, double *x, double *y)
{
  int retour;
  Vecteur3D v_directeur, point_vise;

  if ( NULL == faisceau ) return -1;

  /* Direction du faisceau */
  REPERE_VecteurNormal( &(faisceau->orientation), &v_directeur );
  if ( 0.0 == v_directeur.y ) {
    (void) fprintf(stderr, "FAISCEAU_ProjeterCentre : Faisceau parallle  l'chantillon\n");
    return -2;
  }

  /* Intersection de cette droite avec le plan y=profondeur */
  retour = U3D_Inter_Droite_Plan_YCt( &(faisceau->orientation.position), &v_directeur, profondeur, &point_vise);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "FAISCEAU_ProjeterCentre : ERREUR dans U3D_Inter_Droite_Plan_YCt()\n");
    return -3;
  }

  if ( x != NULL ) *x = point_vise.x;
  if ( y != NULL ) *y = point_vise.z;

  /* Calcul termin sans encombre */
  return 0;
}

int FAISCEAU_ProjeterLimites(Faisceau *faisceau, double profondeur, double *x_coins, double *y_coins)
{
  Vecteur3D v_directeur, point, point_vise;

  if ( NULL == faisceau ) return -1;

  /* Direction du faisceau */
  REPERE_VecteurNormal( &(faisceau->orientation), &v_directeur );
  if ( 0.0 == v_directeur.y ) {
    (void) fprintf(stderr, "FAISCEAU_ProjeterLimites : Faisceau parallle  l'chantillon\n");
    return -2;
  }

  /* Coordonnes du 5e point : centre du faisceau */
  point.x = 0.0;
  point.y = 0.0;
  point.z = 0.0;

  REPERE_LocalVersAbsolu( &point, &(faisceau->orientation), &point );

  U3D_Inter_Droite_Plan_YCt( &point, &v_directeur, profondeur, &point_vise);
  x_coins[ 4 ] = point_vise.x;
  y_coins[ 4 ] = point_vise.z;

  /* Contrle de cohrence des changements de repre */
/*   printf( "(%g, %g, %g) = (%g, %g, %g)\n", faisceau->orientation.position.x, faisceau->orientation.position.y, faisceau->orientation.position.z,  */
/* 	  point.x, point.y, point.z ); */
/*   REPERE_AbsoluVersLocal( &point, &(faisceau->orientation), &point ); */
/*   printf( "(%g, %g, %g) = (%g, %g, %g)\n", 0.0, 0.0, 0.0, point.x, point.y, point.z ); */

  if ( faisceau->proprietes & FAISCEAU_PONCTUEL ) {
    unsigned int i;

#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FAISCEAU_ProjeterLimites : Faisceau ponctuel !\n");
#endif

    for ( i = 0 ; i < 4 ; i++ ) {
      x_coins[ i ] = x_coins[ 4 ];
      y_coins[ i ] = y_coins[ 4 ];
    }

    return 0;
  }

  if ( faisceau->proprietes & FAISCEAU_FOCALISE ) {
    (void) fprintf(stderr, "FAISCEAU_ProjeterLimites :  FAIRE --- faisceau focalis\n");
    return -5;
  }

  /* Faisceau rectangulaire... */
  /* 1. Coin haut-gauche */
  point.x = - faisceau->largeur/2.0;
  point.y = 0.0;
  point.z = + faisceau->hauteur/2.0;

  REPERE_LocalVersAbsolu( &point, &(faisceau->orientation), &point );

  U3D_Inter_Droite_Plan_YCt( &point, &v_directeur, profondeur, &point_vise);
  x_coins[ 0 ] = point_vise.x;
  y_coins[ 0 ] = point_vise.z;

  /* Contrle de cohrence des changements de repre */
/*   REPERE_AbsoluVersLocal( &point, &(faisceau->orientation), &point ); */
/*   printf( "(%g, %g, %g) = (%g, %g, %g)\n", -faisceau->largeur / 2.0, 0.0, faisceau->hauteur / 2.0, point.x, point.y, point.z ); */

  /* 2. Coin haut-droite */
  point.x = + faisceau->largeur/2.0;
  point.y = 0.0;
  point.z = + faisceau->hauteur/2.0;

  REPERE_LocalVersAbsolu( &point, &(faisceau->orientation), &point );

  U3D_Inter_Droite_Plan_YCt( &point, &v_directeur, profondeur, &point_vise);
  x_coins[ 1 ] = point_vise.x;
  y_coins[ 1 ] = point_vise.z;

  /* 3. Coin bas-gauche */
  point.x = - faisceau->largeur/2.0;
  point.y = 0.0;
  point.z = - faisceau->hauteur/2.0;

  REPERE_LocalVersAbsolu( &point, &(faisceau->orientation), &point );

  U3D_Inter_Droite_Plan_YCt( &point, &v_directeur, profondeur, &point_vise);
  x_coins[ 3 ] = point_vise.x;
  y_coins[ 3 ] = point_vise.z;

  /* 2. Coin bas-droite */
  point.x = + faisceau->largeur/2.0;
  point.y = 0.0;
  point.z = - faisceau->hauteur/2.0;

  REPERE_LocalVersAbsolu( &point, &(faisceau->orientation), &point );

  U3D_Inter_Droite_Plan_YCt( &point, &v_directeur, profondeur, &point_vise);
  x_coins[ 2 ] = point_vise.x;
  y_coins[ 2 ] = point_vise.z;

  /*Calcul termin */
  return 0;
}

int ECH_TrouverSortieFaisceau(Echantillon *echantillon, Vecteur3D *point_entree, Vecteur3D *direction, Vecteur3D *point_sortie, boolean tester_face_entree)
{
  double t;

  if ( NULL == echantillon ) return -1;
  if ( ( NULL == point_entree ) || ( NULL == direction ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : direction ou entre du faisceau non indique\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -2;
  }
  if ( NULL == point_sortie ) {
    (void) fprintf(stderr, "TrouverSortieFaisceau : impossible de mmoriser le rsultat\n");
    return -3;
  }

  /* On cherche par les faces d'entre (Y=0) et de sortie (Y=epaisseur) */
  if ( 0.0 != direction->y ) {
    /* Face oppose : face de sortie la plus probable */
    t = ( echantillon->epaisseur - point_entree->y ) / ( direction->y );
    if ( t >= 0.0 ) { /* On va bien dans la bonne direction */
      point_sortie->x = point_entree->x + direction->x * t;
      point_sortie->y = echantillon->epaisseur;
      point_sortie->z = point_entree->z + direction->z * t;

      /* Contrle : ce point est-il bien dans l'chantillon ? */
      if ( ( point_sortie->x >= -echantillon->larg / 2.0 ) && ( point_sortie->x <= echantillon->larg / 2.0 ) &&
	   ( point_sortie->z >= -echantillon->haut / 2.0 ) && ( point_sortie->z <= echantillon->haut / 2.0 ) )
	return 0;
    }

    if ( VRAI == tester_face_entree ) {
      /* Face d'entre : souvent le cas en fluorescence, mais interdit en transmission ! */
      t = - point_entree->y / ( direction->y );
      if ( t >= 0.0 ) {
	point_sortie->x = point_entree->x + direction->x * t;
	point_sortie->y = 0.0;
	point_sortie->z = point_entree->z + direction->z * t;

	/* Contrle : ce point est-il bien dans l'chantillon ? */
	if ( ( point_sortie->x >= -echantillon->larg / 2.0 ) && ( point_sortie->x <= echantillon->larg / 2.0 ) &&
	     ( point_sortie->z >= -echantillon->haut / 2.0 ) && ( point_sortie->z <= echantillon->haut / 2.0 ) )
	return 0;
      }
    }
  }

  /* On cherche par les faces latrales : plans X = Cte */
  if ( 0.0 != direction->x ) {
    /* Face de gauche : X = -largeur/2 */ 
    t = ( -echantillon->larg / 2.0 - point_entree->x ) / ( direction->x );
    if ( t >= 0.0 ) { /* On va bien dans la bonne direction */
      point_sortie->x = -echantillon->larg / 2.0;
      point_sortie->y = point_entree->y + direction->y * t;
      point_sortie->z = point_entree->z + direction->z * t;

      /* Contrle : ce point est-il bien dans l'chantillon ? */
      if ( ( point_sortie->y >= 0.0 ) && ( point_sortie->y <= echantillon->epaisseur ) &&
	   ( point_sortie->z >= -echantillon->haut / 2.0 ) && ( point_sortie->z <= echantillon->haut / 2.0 ) )
/* 	(void) fprintf(stderr, "%s,%s l.%u : Sortie par la face de gauche\n", */
/* 		       __FILE__, __FUNCTION__, __LINE__); */
	return 0;
    }

    /* Face de droite : X = +largeur/2 */ 
    t = ( +echantillon->larg / 2.0 - point_entree->x ) / ( direction->x );
    if ( t >= 0.0 ) { /* On va bien dans la bonne direction */
      point_sortie->x = +echantillon->larg / 2.0;
      point_sortie->y = point_entree->y + direction->y * t;
      point_sortie->z = point_entree->z + direction->z * t;

      /* Contrle : ce point est-il bien dans l'chantillon ? */
      if ( ( point_sortie->y >= 0.0 ) && ( point_sortie->y <= echantillon->epaisseur ) &&
	   ( point_sortie->z >= -echantillon->haut / 2.0 ) && ( point_sortie->z <= echantillon->haut / 2.0 ) ) {
/* 	(void) fprintf(stderr, "%s,%s l.%u : Sortie par la face de droite\n", */
/* 		       __FILE__, __FUNCTION__, __LINE__); */
	return 0;
      }
    }
  }

  /* On cherche par les faces horizontales : plans Z = Cte */
  if ( 0.0 != direction->z ) {
    /* Base : Z = -hauteur/2 */ 
    t = ( -echantillon->haut / 2.0 - point_entree->z ) / ( direction->z );
    if ( t >= 0.0 ) { /* On va bien dans la bonne direction */
      point_sortie->x = point_entree->x + direction->x * t;
      point_sortie->y = point_entree->y + direction->y * t;
      point_sortie->z = -echantillon->haut / 2.0;

      /* Contrle : ce point est-il bien dans l'chantillon ? */
      if ( ( point_sortie->y >= 0.0 ) && ( point_sortie->y <= echantillon->epaisseur ) &&
	   ( point_sortie->x >= -echantillon->larg / 2.0 ) && ( point_sortie->x <= echantillon->larg / 2.0 ) )
	return 0;
    }

    /* Couvercle : Z = +hauteur/2 */ 
    t = ( +echantillon->haut / 2.0 - point_entree->z ) / ( direction->z );
    if ( t >= 0.0 ) { /* On va bien dans la bonne direction */
      point_sortie->x = point_entree->x + direction->x * t;
      point_sortie->y = point_entree->y + direction->y * t;
      point_sortie->z = +echantillon->haut / 2.0;

      /* Contrle : ce point est-il bien dans l'chantillon ? */
      if ( ( point_sortie->y >= 0.0 ) && ( point_sortie->y <= echantillon->epaisseur ) &&
	   ( point_sortie->x >= -echantillon->larg / 2.0 ) && ( point_sortie->x <= echantillon->larg / 2.0 ) )
	return 0;
    }
  }

  (void) fprintf(stderr, "ECH_TrouverSortieFaisceau : Le faisceau ne croise aucune face de l'chantillon !\n");
  (void) fprintf(stderr, "  x = %g + t * %g\n  y = %g + t * %g\n  z = %g + t * %g\n",
		 point_entree->x, direction->x, point_entree->y, direction->y, point_entree->z, direction->z );
  return -1;
}

/* -------- Aspect du faisceau de fluorescence quand il arrive sur un dtecteur -------- */

Faisceau *FAISCEAU_GeometrieFluorescence(Detecteur *detecteur, double *table_E, unsigned long nbr_E)
{
  Faisceau *f_detecteur = NULL;
  unsigned int proprietes = 0;

  if ( NULL == detecteur ) return NULL;
  if ( NULL == table_E ) return NULL;

  if ( nbr_E > 1 ) { /* On suit plus d'une seule nergie */
    proprietes |= FAISCEAU_POLYCHROMATIQUE;
  }

  if ( detecteur->proprietes & DETECTEUR_A_L_INFINI ) {
    /* Dtecteur suppos plac  l'infini =>
       faisceau parallle allant dans la direction du dtecteur */
    if ( detecteur->proprietes & DETECTEUR_INHOMOGENE ) {
      (void) fprintf(stderr, "FAISCEAU_GeometrieFluorescence :  FAIRE --- Dtecteur  surface inhomogne\n");
    } else {
      /* Dtecteur homogne -> un faisceau ponctuel convient parfaitement ! */
      f_detecteur = FAISCEAU_Creer( 0.0 , 0.0, proprietes | FAISCEAU_PONCTUEL, 0, 1.0 );
    }
  } else {
    if ( detecteur->proprietes & DETECTEUR_INHOMOGENE ) {
      (void) fprintf(stderr, "FAISCEAU_GeometrieFluorescence :  FAIRE --- dtecteur proche, angle solide variable\n");
    } else { /* On peut alors le supposer ponctuel... */
      f_detecteur = FAISCEAU_Creer( 0.0 , 0.0, proprietes | FAISCEAU_PONCTUEL, 0, 1.0 );
    }
  }

  if ( NULL == f_detecteur ) {
    (void) fprintf(stderr, "FAISCEAU_GeometrieFluorescence : ERREUR  la cration du faisceau\n");
    return NULL;
  }

  if ( proprietes & FAISCEAU_POLYCHROMATIQUE ) {
    /* On prpare le graphe qui va contenir le spectre mesur */
    f_detecteur->g_E = GRF_nv_TableX( NULL, GR_FLUOX, GRF_SERIE, NULL, VRAI, table_E, nbr_E, FAUX );
    if ( NULL == f_detecteur->g_E ) {
      (void) fprintf(stderr, "FAISCEAU_GeometrieFluorescence : ERREUR dans GRF_nv_GrilleXP()\n");
      FAISCEAU_Effacer( f_detecteur );
      return NULL;
    }
#ifdef _DEVELOPPE_
    GRF_titre( f_detecteur->g_E, "Faisceau vu par le dtecteur" );
#endif
  }

  /* Construction du faisceau finie */
  return f_detecteur;
}

/* -------- Connaissant le point de dpart et la direction, quelle sera la prochaine case
   traverse par la droite [= faisceau] considre ? --------

   Renvoie la distance parcourue dans la case considre

  echantillon : l'chantillon considr [pour la grille 3D]
  direction   : vecteur directeur de la droite
  p_entree    : point d'entre dans la case [rel]
  i_x, i_y, i_z : en ENTRE, coordonnes de la case de dpart
                  en SORTIE, coordonnes de la case d'arrive [suivante]
  sortie      : VRAI si l'on sort de l'chantillon
 * -------- */

double ECH_TrouverCaseSuivante(Echantillon *echantillon, Vecteur3D *direction, Vecteur3D *p_entree,
			       unsigned int *i_x, unsigned int *i_y, unsigned int *i_z, boolean *sortie)
{
  Vecteur3D point_sortie, point_entree;
  Vecteur3D v_directeur;
  double x_case, y_case, z_case;
  double distance = -1.0;
  int d_ix, d_iy, d_iz;

#ifdef _DEVELOPPE_
  if ( NULL == echantillon ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun chantillon indiqu\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1.0;
  }
#endif
  point_sortie = *p_entree;
  *sortie = FAUX;

#ifdef _DEVELOPPE_
/*   { */
/*     unsigned int i, j, k; */

/*     ECH_XYZ_v_IJK( echantillon, p_entree->x, p_entree->y, p_entree->z, &i, &j, &k ); */
/*     if ( ( i != *i_x ) || ( j != *i_y ) || ( k != *i_z ) ) { */
/*       (void) fprintf(stderr, "%s,%s l.%u : Incohrence entre les index de cases et les coordonnes\n", */
/* 		     __FILE__, __FUNCTION__, __LINE__); */
/*       printf( "Point d'entre en (%g, %g, %g) ==> I=%u, J=%u, K=%u\n", p_entree->x, p_entree->y, p_entree->z, i, j, k ); */
/*       printf( " et donn comme en case I=%u, J=%u, K=%u\n", *i_x, *i_y, *i_z ); */
/*       FIN_ANTICIPEE( FIN_ERREUR_GRAVE ); */
/*     } else { */
/*       printf( "%s : Point d'entre en (%g, %g, %g) ==> I=%u, J=%u, K=%u\n", __FUNCTION__, p_entree->x, p_entree->y, p_entree->z, i, j, k ); */
/*     } */
/*   } */
#endif

  /* Cas simples : droites selon les axes */
  if ( ( 0.0 == direction->z ) && ( 0.0 == direction->y ) ) {
    distance = echantillon->pas_x;
    if ( direction->x > 0.0 ) { /* Droite selon les X positifs */
      point_sortie.x += echantillon->pas_x;
      (*i_y)++;
      if ( *i_y >= echantillon->nbr_x ) *sortie = VRAI;
    } else {			/* Droite selon les X ngatifs */
      point_sortie.x -= echantillon->pas_x;
      if ( *i_y > 0 ) {
	(*i_y)--;
      } else {
	*sortie = VRAI;
      }
    }

    return distance;
  }
  if ( ( 0.0 == direction->z ) && ( 0.0 == direction->x ) ) {
    distance = echantillon->pas_y;
    if ( direction->y > 0.0 ) { /* Droite selon les Y positifs */
      point_sortie.y += echantillon->pas_y;
      (*i_y)++;
      if ( *i_y >= echantillon->nbr_y ) *sortie = VRAI;
    } else {			/* Droite selon les Y ngatifs */
      point_sortie.y -= echantillon->pas_y;
      if ( *i_y > 0 ) {
	(*i_y)--;
      } else {
	*sortie = VRAI;
      }
    }

    return distance;
  }
  if ( ( 0.0 == direction->x ) && ( 0.0 == direction->y ) ) {
    distance = echantillon->pas_z;
    if ( direction->z > 0.0 ) { /* Droite selon les Z positifs */
      point_sortie.z += echantillon->pas_z;
      (*i_z)++;
      if ( *i_z >= echantillon->nbr_z ) *sortie = VRAI;
    } else {			/* Droite selon les Z ngatifs */
      point_sortie.z -= echantillon->pas_z;
      if ( *i_z > 0 ) {
	(*i_z)--;
      } else {
	*sortie = VRAI;
      }
    }

    return distance;
  }

  /* On trouve sur quelle face est le point d'entre */
  /* a. Coordonnes du coin BG de la case */
  ECH_IJK_v_XYZ( echantillon, *i_x, *i_y, *i_z, &x_case, &y_case, &z_case );
  v_directeur = *direction;

  /* b. On se ramne entre 0 et 1 [Attention aux cases des bords] */
  point_entree.x = (p_entree->x - x_case); point_entree.x /= echantillon->pas_x;
  point_entree.y = (p_entree->y - y_case); point_entree.y /= echantillon->pas_y;
  point_entree.z = (p_entree->z - z_case); point_entree.z /= echantillon->pas_z;
  v_directeur.x /= echantillon->pas_x;
  v_directeur.y /= echantillon->pas_y;
  v_directeur.z /= echantillon->pas_z;

  /* ==> On est maintenant dans un cube de ct 1 plac en (0, 0, 0 ) */

  /* On se dbarasse d'ventuels problmes d'arrondi */
  if ( fabs( point_entree.x ) < 1.0e-5 ) point_entree.x = 0.0;
  if ( fabs( point_entree.y ) < 1.0e-5 ) point_entree.y = 0.0;
  if ( fabs( point_entree.z ) < 1.0e-5 ) point_entree.z = 0.0;

  if ( fabs( 1.0 - point_entree.x ) < 1.0e-5 ) point_entree.x = 1.0;
  if ( fabs( 1.0 - point_entree.y ) < 1.0e-5 ) point_entree.y = 1.0;
  if ( fabs( 1.0 - point_entree.z ) < 1.0e-5 ) point_entree.z = 1.0;

#ifdef _DEVELOPPE_
  if ( ( point_entree.x < 0.0 ) || ( point_entree.x > 1.0 ) ||
       ( point_entree.y < 0.0 ) || ( point_entree.y > 1.0 ) ||
       ( point_entree.z < 0.0 ) || ( point_entree.z > 1.0 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Point d'entre absurde [(%g, %g, %g) : (%u, %u, %u)]\n",
		   __FILE__, __FUNCTION__, __LINE__, point_entree.x, point_entree.y, point_entree.z, *i_x, *i_y, *i_z);
    printf("E=(%g, %g, %g) case=(%g, %g, %g)=(%u,%u,%u) Dx=%g, Dy=%g, Dz=%g\n", p_entree->x, p_entree->y, p_entree->z,
	   x_case, y_case, z_case, *i_x, *i_y, *i_z, echantillon->pas_x, echantillon->pas_y, echantillon->pas_z );
    printf("La case occupe l'espace de (%g, %g, %g)  (%g, %g, %g)\n",
	   x_case, y_case, z_case, x_case + echantillon->pas_x, y_case + echantillon->pas_y, z_case + echantillon->pas_z );
/*     FIN_ANTICIPEE( FIN_ERREUR_GRAVE ); */
  }
#endif

  /* d. On trouve les index de la case suivante et les coordonnes du point de sortie de la case */
  IndexCaseSuivante( &point_entree, &v_directeur, &point_sortie, &d_ix, &d_iy, &d_iz );

#ifdef _DEVELOPPE_
  if ( ( 0 == d_ix ) && ( 0 == d_iy ) && ( 0 == d_iz ) ) {
    printf("\nE=(%g, %g, %g) case=(%g, %g, %g)=(%u,%u,%u) Dx=%g, Dy=%g, Dz=%g\n", p_entree->x, p_entree->y, p_entree->z,
	   x_case, y_case, z_case, *i_x, *i_y, *i_z, echantillon->pas_x, echantillon->pas_y, echantillon->pas_z );
    printf( "E=(%g,%g,%g) u=(%g,%g,%g)\n",
	    point_entree.x, point_entree.y, point_entree.z, v_directeur.x,  v_directeur.y, v_directeur.z);
    printf( "S=(%g,%g,%g)=(%d,%d,%d) ", point_sortie.x, point_sortie.y, point_sortie.z, d_ix, d_iy, d_iz );
    printf("\n");
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'incrmentation de case !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -5781247.0;
  }
#endif

  /* e. On repasse le point de sortie dans le repre original */
  point_sortie.x *= echantillon->pas_x;
  point_sortie.y *= echantillon->pas_y;
  point_sortie.z *= echantillon->pas_z;

  point_sortie.x += x_case;
  point_sortie.y += y_case;
  point_sortie.z += z_case;

#ifdef _DEVELOPPE_
/*   printf( "==> (%g,%g,%g)\n", point_sortie.x, point_sortie.y, point_sortie.z ); */
#endif

  /* Distance entre les deux points */
  distance = U3D_Distance( p_entree, &point_sortie );

  /* Nouveau point de sortie */
  *p_entree = point_sortie;

  /* Contrle de sortie */
  if ( ( 0 == *i_x ) && ( d_ix == -1 ) ) { *sortie = VRAI; return distance; }
  if ( ( 0 == *i_y ) && ( d_iy == -1 ) ) { *sortie = VRAI; return distance; }
  if ( ( 0 == *i_z ) && ( d_iz == -1 ) ) { *sortie = VRAI; return distance; }

  /* Nouveaux index de case */
  *i_x += d_ix;
  *i_y += d_iy;
  *i_z += d_iz;

  if ( ( ( echantillon->nbr_x - 1 ) == *i_x ) && ( +1 == d_ix ) ) {
    *sortie = VRAI;
#ifdef _DEVELOPPE_
    printf( "Sortie en X\n" );
#endif
    return distance;
  }
  if ( ( ( echantillon->nbr_y - 1 ) == *i_y ) && ( +1 == d_iy ) ) {
    *sortie = VRAI;
#ifdef _DEVELOPPE_
    printf( "Sortie en Y\n" );
#endif
    return distance;
  }
  if ( ( ( echantillon->nbr_z - 1 ) == *i_z ) && ( +1 == d_iz ) ) {
    *sortie = VRAI;
#ifdef _DEVELOPPE_
    printf( "Sortie en Z\n" );
#endif
    return distance;
  }

#ifdef _DEVELOPPE_
  {
    unsigned int i, j, k;
    double x, y, z;
    int retour;

    retour = ECH_XYZ_v_IJK( echantillon, p_entree->x, p_entree->y, p_entree->z, &i, &j, &k );
    if ( retour != 0 ) {
      /* Ne devrait jamais se produire avec les contrles au-dessus ! */
      (void) fprintf(stderr, "%s,%s l.%u : Point de sortie hors de l'chantillon [dbordements... ?]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      printf( "Point de sortie en (%g, %g, %g) ==> I=%u, J=%u, K=%u\n", p_entree->x, p_entree->y, p_entree->z, i, j, k );
      printf( " et I=%u, J=%u, K=%u [avec dI=%d, dJ=%d, dK=%d] [avec Lx=%u, Ly=%u et Lz=%u cases\n", *i_x, *i_y, *i_z, d_ix, d_iy, d_iz, echantillon->nbr_x, echantillon->nbr_y, echantillon->nbr_z );
/*        FIN_ANTICIPEE( FIN_ERREUR_GRAVE ); */
      *sortie = VRAI;
    } else {
      ECH_IJK_v_XYZ( echantillon, i, j, k, &x, &y, &z );
      if ( ( i != *i_x ) || ( j != *i_y ) || ( k != *i_z ) ) {
	(void) fprintf(stderr, "%s,%s l.%u : Incohrence entre les index de cases et les coordonnes [en rsultats]\n",
		       __FILE__, __FUNCTION__, __LINE__);
	printf( "Point de sortie en (%g, %g, %g) ==> I=%u, J=%u, K=%u\n", p_entree->x, p_entree->y, p_entree->z, i, j, k );
	printf( " et donn comme en case I=%u, J=%u, K=%u [avec dI=%d, dJ=%d, dK=%d]\n", *i_x, *i_y, *i_z, d_ix, d_iy, d_iz );
	printf( " la case le contenant va de (%g, %g, %g)  (%g, %g, %g)\n", x, y, z, x + echantillon->pas_x, y + echantillon->pas_y, z + echantillon->pas_z );
/* 	FIN_ANTICIPEE( FIN_ERREUR_GRAVE ); */
      }
    }
  }
#endif

  /* Termin */
  return distance;
}

/* -------- Comment dcaler les index pour passer  la case suivante ? -------- */

#define EPSILON_INTERSECTION 0.0 /* 1.0e-6 */ /* Pour tre sr de passer dans l'autre case, malgr les arrondis, on dborde lgrement */

static int IndexCaseSuivante(Vecteur3D *u0, Vecteur3D *u, Vecteur3D *sortie, int *d_ix, int *d_iy, int *d_iz)
{
  boolean inter_Fond = FAUX, inter_Droite = FAUX, inter_Haut = FAUX,
    inter_Devant = FAUX, inter_Gauche = FAUX, inter_Bas = FAUX;

  *d_ix = 0; *d_iy = 0; *d_iz = 0;

  /* Cas gnral : il faudrait un algorithme optimis type Kaufman...
     Mais adaptable pour calculer la distance aussi ?
  */

  /* Quelle face croise-t-on en X ? */
  if ( u->x > 0.0 ) {
    inter_Droite = Intersection_FaceX( u0, u, 1.0 + EPSILON_INTERSECTION, sortie );
  } else if ( u->x < 0.0 ) {
    inter_Gauche = Intersection_FaceX( u0, u, -EPSILON_INTERSECTION, sortie );
  }

  /* Quelle face croise-t-on en Y ? */
  if ( u->y > 0.0 ) {
    inter_Fond   = Intersection_FaceY( u0, u, 1.0 + EPSILON_INTERSECTION, sortie );
  } else if ( u->y < 0.0 ) {
    inter_Devant = Intersection_FaceY( u0, u, -EPSILON_INTERSECTION, sortie );
  }

  /* Quelle face croise-t-on en Z ? */
  if ( u->z > 0.0 ) {
    inter_Haut  = Intersection_FaceZ( u0, u, 1.0 + EPSILON_INTERSECTION, sortie );
  } else if ( u->z < 0.0 ) {
    inter_Bas   = Intersection_FaceZ( u0, u, -EPSILON_INTERSECTION, sortie );
  }

  /* Conclusion */
  if ( VRAI == inter_Droite ) {
    *d_ix = +1;
  }
  if ( VRAI == inter_Fond ) {
    *d_iy = +1;
  }
  if ( VRAI == inter_Haut ) {
    *d_iz = +1;
  }

  if ( VRAI == inter_Gauche ) {
    *d_ix = -1;
  }
  if ( VRAI == inter_Devant ) {
    *d_iy = -1;
  }
  if ( VRAI == inter_Bas ) {
    *d_iz = -1;
  }

#ifdef _DEVELOPPE_
  if ( ( 0 == *d_ix ) && ( 0 == *d_iy ) && ( 0 == *d_iz ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune intersection trouve [u0=(%g, %g, %g) u=(%g, %g, %g)\n",
		   __FILE__, __FUNCTION__, __LINE__, u0->x, u0->y, u0->z, u->x, u->y, u->z);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
#endif

  return 0;
}

/* -------- Trouver l'intersection avec la face indique  -------- */

static boolean Intersection_FaceX(Vecteur3D *u0, Vecteur3D *u, double x, Vecteur3D *sortie)
{
  Vecteur3D p_sur_plan_face;
  double t;

  /* Coordonnes du point d'intersection de la droite et du plan de la face de sortie */
  t = ( x - u0->x ) / u->x;

  p_sur_plan_face.x = x;
  p_sur_plan_face.y = u0->y + u->y * t;
  p_sur_plan_face.z = u0->z + u->z * t;

  /* Le point est sur la face si deux autres coordonnes sont comprises entre 0 et 1
     [On garde cependant une marge de 1/100 pour compenser les arrondis... */
  if ( ( p_sur_plan_face.y < -1.0e-5 ) || ( ( p_sur_plan_face.y - 1.0 ) > 1.0e-5 ) ) return FAUX;
  if ( ( p_sur_plan_face.z < -1.0e-5 ) || ( ( p_sur_plan_face.z - 1.0 ) > 1.0e-5 ) ) return FAUX;

  /* Le point de sortie appartient bien  la face indique */
  *sortie = p_sur_plan_face;

  return VRAI;  
}

static boolean Intersection_FaceY(Vecteur3D *u0, Vecteur3D *u, double y, Vecteur3D *sortie)
{
  Vecteur3D p_sur_plan_face;
  double t;

  /* Coordonnes du point d'intersection de la droite et du plan de la face de sortie */
  t = ( y - u0->y ) / u->y;

  p_sur_plan_face.x = u0->x + u->x * t;
  p_sur_plan_face.y = y;
  p_sur_plan_face.z = u0->z + u->z * t;

  /* Le point est sur la face si deux autres coordonnes sont comprises entre 0 et 1 */
  if ( ( p_sur_plan_face.x < -1.0e-5 ) || ( ( p_sur_plan_face.x - 1.0 ) > 1.0e-5 ) ) return FAUX;
  if ( ( p_sur_plan_face.z < -1.0e-5 ) || ( ( p_sur_plan_face.z - 1.0 ) > 1.0e-5 ) ) return FAUX;

  /* Le point de sortie appartient bien  la face indique */
  *sortie = p_sur_plan_face;

  return VRAI;  
}

static boolean Intersection_FaceZ(Vecteur3D *u0, Vecteur3D *u, double z, Vecteur3D *sortie)
{
  Vecteur3D p_sur_plan_face;
  double t;

  /* Coordonnes du point d'intersection de la droite et du plan de la face de sortie */
  t = ( z - u0->z ) / u->z;

  p_sur_plan_face.x = u0->x + u->x * t;
  p_sur_plan_face.y = u0->y + u->y * t;
  p_sur_plan_face.z = z;

  /* Le point est sur la face si deux autres coordonnes sont comprises entre 0 et 1 */
  if ( ( p_sur_plan_face.x < -1.0e-5 ) || ( ( p_sur_plan_face.x - 1.0 ) > 1.0e-5 ) ) return FAUX;
  if ( ( p_sur_plan_face.y < -1.0e-5 ) || ( ( p_sur_plan_face.y - 1.0 ) > 1.0e-5 ) ) return FAUX;

  /* Le point de sortie appartient bien  la face indique */
  *sortie = p_sur_plan_face;

  return VRAI;  
}
