/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS

   Module d'affichage graphique des rsultats
   Calcul des bornes en X, en Y
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"
#include "interface.h"
#include "figures.h"		/* Inclus tracer.h */

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static int BLOC_GR_BornesXY( Bloc_Figure *bloc );

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */


/* ~-~-~-~-~-~-~-~-~ Recherche des bornes pour un bloc de figure ~-~-~-~-~-~-~-~-~ */

int BLOC_ChoisirBornesOptimales(unsigned int methode, double bornes[ 6 ], void *infos)
{
  Bloc_Figure *bloc = NULL;
  int retour;

  /* On rcupre le bloc : info passe en paramtre */
  bloc = (Bloc_Figure *) infos;

  /* On appelle l'optimisation de blocs */
  retour = BLOC_OptimiserBornes( bloc, (int) methode );
  if ( retour != 0 ) return retour;

  /* On mmorise les bornes */
  bornes[ 0 ] = bloc->dessin.axe_X.min;
  bornes[ 1 ] = bloc->dessin.axe_X.max;

  bornes[ 2 ] = bloc->dessin.axe_Y.min;
  bornes[ 3 ] = bloc->dessin.axe_Y.max;

  /* Termin */
  return 0;
}
 
int BLOC_OptimiserBornes(Bloc_Figure *bloc, int methode)
{
  if ( NULL == bloc ) return -1;

  switch ( bloc->type )
  {
  case BLOCFIGURE_TYPE_GRAPHES:
    if ( BORNES_XY == methode ) {
      return BLOC_GR_BornesXY( bloc );
    } else {
      OPTION_A_FAIRE( "Optimisation de bornes [mthode autre que optimales X/Y]" );
    }
    break;

  default:
    OPTION_A_FAIRE( "Bloc de type autre que graphe" );
  }

  return +5;
}

/* -------- Bloc de graphes, X et Y optimaux pour tous les graphes -------- */

static int BLOC_GR_BornesXY( Bloc_Figure *bloc )
{
  double x_min, x_max, y_min, y_max;
  struct axes_graphe *graphes;
  unsigned long i;

  if ( 0 == bloc->donnees.graphes.nbr_graphes ) {
    /* Aucun graphe : on ne change rien */
    return +1;
  }

  /* Premires valeurs : du premier graphe */
  graphes = bloc->donnees.graphes.infos_graphe;
  x_min = graphes->graphes->x_min;
  x_max = graphes->graphes->x_max;
  y_min = graphes->graphes->y_min;
  y_max = graphes->graphes->y_max;

  /* On voit s'il faut tendre */
  for ( i = 1 ; i < bloc->donnees.graphes.nbr_graphes ; i++ ) {
    if ( NULL == graphes[ i ].graphes ) {
      /* Graphe t de la figure... Ne devrait pas arriver souvent, mais bon... */
      AVERTISSEMENT( "Le graphe %lu du bloc est NULL !", i );
      continue;
    }
    if ( graphes[ i ].graphes->x_min < x_min ) x_min = graphes[ i ].graphes->x_min;
    if ( graphes[ i ].graphes->x_max > x_max ) x_max = graphes[ i ].graphes->x_max;

    if ( graphes[ i ].graphes->y_min < y_min ) y_min = graphes[ i ].graphes->y_min;
    if ( graphes[ i ].graphes->y_max > y_max ) y_max = graphes[ i ].graphes->y_max;
  }

  /* On mmorise les bornes */
  bloc->dessin.axe_X.min = x_min;
  bloc->dessin.axe_X.max = x_max;
  bloc->dessin.axe_Y.min = y_min;
  bloc->dessin.axe_Y.max = y_max;

  /* Termin */
  return 0;
}
