/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998, Emmanuel CURIS

   Module d'affichage graphique des rsultats

   Trac des cadres (lgendes, encadrement,...)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "tracer.h"

#include "cadre.txt"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */

/* -------- Trac de ce qui concerne une rgion de dessin 2D -------- */

void TRC_DessineRegion2D(Fenetre *fenetre, InfosRegion2D *region)
{
  int retour;
  char *legende;

  if ( NULL == region ) return;
  if ( ( region->largeur < 2 ) || ( region->hauteur < 2 ) ) {
    /* Rgion trop petite : on ne va pas tracer ! */
    if ( 1 == region->largeur ) {
      if ( 1 == region->hauteur ) {
	OPTION_A_FAIRE( "Rgion rduite  un point" );
      } else {
	OPTION_A_FAIRE( "Rgion rduite  une droite verticale" );
      }
    } else if ( 1 == region->hauteur ) {
	OPTION_A_FAIRE( "Rgion rduite  une droite horizontale" );
    } else {
      AVERTISSEMENT( "Rgion trop petite : L=%u, H=%u", region->largeur, region->hauteur );
    }
    return;
  }

  /* 1. Trac du fond (s'il existe) */
  if ( region->couleur_fond >= 0 ) {
    TRC_Couleur( region->couleur_fond );
    TRC_RectanglePlein( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur );
  }

  /* 2. Le trac du cadre, si ncessaire */
  if ( region->proprietes & CADRE2D_AVEC_CADRE ) {
#ifdef _DEVELOPPE_
    if ( region->y_BG < region->hauteur ) {
      (void) fprintf(stderr, "%s, %s l.%u : Rgion HORS-FENTRE [X=%d, Y=%d, L=%u, H=%u]\n",
		     __FILE__, __FUNCTION__, __LINE__, region->x_BG, region->y_BG, region->largeur, region->hauteur);
      /* Ce n'est pas un problme quand c'est un axe cran, c'est mme normal ! */
    }
#endif
    TRC_Style( BI_CONTINU );
    TRC_Epaisseur( 1 );
    TRC_Couleur( region->couleur_bord );
    TRC_Rectangle( fenetre, region->x_BG, region->y_BG - region->hauteur + 1, region->largeur, region->hauteur );
  }

  /* Contrle de l'axe des X */
  retour = AXE_ValiderBornes( &(region->axe_X) );
  if ( retour != 0 ) {
    switch ( retour )
    {
    case -3:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_NaN_X );
      break;

    case -2:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Inf_X );
      break;

    case +1:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Id_X );
      break;

    case +2:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Abs_X );
      break;

    default:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_X );
    }
  } else {
    /* 1. Grille ventuelle */
    (void) TRC_Grille( fenetre, region, &(region->axe_X) );

    /* 2. Graduation en bas, toujours prsente */
    (void) TRC_AxeGradue( fenetre, region->x_BG, region->y_BG, region->x_BG + region->largeur - 1, region->y_BG,
			  &(region->axe_X), 2, 8 );
  }

  /* 3. La lgende */
  legende = AXE_LireLegende( &(region->axe_X) );
  if ( legende != NULL ) {
    TRC_Couleur( region->axe_X.couleur_legende );
    TRC_Texte( fenetre, region->x_BG + (region->largeur / 2), region->y_BG + 20, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, legende );

    free( legende ); legende = NULL;
  }

  /* Contrle de l'axe des Y */
  retour = AXE_ValiderBornes( &(region->axe_Y) );
  if ( retour != 0 ) {
    switch ( retour )
    {
    case -3:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_NaN_Y );
      break;

    case -2:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Inf_Y );
      break;

    case +1:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Id_Y );
      break;

    case +2:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Abs_Y );
      break;

    default:
      TRC_Z_Message( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur, texte_erreur_bornes_Y );
    }
  } else {
    /* 1. Grille ventuelle */
    (void) TRC_Grille( fenetre, region, &(region->axe_Y) );

    /* 2. Graduation  gauche, toujours prsente */
    (void) TRC_AxeGradue( fenetre, region->x_BG, region->y_BG, region->x_BG, region->y_BG - region->hauteur + 1,
			  &(region->axe_Y), 2, 8 );
  }

  /* 3. La lgende */
  legende = AXE_LireLegende( &(region->axe_Y) );
  if ( legende != NULL ) {
    TRC_Couleur( region->axe_Y.couleur_legende );
    TRC_Texte( fenetre, region->x_BG - 50, region->y_BG - (region->hauteur / 2), BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_BAS, 90, legende );

    free( legende ); legende = NULL;
  }

  /* On place les axes, si besoin est */
  TRC_PlacerAxes( fenetre, region );
}

/* -------- Calcule les dimensions relatives d'une rgion 2D dans la rgion englobante -------- */

int TRC_AbsoluVRelatif(InfosRegion2D *region, int x_BG, int y_BG, unsigned int largeur, unsigned int hauteur)
{
  int vrai_x_BG, vrai_y_BG;

  if ( NULL == region ) return -1;

  /* 1. On corrige le dcalage d'origine */
  vrai_x_BG = (double) region->x_BG - x_BG;
  vrai_y_BG = (double) region->y_BG - y_BG;

  /* 2. Position relative du coin BG dans la rgion englobante */
  region->x_BG_pc = ( 100.0 * (double) vrai_x_BG ) / (double) largeur;
  region->y_BG_pc = ( 100.0 * (double) vrai_y_BG ) / (double) hauteur;

  /* 3. Taille relative de la rgion dans la rgion englobante */
  region->larg_pc = ( 100.0 * (double) region->largeur ) / (double) largeur;
  region->haut_pc = ( 100.0 * (double) region->hauteur ) / (double) hauteur;

  /* Termin */
  return 0;
}

/* -------- Calcule les dimensions cran d'une rgion 2D en connaissant les dimensions de la rgion englobant -------- */

int TRC_RelatifVAbsolu(InfosRegion2D *region, int x_BG, int y_BG, unsigned int largeur, unsigned int hauteur)
{
  if ( NULL == region ) return -1;

  /* Dimensions */
  region->largeur = rint( region->larg_pc * (double) largeur / 100.0 );
  region->hauteur = rint( region->haut_pc * (double) hauteur / 100.0 );

  /* Position */
  region->x_BG = x_BG + (int) rint( region->x_BG_pc * (double) largeur / 100.0 );
  if ( region->proprietes & CADRE2D_ECRAN ) {
    region->y_BG = y_BG - (int) rint( region->y_BG_pc * (double) hauteur / 100.0 );
  } else {
    region->y_BG = y_BG + (int) rint( region->y_BG_pc * (double) hauteur / 100.0 );
  }

  return 0;
}

/* -------- Renvoie VRAI si le point est dans le rectangle indiqu -------- */

inline boolean TRC_EstDansZone2D(double x_min, double y_min, double x_max, double y_max, double x, double y)
{
  if ( x < x_min ) return FAUX;
  if ( x > x_max ) return FAUX;
  if ( y < y_min ) return FAUX;
  if ( y > y_max ) return FAUX;

  return VRAI;
}

/* -------- Renvoie la position du point (x,y) dans la zone (bord, centre,...) -------- */

unsigned int TRC_PositionDansRegion2D(InfosRegion2D *region, int x, int y, unsigned int marge)
{
  int x_milieu, y_milieu;
  unsigned int position = TRC_EST_AU_CENTRE;

  if ( NULL == region ) return TRC_EST_HORS_REGION;

  /* On limine les points hors rgion, marges incluses */
  if ( x < (int) ( region->x_BG - marge ) ) return TRC_EST_HORS_REGION;
  if ( x > (int) ( region->x_BG + region->largeur + marge ) ) return TRC_EST_HORS_REGION;
  if ( y > (int) ( region->y_BG + marge ) ) return TRC_EST_HORS_REGION;
  if ( y < (int) ( region->y_BG - region->hauteur - marge ) ) return TRC_EST_HORS_REGION;

  /* On place les bits bord  bord */
  if ( x <= (int) ( region->x_BG + marge ) )                   position |= TRC_EST_BORD_GAUCHE;
  if ( x >= (int) ( region->x_BG + region->largeur - marge ) ) position |= TRC_EST_BORD_DROIT;
  if ( y >= (int) ( region->y_BG - marge ) )                   position |= TRC_EST_BORD_BAS;
  if ( y <= (int) ( region->y_BG - region->hauteur + marge ) ) position |= TRC_EST_BORD_HAUT;

  /* Calcul des milieux */
  x_milieu = region->x_BG + ( region->largeur / 2 );
  y_milieu = region->y_BG - ( region->hauteur / 2 );

  /* On place les bits pour le milieu */
  if ( ( x <= ( x_milieu + (int) marge ) ) && ( x >= ( x_milieu - (int) marge ) ) ) position |= TRC_EST_MILIEU_HOR;
  if ( ( y <= ( y_milieu + (int) marge ) ) && ( y >= ( y_milieu - (int) marge ) ) ) position |= TRC_EST_MILIEU_VER;

  return position;
}

/* -------- Intersection d'un segment et d'un cadre : garder la partie intrieure au cadre
   Ne renvoie rien

 x_min, y_min       : coordonnes du coin BG de la rgion valable
 x_max, y_max       : coordonnes du coin HD de la rgion valable
 x_dedans, y_dedans : coordonnes du point dans la rgion
 x_dehors, y_dehors : coordonnes du point hors de la rgion
 x_bord, y_bord     : coordonnes de la limite du segment, pour arriver sur le bord de la rgion
 -------------------------------------------------------------------------------- */

void TRC_PointIntersection(double x_min, double y_min, double x_max, double y_max,
			   double x_dedans, double y_dedans, double x_dehors, double y_dehors,
			   double *x_bord, double *y_bord)
{
  double x_vrai, y_vrai;

  if ( x_dehors < x_min ) { /* On tait dehors  gauche */
    x_vrai = x_min;
    y_vrai = y_dehors + ( x_vrai - x_dehors ) * ( y_dedans - y_dehors ) / ( x_dedans - x_dehors );
    if ( y_vrai < y_min ) { /* Point hors : coin BG */
      y_vrai = y_min;
      x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
    } else if ( y_dehors > y_max ) { /* Point hors : coin HG */
      y_vrai = y_max;
      x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
    }
  } else if ( x_dehors > x_max ) { /* On tait dehors  droite */
    x_vrai = x_max;
    y_vrai = y_dehors + ( x_vrai - x_dehors ) * ( y_dedans - y_dehors ) / ( x_dedans - x_dehors );
    if ( y_vrai < y_min ) { /* Point hors : coin BG */
      y_vrai = y_min;
      x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
    } else if ( y_dehors > y_max ) { /* Point hors : coin HG */
      y_vrai = y_max;
      x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
    }
  } else if ( y_dehors > y_max ) { /* On tait dehors par le flanc haut */
    y_vrai = y_max;
    x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
  } else { /* On tait dehors par le flanc bas */
    y_vrai = y_min;
    x_vrai = x_dehors + ( y_vrai - y_dehors ) * ( x_dedans - x_dehors ) / ( y_dedans - y_dehors );
  }

  *x_bord = x_vrai;
  *y_bord = y_vrai;
}

/* -------- Calcul des chelles en X et en Y pour une zone 2D --------
 |   Renvoie 0 si tout s'est bien pass
 |
 | region : la rgion du plan reprsente  l'cran
 | axes_secondaires : pour quels axes faut-il utiliser les axes secondaires ? |
 | echelle_X        : le facteur d'chelle pour les X [en vrai/pixel]         |
 | echelle_Y        : le facteur d'chelle pour les Y [en vrai/pixel]         |
 \--------------------------------------------------------------------------- */

int TRC_Z2D_Echelles(InfosRegion2D *region, unsigned int axes_secondaires, double *echelle_X, double *echelle_Y)
{
  InfosAxe *axe;

  if ( NULL == region ) return -1;

  /* Si l'on s'intresse  l'chelle en X */
  if ( echelle_X != NULL ) {
    /* On rcupre les axes  utiliser */
    if ( ( axes_secondaires & AXE_X ) && ( region->X_secondaire != NULL ) ) {
      axe = region->X_secondaire;
    } else {
      axe = &(region->axe_X);
    }

    *echelle_X = AXE_Echelle( axe, region->x_BG, region->x_BG + region->largeur - 1 );
    if ( 0.0 == *echelle_X ) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur en calculant l'chelle en X\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }
  }

  /* Si l'on s'intresse  l'chelle en Y */
  if ( echelle_Y != NULL ) {
    if ( ( axes_secondaires & AXE_Y ) && ( region->Y_secondaire != NULL ) ) {
      axe = region->Y_secondaire;
    } else {
      axe = &(region->axe_Y);
    }

    /* Attention, il faut un facteur d'chelle positif => on permute min et max */
    *echelle_Y = AXE_Echelle( axe, region->y_BG - ( region->hauteur - 1 ), region->y_BG );
    if ( 0.0 == *echelle_Y ) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur en calculant l'chelle en Y\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -3;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Convertir des coordonnes cran en coordonnes relles d'une zone 2D --------
 |   Renvoie 0 si tout s'est bien pass
 |
 | region : la rgion du plan reprsente  l'cran
 | axes_secondaires : pour quels axes faut-il utiliser les axes secondaires ? |
 | x_ecran, y_ecran : coordonnes ABSOLUES du point  l'cran                 |
 | x, y             : coordonnes dans le systme d'axes de la rgion du plan |
 \--------------------------------------------------------------------------- */

int TRC_Z2D_EcranVXY(InfosRegion2D *region, unsigned int axes_secondaires, int x_ecran, int y_ecran, double *x, double *y)
{
  double echelle_X, echelle_Y;
  int retour;
  InfosAxe *axe_X, *axe_Y;

  /* Calcul des chelles en X & en Y */
  retour = TRC_Z2D_Echelles( region, axes_secondaires, &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans TRC_Z2D_Echelles, code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -1;
  }

  /* On rcupre les axes  utiliser */
  if ( ( axes_secondaires & AXE_X ) && ( region->X_secondaire != NULL ) ) {
    axe_X = region->X_secondaire;
  } else {
    axe_X = &(region->axe_X);
  }

  if ( ( axes_secondaires & AXE_Y ) && ( region->Y_secondaire != NULL ) ) {
    axe_Y = region->Y_secondaire;
  } else {
    axe_Y = &(region->axe_Y);
  }

  /* Conversion */
  if ( x != NULL ) *x = TRC_ConvertitX_EvP( x_ecran, axe_X->min, region->x_BG, echelle_X );
  if ( y != NULL ) *y = TRC_ConvertitY_EvP( y_ecran, axe_Y->min, region->y_BG, echelle_Y );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Convertir des coordonnes relles d'une zone 2D en coordonnes cran --------
 |   Renvoie 0 si tout s'est bien pass
 |
 | region           : la rgion du plan reprsente  l'cran                 |
 | axes_secondaires : pour quels axes faut-il utiliser les axes secondaires ? |
 | x_ecran, y_ecran : coordonnes ABSOLUES du point  l'cran                 |
 |                    on arrondit  l'entier le plus proche, en X et en Y     |
 | x, y             : coordonnes dans le systme d'axes de la rgion du plan |
 \--------------------------------------------------------------------------- */

int TRC_Z2D_XYVEcran(InfosRegion2D *region, unsigned int axes_secondaires, double x, double y, int *x_ecran, int *y_ecran)
{
  double echelle_X, echelle_Y;
  int retour;
  InfosAxe *axe_X, *axe_Y;

  /* Calcul des chelles en X & en Y */
  retour = TRC_Z2D_Echelles( region, axes_secondaires, &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans TRC_Z2D_Echelles, code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -1;
  }

  /* Slection des axes [en double ici, mais pas gnant ; en revanche, second contrle inutile] */
  if ( ( axes_secondaires & AXE_X ) && ( region->X_secondaire != NULL ) ) {
    axe_X = region->X_secondaire;
  } else {
    axe_X = &(region->axe_X);
  }

  if ( ( axes_secondaires & AXE_Y ) && ( region->Y_secondaire != NULL ) ) {
    axe_Y = region->Y_secondaire;
  } else {
    axe_Y = &(region->axe_Y);
  }

  /* Conversion */
  if ( x_ecran != NULL ) *x_ecran = TRC_ConvertitX_PvE( x, axe_X->min, region->x_BG, echelle_X );
  if ( y_ecran != NULL ) *y_ecran = TRC_ConvertitY_PvE( y, axe_Y->min, region->y_BG, echelle_Y );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Initialiser une rgion 2D pour l'adapter  un graphe & une fentre -------- */

int TRC_Z2D_Initialiser(InfosRegion2D *region, Fenetre *fenetre, struct graphe *graphe)
{
  if ( NULL == region ) return -1;

  if ( fenetre != NULL ) {
    /* La rgion occupe toute la fentre  l'cran */
    region->x_BG = 0; region->y_BG = fenetre->w_haut - 1;
    region->largeur = fenetre->w_larg;
    region->hauteur = fenetre->w_haut;

    /* On met  NULL ce qui ne sert  rien, pour viter d'ventuelles mauvaises surprises ! */
    region->axe_X.legende = NULL; region->axe_X.unite   = NULL;
    region->axe_Y.legende = NULL; region->axe_Y.unite   = NULL;
    region->X_secondaire  = NULL;
    region->Y_secondaire  = NULL;

    /* On met une couleur pour le cadre */
    region->couleur_bord = NOIR;
  }

  if ( graphe != NULL ) {
    region->axe_X.min = graphe->x_min;
    region->axe_X.max = graphe->x_max;

    region->axe_Y.min = graphe->y_min;
    region->axe_Y.max = graphe->y_max;
  }

  /* Tout s'est bien pass */
  return 0;
}
