/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Module d'affichage graphique

   Figures gomtriques de base
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */

/* -------- Calculer la portion de [segment de ] droite qui tombe dans le rectangle indiqu
     On travaille en double car utilisable dans l'espace rel, avant conversion en coordonnes cran !

     Renvoie  0 si tout va bien,
             >0 si le segment est entirement hors du rectangle,
	     <0 en cas d'erreur

     Les coordonnes du segment aprs intersection REMPLACENT celles avant dans x0, y0, x1, y1
   -------- */

int TRC_LimiterSegment(double x_min, double x_max, double y_min, double y_max, double *x0, double *y0, double *x1, double *y1)
{
  double pente, OalO;

  /* Contrles */
  if ( ( NULL == x0 ) || ( NULL == y0 ) || ( NULL == x1 ) || ( NULL == y1 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : NULL pass pour les segments !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  if ( x_max < x_min ) {
    double x = x_max; x_max = x_min; x_min = x;
  }
  if ( y_max < y_min ) {
    double y = y_max; y_max = y_min; y_min = y;
  }
  if ( ( x_max == x_min ) || ( y_max == y_min ) ) return +1; /* Rduit  un point : sans intrt ! */
  
  /* On s'arrange pour avoir x0 minimal, x1 maximal */
  if ( *x0 > *x1 ) {
    double xy;

    xy = *x0; *x0 = *x1; *x1 = xy;
    xy = *y0; *y0 = *y1; *y1 = xy;
  }

  /* Cas faciles : segment hors de la zone */
  if ( ( *x1 <= x_min ) || ( *x0 >= x_max ) ) return +2;
  if ( *y1 > *y0 ) {		/* Droite croissante */
    if ( ( *y1 <= y_min ) || ( *y0 <= y_max ) ) return +3;
  } else {			/* Droite dcroissante */
    if ( ( *y0 <= y_min ) || ( *y1 <= y_max ) ) return +4;
  }

  /* Si on est l, au moins une partie du segment est dans le rectangle */

  /* Cas facile : droite verticale */
  if ( *x0 == *x1 ) {
    /* *x0 = *x1 = X, forcment dans la zone => il suffit de regarder les Y ! */

    /* On s'arrange pour avoir *y0 le plus petit des deux */
    if ( *y0 > *y1 ) {
      double y = *y0; *y0 = *y1; *y1 = y;
    }
    if ( *y0 < y_min ) *y0 = y_min;
    if ( *y1 > y_max ) *y1 = y_max;
  }

  /* Cas facile : droite horizontale */
  if ( *y0 == *y1 ) {
    /* *y0 = *y1 = Y, forcment dans la zone => il suffit de regarder les X ! */

    if ( *x0 < x_min ) *x0 = x_min;
    if ( *x1 > x_max ) *x1 = x_max;
  }

  /* Calcul de l'quation de la droite : Y = Y0 + pente * (X - X0) = pente * X + (Y0 - pente * X0) */
  pente = ( *y1 - *y0 ) / ( *x1 - *x0 );
  OalO  = *y0 - pente * *x0;

  /* Point de dpart */
  if ( *x0 < x_min ) {
    double y_entree;

    /* X hors de la zone */
    /* a. Calculer le Y pour le X d'entre de zone */
    y_entree = OalO + pente * x_min;

    /* b. Ce Y convient-il ? */
    if ( y_entree < y_min ) {
      /* On entre donc par le bord bas */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y0 = y_min;
      *x0 = ( y_min - OalO ) / pente;
    } else if ( y_entree > y_max ) {
      /* On entre donc par le bord haut */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y0 = y_max;
      *x0 = ( y_max - OalO ) / pente;
    } else {
      /* Y de dbut correct : on entre par le ct gauche */
      *x0 = x_min;
      *y0 = y_entree;
    }
  } else {
    /* X est dans la zone, mais qu'en est-il de Y ? */
    if ( *y0 < y_min ) {
      /* On entre donc par le bord bas */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y0 = y_min;
      *x0 = ( y_min - OalO ) / pente;
    } else if ( *y0 > y_max ) {
      /* On entre donc par le bord haut */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y0 = y_max;
      *x0 = ( y_max - OalO ) / pente;
    } else {
      /* Point entirement dans la zone : rien  faire ! */
    }
  }

  /* Contrles : passe-t-on en-dessous ou au-dessus de la zone ? */
  if ( *x0 > x_max ) return +5;	/* Le point d'entre est aprs la zone : segment [x0, y0], [x1, y1] passe dessus ou dessous ! */

  /* Point d'arrive */
  if ( *x1 > x_max ) {
    double y_sortie;

    /* X hors de la zone */
    /* a. Calculer le Y pour le X de sortie de zone */
    y_sortie = OalO + pente * x_min;

    if ( y_sortie < y_min ) {
      /* On sort par le ct bas */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y1 = y_min;
      *x1 = ( y_min - OalO ) / pente;
    } else if ( y_sortie > y_max ) {
      /* On sort par le bord haut */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y1 = y_max;
      *x1 = ( y_max - OalO ) / pente;
    } else {
      /* Y de fin correct : on entre par le ct droit */
      *x1 = x_max;
      *y1 = y_sortie;
    }
  } else {
    /* X est dans la zone, mais qu'en est-il de Y ? */
    if ( *y1 < y_min ) {
      /* On sort donc par le bord bas */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y1 = y_min;
      *x1 = ( y_min - OalO ) / pente;
    } else if ( *y1 > y_max ) {
      /* On sort donc par le bord haut */
      /* ==> il suffit d'inverser l'quation de la droite */
      *y1 = y_max;
      *x1 = ( y_max - OalO ) / pente;
    } else {
      /* Point entirement dans la zone : rien  faire ! */
    }
  }

  /* Termin sans souci */
  return 0;
}

/* -------- Tracer un sparateur dans une fentre --------
   fenetre : la fentre o placer le sparateur
   sens    : le sens de trac du sparateur --- si VERTICAL, trace un sparateur vertical
   x_debut : coordonnes du point de dpart du sparateur [en coordonnes internes  la zone de travail]
   y_debut :
   dimension : du sparateur, en pixels [va donc de debut  debut + dimension - 1]
*/

void TRC_Separateur(Fenetre *fenetre, int sens, int x_debut, int y_debut, int dimension)
{
  if ( NULL == fenetre ) return;

  TRC_Couleur( NOIR );
  TRC_Style( BI_CONTINU );
  TRC_Epaisseur( 1 );

  /* On dessine : le sparateur est un simple trait noir continu, actuellement */
  if ( VERTICAL == sens ) {
    TRC_LigneV( fenetre, x_debut, y_debut, dimension );
  } else {
    TRC_LigneH( fenetre, x_debut, y_debut, dimension );
  }
}

/* ~-~-~-~-~-~-~-~ Primitives de dessin 3D [OpenGL] ~-~-~-~-~-~-~-~ */

void TRC_Couleur_OpenGL(int couleur)
{
#ifdef AVEC_OPEN_GL_
  switch ( couleur )
  {
  case BLANC:  glColor3ub( 255, 255, 255 ); return;
  case NOIR:   glColor3ub(   0,   0,   0 ); return;
  case ROUGE:  glColor3ub( 255,   0,   0 ); return;
  case VERT:   glColor3ub(   0, 255,   0 ); return;
  case BLEU:   glColor3ub(   0,   0, 255 ); return;
  case CYAN:   glColor3ub(   0, 255, 255 ); return;
  case JAUNE:  glColor3ub( 255, 255,   0 ); return;
  case VIOLET: glColor3ub( 255,   0, 255 ); return;
  case GRIS:   glColor3ub( 128, 128, 128 ); return;
  case GRIS_FONCE:   glColor3ub(  64,  64,  64 ); return;
  case ROUGE_FONCE:  glColor3ub( 128,   0,   0 ); return;
  case VERT_FONCE:   glColor3ub(   0, 128,   0 ); return;
  case BLEU_FONCE:   glColor3ub(   0,   0, 128 ); return;
  case CYAN_FONCE:   glColor3ub(   0, 128, 128 ); return;
  case JAUNE_FONCE:  glColor3ub( 128, 128,   0 ); return;
  case VIOLET_FONCE: glColor3ub( 128,   0, 128 ); return;
  default:
    (void) fprintf(stderr, "%s,%s l.%u : Index de couleur inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, couleur);
  }
#else
  (void) fprintf( stderr, "%s : couleur = %d, pas d'OpenGL actif\n", __FUNCTION__, couleur );
#endif /* AVEC_OPEN_GL_ */
}

/* -------- Bote en 3D : parallllpipde avec cts parallles aux axes -------- */

void TRC_Boite3D(Vecteur3D *coin_BGAv, Vecteur3D *coin_HDAr, int couleur_faces,
		 int couleur_bord, double epaisseur_bord)
{
  if ( ( NULL == coin_BGAv ) || ( NULL == coin_HDAr ) ) {
    (void) fprintf( stderr, "%s : Coin manquant ! \n", __FUNCTION__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* 1. Trac des faces, si ncessaire */
  if ( couleur_faces >= 0 ) {
    OPTION_A_FAIRE( "Tracer les faces" );
  }

  /* 2. Trac du bord, si ncessaire */
  if ( ( couleur_bord >= 0 ) && ( epaisseur_bord > 0.0 ) ) {
#ifdef AVEC_OPEN_GL_
    glLineWidth( epaisseur_bord ); /* paisseur dcente... */
    TRC_Couleur_OpenGL( couleur_bord );

    glBegin(GL_LINE_LOOP); { /* Face basse */
      glVertex3f( coin_BGAv->x, coin_BGAv->y, coin_BGAv->z ); /* Coin GBAv */
      glVertex3f( coin_BGAv->x, coin_HDAr->y, coin_BGAv->z ); /* Coin GBAr */
      glVertex3f( coin_HDAr->x, coin_HDAr->y, coin_BGAv->z ); /* Coin DBAr */
      glVertex3f( coin_HDAr->x, coin_BGAv->y, coin_BGAv->z ); /* Coin DBAv */
    } glEnd();

    glBegin(GL_LINE_LOOP); { /* Face haute */
      glVertex3f( coin_BGAv->x, coin_BGAv->y, coin_HDAr->z ); /* Coin GHAv */
      glVertex3f( coin_BGAv->x, coin_HDAr->y, coin_HDAr->z ); /* Coin GHAr */
      glVertex3f( coin_HDAr->x, coin_HDAr->y, coin_HDAr->z ); /* Coin DHAr */
      glVertex3f( coin_HDAr->x, coin_BGAv->y, coin_HDAr->z ); /* Coin DHAv */
    } glEnd();

    glBegin(GL_LINES); { /* Artes verticales, reliant les deux */
      glVertex3f( coin_BGAv->x, coin_BGAv->y, coin_BGAv->z ); /* Coin GBAv */
      glVertex3f( coin_BGAv->x, coin_BGAv->y, coin_HDAr->z ); /* Coin GHAv */

      glVertex3f( coin_BGAv->x, coin_HDAr->y, coin_BGAv->z ); /* Coin GBAr */
      glVertex3f( coin_BGAv->x, coin_HDAr->y, coin_HDAr->z ); /* Coin GHAr */

      glVertex3f( coin_HDAr->x, coin_HDAr->y, coin_BGAv->z ); /* Coin DBAr */
      glVertex3f( coin_HDAr->x, coin_HDAr->y, coin_HDAr->z ); /* Coin DHAr */

      glVertex3f( coin_HDAr->x, coin_BGAv->y, coin_BGAv->z ); /* Coin DBAv */
      glVertex3f( coin_HDAr->x, coin_BGAv->y, coin_HDAr->z ); /* Coin DHAv */
    } glEnd();
#endif /* AVEC_OPEN_GL_ */
  }
}
