/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 1999, Emmanuel CURIS
   Cration du fichier : 2 aot 1999

   Options de dessin, au sens large
   Tout ce qui concerne le choix de zones de redessin
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define TAILLE_BORDURES 5 /* Taille des bordures en pixels, doit tre > 0 */

#define TRC_Zone_InitialiserDessin { TRC_Mode( BI_MODE_INVERSE ); TRC_Style( BI_POINTS ); TRC_Couleur( ROUGE ); }

static struct {
  int x;
  unsigned int largeur;
} coord_X;

static struct {
  int y;
  unsigned int hauteur;
} coord_Y;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void RCT_barre_horizontale(Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_barre_verticale  (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_coin_haut_gauche (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_coin_haut_droit  (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_coin_bas_gauche  (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_coin_bas_droit   (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_deplace          (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_ligne            (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RCT_zoom             (Fenetre *fenetre, int x, int y, int vx_x, int vx_y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Dfinir un rectangle que l'on dplace --------
   Renvoie VRAI si on a pu dfinir le rectangle, FAUX en cas de problme ou d'annulation

   Fenetre = la fentre dans laquelle on doit travailler
   Le rectangle fait (larg, haut).
   Il est trac avec un dcalage de (dx, dy) par rapport  la souris [xHG = x + dx, yHG = y + dy]

   xHG_fin, yHG_fin : coordonnes finales du coin HG du rectangle dplac
 */

int prepare_deplace(Fenetre *fenetre, int dx, int dy, unsigned int larg, unsigned int haut, int *xHG_fin, int *yHG_fin)
{
  int etat, retour;
  int x_clic, y_clic;

  coord_X.x = dx; coord_Y.y = dy;
  coord_X.largeur = larg; coord_Y.hauteur = haut;

  /* On attend le clic de fin */
  retour = BI_AttendClic(fenetre, 0, RCT_deplace, &x_clic, &y_clic, &etat); 
  if ( retour > 1 ) return FAUX; /* Annulation */

  /* On calcule les nouvelles coordonnes du coin HG du rectangle dplac */
  *xHG_fin = x_clic + dx;
  *yHG_fin = y_clic + dy;

  /* On a termin en confirmant */
  return VRAI;
}

static void RCT_deplace(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  /* Paramtres du trac des lignes */
  TRC_Couleur( ROUGE );
  TRC_Style( 3 );
  vswr_mode(_PRG_station->handle, MD_INVERSE);
  
  if ( (vx_x >= fenetre->w_x) && (vx_x <= fenetre->w_x + (int) fenetre->w_larg - 1) &&
       (vx_y >= fenetre->w_y) && (vx_y <= fenetre->w_y + (int) fenetre->w_haut - 1) ) {
    TRC_Rectangle( fenetre, vx_x + coord_X.x, vx_y + coord_Y.y, coord_X.largeur, coord_Y.hauteur );
  }

  if ( (x >= fenetre->w_x) && (x <= fenetre->w_x + (int) fenetre->w_larg - 1) &&
       (y >= fenetre->w_y) && (y <= fenetre->w_y + (int) fenetre->w_haut - 1) ) {
    TRC_Rectangle( fenetre, x + coord_X.x, y + coord_Y.y, coord_X.largeur, coord_Y.hauteur );
  }
}

/* -------- Dfinir un rectangle en vue de faire un grossissement --------
   Renvoie VRAI si on a bien dfini ce rectangle, FAUX si on a annul
   Dmarre le rectangle en (x_dep, y_dep) et renvoie dans (x_fin, y_fin) l'autre coordonne
   Fenetre = la fentre dans laquelle on doit travailler
 */

int prepare_zoom(Fenetre *fenetre, int x_dep, int y_dep, int *x_fin, int *y_fin)
{
  int etat, retour;

  coord_X.x = x_dep;
  coord_Y.y = y_dep;
  retour = BI_AttendClic(fenetre, 0, RCT_zoom, x_fin, y_fin, &etat);

  return (retour > 1) ? FAUX : VRAI;
}

static void RCT_zoom(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  TRC_Mode( MD_INVERSE );
  TRC_Couleur( ROUGE );
  TRC_Style( 3 );
  TRC_Epaisseur( 1 );

  if ( (vx_x >= fenetre->w_x) && (vx_x <= fenetre->w_x + (int) fenetre->w_larg - 1) &&
       (vx_y >= fenetre->w_y) && (vx_y <= fenetre->w_y + (int) fenetre->w_haut - 1) ) {
    if ( vx_x >= coord_X.x ) {
      if ( vx_y >= coord_Y.y ) {
	TRC_Rectangle( fenetre, coord_X.x, coord_Y.y, vx_x - coord_X.x, vx_y - coord_Y.y );
      } else {
	TRC_Rectangle( fenetre, coord_X.x, vx_y, vx_x - coord_X.x, coord_Y.y - vx_y );
      }
    } else {
      if ( vx_y >= coord_Y.y ) {
	TRC_Rectangle( fenetre, vx_x, coord_Y.y, coord_X.x - vx_x, vx_y - coord_Y.y );
      } else {
	TRC_Rectangle( fenetre, vx_x, vx_y, coord_X.x - vx_x, coord_Y.y - vx_y );
      }
    }
  }

  if ( (x >= fenetre->w_x) && (x <= fenetre->w_x + (int) fenetre->larg - 1) &&
       (y >= fenetre->w_y) && (y <= fenetre->w_y + (int) fenetre->haut - 1) ) {
    if ( x >= coord_X.x ) {
      if ( y >= coord_Y.y ) {
	TRC_Rectangle( fenetre, coord_X.x, coord_Y.y, x - coord_X.x, y - coord_Y.y );
      } else {
	TRC_Rectangle( fenetre, coord_X.x, y, x - coord_X.x, coord_Y.y - y );
      }
    } else {
      if ( y >= coord_Y.y ) {
	TRC_Rectangle( fenetre, x, coord_Y.y, coord_X.x - x, y - coord_Y.y );
      } else {
	TRC_Rectangle( fenetre, x, y, coord_X.x - x, coord_Y.y - y );
      }
    }
  }

  TRC_Mode( MD_REMPLACE ); /* Mode recouvrir */
}

/* -------- Dfinir une position horizontale,  l'aide d'un rectangle
   Renvoie VRAI si on a bien dfini ce rectangle, FAUX si on a annul
   Dmarre la barre en x_dep, la finit en x_fin et renvoie dans y_fin la position finale
   Fenetre = la fentre dans laquelle on doit travailler
*/

int prepare_barre_horizontale(Fenetre *fenetre, int x_dep, unsigned int largeur, int *y_fin)
{
  int etat, retour, rien;

  coord_X.x = x_dep;
  coord_X.largeur = largeur;
  retour = BI_AttendClic(fenetre, 0, RCT_barre_horizontale, &rien, y_fin, &etat);

  return (retour > 1) ? FAUX : VRAI;
}

static void RCT_barre_horizontale(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  TRC_Mode( MD_INVERSE );
  TRC_Couleur( ROUGE );
  TRC_Style( 3 );
  TRC_Epaisseur( 1 );

  if ( (vx_x >= fenetre->w_x) && (vx_x <= fenetre->w_x + (int) fenetre->w_larg - 1) &&
       (vx_y >= fenetre->w_y) && (vx_y <= fenetre->w_y + (int) fenetre->w_haut - 1) ) {
    TRC_Rectangle( fenetre, coord_X.x, vx_y - 2 * TAILLE_BORDURES, coord_X.largeur, 2 * TAILLE_BORDURES );
  }

  if ( (x >= fenetre->w_x) && (x <= fenetre->w_x + (int) fenetre->larg - 1) &&
       (y >= fenetre->w_y) && (y <= fenetre->w_y + (int) fenetre->haut - 1) ) {
    TRC_Rectangle( fenetre, coord_X.x, y - 2 * TAILLE_BORDURES, coord_X.largeur, 2 * TAILLE_BORDURES );
  }

  TRC_Mode( MD_REMPLACE ); /* Mode recouvrir */
}

/* -------- Dfinir une position verticale,  l'aide d'un rectangle
   Renvoie VRAI si on a bien dfini ce rectangle, FAUX si on a annul
   Dmarre la barre en y_dep, la finit en y_fin et renvoie dans x_fin la position finale
   Fenetre = la fentre dans laquelle on doit travailler
*/

int prepare_barre_verticale(Fenetre *fenetre, int y_dep, int *x_fin, unsigned int hauteur)
{
  int etat, retour, rien;

  coord_Y.y = y_dep; coord_Y.hauteur = hauteur;
  retour = BI_AttendClic(fenetre, 0, RCT_barre_verticale, x_fin, &rien, &etat);

  return (retour > 1) ? FAUX : VRAI;
}

static void RCT_barre_verticale(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  TRC_Mode( MD_INVERSE );
  TRC_Couleur( ROUGE );
  TRC_Style( 3 );
  TRC_Epaisseur( 1 );

  if ( (vx_x >= fenetre->w_x) && (vx_x <= fenetre->w_x + (int) fenetre->w_larg - 1) &&
       (vx_y >= fenetre->w_y) && (vx_y <= fenetre->w_y + (int) fenetre->w_haut - 1) ) {
    TRC_Rectangle( fenetre, vx_x - 2 * TAILLE_BORDURES, coord_Y.y, 2 * TAILLE_BORDURES, coord_Y.hauteur );
  }

  if ( (x >= fenetre->w_x) && (x <= fenetre->w_x + (int) fenetre->larg - 1) &&
       (y >= fenetre->w_y) && (y <= fenetre->w_y + (int) fenetre->haut - 1) ) {
    TRC_Rectangle( fenetre, x - 2 * TAILLE_BORDURES, coord_Y.y, 2 * TAILLE_BORDURES, coord_Y.hauteur );
  }

  TRC_Mode( MD_REMPLACE ); /* Mode recouvrir */
}

/* -------- Dfinir la position d'un coin,  l'aide d'une sorte de L
   Renvoie VRAI si on a bien dfini ce L, FAUX si on a annul
   x_fixe, y_fixe indiquent les coordonnes fixes ; x et y les coordonnes mobiles
   type contient le type de coin  grer (mme code que pour le lieu de clic)
   Fenetre = la fentre dans laquelle on doit travailler
*/

int prepare_coin(Fenetre *fenetre, int x_fixe, int y_fixe, int *x, int *y, int type)
{
  int etat, retour;
  void (*fct_RCT)(Fenetre *, int, int, int, int);

  switch (type)
  {
  case TRC_EST_COIN_HG:
    fct_RCT = RCT_coin_haut_gauche;
    break;

  case TRC_EST_COIN_BG:
    fct_RCT = RCT_coin_bas_gauche;
    break;

  case TRC_EST_COIN_BD:
    fct_RCT = RCT_coin_bas_droit;
    break;

  case TRC_EST_COIN_HD:
    fct_RCT = RCT_coin_haut_droit;
    break;

  default:
    fprintf(stderr, "prepare_coin : mode incorrect\n");
    return FAUX;
  }

  coord_X.x = x_fixe; coord_Y.y = y_fixe;
  retour = BI_AttendClic(fenetre, 0, fct_RCT, x, y, &etat);
  return (retour > 1) ? FAUX : VRAI;
}

static void RCT_coin_haut_gauche(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  Point_Ecran coins[ 7 ];

  TRC_Zone_InitialiserDessin;

  coins[ 0 ].x = coord_X.x;
  coins[ 1 ].x = coord_X.x;
  coins[ 3 ].y = coord_Y.y;
  coins[ 4 ].y = coord_Y.y;
  coins[ 6 ].x = coord_X.x;
  
  if ( ( vx_x >= 0 ) && ( vx_x <= (int) (fenetre->w_larg - 1) ) &&
       ( vx_y >= 0 ) && ( vx_y <= (int) (fenetre->w_haut - 1) ) ) { /* On efface l'ancien */
    coins[ 0 ].y = vx_y - TAILLE_BORDURES;
    if ( coins[ 0 ].y < 0 ) coins[ 0 ].y = 0;
    coins[ 5 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].y = vx_y + TAILLE_BORDURES;;
    coins[ 2 ].y = coins[ 1 ].y;

    coins[ 2 ].x = vx_x + TAILLE_BORDURES;
    coins[ 3 ].x = coins[ 2 ].x;

    coins[ 4 ].x = vx_x - TAILLE_BORDURES;
    if ( coins[ 4 ].x < 0 ) coins[ 4 ].x = 0;
    coins[ 5 ].x = coins[ 4 ].x;

    TRC_LigneBrisee( fenetre, 7, coins );

    TRC_Ligne( fenetre, vx_x, coord_Y.y, vx_x, vx_y );
    TRC_Ligne( fenetre, vx_x, vx_y, coord_X.x, vx_y );
  }

  if ( ( x >= 0 ) && ( x <= (int) (fenetre->w_larg - 1) ) &&
       ( y >= 0 ) && ( y <= (int) (fenetre->w_haut - 1) ) ) {
    coins[ 0 ].y = y - TAILLE_BORDURES;
    if ( coins[ 0 ].y < 0 ) coins[ 0 ].y = 0;
    coins[ 5 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].y = y + TAILLE_BORDURES;;
    coins[ 2 ].y = coins[ 1 ].y;

    coins[ 2 ].x = x + TAILLE_BORDURES;
    coins[ 3 ].x = coins[ 2 ].x;

    coins[ 4 ].x = x - TAILLE_BORDURES;
    if ( coins[ 4 ].x < 0 ) coins[ 4 ].x = 0;
    coins[ 5 ].x = coins[ 4 ].x;

    TRC_LigneBrisee( fenetre, 7, coins );

    TRC_Ligne( fenetre, x, coord_Y.y, x, y );
    TRC_Ligne( fenetre, x, y, coord_X.x, y );
  }
}

static void RCT_coin_haut_droit(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  Point_Ecran coins[ 7 ];

  TRC_Zone_InitialiserDessin;

  coins[ 0 ].x = coord_X.x;
  coins[ 2 ].y = coord_Y.y;
  coins[ 3 ].y = coord_Y.y;
  coins[ 5 ].x = coord_X.x;
  coins[ 6 ].x = coord_X.x;

  if ( ( vx_x >= 0 ) && ( vx_x <= (int) fenetre->w_larg - 1 ) &&
       ( vx_y >= 0 ) && ( vx_y <= (int) fenetre->w_haut - 1 ) ) { /* On efface l'ancien */
    coins[ 0 ].y = vx_y - TAILLE_BORDURES;
    if ( coins[ 0 ].y < 0 ) coins[ 0 ].y = 0;
    coins[ 1 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].x = vx_x + TAILLE_BORDURES;
    coins[ 2 ].x = coins[ 1 ].x;

    coins[ 3 ].x = vx_x - TAILLE_BORDURES;
    if ( coins[ 3 ].x < 0 ) coins[ 3 ].x = 0;
    coins[ 4 ].x = coins[ 3 ].x;

    coins[ 4 ].y = vx_y + TAILLE_BORDURES;
    coins[ 5 ].y = coins[ 4 ].y;

    TRC_LigneBrisee( fenetre, 7, coins );
  }

  if ( ( x >= 0 ) && ( x <= (int) fenetre->w_larg - 1 ) &&
       ( y >= 0 ) && ( y <= (int) fenetre->w_haut - 1 ) ) { /* On met le nouveau */
    coins[ 0 ].y = y - TAILLE_BORDURES;
    if ( coins[ 0 ].y < 0 ) coins[ 0 ].y = 0;
    coins[ 1 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].x = x + TAILLE_BORDURES;
    coins[ 2 ].x = coins[ 1 ].x;

    coins[ 3 ].x = x - TAILLE_BORDURES;
    if ( coins[ 3 ].x < 0 ) coins[ 3 ].x = 0;
    coins[ 4 ].x = coins[ 3 ].x;

    coins[ 4 ].y = y + TAILLE_BORDURES;
    coins[ 5 ].y = coins[ 4 ].y;

    TRC_LigneBrisee( fenetre, 7, coins );
  }
}

static void RCT_coin_bas_gauche(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  Point_Ecran coins[ 7 ];

  TRC_Zone_InitialiserDessin;

  coins[ 0 ].y = coord_Y.y;
  coins[ 2 ].x = coord_X.x;
  coins[ 3 ].x = coord_X.x;
  coins[ 5 ].y = coord_Y.y;
  coins[ 6 ].y = coord_Y.y;

  if ( ( vx_x >= 0 ) && ( vx_x <= (int) (fenetre->w_larg - 1) ) &&
       ( vx_y >= 0 ) && ( vx_y <= (int) (fenetre->w_haut - 1) ) ) { /* On efface l'ancien */
    coins[ 0 ].x = vx_x + TAILLE_BORDURES;
    coins[ 1 ].x = coins[ 0 ].x;
    coins[ 6 ].x = coins[ 0 ].x;

    coins[ 1 ].y = vx_y - TAILLE_BORDURES;
    if ( coins[ 1 ].y < 0 ) coins[ 1 ].y = 0;
    coins[ 2 ].y = coins[ 1 ].y;

    coins[ 3 ].y = vx_y + TAILLE_BORDURES;
    coins[ 4 ].y = coins[ 3 ].y;

    coins[ 4 ].x = vx_x - TAILLE_BORDURES;
    if ( coins[ 4 ].x < 0 ) coins[ 4 ].x = 0;
    coins[ 5 ].x = coins[ 4 ].x;

    TRC_LigneBrisee( fenetre, 7, coins );
  }

  if ( ( x >= 0 ) && ( x <= (int) (fenetre->w_larg - 1) ) &&
       ( y >= 0 ) && ( y <= (int) (fenetre->w_haut - 1) ) ) { /* On met le nouveau */
    coins[ 0 ].x = x + TAILLE_BORDURES;
    coins[ 1 ].x = coins[ 0 ].x;
    coins[ 6 ].x = coins[ 0 ].x;

    coins[ 1 ].y = y - TAILLE_BORDURES;
    if ( coins[ 1 ].y < 0 ) coins[ 1 ].y = 0;
    coins[ 2 ].y = coins[ 1 ].y;

    coins[ 3 ].y = y + TAILLE_BORDURES;
    coins[ 4 ].y = coins[ 3 ].y;

    coins[ 4 ].x = x - TAILLE_BORDURES;
    if ( coins[ 4 ].x < 0 ) coins[ 4 ].x = 0;
    coins[ 5 ].x = coins[ 4 ].x;

    TRC_LigneBrisee( fenetre, 7, coins );

  }
}

static void RCT_coin_bas_droit(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  Point_Ecran coins[ 7 ];

  TRC_Zone_InitialiserDessin;

  coins[ 0 ].x = coord_X.x;
  coins[ 2 ].y = coord_Y.y;
  coins[ 3 ].y = coord_Y.y;
  coins[ 5 ].x = coord_X.x;
  coins[ 6 ].x = coord_X.x;

  if ( ( vx_x >= 0 ) && ( vx_x <= (int) fenetre->w_larg - 1 ) &&
       ( vx_y >= 0 ) && ( vx_y <= (int) fenetre->w_haut - 1 ) ) { /* On efface l'ancien */
    coins[ 0 ].y = vx_y + TAILLE_BORDURES;
    coins[ 1 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].x = vx_x + TAILLE_BORDURES;
    coins[ 2 ].x = coins[ 1 ].x;

    coins[ 3 ].x = vx_x - TAILLE_BORDURES;
    if ( coins[ 3 ].x < 0 ) coins[ 3 ].x = 0;
    coins[ 4 ].x = coins[ 3 ].x;

    coins[ 4 ].y = vx_y - TAILLE_BORDURES;
    if ( coins[ 4 ].y < 0 ) coins[ 4 ].y = 0;
    coins[ 5 ].y = coins[ 4 ].y;

    TRC_LigneBrisee( fenetre, 7, coins );
  }

  if ( ( x >= 0 ) && ( x <= (int) fenetre->w_larg - 1 ) &&
       ( y >= 0 ) && ( y <= (int) fenetre->w_haut - 1 ) ) { /* On met le nouveau */
    coins[ 0 ].y = y + TAILLE_BORDURES;
    coins[ 1 ].y = coins[ 0 ].y;
    coins[ 6 ].y = coins[ 0 ].y;

    coins[ 1 ].x = x + TAILLE_BORDURES;
    coins[ 2 ].x = coins[ 1 ].x;

    coins[ 3 ].x = x - TAILLE_BORDURES;
    if ( coins[ 3 ].x < 0 ) coins[ 3 ].x = 0;
    coins[ 4 ].x = coins[ 3 ].x;

    coins[ 4 ].y = y - TAILLE_BORDURES;
    if ( coins[ 4 ].y < 0 ) coins[ 4 ].y = 0;
    coins[ 5 ].y = coins[ 4 ].y;

    TRC_LigneBrisee( fenetre, 7, coins );
  }
}

/* -------- Dfinir la position d'un point, reli  un point fixe -------- */

int prepare_ligne(Fenetre *fenetre, int x_fixe, int y_fixe, int *x_mobile, int *y_mobile)
{
  int retour, etat;

  if (fenetre == NULL) return 0;
  if ( (x_mobile == NULL) || (y_mobile == NULL) ) {
    fprintf(stderr, "prepare_ligne [Tracer/zones.c] : x_mobile ou y_mobile NULL\n");
    return 0;
  }

  coord_X.x = x_fixe; coord_Y.y = y_fixe;
  retour = BI_AttendClic( fenetre, 0, RCT_ligne, x_mobile, y_mobile, &etat );
  return (retour > 1) ? FAUX : VRAI;
}

static void RCT_ligne(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  TRC_Zone_InitialiserDessin;

  if ( ( vx_x >= 0 ) && ( vx_x <= (int) fenetre->w_larg - 1) &&
       ( vx_y >= 0 ) && ( vx_y <= (int) fenetre->w_haut - 1) ) {
    TRC_Ligne( fenetre, coord_X.x, coord_Y.y, vx_x, vx_y );
  }

  if ( (x >= fenetre->w_x) && (x <= fenetre->w_x + (int) fenetre->larg - 1) &&
       (y >= fenetre->w_y) && (y <= fenetre->w_y + (int) fenetre->haut - 1) ) {
    TRC_Ligne( fenetre, coord_X.x, coord_Y.y, x, y );
  }
}
