/* Calcul d'une densite a partir d'une formule chimique
   (C) juin 2002, Emmanuel Curis
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> /* M_PI */

#include "chimie_utils.h"
#include "deboguage.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Calcul de la masse molaire -------- */

double Chimie_MasseMolaire(Composition *formule)
{
  double masse_molaire = 0.0;
  unsigned int i;

  if ( NULL == formule ) return -1.0;

  for ( i = 0 ; i < 120 ; i++ ) {
    if ( 0.0 == formule->atomes[ i ] ) continue;

    masse_molaire += formule->atomes[ i ] * ATM_Z_vers_masse( i + 1, ATM_MASSE_MOLAIRE );
  }

  return masse_molaire;
}

double Chimie_EstimeDensite(Composition *formule, int methode)
{
  double masse = 0.0, volume = 0.0;
  unsigned int i;
  float nbr_atomes = 0;

  ENTREE_FONCTION("Chimie_EstimeDensite", "formule=%p methode=%d", formule, methode);

  if (NULL == formule) {
    SORTIE_FONCTION("Chimie_EstimeDensite", "%g [ERREUR : formule==NULL]", -1.0);
    return -1.0;
  }

  switch ( methode )
  {
  }

  /* Calcul simple : masse totale d'une molecule = somme des masses de chacun des atomes
                     volume total d'une molecule = somme des volumes de chacun des atomes */
  for (i = HYDROGENE ; i <= URANIUM  ; i++) {
    double m_i, r_i;

    if ( 0.0 == formule->atomes[ i - 1 ] ) continue;
    if ( i > HYDROGENE ) {
      nbr_atomes += formule->atomes[ i - 1 ];
    }

    /* La masse */
    m_i = ATM_Z_vers_masse(i, ATM_MASSE_ATOMIQUE);
    masse  += formule->atomes[ i - 1 ] * m_i;

    /* Le volume : masse / masse volumique*/
/*     rho_i = ATM_Z_vers_masse(i, ATM_MASSE_VOLUMIQUE); */
/*     if ( rho_i <= 0.0 ) { */
/*       (void) fprintf(stderr, "Chimie_EstimeDensite : densite absurde, Z = %u ==> rho = %f\n", i, rho_i); */
/*     } else { */
/*       volume += formule->atomes[ i - 1 ] * ( m_i / rho_i ); */
/*     } */

    /* Le volume : atome spherique, de rayon le rayon covalent */
    r_i = ATM_Z_vers_rayon(i, ATM_RAYON_COVALENT);
    if (r_i <= 0.0) {
      (void) fprintf(stderr, "Chimie_EstimeDensite : rayon covalent absurde, Z = %u ==> rc = %f\n", i, r_i);
    } else {
      r_i *= 1.0e-10; /* 1 pm = 1e-10 cm => rayon en centimetres */
      r_i = r_i * r_i * r_i;
      r_i *= 4.0/3.0 * M_PI; /* volume d'un atome en cm^3 */
      volume += r_i;
    }
  }

  printf("Estimation a 18 A^3 : rho = %f\n", masse / (nbr_atomes * (18.0e-24)));
  
  if (volume == 0.0) {
    (void) fprintf(stderr, "Chimie_EstimeDensite : volume estime nul !\n");
    SORTIE_FONCTION("Chimie_EstimeDensite", "%g [ERREUR : Volume estim nul]", -1.0);
    return -1.0;
  }

  SORTIE_FONCTION("Chimie_EstimeDensite", "%g", masse / volume);
  return (masse / volume);
}

#ifdef AUTONOME

int main(int n_args, char *formules[])
{
  Composition *compo;
  int i;

  for (i = 1 ; i < n_args ; i++) {
    double rho;

    compo = Chimie_AnalyseFormule(formules[i]);
    rho = Chimie_EstimeDensite(compo, 0);
    printf("Masse volumique estimee pour %s : %f g/cm^3\n", formules[i], rho);
    free(compo);
  }

  return 0;
}

#endif /* AUTONOME */
