/* Fonctions utilitaires générales
   Développées pour LASE, mais plus génériques
   (C) 1996 - 2003, Emmanuel CURIS

   Écriture "avancée" des fichiers en ASCII
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "ascii_utils.h"
#include "txt_utils.h" /* Pour TXT_CNV_vNombre_D */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

inline static int EA_AvecErreur(FILE *fichier, unsigned long nbr_points, unsigned int nbr_colonnes, double **tables, double **erreurs,
				struct EA_infos_export *options);

inline static int EA_SansErreur(FILE *fichier, unsigned long nbr_points, unsigned int nbr_colonnes, double **tables,
				struct EA_infos_export *options);

inline static int EA_EcrireSeparateur(FILE *fichier, int separateur);
inline static int EA_EcrireValeur    (FILE *fichier, double valeur, double erreur);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Écriture simultanée de plusieurs tables numériques en colonnes dans un fichier -------- */

int EA_MultiColonnes(FILE *fichier, unsigned long nbr_points, unsigned int nbr_colonnes, double **tables, double **erreurs,
		     struct EA_infos_export *options)
{
  struct EA_infos_export infos_locales = { 0, 0, 6 };
  int retour;

  if ( ( 0 == nbr_points ) || ( 0 == nbr_colonnes ) ) return 0;
  if ( ( NULL == fichier ) || ( NULL == tables ) ) return -1;
  if ( NULL == options ) {
    (void) fprintf(stderr, "%s,%s l.%u : ATTENTION, pas d'options indiquées.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    options = &infos_locales;
  }

  if ( NULL == erreurs ) {
    retour = EA_SansErreur( fichier, nbr_points, nbr_colonnes, tables, options );
  } else {
    retour = EA_AvecErreur( fichier, nbr_points, nbr_colonnes, tables, erreurs, options );
  }
  if ( retour != 0 ) {
    (void) fprintf( stderr, "Avertir d'un problème à la sauvegarde..." );
  }

  return 0;
}

/* -------- Export multi-valeurs avec écriture des incertitudes -------- */

inline static int EA_AvecErreur(FILE *fichier, unsigned long nbr_points, unsigned int nbr_colonnes, double **tables, double **erreurs,
				struct EA_infos_export *options)
{
  int retour;
  unsigned long i;
  unsigned int idx_colonne;

  for ( i = 0 ; i < nbr_points ; i++ ) {
    /* Les colonnes sauf la dernière */
    for ( idx_colonne = 0 ; idx_colonne < nbr_colonnes - 1 ; idx_colonne++ ) {
      retour = 0;

      if ( NULL == tables[ idx_colonne ] ) {
	if ( options->proprietes & EA_EXPORT_COLONNES_VIDES ) {
	  retour  = EA_EcrireValeur( fichier, 0.0, 0.0 );
	  retour |= EA_EcrireSeparateur( fichier, options->separateur );
	}
      } else {
	if ( erreurs[ idx_colonne ] != NULL ) {
	  retour  = EA_EcrireValeur( fichier, *(tables[ idx_colonne ] + i ), *(erreurs[ idx_colonne ] + i ) );
	} else {
	  retour  = EA_EcrireValeur( fichier, *(tables[ idx_colonne ] + i ), 0.0 );
	}
	retour |= EA_EcrireSeparateur( fichier, options->separateur );
      }

      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire la colonne %u\n",
		       __FILE__, __FUNCTION__, __LINE__, idx_colonne + 1);
      }
    }

    /* La dernière colonne [à part, car pas de séparateur !] */
    retour = 0;
    if ( NULL == tables[ nbr_colonnes - 1 ] ) {
      if ( options->proprietes & EA_EXPORT_COLONNES_VIDES ) {
	retour = EA_EcrireValeur( fichier, 0.0, 0.0 );
      }
    } else {
      if ( erreurs[ nbr_colonnes - 1 ] != NULL ) {
	retour  = EA_EcrireValeur( fichier, *(tables[ nbr_colonnes - 1 ] + i ), *(erreurs[ nbr_colonnes - 1 ] + i ) );
      } else {
	retour  = EA_EcrireValeur( fichier, *(tables[ nbr_colonnes - 1 ] + i ), 0.0 );
      }
    }
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire la dernière colonne\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    /* Passage à la ligne suivante */
    (void) fputc( '\n', fichier );
  }

  /* Terminé, tout s'est bien passé */
  return 0;
}

/* -------- Export multi-valeurs avec valeurs supposées exactes -------- */

inline static int EA_SansErreur(FILE *fichier, unsigned long nbr_points, unsigned int nbr_colonnes, double **tables,
				struct EA_infos_export *options)
{
  int retour;
  unsigned long i;
  unsigned int idx_colonne;

  for ( i = 0 ; i < nbr_points ; i++ ) {
    /* Les colonnes sauf la dernière */
    for ( idx_colonne = 0 ; idx_colonne < nbr_colonnes - 1 ; idx_colonne++ ) {
      retour = 0;

      if ( NULL == tables[ idx_colonne ] ) {
	if ( options->proprietes & EA_EXPORT_COLONNES_VIDES ) {
	  retour  = EA_EcrireValeur( fichier, 0.0, 0.0 );
	  retour |= EA_EcrireSeparateur( fichier, options->separateur );
	}
      } else {
	retour  = EA_EcrireValeur( fichier, *(tables[ idx_colonne ] + i ), 0.0 );
	retour |= EA_EcrireSeparateur( fichier, options->separateur );
      }

      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire la colonne %u\n",
		       __FILE__, __FUNCTION__, __LINE__, idx_colonne + 1);
      }
    }

    /* La dernière colonne [à part, car pas de séparateur !] */
    retour = 0;      
    if ( NULL == tables[ nbr_colonnes - 1 ] ) {
      if ( options->proprietes & EA_EXPORT_COLONNES_VIDES ) {
	retour = EA_EcrireValeur( fichier, 0.0, 0.0 );
      }
    } else {
      retour  = EA_EcrireValeur( fichier, *(tables[ nbr_colonnes - 1 ] + i ), 0.0 );
    }
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire la dernière colonne\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    /* Passage à la ligne suivante */
    (void) fputc( '\n', fichier );
  }

  /* Terminé, tout s'est bien passé */
  return 0;
}

/* -------- Écrire un séparateur -------- */

inline static int EA_EcrireSeparateur(FILE *fichier, int separateur)
{
  int retour;

  if ( 0 == separateur ) {
    retour = fputc( ' ', fichier );
  } else {
    retour = fputc( separateur, fichier );
  }

  if ( EOF == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire le séparateur\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* Sauvegarde réussie */
  return 0;
}

/* -------- Écrire une valeur et son erreur -------- */

inline static int EA_EcrireValeur(FILE *fichier, double valeur, double erreur)
{
  char txt_valeur[ 100 ];
  int retour;

  (void) TXT_CNV_D_vTexte( valeur, erreur, 4, 7, 3, txt_valeur, NULL );
  retour = fputs( txt_valeur, fichier );
  if ( EOF == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant écrire la valeur %g ± %g [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, valeur, erreur, txt_valeur);
    return -1;
  }

  return 0;
}
