/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   criture "avance" des fichiers en binaire
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bin_utils.h"
#include "utils_3d.h"

/* -------- Sauvegarde d'une chane de caractres --------
    Renvoie 0 si tout s'est bien pass, != 0 en cas de problme

  Format de sauvegarde :
    lg_chaine : unsigned long --- nombre de caractres dans la chane
    <<chaine>> : la chane (sans son 0 final)
*/

inline int BIN_SV_chaine(FILE *fichier, const char *chaine)
{
  unsigned long lg_chaine;
  size_t n_ecrit;

  if ( NULL == fichier ) return -1;
  if ( NULL == chaine ) {
    lg_chaine = 0UL;
    n_ecrit = fwrite( &lg_chaine, sizeof( unsigned long ), 1, fichier );
    if ( n_ecrit != 1 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauvegarder la longueur de chane / chane nulle\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }

    return 0;
  }

  lg_chaine = strlen( chaine );
  n_ecrit = fwrite( &lg_chaine, sizeof( unsigned long ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauvegarder la longueur de chane [%lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, lg_chaine);
    return -2;
  }
  n_ecrit = fwrite( chaine, sizeof( char ), lg_chaine, fichier );
  if ( n_ecrit != lg_chaine ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauvegarder la chane [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, chaine);
    return -4;
  }

  /* Pas de problme */
  return 0;
}

/* -------- Sauvegarde d'une orientation 3D --------
 << Vecteur 3D >> Centre du repre
 double           Angle phi
 double           Angle theta
 double           Angle alpha
 */

inline int BIN_SV_orientation(FILE *fichier, struct orientation_3d *orientation)
{
  int retour, n_ecrit;

  if ( NULL == fichier ) return -1;
  if ( NULL == orientation ) {
    (void) fprintf(stderr, "%s,%s l.%u : Orientation nulle !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* Position du centre du repre */
  retour = BIN_SV_vecteur_3d( fichier, &(orientation->position) );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans BIN_SV_vecteur_3d [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -4;
  }

  /* Angles dfinissant le repre */
  n_ecrit = fwrite( &(orientation->phi), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver phi [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->phi );
    return -5;
  }
  n_ecrit = fwrite( &(orientation->theta), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver theta [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->theta );
    return -5;
  }
  n_ecrit = fwrite( &(orientation->alpha), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver alpha [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, orientation->alpha );
    return -5;
  }

  /* Pas de problme */
  return 0;
}

/* -------- Sauvegarde d'un vecteur  3 dimensions --------
 double X
 double Y
 double Z
*/

inline int BIN_SV_vecteur_3d(FILE *fichier, struct vect_3d *vecteur)
{
  int n_ecrit;

  if ( NULL == fichier ) return -1;
  if ( NULL == vecteur ) {
    (void) fprintf(stderr, "%s,%s l.%u : Vecteur nul !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* Coordonnes X, Y et Z */
  n_ecrit = fwrite( &(vecteur->x), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver X [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->x );
    return -5;
  }
  n_ecrit = fwrite( &(vecteur->y), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver Y [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->y );
    return -5;
  }
  n_ecrit = fwrite( &(vecteur->z), sizeof( double ), 1, fichier );
  if ( n_ecrit != 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de sauver Z [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, vecteur->z );
    return -5;
  }

  /* Pas de problme */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Divers ~-~-~-~-~-~-~-~-~ */

/* -------- Inclure un fichier dans le fichier ouvert -------- */

inline int BIN_InclureFichier(FILE *fichier, const char *nom_fichier)
{
  FILE *fichier_insere;
  int c;

  if ( ( NULL == fichier ) || ( NULL == nom_fichier ) ) return -1;

  fichier_insere = fopen( nom_fichier, "rb" );
  if ( NULL == fichier_insere ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'insrer le fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    return -10;
  }

  /* Copie brutale */
  c = fgetc( fichier_insere );
  while ( c != EOF ) {
    c = fputc( c, fichier );
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR d'criture !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      (void) fclose( fichier_insere );
      return +1;
    }

    c = fgetc( fichier_insere );
  }

  /* Termin : on ferme le fichier d'origine */
  (void) fclose( fichier_insere );
  return 0;
}
