/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2002, Emmanuel CURIS

   Conversion symbole chimique -> numro atomique correspondant
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "chimie_utils.h"

/* ------------------ Renvoi le numro atomique de l'lment de symbole considr ---------------\
 |    Renvoie 0 si le code est inconnu.                                                          |
 |    Si code du type Fe1 : utilise que Fe pour dterminer...                                |
 |                                                                                               |
 | symbole : chane dont les 2 premiers caractres doivent tre le symbole                       |
 |           (note : les espaces initiaux sont ignors)                                          |
 \---------------------------------------------------------------------------------------------- */

int ATM_num_atomique(char *symbole)
{
  char lettre_1, lettre_2;

  if (symbole == NULL) return 0; /* Aucun symbole envoy */

  while( isspace(*symbole) )
    symbole++;

  lettre_1 = symbole[0];

  if ( (lettre_1 < 'A') && (lettre_1 > 'Z') )
    return 0; /* Symbole sans signification chimique, a priori */

  lettre_2 = symbole[1];

  if ( (lettre_2 < 'a') && (lettre_2 > 'z') ) { /* On a donc un symbole  une lettre seulement */
    switch (lettre_1)
    {
    case 'B': return BORE;
    case 'C': return CARBONE;
    case 'D': return HYDROGENE; /* D = Deutrium = Hydrogne donc... */
    case 'F': return FLUOR;
    case 'H': return HYDROGENE;
    case 'I': return IODE;
    case 'N': return AZOTE;
    case 'O': return OXYGENE;
    case 'P': return PHOSPHORE;
    case 'S': return SOUFRE;
    case 'T': return HYDROGENE; /* T = Tritium donc hydrogne... */
    case 'U': return URANIUM;
    case 'V': return VANADIUM;
    case 'W': return TUNGSTENE;
    case 'Y': return YTTRIUM;
    default:
      return 0; /* Symbole non dfini */
    }
  }

  /* On est ici -> on utilise un symbole  deux lettres... */

  switch (lettre_1) /* Premire lettre du symbole */
  {
  case 'A':
    switch (lettre_2) /* Seconde lettre du symbole */
    {
    case 'c': return ACTINIUM; /* Ac */
    case 'g': return ARGENT;   /* Ag */
    case 'l': return ALUMINIUM;/* Al */
    case 'm': return AMERICIUM;/* Am */
    case 'r': return ARGON;    /* Ar */
    case 's': return ARSENIC;  /* As */
    case 't': return ASTATE;   /* At */
    case 'u': return OR;       /* Au */
    case 'z': printf("Symbole archaque : Az\n"); return AZOTE;
    default: return 0; /* Pas de symbole dfini */
    }

  case 'B':
    switch (lettre_2)
    {
    case 'a': return BARYUM;    /* Ba */
    case 'e': return BERYLLIUM; /* Be */
    case 'i': return BISMUTH;   /* Bi */
    case 'k': return BERKELIUM; /* Bk */
    case 'h': return BOHRIUM;   /* Bh */
    case 'r': return BROME;     /* Br */
    default: return BORE;       /* B  */
    }

  case 'C':
    switch (lettre_2)
    {
    case 'a': return CALCIUM;     /* Ca */
    case 'd': return CADMIUM;     /* Cd */
    case 'e': return CERIUM;      /* Ce */
    case 'f': return CALIFORNIUM; /* Cf */
    case 'l': return CHLORE;      /* Cl */
    case 'm': return CURIUM;      /* Cm */
    case 'o': return COBALT;      /* Co */
    case 'r': return CHROME;      /* Cr */
    case 's': return CESIUM;      /* Cs */
    case 'u': return CUIVRE;      /* Cu */
    default:  return CARBONE;     /* C  */
    }

  case 'D':
    switch (lettre_2)
    {
    case 'b': return DUBNIUM;     /* Db */
    case 's': return DARMSTADTIUM;/* Ds */
    case 'y': return DYSPROSIUM;  /* Dy */
    default: return HYDROGENE; /* D = deutrium, donc hydrogne */
    }

  case 'E':
    switch (lettre_2)
    {
    case 'r': return ERBIUM;      /* Er */
    case 's': return EINSTEINIUM; /* Es */
    case 'u': return EUROPIUM;    /* Eu */
    default: return 0;
    }

  case 'F':
    switch (lettre_2)
    {
    case 'e': return FER;      /* Fe */
    case 'm': return FERMIUM;  /* Fm */
    case 'r': return FRANCIUM; /* Fr */
    default: return FLUOR;     /* F = fluor */
    }

  case 'G':
    switch (lettre_2)
    {
    case 'a': return GALLIUM;    /* Ga */
    case 'd': return GADOLINIUM; /* Gd */
    case 'e': return GERMANIUM;  /* Ge */
    default: return 0;
    }

  case 'H':
    switch (lettre_2)
    {
    case 'a': return HAFNIUM; /* Ha [plus utilis ?] */
    case 'e': return HELIUM;  /* He */
    case 'f': return HAFNIUM; /* Hf */
    case 'g': return MERCURE; /* Hg */
    case 'o': return HOLMIUM; /* Ho */
    case 's': return HASSIUM; /* Hs */
    default: return HYDROGENE;
    }

  case 'I':
    if (lettre_2 == 'n') return INDIUM;  /* In */
    if (lettre_2 == 'r') return IRIDIUM; /* Ir */
    return IODE;

  case 'K':
    if (lettre_2 == 'r') return KRYPTON; /* Kr */
    return POTASSIUM;

  case 'L':
    if (lettre_2 == 'a') return LANTHANE;   /* La */
    if (lettre_2 == 'i') return LITHIUM;    /* Li */
    if (lettre_2 == 'r') return LAWRENCIUM; /* Lr */
    if (lettre_2 == 'u') return LUTETIUM;   /* Lu */
    if (lettre_2 == 'w') return LAWRENCIUM; /* Lw - 2 symboles pendant un temps... */
    return 0;

  case 'M':
    if (lettre_2 == 'd') return MENDELEVIUM; /* Md */
    if (lettre_2 == 'g') return MAGNESIUM;   /* Mg */
    if (lettre_2 == 'n') return MANGANESE;   /* Mn */
    if (lettre_2 == 'o') return MOLYBDENE;   /* Mo */
    if (lettre_2 == 't') return MEITNERIUM;  /* Mt */
    return 0;
    
  case 'N':
    if (lettre_2 == 'a') return SODIUM;    /* Na */
    if (lettre_2 == 'b') return NIOBIUM;   /* Nb */
    if (lettre_2 == 'd') return NEODYME;   /* Nd */
    if (lettre_2 == 'e') return NEON;      /* Ne */
    if (lettre_2 == 'i') return NICKEL;    /* Ni */
    if (lettre_2 == 'o') return NOBELIUM;  /* No */
    if (lettre_2 == 'p') return NEPTUNIUM; /* Np */
    return AZOTE;

  case 'O':
    if (lettre_2 == 's') return OSMIUM;  /* Os */
    return OXYGENE;

  case 'P':
    if (lettre_2 == 'a') return PROTACTINIUM; /* Pa */
    if (lettre_2 == 'b') return PLOMB;        /* Pb */
    if (lettre_2 == 'd') return PALLADIUM;    /* Pd */
    if (lettre_2 == 'm') return PROMETHIUM;   /* Pm */
    if (lettre_2 == 'o') return POLONIUM;     /* Po */
    if (lettre_2 == 'r') return PRASEODYME;   /* Pr */
    if (lettre_2 == 't') return PLATINE;      /* Pt */
    if (lettre_2 == 'u') return PLUTONIUM;    /* Pu */
    return PHOSPHORE;

  case 'R':
    if (lettre_2 == 'a') return RADIUM;    /* Ra */
    if (lettre_2 == 'b') return RUBIDIUM;  /* Rb */
    if (lettre_2 == 'e') return RHENIUM;   /* Re */
    if (lettre_2 == 'f') return RUTHERFORDIUM; /* Rf */
    if (lettre_2 == 'h') return RHODIUM;   /* Rh */
    if (lettre_2 == 'n') return RADON;     /* Rn */
    if (lettre_2 == 'u') return RUTHENIUM; /* Ru */
    return 0;

  case 'S':
    if (lettre_2 == 'b') return ANTIMOINE; /* Sb */
    if (lettre_2 == 'c') return SCANDIUM;  /* Sc */
    if (lettre_2 == 'e') return SELENIUM;  /* Se */
    if (lettre_2 == 'g') return SEABORGIUM;/* Sg */
    if (lettre_2 == 'i') return SILICIUM;  /* Si */
    if (lettre_2 == 'm') return SAMARIUM;  /* Sm */
    if (lettre_2 == 'n') return ETAIN;     /* Sn */
    if (lettre_2 == 'r') return STRONTIUM; /* Sr */
    return SOUFRE;

  case 'T':
    if (lettre_2 == 'a') return TANTALE;    /* Ta */
    if (lettre_2 == 'b') return TERBIUM;    /* Tb */
    if (lettre_2 == 'c') return TECHNETIUM; /* Tc */
    if (lettre_2 == 'e') return TELLURE;    /* Te */
    if (lettre_2 == 'h') return THORIUM;    /* Th */
    if (lettre_2 == 'i') return TITANE;     /* Ti */
    if (lettre_2 == 'l') return THALLIUM;   /* Tl */
    if (lettre_2 == 'm') return THULIUM;    /* Tm */
    if (lettre_2 == 'u') { printf("Symbole archaque : Tu\n"); return TUNGSTENE; }
    return HYDROGENE; /* T = tritium donc hydrogne */

  case 'U': return URANIUM;

  case 'V': return VANADIUM;

  case 'W': return TUNGSTENE;

  case 'X':
    if (lettre_2 == 'e') return XENON;      /* Xe */
    return 0;

  case 'Y':
    if (lettre_2 == 'b') return YTTERBIUM;  /* Yb */
    return YTTRIUM;

  case 'Z':
    if (lettre_2 == 'n') return ZINC;       /* Zn */
    if (lettre_2 == 'r') return ZIRCONIUM;  /* Zr */
    return 0;
  }

  return 0; /* Par dfaut, aucun atome dfini */
}
