/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   Travail dans l'espace :changements de repres, localisations,...
*/

#ifndef UTILS_3D_
# define UTILS_3D_ /* Protection contre l'inclusion multiple */

/* Coordonnes d'un point, composantes d'un vecteur */

typedef struct vect_3d {
  double x;
  double y;
  double z;
} Vecteur3D;

typedef struct err_coord_3d {
  double var_x, var_y, var_z;    /* Variances des coordonnes */
  double cov_xy, cov_xz, cov_yz; /* Covariances entre coordonnes */
} Vecteur3D_Erreurs;

/* Dfinition d'un repre orthonorm direct  partir du repre absolu [orthonorm, direct]
   Ce repre est obtenu en effectuant dans l'ordre les oprations suivantes :
   1. Translation vers le centre du repre
   2. Rotation de l'angle alpha autour de l'axe des Y
   3. Rotation de l'angle thta autour de l'axe des Z
   4. Rotation de l'angle phi autour de l'axe des X

   Quand ce repre est utilis pour dfinir un plan dans l'espace,
   la normale au plan est l'axe Y et les axes X et Z forment un repre orthonorm direct du plan
 */

typedef struct orientation_3d {
  Vecteur3D position; /* Position du centre du repre */
  double phi;         /* Rotation autour de l'axe des X [radians] */
  double theta;       /* Rotation autour de l'axe des Z [radians] */
  double alpha;       /* Rotation autour de l'axe des Y [radians] */
} Orientation3D;

int REPERE_AbsoluVersLocal(Vecteur3D *coord_absolues, Orientation3D *repere, Vecteur3D *coord_locales );
int REPERE_LocalVersAbsolu(Vecteur3D *coord_locales , Orientation3D *repere, Vecteur3D *coord_absolues);
int REPERE_VecteurNormal  ( Orientation3D *repere, Vecteur3D *normale );

int REPERE_TrouverRotations_Y(Vecteur3D *vect_Y, double *theta, double *phi);

int REPERE_CartesienVersSpherique( Vecteur3D *xyz, double *r, double *theta, double *phi );
int REPERE_SpheriqueVersCartesien( double r, double theta, double phi, Vecteur3D *xyz );

inline double U3D_Distance (Vecteur3D *point_1, Vecteur3D *point_2);
inline int    U3D_Normalise(Vecteur3D *vecteur, Vecteur3D_Erreurs *var_vecteur);

double U3D_ProduitScalaire(Vecteur3D *v_1, Vecteur3D *v_2,
			   Vecteur3D_Erreurs *err_1, Vecteur3D_Erreurs *err_2, double *erreur);

/* Travail sur une droite */
int U3D_VecteurDirecteur    (Vecteur3D *point_1, Vecteur3D *point_2, Vecteur3D *vecteur,
			     Vecteur3D_Erreurs *err_1, Vecteur3D_Erreurs *err_2, Vecteur3D_Erreurs *err_vecteur);
int U3D_PositionSurDroite_PP(Vecteur3D *origine, Vecteur3D *point_2, double t, Vecteur3D *position,
			     Vecteur3D_Erreurs *err_origine, Vecteur3D_Erreurs *err_point_2, double err_t,
			     Vecteur3D_Erreurs *err_position);
int U3D_PositionSurDroite_VP(Vecteur3D *vecteur_directeur, Vecteur3D *origine, double t, Vecteur3D *position,
			     Vecteur3D_Erreurs *err_v_directeur, Vecteur3D_Erreurs *err_origine, double err_t,
			     Vecteur3D_Erreurs *err_position);

#define U3D_PROPAGATION_NUL     0 /* Vecteur nul ! */
#define U3D_PROPAGATION_SELON_X 1 /* Selon quel axe a-t-on intrt  avancer tranche par tranche ? */
#define U3D_PROPAGATION_SELON_Y 2
#define U3D_PROPAGATION_SELON_Z 3
inline int U3D_DirectionPreferentielle(Vecteur3D *vecteur);

int U3D_Inter_Droite_Plan_YCt(Vecteur3D *pt_droite, Vecteur3D *v_directeur, double y, Vecteur3D *intersection);

#endif /* UTILS_3D_ */
