/*  Programme d'estimation du biais sur   partir des donnes sur I0 et I1
    (C) fvrier 2000, Emmanuel CURIS

    Mode d'emploi : biais_mu -f/-t fichier_i0 fichier_i1 [-o sortie]
    Par dfaut, sort sur la sortie standard en ASCII.
    Sort de toute faon en ASCII...

    NB : sale parce que ne ferme pas sortie en quittant (compte sur exit)...
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "fichiers.h"

#define MODE_FLUO  0
#define MODE_TRANS 1

short int mode_manip = MODE_TRANS; /* Transmission par dfaut */

char *nom_i0 = NULL, *nom_i1 = NULL, *nom_sortie = NULL;

Graphe *grf_i0, *grf_i1, *biais;
Graphe *premier_graphe, *dernier_graphe, *graphe_courant;
unsigned long nbr_graphes = 0, num_graphe = 0;

Graphe *calc_biais_trans(void);
Graphe *calc_biais_fluo (void);

int main(int n_args, char **args)
{
  unsigned long i;
  FILE *sortie = stdout;

  if (n_args < 3)
  {
    fprintf(stderr, "biais_mu : indiquer au moins 2 fichiers en entre !\n");
    return -1;
  }

  /* On commence par chercher les commandes de contrle : commencent par - */

  for (i = 1 ; i < n_args ; i++)
  {
    if ( args[i] == NULL ) continue;

    if ( args[i][0] == '-' )
    {
      char *parametre = args[i];
      args[i] = NULL; /* On oublie ce paramtre */

      if (parametre[1] == 't') /* -t = calculs pour la transmission */
      {
	printf("Calculs pour la transmission\n");
	mode_manip = MODE_TRANS;
      }
      else if (parametre[1] == 'f') /* -f = calculs pour la fluorescence */
      {
	printf("Calculs pour la fluorescence\n");
	mode_manip = MODE_FLUO;
      }
      else if (parametre[1] == '-') /* -- = aprs, que des noms de fichier */
	break;
      else if (parametre[1] == 'o') /* -o = on indique un fichier de sortie */
      {
	if ( i + 1 < n_args )
	{
	  nom_sortie = args[i + 1];
	  args[i + 1] = NULL;

	  sortie = fopen(nom_sortie, "wb");
	  if (sortie == NULL)
	  {
	    fprintf(stderr, "biais_mu : impossible de crer le fichier %s\n", nom_sortie);
	    return -1;
	  }

	  fprintf(sortie, "# Estimation du biais, (c) 2000 Emmanuel CURIS\n");
	  fprintf(sortie, "# Colonne : 1 = X, 2 = biais\n");
	}
	else
	{
	  fprintf(stderr, "biais_mu : -o doit tre suivi d'un nom de fichier\n");
	  return -1;
	}
      }
      else
      {
	fprintf(stderr, "biais_mu : option inconnue, %s\n", parametre);
	return -1;
      }
    }
  }

  /* Une fois que l'on est l, on cherche les paramtres restants pour avoir les fichiers en entre */

  for (i = 1 ; i < n_args ; i++)
  {
    if (args[i] != NULL)
    {
      if (nom_i0 == NULL)
	nom_i0 = args[i];
      else if (nom_i1 == NULL)
	nom_i1 = args[i];
      else
      {
	fprintf(stderr, "biais_mu : fichier ignor, %s\n", args[i]);
      }
    }
  }

  if (nom_i0 == NULL)
  {
    fprintf(stderr, "biais_mu : pas de fichier indiqu pour les valeurs de I0 !\n");
    return -1;
  }
  grf_i0 = ch_interne(nom_i0);
  if (grf_i0 == NULL)
  {
    fprintf(stderr, "biais_mu : erreur au chargement du fichier des I0 !\n");
    return -1;
  }

  if (nom_i1 == NULL)
  {
    fprintf(stderr, "biais_mu : pas de fichier indiqu pour les valeurs de I1 !\n");
    return -1;
  }
  grf_i1 = ch_interne(nom_i1);
  if (grf_i1 == NULL)
  {
    fprintf(stderr, "biais_mu : erreur au chargement du fichier des I1 !\n");
    GRF_efface(grf_i0, VRAI, VRAI);
    return -1;
  }

  if (grf_i0->nombre_point != grf_i1->nombre_point)
  {
    fprintf(stderr, "biais_mu : les graphes I0 & I1 n'ont pas le mme nombre de points !\n");
    GRF_efface(grf_i0, VRAI, VRAI);
    GRF_efface(grf_i1, VRAI, VRAI);
    return -1;
  }

  if (mode_manip == MODE_TRANS)
  {
    biais = calc_biais_trans();
  }
  else if (mode_manip == MODE_FLUO)
  {
    biais = calc_biais_fluo();
  }

  if (biais == NULL) return -1;

  printf("Encadrement du biais : min = %g, max = %g\n", biais->y_min, biais->y_max);

  for (i = 0 ; i < biais->nombre_point ; i++)
  {
    fprintf(sortie, "%g %g\n", biais->x[i], biais->y[i]);
  }

  return 0;
}

/* ------ Calcul effectif des biais ------ */

/* ------ biais en fluorescence :

                I1 V(I1)
     E(M) =  + --------
                  I0^3

  en se limitant au premier ordre.
*/

Graphe *calc_biais_fluo()
{
  Graphe *biais;
  unsigned long i;
  register double *x_biais, *x, *i0, *i1, *vi0 = NULL, *vi1 = NULL, *y;

  biais = GRF_creer(grf_i0->nombre_point, GR_SSTYPE, GR_NORMAL, "Biais estim",
		    grf_i0->axe_x, "Biais", grf_i0->unit_x, SS_UNITE);
  if (biais == NULL)
    return NULL;

  if (grf_i0->erreur_y == NULL)
  {
    printf("biais_mu : pas de variance pour I0 -> modle de Poisson\n");
  }
  else
    vi0 = grf_i0->erreur_y->y;

  if (grf_i1->erreur_y == NULL)
  {
    printf("biais_mu : pas de variance pour I1 -> modle de Poisson\n");
  }
  else
    vi1 = grf_i1->erreur_y->y;

  for (i = 0, x = grf_i0->x, x_biais = biais->x, i0 = grf_i0->y, i1 = grf_i1->y, y = biais->y ;
       i < biais->nombre_point ; i++, x++, x_biais++, i0++, i1++, y++)
  {
    double var_i0, var_i1;

    *x_biais = *x;

    if (vi0)
    {
      var_i0 = *vi0 * *vi0;
      vi0++;
    }
    else
      var_i0 = *i0; /* loi de Poisson, V(X) = E(X) */

    if (vi1)
    {
      var_i1 = *vi1 * *vi1;
      vi1++;
    }
    else
      var_i1 = *i1;

    *y = *i1 * var_i0 / (*i0 * *i0 * *i0);
  }

  GRF_bornes(biais, ORDONNE, DESORDONNE);
  return biais;
}

/* ------ biais en transmission :

                1 [ V(I1)   V(I0) ]
     E(M) =  + - [ ----- - ----- ]
                2 [  I1     I0  ]

  en se limitant au premier ordre.
*/

Graphe *calc_biais_trans()
{
  Graphe *biais;
  unsigned long i;
  register double *x_biais, *x, *i0, *i1, *vi0 = NULL, *vi1 = NULL, *y;

  biais = GRF_creer(grf_i0->nombre_point, GR_SSTYPE, GR_NORMAL, "Biais estim",
		    grf_i0->axe_x, "Biais", grf_i0->unit_x, SS_UNITE);
  if (biais == NULL)
    return NULL;

  if (grf_i0->erreur_y == NULL)
  {
    printf("biais_mu : pas de variance pour I0 -> modle de Poisson\n");
  }
  else
    vi0 = grf_i0->erreur_y->y;

  if (grf_i1->erreur_y == NULL)
  {
    printf("biais_mu : pas de variance pour I1 -> modle de Poisson\n");
  }
  else
    vi1 = grf_i1->erreur_y->y;

  for (i = 0, x = grf_i0->x, x_biais = biais->x, i0 = grf_i0->y, i1 = grf_i1->y, y = biais->y ;
       i < biais->nombre_point ; i++, x++, x_biais++, i0++, i1++, y++)
  {
    double var_i0, var_i1;

    *x_biais = *x;

    if (vi0)
    {
      var_i0 = *vi0 * *vi0;
      vi0++;
    }
    else
      var_i0 = *i0; /* loi de Poisson, V(X) = E(X) */

    if (vi1)
    {
      var_i1 = *vi1 * *vi1;
      vi1++;
    }
    else
      var_i1 = *i1;

    *y = 0.5 * ( var_i1 / (*i1 * *i1) - var_i0 / (*i0 * *i0) );
  }

  GRF_bornes(biais, ORDONNE, DESORDONNE);
  return biais;
}

/* ------ Fonctions pour assurer l'dition de liens.  amliorer ! ------ */

int EXF_ERREUR(unsigned int code, ...)
{
  fprintf(stderr, "biais_mu : erreur module exafs code %d\n", code);
  return 0;
}
