/* Calcul du coefficient r du test de Durbin-Watson
   (C) Emmanuel Curis, fvrier 2000

   Mode d'emploi : durbin_watson [-a] [-d] fichier
     -a : le fichier est en ASCII : une colonne de chiffres, tout btement.
     -d : on veut une sortie dtaille
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "fichiers.h"

unsigned long nbr_graphes, num_graphe;
char vrai_message[250];
boolean ascii = FAUX, detail = FAUX;

static Graphe *ch_ascii(const char *nom);
static double calc_dw(Graphe *graphe);

int main(int n_args, char *args[])
{
  int i = 1;
  double d, u;
  unsigned int nbr_grf = 0;

  if (n_args < 2)
  {
    fprintf(stderr, "durbin-watson : indiquez un fichier !\n");
    return -1;
  }

  for (i = 1 ; i < n_args ; i++)
  {
    if ( args[i][0] == '-')
    {
      if ( args[i][1] == 'a' ) /* Import de fichier ASCII */
	ascii = VRAI; /* Les suivants en ASCII */
      else if ( args[i][1] == 'd' ) /* Sortie dtaille */
	detail = VRAI;
    }
    else /* Nom de fichier : on charge */
    {
      Graphe *graphe = NULL;

      if (ascii == VRAI)
      {
	graphe = ch_ascii(args[i]);
      }
      else
      {
	graphe = ch_interne(args[i]);
      }
      if (graphe == NULL)
      {
	fprintf(stderr, "durbin-watson : le fichier %s n'a pas pu tre charg !\n", args[1]);
	return -1;
      }

      nbr_grf++;
      d = calc_dw(graphe);
      if (detail == FAUX) /* On ne veut pas les dtails : on sort d, u et p(|U|>|u|) sur une ligne point */
      {
	printf("%u %.20g", nbr_grf, d);
	if (graphe->nombre_point > 25)
	{
	  /* D'aprs  Modlisation et estimation des erreurs de mesure ,
	     u dfini ici suit une loi de Laplace-Gauss centre rduite.

	     Sous H0, on a donc u = 0 : ds que u != 0, on dit rejeter
	     H0.  Plus |u| est faible, plus on a de chande de rejeter
	      tort H0 ==> le degr de signification est p(|U| > |u|),
	     qui vaut bien 1 si u = 0.
	  */

	  u  = (1 - d/2.0);
	  u *= sqrt( (graphe->nombre_point - 1.0) * (graphe->nombre_point + 1.0) );
	  u /= sqrt( graphe->nombre_point - 2.0 );
	  printf(" %.20g %.20g", u, 1.0 - erf( fabs(u) / sqrt(2.0) ) );
	}
	printf("\n");
      }
      else
      {
	printf("Coefficient d de Durbin-Watson : %f\n", d);
	if (graphe->nombre_point > 25)
	{
	  /* D'aprs  Modlisation et estimation des erreurs de mesure ,
	     u dfini ici suit une loi de Laplace-Gauss centre rduite.

	     Sous H0, on a donc u = 0 : ds que u != 0, on dit rejeter
	     H0.  Plus |u| est faible, plus on a de chande de rejeter
	      tort H0 ==> le degr de signification est p(|U| > |u|),
	     qui vaut bien 1 si u = 0.
	  */

	  u  = (1.0 - d/2.0);
	  u *= sqrt( (graphe->nombre_point - 1.0) * (graphe->nombre_point + 1.0) );
	  u /= sqrt( graphe->nombre_point - 2.0 );
	  printf("Coefficient u associ          : %f\n", u);
	  printf("p(|U| > u) = %.20g doit tre maximale\n", 1.0 - erf( fabs(u) / sqrt(2.0) ) );

	  /* Autre modle : d environ normal, moyenne 2 et variance 4/N, donc (D-2) sqrt(n)/2 suit N(0,1) */
	  u  = d - 2.0;
	  u /= (2 / sqrt( (double) graphe->nombre_point )); 
	  printf("Coefficient u' associ          : %f\n", u);
	  printf("p(|U| > u') = %.20g doit tre maximale\n", 1.0 - erf( fabs(u) / sqrt(2.0) ) );
	}
      }

      GRF_efface(graphe, VRAI, VRAI);
    }
  }

  return 0;
}

/* -------- Calcul effectif --------

         n   [             ]2
         S   [ e  - e      ]
       i = 2 [  i    i - 1 ]
   d = ---------------------
         n   [             ]
         S   [ e  - < e  > ]
       i = 1 [  i      i   ]
 */

static double calc_dw(Graphe *graphe)
{
  unsigned long i;
  register double *y = graphe->y;
  double num = 0.0, den;
  double moyenne = 0.0;

  for (i = 0 ; i < graphe->nombre_point ; i++, y++)
    moyenne += *y;
  moyenne /= (double) (graphe->nombre_point);
  if (detail == VRAI)
    printf("  moyenne exprimentale = %f\n", moyenne);

  y = graphe->y;
  den = (*y - moyenne)  * (*y - moyenne); y++;
  for (i = 1 ; i < graphe->nombre_point ; i++, y++)
  {
    den += (*y - moyenne) * (*y - moyenne);
    num += ( *y - *(y - 1) ) * ( *y - *(y - 1) );
  }

  if (den == 0.0)
  {
    fprintf(stderr, "durbin-watson : graphe nul !\n");
    return 0.0; /* Parfaitement corrl... */
  }

  return num/den;
}

/* -------- Chargement d'un fichier ASCII : suppose que les Y sont dans la 1re colonne,
   ignore le reste jusqu'en fin de ligne -------- */

static Graphe *ch_ascii(const char *nom)
{
  FILE *fichier;
  unsigned long i = 0, nbr_lignes;
  register double *y;
  Graphe *graphe;

  fichier = fopen(nom, "rb");
  if (fichier == NULL) return NULL;

  nbr_lignes = compte_lignes(fichier, '#');
  if (nbr_lignes < 2) /* Trop peu de donnes */
  {
    printf("Le fichier ne contient pas assez de lignes\n");

    fclose(fichier);
    return NULL;
  }

  graphe = GRF_creer(nbr_lignes, GR_SSTYPE, GR_NORMAL, "Donnes exprimentales",
		     NULL, NULL, SS_UNITE, SS_UNITE);
  if (graphe == NULL)
  {
    (void) fclose(fichier);
    return NULL;
  }

  y = graphe->y;
  do
  {
    int c = fgetc(fichier);
    if (c != '#') /* Ce n'est pas une ligne de commentaire */
    {
      ungetc(c, fichier);
      fscanf(fichier, "%lf", y);
      /*      printf("Valeur lue : %f\n", *y); */
      y++;
      i++;
    }

    ignore_ligne(fichier);
  } while (i < nbr_lignes);

  (void) fclose(fichier);
  return graphe;
}
