/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996-2002

   Routines gnrales, spcifiques de LASE (car utilisant ses structures) mais sans intrt scientifique
     (affichages, interface graphique,...)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h" /* Pour RCT_pointeur */
#include "interface.h"

#include "atomes.h" /* Pour GER_liste_molecule */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ; fonctions externes  ce module mais d'usage limit ~~~~~~~~ */

/* ~~~~~~~~ Traitement interne des chanes de caractres ~~~~~~~~~~~ */

/* -------- Conversion d'une chane avec conventions locales en chane utilisable par sprintf 

   Renvoie la chane de masque pour sprintf,  librer avec free() aprs utilisation
   chaine = la chane  traiter
   type_var = comment convertir le # : long, int, float, double ?
   Renvoie dans nbr_params le nombre de paramtres qu'il faudra passer  printf

   Transforme % en %%, \ en \\ et # en %ld, %d, %f ou %lf suivant les cas
*/

char *conv_chaine(const char *chaine, int type_var, int *nbr_params)
{
  char *nv_chaine;
  unsigned long nbr_diese, nbr_doubles;
  unsigned long lg_total, i, j;

  if (chaine == NULL) return NULL;
  lg_total = strlen(chaine) + 1; /* NPO le 0 final */
  nbr_diese = TXT_compte_caractere(chaine, '#');
  nbr_doubles  = TXT_compte_caractere(chaine, '%');
  nbr_doubles += TXT_compte_caractere(chaine, '\\');

  lg_total += nbr_doubles;
  if ( (type_var == DIESE_LONG) || (type_var == DIESE_DOUBLE) )
    lg_total += 2 * nbr_diese;
  else
    lg_total += nbr_diese;

  nv_chaine = (char *) malloc(sizeof(char) * lg_total);
  if (nv_chaine == NULL) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "interface.c", "conv_chaine", "nv_chaine");
    *nbr_params = 0;
    return NULL;
  }

  for (i = 0, j = 0 ; i <= strlen(chaine) ; i++, j++) /* Copie du 0 final aussi ! */
  {
    switch (chaine[i])
    {
    case '%':
      nv_chaine[j++] = '%';
      nv_chaine[j] = '%';
      break;

    case '\\':
      nv_chaine[j++] = '\\';
      nv_chaine[j] = '\\';
      break;

    case '#':
      nv_chaine[j++] = '%';
      if ( (type_var == DIESE_LONG) || (type_var == DIESE_DOUBLE) )
	nv_chaine[j++] = 'l';
      if ( (type_var == DIESE_LONG) || (type_var == DIESE_INT) )
	nv_chaine[j] = 'd';
      else if ( (type_var == DIESE_FLOAT) || (type_var == DIESE_DOUBLE) )
	nv_chaine[j] = 'f';
      break;

    default:
      nv_chaine[j] = chaine[i];
    }
  }

  *nbr_params = (int) nbr_diese;

  return nv_chaine;
}
