/* Essai pour sauver une image depuis LASE
    rendre plus propre...

   Idalement, on devrait demander la taille, crer un pixmap,
   dessiner dedans le graphe  sauver, puis crer l'image et la sauver

   On essaye d'utiliser la lib png pour faire le travail

   (c) mars 2001, Emmanuel Curis
*/

#define _LIBRAIRIE_X_ /* Car on va au coeur de X */

#include <stdio.h>
#include <stdlib.h>

#include <png.h>

#include "systeme.h"

#include "fichiers.h" /* Pour sv_image() */

/* On sauve la fentre indique */

int sv_image(Fenetre *fenetre)
{
  FILE *fichier;
  XImage *image;
  png_structp s_png;
  png_infop infos_png;
  png_color_8 bits_couleurs;
  int nbr_bits = 0, type_image;
  unsigned char **lignes = NULL;
  unsigned long i;

  if (fenetre == NULL) return -1;

  /* On coupe un ventuel rticule */
  if (fenetre->rect) {
    (fenetre->rect)(fenetre, -1, -1, fenetre->vx_x, fenetre->vx_y);
  }
  image = XGetImage( BI_Display, fenetre->handle,
		     fenetre->w_x, fenetre->w_y, fenetre->w_larg, fenetre->w_haut, ~0UL, ZPixmap );
  if (image == NULL) {
    (void) fprintf(stderr, "Impossible de crer l'image\n");
    return -2;
  }
  if (fenetre->rect) { /* On remet l'ventuel rticule */
    (fenetre->rect)(fenetre, fenetre->vx_x, fenetre->vx_y, -1, -1);
  }

  /* On initialise la table des lignes... */
  lignes = (unsigned char **) malloc( sizeof(unsigned char *) * image->height );
  if (lignes == NULL) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "sv_image.c", "sv_image", "lignes");
    XDestroyImage(image);
    return -1;
  }
  for (i = 0 ; i < (unsigned) image->height ; i++)
    lignes[i] = (unsigned char *) (image->data + i * image->bytes_per_line);

  /* Informations sur l'image */
  printf("Largeur de l'image : %d (+ %d de dcalage)\n", image->width, image->xoffset);
  printf("Hauteur de l'image : %d\n", image->height);
  printf("Octets par ligne : %d\n", image->bytes_per_line);
  printf("Ordre des octets : %d", image->byte_order);
  if (image->byte_order == LSBFirst)
    printf(" [LSB : intel]\n");
  else
    printf(" [MSB : motorola]\n");
  printf("Ordre des bits : %d", image->bitmap_bit_order);
  if (image->bitmap_bit_order == LSBFirst)
    printf(" [LSB : 10000000 = 1]\n");
  else
    printf(" [MSB : 00000001 = 1]\n");
  printf("Bitmap unit : %d\n", image->bitmap_unit);
  printf("Bitmap pad  : %d\n", image->bitmap_pad);
  printf("Profondeur  : %d\n", image->depth);
  printf("Bits par pixel : %d\n", image->bits_per_pixel);
  if (image->bits_per_pixel == 32) /* 32 bits : RGBA */
    type_image = PNG_COLOR_TYPE_RGB;
/*    type_image = PNG_COLOR_TYPE_RGB_ALPHA; */
  else if (image->bits_per_pixel == 24) /* 24 bits : RGB */
    type_image = PNG_COLOR_TYPE_RGB;
  else {
    printf("  Type de l'image  dterminer...\n"); /* On essaye RGB */
/*    if (image->red_mask == 0)  */
      type_image = PNG_COLOR_TYPE_PALETTE;
/*    else 
      type_image = PNG_COLOR_TYPE_RGB; */
  }
  printf("Masque du rouge : %lx\n", image->red_mask);
  printf("Masque du vert  : %lx\n", image->green_mask);
  printf("Masque du bleu  : %lx\n", image->blue_mask);

  /* Le infos_png->bit_depth contient le nombre de bits par couleur */
  /* On triche : on compte sur le masque du rouge le nombre de 1... */
  if ( PNG_COLOR_TYPE_RGB == type_image ) {
    unsigned int bit,

    nbr_bits = 0;
    for (bit = 0 ; bit < sizeof(unsigned long) * 8 ; bit++) /* Suppose qu'un char fait 8 bits ! */
      if ( image->red_mask & (1UL << bit) )
	nbr_bits++;
    printf("Nombre de bits pour le rouge : %d\n", nbr_bits);
    if (nbr_bits == 0) { /* Problme  - image en niveaux de gris ? ou Palette... */
      type_image = PNG_COLOR_TYPE_PALETTE;
      nbr_bits = image->bits_per_pixel;
      bits_couleurs.gray  = nbr_bits;
    }
    bits_couleurs.red   = nbr_bits; /* Il faudrait contrler que c'est correct */
    bits_couleurs.green = nbr_bits;
    bits_couleurs.blue  = nbr_bits;
    bits_couleurs.alpha = nbr_bits;
  } else {
    nbr_bits = image->bits_per_pixel;
  }

  fichier = fopen("test.png", "wb");
  if (fichier == NULL) {
    fprintf(stderr, "Erreur  l'ouverture du fichier\n");
    XDestroyImage(image);
    free(lignes);
    return -3;
  }

  s_png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (s_png == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "sv_image.c", "sv_image", "s_png");
    XDestroyImage(image);
    (void) fclose(fichier);
    free(lignes);
    return -4;
  }

  infos_png = png_create_info_struct(s_png);
  if (infos_png == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "sv_image.c", "sv_image", "infos_png");
    png_destroy_write_struct(&s_png, NULL);
    XDestroyImage(image);
    (void) fclose(fichier);
    free(lignes);
    return -5;
  }

  png_init_io(s_png, fichier);
  /* On utilise les paramtres par dfaut pour l'instant */

  /* Versions rcentes de la lib PNG requises : on utilise la fonction fournie pour l'init. */
  png_set_IHDR(s_png, infos_png, (unsigned) image->width, (unsigned) image->height, nbr_bits, type_image,
	       PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

  if (type_image == PNG_COLOR_TYPE_PALETTE) { /* On doit sauver la palette */
    unsigned int i, nbr_couleurs;
    png_color *palette;

    nbr_couleurs = 1 << (image->bits_per_pixel);
    palette = (png_color *) calloc( nbr_couleurs, sizeof(png_color));
    if (palette == NULL) {
      (void) fprintf(stderr, "sv_image : impossible de rserver la palette\n");
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "sv_image.c", "sv_image", "palette");
      (void) fclose(fichier);
      return -6;
    }

    for (i = 0 ; i < nbr_couleurs ; i++) {
      palette[i].red   = _PRG_station->couleurs[i].red;
      palette[i].green = _PRG_station->couleurs[i].green;
      palette[i].blue  = _PRG_station->couleurs[i].blue;
    }
    png_set_PLTE(s_png, infos_png, palette, (signed) nbr_couleurs);
    free(palette);
  }
  else {
    png_set_sBIT(s_png, infos_png, &bits_couleurs); /* Infos sur le codage */
  }

  infos_png->num_trans = 0; /* Pas de transparence */
  infos_png->num_text = 0; /* Pas de commentaires actuellement */

  png_write_info(s_png, infos_png);

  if (image->byte_order == LSBFirst) /* Codage LSB, or PNG demande un codage MSB */
    png_set_swap(s_png);

  if (type_image == PNG_COLOR_TYPE_RGB) { /* Pas de palette */
    /* On se dbarrasse de l'octet de transparence */
    if (image->bits_per_pixel == 32) {
      /* Quel est l'octet  faire sauter ? */
      if (image->byte_order == LSBFirst) {
	if ( (image->red_mask != 0xFF) && (image->blue_mask != 0xFF) )
	  /* Donc octet 1 = X/A, or on est en LSB donc on a en mmoire XRGB ou XBGR */
	  png_set_filler(s_png, 0, PNG_FILLER_BEFORE); /* Conversion ARGB -> RGB ou ABGR -> BGR */
	else
	  /* Donc octet 4 = X/A, or on est en LSB donc on a en mmoire RGBX ou BGRX */
	  png_set_filler(s_png, 0, PNG_FILLER_AFTER); /* Conversion RGBA -> RGB ou BGRA -> BGR */
      } else {
	if ( (image->red_mask != 0xFF) && (image->blue_mask != 0xFF) )
	  /* Donc octet 1 = X/A, or on est en MSB donc on a en mmoire RGBX ou BGRX */
	  png_set_filler(s_png, 0, PNG_FILLER_AFTER); /* Conversion ARGB -> RGB ou ABGR -> BGR */
	else
	  /* Donc octet 4 = X/A, or on est en MSB donc on a en mmoire XRGB ou XBGR */
	  png_set_filler(s_png, 0, PNG_FILLER_BEFORE); /* Conversion RGBA -> RGB ou BGRA -> BGR */
      }
    }
    
    /* On change ventuellement le bleu et le rouge */
    if ( ( (image->byte_order == LSBFirst) && (image->red_mask > image->blue_mask) ) ||
	 ( (image->byte_order == MSBFirst) && (image->red_mask < image->blue_mask) ) )
      /* On a donc en mmoire BGR, on veut RGB */
      png_set_bgr(s_png);

    /*  png_set_swap_alpha(s_png);*/ /* Conversion ARGB -> RGBA */
  } else { /* Image avec palette */
#ifdef _DEVELOPPE_
    printf("PNG -> Image avec palette, rien de spcial  faire\n");
#endif
  }

  /* On crit l'image */
  png_write_image(s_png, lignes);

  png_write_end(s_png, NULL);

  png_destroy_write_struct(&s_png, &infos_png);

  free(lignes);
  (void) fclose(fichier);
  XDestroyImage(image);
  return 0; /* Tout s'est bien pass */
}
